/**************************************************/
/*                                                */
/* File: DyConfig.c                               */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include "dataDisplay.h"
#include "URootAccess.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*                        ************                     */
/*                        *  CONFIG  *                     */
/*                        ************                     */


/*============================ PUT CONFIG =========================*/

/*------------------------------------ DyConfigPutInputs ---*/

void DyConfigPutInputs(FILE *fp)
{
 int i,j, nfound, already;
 char tag[65536];
 char chinputname[256][256];
 struct DyCh *ch;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Put all the inputs of the channels (even those not used by the plots) ---*/

 nfound = 0;
 for (i=0; i<Dy->nch; i++)
    {ch = Dy->ch[i];
     if (strlen(ch->name) < 2) continue;
     if (strlen(ch->inputname) < 2) continue;
     already = OFF;
     for (j=0; j<nfound; j++)
        {if (strcmp(ch->inputname, chinputname[j]) == 0) {already=ON; break;}}
     if (already == ON) continue;
     fprintf(fp,"\n");
     fprintf(fp,"DY_INPUT \n");
     fprintf(fp," chindex %d  %s\n", i, ch->name);
     fprintf(fp," inputtype %d  %s\n", ch->inputtype, Dy->inputtypeName[ch->inputtype]);
     fprintf(fp," inputname %s \n", ch->inputname);
     strcpy(chinputname[nfound], ch->inputname);
     nfound++;
    }

   if (Dy->InputType != INFILE)
     {DyMakeTag(tag);
      fprintf(fp,"\n");
      fprintf(fp, "DY_TAG \042%s\042 \n", tag);
     }

 return;
}

/*------------------------------------ DyConfigPutOpch ---*/

void DyConfigPutOpch(FILE *fp)
{
 int i;
 struct DyCh *ch;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Put all the user-defined channels (even those not used by the plots) ---*/

 for (i=0; i<Dy->nch; i++)
    {ch = Dy->ch[i];
     if (strncmp(ch->name, OPSIGNAL, strlen(OPSIGNAL)) != 0) continue;
     fprintf(fp,"\n");
     fprintf(fp,"DY_OPCH \n");
     fprintf(fp," opstring \042 %s \042 \n", ch->op_string);
    }

 return;
}

/*------------------------------------------- DyConfigPutPlot ---*/

void DyConfigPutPlot (int iplot, FILE *fp)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 if (iplot <= 0) return;

 plot = Plot[iplot];

 fprintf(fp, "\n");
 fprintf(fp, "DY_PLOT %d %s \n", iplot, plot->name);
 fprintf(fp, " type           %d %s \n", plot->type, Dy->typeName[plot->type]);
 fprintf(fp, " numpad         %d \n", plot->numpad);
 fprintf(fp, " superposed     %d \n", plot->superposed);
 fprintf(fp, " hidden         %d \n", plot->hidden);

 fprintf(fp, " chx_name       %s \n", plot->chx->name);
 fprintf(fp, " chx_inputname  %s \n", plot->chx->inputname);
 fprintf(fp, " chx_inputtype  %d \n", plot->chx->inputtype);
 fprintf(fp, " chx_sampFreq   %g \n", plot->chx->sampFreq);
 fprintf(fp, " chx_resampFreq %g \n", plot->chx->resampFreq);
 fprintf(fp, " chx_realunits  %d \n", plot->chx->realunits);
 if (plot->chx->vect2d_duration > 0)
   {fprintf(fp, " chx_vect2d_duration  %g \n", plot->chx->vect2d_duration);
    fprintf(fp, " chx_vect2d_deltat  %g \n", plot->chx->vect2d_deltat);
   }
/**
 if (plot->chx->inputtype == INDY)
   {fprintf(fp, " chx_opstring  \042 %s \042 \n", plot->chx->op_string);}
**/

 if (plot->chy != NULL &&
     (plot->type == PLOTTR || plot->type == PLOTCOHE || plot->type == PLOT2D
      || plot->type == PLOTTRTIME || plot->type == PLOTCOHETIME))
   {
    fprintf(fp, " chy_name       %s \n", plot->chy->name);
    fprintf(fp, " chy_inputname  %s \n", plot->chy->inputname);
    fprintf(fp, " chy_inputtype  %d \n", plot->chy->inputtype);
    fprintf(fp, " chy_sampFreq   %g \n", plot->chy->sampFreq);
    fprintf(fp, " chy_resampFreq %g \n", plot->chy->resampFreq);
    fprintf(fp, " chy_realunits  %d \n", plot->chy->realunits);
    if (plot->chy->vect2d_duration > 0)
      {fprintf(fp, " chy_vect2d_duration  %g \n", plot->chy->vect2d_duration);
       fprintf(fp, " chy_vect2d_deltat  %g \n", plot->chy->vect2d_deltat);
      }
    fprintf(fp, " gridz          %d \n", plot->gridz);
    fprintf(fp, " logz           %d \n", plot->logz);
   }

 fprintf(fp, " gridx          %d \n", plot->gridx);
 fprintf(fp, " gridy          %d \n", plot->gridy);
 fprintf(fp, " logx           %d \n", plot->logx);
 fprintf(fp, " logy           %d \n", plot->logy);

 if (plot->type == PLOTFFT)
   {
    fprintf(fp, " fft_duration   %g \n", plot->fft_duration);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " max_average    %ld \n", plot->maxAverage);
    fprintf(fp, " refresh_fft    %ld \n", plot->refresh_fft);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " ymin           %g \n", plot->ymin);
    fprintf(fp, " ymax           %g \n", plot->ymax);
    fprintf(fp, " hertz          %d \n", plot->hertz);
    fprintf(fp, " rmsfft         %d \n", plot->rmsfft);
    fprintf(fp, " decayfft       %d \n", plot->decayfft);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
   }

 else if (plot->type == PLOTTIME)
   {
    fprintf(fp, " tsize         %g \n", plot->tsize);
    fprintf(fp, " tstep         %g \n", plot->tstep);
    fprintf(fp, " ymin          %g \n", plot->ymin);
    fprintf(fp, " ymax          %g \n", plot->ymax);
    fprintf(fp, " yoffset       %g \n", plot->yoffset);
    fprintf(fp, " yscale        %g \n", plot->yscale);
    fprintf(fp, " bits          %d \n", plot->bits);
    fprintf(fp, " nodc          %d \n", plot->nodc);
    fprintf(fp, " autoscale     %d \n", plot->autoscale);
    fprintf(fp, " filter_fmin   %g \n", plot->filter_fmin);
    fprintf(fp, " filter_fmax   %g \n", plot->filter_fmax);
   }

 else if (plot->type == PLOT1D) 
   {
    fprintf(fp, " refreshdt %g \n", plot->refreshDt);
    fprintf(fp, " nbin      %ld \n", plot->xnBin);
    fprintf(fp, " xmin      %g \n", plot->xmin);
    fprintf(fp, " xmax      %g \n", plot->xmax);
    fprintf(fp, " xoffset   %g \n", plot->xoffset);
    fprintf(fp, " xscale    %g \n", plot->xscale);
    fprintf(fp, " stat      %d \n", plot->stat);
    fprintf(fp, " nodc      %d \n", plot->nodc);
    fprintf(fp, " autoedge  %d \n", plot->autoedge);
   }

 else if (plot->type == PLOT2D)
   {
    fprintf(fp, " refreshdt %g \n", plot->refreshDt);
    fprintf(fp, " xnbin     %ld \n", plot->xnBin);
    fprintf(fp, " xmin      %g \n", plot->xmin);
    fprintf(fp, " xmax      %g \n", plot->xmax);
    fprintf(fp, " ynbin     %ld \n", plot->ynBin);
    fprintf(fp, " ymin      %g \n", plot->ymin);
    fprintf(fp, " ymax      %g \n", plot->ymax);
    fprintf(fp, " stat      %d \n", plot->stat);
    fprintf(fp, " autoedge  %d \n", plot->autoedge);
    fprintf(fp, " timeshift %g \n", plot->timeshift);
    fprintf(fp, " type2d    %d \n", plot->type2d);
   }

 else if (plot->type == PLOTTR)
   {
    fprintf(fp, " gridx2         %d \n", plot->gridx2);
    fprintf(fp, " gridy2         %d \n", plot->gridy2);
    fprintf(fp, " logx2          %d \n", plot->logx2);
    fprintf(fp, " logy2          %d \n", plot->logy2);
    fprintf(fp, " fft_duration   %g \n", plot->fft_duration);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " max_average    %ld \n", plot->maxAverage);
    fprintf(fp, " refresh_fft    %ld \n", plot->refresh_fft);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " ymin           %g \n", plot->ymin);
    fprintf(fp, " ymax           %g \n", plot->ymax);
    fprintf(fp, " hertz          %d \n", plot->hertz);
    fprintf(fp, " rmsfft         %d \n", plot->rmsfft);
    fprintf(fp, " decayfft       %d \n", plot->decayfft);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
    fprintf(fp, " timeshift      %g \n", plot->timeshift);
   }

 else if (plot->type == PLOTCOHE)
   {
    fprintf(fp, " fft_duration   %g \n", plot->fft_duration);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " max_average    %ld \n", plot->maxAverage);
    fprintf(fp, " refresh_fft    %ld \n", plot->refresh_fft);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " ymin           %g \n", plot->ymin);
    fprintf(fp, " ymax           %g \n", plot->ymax);
    fprintf(fp, " hertz          %d \n", plot->hertz);
    fprintf(fp, " rmsfft         %d \n", plot->rmsfft);
    fprintf(fp, " decayfft       %d \n", plot->decayfft);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
    fprintf(fp, " timeshift      %g \n", plot->timeshift);
    fprintf(fp, " crosscohe      %d \n", plot->crossCohe);
    fprintf(fp, " crosscohe_fmin %g \n", plot->crossCohe_fmin);
    fprintf(fp, " crosscohe_fmax %g \n", plot->crossCohe_fmax);
   }

 else if (plot->type == PLOTAUDIO)
   {
    fprintf(fp, " audio_t0       %g \n", plot->audio_t0);
    fprintf(fp, " audio_duration %g \n", plot->audio_duration);
    fprintf(fp, " soundonline    %d \n", plot->soundonline);
    fprintf(fp, " sound16bits    %d \n", plot->sound16bits);
   }

 else if (plot->type == PLOTFFTTIME)
   {
    fprintf(fp, " dfreq          %g \n", plot->dfreq);
    fprintf(fp, " dtime          %g \n", plot->dtime);
    fprintf(fp, " tstep_fft      %g \n", plot->tstep_fft);
    fprintf(fp, " tsize          %g \n", plot->tsize);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " zmin           %g \n", plot->zmin);
    fprintf(fp, " zmax           %g \n", plot->zmax);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " median_norm    %d \n", plot->median_norm);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
    fprintf(fp, " type2d         %d \n", plot->type2d);
   }

 else if (plot->type == PLOTRAW) 
   {
    fprintf(fp, " refreshdt    %g \n", plot->refreshDt);
    fprintf(fp, " max_average  %ld \n", plot->maxAverage);
    fprintf(fp, " ymin         %g \n", plot->zmin);
    fprintf(fp, " ymax         %g \n", plot->zmax);
    fprintf(fp, " autoscale    %d \n", plot->autoscale);
    fprintf(fp, " decayfft     %d \n", plot->decayfft);
    fprintf(fp, " type2d       %d \n", plot->type2d);
   }

 else if (plot->type == PLOT1DTIME)
   {
    fprintf(fp, " dtime      %g \n", plot->dtime);
    fprintf(fp, " tsize      %g \n", plot->tsize);
    fprintf(fp, " tstep      %g \n", plot->tstep);
    fprintf(fp, " refreshdt  %g \n", plot->refreshDt);
    fprintf(fp, " nbin       %ld \n", plot->xnBin);
    fprintf(fp, " ymin       %g \n", plot->ymin);
    fprintf(fp, " ymax       %g \n", plot->ymax);
    fprintf(fp, " zmin       %g \n", plot->zmin);
    fprintf(fp, " zmax       %g \n", plot->zmax);
    fprintf(fp, " xoffset    %g \n", plot->xoffset);
    fprintf(fp, " xscale     %g \n", plot->xscale);
    fprintf(fp, " autoedge   %d \n", plot->autoedge);
    fprintf(fp, " nodc       %d \n", plot->nodc);
    fprintf(fp, " autoscale  %d \n", plot->autoscale);
    fprintf(fp, " type2d     %d \n", plot->type2d);
   }

 else if (plot->type == PLOTRAWTIME)
   {
    fprintf(fp, " tsize        %g \n", plot->tsize);
    fprintf(fp, " tstep        %g \n", plot->tstep);
    fprintf(fp, " refreshdt    %g \n", plot->refreshDt);
    fprintf(fp, " max_average  %ld \n", plot->maxAverage);
    fprintf(fp, " zmin         %g \n", plot->zmin);
    fprintf(fp, " zmax         %g \n", plot->zmax);
    fprintf(fp, " nodc         %d \n", plot->nodc);
    fprintf(fp, " decayfft     %d \n", plot->decayfft);
    fprintf(fp, " autoscale    %d \n", plot->autoscale);
   }

 else if (plot->type == PLOTTRTIME)
   {
    fprintf(fp, " dfreq          %g \n", plot->dfreq);
    fprintf(fp, " dtime          %g \n", plot->dtime);
    fprintf(fp, " tstep_fft      %g \n", plot->tstep_fft);
    fprintf(fp, " tsize          %g \n", plot->tsize);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " zmin           %g \n", plot->zmin);
    fprintf(fp, " zmax           %g \n", plot->zmax);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
    fprintf(fp, " type2d         %d \n", plot->type2d);
   }

 else if (plot->type == PLOTCOHETIME)
   {
    fprintf(fp, " dfreq          %g \n", plot->dfreq);
    fprintf(fp, " dtime          %g \n", plot->dtime);
    fprintf(fp, " tstep_fft      %g \n", plot->tstep_fft);
    fprintf(fp, " tsize          %g \n", plot->tsize);
    fprintf(fp, " tstep          %g \n", plot->tstep);
    fprintf(fp, " fmin           %g \n", plot->fmin);
    fprintf(fp, " fmax           %g \n", plot->fmax);
    fprintf(fp, " zmin           %g \n", plot->zmin);
    fprintf(fp, " zmax           %g \n", plot->zmax);
    fprintf(fp, " nodc           %d \n", plot->nodc);
    fprintf(fp, " autoscale      %d \n", plot->autoscale);
    fprintf(fp, " type2d         %d \n", plot->type2d);
   }

 else
   {
    printf("DyConfigPutPlot> Unkown plot %d  type=%d  name=%s  title=%s\n", iplot, plot->type, plot->name, plot->title);
   }

 return;
}

/*------------------------------------------- DyConfigPutTiming ---*/

void DyConfigPutTiming (FILE *fp)
{
 DyDebugPrint(__FUNCTION__,0);
 
 fprintf(fp, "\n"); 
 fprintf(fp, "DY_TIMING \n");
 fprintf(fp, " starttime %f  (%s) \n", Dy->startTime, FrStrUTC(Dy->startTime, 0));
 fprintf(fp, " duration  %f \n", Dy->duration);

 return;
}

/*------------------------------------------- DyConfigPutPads ---*/

void DyConfigPutPads (FILE *fp)
{
 DyDebugPrint(__FUNCTION__,0);

 fprintf(fp,"\n");
 fprintf(fp, "DY_PADS \n");
 fprintf(fp, " ncol %d \n", Dy->nCol);
 fprintf(fp, " nrow %d \n", Dy->nRow);

 return;
}

/*------------------------------------------- DyConfigPutTrigger ---*/

void DyConfigPutTriggers (FILE *fp)
{
 fprintf(fp,"\n");
 fprintf(fp, "DY_TRIGGERS\n");
 fprintf(fp," triggertype   %d \n", Dy->triggerType);
 if (Dy->triggerType == TRIG_ASCII)  fprintf(fp," trigasciifile %s \n", Dy->trigAsciiFileName);
 if (Dy->triggerType == TRIG_EVENTS) fprintf(fp," trigeventname %s \n", Dy->trigEventName);
 fprintf(fp," triggerused   %d \n", Dy->triggerUsed);
 fprintf(fp," triggerpause  %d \n", Dy->triggerPause);

 return;
}

/*------------------------------------------- DyConfigPutOptions ---*/

void DyConfigPutOptions (FILE *fp)
{
 char dummy[LSTRING_MAX];

 fprintf(fp,"\n");
 fprintf(fp, "DY_OPTIONS \n");
 fprintf(fp," debug            %d \n", Dy->debug);
 fprintf(fp," debugtype        %d \n", Dy->debugType);
 fprintf(fp," noplot           %d \n", Dy->plotsNotDisplayed);
 fprintf(fp," squareplot       %d \n", Dy->squarePlot);
 fprintf(fp," autodeselectplot %d \n", Dy->AutoDeselectPlot);
 fprintf(fp," autodeselectch   %d \n", Dy->AutoDeselectCh);
 fprintf(fp," colorpalette     %d \n", Dy->colorpalette);
 fprintf(fp," waitdata         %d \n", Dy->WaitData);
 fprintf(fp," fullwhite        %d \n", Dy->fullwhite);
 fprintf(fp," linewidth        %d \n", Dy->linewidth);
 fprintf(fp," no_autostop      %d \n", Dy->no_autostop);

 if (Dy->output_flag > 0)
   {strcpy(dummy, Dy->output_dir);
    DyFilterString(dummy,'&');
    fprintf(fp," output_dir       %s \n", dummy);
    fprintf(fp," output_nframe    %d \n", Dy->output_nframe_perfile);
    strcpy(dummy, Dy->output_tag);
    DyFilterString(dummy,'&');
    fprintf(fp," output_tag       %s \n", dummy);
    fprintf(fp," output_flag      %d \n", Dy->output_flag);
   }

 if (Dy->Ddpasswd != NULL)
   fprintf(fp," ddpasswd       %s \n", Dy->Ddpasswd);

 return;
}

/*------------------------------------------- DyConfigPutRefplots ---*/

void DyConfigPutRefplots (FILE *fp)
{
 int i;

 if (Dy->refplot_indelebile == OFF || Dy->nreffile == 0) return;

 fprintf(fp,"\n");
 fprintf(fp, "DY_REFERENCE_PLOTS \n");
 for (i=0; i<Dy->nreffile; i++)
    {fprintf(fp, " ddref_file %s \n", Dy->reffilename[i]);
     fprintf(fp, " ddref_color %d \n", Dy->refcolor[i]);
    }

 return;
}


/*------------------------------------------- DyConfigCreate ---*/

int DyConfigCreate (char *configfileName, int noroot)
{
 int i, err;
 FILE *fp;
 const char* canvasmacro;
 time_t tt;
 char date_heure[24];

 DyDebugPrint(__FUNCTION__,0);

 /*--- If no plot and no frame written at output, do not save any config ---*/

 if (Dy->nplot <= 0 && Dy->output_flag == OFF)
    {printf("DyConfigCreate> No plot, no output frame. Thus no configuration to save in %s.\n", configfileName);
     return(OFF);
    }

 /*--- Open the config file ---*/

 fp = fopen(configfileName,"w");
 if (fp==NULL)
    {printf("DyConfigCreate> Cannot open file %s to save current configuration.\n", configfileName);
     return(OFF);
    }

 /*--- Put the header in the config file ---*/
 /*--- You can use also the C functions: cuserid, gethostname, DisplayString ---*/

 tt = time(NULL);
 strftime(date_heure,24,"%d_%b_%Y_%T",localtime(&tt));
 fprintf(fp,"/*** Made on %s by %s using dataDisplay %s on %s (display=%s) ***/\n\n",
            date_heure, getenv("USER"), DYVERSION, getenv("HOST"), getenv("DISPLAY"));

 /*--- Put the timing information (startTime...) ---*/

 DyConfigPutTiming(fp);
 
 /*--- Put the general options ---*/

 DyConfigPutOptions(fp);

 /*--- Put the list of inputs needed ---*/

 DyConfigPutInputs(fp);

 /*--- Put the padding information (row, col) ---*/

 DyConfigPutPads(fp);

 /*--- Put the user-defined channels ---*/

 DyConfigPutOpch(fp);

 /*--- Put the plots ---*/

 for (i=1; i<=Dy->nplot; i++) DyConfigPutPlot(i,fp);

 /*--- Put the Reference plots ---*/

 DyConfigPutRefplots(fp);

 /*--- Put the triggers ---*/

 DyConfigPutTriggers(fp);

 /*--- Save the global canvas configuration as a C++ macro --- */

 if (noroot == OFF)
    {canvasmacro = RA_CanvasMacroString();
     fprintf(fp,"\n");
     fprintf(fp,"/*<\n%s\n>*/\n",canvasmacro);
    }

 /*--- Put also in the .car file all the .C and .h files associated to User Fct used ---*/
 fprintf(fp,"\n");
 DyMakeAlgoString(fp);

 /*--- Close config file and check that it was really written on disk ---*/

 fprintf(fp,"\n");
 err = fprintf(fp,"/* This is the End */\n");
 fclose(fp);

 if (err < 0)   /* err=ERNOSPC would mean that there is no free space left on device */
    {printf("DyConfigCreate> Could not write the file %s. Maybe no more space left on device.\n", configfileName);
     return(OFF);
    }

 return(ON);
}


/*============================ GET CONFIG =========================*/

/*----------------------------------------------- DyConfigGetInBuffer ---*/

char *DyConfigGetInBuffer (char *config_filename)
{
 FILE *fp;
 char *configbuf;
 char s;
 long n;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check that config_filename is a file ---*/

 if (is_a_file(config_filename) == 0)
    {printf("\nDyConfigGetInBuffer> %s not found\n", config_filename);
     return(NULL);
    }

 /*--- Open the config file ---*/

 fp = fopen(config_filename,"r");
 if (fp == NULL)
    {printf("\nDyConfigGetInBuffer> Could not open %s \n", config_filename);
     return(NULL);
    }

 /*--- Get size of config buffer, then malloc ---*/

 n = 0;
 while(!feof(fp)) {s = getc(fp); n++;}
 fclose(fp);
 configbuf = (char *)calloc(n+1,sizeof(char));
 strcpy(configbuf,"");

 /*--- Fill in the config buffer ---*/

 fp = fopen(config_filename,"r");
 while((s = getc(fp)) != EOF)  sprintf(configbuf,"%s%c",configbuf,s);
 fclose(fp);
 sprintf(configbuf,"%s%c",configbuf,'\0');

 return(configbuf);
}


/*---------------------------------------- DyConfigGetFile ---*/

int DyConfigGetFile (char *result)
{
 int i, nword, nameLength, commentflag;
 char **wordlist, *name, *config;
 char line[LSTRING_MAX], configfile[LSTRING_MAX], dummy[LSTRING_MAX];
 FILE *fp;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get only the first word of what was provided by user ---*/

 strcpy(configfile,strtok(result," "));

 /*--- Check that it is a file and gets its location ---*/

 if (is_a_file(configfile) == 0) {printf("DyConfigGetFile> %s is not a file \n", configfile); return(1);}

 /*--- Read a first time the config file to put it in a buffer (needed only for DyConfigGetCanvas) ---*/

 config = DyConfigGetInBuffer(configfile);

 /*--- Open the config file ---*/

 fp = fopen(configfile,"r");
 if (fp == NULL)
    {printf("DyConfigGetFile> Could not open %s \n", configfile);
     return(2);
    }

 /*--- Get location directory of the config file, so that next query will go there ---*/

 Dy->lastconfig_dir = DyDirName(Dy->lastconfig_dir, configfile);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyConfigGetFile> From \"%s\" got configfile=%s  config_dir=%s\n", result, configfile, Dy->lastconfig_dir);}

 /*--- Read the config and fill the wordlist array ---*/

 wordlist = (char **)calloc(65536,sizeof(char *));
 nword = 0;
 commentflag = OFF;

 while(fgets(line, LSTRING_MAX, fp) != NULL)
   {
    if (line[0] == '#') continue;
    if (strstr(line,"/*") != NULL) commentflag = ON;
    if (strstr(line,"*/") != NULL) {commentflag = OFF; continue;}
    if (commentflag == ON) continue;
    if (strlen(line) < 2) continue;

    name = strtok(line," ");
    while (name != NULL)
      {
       if (name[0] == '\n') {name = strtok(NULL," "); continue;}
       nameLength = strlen(name);
       wordlist[nword] = (char *)calloc(nameLength+1, sizeof(char));
       memcpy(wordlist[nword], name, nameLength);
       wordlist[nword][nameLength] = '\0';
       nword++;
       name = strtok(NULL," ");
      }
   }
 fclose(fp);

 /*--- Reset the user interface, channels and plots parameters ---*/

 DyXformRaz(NULL,0);

 /*--- Get the canvas configuration saved in the configuration file ---*/

 DyConfigGetCanvas(config);

 /*--- Get the user functions saved in the configuration file ---*/
/*
 DyConfigGetAlgo(config);
*/

 /*--- Fill the parameters ---*/

 Dy->nplot = 0;

 for (i=0; i<nword; i++)
    {
     if (strcmp(wordlist[i],"DY_TIMING") == 0)          DyConfigGetTiming(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_OPTIONS") == 0)         DyConfigGetOptions(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_INPUT") == 0)           DyConfigGetInput(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_TAG") == 0)             DyConfigGetTag(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_PADS") == 0)            DyConfigGetPads(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_OPCH") == 0)            DyConfigGetOpch(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_PLOT") == 0)            DyConfigGetPlot(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_REFERENCE_PLOTS") == 0) DyConfigGetRefPlots(wordlist, &i, nword);
     if (strcmp(wordlist[i],"DY_TRIGGERS") == 0)        DyConfigGetTriggers(wordlist, &i, nword);
    }

 /*--- Free the temporary word array used to get the config ---*/

 for (i=0; i<nword; i++) free(wordlist[i]);
 free(wordlist);

 /*--- Set GUI startime, duration and buttons with the values taken from config ---*/

 DyXformSetStartTime(0); 
 DyXformSetButtons();

 /*--- If starting with a config, remove output frames written previously in the output dir ---*/

 if (Dy->AutoConfFlag == ON) {sprintf(dummy,"rm -rf %s/Dy-*.gwf", Dy->output_dir); system(dummy);}

 return(0);
}

/*------------------------------------------- DyConfigGetAlgo ---*/

void DyConfigGetAlgo(char *config)
{
 return;
}


/*------------------------------------------- DyConfigGetCanvas ---*/

void DyConfigGetCanvas(char *config)
{
 char* cmacrostart;
 char* cmacroend;

 /*--- Build a new canvas from the canvas config macro read in the config file */

 cmacrostart = strstr(config,"/*<");
 if (cmacrostart)
    {cmacrostart +=3;
     if (Dy->cmacrocopy) free(Dy->cmacrocopy);
     Dy->cmacrocopy = (char *)calloc(strlen(cmacrostart),sizeof(char));
     cmacroend = strstr(cmacrostart,">*/");
     strncpy(Dy->cmacrocopy,cmacrostart,(int)(cmacroend-cmacrostart));
    }

 return;
}


/*------------------------------------------- DyConfigGetTiming ---*/

void DyConfigGetTiming(char **wordlist, int *iaddress, int nword)
{
 int i;

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {if (strcmp(wordlist[i],"starttime") == 0) {i++; Dy->startTime = atof(wordlist[i]);}
    if (strcmp(wordlist[i],"duration") == 0)  {i++; Dy->duration = atof(wordlist[i]);}
    i++;
   }
 i--;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetOptions ---*/

void DyConfigGetOptions(char **wordlist, int *iaddress, int nword)
{
 int i;

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"debug") == 0)            {i++; Dy->debug = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"debugtype") == 0)        {i++; Dy->debugType = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"noplot") == 0)           {i++; Dy->plotsNotDisplayed = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"squareplot") == 0)       {i++; Dy->squarePlot = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"autodeselectplot") == 0) {i++; Dy->AutoDeselectPlot = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"autodeselectch") == 0)   {i++; Dy->AutoDeselectCh = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"colorpalette") == 0)     {i++; Dy->colorpalette = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"output_flag") == 0)      {i++; Dy->output_flag = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"output_dir") == 0)       {i++; strcpy(Dy->output_dir,wordlist[i]);}
    if (strcmp(wordlist[i],"output_nframe") == 0)    {i++; Dy->output_nframe_perfile = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"output_tag") == 0)       {i++; strcpy(Dy->output_tag,wordlist[i]);}
    if (strcmp(wordlist[i],"waitdata") == 0)         {i++; Dy->WaitData = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"fullwhite") == 0)        {i++; Dy->fullwhite = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"linewidth") == 0)        {i++; Dy->linewidth = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"no_autostop") == 0)      {i++; Dy->no_autostop = atoi(wordlist[i]);}
    i++;
   }
 i--;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetInput ---*/

void DyConfigGetInput(char **wordlist, int *iaddress, int nword)
{
 int i, inputtype;
 char inputname[LSTRING_MAX];

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"inputtype") == 0) {i++; inputtype = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"inputname") == 0) {i++; strcpy(inputname,wordlist[i]);}
    i++;
   }
 i--;


 /*--- Get frame offline from frame files or wave audio files ---*/

 if (inputtype == INFILE || inputtype == INWAVE)
   {
    DyReadFirstFrame(inputname, inputtype);
   }
 /*--- Get frames online from a Cm server ---*/

 if (inputtype == INCM)
   {
    strcpy(Dy->CmServer, inputname);
    DyReadFirstFrame(NULL, inputtype);
   }

 /*--- Get frames from shared memory ---*/

 if (inputtype == INSHM)
   {strcpy(Dy->shmName, inputname);
    DyReadFirstFrame(NULL, inputtype);
   }

 /*--- Get frames online from a Dd server ---*/

 if (inputtype == INDD)
    {
     strcpy (Dy->Ddinputfile, inputname);
     i++;
     strcpy (Dy->DdServer, wordlist[i]);
     DyReadFirstFrame (NULL, inputtype);
    }

 Dy->InputType = inputtype;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetTag ---*/

void DyConfigGetTag(char **wordlist, int *iaddress, int nword)
{
 int i, length;

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    length = strlen(wordlist[i]);
    if (strcmp(wordlist[i],"\042") == 0) {i++; continue;}
    if (wordlist[i][0] == '\042')
      {if (wordlist[i][length-1] == '\042') wordlist[i][length-1]='\0';
       strcat(Dy->input_tag, &wordlist[i][1]);
       strcat(Dy->input_tag," ");
      }
    else if (wordlist[i][length-1] == '\042')
      {wordlist[i][length-1]='\0';
       strcat(Dy->input_tag, wordlist[i]);
       strcat(Dy->input_tag," ");
      }
    else
      {strcat(Dy->input_tag, wordlist[i]);
       strcat(Dy->input_tag," ");
      }
    i++;
   }
 i--;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetPads ---*/

void DyConfigGetPads(char **wordlist, int *iaddress, int nword)
{
 int i;

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"ncol") == 0) {i++; Dy->nCol = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"nrow") == 0) {i++; Dy->nRow = atoi(wordlist[i]);}
    i++;
   }
 i--;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetOpch ---*/

void DyConfigGetOpch(char **wordlist, int *iaddress, int nword)
{
 int i;
 struct DyCh *opch;
 char opstring[LBIGSTRING_MAX];
 int length;

 i = *iaddress;

 i++;

/**
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"opstring") == 0)
      {i++; i++; strcpy(opstring, wordlist[i]);
       i++;
       while(strcmp(wordlist[i],"\042") != 0)
         {strcat(opstring," ");
          strcat(opstring, wordlist[i]);
          i++;
         }
      }
    i++;
   }
**/

 strcpy(opstring,"");
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"opstring") == 0) {i++; continue;}
    length = strlen(wordlist[i]);
    if (strcmp(wordlist[i],"\042") == 0) {i++; continue;}
    if (wordlist[i][0] == '\042')
      {if (wordlist[i][length-1] == '\042') wordlist[i][length-1]='\0';
       strcat(opstring, &wordlist[i][1]);
      }
    else if (wordlist[i][length-1] == '\042')
      {wordlist[i][length-1]='\0';
       strcat(opstring," ");
       strcat(opstring, wordlist[i]);
      }
    else
      {strcat(opstring," ");
       strcat(opstring, wordlist[i]);
      }
    i++;

   }
 i--;

 opch = DyGetOpString(opstring);

 if (opch == NULL)
   {printf("DyConfigGetOpch> No new channel created for this combination of channel: opstring=%s\n", opstring);}

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetPlot ---*/

void DyConfigGetPlot(char **wordlist, int *iaddress, int nword)
{
 int i, j, is2d;
 struct DyPlot *plot;
 char plotname[LSTRING_MAX];
 int plotid;

 i = *iaddress;

 i++;
 plotid = atoi(wordlist[i]);
 i++;
 strcpy(plotname,wordlist[i]);
 i++;

 /*--- Create a new plot and init its parameters ---*/

 Dy->nplot++;
 plot = Plot[Dy->nplot];
 DyPlotInit(Dy->nplot,ON);

 /*--- By default init one channel ---*/

 plot->chx = (struct DyCh *)calloc(1, sizeof(struct DyCh));

 /*--- A flag used to prevent doing twice a malloc for chy ---*/

 is2d = OFF;

 /*--- Loop over the parameters and read them according to the plot type ---*/

 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    /*--- Get plot generic parameters ---*/

    if (strcmp(wordlist[i],"type") == 0)           {i++; plot->type = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"numpad") == 0)         {i++; plot->numpad = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"superposed") == 0)     {i++; plot->superposed = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"hidden") == 0)         {i++; plot->hidden = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"chx_name") == 0)       {i++; strcpy(plot->chx->name,wordlist[i]);}
    if (strcmp(wordlist[i],"chx_inputname") == 0)  {i++; strcpy(plot->chx->inputname,wordlist[i]);}
    if (strcmp(wordlist[i],"chx_inputtype") == 0)  {i++; plot->chx->inputtype = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"chx_sampFreq") == 0)   {i++; plot->chx->sampFreq = atof(wordlist[i]);}
    if (strcmp(wordlist[i],"chx_resampFreq") == 0) {i++; plot->chx->resampFreq = atof(wordlist[i]);}
    if (strcmp(wordlist[i],"chx_realunits") == 0)  {i++; plot->chx->realunits = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"chx_vect2d_duration") == 0) {i++; plot->chx->vect2d_duration = atof(wordlist[i]);} 
    if (strcmp(wordlist[i],"chx_vect2d_deltat") == 0)   {i++; plot->chx->vect2d_deltat = atof(wordlist[i]);} 

    if (strncmp(wordlist[i],"chy_",4) == 0 && is2d == OFF)  {is2d = ON; plot->chy = (struct DyCh *)calloc(1, sizeof(struct DyCh));}
    if (strcmp(wordlist[i],"chy_name") == 0)       {i++; strcpy(plot->chy->name,wordlist[i]);}
    if (strcmp(wordlist[i],"chy_inputname") == 0)  {i++; strcpy(plot->chy->inputname,wordlist[i]);}
    if (strcmp(wordlist[i],"chy_inputtype") == 0)  {i++; plot->chy->inputtype = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"chy_sampFreq") == 0)   {i++; plot->chy->sampFreq = atof(wordlist[i]);}
    if (strcmp(wordlist[i],"chy_resampFreq") == 0) {i++; plot->chy->resampFreq = atof(wordlist[i]);}
    if (strcmp(wordlist[i],"chy_realunits") == 0)  {i++; plot->chy->realunits = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"chy_vect2d_duration") == 0) {i++; plot->chy->vect2d_duration = atof(wordlist[i]);} 
    if (strcmp(wordlist[i],"chy_vect2d_deltat") == 0)   {i++; plot->chy->vect2d_deltat = atof(wordlist[i]);} 

    if (strcmp(wordlist[i],"gridx") == 0)          {i++; plot->gridx = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"gridy") == 0)          {i++; plot->gridy = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"gridz") == 0)          {i++; plot->gridz = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"gridx2") == 0)         {i++; plot->gridx2 = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"gridy2") == 0)         {i++; plot->gridy2 = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"logx") == 0)           {i++; plot->logx = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"logy") == 0)           {i++; plot->logy = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"logz") == 0)           {i++; plot->logz = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"logx2") == 0)          {i++; plot->logx2 = atoi(wordlist[i]);} 
    if (strcmp(wordlist[i],"logy2") == 0)          {i++; plot->logy2 = atoi(wordlist[i]);} 

    /*--- Get plot specific parameters ---*/

    if (plot->type == PLOTFFT)
      {
       if (strcmp(wordlist[i],"fft_duration") == 0) {i++; plot->fft_duration = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"max_average") == 0)  {i++; plot->maxAverage = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"refresh_fft") == 0)  {i++; plot->refresh_fft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)         {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)         {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"hertz") == 0)        {i++; plot->hertz = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"rmsfft") == 0)       {i++; plot->rmsfft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"decayfft") == 0)     {i++; plot->decayfft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
      }

    else if (plot->type == PLOTTIME)
      {
       if (strcmp(wordlist[i],"tsize") == 0)     {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)     {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)      {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)      {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"bits") == 0)      {i++; plot->bits = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)      {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0) {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"yoffset") == 0)   {i++; plot->yoffset = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"yscale") == 0)    {i++; plot->yscale = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"filter_fmin") == 0) {i++; plot->filter_fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"filter_fmax") == 0) {i++; plot->filter_fmax = atof(wordlist[i]);} 
      }

    else if (plot->type == PLOT1D)
      {
       if (strcmp(wordlist[i],"refreshdt") == 0) {i++; plot->refreshDt = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nbin") == 0)      {i++; plot->xnBin = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"xmin") == 0)      {i++; plot->xmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xmax") == 0)      {i++; plot->xmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xoffset") == 0)   {i++; plot->xoffset = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xscale") == 0)    {i++; plot->xscale = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoedge") == 0)  {i++; plot->autoedge = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"stat") == 0)      {i++; plot->stat = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)      {i++; plot->nodc = atoi(wordlist[i]);} 
      }

    else if (plot->type == PLOT2D)
      {
       if (strcmp(wordlist[i],"refreshdt") == 0) {i++; plot->refreshDt = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xnbin") == 0)     {i++; plot->xnBin = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"xmin") == 0)      {i++; plot->xmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xmax") == 0)      {i++; plot->xmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ynbin") == 0)     {i++; plot->ynBin = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)      {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)      {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"timeshift") == 0) {i++; plot->timeshift = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoedge") == 0)  {i++; plot->autoedge = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"stat") == 0)      {i++; plot->stat = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)    {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    else if (plot->type == PLOTTR)
      {
       if (strcmp(wordlist[i],"fft_duration") == 0) {i++; plot->fft_duration = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"max_average") == 0)  {i++; plot->maxAverage = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"refresh_fft") == 0)  {i++; plot->refresh_fft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)         {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)         {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"hertz") == 0)        {i++; plot->hertz = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"rmsfft") == 0)       {i++; plot->rmsfft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"decayfft") == 0)     {i++; plot->decayfft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"timeshift") == 0)    {i++; plot->timeshift = atof(wordlist[i]);} 
      }

    else if (plot->type == PLOTCOHE)
      {
       if (strcmp(wordlist[i],"fft_duration") == 0) {i++; plot->fft_duration = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"max_average") == 0)  {i++; plot->maxAverage = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"refresh_fft") == 0)  {i++; plot->refresh_fft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)         {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)         {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"hertz") == 0)        {i++; plot->hertz = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"rmsfft") == 0)       {i++; plot->rmsfft = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"decayfft") == 0)     {i++; plot->decayfft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"timeshift") == 0)    {i++; plot->timeshift = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"crosscohe") == 0)    {i++; plot->crossCohe = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"crosscohe_fmin") == 0) {i++; plot->crossCohe_fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"crosscohe_fmax") == 0) {i++; plot->crossCohe_fmax = atof(wordlist[i]);} 
      }

    if (plot->type == PLOTFFTTIME)
      {
       if (strcmp(wordlist[i],"dfreq") == 0)        {i++; plot->dfreq = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"dtime") == 0)        {i++; plot->dtime = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep_fft") == 0)    {i++; plot->tstep_fft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tsize") == 0)        {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmin") == 0)         {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmax") == 0)         {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"median_norm") == 0)  {i++; plot->median_norm = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)       {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOTRAW)
      {
       if (strcmp(wordlist[i],"refreshdt") == 0)    {i++; plot->refreshDt = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"max_average") == 0)  {i++; plot->maxAverage = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)         {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)         {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"decay") == 0)        {i++; plot->decay = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)       {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOTRAWTIME)
      {
       if (strcmp(wordlist[i],"tsize") == 0)     {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)     {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"refreshdt") == 0) {i++; plot->refreshDt = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"max_average") == 0)  {i++; plot->maxAverage = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmin") == 0)      {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmax") == 0)      {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)      {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0) {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"decay") == 0)        {i++; plot->decay = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)    {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOT1DTIME)
      {
       if (strcmp(wordlist[i],"dtime") == 0)     {i++; plot->dtime = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tsize") == 0)     {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)     {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"refreshdt") == 0) {i++; plot->refreshDt = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nbin") == 0)      {i++; plot->ynBin = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymin") == 0)      {i++; plot->ymin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"ymax") == 0)      {i++; plot->ymax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmin") == 0)      {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmax") == 0)      {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xoffset") == 0)   {i++; plot->xoffset = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"xscale") == 0)    {i++; plot->xscale = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoedge") == 0)  {i++; plot->autoedge = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"stat") == 0)      {i++; plot->stat = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)      {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0) {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)    {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOTTRTIME)
      {
       if (strcmp(wordlist[i],"dfreq") == 0)        {i++; plot->dfreq = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"dtime") == 0)        {i++; plot->dtime = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep_fft") == 0)    {i++; plot->tstep_fft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tsize") == 0)        {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmin") == 0)         {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmax") == 0)         {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)       {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOTCOHETIME)
      {
       if (strcmp(wordlist[i],"dfreq") == 0)        {i++; plot->dfreq = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"dtime") == 0)        {i++; plot->dtime = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep_fft") == 0)    {i++; plot->tstep_fft = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tsize") == 0)        {i++; plot->tsize = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"tstep") == 0)        {i++; plot->tstep = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmin") == 0)         {i++; plot->fmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"fmax") == 0)         {i++; plot->fmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmin") == 0)         {i++; plot->zmin = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"zmax") == 0)         {i++; plot->zmax = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"nodc") == 0)         {i++; plot->nodc = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"autoscale") == 0)    {i++; plot->autoscale = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"type2d") == 0)       {i++; plot->type2d = atoi(wordlist[i]);} 
      }

    if (plot->type == PLOTAUDIO)
      {
       if (strcmp(wordlist[i],"audio_t0") == 0)       {i++; plot->audio_t0 = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"audio_duration") == 0) {i++; plot->audio_duration = atof(wordlist[i]);} 
       if (strcmp(wordlist[i],"soundonline") == 0)    {i++; plot->soundonline = atoi(wordlist[i]);} 
       if (strcmp(wordlist[i],"sound16bits") == 0)    {i++; plot->sound16bits = atoi(wordlist[i]);} 
      }

    i++;
   }

 i--;

 /*--- If channel is user defined, get its definition from the channels list ---*/

 if (plot->chx->inputtype == INDY)
   {for (j=Dy->nch-1; j>=0; j--)
       {if (strcmp(plot->chx->name, Dy->ch[j]->name) == 0) {DyChCloneOpch(plot->chx, Dy->ch[j]); break;}}
   }
 if (plot->chy && plot->chy->inputtype == INDY)
   {for (j=Dy->nch-1; j>=0; j--)
       {if (strcmp(plot->chy->name, Dy->ch[j]->name) == 0) {DyChCloneOpch(plot->chy, Dy->ch[j]); break;}}
   }

 /*--- Set the name and the title of the plot and set the secondary parameters of the plot ---*/

 Dy->nplotcreated++;
 sprintf(plot->name, "%s%d", DYPLOT_PREFIX, Dy->nplotcreated);
 DyPlotSetTitle(Dy->nplot);
 DyPlotSetSecondParam(Dy->nplot);


 /*--- Add this plot in the GUI plots browser ---*/

 DyXformUpdatePlotBrowser(Dy->nplot, ADD_PLOT);

 *iaddress = i;

 return;
}


/*------------------------------------------- DyConfigGetRefPlots ---*/

void DyConfigGetRefPlots(char **wordlist, int *iaddress, int nword)
{
 int i, color;
 char filename[LSTRING_MAX];

 i = *iaddress;

 i++;

 DyXformRaz(NULL,6);

 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"ddref_file") == 0)  {i++; strcpy(filename,wordlist[i]);} 
    if (strcmp(wordlist[i],"ddref_color") == 0)
      {i++; color = atoi(wordlist[i]); 
       DyRefPlotAdd(filename,color);
       RA_RefPlotRead(filename,color);
       DyXformRefPlotFillBrowser();
       Dy->lastrefplot_dir = DyDirName(Dy->lastrefplot_dir, filename);
      }
    i++;
   }

 i--;

 *iaddress = i;

 return;
}

/*------------------------------------------- DyConfigGetTriggers ---*/

void DyConfigGetTriggers(char **wordlist, int *iaddress, int nword)
{
 int i;

 i = *iaddress;

 i++;
 while (i < nword && strncmp(wordlist[i],"DY_",3) != 0)
   {
    if (strcmp(wordlist[i],"triggertype") == 0) {i++; Dy->triggerType = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"triggerused") == 0) {i++; Dy->triggerUsed = atoi(wordlist[i]);}
    if (strcmp(wordlist[i],"triggerpause") == 0) {i++; Dy->triggerPause = atoi(wordlist[i]);}
    i++;
   }
 i--;

 *iaddress = i;

 return;
}

 
/*------------------------------------------- DyMakeAlgoString ---*/

void DyMakeAlgoString (FILE *fp)
{
 FILE *fpalgo;
 char str[LBIGSTRING_MAX], algoname[LSTRING_MAX],  algofile[LSTRING_MAX];
 int i;

 DyDebugPrint(__FUNCTION__,0);

 for (i=0; i<Dy->nch; i++)
    {
     if (strncmp(Dy->ch[i]->name,OPSIGNAL,strlen(OPSIGNAL)) == 0)
       {
        strcpy(algoname,Dy->ch[i]->op_classname);

        /*--- Looking for already existing algo file ---*/
        sprintf(algofile, "%s.C", algoname);
        fpalgo = fopen(algofile, "r");

        /*--- If not found, try a generic place ---*/
        if (fpalgo == NULL)
          {sprintf(algofile, "%s/%s/%s.C", DYALGO_DIR, getenv("UNAME"), algoname);
           fpalgo = fopen(algofile, "r");
          }

        if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1)
          {printf("DyMakeAlgoString> channel%d: found algo %s... searching for %s\n", i, algoname, algofile);}

        /*--- If file found, get its content and put its content in the config ---*/
        if (fpalgo != NULL)
          {
           fprintf(fp, "\n%s %s %s\n", ALGO_BEGIN, algoname, algofile);
           while (fgets(str,2048,fpalgo)) fprintf(fp, "%s", str);
           fprintf(fp, "\n%s\n", ALGO_END);
          }
        else
          {printf("DyMakeAlgoString> %s not found \n", algofile);}

        /*--- Now search for the .h file ---*/
        sprintf(algofile, "%s.h", algoname);
        fpalgo = fopen(algofile, "r");

        /*--- If not found, try a generic place ---*/
        if (fpalgo == NULL)
          {sprintf(algofile, "%s/%s/%s.h", DYALGO_DIR, getenv("UNAME"), algoname);
           fpalgo = fopen(algofile, "r");
          }

        if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1)
          {printf("DyMakeAlgoString> channel%d: found algo %s... searching for %s\n", i, algoname, algofile);}

        /*--- If file found, get its content and put its content in the config ---*/
        if (fpalgo != NULL)
          {
           fprintf(fp, "\n%s %s %s\n", ALGO_BEGIN, algoname, algofile);
           while (fgets(str,2048,fpalgo)) fprintf(fp, "%s", str);
           fprintf(fp, "\n%s\n", ALGO_END);
          }
        else
          {printf("DyMakeAlgoString> %s not found \n", algofile);}
       }
    }

 return;
}

