/**************************************************/
/*                                                */
/* File: DyInput.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/* regroup the following previous files:          */
/* DyFiles.c  DyCm.c DyFd.c  DyFdShm.c  DyDd.c    */
/*                                                */
/**************************************************/

#include "fselector.h"
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*--------------------------------------------------- DyChOpenInput ------*/
void DyChOpenInput (struct DyCh *ch)
{
 int i, k, iwait, result;
 char dummy[NPLOT_MAX*LSTRING_MAX];
 char tag[NPLOT_MAX*LSTRING_MAX];
 char *buf;
 long wave_nData;

 if (ch == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 if (ch->inputname == NULL) {printf("No input defined for channel %s \n", ch->name); return;}


 /*--- If ch is an operation on channels, open inputs of the channels ---*/

 if (ch->inputtype == INDY)
   {
    for (i=0; i<ch->op_nch; i++) DyChOpenInput(ch->op_chin[i]);
    return;
   }

 /*--- Initialize the ifile and shm pointers ---*/

 ch->ifile = NULL;
 ch->shm   = NULL;

 /*--- Check if in other plots, there is already a ifile or shm opened ---*/
 /*--- This prevent to open a file descriptor for each channel we plot ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     if (strcmp(Plot[i]->chx->inputname,ch->inputname) == 0)
       {
        if (Plot[i]->chx->inputtype != ch->inputtype) continue;
        if (Plot[i]->chx->inputtype == INDY)
          {for (k=0; k<Plot[i]->chx->op_nch; k++) {if (DyChOpenClone(ch, Plot[i]->chx->op_chin[k]) == ON) return;}}
        else
          {if (DyChOpenClone(ch, Plot[i]->chx) == ON) return;}
       }

     if (Plot[i]->chy && strcmp(Plot[i]->chy->inputname,ch->inputname) == 0)
       {
        if (Plot[i]->chy->inputtype != ch->inputtype) continue;
        if (Plot[i]->chy->inputtype == INDY)
          {for (k=0; k<Plot[i]->chy->op_nch; k++) {if (DyChOpenClone(ch, Plot[i]->chy->op_chin[k]) == ON) return;}}
        else
          {if (DyChOpenClone(ch, Plot[i]->chy) == ON) return;}
       }
    }

 /*--- If input is a file and no previously opened ifile has been found, set ifile for this channel ---*/

 if (ch->inputtype == INFILE && ch->ifile == NULL)
   {
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChOpenInput> Open file %s for channel %s\n", ch->inputname, ch->name);
    ch->ifile = FrFileINew(ch->inputname);
   }

 /*--- If input is a shm and no previously opened shm has been found, set shm for this channel ---*/

 else if (ch->inputtype == INSHM && ch->shm == NULL)
   {
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChOpenInput> Open shm for channel %s\n", ch->name);
    iwait = 0;
    while ((ch->shm = FdShmOpen (ch->inputname, Dy->CmName)) == NULL  &&  Dy->Order != STOP)
       {fl_check_forms();
        if (iwait%DY_TIMEOUT2 == 0)
           {sprintf(dummy,"Could not open shm %s since %d sec", ch->inputname, iwait);
            DyInfoLine(dummy);
           }
        sleep(1);
        iwait++;
       }
    }

 /*--- If input is a Cm server, connect to it and set the waiting loop parameters ---*/

 else if (ch->inputtype == INCM && ch->ifile == NULL)
   {
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChOpenInput> Open Cm connection to %s for channel %s\n", ch->inputname, ch->name);
/**
    result = DyCmConnect();
    if (result == -1)
      {Dy->Order = STOP;
       sprintf(dummy,"Not been able to connect with the Cm name %s", Dy->CmName);
       DyInfoLine(dummy);
       return;
      }
**/
    ch->ifile = FrFileIInMemoryNew();
    Dy->CmMffile = ch->ifile;
    if (Dy->CmMftspan < Dy->maxdt / 2) Dy->CmMftspan = 2 * Dy->maxdt;
    Dy->CmMftspan = 2 * Dy->maxdt;
    if (Dy->CmMftspan < 16) Dy->CmMftspan = 16;
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChOpenInput> maxdt=%g  CmMftspan=%g \n", Dy->maxdt, Dy->CmMftspan);
    DyMakeTag(tag);
    DyFdRequestFrames(ch->inputname, Dy->CmName, tag, DYCM_QSIZE, 0);
    sprintf(dummy,"DyChOpenInput> Sent frames request to %s", ch->inputname);
    DyInfoLine(dummy);
    if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0) printf("DyChOpenInput> tag=\"%s\"\n",tag);
   }

 /*--- If input is a Wave file, open it and read its data with a special Read function ---*/

 else if (ch->inputtype == INWAVE)
   {
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChOpenInput> Open a wave file for channel %s\n", ch->name);
    ch->wavefile = fopen(ch->inputname,"r");
    if (ch->wavefile == NULL)
      {sprintf(dummy, "Cannot open file %s", ch->inputname);
       DyInfoPanel(dummy,0);
       return;
      }

    buf = (char *)(&(wavehdr));
    fread(buf, 1, sizeof(wavehdr), ch->wavefile);
    DySoundReadHeader(buf);

    if (wavehdr.riff[0] != 'R')
      {printf("DyChOpenInput> %s if not a wav file \n", ch->inputname); Dy->Order = STOP; return;}
    if (wavehdr.bits_per_samp != 8 && wavehdr.bits_per_samp != 16)
      {printf("DyChOpenInput> %s is not 8 or 16 bits wav file \n", ch->inputname); Dy->Order = STOP; return;}
    if (wavehdr.format_tag != 1 && wavehdr.format_tag != 259)
      {printf("DyChOpenInput> %s is not PCM format wav file \n", ch->inputname); Dy->Order = STOP; return;}
    if (wavehdr.num_chans == 256 * buf[30] + buf[31])
      {printf("DyChOpenInput> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in big endian\n", wavehdr.num_chans, buf[30], buf[31]);}
    else
      {printf("DyChOpenInput> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in little endian\n", wavehdr.num_chans, buf[30], buf[31]);}

    wave_nData = wavehdr.dlength / (wavehdr.bytes_per_samp / wavehdr.num_chans);
    printf("DyChOpenInput> Wave file contains %d bytes with %d bytes per sample (%d bits) and %d channels --> nData=%ld\n",
            wavehdr.dlength, wavehdr.bytes_per_samp, wavehdr.bits_per_samp, wavehdr.num_chans, wave_nData);

    inputFrame = FrameHNew("frame_from_wave");
    FrAdcDataNew (inputFrame, AUDIOCH_NAME, wavehdr.srate, (int)(wavehdr.srate * 1/ch->sampFreq), 16);
   }

 else
   {printf("  DyChOpenInput> Input type %d not known for channel %s\n", ch->inputtype, ch->name);}

 return;
}

/*------------------------------------------- DyChOpenClone ---*/

int DyChOpenClone(struct DyCh *ch, struct DyCh *ch0)
{
 if (ch->inputtype == INFILE && ch0->ifile != NULL) {ch->ifile = ch0->ifile; return(ON);}
 if (ch->inputtype == INCM && ch0->ifile != NULL)   {ch->ifile = ch0->ifile; return(ON);}
 if (ch->inputtype == INSHM && ch0->shm != NULL)    {ch->shm   = ch0->shm;   return(ON);}

 return(OFF);
}


/*------------------------------------------- DyChCloseInput ---*/

void DyChCloseInput(struct DyCh *ch)
{
 int i, k;

 DyDebugPrint(__FUNCTION__,0);

 if (ch == NULL) return;
 
 /*--- If ch is an operation on channels, open inputs of the channels ---*/

 if (ch->inputtype == INDY)
   {
    if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChCloseInput> %s : close inputs of its input channels \n", ch->name);
    for (i=0; i<ch->op_nch; i++) DyChCloseInput(ch->op_chin[i]);
    return;
   }

 /*--- Close the ifile or shm of this channel ---*/

 if (ch->inputtype == INFILE && ch->ifile != NULL)
   {if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChCloseInput> Close file %s for channel %s\n", ch->inputname, ch->name);
    FrFileIEnd(ch->ifile);
    ch->ifile = NULL;
   }

 else if (ch->inputtype == INSHM && ch->shm != NULL)
   {if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChCloseInput> Close shm for channel %s\n", ch->name);
    FdShmClose(ch->shm, Dy->CmName);
    ch->shm = NULL;
   }

 else if (ch->inputtype == INCM && ch->ifile != NULL)
   {if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChCloseInput> Close Cm connection to %s for channel %s\n", ch->inputname, ch->name);
    DyFdRemoveFramesRequest (Dy->CmServer, Dy->CmName);
/*
    DyCmDisconnect();
*/
    FrFileIEnd(ch->ifile);
    ch->ifile = NULL;
    Dy->CmMffile = NULL;
   }


 else if (ch->inputtype == INWAVE && ch->wavefile != NULL)
   {if (((Dy->debugType)&(1<<DBG_READ))) printf("DyChCloseInput> Close a wave file for channel %s\n", ch->name);
    fclose(ch->wavefile);
    ch->wavefile = NULL;
   }

 else
   {return;}

 /*--- Check in other plots if same ifile or shm is used. If yes, then put the ifile or shm pointer to NULL ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->chx->inputtype == INDY) 
       {for (k=0; k<Plot[i]->chx->op_nch; k++) DyChCloseClone(ch,Plot[i]->chx->op_chin[k]);}
     else if (Plot[i]->chx->inputtype == ch->inputtype)
       {DyChCloseClone(ch,Plot[i]->chx);}

     if (Plot[i]->chy)
       {
        if (Plot[i]->chy->inputtype == INDY)
          {for (k=0; k<Plot[i]->chy->op_nch; k++) DyChCloseClone(ch,Plot[i]->chy->op_chin[k]);}
        else if (Plot[i]->chy->inputtype == ch->inputtype)
          {DyChCloseClone(ch,Plot[i]->chy);}
       }
    }

 return;
}


/*------------------------------------------- DyChCloseClone ---*/
/*--- Compare ch0 to ch and put to NULL ch0->ifile if ch0 has same input as ch->ifile already put to NULL ---*/

void DyChCloseClone(struct DyCh *ch, struct DyCh *ch0)
{
 if (strcmp(ch0->inputname,ch->inputname) != 0) return;

 if (ch->inputtype == INFILE && ch0->ifile != NULL)    ch0->ifile = NULL;
 if (ch->inputtype == INCM && ch0->ifile != NULL)      ch0->ifile = NULL;
 if (ch->inputtype == INSHM && ch0->shm != NULL)       ch0->shm = NULL;
 if (ch->inputtype == INWAVE && ch0->wavefile != NULL) ch0->wavefile = NULL;

 return;
}


/**************************************************/
/*                                                */
/* File: DyFiles.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*                        ***********                     */
/*                        *  FILES  *                     */
/*                        ***********                     */


/*--------------------------------------------------- DyAsciiToFrame ------*/
struct FrameH *DyAsciiToFrame (char *asciifileName, double framedt)
{
 double gtime1, gtime2, gtime3, gtime, value, vectdt;
 char line[LSTRING_MAX], dummy[LSTRING_MAX];
 FILE *file;
 int nline, j, is, vectnData;
 struct FrAdcData *adc;
 struct FrameH *frame;
 int commentflag;

 DyDebugPrint(__FUNCTION__,0);


 /*--- Open file and read the first lines ---*/

 file = fopen(asciifileName,"r");

 if (file == NULL)
    {sprintf(dummy,"Cannot open ascii file %s", asciifileName);
     DyInfoLine(dummy); 
     Dy->firstTime = 0.;
     return(NULL);
    }

 commentflag = OFF;
 nline = 0;
 while(fgets(line, LSTRING_MAX, file) != NULL)
   {
    if (line[0] == '#') continue;
    if (strstr(line,"/*") != NULL) commentflag = ON;
    if (strstr(line,"*/") != NULL) {commentflag = OFF; continue;}
    if (commentflag == ON) continue;
    if (strlen(line) < 2) continue;

    sscanf(line,"%lf %lf",&gtime, &value);
printf("nline=%d  gtime=%f value=%g  line=%s \n", nline, gtime, value, line);
    if (nline == 0) gtime1 = gtime;
    if (nline == 1) gtime2 = gtime;
    if (nline == 2) gtime3 = gtime;
    nline++;
    if (nline == 4) break;
   }
 fclose(file);


 /*--- Check that format is gtime followed by a value ---*/

 if (gtime1 == 0 && gtime2 == 0)
   {printf("DyAsciiToFrame> Ascii file %s has not the correct format \n", asciifileName); return(NULL);}
 if (gtime3 - gtime2 != gtime2 - gtime1)
   {printf("DyAsciiToFrame> Ascii file %s has not a list of sampled values \n", asciifileName); return(NULL);}

 /*--- Get the sampling rate ---*/

 vectdt = gtime2 - gtime1;
 if (vectdt > 0) vectnData = (int)(framedt/vectdt);
 else            {printf("DyAsciiToFrame> vectdt=%g  gtime1=%f gtime2=%f \n", vectdt, gtime1, gtime2); return(NULL);}

 /*--- Prepare the frame and the adc channel which will contain the data ---*/

 if (gtime1 > 700000000) Dy->startTime = gtime1;

 frame         = FrameHNew("frame_from_ascii");
 frame->GTimeS = (int)Dy->startTime;
 frame->dt     = framedt;
 sprintf(dummy,"ascii_data_%d", Dy->nch);
 adc = FrAdcDataNew (frame, dummy, 1/vectdt, vectnData, -64);

 /*--- Now read the ascii file to fill the frame ---*/

 file = fopen(asciifileName,"r");
 commentflag = OFF;
 is = 0;
 while(fgets(line, LSTRING_MAX, file) != NULL)
   {
    if (line[0] == '#') continue;
    if (strstr(line,"/*") != NULL) commentflag = ON;
    if (strstr(line,"*/") != NULL) {commentflag = OFF; continue;}
    if (commentflag == ON) continue;
    if (strlen(line) < 2) continue;

    sscanf(line,"%lf %lf",&gtime, &value);
    adc->data->dataD[is] = value;
    is++;
    if (is == vectnData) break;
   }

 if (is < vectnData) {for (j=is; j<vectnData; j++) adc->data->dataD[j]=0;}

 fclose(file);

 return(frame);
}


/*--------------------------------------------------- DyWaveToFrame ------*/
struct FrameH *DyWaveToFrame (char *wavefileName, double framedt)
{
 unsigned char valuec;
 short value;
 char *buf, dummy[LSTRING_MAX];
 FILE *file;
 int j, i, is, nData, vectnData;
 struct FrAdcData *adc;
 struct FrameH *frame;

 DyDebugPrint(__FUNCTION__,0);

 file = fopen(wavefileName,"r");
 if (file == NULL)
    {sprintf(dummy,"Cannot open wave audio file %s", wavefileName);
     DyInfoLine(dummy); 
     Dy->firstTime = 0.;
     return(NULL);
    }

 buf = (char *)(&(wavehdr));
 fread(buf, 1, sizeof(wavehdr), file);
 DySoundReadHeader(buf);

 if (wavehdr.riff[0] != 'R')
    {sprintf(dummy,"%s if not a wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }
 if (wavehdr.bits_per_samp != 8 && wavehdr.bits_per_samp != 16)
    {sprintf(dummy,"%s is not 8 or 16 bits wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }
 if (wavehdr.format_tag != 1 && wavehdr.format_tag != 259)
    {sprintf(dummy,"%s is not PCM format wav file", wavefileName);
     DyInfoLine(dummy); 
     return(NULL);
    }

 vectnData = (int)(wavehdr.srate * framedt);
 nData     = wavehdr.dlength / (wavehdr.bytes_per_samp / wavehdr.num_chans);

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug>0)
    {
     if (wavehdr.num_chans == 256 * buf[30] + buf[31])
        {printf("DyWaveToFrame> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in big endian\n", wavehdr.num_chans, buf[30], buf[31]);}
     else
        {printf("DyWaveToFrame> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in little endian\n", wavehdr.num_chans, buf[30], buf[31]);}
     printf("DyWaveToFrame> Wave file contains %d bytes with %d bytes per sample (%d bits) and %d channels --> nData=%d\n",
            wavehdr.dlength, wavehdr.bytes_per_samp, wavehdr.bits_per_samp, wavehdr.num_chans, nData);
     printf("DyWaveToFrame> Frame length = %g --> vectnData=%d\n", framedt, vectnData);
    }

 frame         = FrameHNew("frame_from_wave");
 frame->GTimeS = (int)Dy->startTime;
 frame->dt     = framedt;
 sprintf(dummy,"audio_data_%d", Dy->nch);
 adc = FrAdcDataNew (frame, dummy, wavehdr.srate, vectnData, 16);

 i = 0;
 for (is=0; is<vectnData; is++)
    {
     if (wavehdr.bits_per_samp == 16) fread(&value, sizeof(short), 1, file);
     else                             fread(&valuec, sizeof(unsigned char), 1, file);
     if (wavehdr.num_chans == 2 && i%2 == 1) is--;
     if (wavehdr.bits_per_samp == 16) adc->data->dataS[is] = value;
     else                             adc->data->dataS[is] = (short)(valuec*256 - 32768);
     i++;
     if (i > nData) break;
    }

 if (is < vectnData) {for (j=is; j<vectnData; j++) adc->data->dataS[j]=0;}

 fclose(file);

 return(frame);
}


/*------------------------------------------ DyReadFirstFrame ---*/

int DyReadFirstFrame (char *chosen_file, long arg)
{
 double endTime, oldstart;
 long result;
 struct FrFile *iFile;
 char dummy[LSTRING_MAX];
#ifndef NOCM
 CmConnectCondition condition;
 CmConnectCondition condition2;
#endif
 long iwait;
 int nadc;
 char *chList;
 int nBytes;
 long shm_iread;
 struct ShmInfo *shm;
 struct FrAdcData *adc;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0)
   {printf("DyReadFirstFrame> arg=%ld  InputType=%ld  previousInputType=%ld \n", arg, Dy->InputType, Dy->previousInputType);}

 /*--- If current data were taken online or are going to be taken online, reset channels list ---*/
 if (arg == INCM || Dy->previousInputType == INCM) DyXformRaz(NULL,1);

 Dy->previousInputType = Dy->InputType;
 Dy->InputType = INOFF;

 /*--- data from ascii file ---*/

 if (arg == INASCII)
    {
     if (chosen_file == NULL) return(-2);

     if (((Dy->debugType)&(1<<DBG_READ))) printf("DyReadFirstFrame> Input=ascii\n");
     DyXformNow(NULL,0);

     /*--- Get firstFrame and show its time stamp ---*/
     Dy->Frame = DyAsciiToFrame(chosen_file,10.);
     if (Dy->Frame == NULL) return(-1);

     Dy->firstTime = (double)Dy->Frame->GTimeS;

     /*--- get channels list from frame ---*/
     Dy->InputType = arg;
     DyChFill(Dy->Frame, chosen_file);   /* Get channel ascii */
    }

 /*--- data from wave audio file ---*/

 if (arg == INWAVE)
    {
     if (chosen_file == NULL) return(-2);

     if (((Dy->debugType)&(1<<DBG_READ))) printf("DyReadFirstFrame> Input=wave file\n");
     DyXformNow(NULL,0);

     /*--- Get firstFrame and show its time stamp ---*/
     Dy->Frame = DyWaveToFrame(chosen_file,1.);
     if (Dy->Frame == NULL) return(-1);

     Dy->firstTime = (double)Dy->Frame->GTimeS;

     /*--- get channels list from frame ---*/
     Dy->InputType = arg;
     DyChFill(Dy->Frame, chosen_file);   /* Get channel audio */
    }

 /*--- frame from one or several files ---*/

 if (arg == INFILE)
    {
     if (chosen_file == NULL) return(-2);

     if (((Dy->debugType)&(1<<DBG_READ))) printf("DyReadFirstFrame> Input=frames files\n");

     /*--- Get firstFrame and show its time stamp ---*/

     iFile  = FrFileINew(chosen_file);
     if (iFile == NULL)
        {sprintf(dummy,"Could not open file %s", chosen_file);
         DyInfoLine(dummy);
         Dy->firstTime = 0.;
         endTime       = 0.;
        }
     else
        {Dy->firstTime = FrFileITStart(iFile);
         endTime       = FrFileITEnd(iFile);
         FrFileIEnd(iFile);
        }

    if (Dy->startTime > endTime || Dy->startTime < Dy->firstTime)
        {
         oldstart = Dy->startTime;
         if (Dy->startTime < Dy->firstTime) Dy->startTime = Dy->firstTime;
         if (Dy->startTime > endTime)       Dy->startTime = endTime - 300;
         if (((Dy->debugType)&(1<<DBG_OTHERS)) && oldstart > 0)
            {printf("DyReadFirstFrame> Start GPS not within %s range [%f - %f].\n", chosen_file, Dy->firstTime, endTime);
             printf("DyReadFirstFrame> I update it from %f to %f (%s)\n", oldstart, Dy->startTime, FrStrGTime((int)Dy->startTime));
            }
         DyXformSetStartTime(0);
        }

     /*--- keep in memory the last file read ---*/
     strcpy(Dy->fileName,chosen_file);

     /*--- get channels list and rebuild frame from channels list ---*/
     iFile = FrFileINew(chosen_file);
     if (iFile == NULL) {sprintf(dummy,"Could not open chosen file %s", chosen_file); DyInfoLine(dummy); return(-1);}

     if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyReadFirstFrame> Getting ChannelList for startTime=%f\n", Dy->startTime);
     chList = FrFileIGetChannelList (iFile, Dy->startTime);
     if (chList == NULL) {FrFileIEnd(iFile); return(-1);}
     Dy->Frame = DyCharChListToFrame (chList);
     free(chList);
     FrFileIEnd(iFile);

     if (Dy->Frame == NULL) {sprintf(dummy,"No frame at gps=%f in file %s", Dy->startTime, chosen_file); DyInfoLine(dummy); return(-1);}
     else                   {sprintf(dummy,"Frame read successfully from file %s", chosen_file); DyInfoLine(dummy);}

     Dy->InputType = arg;
     DyChFill(Dy->Frame, chosen_file);   /* Get channels of the frame */
    }


 /*--- frame from a Cm server ---*/

#ifndef NOCM

 if (arg == INCM)
    {
/**
     if (DyCmConnect() == -1) return(-2);
**/
     DyFdRequestChannelsList (Dy->CmServer,0);
     sprintf(dummy, "Requesting channels list to %s", Dy->CmServer);
     DyInfoLine (dummy);
     Dy->Order = WAIT;
     condition = CmMessageWaitWithTimeout((double)DY_TIMEOUT2);

     if (condition == CmConnectTimeoutDetection)
       {sprintf(dummy,"Could not get channels list from %s. Request now full frame. Please, wait...", Dy->CmServer);
        DyInfoLine(dummy);
        DyFdRequestFrames(Dy->CmServer, Dy->CmName, "*", 0, 0);
        condition2 = DyCmWait(DY_TIMEOUT1, Dy->CmServer);
/**
        if (condition2 == CmConnectTimeoutDetection) {DyCmDisconnect(); return(-1);}
**/
        if (condition2 == CmConnectTimeoutDetection) return(-1);

        /*--- Take the frame received, copy it into Dy->Frame ---*/
        if (Dy->Frame) FrameFree(Dy->Frame);
        Dy->Frame = FrameCopy(inputFrame);
        DyFdRemoveFramesRequest (Dy->CmServer, Dy->CmName);
       }
     else
       {Dy->Frame = DyCharChListToFrame(Dy->chlist);}

     Dy->Order = STOP;

     /*--- if channels available in Dy->Frame, fill the channels list and update start time ---*/

     Dy->firstTime = (double)Dy->Frame->GTimeS;

     if (Dy->startTime <= 0  && Dy->Frame)
        {
         Dy->startTime = (double)Dy->Frame->GTimeS;
         DyXformSetStartTime(Dy->Frame->ULeapS);
        }

     Dy->InputType = arg;
     DyChFill(Dy->Frame, Dy->CmServer);   /* Get channels of the frame */
/*
     DyCmDisconnect();
*/
    }
#endif


 /*--- frame from the Data Distribution server ---*/

#ifndef NOCM

 if (arg == INDD)
    {
/**
     if (DyCmConnect() == -1) return(-2);
**/
     sprintf(dummy, "Requesting channels list to %s...", Dy->DdServer);
     DyInfoLine (dummy);

     result = DyDdChlistRequest (Dy->DdServer, Dy->Ddinputfile, (int)Dy->startTime);

     if (result >= 0)
        {
         Dy->firstTime = (double)Dy->Frame->GTimeS;

         if (Dy->startTime > DyGetGPSNow() - 2400)
            {Dy->startTime = Dy->startTime - 2400;
             DyXformSetStartTime(Dy->Frame->ULeapS);
            }

         if (Dy->startTime <= 0  && Dy->Frame)
            {
             Dy->startTime = (double)Dy->Frame->GTimeS;
             DyXformSetStartTime(Dy->Frame->ULeapS);
            }

         Dy->InputType = arg;
         DyChFill(Dy->Frame, Dy->DdServer);   /* Get channels of the frame */

         if (((Dy->debugType)&(1<<DBG_READ)) && Dy->Frame)
            {nadc = 0; if (Dy->Frame->rawData) {for (adc=Dy->Frame->rawData->firstAdc; adc!=NULL; adc=adc->next) nadc++;}
             printf("DyReadFirstFrame> One frame (fr=%d gps=%d nadc=%d) received from %s\n",
                     Dy->Frame->frame, Dy->Frame->GTimeS, nadc, Dy->CmServer);
            }
        }

     else
        {
         if (result == -1) sprintf(dummy, "No list received from %s", Dy->DdServer);
         if (result == -2) sprintf(dummy, "Files list received from %s", Dy->DdServer);
         DyInfoLine (dummy);
         return(result);
        }
/**
     DyCmDisconnect();
**/
    }
#endif


 /*--- frames from a shared memory ---*/

 if (arg == INSHM)
    {
     Dy->Order = WAIT;

     iwait = 0;
     while ((shm = FdShmOpen(Dy->shmName, Dy->CmName)) == NULL  &&  Dy->Order != STOP)
        {DyXformCheckGUI();
         if (iwait%DY_TIMEOUT2 == 0) {sprintf(dummy, "Could not open shared memory %s since %ld sec", Dy->shmName, iwait); DyInfoLine(dummy);}
         sleep(1);
         iwait++;
        }
     if (Dy->Order == STOP) return(-1);
     else                  {sprintf(dummy, "Opened shared memory %s", Dy->shmName); DyInfoLine (dummy); sleep(1); FdShmCsmIni(shm, &shm_iread);}
     Dy->Order = WAIT;

     while ((Dy->Frame = FdShmRead(shm, &(shm_iread), -1, &nBytes)) == NULL  &&  Dy->Order != STOP)
        {DyXformCheckGUI();
         sprintf(dummy, "Frame=NULL. Reading frame again from shared memory %s (iread=%ld iwrite=%ld)", Dy->shmName, shm_iread, shm->iwrite);
         DyInfoLine (dummy);
         sleep(1);
        }
     if (Dy->Order == STOP) {FdShmClose(shm, Dy->CmName); return(-1);}
     FdShmClose(shm, Dy->CmName);

     Dy->firstTime = (double)Dy->Frame->GTimeS;

     if (((Dy->debugType)&(1<<DBG_READ)))
        {printf("DyReadFirstFrame> One frame (fr=%d gps=%d) read in shared memory %s\n", Dy->Frame->frame, Dy->Frame->GTimeS, Dy->shmName);}

     Dy->InputType = arg;
     DyChFill(Dy->Frame, Dy->shmName);   /* Get channels of the frame */
    }


 /*--- Keep in previousInputType the current InputType ---*/

 Dy->previousInputType = Dy->InputType;

 /*--- Initialize GPS time counter ---*/

 DyXformUpdateCounters (Dy->firstTime, Dy->Frame->ULeapS);

 return(0);
}



/*------------------------------------------- DyAddFile ---*/

void DyAddFile (char *result, int arg)
{
 int i, nf, len;
 char *str, *chaine;
 char **filefound;

 filefound = (char **)calloc(NFILE_MAX,sizeof(char *));
 nf = 0;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
    {printf("DyAddFile> Adding following files: %s \n", result);}

 chaine = (char *)malloc(strlen(result)+1);
 strcpy(chaine, result);

 str = strtok(chaine," ");

 while (str != NULL)
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyAddFile> Looking for %s .. nf=%d\n", str, nf);
     if (is_a_file(str) != 0)
       {len = strlen(str);
        filefound[nf] = (char *)calloc(len+1, sizeof(char));
        memcpy(filefound[nf], str, len);
        filefound[nf][len] = '\0';
        nf++;
       }
     str = strtok(NULL," ");
    }

 free(chaine);
 for (i=0; i<nf; i++) DyReadFirstFrame(filefound[i], arg);

 return;
}


/*------------------------------------------- DyBaseName ---*/

char *DyBaseName (char *allname)
{
 long i,i0;
 const char *filename;

 if (!allname) return(NULL);

 i0 = -1;
 for (i=strlen(allname)-1; i>= 0; i--) {if (allname[i] == '/') {i0 = i; break;}}
 filename = &allname[i0+1];

 return((char *)filename);
}


/*------------------------------------------- DyDirName ---*/

char *DyDirName (char *dirname, char *allname)
{
 long i,i0;

 DyDebugPrint(__FUNCTION__,0);

 if (!allname) return(NULL);

 if (dirname) free(dirname);
 dirname = (char *)calloc(strlen(allname)+1,sizeof(char));
 
 i0 = 0;
 for (i=0; i<strlen(allname); i++)
     {if (allname[i] == '/') i0 = i;}
 
 if (i0 > 0)
    {dirname = (char *)calloc(i0+1,sizeof(char));
     strncpy(dirname, allname, i0);
     dirname[i0] = '\0';
    }
 else
    {dirname = (char *)calloc(3,sizeof(char));
     strncpy(dirname,"./",2);
     dirname[2] = '\0';
    }
    
 return(dirname);
}


/*------------------------------------------- is_a_file ---*/

long is_a_file (char *str)
{
 DIR *fd;
 FILE *fp;

 fd = opendir(str);
 if (fd)  {closedir(fd); return 0;}

 fp = fopen(str,"r");
 if (fp)  {fclose(fp); return 1;}

 return 0;
}


/*------------------------------------------- is_a_dir ---*/

long is_a_dir (char *str)
{
 DIR *fd;

 fd = opendir(str);
 if (fd)  {closedir(fd); return 1;}

 return 0;
}



/*------------------------------------------- DyAddShm ---*/

void DyAddShm(char *shmname)
{
 DyDebugPrint(__FUNCTION__,0);

 if (!shmname) return;

 DyXformClosePanel(getshm->close,GETSHM);
 strcpy(Dy->shmName,shmname);
 DyReadFirstFrame (NULL, INSHM);

 return;
}


/*----------------------------------------- DyGetShmList ---*/

int DyGetShmList()
{
 int nshm;
 FILE *fp;
 char buff[256], *shmid, *str2;
 struct ShmInfo *shm;

 DyDebugPrint(__FUNCTION__,0);

 nshm = 0;

 fp = popen("ipcs -m","r");
 if (fp)
    {fgets(buff,256,fp);
     while (!feof(fp))
       {if (strncmp(buff,"0x",2) == 0)
           {shmid = strstr(buff," ") + 1; str2 = strstr(shmid," ");
            *str2='\0';
            shm = FdShmOpen(shmid,Dy->CmName);
            if (shm) {strcpy(Dy->shmlist[nshm],shm->producerName); nshm++; FdShmClose(shm,Dy->CmName);}
           }
        fgets(buff,256,fp);
       }
     pclose(fp);
    }
 Dy->nshm = nshm;

 DySortShmList();

 return(nshm);
}



/**************************************************/
/*                                                */
/* File: DyCm.c                                   */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#ifndef NOCM

/*--------------------------------------------------- DyChListHandler ------------*/
CmMessageStatus DyChListHandler (CmMessage message, char* sender, char* serverName)
{
 int gps;
 int nadc, nsms, nproc, nsim, length, ll;
 char *listadc, *listsms, *listproc, *listsim;
 char *keyadc, *keysms, *keyproc, *keysim;
 char *str, *buf, name[LSTRING_MAX];
 double rate;

 DyDebugPrint(__FUNCTION__,0);

 gps     = CmMessageGetInt(message);

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
   {printf("DyChListHandler> Received the list of channel provided by %s for gtime=%d \n", sender, gps);}

 /*--- Get the list of ADC channels ---*/
 keyadc  = CmMessageGetText(message);
 nadc    = CmMessageGetInt(message);
 listadc = CmMessageGetText(message);
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyChListHandler> nadc=%d strlen(listadc)=%d\n",nadc, (int)strlen(listadc));}

 /*--- Get the list of PROC channels ---*/
 keyproc  = CmMessageGetText(message);
 nproc    = CmMessageGetInt(message);
 listproc = CmMessageGetText(message);
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyChListHandler> nproc=%d strlen(listproc)=%d\n",nproc, (int)strlen(listproc));}

 /*--- Get the list of SER channels ---*/
 keysms  = CmMessageGetText(message);
 nsms    = CmMessageGetInt(message);
 listsms = CmMessageGetText(message);
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyChListHandler> nsms=%d strlen(listsms)=%d\n",nsms, (int)strlen(listsms));}

 /*--- Get the list of SIM channels ---*/
 keysim  = CmMessageGetText(message);
 nsim    = CmMessageGetInt(message);
 listsim = CmMessageGetText(message);
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyChListHandler> nsim=%d strlen(listsim)=%d\n",nsim, (int)strlen(listsim));}

 /*--- Prepare a string which will contain the full list of channels ---*/
 if (Dy->chlist != NULL) {free(Dy->chlist); Dy->chlist = NULL;}
 length = strlen(listadc) + strlen(listproc) + strlen(listsms) + strlen(listsim) + 256;
 Dy->chlist = (char *)malloc(2*length*sizeof(char));

 /*--- Init the string like the one made by FrFileIGetChannelsList ---*/
 buf = Dy->chlist;
 buf += sprintf(buf,"FIRSTFRAME %d 37 1 0 0 \n", gps);

 /*--- Put in the string the ADC channel names and sample rates ---*/
 str = listadc-1;
 ll = strlen(listadc);
 while (str != NULL && ll > 0)
   {sscanf(str+1,"%s %lf",name, &rate);
    buf += sprintf(buf,"ADC %s\t%g 1. 0. \n", name, rate);
    str = strstr(str+1,"\n");
   }

 /*--- Put in the string the PROC channel names and sample rates ---*/
 str = listproc-1;
 ll = strlen(listproc);
 while (str != NULL && ll > 0)
   {sscanf(str+1,"%s %lf",name, &rate);
    buf += sprintf(buf,"PROC %s\t%g 1. 0. \n", name, rate);
    str = strstr(str+1,"\n");
   }

 /*--- Put in the string the SER channel names and sample rates ---*/
 str = listsms-1;
 ll = strlen(listsms);
 while (str != NULL && ll > 0)
   {sscanf(str+1,"%s %lf",name, &rate);
    buf += sprintf(buf,"SER %s\t%g 1. 0. \n", name, rate);
    str = strstr(str+1,"\n");
   }

 /*--- Put in the string the SIM channel names and sample rates ---*/
 str = listsim-1;
 ll = strlen(listsim);
 while (str != NULL && ll > 0)
   {sscanf(str+1,"%s %lf",name, &rate);
    buf += sprintf(buf,"SIM %s\t%g 1. 0. \n", name, rate);
    str = strstr(str+1,"\n");
   }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1)
   {printf("DyChListHandler> Dy->chlist = %s \n", Dy->chlist);}

 return(CmMessageBreak);
}

#endif

#ifndef NOCM

/*-------------------------------------------------- DyCmFrHandler ----*/

CmMessageStatus DyCmFrHandler (CmMessage message, char *sender, char *serverName)
{
 int run, fnum, gtimeS, gtimeN, trig, nBytes;
 char *buf, *name;
 double tSpan;
 double timeNeeded;

 DyDebugPrint(__FUNCTION__,0);

 if (message == NULL) 
    {
     printf("DyCmFrHandler> Cm message received is null\n");
     return(CmMessageOk);
    }

// DyDebugTimeNeeded(__FUNCTION__,"start");

 /*----- decode Cm message -----*/

 name   = CmMessageGetText  (message);
 run    = CmMessageGetInt   (message);
 fnum   = CmMessageGetInt   (message);
 gtimeS = CmMessageGetInt   (message);
 gtimeN = CmMessageGetInt   (message);
 trig   = CmMessageGetInt   (message);
 nBytes = CmMessageGetInt   (message);
 buf    = CmMessageGetArray (message, CmMessageChar, &nBytes);

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0) 
   {printf("\033[1m DyCmFrHandler> Received from %s the frame %s gtimeS=%d gtimeN=%d \033[0m \n", sender, name, gtimeS, gtimeN);}

 /*--- Put the last frame received in inputFrame ---*/

 if (inputFrame) {FrameFree(inputFrame); inputFrame=NULL;}
 inputFrame = FrameReadFromBuf (buf, nBytes, -1);
 inputFrameSize = nBytes;

 /*--- If frame is null, wait for an other one ---*/

 if (inputFrame == NULL)
   {
    printf("DyCmFrHandler> Could not read the frame received from %s\n", sender);
    inputFrameSize = 0;
    return(CmMessageOk);
   }

 /*--- if received a no_frame, it means that we stop ---*/

 if (strcmp(inputFrame->name,"NO FRAME") == 0)
   {
    FrameFree(inputFrame);
    inputFrame = NULL;
    Dy->Order = STOP;
    return(CmMessageBreak);
   }

 /*--- Put buf in list of frames in memory ---*/

 if (Dy->Order == CONTINUE) 
   {
    tSpan = FrFileIInMemoryAddBuffer(Dy->CmMffile, buf, nBytes, Dy->CmMftspan);
    if (tSpan <= 0)
      {printf("DyCmFrHandler> Could not put in buffer the frame %d from %s", gtimeS, sender);}
    else if (((Dy->debugType)&(1<<DBG_READ))) 
      {printf("\033[1m DyCmFrHandler> Put in buffer the frame %d (%.2fkB) received from %s : nFrame=%g/%g \033[0m\n",
               gtimeS, (double)nBytes/1024., sender, tSpan, Dy->CmMftspan);
      }
    /*--- write the frame on disk if requested ---*/
    if (Dy->output_flag == ON) DyOutputWrite (inputFrame);
   }

// DyDebugTimeNeeded(__FUNCTION__,"stop");

 return(CmMessageBreak);
}

#endif

#ifndef NOCM

/*------------------------------------------ DyCmWait ---*/

CmConnectCondition DyCmWait (int timeout, char *servername)
{
 int timeout2, iwait;
 CmConnectCondition condition;
 char dummy[LSTRING_MAX];

 if (timeout == 0) timeout = 99999999;

 if (timeout < 11) timeout2 = timeout / 2;
 else              timeout2 = 10;

 iwait = 1;
 Dy->Order = WAIT;
 while (Dy->Order != STOP)
    {
     condition = CmMessageWaitWithTimeout(1.);
     if (condition == CmConnectBreakDetection) break;
     if (iwait%timeout2 == 0) printf("DyCmWait> No answer from %s since %d seconds (timeout=%ds) \n", servername, iwait, timeout);
     iwait++;
     if (iwait > timeout) Dy->Order = STOP;
     if (timeout>5 && iwait == 5)
        {sprintf(dummy,"Push \042stop\042 button to stop waiting for an answer from %s", servername);
         DyXformShowOneliner(dummy,1);
        }
     DyXformCheckGUI();
    }

 DyXformHideOneliner();

 return(condition);
}

#endif


/*------------------------------------------ DyCmConnect ---*/

int DyCmConnect ()
{
 char localName[LSTRING_MAX], localName2[LSTRING_MAX];
 char hostname[LSTRING_MAX], *str;

#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

 /*--- Set the CmDomain to getenv("CMDOMAIN") ---*/
 DyCmSetCmDomain(NULL);

 /*--- Use the datadisplay name as Cm name ---*/
 strcpy(Dy->CmName, Dy->DyName);

 /*--- If Cm connection failed, put info to user ---*/

 if (!CmMessageOpenServer(Dy->CmName))
     {printf("DyCmConnect> Could not declare myself with Cm Name %s \n", Dy->CmName);
      DyInfoLine("This Cm Name may be already used.");
      return(-1);
     }

 /*--- If success, keep in Dy->CmName the Cm name used and put connect flag ON ---*/

 printf("DyCmConnect> My Cm Name is %s. Now connected.\n", Dy->CmName);
 Dy->CmConnect = ON;

 /*--- Install reception handlers ---*/

 CmMessageInstallHandler (DyChListHandler, CMTYPE_CHLIST);
 CmMessageInstallHandler (DyCmFrHandler, CMTYPE_FDFRAME);
 CmMessageInstallHandler (DyCmInfoHandler, CMTYPE_DYINFO);
 CmMessageInstallHandler (DyDdHandlerFFL,DDFFLDIR);
 CmMessageInstallHandler (DyNamesHandler,CMTYPE_NSNAMES);
 CmMessageInstallHandler (DyCmQuitHandler, CMTYPE_DYQUIT);

#endif

 return(0);
}


/*------------------------------------------ DyCmDisconnect ---*/

void DyCmDisconnect()
{
 char clientName[LSTRING_MAX];

#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

/**
 if (Dy->CmConnect == OFF)  return;
**/

 /*--- Uninstall reception handlers ---*/

 CmMessageUninstallHandler (CMTYPE_CHLIST);
 CmMessageUninstallHandler (CMTYPE_FDFRAME);
 CmMessageUninstallHandler (CMTYPE_DYINFO);
 CmMessageUninstallHandler (CMTYPE_DYQUIT);
 CmMessageUninstallHandler (DDFFLDIR);
 CmMessageUninstallHandler (CMTYPE_NSNAMES);

 /*--- Remove my Cm name from the Cm server ---*/

 strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
 CmMessageCloseServer(clientName);
 printf("DyCmDisconnect> My Cm Name was %s. Now Disconnected.\n", clientName);

 Dy->CmConnect = OFF;

#endif

 return;
}


/*------------------------------------------ DyUpdateCmList ---*/

void DyUpdateCmList()
{
#ifndef NOCM

 CmMessage message;
 CmConnectCondition condition;
 CmConnect server;
 char* destination, clientName[LSTRING_MAX];
 int connected=OFF;

 DyDebugPrint(__FUNCTION__,0);

/* ICI */
 /*--- Does not work, because setenv CMDOMAIN done after the start of dataDisplay is not seen here ---*/
 DyCmSetCmDomain(NULL);

/**
 if (Dy->CmConnect == OFF)
    {if (!CmMessageOpenMultipleServer("DyCmNamesAsker"))
            {printf("DyUpdateCmList> Cannot declare myself with Cm Name DyCmNamesAsker_1 \n");
             return;
            }
     strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
     printf("DyUpdateCmList> Now connected. My temporary Cm Name is %s \n",clientName);
     connected = ON;
    }

 CmMessageInstallHandler(DyNamesHandler,"NSNames");
**/

 server      = CmConnectGetNameServer();
 destination = CmConnectGetName(server);

 message = CmMessageNew();
 CmMessageSetType(message, "NSGetNames");
 CmMessageSend(message, destination);
 CmMessageDelete(message);

 condition = DyCmWait (DY_TIMEOUT2, destination);

 if (condition == CmConnectTimeoutDetection)
    {DyInfoLine("Timeout reached. I could not update the list of Cm servers.");}

/**
 CmMessageUninstallHandler("NSNames");

 if (connected == ON)
    {strcpy(clientName,CmConnectGetName(CmConnectWhoAmI()));
     CmMessageCloseServer(clientName);
     printf("DyUpdateCmList> my temporary Cm Name was %s. Now Disconnected\n",clientName);
    }
**/

#endif

 return;
}


/*----------------------------------- DyCmSetCmDomain ---*/

void DyCmSetCmDomain (char* cmdomain)
{
 DyDebugPrint(__FUNCTION__,0);

 if (cmdomain == NULL) strcpy(Dy->CmDomain,getenv("CMDOMAIN"));
 else                  strcpy(Dy->CmDomain, cmdomain);

 CmConnectSetDomain(Dy->CmDomain);
 CmNameServerGetInfos();
 
 // unsetenv("CMDOMAIN"); setenv("CMDOMAIN", Dy->CmDomain, 1);

 return;
}


#ifndef NOCM

/*----------------------------------- DyNamesHandler ---*/

CmMessageStatus DyNamesHandler (CmMessage message, char* sender, char* serverName)
{
 const char *name;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->DdNS == ON)
    {
     Dy->ndd = 0;
     while (CmMessageGetItemType (message) == CmMessageItemText)
        {name = CmMessageGetText (message);
         if (strncmp(name,DDSERVER,10) == 0) {strcpy(Dy->ddlist[Dy->ndd], name); (Dy->ndd)++;}
        }
     DySortDdList();
     DyXformDdFillBrowser();
    }

 else
    {
     Dy->ncm = 0;
     while (CmMessageGetItemType (message) == CmMessageItemText)
        {name = CmMessageGetText (message);
         if (strcmp(name,"<unknown>") == 0) continue;
         if (strcmp(name,serverName) == 0)  continue;

         if (Dy->CmFullList == ON)
            {strcpy(Dy->cmlist[Dy->ncm], name); (Dy->ncm)++;}
         else
           {if (strstr(name, "Dy") != NULL)         {strcpy(Dy->cmlist[Dy->ncm], name); (Dy->ncm)++;}
            else if (strstr(name, "Users") != NULL) {strcpy(Dy->cmlist[Dy->ncm], name); (Dy->ncm)++;}
            else if (strstr(name, "Shift") != NULL) {strcpy(Dy->cmlist[Dy->ncm], name); (Dy->ncm)++;}
           }
        }
     DySortCmList();
     DyXformCmFillBrowser();
    }

 return (CmMessageBreak);
}


/*----------------------------------- DyCmInfoHandler ---*/
/* handler Cm to receive an alarm message sent to the dataDisplay user, or to trigger a check
or to produce a jpg picture of current canvas or to stop the dataDisplay */

CmMessageStatus DyCmInfoHandler (CmMessage message, char* sender, char* serverName)
{
 char dummy[LSTRING_MAX];
 char *infotype;
 int alarm=0;

 DyDebugPrint(__FUNCTION__,0);

 infotype = CmMessageGetText (message);
 alarm    = CmMessageGetInt(message);

 if (infotype == NULL) return(CmMessageOk);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) 
   {printf("DyCmInfoHandler> Received message from %s : %s \n", sender, infotype);}

 if (strstr(infotype,"assword") && Dy->Ddpasswd) {free(Dy->Ddpasswd); Dy->Ddpasswd = NULL;}

 if (strcmp(infotype,"quit") == 0)        DyXformQuit(NULL,10);
 else if (strcmp(infotype,"stop") == 0)   Dy->Order = STOP;
 else if (strcmp(infotype,"checks") == 0) DyXformChecks(NULL,0);
 else if (strcmp(infotype,"jpeg") == 0)   {sprintf(dummy, "%s/jpeg/%s.jpg", DYLOG, Dy->curConfig); DySavePlot(dummy);}
 else                                     DyInfoPanel(infotype, alarm);

 return (CmMessageOk);
}


/*----------------------------------- DyCmQuitHandler ---*/
/* handler Cm to receive a CfgExit message that stop the dataDisplay */

CmMessageStatus DyCmQuitHandler (CmMessage message, char* sender, char* serverName)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyCmQuitHandler> Received Cm message CfgExit from %s \n", sender);
 DyInfoLine(dummy);

 DyXformQuit(NULL,10);

 return (CmMessageOk);
}

#endif


/**************************************************/
/*                                                */
/* File: DyFd.c                                   */
/* COPY OF SOME FD LIBRARY FUNCTIONS (v4r13)      */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#ifndef NOCM

/*---------------------------------------------- DyFdRequestChannelsList ---------*/

int DyFdRequestChannelsList (char *source, int gps)
{
 CmMessage message;
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();
 CmMessagePutInt(message, gps);

 CmMessageSetType (message, CMTYPE_CHLIST_REQUEST);

 ok = CmMessageSend(message, source);

 CmMessageDelete(message);

 return(ok);
}


/*------------------------------------------------------ DyFdRequestFrames --------*/

int DyFdRequestFrames (char *source, char* destination, char *tag, int queueSize, int nRetry)
{
 int ok;
 CmMessage message;

 DyDebugPrint(__FUNCTION__,0);

 message= CmMessageNew();

 CmMessagePutText(message, destination);
 CmMessagePutText(message, tag);
 CmMessagePutInt (message, queueSize);
 CmMessagePutInt (message, nRetry);
 CmMessagePutInt (message, 0);  /* no checksum computation for the frames that will be sent */

 CmMessageSetType (message, CMTYPE_FDADDFRAME);

 ok = CmMessageSend (message, source);

 CmMessageDelete (message);

 if (((Dy->debugType)&(1<<DBG_READ)))
   {printf("DyFdRequestFrames> Asked to %s to add %s in the frames receivers list\n", destination, source);}

 return(ok);
}


/*------------------------------------------- DyFdRemoveFramesRequest -------------*/
int DyFdRemoveFramesRequest (char* dest, char* serverName)
{
 int ok;
 CmMessage message;

 DyDebugPrint(__FUNCTION__,0);

 message= CmMessageNew();
 CmMessagePutText (message, serverName);
 CmMessageSetType (message, CMTYPE_FDREMOVEFRAME);

 ok = CmMessageSend (message, dest);

 CmMessageDelete (message);

 /*--- Make sure that the connection with CmServer is terminated (CmMessageCloseServer is not sufficient) ---*/
 sleep(2);
 CmConnectKill(CmConnectGetReference(dest));

 if (((Dy->debugType)&(1<<DBG_READ)))
   {printf("DyFdRemoveFramesRequest> Asked to %s to remove %s from the frames receivers list\n", dest, serverName);}

 return(ok);
}

#endif


/*************************** FdShm.c *****************************/
/*                                                               */
/*               FRAMES IN SHARED MEMORY                         */
/*                     LIBRARY                                   */
/* MODULE  : FdShm.c                                             */
/* AUTHOR  : D. Verkindt, LAPP (verkindt@lapp.in2p3.fr)          */
/* VERSION : v5r0p3                                              */
/* DATE: 27 Jan 2006                                             */
/*                                                               */
/*   This library allows a server (for instance the Main Frame   */
/*   Builder producer) to create a shared memory segment and     */
/*   to put frames into this shared memory. It allows also       */
/*   several consumers (for instance the Main Frame Builder      */
/*   consumers) to read those frames in shared memory.           */
/*                                                               */
/************************** FdShm.c ******************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include "dataDisplay.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif


/*------------------------------------------ FdShmInfoFree --------*/

void FdShmInfoFree (struct ShmInfo *shm)
{
 if (shm == NULL) return;
 if (shm->frameSh)    free(shm->frameSh);
 if (shm->frameSize)  free(shm->frameSize);
 free(shm);
}


/*----------------------------------------- FdShmGetIwrite ----------*/

long FdShmGetIwrite (struct ShmInfo *shm)
{
 return(shm->iwrite);
}


/*----------------------------------------- FdShmGetnFrameMax ----------*/

long FdShmGetnFrameMax (struct ShmInfo *shm)
{
 long nFrameMax;
 if (shm->dynamic > 0) nFrameMax = shm->iwriteMax;
 else                  nFrameMax = shm->nFrameMax;
 return(nFrameMax);
}


/*----------------------------------------- FdShmSetDynamic ----------*/

void FdShmSetDynamic (struct ShmInfo *shm, long dynamic)
{
 shm->dynamic = dynamic;
 return;
}


/*------------------------------------------ FdShmFree --------*/

void FdShmFree (struct ShmInfo *shm, char *name)
{
 key_t key;
 int shmid;
 
 if (name == NULL  ||  shm == NULL) return;

 key = FdShmMakeKey(name);

 if ((shmid = shmget(key, 0, 0)) != -1) 
    {shmctl(shm->shmid, IPC_RMID, 0);
     if (((Dy->debugType)&(1<<DBG_READ))) printf("  FdShmFree> %s Shared memory segment marked for deletion\n", name);
    }
 else
    {if (((Dy->debugType)&(1<<DBG_READ))) printf("  FdShmFree> Consumer %s detached\n", name);
     shm->nConsumer--;
    }

 shmdt((void *)shm);

 return;
}


/*------------------------------------------ FdShmClose --------*/
/* only for consumers */

void FdShmClose (struct ShmInfo *shm, char *name)
{
 if (name == NULL  ||  shm == NULL) return;

 if (strcmp(shm->producerName, name) == 0) 
    {printf("FdShmClose> has no effect for a producer (%s). Use FdShmFree.\n", name);
     return;
    }
 else
    {if (((Dy->debugType)&(1<<DBG_READ))) printf("  FdShmClose> Consumer %s detached from shared memory\n", name);
     shm->nConsumer--;
    }

 shmdt((void *)shm);

 return;
}


/*------------------------------------------- FdShmOpen -------*/
/* this function is only for a consumer */

struct ShmInfo *FdShmOpen (char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 key_t key;
 int shmid;

 shmid = atoi(idName);

 if (shmid == 0)
    { 
     key = FdShmMakeKey(idName);
     if ((shmid = shmget(key, 0, 0)) == -1)  return(NULL);
     if (((Dy->debugType)&(1<<DBG_READ))) printf("FdShmOpen> Connected to shm segment: key=%d  shmid=%d\n", key, shmid);
    }


 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((segptr = shmat(shmid, 0, 0)) == (void *)-1)
    {printf("FdShmOpen> Could not attach Shm segment shmid=%d\n", shmid);
     return(NULL);
    }

 if (segptr == NULL)
    {printf("FdShmOpen> shm segptr is NULL\n");
     return(NULL);
    }

 temp = (struct ShmInfo *)segptr;

 if (((Dy->debugType)&(1<<DBG_READ)))
    {printf("FdShmOpen> segptr=%lx(%ld)  shm=%lx(%ld) (shmid=%d)\n",
         (long)temp->segptr, (long)temp->segptr, (long)temp, (long)temp, shmid);
    }

 temp->nConsumer++;

 if (((Dy->debugType)&(1<<DBG_READ))) 
    {printf("FdShmOpen> New consumer %s (%ld) connected to shm %s\n", name, temp->nConsumer, temp->producerName);}

 return(temp);
}

/*------------------------------------------- FdShmMakeKey -------*/

int FdShmMakeKey (char *name)
{
 int i, length, key, val, nbytes;
 
 length = strlen(name);

 key = 0;
 for (i=0; i<length; i=i+4)
     {val = 0;
      if (length-i < 4) nbytes=length-i;
      else              nbytes = 4;
      memcpy(&val, &name[i], nbytes);
      key += val;
      if (key < 0) key += INT_MAX;
     }

 return(key);
}


/*------------------------------------------- FdShmCsmIni -------*/

long FdShmCsmIni (struct ShmInfo *temp, long *shmiread)
{
 if (temp) *shmiread = temp->iwrite - 1;

 return(*shmiread);
}


/*------------------------------------------- FdShmRead -------------*/

struct FrameH *FdShmRead (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes)
{
 struct FrameH *frame;
 long address, iread, iwrite;

 if (shm == NULL) {*nBytes = -1; return((struct FrameH*)NULL);}

 if (shm->dynamic > 0)
    {frame = FdShmReadDynamic (shm, shmiread, compress, nBytes);
     return(frame);
    }

 iwrite = shm->iwrite;

 /*--- don't try to read a frame currently written ---*/
 if (*shmiread >= iwrite)  {*nBytes = -2; return((struct FrameH*)NULL);}

 /*--- if too late with respect to producer, go behind it (skipping nFrameMax frames) ---*/
 if (*shmiread <= iwrite - shm->nFrameMax)
    {
     printf("FdShmRead> late in reading shm (%ld/%ld). Skipping %ld frames, I go to iread=%ld\n",
             *shmiread, iwrite, iwrite-*shmiread, iwrite-1);
     *shmiread = iwrite - 1;
    }

 /*--- ok, read the frame in share memory ---*/

 iread = (*shmiread) % shm->nFrameMax;

 *nBytes = (int)(shm->frameSize[iread]);
 address = shm->frameSh[iread] + (long) shm - (long) shm->segptr;

 if (*nBytes <= 0)
    {
     *nBytes = -3;
     printf("FdShmRead> nBytes <=0\n");
     return((struct FrameH*)NULL);
    }
 if (address == (long)NULL)
    {*nBytes = -4;
     printf("FdShmRead> address=NULL\n");
     return((struct FrameH*)NULL);
    }

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL)
    {printf("FdShmRead> Not been able to read frame (%d bytes) at adress %ld+%ld (%ld)  (iread=%ld iwrite=%ld) (compress=%ld)\n",
            *nBytes, (long)shm->segptr, shm->frameSh[iread], address, *shmiread, iwrite, compress);
     *nBytes = 0;
    }

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug>0)
    {
     if (frame)
        {printf("      FdShmRead> frameGPS=%d  frameSize=%d  iwrite=%ld  iread=%ld  time=%ld\n",
                        frame->GTimeS,  *nBytes, shm->iwrite, *shmiread, (long)time(NULL));
        }
     else
        {printf("      FdShmRead> frame=NULL frameSize=%d  iwrite=%ld  iread=%ld  time=%ld\n",
                        *nBytes, shm->iwrite, *shmiread, (long)time(NULL));
        }
    }

 (*shmiread)++;

 return(frame);
}


/*------------------------------------------- FdShmReadDynamic  -------------*/

struct FrameH *FdShmReadDynamic (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes)
{
 struct FrameH *frame;
 long address, iwrite, iwriteMod, iwriteMax;
 long writeAddress, readAddress, writeSize;
 static long ireadMod=-1;
 long segEnd;

 if (shm == NULL) {*nBytes = -1; return((struct FrameH *)NULL);}

 iwrite    = shm->iwrite;
 iwriteMod = shm->iwriteMod;
 iwriteMax = shm->iwriteMax;

 if (*shmiread >= iwrite) {*nBytes = -2; return((struct FrameH *)NULL);}

 segEnd  = shm->frameSh[0] + shm->size - shm->statusSize;

 if (ireadMod == -1)
    {if (iwriteMod > 0) ireadMod = iwriteMod - 1;
     else               ireadMod = iwriteMax;
     *shmiread = iwrite - 1;
    }

 if (shm->frameSh[ireadMod] == 0) sleep(1);

 if (shm->frameSh[ireadMod] > segEnd-shm->frameSizeMax || shm->frameSh[ireadMod] == 0) ireadMod = 0;

 writeAddress = shm->frameSh[iwriteMod];
 writeSize    = shm->frameSize[iwriteMod];
 readAddress  = shm->frameSh[ireadMod];

 if ((writeAddress+writeSize >= readAddress) && (readAddress > writeAddress))
    {
     printf("FdShmRead> late in reading shm (iread=%ld iwrite=%ld). Skeeping %ld frames, I go to iread=%ld "
            " (readAddress[%ld]-writeAddress[%ld]=%lx-%lx=%ld  writeSize=%ld)\n",
            *shmiread, iwrite, iwrite-*shmiread, iwrite-1, ireadMod, iwriteMod,
            readAddress, writeAddress, readAddress-writeAddress, writeSize);
     *shmiread = iwrite - 1;
     if (iwriteMod > 0) ireadMod = iwriteMod - 1;
     else               ireadMod = iwriteMax;
    }

 *nBytes = shm->frameSize[ireadMod];
 address = shm->frameSh[ireadMod] + (long) shm - (long) shm->segptr;

 if (*nBytes <= 0)
    {*nBytes = -3;
     printf("FdShmRead> nBytes <=0\n");
     return(NULL);
    }
 if (address == (long)NULL)
    {*nBytes = -4;
     printf("FdShmRead> address=NULL\n");
     return(NULL);
    }

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL)
    {printf("FdShmRead> Not been able to read frame (%d bytes) at adress %ld+%ld (%ld)\n"
            "  (iread=%ld iwrite=%ld ireadMod=%ld iwriteMod=%ld iwriteMax=%ld) (compress=%ld)\n",
             *nBytes, (long)shm->segptr, shm->frameSh[ireadMod], address,
             *shmiread, iwrite, ireadMod, iwriteMod, iwriteMax, compress);
     *nBytes = 0;
    }

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug>0)
    {
     if (frame)
        {printf("      FdShmReadDynamic> frameGPS=%d  frameSize=%d  iwrite=%ld  iread=%ld  iwriteMod=%ld  ireadMod=%ld  iwriteMax=%ld  time=%ld\n",
                        frame->GTimeS,  *nBytes, iwrite, *shmiread, iwriteMod, ireadMod, iwriteMax, (long)time(NULL));
        }
     else
        {printf("      FdShmReadDynamic> frame=NULL frameSize=%d  iwrite=%ld  iread=%ld  iwriteMod=%ld  ireadMod=%ld  iwriteMax=%ld  time=%ld\n",
                         *nBytes, iwrite, *shmiread, iwriteMod, ireadMod, iwriteMax, (long)time(NULL));
        }
    }

 (*shmiread)++;
 ireadMod++;

 return(frame);
}

/**************************************************/
/*                                                */
/* File: DyDd.c                                   */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*------------------------------------------- DyDdPauseRequest ---*/

void DyDdPauseRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256];

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();
 CmMessagePutText (message,"PAUSE");
 CmMessageSetType (message,DATAREQUEST);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 CmMessageSend (message,DsSender);
 CmMessageDelete (message);

#endif

 return;
}


/*------------------------------------------- DyDdContinueRequest ---*/

void DyDdContinueRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256];

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();
 CmMessagePutText (message,"CONTINUE");
 CmMessageSetType (message,DATAREQUEST);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 CmMessageSend (message,DsSender);
 CmMessageDelete (message);

#endif

 return;
}

/*------------------------------------------- DyDdFFLRequest ---*/

long DyDdFFLRequest (char *DdServer)
{
#ifndef NOCM

 CmMessage message;
 int answer;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Ddpasswd == NULL)
    {answer = DyXformGetPasswd();
     if (answer == 0) return(0);
    }

/**
 if (DyCmConnect() == -1) return(-1);

 CmMessageInstallHandler(DyDdHandlerFFL,DDFFLDIR);
**/

 message = CmMessageNew();
 CmMessagePutText (message,DDFFLDIR);
 CmMessagePutText (message,Dy->Ddpasswd);
 CmMessagePutText (message," ");
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);
 CmMessageDelete  (message);

 DyCmWait(DY_TIMEOUT1, DdServer);

/**
 CmMessageUninstallHandler(DDFFLDIR);
 DyCmDisconnect();
**/

 if (Dy->Order == STOP) return(-1);
 Dy->Order = STOP;

#endif

 return(0);
}


/*------------------------------------------- DyDdChlistRequest ---*/

long DyDdChlistRequest (char *DdServer, char *inputfile, int gpsStart)
{
#ifndef NOCM

 CmMessage message;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 CmMessageInstallHandler(DyDdHandlerChlist,DDCHLIST);
 CmMessageInstallHandler(DyDdHandlerFilelist,DDFILELIST);

 message = CmMessageNew();

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
    {printf("    DyDdChlistRequest> Sending Channels list request:  requester=%s  name=%s  inputfile=%s  gpsStart=%d \n",
             Dy->CmName, DDCHLIST, inputfile, gpsStart);
    }

 CmMessagePutText (message,DDCHLIST);
 CmMessagePutText (message,inputfile);
 if (Dy->Ddpasswd) CmMessagePutText (message, Dy->Ddpasswd);
 else              CmMessagePutText (message,"nopasswd");
 CmMessagePutInt  (message,gpsStart);
 CmMessagePutInt  (message,0);
 CmMessagePutInt  (message,0);

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);
 CmMessageDelete  (message);

 sprintf(dummy,"Requested files list or channels list to %s", Dy->DdServer);
 DyInfoLine (dummy);

 Dy->ddfilelist_flag = OFF;

 DyCmWait(DY_TIMEOUT1, DdServer);

 CmMessageUninstallHandler(DDCHLIST);
 CmMessageUninstallHandler(DDFILELIST);

 if (Dy->Order == STOP) return(-1);
 Dy->Order = STOP;

 if (Dy->Ddchlist != NULL)
    {
     if (Dy->Frame) FrameFree(Dy->Frame);
     Dy->Frame = DyCharChListToFrame(Dy->Ddchlist);
     free(Dy->Ddchlist); Dy->Ddchlist = NULL;
     return(0);
    }
 else if (Dy->ddfilelist_flag == ON)
    {return(-2);}
 else
    {return(-1);}

#endif

 return(0);
}


/*------------------------------------------- DyDdDataRequest ---*/

long DyDdDataRequest (char *DdServer, char *inputfile, char *tag, int gpsStart, int nSeconds)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256], *listofgps, *p;
 int itry, i;

 DyDebugPrint(__FUNCTION__,0);

 message = CmMessageNew();

 if (((Dy->debugType)&(1<<DBG_READ)))
    {printf("    DyDdDataRequest> Sending Data request to %s:  name=%s  inputfile=%s  tag=%s  gpsStart=%d  nSeconds=%d\n",
             Dy->DdServer, Dy->CmName, inputfile, tag, gpsStart, nSeconds);
    }

 CmMessagePutText (message,Dy->CmName);
 CmMessagePutText (message,inputfile);
 CmMessagePutText (message,tag);
 CmMessagePutInt  (message,gpsStart);
 CmMessagePutInt  (message,nSeconds);
 CmMessagePutInt  (message,Dy->skipFrame);

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_ASCII) 
    {CmMessagePutText (message,TRIGASCII);
     listofgps = (char *)malloc(10*Dy->nevent_ascii+2);
     p = listofgps;
     for (i=0; i<Dy->nevent_ascii; i++) p += sprintf(p," %d", (int)(Dy->AsciiGps[i]));
     CmMessagePutText (message,listofgps);
    }

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_EVENTS) 
    {CmMessagePutText (message,TRIGEVENTS);
     CmMessagePutText (message,Dy->conditionName);
     CmMessagePutDouble (message,Dy->conditionTh);
    }

 CmMessageSetType (message,DATAREQUEST);
 CmMessageSend    (message,DdServer);

 sprintf(DsSender, "%s_%s",DdServer, Dy->CmName);
 itry = 0;
 while (!CmMessageSend(message,DsSender) && itry<5)
    {itry++;
     sleep(1);
    }

 CmMessageDelete (message);

#endif

 return(0);
}


/*------------------------------------------- DyDdSendWaitingTime ---*/

long DyDdSendWaitingTime (double waiting_time)
{
#ifndef NOCM

 CmMessage message;
 char DsSender[256], dummy[LSTRING_MAX];

 sprintf(dummy,"DyDdSendWaitingTime> waiting_time=%fs", waiting_time);
 DyDebugPrint(dummy,0);

 sprintf(DsSender, "%s_%s",Dy->DdServer, Dy->CmName);

 message = CmMessageNew();

 CmMessagePutFloat (message,waiting_time);
 CmMessageSetType (message,DDWAIT);
 CmMessageSend (message,DsSender);

 CmMessageDelete (message);

#endif

 return(0);
}



#ifndef NOCM

/*------------------------------------------- DyDdHandlerFFL ---*/

CmMessageStatus DyDdHandlerFFL (CmMessage message, char* sender, char* servername)
{
 char *ffldir;

 ffldir = CmMessageGetText(message);
 if (ffldir) strcpy(Dy->Ddffldir, ffldir);

 if (((Dy->debugType)&(1<<DBG_READ))) printf("DyDdHandlerFFL> Received ffldir=%p\n", ffldir);

 return(CmMessageBreak);
}

#endif


#ifndef NOCM

/*------------------------------------------- DyDdHandlerChlist ---*/

CmMessageStatus DyDdHandlerChlist (CmMessage message, char* sender, char* servername)
{
 char *chlist;

 DyDebugPrint(__FUNCTION__,0);

 chlist = CmMessageGetText(message);
 if (chlist) Dy->Ddchlist = my_strcpy(Dy->Ddchlist, chlist);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0 && chlist)
    {printf("DyDdHandlerChlist> Received chlist=%p (%d bytes)\n", chlist, (int)strlen(chlist));}

 return(CmMessageBreak);
}

#endif


#ifndef NOCM

/*------------------------------------------- DyDdHandlerFilelist ---*/

CmMessageStatus DyDdHandlerFilelist (CmMessage message, char* sender, char* servername)
{
 char *filelist;
 char *str;

 DyDebugPrint(__FUNCTION__,0);

 filelist = CmMessageGetText(message);

 Dy->nddfile = 0;

 if (filelist) {strcpy(Dy->ddfilelist[Dy->nddfile],".."); (Dy->nddfile)++;}

 str = strtok(filelist," ");
 while (str != NULL)
    {strcpy(Dy->ddfilelist[Dy->nddfile],str);
     (Dy->nddfile)++;
     str = strtok(NULL," ");
    }

 if (Dy->nddfile > 0) DyXformDdFileFillBrowser();
 if (Dy->nddfile > 0) Dy->ddfilelist_flag = ON;

 return(CmMessageBreak);
}

#endif


