/*******************************************************/
/*                                                     */
/* File: DyUtils.c                                     */
/* Copyright (C) 2003, D. Verkindt                     */
/* For the licensing terms see the LICENSE file.       */
/*                                                     */
/* regroup the following previous files:               */
/* DyUtil.c  DyVar.c  DyVisu.c  DyFrOutput.c DyDump.c  */
/*                                                     */
/*******************************************************/

#define _GNU_SOURCE 
#include "dataDisplay.h"
#include "URootAccess.h"
#include "URootProcess.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

#ifdef NOCM
#define CHANNELS_LIST   "ChannelsList"  
#endif

/**************************************************/
/*                                                */
/* File: DyUtil.c                                 */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*----------------------------------------------- DySetName ---*/

void DySetName()
{
 char localName[LSTRING_MAX], localName2[LSTRING_MAX];

 strcpy(Dy->DyName,DEFAULT_DYNAME);

 /*--- Set dynamically a dataDisplay name ---*/

 if (getenv("USER") != NULL)
    sprintf(localName,"%s_%s_%d", DEFAULT_DYNAME, getenv("USER"), getpid());
 else
    sprintf(localName,"%s_unknown_%d", DEFAULT_DYNAME, getpid());

 /*--- Check that the name built does not contains forbidden characters ---*/
 if (DyCheckString(localName))
   {strcpy(localName2, DyCheckString(localName));
    strcpy(Dy->DyName,localName2);
   }
 
 return;
}


/*----------------------------------------------- Dy_quickselect ---*/
/* Find the kth lowest value on n elements of an array, median value is for k=n/2 */
/*
  This Quickselect routine is based on the algorithm described in
  "Numerical recipes in C", Second Edition,
  Cambridge University Press, 1992, Section 8.5, ISBN 0-521-43108-5
  This code by Nicolas Devillard - 1998. Public domain.
*/

double Dy_quickselect(double *arr, int n, int k)
{
 unsigned long i,ir,j,l,mid;
 float a,temp;

 l  = 0;
 ir = n-1;
 for (;;)
    {
     if (ir <= l+1)
       { 
        if (ir == l+1 && arr[ir] < arr[l]) DY_SWAP(arr[l],arr[ir]);
        return(arr[k]);
       }
     else
       {
        mid = (l+ir) >> 1; 
        DY_SWAP(arr[mid],arr[l+1]);
        if (arr[l] > arr[ir]) DY_SWAP(arr[l],arr[ir]);
        if (arr[l+1] > arr[ir]) DY_SWAP(arr[l+1],arr[ir]);
        if (arr[l] > arr[l+1])  DY_SWAP(arr[l],arr[l+1]);
        i=l+1; 
        j=ir;
        a=arr[l+1]; 
        for (;;)
           { 
            do i++; while (arr[i] < a); 
	    do j--; while (arr[j] > a); 
	    if (j < i) break; 
	    DY_SWAP(arr[i],arr[j]);
           } 
        arr[l+1]= arr[j]; 
        arr[j]  = a;
        if (j >= k) ir = j-1; 
        if (j <= k) l = i;
       }
    }

}


/*-------------------------------------------------- DySoundFillHeader ------*/

void DySoundFillHeader(struct soundhdr *hdr, long srate, long nBytes, short bits_per_sample)
{
 struct soundhdr myhdr = {"RIF",88244,"WAV","fmt",16,1,1,44100,88200,2,16,"dat",88200};
 myhdr.riff[3] = 'F';
 myhdr.wave[3] = 'E';
 myhdr.fmt[3] = ' ';
 myhdr.data[3] = 'a';
 myhdr.format_tag = 1;
 myhdr.num_chans  = 1;
 myhdr.bits_per_samp = bits_per_sample;
 myhdr.bytes_per_samp = myhdr.bits_per_samp / 8;
 myhdr.srate = (int)srate;
 myhdr.bytes_per_sec = (int)srate * myhdr.bytes_per_samp;
 myhdr.dlength = (int)nBytes;
 myhdr.flength = myhdr.dlength + sizeof(myhdr);
 *hdr = myhdr;

 return;
}


/*--------------------------------------------- DySoundReadHeader ---*/

void DySoundReadHeader (char *buffer)
{
 char format_tag_name[16];
 struct soundhdr myhdr;

 DyDebugPrint(__FUNCTION__,0);

 memcpy((char *)(&myhdr),buffer,sizeof(myhdr));

 strcpy(format_tag_name,"unknown");
 if (myhdr.format_tag == 1) strcpy(format_tag_name,"PCM");
 if (myhdr.format_tag == 257) strcpy(format_tag_name,"Mu-Law");
 if (myhdr.format_tag == 258) strcpy(format_tag_name,"A-Law");
 if (myhdr.format_tag == 259) strcpy(format_tag_name,"ADPCM");

 printf("DySoundReadHeader> Header size: %ld bytes \n", sizeof(myhdr));
 printf("DySoundReadHeader> riff: %c  %c  %c  %c \n", myhdr.riff[0], myhdr.riff[1], myhdr.riff[2], myhdr.riff[3]);
 printf("DySoundReadHeader>       %x %x %x %x \n", buffer[0], buffer[1], buffer[2], buffer[3]);
 printf("DySoundReadHeader> File length: %d bytes \n", myhdr.flength);
 printf("DySoundReadHeader> wave: %c%c%c%c \n", myhdr.wave[0], myhdr.wave[1], myhdr.wave[2], myhdr.wave[3]);
 printf("DySoundReadHeader> fmt: %c%c%c%c \n", myhdr.fmt[0], myhdr.fmt[1], myhdr.fmt[2], myhdr.fmt[3]);
 printf("DySoundReadHeader> Block size: %d bytes \n", myhdr.block_size);
 printf("DySoundReadHeader> Format: %d (%s)  \n", myhdr.format_tag, format_tag_name);
 printf("DySoundReadHeader> Number of channels: %d  \n", myhdr.num_chans);
 printf("DySoundReadHeader> Sample Rate: %d  \n", myhdr.srate);
 printf("DySoundReadHeader> Number of bytes per sec: %d  \n", myhdr.bytes_per_sec);
 printf("DySoundReadHeader> Number of bytes per sample: %d  \n", myhdr.bytes_per_samp);
 printf("DySoundReadHeader> Number of bits per sample: %d  \n", myhdr.bits_per_samp);
 printf("DySoundReadHeader> data: %c%c%c%c \n", myhdr.data[0], myhdr.data[1], myhdr.data[2], myhdr.data[3]);
 printf("DySoundReadHeader> Data length: %d  \n", myhdr.dlength);

/*
     header attendu (big-endian):
     R I  F F  file_length W A  V E  f m  t space
     5249 4646 2408 0000   5741 5645 666d 7420
     block_size format nch  sampRate    bytes_per_sec
     1000 0000  0100   0200 2256 0000   8858 0100
     byte_per_sample  bits_per_sample d a  t a  data_length
     0400             1000            6461 7461 0008 0000

     header obtenu sur lappc-vi22 avec un canal a 20kHz:
     4952 4646 58b4 0001 4157 4556 6d66 2074
     0010 0000 0001 0001 49d4 0000 93a8 0000
     0002 0010 6164 6174 5888 0001 a5a7 9a9f
*/

 return;
}

/*------------------------------------------------------------ DyChCloneOpch ---*/

void DyChCloneOpch(struct DyCh *chout, struct DyCh *chin)
{
 int i;

 chout->op_nch    = chin->op_nch;
 if (chin->op_classname) strcpy(chout->op_classname, chin->op_classname);
 if (chin->op_string)    strcpy(chout->op_string, chin->op_string);
 for (i=0; i< chin->op_nch; i++) chout->op_chin[i] = DyChClone(chin->op_chin[i]);

 return;
}

/*------------------------------------------------------------ DyChClone ---*/

struct DyCh *DyChClone(struct DyCh *chin)
{
 struct DyCh *chout;
 int i;

 if (chin == NULL) return(NULL);

 chout = (struct DyCh *)calloc(1,sizeof(struct DyCh));

 if (chout == NULL) {printf("DyChClone> Could not clone channel %s \n", chin->name); return(NULL);}

 strcpy(chout->name, chin->name);
 strcpy(chout->inputname, chin->inputname);
 chout->sampFreq        = chin->sampFreq;
 chout->inputtype       = chin->inputtype;
 chout->vect2d_duration = chin->vect2d_duration;
 chout->vect2d_deltat   = chin->vect2d_deltat;
 chout->op_nch          = chin->op_nch;
 if (chin->op_classname) strcpy(chout->op_classname, chin->op_classname);
 if (chin->op_string)    strcpy(chout->op_string, chin->op_string);
 for (i=0; i< chin->op_nch; i++) chout->op_chin[i] = DyChClone(chin->op_chin[i]);

 return(chout);
}

/*------------------------------------------------------------ DySetDefaultStartTime ---*/

double DySetDefaultStartTime()
{
 int i;
 double starttime, endtime;
 struct DyPlot *plot;
 double minstarttime;

 DyDebugPrint(__FUNCTION__,0);

 minstarttime = 1e+30;

 for (i=1; i<=Dy->nplot; i++)
    {
     plot = Plot[i];

     if (plot->hidden == ON) continue;

     if (plot->chx)
       {if (plot->chx->inputtype == INFILE || plot->chx->inputtype == INCM)
          {starttime = FrFileITStart(plot->chx->ifile);
           if (minstarttime > starttime && starttime > 0) minstarttime = starttime;
           if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 1)
             {printf("DySetDefaultStartTime> chx : starttime %f of %s\n", starttime, plot->chx->inputname);}
          }
       }
     if (plot->chy)
       {if (plot->chy->inputtype == INFILE || plot->chy->inputtype == INCM)
          {starttime = FrFileITStart(plot->chy->ifile);
           if (minstarttime > starttime && starttime > 0) minstarttime = starttime;
           if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 1)
             {printf("DySetDefaultStartTime> chy : starttime %f of %s\n", starttime, plot->chy->inputname);}
          }
       }
    }

 /*--- In case of files input, force the start time to be the min start of the input files ---*/

 if (Dy->startTime < minstarttime && minstarttime > 0) Dy->startTime = starttime;

 /*--- In case of Cm input, force the start time to be the GPS time of the last frame received ---*/

 if (Dy->CmMffile != NULL)
   {endtime = FrFileITEnd(Dy->CmMffile);
    if (Dy->startTime > endtime) Dy->startTime = endtime - 1;
    if (Dy->startTime < 0) Dy->startTime = DyGetGPSNow();
   }

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0)
   {printf("DySetDefaultStartTime> startTime=%f \n", Dy->startTime);}

 return(Dy->startTime);
}


/*------------------------------------------------------------ DySetDefaultEndTime ---*/

double DySetDefaultEndTime()
{
 int i;
 double endtime;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 Dy->endTime = DY_NOENDTIME;

 /*--- In case of Cm input, no end time ---*/

 if (Dy->InputType == INCM) return(Dy->endTime);

 /*--- In cas of file input, determine the max endTime of the files used ---*/

 Dy->endTime = 0;

 for (i=1; i<=Dy->nplot; i++)
    {
     plot = Plot[i];

     if (plot->hidden == ON) continue;

     if (plot->chx)
       {if (plot->chx->inputtype == INFILE || plot->chx->inputtype == INCM)
          {endtime = FrFileITEnd(plot->chx->ifile);
           if (Dy->endTime < endtime && endtime > 0) Dy->endTime = endtime;
          }
       }
     if (plot->chy)
       {if (plot->chy->inputtype == INFILE || plot->chy->inputtype == INCM)
          {endtime = FrFileITEnd(plot->chy->ifile);
           if (Dy->endTime < endtime && endtime > 0) Dy->endTime = endtime;
          }
       }
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DySetDefaultEndTime> endTime=%f \n", Dy->endTime);

 return(Dy->endTime);
}



/*------------------------------------------ DyGetGPSNow ---*/

double DyGetGPSNow ()
{
 double gpsnow;

 gpsnow = FrGetCurrentGPS();

 return(gpsnow);
}


/*------------------------------------------- DyGetLatency ---*/

double DyGetLatency (double gps, double uleaps)
{
 double latency;

 if (uleaps <= 0) uleaps = FrGetLeapS(gps);

 latency = (double)(time(NULL)) - FRGPSOFF - FRGPSTAI + uleaps - gps;

 return(latency);
}

/*------------------------------------------- DyTimeFormatToGps ---*/

double DyTimeFormatToGps (const char *timeformat)
{
 char dummy[256], *str;
 double gtime;
 double dec;

  strcpy(dummy, timeformat);
  str = strstr(dummy,"s");
  if (!str) str = strstr(dummy,"GMT");

  if (str)
     {
      if (strstr(dummy,"s") != NULL) sscanf(str+1, "%lf", &dec);
      else                           dec = 0.;
      if (strstr(dummy,"s") != NULL) dummy[str-dummy] = '\0';
      else                           dummy[str-dummy-1] = '\0';
      gtime = DyGTimeFromDate (dummy+2);
      gtime = gtime + dec;
     }
 else
    {gtime = 0.;}

 return(gtime);
}


/*------------------------------------------- DyGpsToTimeString ---*/

char *DyGpsToTimeString (double origin, struct FrameH *frame, double deltat, int nAverage, int decayfft, char *title)
{
 char stringtime[40], title_end[64];
 time_t starttimeutc;
 int origin2;

 DyDebugPrint(__FUNCTION__,0);

 if (deltat == 0 && nAverage <= 1)
    {strcpy(title_end,DYEMPTY);
     if (strstr(title,"dt:") != NULL) strcpy(title_end,strstr(title,"dt:"));
    }

 if (origin-(int)origin >= 0.9999) origin = (double)((int)(origin+0.1));

 if (frame) starttimeutc = FrGPS2UTC((unsigned int)origin, frame->ULeapS);
 else       starttimeutc = FrGPS2UTC((unsigned int)origin,0);
 strftime(stringtime,30,"%b %e %Y %H:%M:%S",gmtime(&starttimeutc));

 origin2 = (int)(10000 * (origin -(int)origin));
 if (origin2%10000 == 9999)                   {origin2 = 0; origin += 1.;}
 else if (origin2-1000*(origin2/1000) == 999)  origin2++;
 else if (origin2-100*(origin2/100) == 99)     origin2++;
 else if (origin2-10*(origin2/10) == 9)        origin2++;

/*
 if (origin2 == 0)             sprintf(title, "%9.0f : %s UTC", origin, stringtime);
 else if (origin2%1000 == 0.)  sprintf(title, "%10.1f : %s UTC", origin, stringtime);
 else if (origin2%100 == 0.)   sprintf(title, "%11.2f : %s UTC", origin, stringtime);
 else if (origin2%10 == 0.)    sprintf(title, "%12.3f : %s UTC", origin, stringtime);
 else                          sprintf(title, "%13.4f : %s UTC", origin, stringtime);
*/

 if (deltat) sprintf(title, "%13.2f : %s UTC", origin, stringtime);
 else        sprintf(title, "%13.4f : %s UTC", origin, stringtime);

/* ICI replacer deltat et naverage par une string optionnelle envoyee par DyPlotSetTime */

 if (deltat > 0.01)    sprintf(title, "%s dt:%.2fs", title, deltat);
 else if (deltat > 0.) sprintf(title, "%s dt:%.3fs", title, deltat);

 if (nAverage > 1 && decayfft == OFF)     sprintf(title, "%s nAv:%d", title, nAverage);
 else if (nAverage > 1 && decayfft == ON) sprintf(title, "%s movAvg", title);

 if (deltat == 0 && nAverage <= 1)
    {if (strcmp(title_end,DYEMPTY) != 0) sprintf(title,"%s %s", title, title_end);}

 return(title);
}



/*********/
/* INIT  */
/*********/

/*------------------------------------------- DySetDefaultColors ---*/

void DySetDefaultColors (int flag)
{
 int i;

 if (flag <= 0)
   {
    Dy->defaultcolor[0] = 593;  /* kBlue-7   */
    Dy->defaultcolor[1] = 807;  /* kOrange+7 */
    Dy->defaultcolor[2] = 418;  /* kGreen+2  */
    Dy->defaultcolor[3] = 429;  /* kCyan-3   */
    Dy->defaultcolor[4] = 626;  /* kRed-6    */
    Dy->defaultcolor[5] = 923;  /* kGray+3 */
    Dy->defaultcolor[6] =   4;  /* kBlue     */
    Dy->defaultcolor[7] =   2;  /* kRed      */
    Dy->defaultcolor[8] = 804;  /* kOrange+4 */
    Dy->defaultcolor[9] = 907;  /* kPink+7   */
   }

 else if (flag == 1)
   {
    Dy->defaultcolor[0] = 1;    /* kBlack     */
    Dy->defaultcolor[1] = 632;  /* kRed       */
    Dy->defaultcolor[2] = 416;  /* kGreen     */
    Dy->defaultcolor[3] = 600;  /* kBlue      */
    Dy->defaultcolor[4] = 616;  /* kMagenta   */
    Dy->defaultcolor[5] = 432;  /* kCyan      */
    Dy->defaultcolor[6] = 800;  /* kOrange    */
    Dy->defaultcolor[7] = 920;  /* kGray      */
    Dy->defaultcolor[8] = 922;  /* kGray+2    */
    Dy->defaultcolor[9] = 618;  /* kMagenta+2 */
   }

 return;
}


/*------------------------------------------- DyIni ---*/

void DyIni (char **argv, int argc)
{
 long i;
 char dummy[LSTRING_MAX];
 int dbgLvl;
/*
 struct rlimit *rlim;
*/

 DyDebugPrint(__FUNCTION__,0);

/*
 if (is_a_dir(FFLDIR)) Dy->Dir = my_strcpy(Dy->Dir, FFLDIR);
 else                  Dy->Dir = my_strcpy(Dy->Dir, ".");
*/
 Dy->Dir = my_strcpy(Dy->Dir, ".");

 /*--- Init channels list ---*/

 Dy->chlist = NULL;
 Dy->nch = 0;

 /*--- Set default plot line colors ---*/

 DySetDefaultColors(0);
 Dy->colorset = 0;

 /*--- Init plot parameters ---*/

 Dy->UpdateDone    = OFF;
 Dy->nplotcreated  = 0;
 Dy->nplot         = 0;
 dbgLvl = Dy->debug;
 Dy->debug = 0;
 for (i=0; i<NPLOT_MAX; i++)
    {Plot[i] = (struct DyPlot *)calloc(1,sizeof(struct DyPlot));
     DyPlotInit(i,ON);
    }
 Dy->debug = dbgLvl;

 /*--- Init for general parameters ---*/

 Dy->no_autostop      = OFF;
 Dy->firstTime        = 0;
 Dy->Frame            = NULL;
 Dy->y2d              = OFF;
 Dy->CmConnect        = OFF;
 Dy->InputType        = INOFF;
 Dy->Order            = STOP;
 Dy->frameChannel     = 0;
 Dy->nCol             = 0;
 Dy->nRow             = 0;
 Dy->cmacrocopy       = NULL;
 Dy->AutoDeselectPlot = ON;
 Dy->AutoDeselectCh   = OFF;
 Dy->DdNS             = OFF;
 Dy->Ddpasswd         = NULL;
 Dy->nicefont         = OFF; 
 Dy->fullwhite        = ON;
 Dy->WaitData         = OFF;
 Dy->linewidth        = 1;

 /*--- Set default parameters for outputing frames ---*/

 Dy->output_nbytes_tot = 0;
 DyOutputDefault();

 /*--- Set default for  various parameters or strings ---*/

 strcpy(Dy->gLastTitle," ");
 strcpy(Dy->gLastTitleY," ");
 strcpy(Dy->lastSearchName," ");

 /*--- Init for triggers ---*/

 Dy->triggerVect       = NULL;
 Dy->triggerUsed       = OFF;
 strcpy(Dy->trigAsciiFileName,"triglist.txt");
 strcpy(Dy->trigEventName,"EVENT_SNR");
 strcpy(Dy->lastTrigString,"Qc_Moni..ScienceMode = 1");

 /*--- Init for Reference plots ---*/

 Dy->refplot_indelebile = ON; 
 Dy->lastrefplot_dir    = my_strcpy (Dy->lastrefplot_dir,".");

 /*--- Set default file names for saved configs or saved plots ---*/

 strcpy (Dy->lastsavedConfig,DEFAULT_DYNAME); strcat(Dy->lastsavedConfig, CONFIG_SUFFIX);
 strcpy (Dy->lastsavedPlots,DEFAULT_DYNAME); strcat (Dy->lastsavedPlots, ROOT_SUFFIX);
 
 /*--- Set default input names and dataDisplay Cm name ---*/

 strcpy(Dy->shmName,DEFAULT_SHMNAME);
 strcpy(Dy->CmServer,DYEMPTY);
 strcpy(Dy->CmDomain,CASCINA);
 strcpy(Dy->DdServer, DDSERVER);
 strcpy(Dy->Ddinputfile, DDFILE);
 strcpy(Dy->DdDir, DDDIR);
 Dy->CmMftspan = 16;

/*--- Limit the memory usage for a single core ---*/
/***
HIHI
core limit
 if (Dy->debug == 0)
    {
     rlim = (struct rlimit *) malloc(sizeof(struct rlimit));
     getrlimit(RLIMIT_CORE,rlim);
     printf("rlim_cur=%d  rlim_max=%d \n", (int)rlim->rlim_cur, (int)rlim->rlim_max);
     rlim->rlim_max = (rlim_t)0;
     setrlimit(RLIMIT_CORE,rlim);
    }
***/

 /*--- Open main panel of data Display ---*/

 DyXformRaisePanel(NULL,MAINPANEL);

 /*--- initialize help ballons ---*/

 DyXformHelpMainPanel(NULL,1);     /*--- initialize help ballons for mainpanel panel   ---*/
 DyXformHelpMainBrowser(NULL,1);   /*--- initialize help ballons for mainbrowser panel ---*/
 DyXformHelpPlottime(NULL,1);      /*--- initialize help ballons for plottime panel    ---*/
 DyXformHelpPlotffttime(NULL,1);   /*--- initialize help ballons for plotffttime panel ---*/
 DyXformHelpPlot1dtime(NULL,1);    /*--- initialize help ballons for plot1dtime panel  ---*/
 DyXformHelpPlotrawtime(NULL,1);   /*--- initialize help ballons for plotrawtime panel ---*/
 DyXformHelpPlottrfct(NULL,1);     /*--- initialize help ballons for plottrfct panel   ---*/
 DyXformHelpPlotcohe(NULL,1);      /*--- initialize help ballons for plotcohe panel    ---*/
 DyXformHelpPlotfft(NULL,1);       /*--- initialize help ballons for plotfft panel     ---*/
 DyXformHelpPlot1d(NULL,1);        /*--- initialize help ballons for plot1d panel      ---*/
 DyXformHelpPlot2d(NULL,1);        /*--- initialize help ballons for plot2d panel      ---*/
 DyXformHelpAudio(NULL,1);         /*--- initialize help ballons for plotaudio panel   ---*/
 DyXformHelpPlotGenePar(NULL,1);   /*--- initialize help ballons for plotgenepar panel ---*/
 DyXformHelpCmConnect(NULL,1);     /*--- initialize help ballons for plotgenepar panel ---*/
 DyXformHelpUserFct(NULL,1);       /*--- initialize help ballons for plotgenepar panel ---*/
 DyXformHelpOptions(NULL,1);       /*--- initialize help ballons for options panel     ---*/
 DyXformHelpSliders(NULL,1);       /*--- initialize help ballons for sliders panel     ---*/
 DyXformHelpOutput(NULL,1);        /*--- initialize help ballons for output panel      ---*/
 DyXformHelpDebug(NULL,1);         /*--- initialize help ballons for debug panel       ---*/
 DyXformHelpReffile(NULL,1);       /*--- initialize help ballons for refplot panel     ---*/

 /*--- Last message ---*/
 if (Dy->AutoConfFlag == OFF)
    {sprintf(dummy, "dataDisplay %s ready to get data. Use Input Menu", DYVERSION);
     DyInfoLine(dummy);
    }

 /*--- Initialization of the defaut scripts to user operations ---*/
/**
 RA_InitDefaultScripts();
**/
 DyXformUpdateUserProcBrowser();

 return;
}


/*--------------------------------------------------- DyRaz ---*/

void DyRaz (long arg)
{
 long i;

 DyDebugPrint(__FUNCTION__,0);

 /*--- remove ROOT window and plots in memory ---*/

 if (arg == 5 || arg == 2 || arg == 0)
    {if (((Dy->debugType)&(1<<DBG_VDYPLOT))) printf("DyRaz> Reset ROOT windows \n");
     RA_MainCanvasDelete();
     Dy->nRow = 0;
     Dy->nCol = 0;
     for (i=0; i<=Dy->nplot; i++) RA_HDelete(Plot[i]->name);
    }

 /*--- reset plots ---*/

 if (arg == 2 || arg == 0) 
    {
     for (i=0; i<=Dy->nplot; i++) DyPlotInit(i,ON);
     Dy->nplot = 0;
    }

 /*--- Initialize channels list ---*/

 if (arg == 1 || arg == 0)
    {
     for (i=0; i<Dy->nch; i++) free(Dy->ch[i]);
     Dy->nch = 0;
    }

 /*--- Initialize options and main parameters ---*/

 if (arg == 0)
    {
     Dy->firstTime  =  0;
     Dy->startTime  = DyGetGPSNow();
     Dy->duration   = -1;
     Dy->Frame      = NULL;
     Dy->y2d        = OFF;
     Dy->CmConnect  = OFF;
     Dy->InputType  = INOFF;
     Dy->Order      = STOP;
    }

 /*--- Inform user about the type of reset done ---*/

 if (arg == 0) DyInfoLine("All reset!");
 if (arg == 1) DyInfoLine("Input data is reset");
 if (arg == 2) DyInfoLine("List of plots is reset");
 if (arg == 5) DyInfoLine("ROOT Canvas removed");
 if (arg == 6) DyInfoLine("Reference plots files removed");

 return;
}


/*------------------------------------------- DyMakeTypeNames ---*/
/* Called once at start of dataDisplay */

void DyMakeTypeNames ()
{
 int i;

 for (i=0; i<64; i++) strcpy(Dy->typeName[i], "unknown");

 /*--- Plots types ---*/

 strcpy(Dy->typeName[PLOTTIME],     "TIME");
 strcpy(Dy->typeName[PLOTFFT],      "FFT");
 strcpy(Dy->typeName[PLOTTR],       "TRFCT");
 strcpy(Dy->typeName[PLOTCOHE],     "COHE");
 strcpy(Dy->typeName[PLOT1D],       "1D");
 strcpy(Dy->typeName[PLOT2D],       "2D");
 strcpy(Dy->typeName[PLOTAUDIO],    "AUDIO");
 strcpy(Dy->typeName[PLOTFFTTIME],  "FFTTIME");
 strcpy(Dy->typeName[PLOTRAW],      "RAW");
 strcpy(Dy->typeName[PLOT1DTIME],   "1DTIME");
 strcpy(Dy->typeName[PLOTRAWTIME],  "RAWTIME");
 strcpy(Dy->typeName[PLOTTRTIME],   "TRFCTTIME");
 strcpy(Dy->typeName[PLOTCOHETIME], "COHETIME");

 /*--- Panels types ---*/

 strcpy(Dy->typeName[MAINPANEL],    "mainpanel");
 strcpy(Dy->typeName[HELPPANEL],    "help");
 strcpy(Dy->typeName[MAINBROWSER],  "mainbrowser");
 strcpy(Dy->typeName[INPUTFILE],    "fileslist");
 strcpy(Dy->typeName[CMSERVERS],    "cmservers");
 strcpy(Dy->typeName[GETDD],        "datasender");

 strcpy(Dy->typeName[OUTPUTDATA],   "outputdata");
 strcpy(Dy->typeName[GENEPAR],      "geneparameters");
 strcpy(Dy->typeName[USERFUNCBROWSER], "userfct");
 strcpy(Dy->typeName[USERFCTHELP],  "userfcthelp");
 strcpy(Dy->typeName[COFFEEPANEL],  "coffee");
 strcpy(Dy->typeName[REFFILE],      "referencefiles");
 strcpy(Dy->typeName[DYOPTIONS],    "dyoptions");
 strcpy(Dy->typeName[DYSLIDERS],    "dysliders");
 strcpy(Dy->typeName[DYDEBUG],      "dydebug");

 /*--- Input types ---*/

 strcpy(Dy->inputtypeName[INFILE],  "GWF_FILE");
 strcpy(Dy->inputtypeName[INCM],    "ONLINE");
 strcpy(Dy->inputtypeName[INDD],    "DATASENDER");
 strcpy(Dy->inputtypeName[INSHM],   "SHARED_MEMORY");
 strcpy(Dy->inputtypeName[INDD],    "NO_INPUT");
 strcpy(Dy->inputtypeName[INWAVE],  "AUDIO_FILE");
 strcpy(Dy->inputtypeName[INASCII], "ASCII_FILE");
 strcpy(Dy->inputtypeName[INDY],    "CHANNELS_OPERATION");

 return;
}


/*------------------------------------------- DyMakeTag ---*/

void DyMakeTag (char *tag)
{
 int id;

 DyDebugPrint(__FUNCTION__,0);

 strcpy(tag," ");

 for (id=1; id<=Dy->nplot; id++)
    {
     if (Plot[id]->hidden == ON) continue;

     if (Plot[id]->chx != NULL) DyAddToTag(tag, Plot[id]->chx);
     if (Plot[id]->chy != NULL) DyAddToTag(tag, Plot[id]->chy);
    }

 if (Dy->output_flag == ON && Dy->output_tag != NULL) strcat(tag, Dy->output_tag);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0) printf("    DyMakeTag> tag=\042%s\042 \n", tag);

 return;
}


/*------------------------------------------- DyAddToTag ---*/

char *DyAddToTag (char *tag, struct DyCh *ch)
{
 char dummy[LSTRING_MAX];
 char smsName[LSTRING_MAX];
 int j;
 char *str;

 /*--- Check if channel is a serData ---*/
 /*--- If yes, add to tag any possible sms name ---*/

 if (ch->sampFreq <= 1.)
   {str = strstr(ch->name,"_");
    while (str)
      {strncpy(smsName,ch->name,str-ch->name);
       smsName[str-ch->name] = '\0';
       sprintf(dummy," %s ",smsName);
       if (strstr(tag, dummy) == NULL) {strcat(tag, smsName); strcat(tag," ");}
       str = strstr(str+1,"_");
      }
    sprintf(dummy," %s ",ch->name);
    if (strstr(tag, dummy) == NULL) {strcat(tag, ch->name); strcat(tag," ");}
   }

 /*--- Otherwise add to tag the channel name ---*/

 else
   {if (ch->inputtype == INDY)
      {for (j=0; j<ch->op_nch; j++)
          {sprintf(dummy," %s ",ch->op_chin[j]->name);
           if (strstr(tag, dummy) == NULL) {strcat(tag, ch->op_chin[j]->name); strcat(tag," ");}
          }
      }
    else
      {sprintf(dummy," %s ",ch->name);
       if (strstr(tag, dummy) == NULL) {strcat(tag, ch->name); strcat(tag," ");}
      }
   }

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1)
   {printf("DyAddToTag> Added to tag the channel %s  (rate=%g) : tag=%s \n", ch->name, ch->sampFreq, tag);}

 return(tag);
}



/*                        *************                     */
/*                        *  RUNNING  *                     */
/*                        *************                     */

/*--------------------------------------------------- DyStart ---*/

void DyStart()
{
 double framebuffer_endtime, framebuffer_endtime2, timeshift;
 int i, config_created, npad, result, nplot_hidden;
 char dummy[LSTRING_MAX],configfilename[LSTRING_MAX],logName[LSTRING_MAX];
 double dymem;
 int k, answer, noframe, uleaps;
 double gtime, timewindow, endtime;
 struct DyPlot *plot;

 sprintf(dummy,"%s> Dy->nplot=%d  config=%s", __FUNCTION__, Dy->nplot, Dy->lastsavedConfig);
 DyDebugPrint(dummy,0);

 Dy->output_nframe_written = 0;
 Dy->nplotdrawn    = 0;

 /*--- If some canvas configuration was taken from a config , use it ---*/

 if (Dy->cmacrocopy)
    {
     RA_MacroCreateCanvas(Dy->cmacrocopy);
     free(Dy->cmacrocopy);
     Dy->cmacrocopy = NULL;
    }

 /*--- Set number of Pads needed in Root Canvas ---*/

 npad = DyPlotCheckPad();  /*--- Set also the numpad and idsame parameters of all plots ---*/
 DyWindowsIni (npad);

 /*--- Choose fast font (20) when running and nice but slow font (62) at stop ---*/
 if (Dy->nicefont == ON) RA_SetLabelFont (DY_FONT_HELVETICA, NULL);
 else                    RA_SetLabelFont (DY_FONT_TIMES, NULL);

 /*---------- Write global title of plots ----------*/
 RA_GlobalTitleDraw();

 /*--- Set the current running state ---*/
 Dy->Order = CONTINUE;
 
 /*--- If running manually, get parameters from GUI ---*/
 if (Dy->AutoConfFlag == OFF) DyXformGetFirstLast (&Dy->startTime, &Dy->duration);

 /*--- Show startTime and duration in the GUI ---*/
 if (Dy->Frame != NULL) uleaps = Dy->Frame->ULeapS;
 else                   uleaps = FrGetLeapS(Dy->startTime);
 DyXformSetStartTime(uleaps);

 /*--- Check for memory used ---*/

 dymem = DyCheckMemory();
 if (dymem > 256. && Dy->BgRun == OFF)
   {sprintf(dummy, "Memory used: %.3fMB may be too high. Do you really want to start?", dymem/1024./1024.);
    answer = DyXformQuestion(dummy);
    if (answer == 0) {DyXformTrigger(STOP); return;}
   }

 /*--- Set the value of mindt which will be used as period of the gtime loop ---*/
 /*--- and the maxdt which will be used to update the CmMftspan value used to store online frames in memory ---*/

 DyPlotSetMindt();
 DyPlotSetMaxdt();


 /*------------------------------------------------------------*/
 /*--------------------- Init of the plots --------------------*/

 sprintf(dummy, "====================== INITIALIZING : nplot=%d", Dy->nplot);
 DyInfoLine(dummy);

 /*--- Open all the inputs needed to read data for the plots ---*/

 if (((Dy->debugType)&(1<<DBG_READ))) printf("DyStart> Open inputs\n");

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     DyChOpenInput(Plot[i]->chx);
     DyChOpenInput(Plot[i]->chy);
    }

 /*--- In case of Cm input, receive first frames in buffer ---*/

 if (Dy->InputType == INCM)
   {
    noframe = DyGetCmFrame();
    if (noframe > 0)
      {
       for (i=1; i<=Dy->nplot; i++)
         {
          if (Plot[i]->hidden == ON) continue;
          DyChCloseInput(Plot[i]->chx);
          DyChCloseInput(Plot[i]->chy);
         }
       return;
      }
   }

 /*--- Set default start time. In case of Cm input, it is set to GPS time of last frame received ---*/
 /*--- Set default end time. In case of files input, it is set to the max endtime of input files ---*/

 DySetDefaultStartTime(); 
 DySetDefaultEndTime();

 /*--- If duration provided by user, force the endTime ---*/

 if (Dy->duration > 0 && Dy->startTime+Dy->duration < Dy->endTime) Dy->endTime = Dy->startTime + Dy->duration;

 /*--- Initialize the plots with data and set their secondary parameters ---*/

 DyPlotAllIni();

 /*------------------------------------------------------------*/
 /*--------------------- Create config file -------------------*/

 /*--- Check write access to current directory and build the default config file name ---*/

 result = access(".", W_OK);
 if (result != 0)
    {printf("DyStart> Cannot write in current directory. Files %s%s and %s%s will be put in your home directory.\n",
             Dy->curConfig, CONFIG_SUFFIX, Dy->curConfig, ROOT_SUFFIX);
     sprintf(Dy->curConfig,"%s/%s", (char *)getenv("HOME"), DEFAULT_DYNAME);
    }
 else
    {sprintf(Dy->curConfig,"%s", DEFAULT_DYNAME);}

 sprintf(configfilename, "%s%s", Dy->curConfig, CONFIG_SUFFIX);
 config_created = DyConfigCreate(configfilename,0);

 if (config_created == OFF)
    {sprintf(dummy, "Not been able to create config file %s", configfilename);
     DyInfoLine (dummy);
     return;
    }
 else
    {sprintf(dummy, "Created config file %s", configfilename);
     DyInfoLine (dummy);
    }


 /*------------------------------------------------------------*/
 /*------------------------- main loop ------------------------*/



 /*--- Set the value of min refreshDt. It will be the reference for canvas update ---*/
 DyPlotSetMinRefreshDt();

 /*--- Set the reference plot used for pads update ---*/
 Dy->iplotref = DyPlotGetUpdater();

 /*--- Get the starting latency ---*/
 Dy->startLatency  = DyGetLatency(Dy->startTime,0);
 Dy->ttstart = time(NULL);

 DyInfoLine("======================== START READING DATA");

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0) 
   {printf("DyStart> Time loop: startTime=%f  endTime=%f  increment=%g sec  minRefreshDt=%g  startLatency=%g sec\n",
            Dy->startTime, Dy->endTime, Dy->mindt, Dy->min_refreshDt, Dy->startLatency);
   }

 /*--- If input online via Cm, loop on the frames received ---*/
 /*--- If input offline from file, loop on internal gtime counter ---*/
 
 if (Dy->InputType == INCM) DyCmLoop();
 else                       DyFileLoop();

 /*--- End procedure for plots ---*/

 DyInfoLine("======================== STOP READING DATA");

 DyPlotEnd();

 /*--- Close all the inputs needed to read data for the plots ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     DyChCloseInput(Plot[i]->chx);
     DyChCloseInput(Plot[i]->chy);
    }

 /*--- Set the Stop button when main loop is finished ---*/
 /*--- This will only set the GUI stop configuration and put Dy->Order to STOP ---*/

 DyXformTrigger(STOP);
 Dy->Order = STOP;

 /*--- End procedure for plots visualization ---*/

 DyVisuEnd();

 /*--- Set AutoConfFlag to OFF ---*/
 Dy->AutoConfFlag = OFF;

 /*--- In case dataDisplay is run in background, save the plots as a picture ---*/
 if (Dy->BgRun == ON) DySavePlot(Dy->AutoConf);

 /*--- In case dataDisplay is run with a quit at stop ---*/
 if (Dy->QuitAtStop == ON) DyXformQuit(NULL,10);

 return;
}


/*--------------------------------------- DyFileLoop ---*/

void DyFileLoop()
{
 int i,k;
 double gtime, endtime;
 int tt;

 DyDebugPrint(__FUNCTION__,0);

 for (gtime=Dy->startTime; gtime<Dy->endTime; gtime=gtime+Dy->mindt)
    {
     /*--- If duration was not provided, check the input files end time before deciding to stop ---*/
     if (Dy->duration <= 0 && gtime > Dy->endTime-2*Dy->mindt) DySetDefaultEndTime();

     /*--- If option to wait for data is ON and no duration was provided, check indefinitely for input update ---*/
     if (Dy->duration <= 0 && gtime >= Dy->endTime-2*Dy->mindt && Dy->WaitData == ON)
       {
        endtime = Dy->endTime;
        while (Dy->endTime <= endtime && Dy->Order != STOP && Dy->WaitData == ON)
           {
            printf("DyFileLoop> Waiting for new data in inputs... \n");
            for (k=1; k<=Dy->nplot; k++)
               {
                if (Plot[k]->hidden == ON) continue;
                DyChCloseInput(Plot[k]->chx);
                DyChCloseInput(Plot[k]->chy);
               }
            sleep(1);
            for (k=1; k<=Dy->nplot; k++)
               {
                if (Plot[k]->hidden == ON) continue;
                DyChOpenInput(Plot[k]->chx);
                DyChOpenInput(Plot[k]->chy);
               }
            DySetDefaultEndTime();
            for (k=0; k<10; k++)
               {
                DyXformCheckGUI();
                if (Dy->Order == STOP) break;
                sleep(1);
               }
           }
       }

     /*--- Update the GPS time on the GUI main panel ---*/

     DyXformUpdateCounters(gtime,0);

     /*--- Loop over plots ---*/

     DyPlotLoop(gtime);

     /*--- Auto-stop if dataDisplay is kept running for a too long time ---*/
     if (Dy->no_autostop == OFF)
       {tt = time(NULL);
        if (tt-Dy->ttstart > ACTIVE_TIMEMAX) Dy->Order = STOP;
       }

     /*--- If stop button pushed during the Plot loop, stop the loop on gtime ---*/
     if (Dy->Order == STOP) gtime = Dy->endTime;
    }

 return;
}


/*--------------------------------------- DyCmLoop ---*/

void DyCmLoop()
{
 double gtime, framelength;
 int i, firstframe, needdata, count;
 double timeNeeded;
 int tt;

 DyDebugPrint(__FUNCTION__,0);

 firstframe = ON;
 needdata = ON;
 count=0;

 while (Dy->Order != STOP)
    {
     /*--- Get a new frame in buffer and update gtime ---*/
     if (needdata == ON)
       {
        DyDebugTimeNeeded("GetCmFrame","start");
        if (((Dy->debugType)&(1<<DBG_READ))) printf("DyCmLoop> Waiting for a new Cm frame...\n");
        DyGetCmFrame();
        gtime = FrFileITEnd(Dy->CmMffile);
        /*--- Update the GPS time on the GUI main panel ---*/
        DyXformUpdateCounters(gtime-framelength,0);
        if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug>0)
           {printf("gtime=%f  frames in buffer are: from %f to %f \n",
            gtime, FrFileITStart(Dy->CmMffile), FrFileITEnd(Dy->CmMffile)-framelength);
           }
        DyDebugTimeNeeded("GetCmFrame","stop");
       }


     /*--- Init the plot time with the last frame in buffer ---*/
     if (firstframe == ON)
       {if (Dy->Frame != NULL) framelength = Dy->Frame->dt;
        else                   framelength = 1;
        for (i=1; i<=Dy->nplot; i++) Plot[i]->gtime = gtime - framelength;
        Dy->startTime = gtime;
        Dy->startLatency = DyGetLatency(gtime-framelength,0);
        firstframe = OFF;
       }

     /*--- Loop over plots and update each one when needed ---*/
     needdata = DyPlotLoop(gtime);

     /*--- Auto-stop if dataDisplay is kept running for a too long time ---*/
     if (Dy->no_autostop == OFF)
       {tt = time(NULL);
        if (tt-Dy->ttstart > ACTIVE_TIMEMAX) Dy->Order = STOP;
       }
    }

 return;
}


/*--------------------------------------------------- DyPlotLoop ---*/

int DyPlotLoop (double gtime)
{
 int i,j, nstep, needdata, npad;
 double timewindow;
 struct DyPlot *plot;
 char dummy[LSTRING_MAX];
 char dummy3[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Order == STOP) return(OFF);

 needdata = OFF;

 /*--- To periodically process any ROOT event like the change of canvas size ---*/
 /*--- Not used here when receiving frames online via Cm, because gSystem->ProcessEvents() activates DyCmFrHandler ---*/
 if (Dy->InputType != INCM) RA_ProcessRootEvents();

 sprintf(dummy,"All plots at gtime=%f", gtime);
 DyDebugTimeNeeded(dummy,"start");

 /*--- Check for user interaction with the GUI ---*/
 DyXformCheckGUI();

 /*--- Loop on plots ---*/
 for (i=1; i<=Dy->nplot; i++)
    {
     /*--- Test if the ROOT canvas has not been closed by the user ---*/
     if (RA_MainCanvasCheck() == OFF)
       {
        npad = DyPlotCheckPad();
        DyWindowsIni(npad);
        printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
        printf("+   Please, do not try to close the plots canvas in run!  +\n");
        printf("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
       }

     plot = Plot[i];

     /*--- If plot is not shown, do not compute it ---*/
     if (plot->hidden == ON) continue;

     /*--- If plot is a superposed one, do not take it in this loop ---*/
     if (plot->superposed == ON) continue;

     sprintf(dummy3,"plot %s - %s : Other than plots", plot->name, plot->title);
     DyDebugTimeNeeded(dummy3,"start");

     /*--- Get the correct timewindow for the plot ---*/
     timewindow = DyPlotGetTimewindow(i);

     /*--- Do not compute plot if data to read is after endTime ---*/

//     nstep = (int)(Dy->maxtimewindow/timewindow);
     nstep = (int)(Dy->maxtimestep/plot->deltat);

     if (((Dy->debugType)&(1<<DBG_READ)))
       {printf("DyPlotLoop> PLOT%d=%s (%s)  plot->gtime=%.1f gtime=%.1f  nstep=%d maxtimestep=%g deltat=%g\n",
               i, plot->name, Dy->typeName[plot->type], plot->gtime, gtime, nstep, Dy->maxtimestep, plot->deltat);
       }

     /*--- Check interaction with the dataDisplay GUI ---*/
      DyCheckRunning();

     DyDebugTimeNeeded(dummy3,"stop");

     while (nstep > 0)
       {
        if (Dy->InputType == INCM && gtime < plot->gtime + 1.1*timewindow) {needdata = ON; break;}
        if (Dy->InputType != INCM && gtime < plot->gtime) break;

        DyPlotProcess(i,timewindow);

        /*--- Process now the plots superposed to this one ---*/
        if (i<Dy->nplot && Plot[i+1]->superposed == ON)
          {for (j=i+1; j<=Dy->nplot; j++)
              {if (Plot[j]->superposed == OFF) break;
               if (Plot[j]->hidden == ON) continue;
               timewindow = DyPlotGetTimewindow(j);
               DyPlotProcess(j,timewindow);
              }
          }
        nstep--;
        if (Dy->InputType == INCM && gtime < plot->gtime + 1.1*timewindow) {needdata = ON; break;}
        if (Dy->InputType != INCM && gtime < plot->gtime) break;
       }

     /*--- If debug sleep is activated, sleep 1s between each plot drawing ---*/
     if (Dy->debugSleep > 0)
       {printf("Sleeping %d seconds ...\n", Dy->debugSleep);
        sleep(Dy->debugSleep);
       }
    }

 DyDebugTimeNeeded(dummy,"stop");

 return(needdata);
}


/*--------------------------------------------------- DyPlotGetTimewindow ---*/

double DyPlotGetTimewindow (int i)
{
 struct DyPlot *plot;
 double timewindow;

 plot = Plot[i];

 /*--- Set the time window used to get data vector ---*/
 if (plot->type == PLOTFFT || plot->type == PLOTTR || plot->type == PLOTCOHE)
   {timewindow = plot->fft_duration;}
 else if (plot->type == PLOT2D)
   {timewindow = plot->refreshDt;}
 else
   {timewindow = plot->deltat;}

 /*--- special treatment for 2d time vectors ---*/
 if (plot->chx && plot->chx->vect2d_duration > 0) timewindow = plot->deltat;
 if (plot->chy && plot->chy->vect2d_duration > 0) timewindow = plot->deltat;

 return(timewindow);
}



/*--------------------------------------------------- DyPlotProcess ---*/

void DyPlotProcess (int i, double timewindow)
{
 struct DyPlot *plot;
 char dummy[256];
 int ii;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[i];

 if (plot->hidden == ON) return;

 sprintf(dummy,"plot %s - %s : Reading data", plot->name, plot->title);
 DyDebugTimeNeeded(dummy,"start");

 /*--- read data vectors ---*/
 if (plot->chx) plot->chx->vect = DyGetVect(plot->chx, plot->gtime, timewindow);
 if (plot->chy) plot->chy->vect = DyGetVect(plot->chy, plot->gtime, timewindow);

 /*--- If plot TRFCT or COHE or 2D done with timeshift between chx and chy ---*/

 if ((plot->type == PLOTTR || plot->type == PLOTCOHE || plot->type == PLOT2D) && plot->timeshift != 0)
   {FrVectFree(plot->chy->vect);
    plot->chy->vect = DyGetVect(plot->chy, plot->gtime-plot->timeshift, timewindow);
   }

 DyDebugTimeNeeded(dummy,"stop");

 /*--- Check 2 for user interaction with the GUI ---*/
// DyXformCheckGUI();

 /*--- process data vectors ---*/

 DyBasicProcessVect(i);
 DyProcessVect(plot->chx, plot->type);
 DyProcessVect(plot->chy, plot->type);

 /*--- Check 3 for user interaction with the GUI ---*/
// DyXformCheckGUI();

 /*--- compute plots functions and fill the plots ---*/

 sprintf(dummy,"plot %s - %s : Computing and Drawing", plot->name, plot->title);
 DyDebugTimeNeeded(dummy,"start");

 if (plot->type == PLOTFFT)           DyPlotFFT(i);
 else if (plot->type == PLOTTIME)     DyPlotTIME(i);
 else if (plot->type == PLOTTR)       DyPlotTRFCT(i);
 else if (plot->type == PLOTCOHE)     DyPlotCOHE(i);
 else if (plot->type == PLOT1D)       DyPlot1D(i);
 else if (plot->type == PLOT2D)       DyPlot2D(i);
 else if (plot->type == PLOTAUDIO)    DyPlotAUDIO(i);
 else if (plot->type == PLOTFFTTIME)  DyPlotFFTTIME(i);
 else if (plot->type == PLOTRAW)      DyPlotRAW(i);
 else if (plot->type == PLOT1DTIME)   DyPlot1DTIME(i);
 else if (plot->type == PLOTRAWTIME)  DyPlotRAWTIME(i);
 else if (plot->type == PLOTTRTIME)   DyPlotTRFCTTIME(i);
 else if (plot->type == PLOTCOHETIME) DyPlotCOHETIME(i);

 DyDebugTimeNeeded(dummy,"stop");

 /*--- Check 4 for user interaction with the GUI (check also for pause button) ---*/
// DyCheckRunning();

 /*--- Increment the plot time by nData*dx[0] = deltat ---*/

 if (plot->type == PLOTFFTTIME || plot->type == PLOTFFT || plot->type == PLOTCOHE || plot->type == PLOTTR ||
     plot->type == PLOTCOHETIME || plot->type == PLOTTRTIME || plot->type == PLOTRAW || plot->type == PLOTRAWTIME) 
   {plot->gtime += plot->deltat;}
 else if (plot->chx->vect != NULL && plot->chx->vect2d_duration == 0)
   {if (plot->chx->vect->GTime > 0) plot->gtime = plot->chx->vect->GTime + plot->chx->vect->nData * plot->chx->vect->dx[0];
    else                            plot->gtime += plot->chx->vect->nData * plot->chx->vect->dx[0];
   }
 else
   {plot->gtime += plot->deltat;}

 return;
}


/*--------------------------------------------------- DyPlotAllIni ---*/

void DyPlotAllIni ()
{
 int i, nsuperposed;
 double xleg, yleg, legsize;
 double starttime, timewindow;
 struct DyPlot *plot;

 starttime = Dy->startTime;

 /*--- If data received online, set starttime as GPS of first frame in reception buffer ---*/

 if (Dy->InputType == INCM)
    {DyGetCmFrame();
     starttime = FrFileITStart(Dy->CmMffile);
    }

 /*--- Init of each plot with data read from file or received online via Cm ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     plot = Plot[i];

     /*--- If plot is not shown, do not initialize it ---*/

     if (plot->hidden == ON) continue;

      plot->gtime = starttime;

     /*--- Check that channel sampFreq is uptodate with current channel sampFreq ---*/

     DyPlotCheckFreq(i);

     /*--- Set the secondary parameters of the plot (those not shown in plot edit panel) ---*/

     DyPlotSetSecondParam(i);

     /*--- Get the value of the timewindow ---*/

     timewindow = DyPlotGetTimewindow(i);
 
     DyPlotIni(i,timewindow);

     /*--- Set legend if several plots are superposed ---*/

     nsuperposed = DyPlotGetNsuperposed(i);
     if (nsuperposed > 0)
       {RA_SetActivePad(plot->numpad, plot->numsubpad);
        DyPlotSetLegendSize(i,&xleg, &yleg, &legsize);
        RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
       }
     else
       {RA_HSetTitle(plot->name, plot->title);}

     /*--- If requested, set a fast ugly font to be used for the axis labels of this plot ---*/
 
     if (Dy->nicefont == OFF) RA_SetLabelFont(DY_FONT_TIMES, plot->name);

     /*--- Set the label size and title size and offset ---*/
/**
     DyPlotSetAllSize();
     RA_SetLabelSize(0.01,Plot[i]->name);
**/
    }

 /*--- Compute the max timewindow value and the max time step value (used later in DyPlotLoop) ---*/

 DyGetMaxTimeStep();

 return;
}


/*--------------------------------------------------- DyGetMaxTimeStep ---*/

void DyGetMaxTimeStep()
{
 int i;
 double timewindow;

 Dy->maxtimewindow = 0;
 Dy->maxtimestep   = 0;

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     timewindow = DyPlotGetTimewindow(i);
     if (timewindow > Dy->maxtimewindow) Dy->maxtimewindow = timewindow;
     if (Plot[i]->deltat > Dy->maxtimestep) Dy->maxtimestep = Plot[i]->deltat;
    }

 return;
}


/*--------------------------------------------------- DyPlotIni ---*/

void DyPlotIni (int i, double timewindow)
{
 struct DyPlot *plot;

 plot = Plot[i];

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("DyPlotIni> plot%d  %s  %s  gtime=%.2f  deltat=%g  chx->name=%s timewindow=%g \n",
            i, plot->name, Dy->typeName[plot->type], plot->gtime, plot->deltat, plot->chx->name, timewindow);
   }

 if (plot->chx) plot->chx->mean = 0;
 if (plot->chy) plot->chy->mean = 0;

 /*--- Read a first time the data vectors, because needed in DyProcessVectIni and in plots Ini ---*/

 if (plot->chx) plot->chx->vect = DyGetVect(plot->chx, plot->gtime, timewindow);
 if (plot->chy) plot->chy->vect = DyGetVect(plot->chy, plot->gtime, timewindow);

 /*--- If plot TRFCT or COHE or 2D done with timeshift between chx and chy ---*/

 if ((plot->type == PLOTTR || plot->type == PLOTCOHE || plot->type == PLOT2D) && plot->timeshift != 0)
   {FrVectFree(plot->chy->vect);
    plot->chy->vect = DyGetVect(plot->chy, plot->gtime-plot->timeshift, timewindow);
   }

 /*--- Pass to ch the passband filter fmin and fmax defined for the plot, if any ---*/
 /* ICI : Put the passband filter at the level of channels instead of plot */

 if (plot->chx) {plot->chx->bpfilter_fmin = plot->filter_fmin; plot->chx->bpfilter_fmax = plot->filter_fmax;}
 if (plot->chy) {plot->chy->bpfilter_fmin = plot->filter_fmin; plot->chy->bpfilter_fmax = plot->filter_fmax;}

 /*--- Ini the buffers needed to process channels ---*/

 DyProcessVectIni(plot->chx);
 DyProcessVectIni(plot->chy);

 /*--- Process a first time the channels so that Plot ini deal with correct data vectors ---*/

 DyBasicProcessVect(i);
 DyProcessVect(plot->chx, plot->type);
 DyProcessVect(plot->chy, plot->type);

 /*--- Call Ini of the plot ---*/

 if (plot->type == PLOTFFT)       DyPlotFFTIni(i);
 if (plot->type == PLOTTIME)      DyPlotTIMEIni(i);
 if (plot->type == PLOT1D)        DyPlot1DIni(i);
 if (plot->type == PLOT2D)        DyPlot2DIni(i);
 if (plot->type == PLOTTR)        DyPlotTRFCTIni(i);
 if (plot->type == PLOTCOHE)      DyPlotCOHEIni(i);
 if (plot->type == PLOTRAW)       DyPlotRAWIni(i);
 if (plot->type == PLOTFFTTIME)   DyPlotFFTTIMEIni(i);
 if (plot->type == PLOT1DTIME)    DyPlot1DTIMEIni(i);
 if (plot->type == PLOTRAWTIME)   DyPlotRAWTIMEIni(i);
 if (plot->type == PLOTTRTIME)    DyPlotTRFCTTIMEIni(i);
 if (plot->type == PLOTCOHETIME)  DyPlotCOHETIMEIni(i);
 if (plot->type == PLOTAUDIO)     DyPlotAUDIOIni(i);

 return;
}


/*--------------------------------------------------- DyPause ---*/

void DyPause ()
{
 Dy->Order = PAUSE;

 if (Dy->InputType == INDD) DyDdPauseRequest (Dy->DdServer);

 DyInfoLine ("Data read paused");

 return;
}


/*--------------------------------------------------- DyContinue ---*/

void DyContinue ()
{
 Dy->Order = CONTINUE;

 if (Dy->InputType == INDD) DyDdContinueRequest (Dy->DdServer);


 return;
}


/*--------------------------------------------------- DyNext ---*/

void DyNext (long arg)
{
 if (arg == 0) Dy->Order = NEXT_UPDATE;
 if (arg == 2) Dy->Order = NEXT_EVENT;

 if (Dy->InputType == INDD) DyDdContinueRequest (Dy->DdServer);

 return;
}


/*---------------------------------------------------  DyStop ---*/

void DyStop()
{

 if (Dy->Order == STOP) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Update the status flag and the AutoConfFlag ---*/

 Dy->Order = STOP;

 return;
}


/*------------------------------------------- DyGetCmFrame ---*/

int DyGetCmFrame()
{
#ifndef NOCM

 char dummy[LSTRING_MAX];
 CmConnectCondition condition;
 long iwait;
 int timeout;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get Cm messages containing frames, if requested ---*/

 iwait = 1;
 timeout = DY_TIMEOUT2*10;

 while (Dy->Order != STOP)
     {
      condition = CmMessageWaitWithTimeout(0.1);
      if (((Dy->debugType)&(1<<DBG_SPEED)) && Dy->debug > 0)
        {printf("DyGetCmFrame> iwait=%d condition=%d (%d) lastframe=%.1f\n",
                iwait, (int)condition, (int)CmConnectBreakDetection, FrFileITEnd(Dy->CmMffile)-1);
        }
      if (condition == CmConnectBreakDetection)
        {if (((Dy->debugType)&(1<<DBG_SPEED)) && Dy->debug > 0) printf("DyGetCmFrame> CmConnectBreak detected !\n");
         break;
        }
      if (iwait > 10 && iwait%timeout == 0 && Dy->InputType == INCM)
         {sprintf(dummy,"No frame received since %g seconds", iwait/10.);
          DyInfoLine(dummy);
         }
      if (iwait > 10) DyXformCheckGUI();
      iwait++;
     }

#endif

 /*--- Put here the process of ROOT events like canvas size change to prevent interaction with CmMessageWait ---*/
 RA_ProcessRootEvents();

 if (Dy->Order == STOP) return(1);
 else                   return(0);
}


/*------------------------------------------- DyCheckRunning ---*/

void DyCheckRunning ()
{
 struct timespec tv;

 tv.tv_sec = 0;
 tv.tv_nsec = 100000000;

 /*--- Do Pause if trigger option chosen, if pause_on_trigger chosen and if trigger start detected ---*/

 if (Dy->triggerUsed == ON && Dy->triggerPause == ON && Dy->triggerEvent == ON)
    {DyXformTrigger(PAUSE);}

 /*--- check the user action ---*/

 DyXformCheckGUI();

 /*--- Checked status of running buttons ---*/

 if (Dy->Order == PAUSE)
    {if (Dy->InputType == INDD) DyDdPauseRequest (Dy->DdServer);}

 /*--- button PAUSE has been pushed ---*/
 while (Dy->Order == PAUSE)
   {nanosleep(&tv, NULL); DyXformCheckGUI();}

 /*--- button NEXT_UPDATE has been pushed ---*/
 if (Dy->Order == NEXT_UPDATE && Dy->UpdateDone == ON)
   {Dy->UpdateDone = OFF; Dy->Order = PAUSE;}

 /*--- button NEXT_EVENT  has been pushed ---*/
 if (Dy->Order == NEXT_EVENT) Dy->Order = CONTINUE;

 return;
}



/*                        ************                     */
/*                        *  DIVERS  *                     */
/*                        ************                     */


/*----------------------------------------------------- quitHandler ---*/

void quitHandler (int arg)
{
 char name[LSTRING_MAX];

 printf("\n");

 /*--- if any plot present, call all "end" procedures and save plots and ntuples ---*/ 

 if (Dy->Order != STOP)
    {
     if (Dy->InputType == INCM || Dy->InputType == INDD) DyFdRemoveFramesRequest (Dy->CmServer, Dy->CmName);
     DyVisuEnd();
     if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("    quitHandler> Reading loop finished\n");
    }

 /*--- Disconnect from Cm Name Server ---*/ 

 if (Dy->nocm == OFF) DyCmDisconnect();
// if (Dy->InputType == INCM || Dy->InputType == INDD) DyCmDisconnect();
 

 /*--- Remove *.C , *.h and *.so files produced by RA_MakeClass (UserFct) ---*/ 

 DyXformRemoveUserProcFiles();

 /*--- Remove *.lis file and print end informations ---*/ 

 sprintf(name,"%s.lis",Dy->curConfig);
 remove(name);
/*
 sprintf(name, "%s%s", Dy->curConfig, CONFIG_SUFFIX);
 DyMakeSiestaFile(name,0);
*/

 if (Dy->QuitAtStop == OFF)
    {
/**     printf("quitHandler>  Plots are in %s%s \n",   Dy->curConfig, ROOT_SUFFIX);  **/
     printf("quitHandler>  Configuration is in %s%s \n", Dy->curConfig, CONFIG_SUFFIX);
     printf("quitHandler>  Aurevoir %s! \n\n", getenv("USER"));
    }
 else
    {
     printf("\n");
     printf("quitHandler>  Run in batch mode with config %s \n", Dy->AutoConf);
     printf("quitHandler>  Aurevoir %s! \n\n", getenv("USER"));
    }

 DyFree();

 exit(0);
}


/*----------------------------------------------------- DyFree ----*/

void DyFree ()
{
 if (Dy->Frame) FrameFree(Dy->Frame);
 if (Dy->Dir) free(Dy->Dir);
 if (Dy->AutoConf) free(Dy->AutoConf);
 if (Dy->Ddchlist) free(Dy->Ddchlist);

 free(Dy);

 return;
}


/*------------------------------------------- my_strcpy ---*/

char *my_strcpy (char *dummy, char *str)
{
 if (!str) return(NULL);
 if (dummy) free(dummy);
 dummy = (char *) calloc(strlen(str)+1,sizeof(char));
 strcpy(dummy, str);

 return(dummy);
}


/*------------------------------------------- max (not used) ---*/

long max (long a, long b)
{
 if (a>b) return a;
 return b;
}


/*------------------------------------------ return_val (not used) ---*/

double return_val (struct FrAdcData *adc, long i)
{
 double val;

 switch(adc->data->type)
   {case FR_VECT_C:  val = (double)adc->data->data[i]; break;
    case FR_VECT_2S: val = (double)adc->data->dataS[i]; break;
    case FR_VECT_4S: val = (double)adc->data->dataI[i]; break;
    case FR_VECT_8S: val = (double)adc->data->dataL[i]; break;
    case FR_VECT_1U: val = (double)adc->data->dataU[i]; break;
    case FR_VECT_2U: val = (double)adc->data->dataUS[i]; break;
    case FR_VECT_4U: val = (double)adc->data->dataUI[i]; break;
    case FR_VECT_8U: val = (double)adc->data->dataUL[i]; break;
    case FR_VECT_4R: val = (double)adc->data->dataF[i]; break;
    case FR_VECT_8R: val = (double)adc->data->dataD[i]; break;
    default: printf("Vector type of adc %s unknown !",adc->name); exit(0);
   }

 return val;
}


/*------------------------------------------- DyGetMinMax ---*/

void DyGetMinMax (struct FrVect *vect, double *vmin, double *vmax)
{
 long i;
 double value;

 if (vect == NULL) {*vmin = 0; *vmax = 0; return;}

 DyDebugPrint(__FUNCTION__,0);

 if (FrVectIsValid(vect) != 0)
    {
     if (((Dy->debugType)&(1<<DBG_OTHERS)))
        {printf("  DyGetMinMax> Found invalid data or zeros in the vector %s.\n",vect->name);}
     *vmin = 0.;
     *vmax = 0.;
     return;
    }

 *vmin =  1e+32;
 *vmax = -1e+32;
 for (i=0; i<vect->nData; i++)
    {
     value = FrVectGetV(vect,i);
     if (vect->next && vect->next->data[i] <= 0) continue;
     if (value > 1e+30) continue;
     if (value < -1e+30) continue;
     if (value>0 && value < 1e-90) continue;
     if (value <0 && value > -1e-90) continue;
     if (i == 0 || *vmin > value) *vmin = value;
     if (i == 0 || *vmax < value) *vmax = value;
    }

 if (*vmin > 1e+30 || *vmin < -1e+30) *vmin = 0;
 if (*vmax > 1e+30 || *vmax < -1e+30) *vmax = 0;

 return;
}


/*------------------------------------------- DyChecks ---*/

void DyChecks ()
{
 int i,nplotmem, nch;

 printf("\n");
 printf("  ------------ TIMING\n");
 printf("  Dy->startTime=%f  Dy->duration=%f  Dy->endTime=%f \n", Dy->startTime, Dy->duration, Dy->endTime);

 printf("\n");
 printf("  ------------ GENERAL PARAMETERS\n");
 printf("  Dy->nplotcreated=%d \n", Dy->nplotcreated);
 printf("  Dy->Order=%d  Dy->InputType=%d (%s)\n", Dy->Order, Dy->InputType, Dy->inputtypeName[Dy->InputType]);
 printf("  Dy->AutoConf=%s  Dy->AutoConfFlag=%d \n", Dy->AutoConf, Dy->AutoConfFlag);
 printf("  Dy->y2d  =%ld  Dy->AutoDeselectPlot=%d  Dy->AutoDeselectCh=%d \n",
           Dy->y2d, Dy->AutoDeselectPlot, Dy->AutoDeselectCh);

 printf("\n");
 printf("  ------------ CHANNELS\n");
 printf("  Dy->nch=%d \n", Dy->nch);
 nch = 20;
 if (nch > Dy->nch) nch = Dy->nch;
 for (i=0; i<nch; i++)
    {if (Dy->ch[i]->sampFreq == 0) break;
     printf("  channel %d : type=%d  name=%s  sampFreq=%g  inputname=%s \n",
            i, Dy->ch[i]->type, Dy->ch[i]->name, Dy->ch[i]->sampFreq, Dy->ch[i]->inputname);
    }

 printf("\n");
 printf("  ------------ TRIGGERS\n");
 printf("  Dy->triggerPause=%d  Dy->triggerUsed=%d  Dy->triggerType=%d  Dy->trigAsciiFileName=\042%s\042  Dy->trigEventName=%s\n",
           Dy->triggerPause, Dy->triggerUsed, Dy->triggerType, Dy->trigAsciiFileName, Dy->trigEventName);
 printf("  Dy->triggerEvent=%d  Dy->lastTrigString=\042%s\042 \n",
           Dy->triggerEvent, Dy->lastTrigString);

 printf("\n");
 printf("  ------------ INPUTS\n");
 if      (Dy->InputType == INCM)     printf("    Data read online from Cm Server %s (my CmName=%s)\n", Dy->CmServer, Dy->CmName);
 else if (Dy->InputType == INDD)     printf("    Data read online from Data Distribution Server %s\n",Dy->DdServer);
 else if (Dy->InputType == INSHM)    printf("    Data read online from shared memory %s\n",Dy->shmName);
 else if (Dy->InputType == INFILE)   printf("    Data read offline from files of directory %s\n", Dy->Dir);

 printf("\n");
 printf("  ------------ OUTPUTS\n");
 printf("  Dy->output_flag=%d  Dy->output_nframe_perfile=%d  Dy->output_nframe_written=%d \n",
           Dy->output_flag, Dy->output_nframe_perfile, Dy->output_nframe_written);
 printf("  Dy->output_dir=%s \n", Dy->output_dir);
 printf("  Dy->output_tag=%s \n", Dy->output_tag);

 printf("\n");
 printf("  ------------ PLOTS\n");
 printf("  Dy->nplots=%d \n", Dy->nplot);
 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON)
       {printf("hidden    Plot[%d]=%s  (type=%d=%s)  deltat=%g  refreshDt=%g  title=%s\n",
               i, Plot[i]->name, Plot[i]->type, Dy->typeName[Plot[i]->type], Plot[i]->deltat, Plot[i]->refreshDt, Plot[i]->title);}
     else
       {printf("shown     Plot[%d]=%s  (type=%d=%s)  deltat=%g  refreshDt=%g  title=%s\n",
                i, Plot[i]->name, Plot[i]->type, Dy->typeName[Plot[i]->type], Plot[i]->deltat, Plot[i]->refreshDt, Plot[i]->title);}
    }

 printf("\n");
 printf("  -----> Plots details \n");
 for (i=1; i<=Dy->nplot; i++)  DyCheckPlot(i);

 nplotmem = RA_CheckAllPlot(0);

 printf("\n");
 printf("  ----> %d Plots in Memory\n", nplotmem);
 RA_CheckAllPlot(1+Dy->debug);
 printf("\n");

 return;
}


/*------------------------------------------- DyCheckPlot ---*/

void DyCheckPlot (long id)
{
 int ndata, k;
 struct DyPlot *plot;

 plot = Plot[id];

 if (plot->hidden == ON) return;

 printf("\n");
 printf("================================= Plot %s  (%s) in pad %d ===========================\n", plot->name, Dy->typeName[plot->type], plot->numpad);
 printf("\n");
 printf("    Plot number                           = id         = %ld \n", id);
 printf("    Plot type                             = type       = %d (%s) \n", plot->type, Dy->typeName[plot->type]);
 printf("    Plot name                             = name       = %s \n", plot->name);
 printf("    Flag to say that plot is hidden       = hidden     = %d\n", plot->hidden);
 printf("    Pad number where plot is shown        = numpad     = %d\n", plot->numpad);
 printf("    Subpad number where plot is shown     = numsubpad  = %d\n", plot->numsubpad);

 printf("    Flag to say that plot is superposed   = superposed = %d\n", plot->superposed);
 printf("    Plot on which is superposed this plot = idsame     = %d\n", plot->idsame);
 printf("    Color of the plot                     = color      = %d\n", plot->color);

 /*--- Information about channels used in the plot ---*/

 if (plot->chx != NULL)
    {
     printf("    Channel x name                        = chx->name         = %s \n", plot->chx->name);
     printf("    Channel x sampling frequency          = chx->sampFreq     = %g \n", plot->chx->sampFreq);
     printf("    Channel x resampling frequency        = chx->resampFreq   = %g \n", plot->chx->resampFreq);
     printf("    Channel x flag to get calibrated data = chx->realunits    = %d \n", plot->chx->realunits);
     printf("    Channel x input name                  = chx->inputname    = %s \n", plot->chx->inputname);
     printf("    Channel x input type                  = chx->inputtype    = %d \n", plot->chx->inputtype);
     if (plot->chx->vect)
       {ndata = plot->chx->vect->nData;
        if (ndata > 3) ndata = 3;
        printf("    Channel x vector                      = chx->vect         = %ld samplings = ", plot->chx->vect->nData);
        for (k=0; k<ndata; k++) printf(" %g", plot->chx->vect->dataD[k]);
        printf(" ...\n");
       }
     else
       {printf("    Channel x vector                      = chx->vect         = NULL \n");}
     if (plot->chx->inputtype == INDY)
       {printf("    Channel x number of input vectors     = chx->op_nch    = %d \n", plot->chx->op_nch);
        for (k=0; k<plot->chx->op_nch; k++)
           {printf("      Channel x input vector %d             = chx->op_chin[%d] = %s \n", k+1, k, plot->chx->op_chin[k]->name);}
       }
    }

 if (plot->chy != NULL)
    {
     printf("    Channel y name                        = chy->name         = %s \n", plot->chy->name);
     printf("    Channel y sampling frequency          = chy->sampFreq     = %g \n", plot->chy->sampFreq);
     printf("    Channel y resampling frequency        = chy->resampFreq   = %g \n", plot->chy->resampFreq);
     printf("    Channel y flag to get calibrated data = chy->realunits    = %d \n", plot->chy->realunits);
     printf("    Channel y input name                  = chy->inputname    = %s \n", plot->chy->inputname);
     printf("    Channel y input type                  = chy->inputtype    = %d \n", plot->chy->inputtype);
     if (plot->chy->vect)
       {ndata = plot->chy->vect->nData;
        if (ndata > 5) ndata = 5;
        printf("    Channel y vector                      = chy->vect         = %ld samplings = ", plot->chy->vect->nData);
        for (k=0; k<ndata; k++) printf(" %g", plot->chy->vect->dataD[k]);
        printf(" ...\n");
       }
     else
       {printf("    Channel y vector                      = chy->vect         = NULL \n");}
     if (plot->chy->inputtype == INDY)
       {printf("    Channel y number of input vectors     = chy->op_nch    = %d \n", plot->chy->op_nch);
        for (k=0; k<plot->chx->op_nch; k++)
           {printf("      Channel y input vector %d             = chy->op_chin[%d] = %s \n", k+1, k, plot->chy->op_chin[k]->name);}
       }
    }

 /*--- Information specific to the plot ---*/

 if (plot->type == PLOTFFT)
    {
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = deltat       = %g (%g%% of fft_duration)\n", plot->deltat, plot->tstep);
     printf("    Number of FFTs before plot refresh    = refresh_fft  = %ld \n", plot->refresh_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g \n", plot->refreshDt);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of samplings in FFT time window   = xnBin        = %ld \n", plot->xnBin);
     printf("    Shift of data between two FFTs        = dNbin        = %ld \n", plot->dNbin);
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Lower limit on y axis of spectrum     = ymin         = %g \n", plot->ymin);
     printf("    Upper limit on y axis of spectrum     = ymax         = %g \n", plot->ymax);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to compute a RMS of the spectrum = rmsfft       = %d \n", plot->rmsfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
    }

 else if (plot->type == PLOTTIME)
    {
     printf("    Time window size (sec)                  = tsize     = %g \n", plot->tsize);
     printf("    Time window shift between two data read = deltat    = %g (%g%% of tsize)\n", plot->deltat, plot->tstep);
     printf("    Nbr of samplings in time window         = xnBin     = %ld \n", plot->xnBin);
     printf("    Shift of data between two data read     = dNbin     = %ld \n", plot->dNbin);
     printf("    Lower limit on y axis                   = ymin      = %g \n", plot->ymin);
     printf("    Upper limit on y axis                   = ymax      = %g \n", plot->ymax);
     printf("    Flag to auto-adjust y axis limits       = autoscale = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data    = nodc      = %d \n", plot->nodc);
     printf("    Scaling factor on the data              = yscale    = %g \n", plot->yscale);
     printf("    Offset applied on the data              = yoffset   = %g \n", plot->yoffset);
     printf("    Min freq of pass-band filter applied    = filter_fmin = %g \n", plot->filter_fmin);
     printf("    Max freq of pass-band filter applied    = filter_fmax = %g \n", plot->filter_fmax);
    }

 else if (plot->type == PLOT1D)
    {
     printf("    Time window shift between two data read = deltat    = %g (%g%% of tsize)\n", plot->deltat, plot->tstep);
     printf("    Time period of plot refresh             = refreshDt = %g \n", plot->refreshDt);
     printf("    Nbr of bins on X axis                   = xnBin     = %ld \n", plot->xnBin);
     printf("    X axis lower edge                       = xmin      = %g \n", plot->xmin);
     printf("    X axis upper edge                       = xmax      = %g \n", plot->xmax);
     printf("    Scaling factor on the data              = xscale    = %g \n", plot->xscale);
     printf("    Offset applied on the data              = xoffset   = %g \n", plot->xoffset);
     printf("    Flag to auto-adjust X axis edges        = autoedge  = %d \n", plot->autoedge);
    }

 else if (plot->type == PLOT2D)
    {
     printf("    Time window shift between two data read = deltat    = %g (%g%% of tsize)\n", plot->deltat, plot->tstep);
     printf("    Time period of plot refresh             = refreshDt = %g \n", plot->refreshDt);
     printf("    Nbr of bins on X axis                   = xnBin     = %ld \n", plot->xnBin);
     printf("    Nbr of bins on Y axis                   = ynBin     = %ld \n", plot->ynBin);
     printf("    X axis lower edge                       = xmin      = %g \n", plot->xmin);
     printf("    X axis upper edge                       = xmax      = %g \n", plot->xmax);
     printf("    Y axis lower edge                       = ymin      = %g \n", plot->ymin);
     printf("    Y axis upper edge                       = ymax      = %g \n", plot->ymax);
     printf("    Flag to auto-adjust X and Y axis edges  = autoedge  = %d \n", plot->autoedge);
     printf("    Time shift between chx and chy          = timeshift = %g \n", plot->timeshift);
    }

 else if (plot->type == PLOTTR)
    {
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = deltat       = %g (%g%% of fft_duration)\n", plot->deltat, plot->tstep);
     printf("    Number of FFTs before plot refresh    = refresh_fft  = %ld \n", plot->refresh_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g \n", plot->refreshDt);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of samplings in FFT time window   = xnBin        = %ld \n", plot->xnBin);
     printf("    Shift of data between two FFTs        = dNbin        = %ld \n", plot->dNbin);
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Lower limit on y axis of spectrum     = ymin         = %g \n", plot->ymin);
     printf("    Upper limit on y axis of spectrum     = ymax         = %g \n", plot->ymax);
     printf("    Time shift between chx and chy        = timeshift    = %g \n", plot->timeshift);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to compute a RMS of the spectrum = rmsfft       = %d \n", plot->rmsfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
    }

 else if (plot->type == PLOTCOHE)
    {
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = deltat       = %g (%g%% of fft_duration)\n", plot->deltat, plot->tstep);
     printf("    Number of FFTs before plot refresh    = refresh_fft  = %ld \n", plot->refresh_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g \n", plot->refreshDt);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of samplings in FFT time window   = xnBin        = %ld \n", plot->xnBin);
     printf("    Shift of data between two FFTs        = dNbin        = %ld \n", plot->dNbin);
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Lower limit on y axis of spectrum     = ymin         = %g \n", plot->ymin);
     printf("    Upper limit on y axis of spectrum     = ymax         = %g \n", plot->ymax);
     printf("    Time shift between chx and chy        = timeshift    = %g \n", plot->timeshift);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to compute a RMS of the spectrum = rmsfft       = %d \n", plot->rmsfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
     printf("    A cross-coherence is computed         = crossCohe    = %d \n", plot->crossCohe);
     printf("    Freq min of cross-coherence plot      = crossCohe_fmin = %g \n", plot->crossCohe_fmin);
     printf("    Freq max of cross-coherence plot      = crossCohe_fmax = %g \n", plot->crossCohe_fmax);
    }

 else if (plot->type == PLOTFFTTIME)
    {
     printf("\n");
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Time resolution                       = dtime        = %g \n", plot->dtime);
     printf("    Time window size (sec)                = tsize        = %g \n", plot->tsize);
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = fft_deltat   = %g (%g%% of fft_duration)\n", plot->fft_deltat, plot->tstep_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g (%g%% of tsize)\n", plot->refreshDt, plot->tstep);
     printf("    Shift of time between two data read   = deltat       = %g \n", plot->deltat);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Lower limit on color axis             = zmin         = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax         = %g \n", plot->zmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of bins on time axis              = xnBin        = %ld \n", plot->xnBin);
     printf("    Nbr of bin on frequency axis          = ynBin        = %ld \n", plot->ynBin);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
     printf("    Flag to normalize by median spectrum  = median_norm  = %d \n", plot->median_norm);
    }

 else if (plot->type == PLOT1DTIME)
    {
     printf("\n");
     printf("    Number of bins on Y axis              = ynBin          = %ld \n", plot->ynBin);
     printf("    Time resolution (stat duration)       = dtime (deltat) = %g (%g) \n", plot->dtime, plot->deltat);
     printf("    Time window size (sec)                = tsize          = %g \n", plot->tsize);
     printf("    Refresh period (sec)                  = refreshDt      = %g (%g%% of tsize)\n", plot->refreshDt, plot->tstep);
     printf("    Lower bound in Y axis                 = ymin           = %g \n", plot->ymin);
     printf("    Upper bound in Y axis                 = ymax           = %g \n", plot->ymax);
     printf("    Lower limit on color axis             = zmin           = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax           = %g \n", plot->zmax);
     printf("    Nbr of bins on time axis              = xnBin          = %ld \n", plot->xnBin);
     printf("    Flag to set ymin and ymax at start    = autoedge       = %d \n", plot->autoedge);
     printf("    Flag to auto-adjust y axis limits     = autoscale      = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc           = %d \n", plot->nodc);
    }

 else if (plot->type == PLOTTRTIME)
    {
     printf("\n");
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Time resolution                       = dtime        = %g \n", plot->dtime);
     printf("    Time window size (sec)                = tsize        = %g \n", plot->tsize);
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = deltat       = %g (%g%% of fft_duration)\n", plot->deltat, plot->tstep_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g (%g%% of tsize)\n", plot->refreshDt, plot->tstep);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Lower limit on color axis             = zmin         = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax         = %g \n", plot->zmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of bins on time axis              = xnBin        = %ld \n", plot->xnBin);
     printf("    Nbr of bin on frequency axis          = ynBin        = %ld \n", plot->ynBin);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
    }

 else if (plot->type == PLOTCOHETIME)
    {
     printf("\n");
     printf("    Frequency resolution                  = dfreq        = %g \n", plot->dfreq);
     printf("    Time resolution                       = dtime        = %g \n", plot->dtime);
     printf("    Time window size (sec)                = tsize        = %g \n", plot->tsize);
     printf("    FFT time window size (sec)            = fft_duration = %g \n", plot->fft_duration);
     printf("    Shift of time window between two FFTs = fft_deltat   = %g (%g%% of fft_duration)\n", plot->fft_deltat, plot->tstep_fft);
     printf("    Refresh period (sec)                  = refreshDt    = %g (%g%% of tsize)\n", plot->refreshDt, plot->tstep);
     printf("    Shift of time between two data read   = deltat       = %g \n", plot->deltat);
     printf("    Lower frequency shown                 = fmin         = %g \n", plot->fmin);
     printf("    Upper frequency shown                 = fmax         = %g \n", plot->fmax);
     printf("    Lower limit on color axis             = zmin         = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax         = %g \n", plot->zmax);
     printf("    Number of FFTs to average             = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Nbr of bins on time axis              = xnBin        = %ld \n", plot->xnBin);
     printf("    Nbr of bin on frequency axis          = ynBin        = %ld \n", plot->ynBin);
     printf("    Flag to set a moving average of FFTs  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to auto-adjust y axis limits     = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc         = %d \n", plot->nodc);
     printf("    Flag to show FFT in 1/Hz              = hertz        = %d \n", plot->hertz);
    }

 else if (plot->type == PLOTRAW)
    {
     printf("\n");
     printf("    Lower limit on color axis             = zmin         = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax         = %g \n", plot->zmax);
     printf("    Number of images to average           = maxAverage   = %ld \n", plot->maxAverage);
     printf("    Flag to set moving average of images  = decayfft     = %d \n", plot->decayfft);
     printf("    Flag to auto-adjust y / z axis limits = autoscale    = %d \n", plot->autoscale);
     printf("    Flag to know if input vector is image = raw2d        = %d \n", plot->raw2d);
     printf("    Number of bins on vect->nx[0] (vectx) = xnBin        = %ld \n", plot->xnBin);
     printf("    Min of vectx                          = xmin         = %g \n", plot->xmin);
     printf("    Max of vectx                          = xmax         = %g \n", plot->xmax);
     if (plot->raw2d == ON)
       {printf("    Number of bins on vect->nx[1] (vecty) = ynBin        = %ld \n", plot->ynBin);
        printf("    Min of vecty                          = ymin         = %g \n", plot->ymin);
        printf("    Max of vecty                          = ymax         = %g \n", plot->ymax);
       }
     printf("\n");
    }

 else if (plot->type == PLOTRAWTIME)
    {
     printf("\n");
     printf("    Time resolution (stat duration)       = dtime (deltat) = %g (%g) \n", plot->dtime, plot->deltat);
     printf("    Time window size (sec)                = tsize          = %g \n", plot->tsize);
     printf("    Refresh period (sec)                  = refreshDt      = %g (%g%% of tsize)\n", plot->refreshDt, plot->tstep);
     printf("    Number of raw vectors to average      = maxAverage     = %ld \n", plot->maxAverage);
     printf("    Lower limit on color axis             = zmin           = %g \n", plot->zmin);
     printf("    Upper limit on color axis             = zmax           = %g \n", plot->zmax);
     printf("    Nbr of bins on time axis              = xnBin          = %ld \n", plot->xnBin);
     printf("    Flag to auto-adjust y axis limits     = autoscale      = %d \n", plot->autoscale);
     printf("    Flag to remove DC part from the data  = nodc           = %d \n", plot->nodc);
    }

 else if (plot->type == PLOTAUDIO)
    {
     printf("\n");
     printf("    Start time of audio recording         = audio_t0       = %g \n", plot->audio_t0);
     printf("    Duration of audio recording           = audio_duration = %g \n", plot->audio_duration);
     printf("    Flag to get audio online              = soundonline    = %d \n", plot->soundonline);
     printf("    Flag to record audio in 16 bits       = sound16bits    = %d \n", plot->sound16bits);
    }

 else
    {printf("   ******************************** \n");
     printf("   NO SPECIFIC INFO for plot %s \n", Dy->typeName[plot->type]);
     printf("   ******************************** \n");
    }

 /*--- Last generic parameters ---*/

 printf("    Type of 2D plot (SCAT=%d, COLZ=%d, ...)   = type2d       = %d \n", SCAT, COLZ, plot->type2d);
 printf("    X grid visible on plot                  = gridx        = %d \n", plot->gridx);
 printf("    Y grid visible on plot                  = gridy        = %d \n", plot->gridy);
 printf("    Z grid visible on plot                  = gridz        = %d \n", plot->gridz);
 printf("    Log scale on X axis                     = logx         = %d \n", plot->logx);
 printf("    Log scale on Y axis                     = logy         = %d \n", plot->logy);
 printf("    Log scale on Z axis                     = logz         = %d \n", plot->logz);

 return;
}


/*------------------------------------------------------- DyUTC2GPS ---*/
/* this function converts a UTC time to a GPS time (inspired from FrameLib)  */

double DyUTC2GPS (double utc)
{
 double gps, uLeapS;

 /*-- first guess ---*/

 gps = utc - FRGPSTAI + 37 - FRGPSOFF;

 /*---- update leap second according to the gps time ----*/

 uLeapS = FrGetLeapS(gps);

 /*--- Set the correct gps time --*/

 gps = utc - FRGPSTAI + uLeapS - FRGPSOFF;

 return(gps);
}


/*-------------------------------------- DyGTimeFromDate ---*/

int DyGTimeFromDate (char *inputDate)   /* inputDate in UTC, output is GTimeS */
{
 char chaine[256];
 char words[32][32], *word;
 int nword, i, dum1, dum2, dum3;
 int year, month, day, hour, minute, second, gtime, uleaps;

 DyDebugPrint(__FUNCTION__,0);

 if (inputDate == NULL)             return(0);
 if (strstr(inputDate,":") == NULL) return(0);

/**
 if (strstr(inputDate,"-") != NULL)
    {length = strlen(inputDate);
     for (i=0; i<length; i++) {if (inputDate[i] == '-') inputDate[i] = ' ';}
    }
**/

 strcpy(chaine,inputDate);
 word  = strtok(chaine," ");
 nword = 0;
 while (word != NULL)
    {
     strcpy(words[nword],word);
     nword++;
     word = strtok(NULL," ");
    }

 year = 0; month = 0; day = 0; hour = 0; minute = 0; second = 0;

 for (i=0; i<nword; i++)
    {

     if (strstr(words[i],"/") == NULL  &&  strstr(words[i],"-") == NULL  &&  strstr(words[i],":") == NULL)
        {
         if (strstr(words[i],"Jan")  || strstr(words[i],"jan"))  month =  1;
         if (strstr(words[i],"Feb")  || strstr(words[i],"feb"))  month =  2;
         if (strstr(words[i],"Fev")  || strstr(words[i],"fev"))  month =  2;
         if (strstr(words[i],"Mar")  || strstr(words[i],"mar"))  month =  3;
         if (strstr(words[i],"Apr")  || strstr(words[i],"apr"))  month =  4;
         if (strstr(words[i],"Avr")  || strstr(words[i],"avr"))  month =  4;
         if (strstr(words[i],"May")  || strstr(words[i],"may"))  month =  5;
         if (strstr(words[i],"Mai")  || strstr(words[i],"mai"))  month =  5;
         if (strstr(words[i],"Jun")  || strstr(words[i],"jun"))  month =  6;
         if (strstr(words[i],"Juin") || strstr(words[i],"juin")) month =  6;
         if (strstr(words[i],"Jul")  || strstr(words[i],"jul"))  month =  7;
         if (strstr(words[i],"Juil") || strstr(words[i],"juil")) month =  7;
         if (strstr(words[i],"Aug")  || strstr(words[i],"aug"))  month =  8;
         if (strstr(words[i],"Aout") || strstr(words[i],"aout")) month =  8;
         if (strstr(words[i],"Sep")  || strstr(words[i],"sep"))  month =  9;
         if (strstr(words[i],"Oct")  || strstr(words[i],"oct"))  month = 10;
         if (strstr(words[i],"Nov")  || strstr(words[i],"nov"))  month = 11;
         if (strstr(words[i],"Dec")  || strstr(words[i],"dec"))  month = 12;

         if (atoi(words[i]) > 59)   year = atoi(words[i]);
         else if (atoi(words[i]) > 0) day  = atoi(words[i]);
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d-%d-%d", &dum1, &dum2, &dum3) == 3)
        { 
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d/%d/%d", &dum1, &dum2, &dum3) == 3)
        { 
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }

     dum1 = 0; dum2 = 0; dum3 = 0;
     if (sscanf(words[i], "%d:%d:%d", &dum1, &dum2, &dum3) == 3)
        {
         if (dum3 > 59)  {day = dum1; month = dum2; year = dum3;}
         if (dum1 > 59)  {year = dum1; month = dum2; day = dum3;}
         else            {hour = dum1; minute = dum2; second = dum3;}
        }
    }

 gtime = (DyJulianDay(year,month,day) - DyJulianDay(1980,1,6)) * 86400.
       + hour*3600. + minute*60. + second + 37 - FRGPSTAI;
 uleaps = FrGetLeapS(gtime);
 gtime = gtime + uleaps - 37;
 
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
    {printf("    DyGTimeFromDate> Found date:  %d/%d/%d  %d:%d:%d\n", day, month, year, hour, minute, second);
     printf("    DyGTimeFromDate> Found gtime: %d  (%s)\n", gtime, FrStrUTC(gtime,0));
    }
           
 return(gtime);
}


/*----------------------------------------- DyJulianDay ---*/

int DyJulianDay(int y, int m, int d)
{
 int jd;

 jd = ( 1461 * ( y + 4800 + ( m - 14 ) / 12 ) ) / 4 +
               ( 367 * ( m - 2 - 12 * ( ( m - 14 ) / 12 ) ) ) / 12 -
               ( 3 * ( ( y + 4900 + ( m - 14 ) / 12 ) / 100 ) ) / 4 +
               d - 32075;
 return(jd);
}



/*----------------------------------------- DyMissingFrames ---*/
/* obsolete */

int DyMissingFrames (int previous, struct FrameH *frame)
{
 int dframe, dgtime, current;
 char errorMsg[LSTRING_MAX];

 if (frame == NULL) return(previous);

 DyDebugPrint(__FUNCTION__,0);

 /*--- Using frame GPS time ---*/

 if (frame->dt > 0.)
    {
     current = frame->GTimeS;
     if ((double)(current-previous) > 1.99*frame->dt  &&  previous > 0)
        {dgtime = (int)(((double)current - (double)previous) / frame->dt)-1;
         sprintf(errorMsg,"%d frames missing (between %d and %d)", dgtime, previous, current);
         DyInfoLine(errorMsg);
        }
    }

 /*--- Using frame number ---*/

 else
    {
     current = frame->frame;
     if (current != previous + 1  &&  previous >= 0)
        {dframe = current - previous;
         if (current < previous) dframe += 32767;
         sprintf(errorMsg,"%d frames missing (between fr%d and fr%d)", dframe, previous, current);
         DyInfoLine(errorMsg);
        }
     }

 return(current);
}


/*-------------------------------------- DyGetMin ---*/

double DyGetMin (double x1, double x2)
{
 if (x1 > x2) return(x2);
 return(x1);
}


/*-------------------------------------- DyGetMax ---*/

double DyGetMax (double x1, double x2)
{
 if (x1 > x2) return(x1);
 return(x2);
}


/*-------------------------------------- DyDebugTimeNeeded ---*/

double DyDebugTimeNeeded (const char *fctname, const char *flag)
{
 double currentTime, dtime;
 static int level = 0;
 int i,i0;

 if (!((Dy->debugType)&(1<<DBG_SPEED))) return(0);

 /*--- If flag is NULL or contains "start", set the starting time of fctname at level [level] ---*/

 if (flag == NULL || strstr(flag,"start") != NULL)
    {
     gettimeofday(&dytv, NULL);
     ptime[level] = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
     strcpy(fctnamep[level],fctname);
     dtime = 0.;
     level++;
     if (level > DBGTIME_LEVELMAX) level = DBGTIME_LEVELMAX;
    }

 /*--- If flag contains "stop", set the stop time and print the time evolved between stop and start ---*/

 if (strstr(flag, "stop") != NULL)
    {
     i0 = -1;
     for (i=level-1; i>=0; i--)
        {if (strcmp(fctname,fctnamep[i]) == 0)
           {gettimeofday(&dytv, NULL);
            currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
            dtime = currentTime - ptime[i];
            if (Dy->debug >= i)
              {if (i == 0)      printf("\033[31;1m%s : time=%.2f ms (level=%d) \033[0m \n", fctname, dtime*1000, i);
               else if (i == 1) printf("\033[34;1m%s : time=%.2f ms  (level=%d) \033[0m \n", fctname, dtime*1000, i);
               else             printf("\033[1m%s : time=%.2f ms  (level=%d) \033[0m \n", fctname, dtime*1000, i);
              }
            ptime[i] = 0.;
            strcpy(fctnamep[i],DYEMPTY);
            i0 = i;
            break;
           }
        }
     if (i0 == -1)
        {printf("**%s - time needed=[not evaluated]\n", fctname);
         dtime = 0.;
        }
     else
        {level = i0;}
    }

 return(dtime);
}


/*-------------------------------------- DyDebugPrint ---*/

void DyDebugPrint (const char *msg, int printflag)
{
 double currentTime;

 /*--- Use this function for functions call only when printflag is on ---*/
 if (!((Dy->debugType)&(1<<DBG_CALLS)) && printflag == 0) return;

 /*--- Print current time with the message ---*/

 if (printflag > 0)
    {gettimeofday(&dytv, NULL);
     currentTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );
     printf("\033[34;1m--%s - current_time=%.3f \033[0m \n", msg, currentTime);
    }

 /*--- Print only the provided message ---*/

 else
    {printf("\033[34;1m--%s \033[0m \n", msg);}

 return;
}


/*-------------------------------------- DyCheckMemory ---*/

double DyCheckMemory()
{
 int i;
 long maxAverage;
 double dymem, memused, tsize, tstep, fft_duration, stat_duration;

 DyDebugPrint(__FUNCTION__,0);

 dymem = 0.;
 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->tsize > 0) tsize = Plot[i]->tsize;
     else                    tsize = 1.;
 
     if (Plot[i]->tstep > 0) tstep = Plot[i]->tstep;
     else                    tstep = 1.;
 
     if (Plot[i]->fft_duration > 0) fft_duration = Plot[i]->fft_duration;
     else                           fft_duration = 1.;

     if (Plot[i]->stat_duration > 0) stat_duration = Plot[i]->stat_duration;
     else                            stat_duration = 1.;

     if (Plot[i]->maxAverage > 0) maxAverage = Plot[i]->maxAverage ;
     else                         maxAverage  = 1.;

     if (Plot[i]->type == PLOT2D)
        {memused = 8. * Plot[i]->xnBin * Plot[i]->ynBin;}

     else if (Plot[i]->type == PLOTFFTTIME)
        {memused = 8. * (Plot[i]->nBin/2) * Plot[i]->tsize / (maxAverage * fft_duration * tstep / 100.);}

     else if (Plot[i]->type == PLOT1DTIME)
        {memused = 8. * Plot[i]->nBin * Plot[i]->tsize / stat_duration;}

     else if (Plot[i]->type == PLOTRAWTIME)
        {memused = 8. * Plot[i]->nBin * Plot[i]->tsize / stat_duration;}

     else
        {memused = 8. * Plot[i]->xnBin;}

     dymem += memused;

     if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
        {printf("DyCheckMemory> plot%d=%s  xnBin=%ld  ynBin=%ld \n"
                "DyCheckMemory> fft_duration=%.4f  tsize=%.4f  maxAverage=%ld \n"
                "DyCheckMemory> memused=%.3fMB=8*%ld*%.2f dymem=%.3fMB\n",
                 i, Plot[i]->name, Plot[i]->xnBin, Plot[i]->ynBin,
                 fft_duration, Plot[i]->tsize, maxAverage, 
                 memused/1024./1024., Plot[i]->xnBin/2, Plot[i]->tsize/(maxAverage*fft_duration*10/100.), dymem/1024./1024.);
        }
    }

 dymem += 10.*1024.*1024.;   /* on 8 June 2004: TOC size for rawdata=8MB  trenddata=13MB */

 if (dymem/1024./1024. > 120.)
    {printf("DyCheckMemory> Estimated memory consumption = %.3f MBytes\n", dymem/1024./1024.);}

 return(dymem/1024./1024.);
}


/*-------------------------------------------------- DyCheckString ----*/

char *DyCheckString (char* name)
{
 unsigned char c;
 char *localName;

 if (name == NULL)       return(NULL);
 if (strlen (name) == 0) return(NULL);

 localName = name;

 while (*localName != 0)
   {
    c = (unsigned char) *localName;
    if (strchr (". \t~!^*()-+\"'|\\<>?/", c) != NULL) *localName = '_'; 
    localName++;
   }

 return(name);
}


/*--------------------------------------------------- DyFilterString ----*/

char *DyFilterString (char* name, unsigned char separator)
{
 unsigned char c;
 char *localName;
 int length;

 if (name == NULL)       return(NULL);
 if (strlen (name) == 0) return(NULL);

 length = strlen(name);
 localName = name;

 while (*localName != 0)
   {
    c = (unsigned char) *localName;
    if (c == separator) *localName = ' '; 
    if (c == ' ' && localName > name && localName < name+length-1) *localName = separator; 
    localName++;
   }

 return(name);
}

/*--------------------------------------------------- DyCountChannels ---*/

int DyCountChannels (struct FrameH *frame)
{
 int nadc, nser, nproc, nsim, nch;
 struct FrAdcData *adc;
 struct FrSerData *ser;
 struct FrProcData *proc;
 struct FrSimData *sim;

 if (frame == NULL) return(0);

 nadc = 0;
 nser = 0;
 if (frame->rawData)
    {adc = frame->rawData->firstAdc; while (adc != NULL) {nadc++; adc = adc->next;}
     ser = frame->rawData->firstSer; while (ser != NULL) {nser++; ser = ser->next;}
    }

 nproc = 0;
 proc = frame->procData;
 while (proc != NULL) {nproc++; proc = proc->next;}

 nsim = 0;
 sim = frame->simData;
 while (sim != NULL) {nsim++; sim = sim->next;}

 nch = nadc + nser;

 return(nch);
}


/**************************************************/
/*                                                */
/* File: DyVar.c                                  */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*                        **************                     */
/*                        *  VARIABLE  *                     */
/*                        **************                     */

/*------------------------------------------- DyDecodeVar ---*/
/* ICI  : obsolete? */

void DyDecodeVar (char *inputString, char **smsName, char **varName)
{
 long length;
 char *temp;

 temp = strstr(inputString,SEP);

 if (temp != NULL && strncmp(inputString,OPSIGNAL,strlen(OPSIGNAL)) != 0)
    {
     *varName = my_strcpy(*varName, &temp[strlen(SEP)]);
     length   = strlen(inputString)-strlen(temp);
     *smsName = (char *) calloc(length+1,sizeof(char));
     strncpy(*smsName,inputString,length);
    }
 else
    {
     *smsName = NULL;
     *varName = NULL;
    }

 return;
}

/*------------------------------------------ DyUpdateChList ---*/

void DyUpdateChList()
{
 int i, nch, iinput, j, alreadyread;
 char inputname[64][LSTRING_MAX];
 int inputtype[64];

 DyDebugPrint(__FUNCTION__,0);

 nch = Dy->nch;

 /*--- Update channels list by connecting to each input once ---*/

 iinput = 0;
 for (i=0; i<Dy->nch; i++)
    {
     alreadyread = OFF;
     for (j=0; j<iinput; j++)
        {if (strcmp(Dy->ch[i]->inputname, inputname[j]) == 0 && Dy->ch[i]->inputtype == inputtype[j])
           {alreadyread = ON; break;}
        }
     if (alreadyread == OFF)
       {strcpy(inputname[iinput],Dy->ch[i]->inputname);
        inputtype[iinput] = Dy->ch[i]->inputtype;
        iinput++;
        DyReadFirstFrame(Dy->ch[i]->inputname, Dy->ch[i]->inputtype);
       }
    }

 /*--- If no channel in the list, try to do an update using available plots ---*/

 if (Dy->nch == 0)
   {
    for (i=1; i<=Dy->nplot; i++)
       {if (i>1 && strcmp(Plot[i]->chx->inputname,Plot[i-1]->chx->inputname) == 0 &&
                  Plot[i]->chx->inputtype == Plot[i-1]->chx->inputtype) continue;
        DyReadFirstFrame(Plot[i]->chx->inputname, Plot[i]->chx->inputtype);
        if (Plot[i]->chy)
          {if (i>1 && strcmp(Plot[i]->chy->inputname,Plot[i-1]->chy->inputname) == 0 &&
                      Plot[i]->chy->inputtype == Plot[i-1]->chy->inputtype) continue;
           DyReadFirstFrame(Plot[i]->chy->inputname, Plot[i]->chy->inputtype);
          }
       }
   }

 printf("DyUpdateChList> Number of channels: %d --> %d \n", nch, Dy->nch);

 return;
}


/*-------------------------------------------------- DyChListToFrame ---*/

struct FrameH *DyChListToFrame (struct FrameH *frame)
{
#ifndef NOCM

 struct FrSerData *smsChList;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 double chFreq, chMin, chMax;
 int i, nData, nlist;
 char chName[LSTRING_MAX], word0[LSTRING_MAX], type[LSTRING_MAX], chlistName[LSTRING_MAX];
 char *chaine, *sentence;

 if (frame == NULL) return(NULL);

 DyDebugPrint(__FUNCTION__,0);

 nlist = 0;
 while (1)
   {
    sprintf(chlistName,"%s_%d",CHANNELS_LIST, nlist);

    if (nlist == 0) smsChList = FrSerDataFind (frame, CHANNELS_LIST, NULL);
    else            smsChList = FrSerDataFind (frame, chlistName, NULL);

   if (((Dy->debugType)&(1<<DBG_OTHERS)))
     {printf("  DyChListToFrame> Reading %s (smsChList=%p)\n", chlistName, smsChList);}

    if (smsChList == NULL) break;

    chaine = (char *)malloc(strlen(smsChList->data)+1);
    strcpy(chaine, smsChList->data);
    sentence = strtok(chaine,"\n");

    while (sentence != NULL)
       {
        sscanf(sentence,"%s",type);

        if (strcmp(type,"ADC") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("        DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            adc = FrAdcDataNew (frame, chName, chFreq, nData, -32);
            adc->data->dataF[0] = chMin;
            adc->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  adc->data->dataF[i] = chMax;
           }
        if (strcmp(type,"PROC") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("        DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            proc = FrProcDataNew (frame, chName, chFreq, nData, -32);
            proc->data->dataF[0] = chMin;
            proc->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  proc->data->dataF[i] = chMax;
           }
        if (strcmp(type,"SIM") == 0)
           {
            sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
            if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>1) printf("        DyChListToFrame> %s %s %g %g %g\n", word0, chName, chFreq, chMin, chMax);
            nData = (int)(chFreq*frame->dt);
            if (nData <= 0) nData = 2;
            sim = FrSimDataNew (frame, chName, chFreq, nData, -32);
            sim->data->dataF[0] = chMin;
            sim->data->dataF[1] = chMax;
            for (i=1; i<nData; i++)  sim->data->dataF[i] = chMax;
           }
        sentence = strtok(NULL,"\n");
       }

    free(chaine);
    nlist++;
   }

#endif

 return(frame);
}


/*-------------------------------------------------- DyCharChListToFrame ---*/

struct FrameH *DyCharChListToFrame (char *chaine)
{
 struct FrSerData *sms;
 struct FrAdcData *adc;
 struct FrProcData *proc;
 struct FrSimData *sim;
 double chFreq, chMin, chMax;
 int nData;
 long length;
 char chName[LSTRING_MAX], word0[LSTRING_MAX], type[LSTRING_MAX], dummy[LBIGSTRING_MAX];
 char *sentence, *chData, *str, *str2;
 struct FrameH *frame;

 if (chaine == NULL) return(NULL);

 DyDebugPrint(__FUNCTION__,0);

 frame = FrameHNew("channelsList");

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1) printf("          DyCharChListToFrame> channelsList=%s\n", chaine);

 sentence = strtok(chaine,"\n");

 /*** ICI : If file contained only serData, the string chaine (chList) returned by
      FrFileIChannelList and passed here contains no SER (to be investigated) ***/

 while (sentence != NULL)
    {
     sscanf(sentence,"%s",type);

     if (strcmp(type,"FIRSTFRAME") == 0)
        {
         sscanf(sentence,"%s %d %hu %lf %d %d", word0,
                          &(frame->GTimeS), &(frame->ULeapS), &(frame->dt),
                          &(frame->frame), &(frame->run));
         if (((Dy->debugType)&(1<<DBG_OTHERS)))
            {printf("        DyCharChListToFrame> FIRSTFRAME: %d %d %g %d %d\n", frame->GTimeS, frame->ULeapS, frame->dt, (int)frame->frame, frame->run);}
        }

/*
     if (strcmp(type,"SMS") == 0  ||  strcmp(type,"SER") == 0)
        {
         sscanf(sentence,"%s %s %lf %s", word0, chName, &chFreq, dummy);
         str  = strstr(sentence,dummy);

         if (str)
            {str2 = strstr(str,"\n");
             if (str2 == NULL) length = strlen(sentence)-((long)str-(long)sentence);
             else              length = (long)str2 - (long)str;
             chData = (char *)calloc(1, length+1);
             strncpy(chData,str,length-1);
             chData[length]='\0';
             sms = FrSerDataNew (frame, chName, frame->GTimeS, chData, chFreq);
             free(chData);
            }
        }
*/

     if (strcmp(type,"ADC") == 0 || strcmp(type,"SMS") == 0  ||  strcmp(type,"SER") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         adc = FrAdcDataNew (frame, chName, chFreq, nData, -32);
         adc->data->nData    = nData;
         adc->data->dataF[0] = chMin;
         adc->data->dataF[1] = chMax;
        }

     if (strcmp(type,"PROC") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         proc = FrProcDataNew (frame, chName, chFreq, nData, -32);
         proc->data->nData    = nData;
         proc->data->dataF[0] = chMin;
         proc->data->dataF[1] = chMax;
        }

     if (strcmp(type,"SIM") == 0)
        {
         sscanf(sentence,"%s %s %lf %lf %lf", word0, chName, &chFreq, &chMin, &chMax);
         nData = (int)(chFreq*frame->dt);
         if (nData <= 0) nData = 2;
         sim = FrSimDataNew (frame, chName, chFreq, nData, -32);
         sim->data->nData    = nData;
         sim->data->dataF[0] = chMin;
         sim->data->dataF[1] = chMax;
        }

     sentence = strtok(NULL,"\n");
    }

 return(frame);
}


/*------------------------------------------ DyGetSmsVarList ---*/

int DyGetSmsVarList (int *ifirst, char **wordlist, struct FrSerData *smsStation)
{
 char *current, name[LSTRING_MAX], *start;
 long length, nword;

 if (smsStation == NULL) return(0);

 /*--- Get the list of words of the string ---*/

 start    = smsStation->data;
 length   = strlen(start);
 current  = start;
 nword    = 0;
 
 while (current-start < length)
    {sscanf(current,"%s",name);
     wordlist[nword] = (char *)calloc(strlen(name)+1, sizeof(char));
     memcpy(wordlist[nword],name,strlen(name));
     while(*current==' ' || *current=='\n') current++;
     current += strlen(name)+1;
     if (current>=start+length) break;
     while(*current==' ' || *current=='\n') current++;
     nword++;
    }

 if (nword < 4)
    {*ifirst = 0;}
 else
    {
     if (strcmp(wordlist[3],"ALL") == 0  ||  strcmp(wordlist[3],"UPD") == 0) *ifirst = 6;
     else                                                                    *ifirst = 0;
    }

 return(nword);
}


/*------------------------------------------- DyChFix2d ----*/
/* this fix is active if the channel name contains "_sample" */
/* and if data vector is found in the frame with nDim>1 and dx[1]>0 */

int DyChFix2d (struct FrameH *frame, struct DyCh *ch)
{
 struct FrVect *vect;

 DyDebugPrint(__FUNCTION__,0);

 if (frame == NULL) {printf("DyChFix2d> %s : frame is NULL\n", ch->name); return(0);}

 vect = FrameFindVect(frame, ch->name);

 if (vect == NULL)     {printf("DyChFix2d> %s : vect is NULL\n", ch->name); return(0);}
 if (vect->nDim <= 1)  {printf("DyChFix2d> %s : vect->nDim=%d \n", ch->name, vect->nDim); return(0);}
 if (vect->dx[1] <= 0) {printf("DyChFix2d> %s : vect->dx[1]=%g \n", ch->name, vect->dx[1]); return(0);}
 if (strstr(vect->unitX[1],"ime") == NULL) {printf("DyChFix2d> %s : unitX[1]=%s \n", vect->unitX[1]); return(0);}

 ch->sampFreq = 1./vect->dx[1];
 ch->resampFreq = 1./vect->dx[1];
 ch->vect2d_deltat = vect->dx[0];
 ch->vect2d_duration = vect->nx[1] * vect->dx[1];
 
 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
   {printf("DyChFix2d> ch->name=%s  sampFreq=%g  vect2d_deltat=%g  vect2d_duration=%g \n",
            ch->name, ch->sampFreq, ch->vect2d_deltat, ch->vect2d_duration);
    if (Dy->debug > 1) FrVectDump(vect,stdout,2);
   }

 return(1);
}


/*------------------------------------------- DyChFill ----*/

void DyChFill (struct FrameH *frame, char *inputname)
{
 struct FrSerData *sms;
 struct FrAdcData *adc;
 struct FrSimData *sim;
 struct FrProcData *proc;
 char dummy[LSTRING_MAX];
 long i, j, i0;
 char  **wordlist;
 int nword, ifirst, nch0;
 struct FrFile *inputf;
 struct FrameH *inputframe;
 double gtime;
 int ntry, ntrymax, found, foundmax;
#ifndef NOCM
 CmConnectCondition condition2;
#endif

 if (frame == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Update the plots already present with the samplingRates of the updated list of channels ---*/

/* ICI Still useful to call UpdateDataType ? */
/**
 if (Dy->AutoConfFlag == OFF) DyUpdateDataType(frame);
**/

 /*--- Keep memory of the current number of channels already in the list ---*/

 nch0 = Dy->nch;

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> In: Dy->nch=%d\n", Dy->nch);

 /*--- Fill channels list with PROC ---*/

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> Filling channels table with procData\n");
 proc = frame->procData;
 while(proc)
    {
     i0 = -1;
     for (i=0; i<nch0; i++)
        {if (strcmp(Dy->ch[i]->name,proc->name) == 0) {i0=i; break;}}
     if (i0 == -1) 
        {
         Dy->ch[Dy->nch] = (struct DyCh *)calloc(1,sizeof(struct DyCh));
         strcpy(Dy->ch[Dy->nch]->name, proc->name);
         strcpy(Dy->ch[Dy->nch]->inputname, inputname);
         Dy->ch[Dy->nch]->inputtype = Dy->InputType;
         Dy->ch[Dy->nch]->sampFreq  = 1./proc->data->dx[0];
         Dy->ch[Dy->nch]->type      = PROCDATA;
         Dy->nch++;
         if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0 && Dy->nch % 1000 == 0)
           {printf("DyChFill> Added %d channels up to now: last one is %s\n", Dy->nch, Dy->ch[Dy->nch-1]->name);}
         if (Dy->nch >= NCHMAX)
           {printf("DyChFill> Can not manage more than %d channels\n", NCHMAX);
            Dy->nch--;
            break;
           }
        }
     else
        {strcpy(Dy->ch[i0]->inputname, inputname);
         for (j=1; j<=Dy->nplot; j++)
            {if (Plot[j]->chx && strcmp(Plot[j]->chx->name,proc->name) == 0) strcpy(Plot[j]->chx->inputname, inputname);
             if (Plot[j]->chy && strcmp(Plot[j]->chy->name,proc->name) == 0) strcpy(Plot[j]->chy->inputname, inputname);
            }
        }
     proc = proc->next;
    }

 /*--- Fill channels list with SIM---*/

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> Filling channels table with simData\n");
 sim = frame->simData;
 while(sim)
    {
     i0 = -1;
     for (i=0; i<nch0; i++)
        {if (strcmp(Dy->ch[i]->name,sim->name) == 0) {i0=i; break;}}
     if (i0 == -1)
        { 
         Dy->ch[Dy->nch] = (struct DyCh *)calloc(1,sizeof(struct DyCh));
         strcpy(Dy->ch[Dy->nch]->name, sim->name);
         strcpy(Dy->ch[Dy->nch]->inputname, inputname);
         Dy->ch[Dy->nch]->inputtype = Dy->InputType;
         Dy->ch[Dy->nch]->sampFreq  = 1./sim->data->dx[0];
         Dy->ch[Dy->nch]->type      = SIMDATA;
         Dy->nch++;
         if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0 && Dy->nch % 1000 == 0)
           {printf("DyChFill> Added %d channels up to now: last one is %s\n", Dy->nch, Dy->ch[Dy->nch-1]->name);}
         if (Dy->nch >= NCHMAX)
           {printf("DyChFill> Can not manage more than %d channels\n", NCHMAX);
            Dy->nch--;
            break;
           }
        }
     else
        {strcpy(Dy->ch[i0]->inputname, inputname);
         for (j=1; j<=Dy->nplot; j++)
            {if (Plot[j]->chx && strcmp(Plot[j]->chx->name,sim->name) == 0) strcpy(Plot[j]->chx->inputname, inputname);
             if (Plot[j]->chy && strcmp(Plot[j]->chy->name,sim->name) == 0) strcpy(Plot[j]->chy->inputname, inputname);
            }
        }
     sim = sim->next;
    }


 /*--- Check if rawdata present ---*/
 if (frame->rawData == NULL)
    {printf("DyChFill> no raw data in this frame \n"); return;}


 /*--- Fill channels list with SMS ---*/

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> Filling channels table with serData\n");
 sms  = frame->rawData->firstSer;
 while (sms != NULL)
    {
     if (strcmp(sms->name,CHANNELS_LIST) != 0)
        {wordlist = (char **)calloc(65536,sizeof(char *));
         nword = DyGetSmsVarList (&ifirst, wordlist, sms);
         for (i=ifirst; i<nword; i=i+2)
            {
             if (strcmp(wordlist[i],"units") == 0) {free(wordlist[i]); continue;}
             if (strcmp(wordlist[i],"slope") == 0) {free(wordlist[i]); continue;}
             if (strcmp(wordlist[i],"bias") == 0)  {free(wordlist[i]); continue;}
             sprintf(dummy,"%s%s%s", sms->name, SEPSMS, wordlist[i]);
             i0 = -1;
             for (j=0; j<nch0; j++)
                {if (strcmp(Dy->ch[j]->name,dummy) == 0) {i0=j; break;}}
             if (i0 == -1)
               { 
                Dy->ch[Dy->nch] = (struct DyCh *)calloc(1,sizeof(struct DyCh));
                strcpy(Dy->ch[Dy->nch]->name, dummy);
                strcpy(Dy->ch[Dy->nch]->inputname, inputname);
                Dy->ch[Dy->nch]->inputtype = Dy->InputType;
                Dy->ch[Dy->nch]->sampFreq  = sms->sampleRate;
                Dy->ch[Dy->nch]->type      = SERDATA;
                Dy->nch++;
                if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0 && Dy->nch % 1000 == 0)
                  {printf("DyChFill> Added %d channels up to now: last one is %s\n", Dy->nch, Dy->ch[Dy->nch-1]->name);}
                if (Dy->nch >= NCHMAX)
                  {printf("DyChFill> Can not manage more than %d channels\n", NCHMAX);
                   Dy->nch--;
                   break;
                  }
               }
             else
               {strcpy(Dy->ch[i0]->inputname, inputname);
                for (j=1; j<=Dy->nplot; j++)
                   {if (Plot[j]->chx && strcmp(Plot[j]->chx->name,dummy) == 0) strcpy(Plot[j]->chx->inputname, inputname);
                    if (Plot[j]->chy && strcmp(Plot[j]->chy->name,dummy) == 0) strcpy(Plot[j]->chy->inputname, inputname);
                   }
               }
             free(wordlist[i]);
            }
         free(wordlist);
        }
     sms = sms->next;
    }


 /*--- Fill channels list with ADC ---*/

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> Filling channels table with adcData\n");
 adc  = frame->rawData->firstAdc;
 while(adc)
    {
     i0 = -1;
     for (i=0; i<nch0; i++)
        {if (strcmp(Dy->ch[i]->name,adc->name) == 0) {i0=i; break;}}
     if (i0 == -1) 
        { 
         Dy->ch[Dy->nch] = (struct DyCh *)calloc(1,sizeof(struct DyCh));
         strcpy((Dy->ch[Dy->nch])->name, adc->name);
         strcpy((Dy->ch[Dy->nch])->inputname, inputname);
         Dy->ch[Dy->nch]->inputtype = Dy->InputType;
         Dy->ch[Dy->nch]->sampFreq  = adc->sampleRate;
         Dy->ch[Dy->nch]->type      = ADCDATA;
         Dy->nch++;
         if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0 && Dy->nch % 1000 == 0)
           {printf("DyChFill> Added %d channels up to now: last one is %s\n", Dy->nch, Dy->ch[Dy->nch-1]->name);}
         if (Dy->nch >= NCHMAX)
           {printf("DyChFill> Can not manage more than %d channels\n", NCHMAX);
            Dy->nch--;
            break;
           }
        }
     else
        {strcpy(Dy->ch[i0]->inputname, inputname);
         for (j=1; j<=Dy->nplot; j++)
            {if (Plot[j]->chx && strcmp(Plot[j]->chx->name,adc->name) == 0) strcpy(Plot[j]->chx->inputname, inputname);
             if (Plot[j]->chy && strcmp(Plot[j]->chy->name,adc->name) == 0) strcpy(Plot[j]->chy->inputname, inputname);
            }
        }
     adc = adc->next;
    }

 /*--- Special treatment for 2d time vectors ---*/

 foundmax = 0;
 for (i=0; i<Dy->nch; i++)
    {if (strstr(Dy->ch[i]->name, "_sample") != NULL) foundmax++;}

 if (foundmax > 0)
   {
    /*--- If _sample channels exists and data received online, request a frame to get their parameters ---*/
    if (Dy->InputType == INCM)
       {
        if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0)
          {printf("DyChFill> Found %d high sampling channels *_sample. Requesting now a full frame from %s to manage them.\n",
                   foundmax, Dy->CmServer);
          }
        DyFdRequestFrames(Dy->CmServer, Dy->CmName, "*_sample*", 0, 0);
        condition2 = DyCmWait(DY_TIMEOUT1, Dy->CmServer);
        if (condition2 != CmConnectTimeoutDetection)
          {
           if (((Dy->debugType)&(1<<DBG_READ)))
             {printf("DyChFill> Got a frame from Cm connection .... to check for 2d time vectors \n");}
           DyFdRemoveFramesRequest (Dy->CmServer, Dy->CmName);
           for (i=0; i<Dy->nch; i++)
              {if (strstr(Dy->ch[i]->name, "_sample") != NULL) found += DyChFix2d(inputFrame, Dy->ch[i]);}
          }
       }

    /*--- If _sample channels exists and data read from file, read a frame to get their parameters ---*/
    else
       {
        inputf = FrFileINew(inputname);
        gtime = FrFileITEnd(inputf) - 1;
        ntrymax=20; ntry=0; found = 0;
        while (ntry < ntrymax && found < foundmax)
          {
           inputframe = FrameReadTChnl(inputf,gtime,"*_sample*");
           for (i=0; i<Dy->nch; i++)
              {if (strstr(Dy->ch[i]->name, "_sample") != NULL && strcmp(Dy->ch[i]->inputname, inputname) == 0)
                 {found += DyChFix2d(inputframe, Dy->ch[i]);
                 }
              }
           ntry++;
           if (inputframe) gtime = gtime - inputframe->dt*1.1;
           else            gtime = gtime - 1.1;
           if (inputframe) FrameFree(inputframe);
          }
        FrFileIEnd(inputf);
       }
   }

 /*--- Update the sampling frequency of the plots channels if needed ---*/
 for (i=1; i<=Dy->nplot; i++) DyUpdatePlotParam(frame,i);

 /*--- Update the inputs of the plots channels if needed ---*/
 if (Dy->previousInputType == INCM || Dy->InputType == INCM)
   {for (i=1; i<=Dy->nplot; i++) DyUpdatePlotInput(i);}

 /*--- Alphabetic sort of the channels list ---*/

 DyChSort();

 /*--- Finally fill the GUI channels browser ---*/

 DyXformChannelFillBrowser();

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  DyChFill> Added %d channels in the browser\n", Dy->nch-nch0);

 return;
}


/*--------------------------------------------- DySortCmList ---*/

void DySortCmList()
{
 int i,j;
 char dummy[LSTRING_MAX];

 if (Dy->ncm < 2) return;

 for (i=0; i<Dy->ncm-1; i++)
    {
     for (j=i+1; j<Dy->ncm; j++)
        {
         if (strcmp(Dy->cmlist[i], Dy->cmlist[j]) > 0)
            {
             strcpy(dummy,Dy->cmlist[i]);
             strcpy(Dy->cmlist[i], Dy->cmlist[j]);
             strcpy(Dy->cmlist[j],dummy);
            }
        }
    }

 return;
}

/*--------------------------------------------- DySortDdList ---*/

void DySortDdList()
{
 int i,j;
 char dummy[LSTRING_MAX];

 if (Dy->ndd < 2) return;

 for (i=0; i<Dy->ndd-1; i++)
    {
     for (j=i+1; j<Dy->ndd; j++)
        {
         if (strcmp(Dy->ddlist[i], Dy->ddlist[j]) > 0)
            {
             strcpy(dummy,Dy->ddlist[i]);
             strcpy(Dy->ddlist[i], Dy->ddlist[j]);
             strcpy(Dy->ddlist[j],dummy);
            }
        }
    }

 return;
}

/*--------------------------------------------- DySortDdFileList ---*/

void DySortDdFileList()
{
 int i,j;
 char dummy[LSTRING_MAX];

 if (Dy->nddfile < 2) return;

 for (i=0; i<Dy->nddfile-1; i++)
    {
     for (j=i+1; j<Dy->nddfile; j++)
        {
         if (strcmp(Dy->ddfilelist[i], Dy->ddfilelist[j]) > 0)
            {
             strcpy(dummy,Dy->ddfilelist[i]);
             strcpy(Dy->ddfilelist[i], Dy->ddfilelist[j]);
             strcpy(Dy->ddfilelist[j],dummy);
            }
        }
    }

 return;
}

/*--------------------------------------------- DySortShmList ---*/

void DySortShmList()
{
 int i,j;
 char dummy[LSTRING_MAX];

 if (Dy->nshm < 2) return;

 for (i=0; i<Dy->nshm-1; i++)
    {
     for (j=i+1; j<Dy->nshm; j++)
        {
         if (strcmp(Dy->shmlist[i], Dy->shmlist[j]) > 0)
            {
             strcpy(dummy,Dy->shmlist[i]);
             strcpy(Dy->shmlist[i], Dy->shmlist[j]);
             strcpy(Dy->shmlist[j],dummy);
            }
        }
    }

 return;
}

/*--------------------------------- Used by DyChSort ---*/
static int DyChCmpName(const void *p1, const void *p2)
  {
   struct DyCh *ch1, *ch2;

   ch1 = *(struct DyCh * const *)p1;
   ch2 = *(struct DyCh * const *)p2;

   return strcmp(ch1->name, ch2->name);
  }

/*--------------------------------- Used by DyChSortByFreq ---*/
static int DyChCmpFreq(const void *p1, const void *p2)
  {
   struct DyCh *ch1, *ch2;

   ch1 = *(struct DyCh * const *)p1;
   ch2 = *(struct DyCh * const *)p2;

   if (ch1->sampFreq == ch2->sampFreq)     return(0);
   else if (ch1->sampFreq > ch2->sampFreq) return(1);
   else                                    return(-1);
  }


/*--------------------------------------------- DyChSort ---*/

void DyChSort()
{
 int i,j;
 struct DyCh *chdummy;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->nch < 2) return;
 if (Dy->nch > 50000) printf("DyChSort> Sorting %d channels. Please, wait...\n", Dy->nch);

 qsort(Dy->ch, Dy->nch, sizeof(struct DyCh *), (int (*) (const void*, const void*)) DyChCmpName);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyChSort> Sorted %d channels \n", Dy->nch);

 return;
}


/*--------------------------------------------- DyChSortByFreq ---*/

void DyChSortByFreq()
{
 int i,j,jstart, jstop;
 struct DyCh *chdummy;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->nch < 2) return;

 qsort(Dy->ch, Dy->nch, sizeof(struct DyCh *), (int (*) (const void*, const void*)) DyChCmpFreq);

 jstart=0;
 for (i=0; i<Dy->nch-1; i++)
    {
     if (Dy->ch[i]->sampFreq != Dy->ch[i+1]->sampFreq)
       {jstop=i+1;
        qsort(&(Dy->ch[jstart]), jstop-jstart, sizeof(struct DyCh *), (int (*) (const void*, const void*)) DyChCmpName);
        jstart = i+1;
       }
    }

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyChSortByFreq> Sorted %d channels \n", Dy->nch);

 return;
}


/*-------------------------------------------- DyChRemove ---*/

void DyChRemove (int i0)
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 if (i0 < 0) return;

 for (i=i0; i<Dy->nch-1; i++)
    {
     DyChCopy(Dy->ch[i], Dy->ch[i+1]);
    }

 free(Dy->ch[Dy->nch-1]);
 (Dy->nch)--;

 return;
}


/*-------------------------------------------- DyChCopy ---*/

void DyChCopy (struct DyCh *chout, struct DyCh *chin)
{
 int i;

 if (chout == NULL) return;
 if (chin == NULL)  return;

 chout->type            = chin->type;
 chout->status          = chin->status;
 strcpy(chout->name, chin->name);
 chout->sampFreq        = chin->sampFreq;
 chout->resampFreq      = chin->resampFreq;
 chout->realunits       = chin->realunits;
 chout->slope           = chin->slope;
 chout->bias            = chin->bias;
 strcpy(chout->inputname, chin->inputname);
 chout->inputtype       = chin->inputtype;
 chout->bpfilter_fmin   = chin->bpfilter_fmin;
 chout->bpfilter_fmax   = chin->bpfilter_fmax;
 chout->vect2d_duration = chin->vect2d_duration;
 chout->vect2d_deltat   = chin->vect2d_deltat;

 chout->op_nch          = chin->op_nch;
 if (chin->op_classname) strcpy(chout->op_classname, chin->op_classname);
 if (chin->op_string)    strcpy(chout->op_string, chin->op_string);
 if (chin->op_cstring)   strcpy(chout->op_cstring, chin->op_cstring);
 for (i=0; i< chin->op_nch; i++)
    {chout->op_chin[i] = DyChClone(chin->op_chin[i]); /* free(chin->op_chin[i]);*/}

 return;
}


/*------------------------------------------ DyChFilter ---*/

int DyChFilter(const char *filtertag)
{
 char name[64][64], dummy[LSTRING_MAX], *str;
 int i, j, nname, todelete, length, namelength, nkeep, ndel, ftype;
 struct DyCh **chkeep, **chdel;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->nch < 3) return(0);

 /*--- Cases of no filter to apply anymore ---*/

 if (filtertag == NULL || strcmp(filtertag,"") == 0 || strcmp(filtertag,"*") == 0)
   {DyChSort();
    return(1);
   }

 /*--- Allocate for filtered channel names storage ---*/

 chkeep = (struct DyCh **)malloc(Dy->nch*sizeof(struct DyCh *));
 chdel  = (struct DyCh **)malloc(Dy->nch*sizeof(struct DyCh *));

 /*--- Search for channels prefix ---*/
 DyChGetPrefix();

 /*--- Search for names in the filter tag ---*/

 strcpy(dummy,filtertag);
 length = strlen(filtertag);
 nname = 0;
 str  = strtok(dummy,"*");
 while (str != NULL) {strcpy(name[nname],str); str = strtok(NULL,"*"); nname++;}
 if (nname == 0) {DyChSort(); return(1);}

 if (filtertag[length-1] != '*' && filtertag[0] != '*')      ftype = 1;
 else if (filtertag[0] != '*')                               ftype = 2;
 else if (filtertag[length-1] != '*' && filtertag[0] == '*') ftype = 3;
 else if (filtertag[length-1] == '*' && filtertag[0] == '*') ftype = 4;
 else                                                        ftype = 5;

 /*--- Search for channel names that do not contain the filter tag names ---*/

 nkeep = 0;
 ndel  = 0;
 for (i=0; i<Dy->nch; i++)
    {
     todelete = 0;
     str = Dy->ch[i]->name;
     namelength = strlen(Dy->ch[i]->name);

     for (j=0; j<nname; j++)
        {
         /*--- If filter tag is one name without wildcard ---*/
         if (nname == 1 && ftype == 1)
            {if (strcasecmp(Dy->ch[i]->name, filtertag) != 0)
                {sprintf(dummy,"%s%s", Dy->chPrefix, filtertag);
                 if (strcasecmp(Dy->ch[i]->name, dummy) != 0) {todelete = 1; break;}
                }
             break;
            }

         /*--- If filter tag does not begin with a wildcard ---*/
         if (j == 0 && ftype == 2)
            {if (strncasecmp(str, name[j], strlen(name[j])) != 0) {todelete = 1; break;}
             else str += strlen(name[j]);
             continue;
            }

         /*--- If filter tag begins with a wildcard but do not end with a wildcard ---*/
         if (j == nname-1 && ftype == 3)
            {if (strcasestr(str, name[j]) == NULL) {todelete = 1; break;}
             else if (strncasecmp(Dy->ch[i]->name + namelength-strlen(name[j]), name[j], strlen(name[j])) != 0) {todelete = 1; break;}
             break;
            }

         /*--- General case, search for name within the remaining part of channel name ---*/

         if (strcasestr(str, name[j]) == NULL) {todelete = 1; break;}
         else str += strlen(name[j]);
        }

     /*--- According to the todelete value, clone the channel in chkeep or chdel array ---*/
     if (todelete == 1) {chdel[ndel] = DyChClone(Dy->ch[i]); ndel++;}
     else               {chkeep[nkeep] = DyChClone(Dy->ch[i]); nkeep++;}
    }

 /*--- Remove all the list of channels

 for (i=0; i<Dy->nch; i++) {free(Dy->ch[i]);}

 /*--- Put back at beginning of the list all the channels selected ---*/

 Dy->nch = 0;
 for (i=0; i<nkeep; i++)
    {Dy->ch[Dy->nch] = DyChClone(chkeep[i]);
     if (Dy->ch[Dy->nch] != NULL) Dy->nch++;
    }
 if (nkeep > 0) DyChSort();
 for (i=0; i<nkeep; i++) free(chkeep[i]);
 free(chkeep);

 /*--- Put back at the end of the list all the deleted channels ---*/

 for (i=0; i<ndel; i++)
    {Dy->ch[Dy->nch] = DyChClone(chdel[i]);
     if (Dy->ch[Dy->nch] != NULL) Dy->nch++;
    }
 for (i=0; i<ndel; i++) free(chdel[i]);
 free(chdel);

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
   {printf("DyChFilter> nch=%d  nkeep=%d  ndel=%d  sizeofch=%d\n", Dy->nch, nkeep, ndel, sizeof(struct DyCh));}

 return(ndel);
}


/*----------------------------------- Search for a prefix in the channels names ---*/

void DyChGetPrefix()
{
 char *str;
 int i, lprefix;

 DyDebugPrint(__FUNCTION__,0);

 strcpy(Dy->chPrefix,"");

 if (Dy->nch < 6) return;

 for (i=0; i<5; i++)
    {
     str = strstr(Dy->ch[i]->name,":");
     if (str && *(str+1) != ' ')
       {lprefix = str-Dy->ch[i]->name+1;
        if (lprefix < 32) {strncpy(Dy->chPrefix, Dy->ch[i]->name, lprefix); *(Dy->chPrefix+lprefix) = '\0'; break;}
       }
    }

 for (i=Dy->nch-5; i<Dy->nch; i++)
    {
     str = strstr(Dy->ch[i]->name,":");
     if (str && *(str+1) != ' ')
       {lprefix = str-Dy->ch[i]->name+1;
        if (lprefix < 32) {strncpy(Dy->chPrefix, Dy->ch[i]->name, lprefix); *(Dy->chPrefix+lprefix) = '\0'; break;}
       }
    }

 return;
}


/*------------------------------------------- DyChDescription ---*/

void DyChDescription (char *chfile, char *chname)
{
 char line[LSTRING_MAX];
 int commentflag, found;
 FILE *fp;

 fp = fopen(chfile,"r");
 if (fp == NULL) return;

 if (strncmp(chname,"V1:",3) == 0) chname = &chname[3];

 found = OFF;
 while(fgets(line, LSTRING_MAX, fp) != NULL)
   {
    if (line[0] == '#') continue;
    if (strstr(line,"/*") != NULL) commentflag = ON;
    if (strstr(line,"*/") != NULL) {commentflag = OFF; continue;}
    if (commentflag == ON) continue;
    if (strlen(line) < 2) continue;
    if (strstr(line,chname) != NULL) {printf("%s", line); found = ON; break;}
   }

 if (found == OFF) printf("%s \t No description found \n", chname);

 fclose(fp);

 return;
}


/*------------------------------------------- DyGetInputVectors ---*/

void DyGetInputVectors(struct DyCh *ch)
{
 int nword;
 char *word, chaine[LBIGSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get from the channel name the list of input vector names separated by OPSEP ---*/

 strcpy(chaine,ch->name);

 word = strtok(chaine,OPSEP);
 nword = 0;
 while (word != NULL)
    {
     strcat(ch->op_string,word);
     nword++;
     word = strtok(NULL,OPSEP);
    }
 ch->op_nch = nword;

 return;
}

/*------------------------------------------- DyGetVect ---*/

struct FrVect *DyGetVect(struct DyCh *ch, double gtime, double duration)
{
 struct FrVect *vect;
 char dummy[LSTRING_MAX];
 struct FrVect *invects[64];
 struct FrVect *vect2;
 double lowFreq, fastFreq;
 int nx1, i, j, j0, dj0;
 long nsample;

 DyDebugPrint(__FUNCTION__,0);
 
 if (ch == NULL)           {printf("DyGetVect> ch is NULL\n"); return(NULL);}
 if (ch->sampFreq == 0)    {printf("DyGetVect> ch=%s ch->sampFreq is 0 \n", ch->name); return(NULL);}
 if (strlen(ch->name) < 2) {printf("DyGetVect> ch->name=%s is of length<2\n", ch->name); return(NULL);}

 /*--- First delete any previous vector read ---*/

 if (ch->vect) {FrVectFree(ch->vect); ch->vect = NULL;}

 /*--- In case input channel is not the output of an operation: read directly the data vector ---*/

 if (ch->inputtype == INFILE)
   {
    if (ch->ifile == NULL) {printf("DyGetVect> ch->ifile is NULL (%s not opened)\n", ch->inputname); return(NULL);}
    if (ch->realunits == ON) vect = FrFileIGetVectDN(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
    else                     vect = FrFileIGetVectD(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
   }

 else if (ch->inputtype == INSHM)
   {
    if (ch->ifile == NULL) {printf("DyGetVect> ch->ifile is NULL (%s not opened)\n", ch->inputname); return(NULL);}
    if (ch->realunits == ON) vect = FrFileIGetVectDN(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
    else                     vect = FrFileIGetVectD(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
   }

 else if (ch->inputtype == INCM)
   {
    if (ch->ifile == NULL) {printf("DyGetVect> ch->ifile is NULL (%s not opened)\n", ch->inputname); return(NULL);}
    if (ch->realunits == ON) vect = FrFileIGetVectDN(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
    else                     vect = FrFileIGetVectD(ch->ifile,ch->name,gtime+0.1/ch->sampFreq,duration);
    if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 1)
      {if (ch->sampFreq > 1)
         {if (vect == NULL)
            {printf("DyGetVect> Cm channel=%s vect=NULL  gtime=%.1f [%.1f,%1.f]\n",
                    ch->name, gtime, FrFileITStart(Dy->CmMffile), FrFileITEnd(Dy->CmMffile));
            }
          else
            {printf("DyGetVect> Cm channel=%s vect=%p nData=%ld gtime=%.1f [%.1f,%1.f]\n",
                    ch->name, vect, vect->nData, vect->GTime, FrFileITStart(Dy->CmMffile), FrFileITEnd(Dy->CmMffile));
            }
         }
       else
         {printf("  DyGetVect> Cm channel %s vect=(serData at %gHz) gtime=%.1f [%.1f,%1.f]\n",
               ch->name, ch->sampFreq, gtime, FrFileITStart(Dy->CmMffile), FrFileITEnd(Dy->CmMffile));
         }
      }
   }

 else if (ch->inputtype == INDY)
   {
    for (j=0; j<ch->op_nch; j++)
       {ch->op_chin[j]->realunits = ON;
        invects[j] = DyGetVect(ch->op_chin[j], gtime, duration);
        if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 1)
          {printf("  DyGetVect> DyOp:  invects[%d]=%s nData=%ld  op_chin=%s\n",
                   j, invects[j]->name, invects[j]->nData, ch->op_chin[j]->name);
          }
       }
    vect = DyCreateOpVect(ch, invects);
    RA_UserProcess (ch->op_classname, ch->op_nch, invects, vect, NULL);
    for (j=0; j<ch->op_nch; j++) {if (invects[j] != NULL) FrVectFree(invects[j]);}
   }

 /*--- If SerData, current Framelib cannot use FrFileIGetVect, meanwhile read the frames ---*/

 if (vect == NULL && ch->inputtype == INCM)
   {vect = DyGetSerDataVect(ch, gtime, duration);}


 /*--- If data vector not found, create an empty one ---*/
 /*--- If data vector found, Check availability and validity of the data vector ---*/

 if (vect == NULL)
   {
    sprintf(dummy,"%s is missing between gps=%.3f and gps=%.3f. Replacing it with zeros", ch->name, gtime, gtime+duration);
    DyInfoLine(dummy);
    if (ch->vect2d_duration > 0) nsample = (int)(ch->vect2d_duration * ch->sampFreq);
    else                         nsample = (int)(ch->resampFreq * duration);
    vect = FrVectNewTS(ch->name, ch->sampFreq, nsample, -64);
    vect->GTime = gtime;
    for (j=0; j<vect->nData; j++) vect->dataD[j] = DY_NOCHANNEL * 1.01;
    vect->next = FrVectNewTS("Available_data", ch->sampFreq, nsample, -64);
    for (j=0; j<vect->next->nData; j++) vect->next->data[j] = 0;
    ch->status = DY_NOCHANNEL;
   }
 else
   {ch->status = DyCheckVect(vect, ch, gtime, duration);}

 if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 0)
   {printf("DyGetVect> Vector %s  GTime=%.3f nData=%ld dx=%g \n",
            vect->name, vect->GTime, vect->nData, vect->dx[0]);
   }


 if (ch->vect2d_duration > 0)
   {
    if (((Dy->debugType)&(1<<DBG_READ)) && Dy->debug > 1)
      {printf("DyGetVect> vect=%s  nDim=%d  dx[1]=%g  unitX[1]=%s nData=%d \n",
              vect->name, vect->nDim, vect->dx[1], vect->unitX[1], vect->nData);
       FrVectDump(vect,stdout,2);
      }
   }

/*
ICI: lors de la suppression d'un plot ne pas oublier de faire un free de ch->vect et de mettre ch->vect a null
*/

 return(vect);
}


/*------------------------------------------- DyGetSerDataVect ---*/

struct FrVect *DyGetSerDataVect(struct DyCh *ch, double gtime, double duration)
{
 struct FrVect *vect;
 struct FrSerData *ser;
 char *str;
 char smsname[LSTRING_MAX], sername[LSTRING_MAX], varname[LSTRING_MAX];
 int is;
 double tt, framedt , previousvalue, value, tt0;
 struct FrameH *fr;

 fr = FrameReadT(ch->ifile,gtime);

 if (fr == NULL) {printf("DyGetSerDataVect> frame is NULL at time %f \n", gtime); return(NULL);}

 framedt = fr->dt;
 if (framedt == 0) framedt = 1;

 ser = NULL;
 for (tt=gtime; tt<gtime+duration; tt=tt+framedt)
    {
     fr = FrameReadT(ch->ifile,tt);
     if (fr == NULL) continue;

     str = strstr(ch->name,"_");
     while (str != NULL)
       {
        strncpy(smsname,ch->name, str-ch->name);
        smsname[str-ch->name] = '\0';
        ser = FrSerDataFind(fr, smsname, NULL);
        if (ser != NULL)
          {strcpy(sername,smsname);
           strcpy(varname,str+1);
           previousvalue = FrSerDataGetValue(fr, sername, varname, DY_NOCHANNEL * 1.01);
           if (previousvalue < DY_NOCHANNEL)
             {tt0 = tt;
              tt = gtime+duration;
              break;
             }
          }
        str = strstr(str+1,"_");
       }
     FrameFree(fr);
    }

 if (ser == NULL) return(NULL);

 vect = FrVectNewTS(ch->name, ch->sampFreq, (int)(duration * ch->sampFreq), -64);
 vect->GTime = tt0;
 is = 0;
 for (tt=tt0; tt<gtime+duration; tt=tt+1./ch->sampFreq)
    {
     fr = FrameReadT(ch->ifile,tt);
     if (fr == NULL) value = DY_NOCHANNEL * 1.01;
     else            value = FrSerDataGetValue(fr, sername, varname, DY_NOCHANNEL * 1.01);
     if (value > DY_NOCHANNEL) ch->status = DY_NOCHANNEL;
     vect->dataD[is] = value;
/*
     if (value > DY_NOCHANNEL) vect->dataD[is] = previousvalue;
     else                      {vect->dataD[is] = value; previousvalue = value;}
*/
     is++;
     if (fr) FrameFree(fr);
    }

 return(vect);
}


/*------------------------------------------- DyCreateOpVect ---*/

struct FrVect *DyCreateOpVect(struct DyCh *ch, struct FrVect **invects)
{
 long i, j, k, kstart, kend, missvect, nData_max;
 struct FrVect *outvect;

 DyDebugPrint(__FUNCTION__,0);
 
 /*--- Check for a fully missing input vector and compute the nData max ---*/

 missvect = OFF;
 nData_max = 0;
 for (i=0; i<ch->op_nch; i++)
    {
     if (invects[i] == NULL)
       {missvect = ON;}
     else
       {if (invects[i]->next && invects[i]->next->data[0] == 0 && invects[i]->next->data[invects[i]->nData-1] == 0) missvect = ON;
        if (nData_max < invects[i]->nData) nData_max = invects[i]->nData;
       }
    }

 /*--- Compute the duration of the output vector ---*/
/*
 duration = nData_max / ch->sampFreq;
*/
 /*--- Create the output vector ---*/
 
 outvect = FrVectNewTS(ch->name, ch->sampFreq, nData_max, -64);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 1)
   {printf("DyCreateOpVect> outvect: %s %g %d \n", outvect->name, ch->sampFreq, nData_max);
    for (i=0; i<ch->op_nch; i++)
       {if (invects[i]) printf("  invects[%d]=%s\n", i, invects[i]->name);
        else            printf("  invects[%d]=NULL\n", i);
       }
   }

 /*--- Set the Gtime of the output vector ---*/

 for (i=0; i<ch->op_nch; i++) {if (invects[i]) {outvect->GTime = invects[i]->GTime; break;}}

 /*--- If one input vector is missing, then do as if no output vector was created ---*/

 if (missvect == ON) {FrVectFree(outvect); return(NULL);}

 /*--- Check for missing sample in the input vectors and manage output vector samplings accordingly ---*/

 for (i=0; i<ch->op_nch; i++)
    {if (invects[i]->next != NULL)
       {if (outvect->next == NULL)
          {outvect->next = FrVectNewTS("Available_data", ch->sampFreq, nData_max, -64);
           for (k=0; k<outvect->next->nData; k++) outvect->next->data[k] = 1;
          } 
        for (j=0; j<invects[i]->nData; j++)
           {if (invects[i]->next->data[j] == 0)
              {kstart = j * nData_max / invects[i]->nData;
               kend   = (j+1) * nData_max / invects[i]->nData;
               for (k=kstart; k<kend; k++) {outvect->dataD[k] = 1.01 * DY_NOCHANNEL; outvect->next->data[k] = 0;}
              }
           }
        }
    }

 return(outvect);
}


/*------------------------------------------- DyCheckVect ---*/

double DyCheckVect(struct FrVect *vect, struct DyCh *ch, double gtime, double duration)
{
 int i;
 double chstatus;
 char dummy[LSTRING_MAX];

 chstatus = ON;

 /*---- Check validity of data ---*/

 if (FrVectIsValid(vect) != 0)
   {
    sprintf(dummy, "DyCheckVect> Corrupted data in %s in some samplings between gps=%f and gps=%f", ch->name, gtime, gtime+duration);
    DyInfoLine(dummy);
    if (vect->next != NULL)
      {for (i=0; i<vect->nData; i++)
          {/*--- If auxiliary vector shows that sampling is corrupted ---*/
           if (vect->next->data[i] == -1) vect->dataD[i] = DY_BADDATA * 1.01;
          }
      }
    chstatus = DY_BADDATA;
   }

 /*--- Check missing samples ---*/

 else if (vect->next != NULL && strncmp(vect->next->name,"Available_data",14) == 0)
   {
    sprintf(dummy,"DyCheckVect> Missing samples in %s somewhere between gps=%f and gps=%f", ch->name, gtime, gtime+duration); 
    DyInfoLine(dummy);
    for (i=0; i<vect->nData; i++)
       {/*--- If auxiliary vector shows that sampling is missing ---*/
        if (vect->next->data[i] == 0) vect->dataD[i] = DY_NOCHANNEL * 1.01;
       }
    chstatus = DY_NOCHANNEL;
   }


 return(chstatus);
}


/*------------------------------------------- DyBasicProcessVect ---*/

void DyBasicProcessVect(int iplot)
{
 struct DyPlot *plot;
 int i,nmean;

 plot = Plot[iplot];

 /*--- Optional: subtract the DC component ---*/

 if (plot->nodc == ON)
   {
    if (plot->chx) DyChZeroMean(plot->chx);
    if (plot->chy) DyChZeroMean(plot->chy);
   }

 /*--- Optional: scale and offset in TIME plot ---*/

 if (plot->type == PLOTTIME && (plot->yscale != 0 || plot->yoffset != 0))
   {
    if (plot->chx) DyChScaleOffset(plot->chx, plot->yscale, plot->yoffset);
   }

 /*--- Optional: scale and offset in 1D plot ---*/

 if (plot->type == PLOT1D && (plot->xscale != 0 || plot->xoffset != 0))
   {
    if (plot->chx) DyChScaleOffset(plot->chx, plot->xscale, plot->xoffset);
   }

 return;
}

/*------------------------------------------- DyChZeroMean ---*/

void DyChZeroMean(struct DyCh *ch)
{
 int nmean, i;

 if (ch == NULL)       return;
 if (ch->vect == NULL) return;

 nmean = 1;
 for (i=0; i<ch->vect->nData; i++)
    {if (ch->vect->dataD[i] > DY_NOCHANNEL || ch->vect->dataD[i] > DY_BADDATA) continue;
     ch->mean += ch->vect->dataD[i];
     nmean++;
    }
 ch->mean = ch->mean / nmean;
 for (i=0; i<ch->vect->nData; i++)
    {if (ch->vect->dataD[i] > DY_NOCHANNEL || ch->vect->dataD[i] > DY_BADDATA) continue;
     ch->vect->dataD[i] -= ch->mean;
    }

 return;
}

/*------------------------------------------- DyChScaleOffset ---*/

void DyChScaleOffset(struct DyCh *ch, double scale, double offset)
{
 int i;

 if (ch == NULL)       return;
 if (ch->vect == NULL) return;

 if (scale != 0)
   {for (i=0; i<ch->vect->nData; i++)
       {if (ch->vect->dataD[i] > DY_NOCHANNEL || ch->vect->dataD[i] > DY_BADDATA) continue;
        ch->vect->dataD[i] *= scale;
       }
   }

 if (offset != 0)
   {for (i=0; i<ch->vect->nData; i++)
       {if (ch->vect->dataD[i] > DY_NOCHANNEL || ch->vect->dataD[i] > DY_BADDATA) continue;
        ch->vect->dataD[i] += offset;
       }
   }

 return;
}


/*------------------------------------------- DyProcessVect ---*/

void DyProcessVect(struct DyCh *ch, int plottype)
{
 double vecttime;
 struct FrVect *oldvect, *vect1;
 int i,j;

 if (ch == NULL)       return;
 if (ch->vect == NULL) return;

 /*--- Apply an antialiasing filter and a resampling ---*/
 DyResample(ch, plottype);

 /*--- Apply a pass-band filter if required ---*/
 DyPassbandfilter(ch);

 return;
}


/*------------------------------------------- DyResample ---*/

void DyResample(struct DyCh *ch, int plottype)
{
 struct FrVect *aafilter_out;
 int resampFactor;
 double vecttime;
 struct FrVect *oldvect, *vect1;
 int i,j;

 if (ch == NULL)       return;
 if (ch->vect == NULL) return;
 if (ch->resampFreq >= ch->sampFreq) return;

 /*--- Take into account missing or corrupted samples ---*/

 if (ch->status == DY_NOCHANNEL)
   {vect1 = FrVectCopy(ch->vect);
    for (i=0; i<vect1->nData; i++) {if (vect1->dataD[i] > DY_NOCHANNEL) vect1->dataD[i] = 0.;}
   }
 else if (ch->status == DY_BADDATA)
   {vect1 = FrVectCopy(ch->vect);
    for (i=0; i<vect1->nData; i++) {if (vect1->dataD[i] > DY_BADDATA) vect1->dataD[i] = 0.;}
   }
 else
   {vect1 = ch->vect;}

 /*--- Set the resampFactor value correctly (nData/ndataout must be an integer ---*/

 resampFactor = (int)(ch->sampFreq/ch->resampFreq + 0.5);

 /*--- Apply an antialiasing filter and a resampling ---*/

 if (plottype != PLOTTIME && plottype != PLOT1D && plottype != PLOT2D)
   {
    aafilter_out = FrvFilterButProc(ch->aafilter,vect1);
/*
    aafilter_out = DyFiltfilt(ch->aafilter,vect1);
*/
   }
 else
   {
    aafilter_out = FrVectCopy(vect1);
   }

 vecttime = ch->vect->GTime;
 oldvect = ch->vect;
 ch->vect = FrVectNew1D(aafilter_out->name, FR_VECT_8R, aafilter_out->nData/resampFactor, aafilter_out->dx[0]*resampFactor,
                        aafilter_out->unitX[0], aafilter_out->unitY);
 ch->vect = FrVectDecimate(aafilter_out, resampFactor, ch->vect);
/*
 ch->vect = FrvDecimateD(aafilter_out, resampFactor, NULL);
*/
 if (ch->vect == NULL)
    {ch->vect = FrVectCopy(aafilter_out);
     printf("DyResample> Could not resample %s from %g Hz to %g Hz (resampFactor=%d) \n",
             ch->name, ch->sampFreq, ch->resampFreq, resampFactor);
    }
 ch->vect->GTime = vecttime;

 if (plottype != PLOTTIME && plottype != PLOT1D && plottype != PLOT2D)
   { /* FrVectFree(aafilter_out); */}
 else
   {FrVectFree(aafilter_out);}

 /*--- Take into account missing or corrupted samples ---*/

 if (ch->status == DY_NOCHANNEL || ch->status == DY_BADDATA)
   {for (i=0; i<ch->vect->nData; i++)
       {for (j=0; j<resampFactor; j++)
           {if (oldvect->dataD[i+j] > DY_NOCHANNEL) {ch->vect->dataD[i] = DY_NOCHANNEL*1.01; break;}
            if (oldvect->dataD[i+j] > DY_BADDATA)   {ch->vect->dataD[i] = DY_BADDATA*1.01; break;}
           }
       }
    FrVectFree(vect1);
   }
 FrVectFree(oldvect);

 return;
}


/*------------------------------------------- DyPassbandfilter ---*/

void DyPassbandfilter(struct DyCh *ch)
{
 struct FrVect *bpfilter_out;
 double vecttime;
 struct FrVect *oldvect, *vect1;
 int i;

 if (ch == NULL)       return;
 if (ch->vect == NULL) return;
 if (ch->bpfilter_fmin <= 0 && ch->bpfilter_fmax <= 0) return;

 /*--- Take into account missing or corrupted samples ---*/

 if (ch->status == DY_NOCHANNEL)
   {vect1 = FrVectCopy(ch->vect);
    for (i=0; i<vect1->nData; i++) {if (vect1->dataD[i] > DY_NOCHANNEL) vect1->dataD[i] = 0.;}
   }
 else if (ch->status == DY_BADDATA)
   {vect1 = FrVectCopy(ch->vect);
    for (i=0; i<vect1->nData; i++) {if (vect1->dataD[i] > DY_BADDATA) vect1->dataD[i] = 0.;}
   }
 else
   {vect1 = ch->vect;}

 /*--- Apply a pass-band filter if required ---*/

 bpfilter_out = FrvFilterButProc(ch->bpfilter,ch->vect);
 vecttime = ch->vect->GTime;
 oldvect = ch->vect;
 ch->vect = FrVectCopy(bpfilter_out);
/*
 bpfilter_out = DyFiltfilt(ch->bpfilter,ch->vect);
 vecttime = ch->vect->GTime;
 oldvect = ch->vect;
 ch->vect = bpfilter_out;
*/
 ch->vect->GTime = vecttime;

 /*--- Take into account missing or corrupted samples ---*/

 if (ch->status == DY_NOCHANNEL || ch->status == DY_BADDATA)
   {for (i=0; i<ch->vect->nData; i++)
       {if (oldvect->dataD[i] > DY_NOCHANNEL) ch->vect->dataD[i] = DY_NOCHANNEL*1.01; 
        if (oldvect->dataD[i] > DY_BADDATA)   ch->vect->dataD[i] = DY_BADDATA*1.01; 
       }
    FrVectFree(vect1);
   }
 FrVectFree(oldvect);

 return;
}


/*------------------------------------------- DyProcessVectIni ---*/

void DyProcessVectIni(struct DyCh *ch)
{
 int j, j1, j2, resampFactor;
 double resampfreq1;

 if (ch == NULL) return;

 /*--- Set the resampFactor value correctly (nData/ndataout must be an integer) ---*/
/**
 if (ch->resampFreq < ch->sampFreq)
   {
    resampFactor = (int)(ch->sampFreq/ch->resampFreq + 0.5);
    resampfreq1 = ch->resampFreq;
    for (j=resampFactor; j>1; j--) {if (ch->vect->nData % j == 0) {j1 = j; break;}}
    for (j=resampFactor; j<ch->vect->nData/2; j++) {if (ch->vect->nData % j == 0) {j2 = j; break;}}
    if ((j1-j)*(j1-j) < (j2-j)*(j2-j)) resampFactor = j1;
    else                               resampFactor = j2;
    ch->resampFreq = ch->sampFreq / resampFactor;
    if (ch->resampFreq != resampfreq1)
      {printf("DyProcessVectIni> channel %s could not be resampled. Forced resampling frequency to %g Hz (resampFactor=%d)\n",
               ch->name, ch->resampFreq, resampFactor);
      }
   }
**/

 /*--- If needed, set up the antialias filter ---*/

 if (ch->resampFreq < ch->sampFreq)
   {ch->aafilter = FrvFilterButNew(16,0,1.1*ch->resampFreq/2,"antialiasing");}

 /*--- If requested, set up a pass band filter ---*/

 if (ch->bpfilter_fmin > 0 || ch->bpfilter_fmax > 0)
   {ch->bpfilter = FrvFilterButNew(16,ch->bpfilter_fmin,ch->bpfilter_fmax,"passband");}

 return;
}


/*------------------------------------- DyFiltfilt ---*/
/*--- Apply the zero-phase band-pass filter on the data vector ---*/

struct FrVect *DyFiltfilt(struct FrvFilterB *filter, struct FrVect *vect)
{
 struct FrVect *vectbuff;
 struct FrVect *vectbuff2;
 struct FrVect *vectout;
 int i;

 vectout = FrVectNewTS(vect->name, 1./vect->dx[0], vect->nData, -64);
 vectout->GTime = vect->GTime;
 FrStrCpy(&vectout->unitY, vect->unitY);

 /*--- Apply band-pass filter on the data vector ---*/
 vectbuff = FrvFilterButProc(filter, vect);

 /*--- Reverse the vector data ---*/
 for (i=0; i<vectbuff->nData; i++) {vectout->dataD[i] = vectbuff->dataD[vectbuff->nData-1-i];}

 /*--- Apply band-pass filter on the data vector again ---*/
 vectbuff2 = FrvFilterButProc(filter, vectout);

 /*--- Reverse the vector data again ---*/
 for (i=0; i<vectbuff2->nData; i++) {vectout->dataD[i] = vectbuff2->dataD[vectbuff2->nData-1-i];}

/**
 FrVectFree(vect);
 vect = vectout;
**/

 return(vectout);
}


/*------------------------------------------- DyChFree ---*/

void DyChFree(struct DyCh *ch)
{
 if (ch == NULL) return;

 if (ch->aafilter) FrvFilterButFree(ch->aafilter);
 if (ch->bpfilter) FrvFilterButFree(ch->bpfilter);
/*
 if (ch->buffer) FrvBufFree(ch->buffer);
 if (ch->vect) FrVectFree(ch->vect);
*/

 free(ch);
 ch = NULL;

 return;
}


/**************************************************/
/*                                                */
/* File: DyVisu.c                                 */
/* Copyright (C) 2003, D. Verkindt, D. Buskulic   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/


/*------------------------------------------- DyWindowsIni ---*/

void DyWindowsIni (long nwindows)
{
/*
 * Opens a Root canvas and divides it so that there is a suitable number of pads
 */
 int nCol, nRow;
 int padwidth, padheight;
 int winwidth, winheight;
 int i;
 static int nColOld=0, nRowOld=0;
 char canvasTitle[LSTRING_MAX], dummy[LSTRING_MAX];

 sprintf(dummy, "DyWindowsIni> nwindows=%ld\n", nwindows);
 DyDebugPrint(dummy,0);

 if (nwindows <=0) return;
 
 /*--- Calculation of the window size: organization of pad and pad size ---*/

 DySetRowCol(nwindows, &nCol, &nRow);

 if (Dy->nCol > 0 && Dy->nRow > 0 && Dy->nCol*Dy->nRow >= nwindows)
    {
     printf("DyWindowsIni> Canvas format forced to %d columns x %d raws\n", Dy->nCol, Dy->nRow);
     nCol = Dy->nCol; nRow = Dy->nRow;
    }

 if (Dy->squarePlot == ON)
    {
     if (nCol == 1)  padwidth = 700 *0.907;
     if (nCol == 2)  padwidth = 380 *0.907;
     if (nCol == 3)  padwidth = 260 *0.907;
     if (nCol == 4)  padwidth = 200 *0.907;
     if (nCol >= 5)  padwidth = 800/nCol *0.907;

     if (nRow == 1)  padheight = 700;
     if (nRow == 2)  padheight = 380;
     if (nRow == 3)  padheight = 260;
     if (nRow == 4)  padheight = 200;
     if (nRow >= 5)  padheight = 800/nRow;

     if (nRow < nCol) padheight = padwidth;

     /* Set the window size */

     winwidth  = nCol*(padwidth+5)+5;
     winheight = nRow*(padheight+5)+32;
     if (winwidth > 800 * 0.907)  winwidth  = 800 *0.907;
     if (winheight > 800) winheight = 800;
    }

 else
    {
     if (nCol == 1)  padwidth = 900;
     if (nCol == 2)  padwidth = 450;
     if (nCol == 3)  padwidth = 330;
     if (nCol == 4)  padwidth = 250;
     if (nCol >= 5)  padwidth = 1000/nCol;

     if (nRow == 1)  padheight = 700;
     if (nRow == 2)  padheight = 380;
     if (nRow == 3)  padheight = 260;
     if (nRow == 4)  padheight = 200;
     if (nRow >= 5)  padheight = 800/nRow;

     if (nRow < nCol) padheight = padwidth;

     /* Set the window size */

     winwidth  = nCol*(padwidth+5)+5;
     winheight = nRow*(padheight+5)+32;
     if (winwidth > 1000) winwidth  = 1000;
     if (winheight > 800) winheight = 800;
    }

 /*--- Set title of the canvas window ---*/

 sprintf(canvasTitle,"%s_plots", Dy->DyName);
 if (Dy->titleCmd != NULL) sprintf(canvasTitle,"%s_plots", Dy->titleCmd);

 /*--- Initialize the canvas window ---*/
 /* If canvas not divided as before, delete it and reconstruct */

 RA_InitDDWindow(canvasTitle,10,10,winwidth,winheight);

if (Dy->AutoConfFlag == OFF || (Dy->AutoConfFlag == ON && !RA_PadIsDivided(nCol*nRow)))
  {     
   if ((!RA_PadIsDivided(nCol*nRow) || nColOld != nCol || nRowOld != nRow) && RA_PadIsDivided(-1))
      {
       RA_MainCanvasDelete();
       RA_InitDDWindow(canvasTitle,10,10,winwidth,winheight);
       RA_PadDivideWithTitle(nCol,nRow);
      }
   else if (!RA_PadIsDivided(-1))
      {RA_PadDivideWithTitle(nCol,nRow);}
  }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT))) printf("  DyWindowsIni> nwindows=%ld  nCol=%d  nRow=%d \n", nwindows, nCol, nRow);

 nColOld = nCol;
 nRowOld = nRow;

 /* Clear the supplementary pads */

 if (nwindows < nCol*nRow)
    {
     for (i=nwindows+1;i<=nCol*nRow;i++) RA_PadCompleteClear(i);
    }
 RA_SetTestTrigPad(-1,0,0);

 /** ICI */
 /*--- Define the label size and title size and offset on x and y axis ---*/
/**
 DyPlotSetAllSize();
 RA_HSetAllSize();
**/
 
 return;
}


/*-------------------------------- DySetRowCol ---*/
/* author: E. Tournefier */

void DySetRowCol (int ntot, int *nCol, int *nRow)
{
 int n1,n2,n3,n4,a,b;
 double x,y;

 DyDebugPrint(__FUNCTION__,0);

 if (ntot <= 0) {*nRow = 1; *nCol = 1; return;}
 if (ntot == 1) {*nRow = 1; *nCol = 1; return;}
 if (ntot == 2) {*nRow = 1; *nCol = 2; return;}
 if (ntot == 5) {*nRow = 2; *nCol = 3; return;}

 y  = ntot;
 x  = sqrt(y);
 n1 = x;
 y  = x-n1;
 if (y > 0.5) n1++;

 n2 = ntot/n1;
 if (n1*n2 < ntot) n2++;

 n3 = n1-1;
 n4 = ntot/n3;
 if (n4*n3 < ntot) n4++;

 a = n1*n2 - ntot;
 b = n3*n4 - ntot;

 if (a <= b)
    {*nRow = n1;
     *nCol = n2;
    }
 else
    {*nRow = n3;
     *nCol = n4;
    }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
    {printf("      DySetRowCol> n1=%d n2=%d n3=%d n4=%d a=%d b=%d\n", n1, n2, n3, n4, a, b);
     printf("      DySetRowCol> nRow=%d  nCol=%d \n", *nRow, *nCol);
    }

/***
 if (ntot == 1)  {*nCol=1; *nRaw=1;}
 if (ntot == 2)  {*nCol=2; *nRaw=1;}
 if (ntot == 3)  {*nCol=3; *nRaw=1;}
 if (ntot == 4)  {*nCol=2; *nRaw=2;}
 if (ntot == 5)  {*nCol=3; *nRaw=2;}
 if (ntot == 6)  {*nCol=3; *nRaw=2;}
 if (ntot == 7)  {*nCol=4; *nRaw=2;}
 if (ntot == 8)  {*nCol=4; *nRaw=2;}
 if (ntot == 9)  {*nCol=3; *nRaw=3;}
 if (ntot > 9)   {*nCol=4; *nRaw=(ntot-1)/4+1;}
 if (ntot > 12)  {*nCol=5; *nRaw=(ntot-1)/5+1;}
***/

 return;
}


/*-------------------------------------------- DyVisuEnd ---*/

void DyVisuEnd ()
{
 int result;
 char ddrootFile[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 RA_RefPlotDraw();
 RA_MainCanvasUpdate();

 /*--- open file for plots and ntuples and save them ---*/

 result = access(".", W_OK);
 if (result != 0) sprintf(ddrootFile,"%s/%s%s", (char *)getenv("HOME"), DEFAULT_DYNAME, ROOT_SUFFIX);
 else             sprintf(ddrootFile,"%s%s", DEFAULT_DYNAME, ROOT_SUFFIX);

if (Dy->plotsNotDisplayed == ON)
   {if (result != 0) DyInfoLine ("Could not write in local directory. Files will be written in your home directory.");
    RA_FileOpenCreate(ddrootFile);
    RA_FileWrite(ddrootFile);
    RA_FileClose(ddrootFile);
   }

 return;
}


/**************************************************/
/*                                                */
/* File: DyFrOutput.c                             */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*------------------------------------------- DyOutputDefault ---*/

void DyOutputDefault()
{
 strcpy(Dy->output_tag,"Qc_Moni* Alp_Main* Alp_Cali* Pr_B1_ACp h*");
 strcpy(Dy->output_dir, DYOUTPUTDIR);
 Dy->output_nframe_perfile = 240;
 Dy->output_flag = OFF;

 return;
}


/*------------------------------------------- DyOutputWrite ---*/

void DyOutputWrite (struct FrameH *frame)
{
 char filename[LSTRING_MAX], dummy[LSTRING_MAX], date_heure[18];
 time_t utc;
 int error, nBytes;
 DIR  *dirp;

 if (Dy->output_flag == OFF) return;

 if (Dy->output_nframe_written == 0)
    {dirp = opendir(Dy->output_dir);
     if (dirp == NULL)
        {printf("Creating directory %s \n", Dy->output_dir);
         sprintf(dummy, "mkdir %s", Dy->output_dir);
         system(dummy);
        }
     Dy->output_nbytes_tot = 0;
    }

 if (Dy->output_nframe_written%2 == 1) DyXformShowWriteAlarm(1);
 else                                  DyXformShowWriteAlarm(0);

 if (Dy->output_nframe_written == Dy->output_nframe_perfile)
    {FrFileOEnd(Dy->output_file);
     Dy->output_file = NULL;
     Dy->output_nframe_written = 0;
    }

 if (Dy->output_file == NULL)
    {utc = frame->GTimeS + FRGPSTAI - frame->ULeapS + FRGPSOFF;
     strftime(date_heure,18,"%d-%b-%Y-%Hh%M",gmtime(&utc));
     sprintf(filename, "%s/Dy-%d-%s-%dF.gwf", Dy->output_dir, frame->GTimeS, date_heure, Dy->output_nframe_perfile);
     printf("DyOutputWrite> Writing output file %s \n", filename);
     Dy->output_file = FrFileONew(filename,-1);
    }

 nBytes = Dy->output_file->nBytes;

 error = FrameWrite (frame, Dy->output_file);

 if (error == 0)
    {Dy->output_nframe_written++;
     Dy->output_nbytes_tot += Dy->output_file->nBytes - nBytes;
     printf("DyOutputWrite> frame=%p  Input_framesize=%d  Output_framesize=%ld  Written on disk=%.3f MB\n",
             frame, inputFrameSize, Dy->output_file->nBytes - nBytes, Dy->output_nbytes_tot/1024./1024.);
     if (Dy->output_nbytes_tot > 10.*1024.*1024.*1024.)
        {Dy->output_flag = OFF;
         printf("DyOutputWrite> Written %.1f kB in %s\n", Dy->output_nbytes_tot/1024., Dy->output_dir);
         DyInfoLine("Written already 10 GB of data! You cannot continue to output data");
         Dy->output_nbytes_tot = 0;
        }
    }

 return;
}


/*------------------------------------------- DyOutputClose ---*/

void DyOutputClose()
{
/**
 FILE *fp;
 struct FrFile *iFile;
 char *output_files;
 char fflname[LSTRING_MAX], filename[LSTRING_MAX];
 DIR  *dirp;
 struct dirent *entry;
 int length, nfile;
**/

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->output_file)
    {FrFileOEnd(Dy->output_file);
     Dy->output_file = NULL;
    }

/***
 length = 0;
 dirp   = opendir(Dy->output_dir);
 while ((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;
    if (strstr(entry->d_name,".gwf") == 0) continue;
    length += 1 + strlen(Dy->output_dir) + strlen(entry->d_name);
   }
 closedir(dirp);
 output_files = (char *)malloc(length+1);

 nfile = 0;
 dirp  = opendir(Dy->output_dir);
 while ((entry = readdir(dirp)) != NULL)
   {
    if (strncmp(entry->d_name,".",1) == 0) continue;
    if (strstr(entry->d_name,".gwf") == 0) continue;
    sprintf(filename,"%s/%s",Dy->output_dir,entry->d_name);
    if (nfile == 0) strcpy(output_files,filename);
    else            {strcat(output_files," "); strcat(output_files,filename);}
    nfile++;
   }
 closedir(dirp);

 if (nfile > 0)
    {sprintf(fflname,"%s/%s", Dy->output_dir, DYOUTPUTFFL);
     printf("DyOutpuClose> Building %s\n", fflname);
     printf("DyOutputClose> Output files: %s\n", output_files);
     fp = fopen(fflname,"w");
     iFile = FrFileINew(output_files);
     printf("DyOutputClose> Output files: iFile=%x  fp=%x\n", iFile, fp);
     FrFileIDumpT (iFile, fp, 1, NULL, 0., 1.e+12);
     FrFileIEnd(iFile);
     fclose(fp);
    }

 free(output_files);
***/

 DyXformShowWriteAlarm(0);
 Dy->output_nframe_written = 0;

 return;
}


/*-------------------------------------------------- DyCheckData ----*/
int DyCheckData (struct FrVect *data)
{
 int i, flag, notValid;
 double value;

 notValid = 0;
 flag     = 0;

 if (data == NULL) return(0);

 for (i=0; i<data->nData; i++)
    {
     if (data->type == FR_VECT_4R)  value = data->dataF[i];
     if (data->type == FR_VECT_8R)  value = data->dataD[i];
     if (data->type == FR_VECT_C)   value = data->data[i];
     if (data->type == FR_VECT_2S)  value = data->dataS[i];
     if (data->type == FR_VECT_8S)  value = data->dataL[i];
     if (data->type == FR_VECT_4S)  value = data->dataI[i];

     if (isnan(value))
        {if (flag == 0) printf("DyCheckData: Got NaN at index %d of vector %s (type %d)\n", i,data->name,data->type);
         flag     = 1;
         notValid = 1;
        }
     else
        {flag = 0;}
    }
			 
 return(notValid);
}



/*-------------------------------------------- DyFrVectDumpToBuf ----*/
/* ICI obsolete ? */

char *DyFrVectDumpToBuf (struct FrVect *vect, char *buf, long dumplevel)
{
 int i, imax;
 char *dC;
 short  *dS;
 int    *dI;  
 long   *dL; 
 float  *dF, ratio;  
 double *dD;
 unsigned char *dU;
 unsigned short *dUS;
 unsigned int *dUI;
 unsigned long *dUL; 

 if (buf  == NULL) return(NULL);
 if (vect == NULL) return(NULL);
 if (dumplevel < 1) return(NULL);

 strcpy(buf," ");

 sprintf(buf,"%s  Vector:%s ndata=%ld nDim=%d", buf, vect->name, vect->nData, vect->nDim);

 if (vect->unitY != NULL) 
    {sprintf(buf,"%s dataUnits=%s", buf, vect->unitY);}
 
 for (i=0; i<vect->nDim; i++)
    {sprintf(buf,"%s      dimension=%d nx=%10ld dx=%g", buf,i,vect->nx[i], vect->dx[i]);
     if (vect->unitX[i] != NULL)
        {sprintf(buf,"%s unit%d=%s\n",buf, i, vect->unitX[i]);}
     else
        {sprintf(buf,"%s unit%d=a.u.\n",buf,i);}
    }

 if (vect->compress == 0)
    {
     if (vect->type == FR_VECT_4R) 
        {dF = (float *) vect->data;
         if (dumplevel == 2)
            {sprintf(buf,"%s(F) %e %e %e ...\n",buf, dF[0],dF[1],dF[2]);}
         if (dumplevel == 3)
            {sprintf(buf,"%s(F) ",buf);
             if (vect->nData/10 < 3) imax=vect->nData;
             else                    imax = vect->nData/10;
             for (i=0; i<imax; i++)
                 {sprintf(buf,"%s%e ", buf, dF[i]);
                  if (i>0 && i%10==0) sprintf(buf,"%s\n",buf);
                 }
             sprintf(buf,"%s...\n",buf);
            }
         if (dumplevel >3)
            {sprintf(buf,"%s(F) ",buf);
             for (i=0; i<vect->nData; i++)
                 {sprintf(buf,"%s%e ", buf, dF[i]);
                  if (i>0 && i%10==0) sprintf(buf,"%s\n",buf);
                 }
             sprintf(buf,"%s...\n",buf);
            }
        }
     else if(vect->type == FR_VECT_8R)  
        {dD = (double *) vect->data;
         sprintf(buf,"%s(D) %e %e %e ...\n",buf, dD[0],dD[1],dD[2]);
        }
     else if(vect->type == FR_VECT_C)  
        {dC = vect->data;
         sprintf(buf,"%s(C) %d %d %d %d %d ...\n", 
                             buf, dC[0],dC[1],dC[2],dC[3],dC[4]);}
     else if(vect->type == FR_VECT_2S)  
        {dS = (short *) vect->data;
         sprintf(buf,"%s(S) %d %d %d %d %d ...\n", 
                            buf, dS[0],dS[1],dS[2],dS[3],dS[4]);}
     else if(vect->type == FR_VECT_8S)  
        {dL = (long *) vect->data;
         sprintf(buf,"%s(L) %ld %ld %ld ...\n",buf, dL[0],dL[1],dL[2]);
        }
     else if(vect->type == FR_VECT_4S)  
        {dI = (int *) vect->data;
         sprintf(buf,"%s(I) %d %d %d ...\n",buf, dI[0],dI[1],dI[2]);
        }
     else if(vect->type == FR_VECT_1U)  
        {dU = (unsigned char *)vect->dataU;
         sprintf(buf,"%s(C) %d %d %d ...\n",buf, dU[0],dU[1],dU[2]);
        }
     else if(vect->type == FR_VECT_2U)  
        {dUS = (unsigned short *) vect->data;
         sprintf(buf,"%s(S) %d %d %d ...\n",buf, dUS[0],dUS[1],dUS[2]);
        }
     else if(vect->type == FR_VECT_8U)  
        {dUL = (unsigned long *) vect->data;
         sprintf(buf,"%s(L) %ld %ld %ld ...\n",buf, dUL[0],dUL[1],dUL[2]);
        }
     else if(vect->type == FR_VECT_4U)  
        {dUI = (unsigned int *) vect->data;
         sprintf(buf,"%s(I) %d %d %d ...\n",buf, dUI[0],dUI[1],dUI[2]);
        }
     else if(vect->type == FR_VECT_C8) 
        {dF = (float *) vect->data;
         sprintf(buf,"%s(CF) (%e,%e) (%e,%e) ...\n",buf, dF[0],dF[1],dF[2],dF[3]);
        }
     else if(vect->type == FR_VECT_C16) 
        {dD = (double *) vect->data;
         sprintf(buf,"%s(CD) (%e,%e) (%e,%e) ...\n",buf, dD[0],dD[1],dD[2],dD[3]);
        }
     else 
        {sprintf(buf,"%sunknown type: %d \n",buf,vect->type );}
    }

 else 
    {ratio = ((float) vect->nBytes) /(vect->nData*vect->wSize);
     sprintf(buf,"%s\n     the vector is %4.2f compressed (%d) nBytes=%ld wSize=%d\n",
             buf,ratio,vect->compress,vect->nBytes, vect->wSize);
    }

 if (vect->next != NULL)
    {sprintf(buf,"%s  Attached information:\n",buf);
     DyFrVectDumpToBuf(vect->next, buf, dumplevel) ;
    }

 return(buf);
}


