/**************************************************/
/*                                                */
/* File: DyXformUser.c                            */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/**
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
**/

#include "dataDisplay.h"
#include "URootProcess.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif


/*                 ******************************                 */
/*                 * User processing and macros *                 */
/*                 ******************************                 */


/*------------------------------------------- DyLoadScript ---*/

void DyLoadScript (char *className)
{
 char dummy[LSTRING_MAX], fileName[LSTRING_MAX];

 sprintf(dummy,"DyLoadScript> className=%s", className);
 DyDebugPrint(dummy,0);

 sprintf(fileName,"%s.C", className);

 if (is_a_file(fileName) == 0)
    {
     if (is_a_dir(DYALGO_DIR))
        {
         sprintf(dummy,"%s/%s", DYALGO_DIR, getenv("UNAME"));
         if (is_a_dir(dummy)) sprintf(fileName, "%s/%s.C", dummy, className);
         else                 sprintf(fileName,"%s/%s.C", DYALGO_DIR, className);
        }
     else
        {printf("DyLoadScript> Algorithm's file not found\n");
         return;
        }
    }

 RA_LoadScript(fileName,1);
 DyXformUpdateUserProcBrowser();

 return;
}


/*------------------------------------------ DyXformMakeNewMacro ---*/

void DyXformMakeNewMacro (FL_OBJECT *obj, long arg)
{
/* Writes a sample macro in a new file. This is just an example macro */
/* that needs to be adapted to a users particular needs               */
/* so it should be edited later, before loading                       */
   
 const char *str;

 DyDebugPrint(__FUNCTION__,0);

 str = fl_show_input("Name of the new processing object",USEROP);
 if (!str) return;
 if (strlen((char *)str) ==0) {
    printf("DyXformMakeNewMacro> need a name \n");
 }

 RA_MakeClass((char*)str, NULL, NULL);

 return;
}

/*------------------------------------------ DyXformLoadInterpretMacro ---*/

void DyXformLoadInterpretMacro (FL_OBJECT *obj, long arg)
{
/* Load a macro without compiling it, i.e. in interpreted mode */

 const char *str;
 long test=1;

 while(test)
    {str = fl_show_fselector("chose a macro file to load",".","*.[c|C|cc]","");
     if (!str) return;
     if (is_a_file((char *)str)) test=0;
     else printf("DyXformLoadInterpretMacro> I need a file \n");
    }

 RA_LoadScript((char*)str, 0);
 DyXformUpdateUserProcBrowser();

 return;
}

/*------------------------------------------ DyXformLoadCompileMacro ---*/

void DyXformLoadCompileMacro (FL_OBJECT *obj, long arg)
{
/* Load a macro and, if it has changed since the last loading, compiles it */
/* prior to loading. That means, creates a new shared library out of       */
/* code contained inside the macro, loads this shared library and update   */
/* the list of already loaded user functions                               */
   
 const char *str;
 long test=1;

 while(test)
    {str = fl_show_fselector("chose a macro file to load",".","*.[c|C|cc]","");
     if (!str) return;
     if (is_a_file((char *)str)) test=0;
     else printf("DyXformLoadCompileMacro> I need a file \n");
    }

 RA_LoadScript((char*)str, 1);
 DyXformUpdateUserProcBrowser();

 return;
}

/*------------------------------------------ DyXformUpdateUserProcBrowser ---*/

void DyXformUpdateUserProcBrowser()
{
/* Updates the user function browser */

 int i, j, j0, nvar;
 char userline[LSTRING_MAX], dummy[LSTRING_MAX], *str3;
 const char *str;

 DyDebugPrint(__FUNCTION__,0);

 RA_GetListOfProc();

 fl_clear_browser(userfuncbrowser->userFuncLoaded);

 if (gProcList)
    {
     for (i=0;i<gProcList->numProcess;i++)
        {
         strcpy(dummy,gProcList->name[i]);
         str3 = dummy+strlen(dummy)-1;
         while (atoi(str3) > 0 || *str3 == '0') str3--;
         *(str3+1) = '\0';
         strcpy(userline,dummy);

         if (!gProcList->procCompiled[i]) strcat(userline," (interpreted)");
         nvar = fl_get_browser_maxline(userfuncbrowser->userFuncLoaded);
         j0 = -1;
         for (j=0; j<nvar; j++)
           {str = fl_get_browser_line(userfuncbrowser->userFuncLoaded,j+1);
            if (strcmp(dummy, str) == 0) {j0 = j; break;}
           }
         if (j0 == -1) fl_add_browser_line (userfuncbrowser->userFuncLoaded,userline);
        }
    }
}


/*------------------------------------------- DyXformSetDefaultOpString ---*/

void DyXformSetDefaultOpString (FL_OBJECT *obj, long arg)
{
 long i, k0, k, nvar, nsig, oplinefound, chfound;
 const char *str;
 char chname[LSTRING_MAX], opstring[LBIGSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_browser(mainbrowser->channels) == 0) return;

 browser = mainbrowser->channels;
 nvar    = fl_get_browser_maxline(browser);

 /*--- Determine the number of opstring channels selected and the total number of lines selected ---*/

 chfound = 0;
 oplinefound = 0;
 for (i=nvar-1; i>=0; i--)
    {if (fl_isselected_browser_line(browser,i+1))
       {
        str = fl_get_browser_line(browser,i+1);
        sscanf(str,"%s",chname);
        if (strncmp(chname,OPSIGNAL,strlen(OPSIGNAL)) == 0)
          {for (k=Dy->nch-1; k>=0; k--) {if (strcmp(Dy->ch[k]->name, chname) == 0) {k0 = k; break;}}
           oplinefound++;
          }
        chfound++;
       }
    }

 /*--- If only one opstring channel selected, use it as default input field ---*/

 if (oplinefound==1 && chfound==1)
   {sprintf(opstring, "%s", Dy->ch[k0]->op_string);
    strcat(opstring," \0");
    fl_set_input(mainbrowser->opstring, opstring);
    return;
   }


 sprintf(opstring, "%s_%s = ", getenv("USER"), USEROP);

 /*--- build the default opstring from selected channels, except channels made by user ---*/

 nsig    = 0;
 for (i=0; i<nvar; i++)
    {if (fl_isselected_browser_line(browser,i+1))
       {
        str = fl_get_browser_line(browser,i+1);
        sscanf(str,"%s",chname);
        if (strncmp(chname,OPSIGNAL,strlen(OPSIGNAL)) == 0) continue;
        if (nsig > 0) strcat(opstring," + ");
        strcat(opstring,chname);
        nsig++;
       }
    }

 /*---add the opstring of channels made by user ---*/

 for (i=nvar-1; i>=0; i--)
    {if (fl_isselected_browser_line(browser,i+1))
       {
        str = fl_get_browser_line(browser,i+1);
        sscanf(str,"%s",chname);
        if (strncmp(chname,OPSIGNAL,strlen(OPSIGNAL)) != 0) continue;
        for (k=Dy->nch-1; k>=0; k--) {if (strcmp(Dy->ch[k]->name, chname) == 0) {k0 = k; break;}}
        if (nsig > 0) strcat(opstring," + ");
        strcat(opstring,strstr(Dy->ch[k0]->op_string,"=")+1);
        nsig++;
       }
    }

 strcat(opstring," \0");

 /*--- put opstring in the input field modifiable by the user ---*/

 fl_set_input(mainbrowser->opstring, opstring);

 return;
}

 
/*------------------------------------------- DyGetOpString ---*/

struct DyCh *DyGetOpString (const char *opstring)
{
 long i, i0, number, maxnumber, found;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX], opstring1[LBIGSTRING_MAX], opstring2[LBIGSTRING_MAX];
 char *p, *p1, className[LSTRING_MAX];
 char *str3;
 struct DyCh *opch;

 DyDebugPrint(__FUNCTION__,0);

 strcpy(opstring1, opstring);

 /*--- build the user function operation string built from user function list ---*/

/**
 RA_GetListOfProc();
 if (gProcList)
   {maxnumber = -1;
    for (i=0;i<gProcList->numProcess;i++)
       {
        strcpy(dummy,gProcList->name[i]);
        str3 = dummy+strlen(dummy)-1;
        while (atoi(str3) > 0 || *str3 == '0') str3--;
        number = atoi(str3+1);
        *(str3+1) = '\0';
        if (strcmp(dummy, userFct) == 0) {i0 = i; maxnumber = number;}
       }
    if (maxnumber >= 0) strcpy(opstring1,gProcList->opstring[i0]);
   }
**/

 /*--- Build the className ---*/

 if (strstr(opstring1,"=") == NULL)
    {strcpy(className,USEROP);}
 else
    {p  = strstr(opstring1,"="); p--;
     while (*p == ' ' && p >= opstring1) p--;
     p1 = opstring1;
     while (*p1 == ' ' && p1 < opstring1+strlen(opstring1)) p1++;
     strncpy(className,p1,p-p1+1);
     className[p-p1+1] = '\0';
     if (DyCheckString(className)) strcpy(className, DyCheckString(className));
    }

 /*--- To prevent problem coming from one instance of VViewerSelector class per user function --*/
 /*--- we add a numbering to className if it already exists in user functions list ---*/
 /* ICI Not used anymore: the user will have to choose a different name for each new channel created */

/**
 RA_GetListOfProc();

 if (gProcList)
    {
     maxnumber = -1;
     found = -1;
     for (i=0;i<gProcList->numProcess;i++)
        {
         strcpy(dummy,gProcList->name[i]);
         str3 = dummy+strlen(dummy)-1;
         while (atoi(str3) > 0 || *str3 == '0') str3--;
         number = atoi(str3+1);
         if (maxnumber < number) maxnumber = number;
         *(str3+1) = '\0';
         if (strcmp(dummy, className) == 0) {found = number; break;}
        }
     if (found == -1)
        {if (maxnumber == 0) strcat(className, "2");
         else                sprintf(className, "%s%ld", className, maxnumber+1);
        }
    }
**/

 /*--- Create the Operation String (opstring2 is the C code line corresponding to opstring) ---*/
 /*--- And put in inputVect the list of input vectors used in opstring ---*/

 strcpy(inputVect,"");
 strcpy(opstring2,"");
 DyMakeOpCString(opstring1, opstring2, inputVect);

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
    {printf("    DySetOpString> className=%s \n",className);
     printf("    DySetOpString> opstring1=%s \n",opstring1);
     printf("    DySetOpString> opstring2=%s \n",opstring2);
     printf("    DySetOpString> inputVect=%s \n",inputVect);
    }

 if (inputVect == NULL || strlen(inputVect) <= 1) return(NULL);

 /*--- Create the corresponding class and load it ---*/

 RA_MakeClass(className, opstring2, opstring1);
 DyLoadScript(className);

 /*--- Add the operation string result as a new channel ---*/

 opch = DyAddUserChannel(className, &inputVect[1]);
 if (opch == NULL) return(NULL);

 strcpy(opch->op_string,opstring1);
 strcpy(opch->op_cstring,opstring2);

 /*--- Put the opch structure in the already existing channel or create a new one ---*/

 i0 = -1;
 for (i=Dy->nch-1; i>=0; i--) {if (strcmp(Dy->ch[i]->name,opch->name) == 0) {i0 = i; break;}}
 if (i0 == -1) Dy->ch[Dy->nch++] = opch;
 else         {free(Dy->ch[i0]); Dy->ch[i0] = opch;}
 printf("==== Dy->nch=%d  i0=%ld  opch->op_nch=%d \n", Dy->nch, i0, opch->op_nch);

 /*--- Put the new channel into the channels browser ---*/

 DyXformAddOneChannel(mainbrowser->channels, opch, i0);

 if (i0 == -1) sprintf(dummy, "Made new signal %s at %.2fHz", opch->name, opch->sampFreq);
 else          sprintf(dummy, "Updated the signal %s at %.2fHz", opch->name, opch->sampFreq);
 DyInfoLine (dummy);

 return (opch);
}


/*-------------------------------------- DyChFillFromOpstring ---*/

struct DyCh *DyChFillFromOpstring (struct DyCh *opch, char *opstring)
{
 char inputVect[LBIGSTRING_MAX], opstring2[LBIGSTRING_MAX];
 char className[LSTRING_MAX];
 char *str;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Create the Operation String (opstring2 is the C code line corresponding to opstring) ---*/
 /*--- And put in inputVect the list of input vectors used in opstring ---*/

 strcpy(inputVect,"");
 strcpy(opstring2,"");
 DyMakeOpCString(opstring, opstring2, inputVect);

 if (inputVect == NULL || strlen(inputVect) <= 1) return(NULL);

 /*--- Create the corresponding class and load it ---*/

 str = strstr(opstring,USEROP);
 strncpy(className,opstring,str-opstring);
/**
 RA_MakeClass(className, opstring2, opstring);
 DyLoadScript(className);
**/

 /*--- Add the operation string result as a new channel ---*/

 opch = DyAddUserChannel(className, &inputVect[1]);
 if (opch == NULL) return(NULL);

 strcpy(opch->op_string,opstring);
 strcpy(opch->op_cstring,opstring2);

 return(opch);
}


/*-------------------------------------- DyAddUserChannel ---*/

struct DyCh *DyAddUserChannel (char *className, char *inputVect)
{
 int i, op_nch;
 char *word, outname[LBIGSTRING_MAX], chaine[LBIGSTRING_MAX];
 struct DyCh *opch;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("  inputVect=%s \n", inputVect);

 if (inputVect == NULL) return(NULL);

 strcpy(outname,OPSIGNAL);
 strcat(outname,className);

 /*--- Get list of input vectors and put information in a new channel structure ---*/
 /*--- Each input vector is associated to a channel structure op_chin belonging to  the new channel structure ---*/

 op_nch = 0;
 strcpy(chaine,inputVect);
 word = strtok(chaine,OPSEP);

 while (word != NULL)
   {
    printf("  DyAddUserChannel> Searching if channel %s  already exists  ..  op_nch=%d\n", word, op_nch);
    for (i=0; i<Dy->nch; i++)
       {if (strcmp(Dy->ch[i]->name,word) == 0)
          {if (op_nch == 0) {opch = (struct DyCh *)malloc(sizeof(struct DyCh)); opch->sampFreq = 0;}
           if (opch->sampFreq < Dy->ch[i]->sampFreq) opch->sampFreq = Dy->ch[i]->sampFreq;
           opch->op_chin[op_nch] = DyChClone(Dy->ch[i]);
           opch->op_ich[op_nch] = i;
           op_nch++;
           break;
          }
       }
/**
    strcat(outname,OPSEP);
    strcat(outname,word);
**/
    word = strtok(NULL,OPSEP);
   }


 /*--- Put outname and op_nch in the opch structure ---*/

 strcpy(opch->inputname, "dataDisplay");
 strcpy(opch->name,outname);
 strcpy(opch->op_classname,className);
 opch->resampFreq = opch->sampFreq;
 opch->vect2d_duration = 0;
 opch->op_nch = op_nch;
 opch->inputtype = INDY;

 return(opch);
}


/*------------------------------------------- DyMakeOpCString ---*/
/* Take opstring as input and build opstring2 (the running operation string) and inputVect (list of input channels) */
void DyMakeOpCString (char *opstring, char *opstring2, char *inputVect)
{
 int nvect;
 unsigned char c, cnext;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyMakeOpCString>  opstring=%s", opstring);

 p = opstring;
 if (strstr(opstring,"=") != NULL) {p = strstr(opstring,"="); p++;}
 p2     = opstring2; *p2='('; p2++; *p2='\0';
 pstart = NULL;
 nvect  = 0;

 while (*p != 0)
   {
    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';

            /*--- if this is really a signal name, not a constant ---*/
            if (atof(signalName) == 0.  &&  c != '(') 
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2 += strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
/**
        else if (c != ' ')
           {*p2 = c; p2++;}
**/
        else 
           {*p2 = c; p2++;}

        p++; continue;
       }
 
    cnext = (unsigned char) *(p+1);
    if (strchr ("+*/(),", c) != NULL || (c == '-' && cnext == ' '))
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';

            /*--- if this is really a signal name, not a constant ---*/
            if (atof(signalName) == 0.  &&  c != '(')
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2+=strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
/**
        else if (c != ' ')
           {*p2 = c; p2++;}
**/
        else 
           {*p2 = c; p2++;}
        p++;
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0.)  /* if this is really a signal name, not a constant or a parameter */
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,") & ("); p2 += 5;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,") | ("); p2 += 5;}
         else
            {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
             strcpy(p2, dummy);
             strcat(inputVect, OPSEP);
             strcat(inputVect, signalName);
             nvect++;
             p2+=strlen(dummy);
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
    }

 *p2=')'; p2++;
 *p2 = '\0';

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
    {printf("    last> opstring=%s  opstring2=%s  inputVect=%s\n",opstring ,opstring2, inputVect);}

 return;
}



/*------------------------------------------- DyOpStringReplaceChannels ---*/
/* ICI obsolete ? */

char *DyOpStringReplaceChannels (char *opstring, char *vectlist)
{
 int i, nvect, nch;
 unsigned char c;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char *chaine, *name, *vectlist2;
 char *channel[256];
 char opstring2[LBIGSTRING_MAX];

 if (vectlist == NULL) return(NULL);

 /*--- List the channels and put them into an array ---*/

 if (strstr(vectlist,"=") != NULL) vectlist2 = strstr(vectlist,"=")+1;
 else                              vectlist2 = vectlist;

 chaine = (char *)malloc(strlen(vectlist2)+1);
 strcpy(chaine,vectlist2);
 name = strtok(chaine," ");
 nch = 0;

 while (name != NULL)
    {channel[nch] = (char *)malloc(strlen(name)+1);
     strcpy(channel[nch],name);
     nch++;
     name = strtok(NULL,OPSEP);
    }
 free(chaine);
 
 /*--- Scan the opstring and replaced each channel found by channel of the array ---*/

 p = opstring;
 if (strstr(opstring,"=") != NULL) {p = strstr(opstring,"="); p++;}

 strncpy(opstring2,vectlist,p-opstring+1);
 p2 = opstring2; p2 += p-opstring+1;
 pstart = NULL;
 nvect  = 0;

 while (*p != 0)
   {
    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.)  /* if this is really a signal name, not a constant or a parameter */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,"AND"); p2 += 3;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,"OR"); p2 += 2;}
                else
                   {strcpy(p2, channel[nvect]);
                    if (nvect < nch) p2+=strlen(channel[nvect]);
                    else             p2+=strlen(channel[nch-1]);
                    nvect++;
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
        else if (c != ' ')
           {*p2 = c; p2++;}

        p++; continue;
       }
 
    if (strchr ("+-*/(),", c) != NULL)
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.)  /* if this is really a signal name, not a constant or a parameter */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,"AND"); p2 += 3;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,"OR"); p2 += 2;}
                else
                   {strcpy(p2, channel[nvect]);
                    if (nvect < nch) p2+=strlen(channel[nvect]);
                    else             p2+=strlen(channel[nch-1]);
                    nvect++;
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++;
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0.)
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,"AND"); p2 += 3;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,"OR"); p2 += 2;}
         else
            {strcpy(p2, channel[nvect]);
             if (nvect < nch) p2+=strlen(channel[nvect]);
             else             p2+=strlen(channel[nch-1]);
             nvect++;
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
    }

 *p2 = '\0';

 strcpy(opstring, opstring2);

 for (i=0; i<nch; i++) free(channel[i]);

 return(opstring);
}


/*------------------------------------------ DyXformRemoveUserProcFiles ---*/

void DyXformRemoveUserProcFiles ()
{
/* Remove *.C , *.h and *.so files produced by RA_MakeClass (UserFct) */

 int i;
 char name[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 RA_GetListOfProc();

 if (gProcList)
    {
     for (i=0;i<gProcList->numProcess;i++)
        {
         sprintf(name,"%s.C",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("    file %s removed \n", name);

         sprintf(name,"%s.h",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("    file %s removed \n", name);

         sprintf(name,"%s_C.so",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("    file %s removed \n", name);

         sprintf(name,"%s_C.d",gProcList->name[i]);
         remove(name);
         if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("    file %s removed \n", name);
        }
    }
 return;
}


/*------------------------------------------- DyXformSetTrigger ---*/
                                            /*  called from Main Browser  */
void DyXformSetTrigger (FL_OBJECT *obj, long arg)
{
 const char *menuItem, *opstring;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_menu(obj) > 0)
    {menuItem = fl_get_menu_text(obj);
     if (menuItem) Dy->trigMenuItem = my_strcpy(Dy->trigMenuItem,(char *)menuItem);
    }

 if (Dy->trigMenuItem == NULL)
    {DyInfoLine("Choose first a trigger source");
     return;
    }

 if (strcmp(Dy->trigMenuItem,TRIGCH) == 0)
    {
     DySetTriggerString();
     Dy->triggerType = TRIG_CH;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
    }

 if (strcmp(Dy->trigMenuItem,TRIGASCII) == 0)
    {
     opstring = fl_show_input("Write here the name of the Ascii file that contains triggers GPS times", Dy->trigAsciiFileName);
     if (opstring == NULL) return;
     else                  strcpy(Dy->trigAsciiFileName, opstring);
     DySetTriggerList();
     Dy->triggerType = TRIG_ASCII;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
     Dy->triggerPause = ON;
     fl_set_button(dyoptions->trigpause,ON);
    }

 if (strcmp(Dy->trigMenuItem,TRIGEVENTS) == 0)
    {
     opstring = fl_show_input("Write here the name of the Ascii file that contains triggers GPS times", Dy->trigEventName);
     if (opstring == NULL) return;
     else                  strcpy(Dy->trigEventName, opstring);
     DyMakeTrigEventList (Dy->trigEventName);
     Dy->triggerType = TRIG_EVENTS;
     Dy->triggerUsed = ON;
     fl_set_button(dyoptions->trigused,ON);
     Dy->triggerPause = ON;
     fl_set_button(dyoptions->trigpause,ON);
    }
 return;
}

/*------------------------------------------- DySetTriggerList ---*/

void DySetTriggerList()
{
 char *trigline, *word, *str;
 int i, triggps, nowgps;
 FILE *trigAsciiFile;

 DyDebugPrint(__FUNCTION__,0);

 trigAsciiFile = fopen(Dy->trigAsciiFileName,"r");
 if (trigAsciiFile == NULL)
    {printf("DySetTriggerList> trigger file %s does not exist.\n", Dy->trigAsciiFileName);
     Dy->triggerUsed = OFF;
     return;
    }

 trigline = (char *)malloc(LSTRING_MAX);

 Dy->nevent_ascii = 0;
 while (trigline && Dy->nevent_ascii < NEVENT_MAX-1)
    {
     trigline = fgets(trigline, LSTRING_MAX, trigAsciiFile);
     if (trigline)
        {word  = strtok(trigline," ");
         while (word != NULL)
            {triggps = atof(word);
             if (triggps == 0)
                {str = strstr(word,"=");
                 if (str) triggps = atof(str+1);
                }
             nowgps  = (int)FrGetCurrentGPS() + 3600;
             if (triggps > 70000000 && triggps < nowgps) word = NULL;
             else                                        word = strtok(NULL," ");
            }
         free(trigline);
         if (triggps > 70000000 && triggps < nowgps)
            {Dy->AsciiGps[Dy->nevent_ascii] = triggps; Dy->nevent_ascii++;}
        }
    }

 if (Dy->nevent_ascii == 0) Dy->triggerUsed = OFF;
 fclose(trigAsciiFile);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0)
    {for (i=0; i<100; i++) printf("  DySetTriggerList> Event %d/%d : GPS=%f \n", i, Dy->nevent_ascii, Dy->AsciiGps[i]);}

 return;
}


/*------------------------------------------- DySetTriggerString ---*/

void DySetTriggerString()
{
 long i, nvar;
 const char *opstring, *str;
 char dummy[LSTRING_MAX], inputVect[LBIGSTRING_MAX], opstring1[LBIGSTRING_MAX], opstring2[LBIGSTRING_MAX];
 char className[LSTRING_MAX];
 char defaultVectList[LBIGSTRING_MAX];
 FL_OBJECT *browser;
 struct DyCh *opch;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DySetTrigString> no frame in memory \n"); return;}

 strcpy(defaultVectList,"");
 strcpy(inputVect,"");
 strcpy(opstring2,"");

 /*--- build the vect list from selected items in the channels browsers ---*/

 if (fl_get_browser(mainbrowser->channels) != 0)
    {browser = mainbrowser->channels;
     nvar    = fl_get_browser_maxline(browser);
     for (i=0; i<nvar; i++)
        {if (fl_isselected_browser_line(browser,i+1))
            {str = fl_get_browser_line(browser,i+1);
             sscanf(str,"%s",dummy);
             if (strncmp(dummy,OPSIGNAL,strlen(OPSIGNAL)) != 0 && strncmp(dummy,TRIGGER,strlen(TRIGGER)) != 0)
                {strcat(defaultVectList," ");
                 strcat(defaultVectList,dummy);
                }
            }
        }
     }

 strcat(defaultVectList," \0");

/**
 if (defaultVectList  &&  strlen(defaultVectList) > 1)
    {
     sprintf(dummy2, "%s_%s = %s", getenv("USER"), TRIGGER, defaultVectList);
     strcpy(defaultVectList, dummy2);
    }
**/

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("    DySetTriggerString> defaultVectList=%s  previoustrigVectList=%s  Dy->lastTrigString=%s\n",
             defaultVectList, Dy->previoustrigVectList, Dy->lastTrigString);
    }


 /*--- If Trigger is from Channels, put vect list in an input field presented to the user ---*/

 if (strlen(defaultVectList) > 1)
    {    
     if (strcmp(defaultVectList, Dy->previoustrigVectList) != 0)
        {opstring = fl_show_input("Write here the trigger condition. Example:", defaultVectList);}
     else
        {opstring = fl_show_input("Write here the trigger condition. Example:", Dy->lastTrigString);}
    }
 else
    {opstring = fl_show_input("Write here the trigger condition. Example:", Dy->lastTrigString);}

 if (opstring == NULL) return;
 else                  strcpy(opstring1, opstring);

 if (strlen(defaultVectList) > 1) strcpy(Dy->previoustrigVectList, defaultVectList);

 strcpy(Dy->lastTrigString,opstring1);

 /*--- Build the className ---*/

/*
 sprintf(className,"%s_%s", getenv("USER"), TRIGGER);
*/
 strcpy(className, TRIGGER);

 /*--- Create the Trigger C String (opstring2 is the set of C code lines corresponding to opstring1) ---*/

 DyMakeTrigEventList(opstring1);
 DyMakeTrigCString(opstring1, opstring2, inputVect);

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("    className=%s \n",className);
     printf("    opstring1=%s \n",opstring1);
     printf("    opstring2=%s \n",opstring2);
     printf("    inputVect=%s \n",inputVect);
    }

 RA_MakeClass(className, opstring2, opstring1);
 DyLoadScript(className);

 /*--- Add the operation string result as a new proc data channel ---*/

 if (strlen(inputVect)>1)
    {opch = DyAddUserChannel(className, &inputVect[1]);
     if (opch == NULL) return;
     DyXformAddOneChannel(mainbrowser->channels, opch, -1);
    }

 Dy->triggerType = TRIG_CH;
 fl_set_button(dyoptions->trigused,ON);
 Dy->triggerUsed = ON;

 return;
}


/*------------------------------------------- DyMakeTrigCString ---*/

void DyMakeTrigCString ( char *opstring, char *opstring2, char *inputVect)
{
 int nvect;
 unsigned char c;
 char *p, *p2, *pstart, signalName[LSTRING_MAX];
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) 
   {printf("  DyMakeTrigCString> opstring=%s", opstring);}

 p      = opstring;
 p2     = opstring2; *p2='('; p2++; *p2='\0';
 pstart = NULL;
 nvect  = 0;

 while (*p != 0)
   {
    c = (unsigned char) *p;

    if (strchr (" \t~!\"'|\\<>=?^", c) != NULL)
       {
        if (pstart != NULL)
           {
            strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0  &&  c != '(' ) /* if this is really a signal name, not a constant */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2 += strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           }
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++; continue;
       }
 
    if (strchr ("+-*/(),", c) != NULL)
       {
        if (pstart != NULL)
           {strncpy(signalName,pstart, p-pstart);
            signalName[p-pstart] = '\0';
            if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0  &&  c != '(' ) /* if this is really a signal name, not a constant */
               {
                if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
                   {strcpy(p2,") & ("); p2 += 5;}
                else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
                   {strcpy(p2,") | ("); p2 += 5;}
                else
                   {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
                    strcpy(p2, dummy);
                    strcat(inputVect, OPSEP);
                    strcat(inputVect, signalName);
                    nvect++;
                    p2+=strlen(dummy);
                   }
               }
            else
               {strcpy(p2, signalName);
                p2+=strlen(signalName);
               }
            pstart = NULL;
           *p2 = c; p2++;
           } 
        else if (c != ' ')
           {*p2 = c; p2++;}
        p++;
        continue;
       }

    if (pstart == NULL) pstart = p;
    p++;
   }

 if (pstart != NULL)
    {strncpy(signalName,pstart, p-pstart);
     signalName[p-pstart] = '\0';
     if (atof(signalName) == 0.  &&  strncmp(signalName,"0",1) != 0) /* if this is really a signal name, not a constant */
        {
         if (strcmp(signalName,"AND") == 0 || strcmp(signalName,"and") == 0)
            {strcpy(p2,") & ("); p2 += 5;}
         else if (strcmp(signalName,"OR") == 0 || strcmp(signalName,"or") == 0)
            {strcpy(p2,") | ("); p2 += 5;}
         else
            {sprintf(dummy,"%s[%d]", VALUEIN, nvect);
             strcpy(p2, dummy);
             strcat(inputVect, OPSEP);
             strcat(inputVect, signalName);
             nvect++;
             p2+=strlen(dummy);
            }
        }
     else
        {strcpy(p2, signalName);
         p2+=strlen(signalName);
        }
     pstart = NULL;
    }

 *p2=')'; p2++; 
 *p2 = '\0';

 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
    {printf("    opstring=%s  opstring2=%s\n", opstring, opstring2);}

 return;
}


/*------------------------------------------- DyMakeTrigEventList ---*/

void DyMakeTrigEventList (char *opstring)
{
 char *chaine, *name, *name2, *name3, dummy[LSTRING_MAX];

 if (opstring == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyMakeTrigEventList>  opstring=%s", opstring);

 chaine = (char *)malloc(strlen(opstring)+1);
 strcpy(chaine, opstring);
 name = strtok(chaine," ");

 while (name != NULL)
    {if (strncmp(name,"EVENT",5) == 0)
        {name2 = strtok(NULL," ");
         if (strcmp(name2,">") == 0 || strcmp(name2,">=") == 0 || strcmp(name2,"=>") == 0)
            {name3 = strtok(NULL," ");
             if (name3)
                {strcpy(Dy->conditionName,name);
                 Dy->conditionTh = atof(name3);
                 if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug>0)
                    {printf("     opstring=%s chaine=%s  name=%s name2=%s name3=%s conditionName=%s conditionTh=%g \n",
                             opstring, chaine, name, name2, name3,  Dy->conditionName, Dy->conditionTh);
                    }
                 break;
                }
            }
        }
     name  = strtok(NULL," ");
    }
 free(chaine);

 return;
}

