/**************************************************/
/*                                                */
/* File: URZoomSeveral.cc                         */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#ifndef __CINT__
#include "dataDisplay.h"
#include "URootAccess.h"
#include "TROOT.h"
#include "VDyPlot.h"
#include "VDyPlot2.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TGaxis.h"
#include "TLegend.h"
#include "TPaveStats.h"
R__EXTERN TCanvas* gDDCanvas;
#endif

#include "TObject.h"

extern "C" {
void URUpdateTimeFreq(TObject* objaxis);
void URZoomXAsLast(TObject* objaxis);
void URZoomAllXAsLast(TObject* objaxis);
void URZoomYAsLast(TObject* objaxis);
void URZoomAllYAsLast(TObject* objaxis);
void URUnZoomAllY(TObject* objaxis);
void URUnZoomAllZ(TObject* objaxis);
void URUnZoomAllX(TObject* objaxis);
void URScaleYAsLast(TObject* objaxis);
void URScaleAllYAsLast();
void URZoomAllStat();
void URZoomAllLabels(TObject* objaxis);
void URLogLinAll(TObject* objaxis);
void URZoomAllLegends(TObject* objaxis);
void URUnZoomY(TObject* objaxis);
void URUnZoomX(TObject* objaxis);
void URUnZoomSame(TVirtualPad* pad, VDyPlot *lastModifiedPlotY);
void URUnZoomSame2(TVirtualPad* pad, VDyPlot2 *lastModifiedPlotY);
void URDrawOptionsAllAsThis(TObject* objaxis);


#ifndef __CINT__


/*-------------------------------- URUpdateTimeFreq ----------*/
void URUpdateTimeFreq(TObject* objaxis)
{
// Zoom all freq plots with the same range as the last plot

   TAxis* axis;
   int padmodified, padmodified2;
   Axis_t plotymin;
   Axis_t plotymax;

   TAxis* axislast = (TAxis*) objaxis;
   TH1 *href = (TH1 *)(axislast->GetParent());
   if (!href) return;

   if (strcmp(href->IsA()->GetName(),"VDyPlot") == 0) return;

   VDyPlot2* h0 = (VDyPlot2 *)href;

   if (strcmp(h0->mPlotType,"FFTTIME")!=0 && strcmp(h0->mPlotType,"TRFCTTIME")!=0 && strcmp(h0->mPlotType,"COHETIME")!=0) return;

   if (axislast == h0->GetXaxis()) return;

   if (axislast == h0->GetYaxis())
      {
       plotymin = axislast->GetBinLowEdge(axislast->GetFirst());
       plotymax = axislast->GetBinLowEdge(axislast->GetLast())
                  + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;
      }

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());

      /*--- Loop on all the objects of the root Canvas ---*/
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             /*--- If Pad encountered, loop on all the objects of this pad ---*/

              while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      axis = h->GetXaxis();

                      /*--- If plot encountered is FFT, COHE or TRFCT type, modify it ---*/

                      if (strcmp(h->mPlotType,"FFT")==0 || strcmp(h->mPlotType,"COHE")==0 || strcmp(h->mPlotType,"TRFCT")==0)
                         {
                          if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                             {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                              padmodified++;
                             }
                         }
                     }

                  /*--- If SubPad encountered, loop on all the objects of this Subpad ---*/

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          /*--- If plot encountered is COHE or TRFCT, modify it ---*/

                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;

                              const char *drawOption = h->GetDrawOption();
                              if (drawOption && (strstr(drawOption,"same") != NULL || strstr(drawOption,"SAME") != NULL)) continue;

                              axis = h->GetXaxis();

                              if (strcmp(h->mPlotType,"FFT")==0 || strcmp(h->mPlotType,"COHE")==0 || strcmp(h->mPlotType,"TRFCT")==0)
                                 {
                                  if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                                     {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                                      padmodified2++;
                                     }
                                 }
                             }
                          if (padmodified2>0) break;
                         }
//                   Update the subpad
                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                         }
                     }

                  /*--- If plot 2D encountered, change it only if it is of the same type as last modified plot ---*/

                  else if (obj2->InheritsFrom(VDyPlot2::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      axis = h->GetYaxis();

                       if (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0)
                         {
                          if (axis->FindBin(plotymin) < axis->FindBin(plotymax))
                            {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                             padmodified = 1;
                            }
                         }
                     }

                  if (padmodified>0) break;
                 }
//           Update the pad
             if (padmodified > 0)
                {
                 ((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    } 
}


/*-------------------------------- URZoomXAsLast ----------*/
void URZoomXAsLast(TObject* objaxis)
{
// Zoom X axis of the selected plot as the last that was zoomed or unzoomed
   
   if (gLastModifiedPlot == 0 && gLastModifiedPlot2 == 0) return;

   TAxis* axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent());
   if (!h0) return;

   if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0 && gLastModifiedPlot != 0)
      {
       VDyPlot *h = (VDyPlot *)(axis->GetParent());

       axis = h->GetXaxis();

       TAxis* axislast = gLastModifiedPlot->GetXaxis();
       Axis_t plotxmin = axislast->GetBinLowEdge(axislast->GetFirst());
       Axis_t plotxmax = axislast->GetBinLowEdge(axislast->GetLast())
                         + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;
       double plotgps = gLastModifiedPlot->GetTimeUTC();

       if (strcmp(h->mPlotType, gLastModifiedPlot->mPlotType) == 0)
          {
           if (strcmp(h->mPlotType,"TIME") == 0)
              {
               double hgps = h->GetTimeUTC();
               double delta = hgps - plotgps;
               if (axis->FindBin(plotxmin-delta) < axis->FindBin(plotxmax-delta))
                  {axis->SetRange(axis->FindBin(plotxmin-delta), axis->FindBin(plotxmax-delta));}
              }
           else
              {
               if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
                  {axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));}
              }
          }

       if (strcmp(h->mPlotType, "FFT") == 0 && 
           (strcmp(gLastModifiedPlot->mPlotType,"COHE") == 0  || strcmp(gLastModifiedPlot->mPlotType,"TRFCT") == 0))
          {
           if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
               axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));
          }

       if (strcmp(h->mPlotType, "TRFCT") == 0 && 
           (strcmp(gLastModifiedPlot->mPlotType,"COHE") == 0  || strcmp(gLastModifiedPlot->mPlotType,"FFT") == 0))
          {
           if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
               axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));
          }

       if (strcmp(h->mPlotType, "COHE") == 0 && 
           (strcmp(gLastModifiedPlot->mPlotType,"FFT") == 0  || strcmp(gLastModifiedPlot->mPlotType,"TRFCT") == 0))
          {
           if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
               axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));
          }
      }

   else if (strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0 && gLastModifiedPlot2 != 0)
      {
       VDyPlot2 *h = (VDyPlot2 *)(axis->GetParent());

       axis = h->GetXaxis();

       TAxis* axislast = gLastModifiedPlot2->GetXaxis();
       Axis_t plotxmin = axislast->GetBinLowEdge(axislast->GetFirst());
       Axis_t plotxmax = axislast->GetBinLowEdge(axislast->GetLast())
                         + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;
       double plotgps = gLastModifiedPlot2->GetTimeUTC();

       if ( (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"1DTIME")==0 ||
             strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0) && 
            (strcmp(gLastModifiedPlot2->mPlotType,"FFTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"1DTIME")==0 ||
             strcmp(gLastModifiedPlot2->mPlotType,"TRFCTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"COHETIME")==0) )
          {
           double hgps = h->GetTimeUTC();
           double delta = hgps - plotgps;
           if (axis->FindBin(plotxmin-delta) < axis->FindBin(plotxmax-delta))
              {axis->SetRange(axis->FindBin(plotxmin-delta), axis->FindBin(plotxmax-delta));}
          }

       else if (strcmp(h->mPlotType, gLastModifiedPlot2->mPlotType) == 0)
          {
           if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
              {axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));}
          }
      }

// Update the corresponding pad
   TVirtualPad* pad = gDDCanvas->GetSelectedPad();
   pad->Modified();
   pad->Update();
}


/*-------------------------------- URZoomAllXAsLast ----------*/
void URZoomAllXAsLast(TObject* objaxis)
{
// Zoom X axis of all the same type plots as the last that was zoomed or unzoomed
// In fact apply the zoom on all the plots with same type of X axis (time or frequency or else)
// h0=plot from which the function is called

 TObject *obj2;
 TAxis* axislast;
 TAxis* axis;
 TAxis* axis2;
 TAxis* axis1;
 double plotgps;

 // Get plot h0 from which axis was selected

 axis = (TAxis*) objaxis;
 TH1 *h0 = (TH1 *)(axis->GetParent()); 
 if (!h0) return;

 // If no plot was already zoomed, no zoom to propagate

 if (gLastModifiedPlot == 0   && strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)  return;
 if (gLastModifiedPlot2 == 0  && strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0) return;

 // If 1D plot, get X axis and GPS time of last zoomed 1D plot

 if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)
    {axislast = gLastModifiedPlot->GetXaxis();
     plotgps = gLastModifiedPlot->GetTimeUTC();
    }

 // If 2D plot, get X axis and GPS time of last zoomed 2D plot

 else
    {axislast = gLastModifiedPlot2->GetXaxis();
     plotgps = gLastModifiedPlot2->GetTimeUTC();
    }

 // Compute the xmin and xmax of the last zoomed plot

 Axis_t plotxmin = axislast->GetBinLowEdge(axislast->GetFirst());
 Axis_t plotxmax = axislast->GetBinLowEdge(axislast->GetLast())
                   + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;


 // Now look at plots available and propagate the zoom to all the plots found 

 TIter next(gROOT->GetList());

 while ((obj2 = next()))
    {
     if ((obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(VDyPlot2::Class()) )
         && strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)
       {
        VDyPlot* h = (VDyPlot *)obj2;
        axis = h->GetXaxis();
        VDyPlot2* h2 = (VDyPlot2 *)obj2;
        axis2 = h2->GetXaxis();
                      
        /*--- If plot encountered, change it only if it is of the same type as last modified plot ---*/

        if (strcmp(h->mPlotType, gLastModifiedPlot->mPlotType) == 0)
          {
           if (strcmp(h->mPlotType,"TIME") == 0)
             {
              double hgps = h->GetTimeUTC();
              double delta = hgps - plotgps;
              if (axis->FindBin(plotxmin-delta) < axis->FindBin(plotxmax-delta))
                {axis->SetRange(axis->FindBin(plotxmin-delta), axis->FindBin(plotxmax-delta));}
             }
           else
             {
              if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
                {axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));}
             }
          }

       /*--- If plot encountered, change it also if it is FFT, COHE or TRFCT type like for last modified plot ---*/

       else if ((strcmp(h->mPlotType,"FFT")==0 || strcmp(h->mPlotType,"COHE")==0 || strcmp(h->mPlotType,"TRFCT")==0) && 
                (strcmp(gLastModifiedPlot->mPlotType,"FFT")==0 || strcmp(gLastModifiedPlot->mPlotType,"COHE")==0 ||
                 strcmp(gLastModifiedPlot->mPlotType,"TRFCT") == 0))
         {
          if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
            {axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));}
         }

       /*--- If plot encountered, change it also if it has a time X axis like for last modified plot ---*/

       else if ((strcmp(h2->mPlotType,"FFTTIME")==0 || strcmp(h2->mPlotType,"COHETIME")==0 || strcmp(h2->mPlotType,"TRFCTTIME")==0) 
                 && strcmp(gLastModifiedPlot->mPlotType,"TIME")==0)
         {
          double hgps = h2->GetTimeUTC();
          double delta = hgps - plotgps;
          if (axis2->FindBin(plotxmin-delta) < axis2->FindBin(plotxmax-delta))
            {axis2->SetRange(axis2->FindBin(plotxmin-delta), axis2->FindBin(plotxmax-delta));}
         }
      }

   /*--- If plot 2D encountered, change it only if it is of the same type as last modified plot ---*/

   else if ((obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(VDyPlot2::Class()) )
            && strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0)
     {
      VDyPlot2* h = (VDyPlot2 *)obj2;
      axis = h->GetXaxis();
      VDyPlot* h1 = (VDyPlot *)obj2;
      axis1 = h1->GetXaxis();

      if ( (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"1DTIME")==0 ||
            strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0) && 
           (strcmp(gLastModifiedPlot2->mPlotType,"FFTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"1DTIME")==0 ||
            strcmp(gLastModifiedPlot2->mPlotType,"TRFCTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"COHETIME")==0) )
        {
         double hgps = h->GetTimeUTC();
         double delta = hgps - plotgps;
         if (axis->FindBin(plotxmin-delta) < axis->FindBin(plotxmax-delta))
           {axis->SetRange(axis->FindBin(plotxmin-delta), axis->FindBin(plotxmax-delta));}
        }

      else if ( (strcmp(h1->mPlotType,"TIME")==0) && 
               (strcmp(gLastModifiedPlot2->mPlotType,"FFTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"1DTIME")==0 ||
                strcmp(gLastModifiedPlot2->mPlotType,"TRFCTTIME")==0 || strcmp(gLastModifiedPlot2->mPlotType,"COHETIME")==0) )
        {
         double hgps = h1->GetTimeUTC();
         double delta = hgps - plotgps;
         if (axis1->FindBin(plotxmin-delta) < axis1->FindBin(plotxmax-delta))
           {axis1->SetRange(axis1->FindBin(plotxmin-delta), axis1->FindBin(plotxmax-delta));}
        }

      else if (strcmp(h->mPlotType, gLastModifiedPlot2->mPlotType) == 0)
        {
         if (axis->FindBin(plotxmin) < axis->FindBin(plotxmax))
           {axis->SetRange(axis->FindBin(plotxmin), axis->FindBin(plotxmax));}
        }
     }
  }

 gDDCanvas->Paint();
 gDDCanvas->Modified();
 gDDCanvas->Update();
}


/*-------------------------------- URZoomYAsLast ----------*/
void URZoomYAsLast(TObject* objaxis)
{
// Zoom Y axis of the selected plot as the last that was zoomed or unzoomed in Y
  
   if (gLastModifiedPlotY == 0 && gLastModifiedPlotY2 == 0) return;
   
   TAxis* axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent()); 

   if (!h0) return;

   if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0 && gLastModifiedPlotY != 0)
      {
       VDyPlot *h = (VDyPlot *)(axis->GetParent()); 

       Axis_t plotymin = gLastModifiedPlotY->GetMinimum();
       Axis_t plotymax = gLastModifiedPlotY->GetMaximum();

       if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0)
          {
           h->SetMinimum(plotymin);
           h->SetMaximum(plotymax);
          }
      }

   else if (strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0 && gLastModifiedPlotY2 != 0)
      {
       VDyPlot2 *h = (VDyPlot2 *)(axis->GetParent());
       axis = h->GetYaxis();

       TAxis* axislast = gLastModifiedPlotY2->GetYaxis();
       Axis_t plotymin = axislast->GetBinLowEdge(axislast->GetFirst());
       Axis_t plotymax = axislast->GetBinLowEdge(axislast->GetLast())
                         + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;

       if ( (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"1DTIME")==0 ||
             strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0) && 
            (strcmp(gLastModifiedPlotY2->mPlotType,"FFTTIME")==0 || strcmp(gLastModifiedPlotY2->mPlotType,"1DTIME")==0 ||
             strcmp(gLastModifiedPlotY2->mPlotType,"TRFCTTIME")==0 || strcmp(gLastModifiedPlotY2->mPlotType,"COHETIME")==0) )
          {
           if (axis->FindBin(plotymin) +1 < axis->FindBin(plotymax))
              {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));}
          }

       else if (strcmp(h->mPlotType, gLastModifiedPlotY2->mPlotType) == 0)
          {
           if (axis->FindBin(plotymin) +1 < axis->FindBin(plotymax))
              {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));}
          }
      }

// Update the pad
   TVirtualPad* pad = gDDCanvas->GetSelectedPad();
   pad->Modified();
   pad->Update();
}



/*-------------------------------- URZoomAllYAsLast ----------*/
void URZoomAllYAsLast(TObject* objaxis)
{
// Zoom Y axis of all the same type plots as the last that was zoomed or unzoomed

   Axis_t plotymin, plotymax;
   Axis_t plotymin2, plotymax2;
   int padmodified, padmodified2;
   TAxis* axislast;
   TAxis* axis;

   axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent()); 
   if (!h0) return;

   if (gLastModifiedPlotY == 0   && strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)  return;
   if (gLastModifiedPlotY2 == 0  && strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0) return;

   if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)
      {axislast = gLastModifiedPlotY->GetYaxis();
       plotymin = gLastModifiedPlotY->GetMinimum();
       plotymax = gLastModifiedPlotY->GetMaximum();
       plotymin2 = plotymin - 0.1 * (plotymax - plotymin);
       plotymax2 = plotymax + 0.1 * (plotymax - plotymin);
      }
   else
      {axislast = gLastModifiedPlotY2->GetYaxis();
       plotymin = axislast->GetBinLowEdge(axislast->GetFirst());
       plotymax = axislast->GetBinLowEdge(axislast->GetLast())
                  + 9.99*axislast->GetBinWidth(axislast->GetLast())/10;
      }

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             while ((obj2 = next()))
                 {
                  /*--- If plot encountered, change it only if it is of the same type as last modified plot ---*/

                  if (obj2->InheritsFrom(VDyPlot::Class()) && strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0)
                         {
                          if ((plotymin >= 0. && ((TPad *)obj)->GetLogy() > 0) || ((TPad *)obj)->GetLogy() == 0) h->SetMinimum(plotymin);
                          if ((plotymax >= 0. && ((TPad *)obj)->GetLogy() > 0) || ((TPad *)obj)->GetLogy() == 0) h->SetMaximum(plotymax);
                           padmodified = 1;
                         }
/***
                      else if ((strcmp(h->mPlotType, "FFT") == 0 || strcmp(h->mPlotType, "TRFCT") == 0 || strcmp(h->mPlotType, "COHE") == 0) && 
                               (strcmp(gLastModifiedPlotY->mPlotType,"COHE") == 0  || strcmp(gLastModifiedPlotY->mPlotType,"FFT") == 0) || strcmp(gLastModifiedPlotY->mPlotType,"TRFCT") == 0))
                         {
                          if ((plotymin >= 0. && ((TPad *)obj)->GetLogy() > 0) || ((TPad *)obj)->GetLogy() == 0) h->SetMinimum(plotymin);
                          if ((plotymax >= 0. && ((TPad *)obj)->GetLogy() > 0) || ((TPad *)obj)->GetLogy() == 0) h->SetMaximum(plotymax);
                           padmodified = 1;
                         }
***/
                     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("ZoomAllYAsLast>step1> plot=%s=%s  plotymin=%f plotymax=%f binmin=%d binmax=%d padmodified=%d\n",
                                  h->GetTitle(), h->GetName(), plotymin, plotymax,
                                  axis->FindBin(plotymin), axis->FindBin(plotymax), padmodified);
                         }
                    }

                  /*--- If SubPad encountered, loop on all the objects of this Subpad ---*/

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                     TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()) && strcmp(h0->IsA()->GetName(),"VDyPlot") == 0)
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              int hNum = atoi(&(h->GetName())[1]);
                              int hlastNum = atoi(&(gLastModifiedPlotY->GetName())[1]);
                              if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0  &&  hNum/1000 == hlastNum/1000)
                                 {
                                  if ((plotymin >= 0. && ((TPad *)obj2)->GetLogy() > 0) || ((TPad *)obj2)->GetLogy() == 0) h->SetMinimum(plotymin);
                                  if ((plotymax >= 0. && ((TPad *)obj2)->GetLogy() > 0) || ((TPad *)obj2)->GetLogy() == 0) h->SetMaximum(plotymax);
                                  padmodified2 = 1;
                                 }
                             }
                          if (padmodified2>0) break;
                         }

//                   Update the subpad
                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                         }
                     }

                  /*--- If plot 2D encountered, change it only if it is of the same type as last modified plot ---*/

                  else if (obj2->InheritsFrom(VDyPlot2::Class()) && strcmp(h0->IsA()->GetName(),"VDyPlot2") == 0)
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      axis = h->GetYaxis();

                      if ( (strcmp(h->mPlotType,"FFTTIME")==0 || strcmp(h->mPlotType,"1DTIME")==0 ||
                            strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"COHETIME")==0) && 
                           (strcmp(gLastModifiedPlotY2->mPlotType,"FFTTIME")==0 || strcmp(gLastModifiedPlotY2->mPlotType,"1DTIME")==0 ||
                            strcmp(gLastModifiedPlotY2->mPlotType,"TRFCTTIME")==0 || strcmp(gLastModifiedPlotY2->mPlotType,"COHETIME")==0) )
                         {
                          if (axis->FindBin(plotymin) +1 < axis->FindBin(plotymax))
                             {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                              padmodified = 1;
                             }
                         }

                      else if (strcmp(h->mPlotType, gLastModifiedPlotY2->mPlotType) == 0)
                         {
                          if (axis->FindBin(plotymin) +1 < axis->FindBin(plotymax))
                             {axis->SetRange(axis->FindBin(plotymin), axis->FindBin(plotymax));
                              padmodified = 1;
                             }
                         }

                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
                         {printf("ZoomAllYAsLast>step2> plot=%s=%s  plotymin=%f plotymax=%f binmin=%d binmax=%d padmodified=%d\n",
                                  h->GetTitle(), h->GetName(), plotymin, plotymax,
                                  axis->FindBin(plotymin), axis->FindBin(plotymax), padmodified);
                         }
 
                     }

                 if (padmodified>0) break;
                }
//           Update the pad
             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    } 
}



/*-------------------------------- URUnZoomAllY ----------*/
void URUnZoomAllY(TObject* objaxis)
{
// UnZoom Y axis of all plots of same type as the last touched in Y

   TAxis* axis;
 
   axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent()); 
   if (!h0) return;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0) URUnZoomSame((TVirtualPad *)obj, (VDyPlot *)h0);
             else                                             URUnZoomSame2((TVirtualPad *)obj, (VDyPlot2 *)h0);

             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0) URUnZoomSame((TVirtualPad *)obj2, (VDyPlot *)h0);
                      else                                             URUnZoomSame2((TVirtualPad *)obj2, (VDyPlot2 *)h0);
                     }
                }
            }
        }
    } 
}



/*-------------------------------- URUnZoomY ----------*/
void URUnZoomY(TObject* objaxis)
{
// UnZoom Y axis of all plots superposed in this pad
   TAxis* axis;
 
   axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent()); 
   if (!h0) return;

   TVirtualPad* pad = gDDCanvas->GetSelectedPad();
   if (strcmp(h0->IsA()->GetName(),"VDyPlot") == 0) URUnZoomSame(pad, (VDyPlot *)h0);
   else                                             URUnZoomSame2(pad, (VDyPlot2 *)h0);
}


/*-------------------------------- URUnZoomSame ----------*/
void URUnZoomSame(TVirtualPad* pad, VDyPlot *currentPlot)
{
// UnZoom Y axis of all plots superposed in this pad

 int imin, imax, imax2, fftcondition, fftcondition0;
 double plotymin, plotymax, value, delta;
 VDyPlot* horig;
 TObject *obj2;

 if (currentPlot == NULL) return; 

 horig = NULL;

 TIter    next(pad->GetListOfPrimitives());
 while ((obj2 = next()))
    {
     if (obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(TH1F::Class()))
       {
        VDyPlot* h = (VDyPlot *)obj2;

        fftcondition0 = strcmp(currentPlot->mPlotType,"FFT")!=0 && strcmp(currentPlot->mPlotType,"COHE")!=0 && strcmp(currentPlot->mPlotType,"TRFCT")!=0;
        fftcondition  = strcmp(h->mPlotType,"FFT")!=0 && strcmp(h->mPlotType,"COHE")!=0 && strcmp(h->mPlotType,"TRFCT")!=0;
        if (strcmp(currentPlot->mPlotType,"FFT")==0 && fftcondition)        return;
        else if (strcmp(currentPlot->mPlotType,"COHE")==0 && fftcondition)  return;
        else if (strcmp(currentPlot->mPlotType,"TRFCT")==0 && fftcondition) return;
        else if (strcmp(h->mPlotType, currentPlot->mPlotType) != 0 && fftcondition0) return;

        /*--- If first plot found in the pad ---*/

        if (horig == NULL)
          {
           /*--- Take X axis edges of first plot found and init the ymin and ymax values ---*/

           horig = h;
           imin = h->GetXaxis()->GetFirst();
           imax = h->GetXaxis()->GetLast();
           plotymin = h->GetBinContent(imin);
           if (imin == 0 && plotymin <= 0.) plotymin = h->GetBinContent(imin+1);
           plotymax = h->GetBinContent(imin);
          }

        /*--- Limit the imax value to non empty bins ---*/

        for (int i=imax;i>=imin;i--) {if (h->GetBinContent(i) != 0.) {imax2 = i; break;}}

        /*--- Compute the ymin and ymax values against the values of this plot ---*/

        for (int i=imin;i<=imax2;i++)
           {value = h->GetBinContent(i);
            if (value > plotymax) plotymax = value;
            if (value<= 0. && pad->GetLogy() > 0) continue;
            if (value < plotymin ) plotymin = value;
           }

        if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
           {printf("URUnZoomSame> plot %s (%s) plotymin=%g  plotymax=%g  getmin=%g  getmax=%g "
                   " imin=%d  imax=%d  uymin=%g  uymax=%g \n",
                   h->GetName(), h->mPlotType, plotymin, plotymax, h->GetMinimum(), h->GetMaximum(),
                   imin, imax, pad->GetUymin(), pad->GetUymax());
           }
       }
   }

 /*--- Put the ymin and ymax values as new Y limits of the first plot found horig ---*/

 if (horig)
   {
    if (strcmp(horig->mPlotType,"TIME") == 0)
      {delta = 0.1 * (plotymax - plotymin);
       plotymin = plotymin - delta;
       plotymax = plotymax + delta;
       if (pad->GetLogy() > 0) plotymin = (plotymin+delta) / 2.;
      }
    else
      {
       if (pad->GetLogy() > 0)
          {plotymin = plotymin / 2.;
           plotymax = plotymax * 2.;
          }
       else
          {plotymin = plotymin * 0.9;
           plotymax = plotymax * 1.1;
          }
      }
   horig->SetMinimum(plotymin);
   horig->SetMaximum(plotymax);
  }

 
 /*--- Put the ymin and ymax values as new Y limits also of the other plots of the pad ---*/

 TIter next2(pad->GetListOfPrimitives());
 while ((obj2 = next2()))
    {
     if (obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(TH1F::Class()))
       {
        VDyPlot* h = (VDyPlot *)obj2;
        h->SetMinimum(plotymin);
        h->SetMaximum(plotymax);
        if (strcmp(h->mPlotType,"1D")==0) h->SetMinimum(0.1);
       }
    }

 /*--- Update the corresponding pad ---*/

 if (horig)
   {
    pad->Modified();
    pad->Update();
   }

}


/*-------------------------------- URUnZoomSame2 ----------*/
void URUnZoomSame2(TVirtualPad* pad, VDyPlot2 *currentPlot)
{
// UnZoom Y axis of all 2D plots present in this pad

 int bmin, bmax, nybin;
 int fftcondition, fftcondition0;
 double ymin, ymax;
 TObject *obj2;
 
 if (currentPlot == NULL) return;

 TIter    next(pad->GetListOfPrimitives());
 while ((obj2 = next()))
    {
     if (obj2->InheritsFrom(VDyPlot2::Class()) || obj2->InheritsFrom(TH2F::Class()))
       {
        VDyPlot2* h = (VDyPlot2 *)obj2;

        fftcondition0 = strcmp(currentPlot->mPlotType,"FFTTIME")!=0 && strcmp(currentPlot->mPlotType,"COHETIME")!=0 && strcmp(currentPlot->mPlotType,"TRFCTTIME")!=0;
        fftcondition  = strcmp(h->mPlotType,"FFTTIME")!=0 && strcmp(h->mPlotType,"COHETIME")!=0 && strcmp(h->mPlotType,"TRFCTTIME")!=0;
        if (strcmp(currentPlot->mPlotType,"FFTTIME")==0 && fftcondition)        return;
        else if (strcmp(currentPlot->mPlotType,"COHETIME")==0 && fftcondition)  return;
        else if (strcmp(currentPlot->mPlotType,"TRFCTTIME")==0 && fftcondition) return;
        else if (strcmp(h->mPlotType, currentPlot->mPlotType) != 0 && fftcondition0) return;

//        if (currentPlot == NULL || (currentPlot && strcmp(h->mPlotType, currentPlot->mPlotType) == 0)
//            || (currentPlot && strcmp(currentPlot->mPlotType,"FFTTIME")==0 && (strcmp(h->mPlotType,"COHETIME")==0 || strcmp(h->mPlotType,"TRFCTTIME")==0))
 //           || (currentPlot && strcmp(currentPlot->mPlotType,"COHETIME")==0 && (strcmp(h->mPlotType,"TRFCTTIME")==0 || strcmp(h->mPlotType,"FFTTIME")==0))
  //          || (currentPlot && strcmp(currentPlot->mPlotType,"TRFCTTIME")==0 && (strcmp(h->mPlotType,"COHETIME")==0 || strcmp(h->mPlotType,"FFTTIME")==0)) )

   //        {
            ymin = h->GetYaxis()->GetXmin();
            ymax = h->GetYaxis()->GetXmax();
            bmin = h->GetYaxis()->FindBin(ymin);
            bmax = h->GetYaxis()->FindBin(ymax);
            nybin = h->GetNbinsY();
            h->GetYaxis()->SetRange(bmin, bmax);
     //      }
//        break;
       }
    }

// Update the corresponding pad
 pad->Modified();
 pad->Update();
}


/*-------------------------------- URUnZoomAllZ ----------*/
void URUnZoomAllZ(TObject* objaxis)
{
// UnZoom Z axis of all 2D plots

   int padmodified;
   TAxis* axis;
 
   axis = (TAxis*) objaxis;
   TH1 *h0 = (TH1 *)(axis->GetParent()); 
   if (!h0) return;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             padmodified = 0;
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             while ((obj2 = next()))
                 {if (obj2->InheritsFrom(VDyPlot2::Class()) || obj2->InheritsFrom(TH2F::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      h->SetMinimum(-1111);
                      h->SetMaximum(-1111);
                      padmodified = 1;
                     }
                 }
             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    } 
}



/*-------------------------------- URUnZoomAllX ----------*/
void URUnZoomAllX(TObject* objaxis)
{
// UnZoom X axis of all plots of same type as the last touched in X

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             gDDCanvas->SetSelectedPad((TPad *)obj);
             URUnZoomX(objaxis);

             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      gDDCanvas->SetSelectedPad((TPad *)obj2);
                      URUnZoomX(objaxis);
                     }
                }
            }
        }
    } 
}


/*-------------------------------- URUnZoomX ----------*/
void URUnZoomX (TObject* objaxis)
{
// UnZoom X axis of all plots superposed in this pad

 int nxbin, bmin, bmax;
 double xmin, xmax;
 TObject *obj2;
 TAxis* axis;

  axis = (TAxis*) objaxis;
  TH1 *h0 = (TH1 *)(axis->GetParent()); 
  if (!h0) return;

  TVirtualPad* pad = gDDCanvas->GetSelectedPad();

  TIter    next(pad->GetListOfPrimitives());
  while ((obj2 = next()))
     {
      if (obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(TH1F::Class()))
         {
          VDyPlot* h = (VDyPlot *)obj2;
          VDyPlot* h01 = (VDyPlot *)h0;

          if (strcmp(h->mPlotType, h01->mPlotType) != 0 &&
              ((strcmp(h->mPlotType, "FFT") != 0 && strcmp(h->mPlotType, "COHE") != 0 && strcmp(h->mPlotType, "TRFCT") != 0) ||
               (strcmp(h01->mPlotType,"FFT") != 0 && strcmp(h01->mPlotType,"COHE") != 0 && strcmp(h01->mPlotType,"TRFCT") != 0)))
             {continue;}

          xmin = h->GetXaxis()->GetXmin();
          xmax = h->GetXaxis()->GetXmax();
          bmin = h->GetXaxis()->FindBin(xmin);
          bmax = h->GetXaxis()->FindBin(xmax);
          nxbin = h->GetNbinsX();
          if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
             {printf("UnZoomX> h=%s=%s nxbin=%d xmin=%g xmax=%g bmin=%d bmax=%d\n",
                      h->GetTitle(), h->GetName(), nxbin, xmin, xmax, bmin, bmax);
             }
          h->GetXaxis()->SetRange(bmin, bmax);
        }

      else if (obj2->InheritsFrom(VDyPlot2::Class()) || obj2->InheritsFrom(TH2F::Class()))
         {
          VDyPlot2* h = (VDyPlot2 *)obj2;
          VDyPlot2* h02 = (VDyPlot2 *)h0;
          VDyPlot* h01 = (VDyPlot *)h0;
/* HOHO */
/*
printf("h0=%s  type=%s  type2=%s \n", h0->GetName(), h01->mPlotType, h02->mPlotType);
*/
          if (strcmp(h02->mPlotType,"2D") != 0 && strstr(h02->mPlotType,"TIME") == 0 &&
              strcmp(h01->mPlotType,"TIME") != 0 && strcmp(h02->mPlotType,"RAW") != 0)
             {continue;}

          if ((strcmp(h->mPlotType,"RAW") != 0 && strcmp(h02->mPlotType,"RAW") == 0) ||
              (strcmp(h->mPlotType,"RAW") == 0 && strcmp(h02->mPlotType,"RAW") != 0))
             {continue;}
          if ((strcmp(h->mPlotType,"2D") != 0 && strcmp(h02->mPlotType,"2D") == 0) ||
              (strcmp(h->mPlotType,"2D") == 0 && strcmp(h02->mPlotType,"2D") != 0))
             {continue;}
          if ((strstr(h->mPlotType,"TIME") != 0 && strstr(h02->mPlotType,"TIME") == 0) ||
              (strstr(h->mPlotType,"TIME") == 0 && strstr(h02->mPlotType,"TIME") != 0))
             {continue;}

          xmin = h->GetXaxis()->GetXmin();
          xmax = h->GetXaxis()->GetXmax();
          bmin = h->GetXaxis()->FindBin(xmin);
          bmax = h->GetXaxis()->FindBin(xmax);
          nxbin = h->GetNbinsX();
          if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug>1)
             {printf("UnZoomX> h=%s=%s nxbin=%d xmin=%g xmax=%g bmin=%d bmax=%d\n",
                      h->GetTitle(), h->GetName(), nxbin, xmin, xmax, bmin, bmax);
             }
          h->GetXaxis()->SetRange(bmin, bmax);
        }
    }

// Update the corresponding pad
   pad->Modified();
   pad->Update();
}


/*-------------------------------- URScaleYAsLast ----------*/
void URScaleYAsLast(TObject* objaxis)
{
// Scale the selected plot as the last that was zoomed or unzoomed

 double scale1, scale2;
 double currentMin, currentMax, newMin, newMax;

   if (gLastModifiedPlotY == 0) return;

   TVirtualPad* pad = gDDCanvas->GetSelectedPad();

   TAxis* axis = (TAxis*) objaxis;
   VDyPlot *h = (VDyPlot *)(axis->GetParent()); 

   if (!h) return;

   currentMin = h->GetMinimum();
   currentMax = h->GetMaximum();

   if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0)
      {
       scale1 = (gLastModifiedPlotY->mOldMaxY - gLastModifiedPlotY->mOldMinY)
               / (gLastModifiedPlotY->mPreviousMaxY - gLastModifiedPlotY->mPreviousMinY);
       scale2 = (gLastModifiedPlotY->mOldMinY - gLastModifiedPlotY->mPreviousMinY)
               / (gLastModifiedPlotY->mPreviousMaxY - gLastModifiedPlotY->mPreviousMinY);

       newMin = currentMin + scale2 * (currentMax-currentMin);
       newMax = newMin     + scale1 * (currentMax-currentMin);

       h->SetMinimum(newMin);
       h->SetMaximum(newMax);
      }

// Update the pad
   pad->Modified();
   pad->Update();
}



/*-------------------------------- URScaleAllYAsLast ----------*/
void URScaleAllYAsLast()
{
// Scale all the same type plots as the last that was zoomed or unzoomed

   int padmodified, padmodified2, nplot;

   if (gLastModifiedPlotY == 0) return;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             while ((obj2 = next()))
                 {
                  nplot = 0;
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0 && nplot==0)
                         {
                          URScaleYAsLast(h->GetYaxis());
                          padmodified = 1;
                         }
                     nplot++;
                    }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          nplot = 0;
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              int hNum = atoi(&(h->GetName())[1]);
                              int hlastNum = atoi(&(gLastModifiedPlotY->GetName())[1]);
                              if (strcmp(h->mPlotType, gLastModifiedPlotY->mPlotType) == 0 && nplot==0 && hNum/1000 == hlastNum/1000)
                                 {
                                  URScaleYAsLast(h->GetYaxis());
                                  padmodified2 = 1;
                                 }
                              nplot++;
                             }
                         }

                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }
}


/*-------------------------------- URZoomAllStat ----------*/
void URZoomAllStat()
{
// Larger StatBox on all the plots

 int padmodified, padmodified2;
 double x1, y1, x1new, y1new;
 TPaveStats* pavestat;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;

                      pavestat= (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                      if (pavestat)
                         {x1    = pavestat->GetX1NDC();
                          y1    = pavestat->GetY1NDC();
                          x1new = x1 - x1/10;
                          y1new = y1 - y1/10;
                          if (y1new < 0.5) y1new = 0.5;
                          pavestat->SetX1NDC(x1new);
                          pavestat->SetY1NDC(y1new);
                          padmodified = 1;
                         }
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              pavestat = (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                              if (pavestat)
                                 {x1    = pavestat->GetX1NDC();
                                  y1    = pavestat->GetY1NDC();
                                  x1new = x1 - x1/10;
                                  y1new = y1 - y1/10;
                                  if (y1new < 0.5) y1new = 0.5;
                                  pavestat->SetX1NDC(x1new);
                                  pavestat->SetY1NDC(y1new);
                                  padmodified2 = 1;
                                 }
                             }
                         }


                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}






/*-------------------------------- URZoomAllLegends ----------*/
void URZoomAllLegends(TObject* objaxis)
{
// Larger legends for all the plots

 int padmodified, padmodified2, only_trfct;
 double lsize, yleg, xleg, xleg2;
 TLegend* leg;

  if (objaxis == NULL) return;

   TAxis* axis = (TAxis*) objaxis;
   VDyPlot *horig = (VDyPlot *)(axis->GetParent()); 

   only_trfct = 0;
   if (horig && strcmp(horig->mPlotType,"TRFCT")==0) only_trfct = 1;


   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             leg = (TLegend*)(((TPad *)obj)->FindObject("DyLegendString"));
             if (leg && only_trfct == 0)
                {lsize = leg->GetTextSize();
                 yleg  = leg->GetY1NDC();
                 xleg  = leg->GetX1NDC();
                 xleg2 = leg->GetX2NDC();
                 if (lsize == 0) lsize = 0.04;
                 lsize += 0.007;
                 leg->SetTextSize(lsize);
                 yleg -= 0.02;
                 leg->SetY1NDC(yleg);
                 if (xleg <1.-xleg2)
                    {xleg2 += 0.05;
                     leg->SetX2NDC(xleg2);
                    }
                 else
                    {xleg -= 0.05;
                     leg->SetX1NDC(xleg);
                    }
                 padmodified = 1;
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      padmodified2 = 0;
 
                      leg = (TLegend*)(((TPad *)obj2)->FindObject("DyLegendString"));
                      if (leg)
                         {lsize = leg->GetTextSize();
                          yleg  = leg->GetY1NDC();
                          xleg  = leg->GetX1NDC();
                          xleg2 = leg->GetX2NDC();
                          if (lsize == 0) lsize = 0.05;
                          lsize += 0.007;
                          leg->SetTextSize(lsize);
                          yleg -= 0.02;
                          leg->SetY1NDC(yleg);
                          if (xleg <1.-xleg2)
                             {xleg2 += 0.05;
                              leg->SetX2NDC(xleg2);
                             }
                          else
                             {xleg -= 0.05;
                              leg->SetX1NDC(xleg);
                             }
                          padmodified2 = 1;
                         }

                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}


/*-------------------------------- URZoomAllLabels ----------*/
void URZoomAllLabels(TObject* objaxis)
{
// Larger Labels on axis of all the plots, and larger GPSorigin, title and unitsY
// If chosen axis is from a TRFCT plot, then changes affect only TRFCT plots

 int padmodified, padmodified2, only_trfct;
 double lsize, yoffset;
 TPaveText* timepave;

   if (objaxis == NULL) return;

   TAxis* axis = (TAxis*) objaxis;
   VDyPlot *horig = (VDyPlot *)(axis->GetParent()); 

   only_trfct = 0;
   if (horig && strcmp(horig->mPlotType,"TRFCT")==0) only_trfct = 1;

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             timepave = (TPaveText*)(((TPad *)obj)->FindObject("title"));
             if (timepave && strcmp(axis->GetName(),"xaxis") == 0 && only_trfct == 0)
                {lsize = timepave->GetTextSize();
                 if (lsize == 0) lsize = 0.05;
                 lsize += 0.01;
                 timepave->SetTextSize(lsize);
                }
             timepave = (TPaveText*)(((TPad *)obj)->FindObject("DyTimeString"));
             if (timepave && strcmp(axis->GetName(),"xaxis") == 0 && only_trfct == 0)
                {lsize = timepave->GetTextSize();
                 if (lsize == 0) lsize = 0.04;
                 lsize += 0.01;
                 timepave->SetTextSize(lsize);
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()) || obj2->InheritsFrom(VDyPlot2::Class()) || obj2->InheritsFrom(TH2F::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      if (strcmp(axis->GetName(),"xaxis") == 0 && only_trfct == 0)
                         {lsize = h->GetLabelSize("X");
                          lsize += 0.01;
                          h->SetLabelSize(lsize, "X");
                         }
                      else if (only_trfct == 0)
                         {lsize = h->GetLabelSize("Y");
                          lsize += 0.01;
                          h->SetLabelSize(lsize, "Y");
                          lsize = h->GetTitleSize("Y");
                          lsize += 0.01;
                          h->SetTitleSize(lsize, "Y");
                          yoffset = h->GetTitleOffset("Y");
                          yoffset -= 0.22;
                          h->SetTitleOffset(yoffset, "Y");
                         }
                      padmodified = 1;  
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("title"));
                      if (timepave && strcmp(axis->GetName(),"xaxis") == 0)
                         {lsize = timepave->GetTextSize();
                          if (lsize == 0) lsize = 0.05;
                          lsize += 0.01;
                          timepave->SetTextSize(lsize);
                         }
                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("DyTimeString"));
                      if (timepave && strcmp(axis->GetName(),"xaxis") == 0)
                         {lsize = timepave->GetTextSize();
                          if (lsize == 0) lsize = 0.04;
                          lsize += 0.01;
                          timepave->SetTextSize(lsize);
                         }

                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              if (strcmp(axis->GetName(),"xaxis") == 0)
                                 {lsize = h->GetLabelSize("X");
                                  lsize += 0.01;
                                  h->SetLabelSize(lsize, "X");
                                  h->SetLabelOffset(-0.01, "X");
                                 }
                              else
                                 {lsize = h->GetLabelSize("Y");
                                  lsize += 0.01;
                                  h->SetLabelSize(lsize, "Y");
                                  lsize = h->GetTitleSize("Y");
                                  lsize += 0.01;
                                  h->SetTitleSize(lsize, "Y");
                                  yoffset = h->GetTitleOffset("Y");
                                  yoffset -= 0.22;
                                  h->SetTitleOffset(yoffset, "Y");
                                 }
                              padmodified2 = 1;
                             }
                         }


                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}




/*-------------------------------- URLogLinAll ----------*/
void URLogLinAll(TObject* objaxis)
{
// Put all plots in logx/linx or logy/liny

 int padmodified, padmodified2;
 static int logxFlip=1;
 static int logyFlip=1;

   if (objaxis == NULL) return;

   TAxis* axis = (TAxis*) objaxis;

   TH1 *h0 = (TH1 *)(axis->GetParent()); 
   if (!h0) return;

   if (strcmp(axis->GetName(),"xaxis") == 0) logxFlip = abs(logxFlip-1);
   else                                      logyFlip = abs(logyFlip-1);

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());
      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      VDyPlot* horig = (VDyPlot *)h0;

                      if ((strcmp(horig->mPlotType,"TRFCT") == 0 || strcmp(horig->mPlotType,"COHE") == 0 || strcmp(horig->mPlotType,"FFT") == 0)
                          && (strcmp(h->mPlotType,"TRFCT") == 0 || strcmp(h->mPlotType,"COHE") == 0 || strcmp(h->mPlotType,"FFT") == 0))
                         {padmodified = 1;
                          if (strcmp(axis->GetName(),"xaxis") == 0) ((TPad *)obj)->SetLogx(logxFlip);
                          else                                      ((TPad *)obj)->SetLogy(logyFlip);
                         }

                      else if (strcmp(horig->mPlotType,h->mPlotType) == 0 && strcmp(h->mPlotType,"TIME") == 0)
                         {padmodified = 1;
                          if (strcmp(axis->GetName(),"xaxis") == 0)
                             {((TPad *)obj)->SetLogx(0);}
                          else
                             {if (logyFlip == 0 || (logyFlip == 1 && h->GetMinimum() > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                         }

                      else if (strcmp(horig->mPlotType,h->mPlotType) == 0)
                         {padmodified = 1;
                          if (strcmp(axis->GetName(),"xaxis") == 0)
                             {if (logxFlip == 0 || (logxFlip == 1 && h->GetXaxis()->GetBinLowEdge(axis->GetFirst()) > 0)) ((TPad *)obj)->SetLogx(logxFlip);}
                          else
                             {if (logyFlip == 0 || (logyFlip == 1 && h->GetMinimum() > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                         }
                      break;
                     }

                  else if (obj2->InheritsFrom(VDyPlot2::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      VDyPlot2* horig = (VDyPlot2 *)h0;

                      if ( (strcmp(horig->mPlotType,"FFTTIME") == 0 || strcmp(horig->mPlotType,"1DTIME") == 0 ||
                            strcmp(horig->mPlotType,"COHETIME") == 0 || strcmp(horig->mPlotType,"TRFCTTIME") == 0) &&
                           (strcmp(h->mPlotType,"FFTTIME") == 0 || strcmp(h->mPlotType,"1DTIME") == 0 ||
                            strcmp(h->mPlotType,"COHETIME") == 0 || strcmp(h->mPlotType,"TRFCTTIME") == 0))
                         {padmodified = 1;
                          if (strcmp(axis->GetName(),"xaxis") == 0)
                             {((TPad *)obj)->SetLogx(0);}
                          else
                             {if (strcmp(h->mPlotType,"1DTIME") == 0)
                                 {if (logyFlip == 0 || (logyFlip == 1 && h->GetYaxis()->GetBinLowEdge(axis->GetFirst()) > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                              else ((TPad *)obj)->SetLogy(logyFlip);
                             }
                         }

                      else if (strcmp(horig->mPlotType,h->mPlotType) == 0)
                         {padmodified = 1;
                          if (strcmp(axis->GetName(),"xaxis") == 0)
                             {if (logxFlip == 0 || (logxFlip == 1 && h->GetXaxis()->GetBinLowEdge(axis->GetFirst()) > 0)) ((TPad *)obj)->SetLogx(logxFlip);}
                          else
                             {if (logyFlip == 0 || (logyFlip == 1 && h->GetYaxis()->GetBinLowEdge(axis->GetFirst()) > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                         }
                      break;
                     }

                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                        {
                         if (obj3->InheritsFrom(VDyPlot::Class()))
                            {
//                             VDyPlot* h = (VDyPlot *)obj3;
                             VDyPlot* horig = (VDyPlot *)h0;

                             if (strcmp(horig->mPlotType,"TRFCT") == 0 || strcmp(horig->mPlotType,"COHE") == 0 || strcmp(horig->mPlotType,"FFT") == 0)
                                {
                                 padmodified2 = 1;
                                 if (strcmp(axis->GetName(),"xaxis") == 0) ((TPad *)obj)->SetLogx(logxFlip);
                                 else                                      ((TPad *)obj)->SetLogy(logyFlip);
                                }
                             break;
                            }
                        }

                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}


/*-------------------------------- URDrawOptionsAllAsThis ----------*/

void URDrawOptionsAllAsThis(TObject* objaxis)
{
 unsigned int i;
 int padmodified, padmodified2;
 char newoption;
 Option_t *option;

   if (objaxis == NULL) return;

   TAxis* axis = (TAxis*) objaxis;
   VDyPlot *horig = (VDyPlot *)(axis->GetParent()); 
   option = horig->GetDrawOption();

   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
     {
      TObject *obj;
      TIter    next(gDDCanvas->GetListOfPrimitives());

      while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      Option_t *optbis = h->GetDrawOption();
                      TString optbis2;
                      newoption = ' ';
                      for (i=0; i<strlen(option); i++) if (option[i] == 'l' || option[i] == 'h') {newoption = option[i]; break;}
                      for (i=0; i<strlen(optbis); i++)
                         {if (optbis[i] == '9') continue; 
                          if (optbis[i] == 'l' || optbis[i] == 'h') optbis2 = optbis2 + newoption;
                          else                                      optbis2 = optbis2 + optbis[i];
                         } 
                      h->SetDrawOption(optbis2.Data());
                      padmodified = 1;  
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              Option_t *optbis = h->GetDrawOption();
                              TString optbis2;
                              newoption = ' ';
                              for (i=0; i<strlen(option); i++) if (option[i] == 'l' || option[i] == 'h') {newoption = option[i]; break;}
                              for (i=0; i<strlen(optbis); i++)
                                 {if (optbis[i] == '9') continue; 
                                  if (optbis[i] == 'l' || optbis[i] == 'h') optbis2 = optbis2 + newoption;
                                  else                                      optbis2 = optbis2 + optbis[i];
                                 } 
                              h->SetDrawOption(optbis2.Data());
                              padmodified2 = 1;
                             }
                         }

                     if (padmodified2 > 0)
                        {((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }
                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }
}

#endif

}
