/**************************************************/
/*                                                */
/* File: URootAccess.cc                           */
/* Copyright (C) 2003, D. Buskulic, D. Verkindt   */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/
 
// Wrappers allowing access to ROOT functions

using namespace std;

#include <fstream>
#include <iostream>

/**
#include <time.h>
**/

#include "FrameL.h"
#include "TSystem.h"
#include "TClass.h"
#include "TEnv.h"
#include "TCanvas.h"
#include "TH1.h"
#include "TH2.h"
#include "TAxis.h"
#include "TStyle.h"
#include "TDirectory.h"
#include "TFile.h"
#include "TKey.h"
#include "TApplication.h"
#include "TROOT.h"
#include "TImage.h"
#include "TColor.h"

#if ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0)
 #include "TAttImage.h"
 #include "TImageDump.h"
#endif

#include "TList.h"
#include "TLegendEntry.h"
#include "TLegend.h"
#include "TPaveText.h"
#include "TPaveStats.h"
#include "TTimer.h"
#include "TObjectTable.h"
#include "TStorage.h"
#include "TFrame.h"
#include "TString.h"
#include "TSystem.h"

#include "TInterpreter.h"
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,3,4)
#include "TClassMenuItem.h"
#endif
#include "TMethod.h"

#include "dataDisplay.h"
#include "URootAccess.h"
#include "VDyPlot.h"
#include "VDyPlot2.h"

/**
#include <sys/time.h>

#ifdef R__ALPHA
#include <sys/table.h>
#endif
**/

#if ROOT_VERSION_CODE <= ROOT_VERSION(3,3,9)
TROOT root("DD", "Data Display");
#endif

TCanvas*         gDDCanvas=NULL;
TVirtualPad*     gLastPad=NULL;
double           gLowestInterval;
TTimer*          gCheckTimer;
TString          gDDCanvasString;
char*            gDyTmpMacro;

//_____________________________________________________________________________
int RA_MainCanvasCheck()
{
/**
 if (gDDCanvas->GetCanvasImpl()==NULL)
   {
    printf(">>>>> window has been closed by user\n");
    return(OFF);
   }
**/
 if (!gROOT->GetListOfCanvases()->FindObject("gDDCanvas"))
    {
     printf(">>>> canvas has disappeared \n");
     return(OFF);
    }
 return(ON);
}


//_____________________________________________________________________________
void RA_Main()
{
// Main for the root event loop

// Sets the timer to call periodically check_forms
 gCheckTimer = new TTimer(50);
 TQObject::Connect(gCheckTimer, "Timeout()", 0,0, "Wrapper_fl_check_forms()");
 gCheckTimer->Start();

// Runs the root event loop
 gApplication->Run();
}


//_____________________________________________________________________________
double RA_GetSize (char *objname)
{
 double size;
 TPaveStats* pavestat;
 TLegend* leg;
 TPaveText* timepave;

 if (objname == NULL) return(0); 

 size = 0;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             if (strcmp(objname,"legend") == 0) 
               {leg = (TLegend*)(((TPad *)obj)->FindObject("DyLegendString"));
                if (leg)
                  {if (size < leg->GetTextSize()) size = leg->GetTextSize();
                   if (Dy->debug > 0) printf("RA_GetSize> Found in %d a DyLegendString: size=%g \n", ((TPad *)obj)->GetNumber(), size);
                  }
               }

             if (strcmp(objname,"timestring") == 0) 
               {timepave = (TPaveText*)(((TPad *)obj)->FindObject("DyTimeString"));
                if (timepave)
                  {if (size < timepave->GetTextSize()) size = timepave->GetTextSize();
                   if (Dy->debug > 0) printf("RA_GetSize> Found in %d a DyTimeString: size=%g \n", ((TPad *)obj)->GetNumber(), size);
                  }
               }

             if (strcmp(objname,"title") == 0) 
               {timepave = (TPaveText*)(((TPad *)obj)->FindObject("title"));
                if (timepave)
                  {if ( size < timepave->GetTextSize()) size = timepave->GetTextSize();
                   if (Dy->debug > 0) printf("RA_GetSize> Found in %d a title string: size=%g \n", ((TPad *)obj)->GetNumber(), size);
                  }
               }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                    {TH1* h = (TH1 *)obj2;
                     if (strcmp(objname,"stat") == 0)
                       {pavestat= (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                        if (pavestat)
                          {if (size < 1-pavestat->GetY1NDC()) size = 1-pavestat->GetY1NDC();
                           if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with stats object: size=%g \n", ((TPad *)obj)->GetNumber(), h->GetName(), size);
                          }
                       }
                     if (strcmp(objname,"labelx") == 0)
                       {if (size < h->GetLabelSize("X")) size = h->GetLabelSize("X");
                        if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with labelX: size=%g \n", ((TPad *)obj)->GetNumber(), h->GetName(), size);
                       }
                     if (strcmp(objname,"labely") == 0)
                       {if (size < h->GetLabelSize("Y")) size = h->GetLabelSize("Y");
                        if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with labelY: size=%g \n", ((TPad *)obj)->GetNumber(), h->GetName(), size);
                       }
                    }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      if (strcmp(objname,"legend") == 0) 
                        {leg = (TLegend*)(((TPad *)obj)->FindObject("DyLegendString"));
                         if (leg)
                           {if (size < leg->GetTextSize()) size = leg->GetTextSize();
                            if (Dy->debug > 0) printf("RA_GetSize> Found in %d a DyLegendString: size=%g \n", ((TPad *)obj2)->GetNumber(), size);
                           }
                        }

                      if (strcmp(objname,"timestring") == 0) 
                        {timepave = (TPaveText*)(((TPad *)obj)->FindObject("DyTimeString"));
                         if (timepave)
                           {if (size < timepave->GetTextSize()) size = timepave->GetTextSize();
                            if (Dy->debug > 0) printf("RA_GetSize> Found in %d a DyTimeString: size=%g \n", ((TPad *)obj2)->GetNumber(), size);
                           }
                        }

                      if (strcmp(objname,"title") == 0) 
                        {timepave = (TPaveText*)(((TPad *)obj)->FindObject("title"));
                         if (timepave)
                           {if ( size < timepave->GetTextSize()) size = timepave->GetTextSize();
                            if (Dy->debug > 0) printf("RA_GetSize> Found in %d a title string: size=%g \n", ((TPad *)obj2)->GetNumber(), size);
                           }
                       }

                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());

                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                            {TH1* h = (TH1 *)obj3;
                             if (strcmp(objname,"stat") == 0)
                               {pavestat= (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                                if (pavestat)
                                  {if (size < 1-pavestat->GetY1NDC()) size = 1-pavestat->GetY1NDC();
                                   if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with stats object: size=%g \n", ((TPad *)obj2)->GetNumber(), h->GetName(), size);
                                  }
                               }
                            if (strcmp(objname,"labelx") == 0)
                               {if (size < h->GetLabelSize("X")) size = h->GetLabelSize("X");
                                if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with labelX: size=%g \n", ((TPad *)obj2)->GetNumber(), h->GetName(), size);
                               }
                            if (strcmp(objname,"labely") == 0)
                               {if (size < h->GetLabelSize("Y")) size = h->GetLabelSize("Y");
                                if (Dy->debug > 0) printf("RA_GetSize> Found in pad %d a plot %s with labelY: size=%g \n", ((TPad *)obj2)->GetNumber(), h->GetName(), size);
                               }
                            }
                         }
                     }
                 }
            }
        }
    }

 return(size);
}


//_____________________________________________________________________________
void RA_ZoomAllStat (double zoomvalue)
{
// Larger or smaller StatBox on all the plots

 int padmodified, padmodified2;
 double x1, y1;
 TPaveStats* pavestat;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                      TH1* h = (TH1 *)obj2;

                      pavestat= (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                      if (pavestat)
                         {x1    = pavestat->GetX1NDC();
                          y1    = pavestat->GetY1NDC();
                          printf("RA_ZoomAllStat> Pad %d : Lower x/y of legend box goes from %g/%g to %g \n", ((TPad *)obj)->GetNumber(), x1, y1, 1-zoomvalue);
                          y1  = 1 - zoomvalue;
                          x1  = 1 - zoomvalue;
                          if (x1 < 0.30) x1 = 0.3;
                          if (x1 > 0.95) x1 = 0.95;
                          if (y1 < 0.30) y1 = 0.3;
                          if (y1 > 0.95) y1 = 0.95;
                          pavestat->SetX1NDC(x1);
                          pavestat->SetY1NDC(y1); 
                          padmodified = 1;
                         }
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              TH1* h = (TH1 *)obj3;
                              pavestat = (TPaveStats *)((h->GetListOfFunctions())->FindObject("stats"));
                              if (pavestat)
                                 {x1    = pavestat->GetX1NDC();
                                  y1    = pavestat->GetY1NDC();
                                  printf("RA_ZoomAllStat> Pad %d : Lower x/y of legend box goes from %g/%g to %g \n", ((TPad *)obj)->GetNumber(), x1, y1, 1-zoomvalue);
                                  y1  = 1 - zoomvalue;
                                  x1  = 1 - zoomvalue;
                                  if (x1 < 0.30) x1 = 0.3;
                                  if (x1 > 0.95) x1 = 0.95;
                                  if (y1 < 0.30) y1 = 0.3;
                                  if (y1 > 0.95) y1 = 0.95;
                                  pavestat->SetX1NDC(x1);
                                  pavestat->SetY1NDC(y1);
                                  padmodified2 = 1;
                                 }
                             }
                         }
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                    }
                }
             if (padmodified > 0) {((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllLegends (double zoomvalue)
{
// Larger or smaller legends for all the plots

 double lsize;
 TLegend* leg;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             leg = (TLegend*)(((TPad *)obj)->FindObject("DyLegendString"));
             if (leg)
                {lsize = leg->GetTextSize();
                 printf("RA_ZoomAllLegends> Pad %d : Text size of legend box goes from %g to %g \n", ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                 lsize = zoomvalue;
                 leg->SetTextSize(lsize);
                 if (lsize > 0.05) leg->SetX2NDC(0.98);
                 else              leg->SetX2NDC(0.9);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      leg = (TLegend*)(((TPad *)obj2)->FindObject("DyLegendString"));
                      if (leg)
                        {lsize = leg->GetTextSize();
                         printf("RA_ZoomAllLegends> Pad %d : Text size of legend box goes from %g to %g \n", ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                         lsize = zoomvalue;
                         leg->SetTextSize(lsize);
                         if (lsize > 0.05) leg->SetX2NDC(0.98);
                         else              leg->SetX2NDC(0.9);
                         ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                        }
                    }
                }
            }
        }
    }

}


//_____________________________________________________________________________
void RA_ZoomAllLabels (double zoomvalue, char *axisname)
{
// Larger Labels on axis of all the plots, and larger GPSorigin, title and unitsY

 int padmodified, padmodified2;
 double lsize, offset;
 char axis[64];

 if (axisname == NULL) strcpy(axis,"xaxis");
 else                  strcpy(axis, axisname);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                      TH1* h = (TH1 *)obj2;
                      if (strcmp(axis,"xaxis") == 0)
                         {lsize = h->GetLabelSize("X");
                          printf("RA_ZoomAllLabels> Pad %d : plot %s : X axis label size goes from %g to %g \n",
                                 ((TPad *)obj)->GetNumber(), h->GetName(), lsize, zoomvalue);
                          lsize = zoomvalue;
                          if (lsize < 0.04) lsize = 0.04;
                          h->SetLabelSize(lsize, "X");
                          offset = 1.3-9.85*lsize;
                          if (offset < 0.1) offset = 0.1;
                          h->SetTitleOffset(offset, "X");
                         }
                      else
                         {lsize = h->GetLabelSize("Y");
                          printf("RA_ZoomAllLabels> Pad %d : plot %s : %s label size goes from %g to %g \n",
                                 ((TPad *)obj)->GetNumber(), h->GetName(), axis, lsize, zoomvalue);
                          lsize = zoomvalue;
                          if (lsize < 0.04) lsize = 0.04;
                          h->SetLabelSize(lsize, "Y");
                          offset = 2.4-20.2*lsize;
                          if (offset < 0.4) offset = 0.4;
                          h->SetTitleOffset(offset, "Y");
                         }
                      padmodified = 1;  
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              TH1* h = (TH1 *)obj3;
                              if (strcmp(axis,"xaxis") == 0)
                                 {lsize = h->GetLabelSize("X");
                                  printf("RA_ZoomAllLabels> Pad %d : plot %s : %s label size goes from %g to %g \n",
                                         ((TPad *)obj)->GetNumber(), h->GetName(), axis, lsize, zoomvalue);
                                  lsize = zoomvalue;
                                  if (lsize < 0.04) lsize = 0.04;
                                  h->SetLabelSize(lsize, "X");
                                  offset = 1.3-9.85*lsize;
                                  if (offset < 0.1) offset = 0.1;
                                  h->SetTitleOffset(offset, "X");
                                 }
                              else
                                 {lsize = h->GetLabelSize("Y");
                                  printf("RA_ZoomAllLabels> Pad %d : plot %s : %s label size goes from %g to %g \n",
                                         ((TPad *)obj)->GetNumber(), h->GetName(), axis, lsize, zoomvalue);
                                  lsize = zoomvalue;
                                  if (lsize < 0.04) lsize = 0.04;
                                  h->SetLabelSize(lsize, "Y");
                                  offset = 2.4-20.2*lsize;
                                  if (offset < 0.4) offset = 0.4;
                                  h->SetTitleOffset(offset, "Y");
                                 }
                              padmodified2 = 1;
                             }
                         }
                      if (padmodified2 > 0){((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                     }
                 }
             if (padmodified > 0) {((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllTimeString (double zoomvalue)
{
// Larger or smaller all plots GPSorigin time string

 double lsize;
 TPaveText* timepave;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             timepave = (TPaveText*)(((TPad *)obj)->FindObject("DyTimeString"));
             if (timepave)
                {lsize = timepave->GetTextSize();
                 printf("RA_ZoomAllTimeString> Pad %d : time string size goes from %g to %g \n",
                         ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                 lsize = zoomvalue;
                 if (lsize < 0.03) lsize = 0.;
                 timepave->SetTextSize(lsize);
                 if (lsize > 0.07) timepave->SetTextAlign(12);
                 else              timepave->SetTextAlign(22);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("DyTimeString"));
                      if (timepave)
                         {lsize = timepave->GetTextSize();
                          printf("RA_ZoomAllTimeString> Pad %d : time string size goes from %g to %g \n",
                                 ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                          lsize = zoomvalue;
                          if (lsize < 0.03) lsize = 0.;
                          timepave->SetTextSize(lsize);
                          if (lsize > 0.07) timepave->SetTextAlign(12);
                          else              timepave->SetTextAlign(22);
                          ((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();
                         }
                     }
                 }
            }
        }
    }
}


//_____________________________________________________________________________
void RA_ZoomAllTitle (double zoomvalue)
{
// Larger or smaller all plots titles

 double lsize;
 TPaveText* timepave;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             timepave = (TPaveText*)(((TPad *)obj)->FindObject("title"));
             if (timepave)
                {lsize = timepave->GetTextSize();
                 printf("RA_ZoomAllTitle> Pad %d : plot title size goes from %g to %g \n",
                         ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                 lsize = zoomvalue;
                 if (lsize < 0.03) lsize = 0.;
                 timepave->SetTextSize(lsize);
                 ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
                }

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      timepave = (TPaveText*)(((TPad *)obj2)->FindObject("title"));
                      if (timepave)
                         {lsize = timepave->GetTextSize();
                          printf("RA_ZoomAllTitle> Pad %d : plot title size goes from %g to %g \n",
                                 ((TPad *)obj)->GetNumber(), lsize, zoomvalue);
                          lsize = zoomvalue;
                          if (lsize < 0.03) lsize = 0.;
                          timepave->SetTextSize(lsize);
                          ((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();
                         }
                     }
                 }
            }
        }
    }

}


//_____________________________________________________________________________
int RA_GetLog (int plot2d)
{
// Say if current pad plot axis is in log scale
  
 if (plot2d > 0)      {if (gPad->GetLogz() > 0) return(1);}
 else if (plot2d < 0) {if (gPad->GetLogx() > 0) return(1);}
 else                 {if (gPad->GetLogy() > 0) return(1);}
 return(0);
}


//_____________________________________________________________________________
void RA_LogLinAll (char *axisname)
{
// Put all plots in logx/linx or logy/liny

 char axis[64];
 int padmodified, padmodified2;
 static int logxFlip=0;
 static int logyFlip=0;

 if (axisname == NULL) strcpy(axis,"xaxis");
 else                  strcpy(axis, axisname);

 if (strcmp(axis,"xaxis") == 0) logxFlip = abs(logxFlip-1);
 else                           logyFlip = abs(logyFlip-1);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             while ((obj2 = next()))
                 {
                  if (obj2->InheritsFrom(TH1::Class()))
                     {
                       TH1 *h = (TH1 *)obj2;
                      padmodified = 1;
                      if (strcmp(axis,"xaxis") == 0)
                         {
/*
                          if (logxFlip == 0 || (logxFlip == 1 &&
                                                h->GetXaxis()->GetBinLowEdge(h->GetXaxis()->GetFirst()) > 0 &&
                                                strstr(h->GetTitle(),"TIME") == NULL)) ((TPad *)obj)->SetLogx(logxFlip);
*/
                          if (logxFlip == 0 || (logxFlip == 1 &&
                                                strstr(h->GetTitle(),"TIME") == NULL)) ((TPad *)obj)->SetLogx(logxFlip);
                         }
                      else
                         {if (logyFlip == 0 || (logyFlip == 1 && h->GetMinimum() > 0)) ((TPad *)obj)->SetLogy(logyFlip);}
                      break;
                     }

                  if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;

                      while ((obj3 = next()))
                        {
                         if (obj3->InheritsFrom(TH1::Class()))
                            {
                             padmodified2 = 1;
                             if (strcmp(axis,"xaxis") == 0) ((TPad *)obj)->SetLogx(logxFlip);
                             else                           ((TPad *)obj)->SetLogy(logyFlip);
                             break;
                            }
                        }

                     if (padmodified2 > 0)
                        {
                         ((TPad *)obj2)->Modified();
                         ((TPad *)obj2)->Update();
                        }
                    }

                }

             if (padmodified > 0)
                {((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            }
        }
    }

}


//______________________________________________________________________________
void RA_HSetLineWidth(char *name, int linewidth)
{
 if (linewidth <= 0) return;

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h && h->InheritsFrom(VDyPlot::Class())) h->SetLineWidth(linewidth);
}

//______________________________________________________________________________
void RA_HSetColor(char *name, int color)
{
// Set histogram line color (1=black, 2=red, 3=green, 4=blue, 5=yellow, 6=pink)

 int num;
 char h2name[256];

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h) h->SetLineColor((Color_t)color);

 sscanf(name+strlen(DYPLOT_PREFIX),"%d",&num);
 sprintf(h2name,"%s%d", DYPLOT_PREFIX, num+1000);
 TH1* h2=(TH1*)gDirectory->Get(h2name);
 if (h2) h2->SetLineColor((Color_t)color);

}


//______________________________________________________________________________
void RA_HSetLineStyle(char *name, int style)
{
// Set histogram line style (1=solid, 2=dash, 3=dash-dot, 4=dot)

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h) h->SetLineStyle(style);
}


//______________________________________________________________________________
void RA_HSetStat(char *name, int flag)
{
// Put stat box on plot if requested 

 int num;
 char h2name[256];

 TH1* h=(TH1*)gDirectory->Get(name);
 if (h) h->SetStats(flag);

 sscanf(name+strlen(DYPLOT_PREFIX),"%d",&num);
 sprintf(h2name,"%s%d", DYPLOT_PREFIX, num+1000);
 TH1* h2=(TH1*)gDirectory->Get(h2name);
 if (h2) h2->SetStats(flag);

}


//______________________________________________________________________________
void RA_HSetLegend (char *name, char *text_orig, int superposed, int nsuperposed, double xorig, double yorig, double leg_size)
{
 TLegendEntry *le;
 TLegend* leg;
 char *p, *text;
 double yoffset, length;

// Add legend for superimposed plots in Pad

 TH1* h=(TH1*)gDirectory->Get(name);

/*
 printf("RA_HSetLegend: name=%s  text_orig=%s  superposed=%d  nsuperposed=%d  h=%p  gPad=%p \n",
        name, text_orig, superposed, nsuperposed, h, gPad);
*/
 if (!h)         return;
 if (!gPad)      return;
 if (!text_orig) return;

 /*--- get legend if it already exists ---*/

 leg = (TLegend*)gPad->FindObject("DyLegendString");

 /*--- Delete it if plots no more superposed ---*/

 if (nsuperposed < 1)
   {if (leg) {delete leg; leg = NULL;}
    return;
   }

 /*--- Delete it (to recreate it) if the plot is the first of superposed plots ---*/

 if (superposed == OFF)
   {if (leg) {delete leg; leg = NULL;}}

 /*--- Does not show in the legend the type of plot for the superposed plots ---*/

 text = (char *)malloc(strlen(text_orig)+1);
 strcpy(text, text_orig);

 p = strstr(text,SEPTITLE);
// if (p != NULL && superposed == ON) *p = '\0';
 if (p != NULL) *p = '\0';
/* HOHO */

 /*--- if Legend does not yet exist, we create it ---*/
/**
printf("plotname=%s  leg=%x  xorig=%g yorig=%g legsize=%g\n",
        h->GetName(), leg, xorig, yorig, leg_size);
**/

 if (leg == NULL)
    {
     xorig = 0.27; length = 0.70;
//     yorig = 0.96-nsuperposed*0.05;
     yoffset = 0.;
     if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
        {if (strcmp(((VDyPlot*)h)->GetPlotType(),"1D") == 0) xorig = 0.10;
         if (strcmp(((VDyPlot*)h)->GetPlotType(),"TRFCT") == 0) yoffset = 0.05;
        }
//     leg = new TLegend(xorig, yorig-yoffset, xorig+length, 0.99);
     leg = new TLegend(xorig, yorig, xorig+length, DY_LEG_YMAX);
     leg->AddEntry(h,text,"l");
     leg->SetName("DyLegendString");
     if (Dy->nicefont == OFF) leg->SetTextFont(DY_FONT_TIMES);
     else                     leg->SetTextFont(DY_FONT_HELVETICA);
     leg->SetFillColor(0);
//     leg->SetFillColor(kWhite);
//     leg->SetFillStyle(4080);
     leg->SetTextSize(0);
//     leg->SetTextSize(leg_size);
     leg->Draw();
    }

 /*--- If legend already exists, we add this plot to it ---*/

 else
    {
     le = leg->AddEntry(h,text,"l");
/*
     if (nsuperposed < 4) le->SetTextColor(h->GetLineColor());
*/
    }

 if (text) free(text);

}


//______________________________________________________________________________
void RA_HSetWarningString (char *text)
{
// Add warning string under plot in the all Pad

 int color=1;
 TPaveText* warningpave;

 if (gPad && text)
    {if (gPad->GetBottomMargin() < 0.08+0.10) gPad->SetBottomMargin(0.08+0.10);}

 if (gPad)
    {
     if (text)
        {if (strcmp(text,DYMSG_NOFRAME) == 0)        color = DYCOL_NOFRAME;
         else if (strcmp(text,DYMSG_NOCHANNEL) == 0) color = DYCOL_NOCHANNEL;
         else if (strcmp(text,DYMSG_BADDATA) == 0)   color = DYCOL_BADDATA ;
         else                                        color = 1;
        }

     warningpave = (TPaveText*)gPad->FindObject("DyWarningString");

     if (!warningpave)
        {if (text)
            {warningpave = new TPaveText(0.05, 0.08, 0.40, 0.12,"bNDC");
             warningpave->SetName("DyWarningString");
             TString warningString = text;
             warningpave->AddText(warningString.Data());
             if (Dy->nicefont == OFF) warningpave->SetTextFont(DY_FONT_TIMES);
             else                     warningpave->SetTextFont(DY_FONT_HELVETICA);
             warningpave->SetTextColor(color);
             warningpave->SetFillColor(0);
             warningpave->Draw("NDC");
            }
        }

     else
        {
         if (text)
            {warningpave->Clear();
             TString warningString = text;
             warningpave->AddText(warningString.Data());
             warningpave->SetTextColor(color);
             warningpave->Draw("NDC");
            }
         else
            {delete warningpave; warningpave = NULL;}
        }
    }
}


//_____________________________________________________________________________
void RA_SetLabelFont (int fonttype, char *plotName)
{
 if (plotName != NULL)
    {
     TH1* h=(TH1*)gDirectory->Get(plotName);
     if (h)
        {h->SetLabelFont(fonttype,"X");
         h->SetLabelFont(fonttype,"Y");
        }
    }
 else
    {
     if (!gStyle) return;
     gStyle->SetTextFont(fonttype);
     gStyle->SetLabelFont(fonttype,"X");
     gStyle->SetLabelFont(fonttype,"Y");
     //   gStyle->SetTitleFont(fonttype);
    }
}


//_____________________________________________________________________________
void RA_SetLabelSize (double size, char *plotName)
{
printf("SetLabelSize: plotname=%s xt_offset=%g yt_offset=%g xt_size=%g yt_size=%g xl_size=%g yl_size=%g \n",
        plotName, Dy->xt_offset, Dy->yt_offset, Dy->xt_size, Dy->yt_size, Dy->xl_size, Dy->yl_size);
 
 if (plotName != NULL)
    {
     TH1* h=(TH1*)gDirectory->Get(plotName);
     if (h)
        {h->SetLabelSize(size,"X");
         h->SetLabelSize(size,"Y");
         h->GetXaxis()->SetTitleOffset(Dy->xt_offset);
         h->GetXaxis()->SetTitleSize(Dy->xt_size);
         h->GetYaxis()->SetTitleOffset(Dy->yt_offset);
         h->GetYaxis()->SetTitleSize(Dy->yt_size);
         h->SetLabelSize(Dy->xl_size,"X");
         h->SetLabelSize(Dy->yl_size,"Y");
        }
    }
 else
    {
     if (!gStyle) return;
     gStyle->SetLabelSize(size,"X");
     gStyle->SetLabelSize(size,"Y");
     gStyle->SetTitleOffset(Dy->xt_offset);
     gStyle->SetTitleSize(Dy->xt_size);
     gStyle->SetTitleOffset(Dy->yt_offset);
     gStyle->SetTitleSize(Dy->yt_size);
     gStyle->SetLabelSize(Dy->xl_size,"X");
     gStyle->SetLabelSize(Dy->yl_size,"Y");
    }
}


//_____________________________________________________________________________
void RA_HSetAllSize ()
{
 if (!gStyle) return;

printf("SetAllSize: xt_offset=%g yt_offset=%g xt_size=%g yt_size=%g xl_size=%g yl_size=%g \n",
        Dy->xt_offset, Dy->yt_offset, Dy->xt_size, Dy->yt_size, Dy->xl_size, Dy->yl_size);
 
 gStyle->SetTitleOffset(Dy->xt_offset);
 gStyle->SetTitleSize(Dy->xt_size);
 gStyle->SetTitleOffset(Dy->yt_offset);
 gStyle->SetTitleSize(Dy->yt_size);
 gStyle->SetLabelSize(Dy->xl_size,"X");
 gStyle->SetLabelSize(Dy->yl_size,"Y");
}


//_____________________________________________________________________________
void RA_SetGrayscale(int arg)
{
 if (!gDDCanvas) return;

 gDDCanvas->SetGrayscale(arg);
 gDDCanvas->Modified();
 gDDCanvas->Update();
}


//_____________________________________________________________________________
void RA_SetCanvasColor(int fullwhite)
{
 if (!gDDCanvas) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (fullwhite == 1) gDDCanvas->SetFillColor(0);
 else                gDDCanvas->SetFillColor(19);

 gDDCanvas->Modified();
 gDDCanvas->Update();

}


//_____________________________________________________________________________
void RA_InitROOT()
{
// Initialize the ROOT environment
   
 DyDebugPrint(__FUNCTION__,0);

 if (Dy->BgRun == ON) gROOT->SetBatch();
// if (Dy->BgRun == ON) gApplication->MakeBatch();

/**
 gEnv->SetValue("Unix.*.Root.UseTTFonts","false");
**/
 new TApplication("App", 0, 0);

 if (Dy->nicefont == OFF)
    {
     gStyle->SetTextFont(DY_FONT_TIMES);
     gStyle->SetTitleFont(DY_FONT_TIMES);
     gStyle->SetLabelFont(DY_FONT_TIMES,"X");
     gStyle->SetLabelFont(DY_FONT_TIMES,"Y");
    }
 else 
    {
     gStyle->SetTextFont(DY_FONT_HELVETICA);
     gStyle->SetTitleFont(DY_FONT_HELVETICA);
     gStyle->SetLabelFont(DY_FONT_HELVETICA,"X");
     gStyle->SetLabelFont(DY_FONT_HELVETICA,"Y");
    }

 gStyle->SetTextSize(0.05);
 gStyle->SetTitleBorderSize(0);
 gStyle->SetTitleSize(0.05);
 gStyle->SetLabelSize(0.05,"X");
 gStyle->SetLabelSize(0.05,"Y");
 gStyle->SetTitleW(0.8);
 gStyle->SetTitleH(0.07);
 //gStyle->SetTitleY(0.99);
 //gStyle->SetTitleX(0.1);
 gStyle->SetTitleStyle(0);
 gStyle->SetStatY(0.90);
 gStyle->SetOptStat(1);
 gStyle->SetNdivisions(505,"X");
 gStyle->SetNdivisions(505,"Y");
 gStyle->SetPadLeftMargin(0.15);
 gStyle->SetPadTopMargin(0.10);
// gStyle->SetPadRightMargin(0.15);
// gStyle->SetPadBottomMargin(0.18);
// gROOT->ForceStyle();
// gROOT->SetStyle("Plain");

 gStyle->SetCanvasBorderMode(0);
 gStyle->SetCanvasColor(19);
 gStyle->SetPadBorderMode(0);
 gStyle->SetPadColor(0);
 gStyle->SetFrameBorderMode(0);
// gStyle->SetFillColor(19);
// gStyle->SetFrameFillColor(19);
 gStyle->SetGridColor(14);

 RA_SetColorPalette(0);
 Dy->colorpalette = 0;

// TStorage::EnableStatistics();
 gSystem->ResetSignal(kSigBus);
 gSystem->ResetSignal(kSigSegmentationViolation);
 gSystem->ResetSignal(kSigSystem);
 gSystem->ResetSignal(kSigPipe);
 gSystem->ResetSignal(kSigIllegalInstruction);
 gSystem->ResetSignal(kSigQuit);
 gSystem->ResetSignal(kSigInterrupt);
 gSystem->ResetSignal(kSigWindowChanged);
 gSystem->ResetSignal(kSigAlarm);
 gSystem->ResetSignal(kSigChild);
 gSystem->ResetSignal(kSigUrgent);
 gSystem->ResetSignal(kSigFloatingException);
 gSystem->ResetSignal(kSigTermination);
 gSystem->ResetSignal(kSigUser1);
 gSystem->ResetSignal(kSigUser2);
      
 gDyTmpMacro = RA_CreateTempFileName(".C");


// Customize the Axis menu if possible

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,3,4)

 TClass* claxis = gROOT->GetClass("TAxis");
 claxis->MakeCustomMenuList();
   
 TList* laxmenu = claxis->GetMenuList();

// Remove the access to class information

 TClassMenuItem* mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(16);
 delete mi;
 mi = (TClassMenuItem*)laxmenu->At(1);
 delete mi;

// Change Unzoom position in the menu

 TClassMenuItem* miUnzoom = (TClassMenuItem*)laxmenu->At(12);
 laxmenu->Remove(miUnzoom);
 laxmenu->AddFirst(miUnzoom);

// Add the other specific commands in the menu

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL Z","URUnZoomAllZ",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoomY","URUnZoomY",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL Y","URUnZoomAllY",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Scale ALL Y as last","URScaleAllYAsLast",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Scale Y as last","URScaleYAsLast",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom Y axis as last","URZoomYAsLast",0,"TObject*",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom ALL Y as last","URZoomAllYAsLast",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoomX","URUnZoomX",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"UnZoom ALL X","URUnZoomAllX",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom X axis as last","URZoomXAsLast",0,"TObject*",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom ALL X as last","URZoomAllXAsLast",0,"",0,0));

 laxmenu->AddFirst(new TClassMenuItem(TClassMenuItem::kPopupSeparator,claxis));

 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All StatBox","URZoomAllStat",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All Legends","URZoomAllLegends",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Zoom All Labels","URZoomAllLabels",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Log/Lin All","URLogLinAll",0,"",0,0));
 laxmenu->AddFirst(new TClassMenuItem(
       TClassMenuItem::kPopupUserFunction,claxis,"Update Time/Freq","URUpdateTimeFreq",0,"",0,0));
// laxmenu->AddFirst(new TClassMenuItem(
//       TClassMenuItem::kPopupUserFunction,claxis,"Draw All as this one","URDrawOptionsAllAsThis",0,"",0,0));

#endif
   
 gDDCanvasString = "";

 /*--- Execute the logon macro as in Root ---*/
/**
 RA_ExecLogon();
**/

/*--- Prevent the use of french locale (which introduce 0, instead of . for decimals) ---*/
 setlocale(LC_ALL,"C");

}


//_____________________________________________________________________________
void RA_ExecLogon()
{
// Copy of TRint's "ExecLogon" method (could not use it since it is private)
//
// Execute logon macro's. There are three levels of logon macros that
// will be executed: the system logon etc/system.rootlogon.C, the global
// user logon ~/.rootlogon.C and the local ./.rootlogon.C. For backward
// compatibility also the logon macro as specified by the Rint.Logon
// environment setting, by default ./rootlogon.C, will be executed.
// No logon macros will be executed when the system is started with
// the -n option.

 DyDebugPrint(__FUNCTION__,0);

 if (gApplication->NoLogOpt()) return;

 TString name = ".rootlogon.C";
 TString sname = "system";
 sname += name;

#ifdef ROOTETCDIR
 char *s = gSystem->ConcatFileName(ROOTETCDIR, sname);
#else
 TString etc = gRootDir;
#ifdef WIN32
 etc += "\\etc";
#else
 etc += "/etc";
#endif
 char *s = gSystem->ConcatFileName(etc, sname);
#endif
 if (!gSystem->AccessPathName(s, kReadPermission)) gApplication->ProcessFile(s);
 delete [] s;

 s = gSystem->ConcatFileName(gSystem->HomeDirectory(), name);
 if (!gSystem->AccessPathName(s, kReadPermission)) gApplication->ProcessFile(s);
 delete [] s;

 // avoid executing ~/.rootlogon.C twice
 if (strcmp(gSystem->HomeDirectory(), gSystem->WorkingDirectory()))
    {if (!gSystem->AccessPathName(name, kReadPermission)) gApplication->ProcessFile(name);}

 // execute also the logon macro specified by "Rint.Logon"
 const char *logon = gEnv->GetValue("Rint.Logon", (char*)0);
 if (logon)
    {char *mac = gSystem->Which(TROOT::GetMacroPath(), logon, kReadPermission);
     if (mac) gApplication->ProcessFile(logon);
     delete [] mac;
    }
}


//_____________________________________________________________________________
int RA_SetColorPalette (int choice)
{
// Set color palette
   
 DyDebugPrint(__FUNCTION__,0);

 if (choice == 0)
   {
    gStyle->SetPalette(1);   /*--- Standard rainbow palette ---*/
   }

 else if (choice == 1)  /*--- rainbow palette more contrasted ---*/
   {int ncolors = 128;
    const Int_t NRGBs = 5;
    Double_t Stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
    Double_t Green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
    Double_t Blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 2)  /*--- rainbow palette more contrasted ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 7;
    Double_t Stops[NRGBs] = { 0.00, 0.20, 0.35, 0.50, 0.65, 0.80, 1.00 };
    Double_t Red[NRGBs]   = { 0.20, 0.00, 0.00, 0.00, 0.97, 0.97, 0.30 };
    Double_t Green[NRGBs] = { 0.00, 0.30, 0.97, 0.40, 0.97, 0.00, 0.00 };
    Double_t Blue[NRGBs]  = { 0.30, 0.97, 0.97, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 3)  /*--- violet-->green-->red-->dark red ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.20, 0.40, 0.97, 0.00, 0.00, 0.97, 0.97, 0.40 };
    Double_t Green[NRGBs] = { 0.00, 0.04, 0.00, 0.97, 0.97, 0.97, 0.00, 0.00 };
    Double_t Blue[NRGBs]  = { 0.20, 0.30, 0.50, 0.97, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 4)   /*--- black-->red-->yellow-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.20, 0.40, 0.60, 0.97, 0.97, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.20, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 5)   /*--- black-->green-->yellow ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.60, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.20, 0.30, 0.70, 0.80, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 6)  /*--- black-->yellow-->blue-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.60, 0.97, 0.00, 0.00, 0.70, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.60, 0.97, 0.20, 0.50, 0.70, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.10, 0.00, 0.00, 0.20, 0.50, 0.70, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 7)  /*--- black-->red-->green-->violet ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.30, 0.00, 0.00, 0.20, 0.40, 0.97, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.00, 0.97, 0.97, 0.00, 0.04, 0.97, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.00, 0.97, 0.00, 0.20, 0.30, 0.00, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else if (choice == 8)  /*--- black-->gray-->white ---*/
   {int ncolors = 64;
    const Int_t NRGBs = 8;
    Double_t Stops[NRGBs] = { 0.00, 0.07, 0.15, 0.35, 0.50, 0.70, 0.90, 1.00 };
    Double_t Red[NRGBs]   = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
    Double_t Green[NRGBs] = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
    Double_t Blue[NRGBs]  = { 0.00, 0.10, 0.20, 0.50, 0.60, 0.80, 0.90, 0.97 };
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,17,0)
    gStyle->CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#else
    TColor::CreateGradientColorTable(NRGBs, Stops, Red, Green, Blue, ncolors);
#endif
    gStyle->SetNumberContours(ncolors);
   }

 else
   {
    gStyle->SetPalette(1);   /*--- Standard rainbow palette ---*/
    choice = 0;
   }

 if (gDDCanvas)
   {gDDCanvas->Paint(); gDDCanvas->Update();}

// RA_MainCanvasUpdate();

 return (choice);
}


//_____________________________________________________________________________
void RA_InitDDWindow(char* title, int posx, int posy, int width, int height)
{
// Initialize global data display canvas
   
 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && (strcmp(title,"CLEAR") == 0))
    {delete gDDCanvas;
     gDDCanvas=0;
    }

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
       {if (obj->InheritsFrom(TPad::Class())) RA_PadClear((void*)obj);}
     gDDCanvas->SetTitle(title);
    }
 else
    {
     gDDCanvas = new TCanvas("gDDCanvas",title,posx,posy,width,height);
     gSystem->ProcessEvents();
     gDDCanvas->ToggleEventStatus();
     gDDCanvas->Resize();
    }

 gDDCanvas->cd();
 RA_SetCanvasColor(Dy->fullwhite);
}


//_____________________________________________________________________________
void RA_MainCanvasClear()
{
// Clear the global canvas
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas)) gDDCanvas->Clear();
}


//_____________________________________________________________________________
void RA_MainCanvasDelete()
{
 if (!gDDCanvas) return;

// Clear the global canvas
   if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas)) {
      delete gDDCanvas;
      gDDCanvas=0;
   }
}


//_____________________________________________________________________________
void RA_MainCanvasUpdate()
{
// Update the global canvas on the screen

 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     if (RA_SystemCheckSwap()) 
        {
         gDDCanvas->Paint();
         gDDCanvas->Modified();
         gDDCanvas->Update();
        } 
    }
}


//_____________________________________________________________________________
void RA_FileOpenCreate(char* name)
{
// Open a new file for creation
   
 TDirectory* tmpdir;
 tmpdir = gDirectory;
 new TFile(name,"RECREATE");
 tmpdir->cd();
}


//_____________________________________________________________________________
void RA_FileWrite(char* name)
{
// Write out all histograms to a file
   
 TH1F* idcurcopy;
 TH2F* idcurcopy2;
 TDirectory* tmpdir;
 char newname[256];

 DyDebugPrint(__FUNCTION__,0);

 tmpdir = gDirectory;

 TFile* f = (TFile*)(gROOT->GetListOfFiles()->FindObject(name));

 if (!f) {printf("RA_FileWrite> file %s not opened\n",name); return;}
 f->cd();

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());

     /*--- Loop on all the objects of the root Canvas ---*/
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());

             while ((obj2 = next()))   /*--- If Pad encountered, loop on all the objects of this pad ---*/
                 {
                  if (obj2->InheritsFrom(VDyPlot::Class()))
                     {
                      VDyPlot* h = (VDyPlot *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                         {printf("RA_FileWrite> Writing as TH1F the VDyPlot plot %s \n", h->GetTitle());}
                      idcurcopy = new TH1F(*((TH1F*)obj2));
                      sprintf(newname,"%s_%f", h->GetName(), h->GetTimeUTC());
printf("VDyPlot %s : title=%s  %f UTC  newname=%s\n", h->GetName(), h->GetTitle(), h->GetTimeUTC(), newname);
                      idcurcopy->SetName(newname);
                      idcurcopy->Write();
                      delete idcurcopy;
                     }
                  else if (obj2->InheritsFrom(VDyPlot2::Class()))
                     {
                      VDyPlot2* h = (VDyPlot2 *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                         {printf("RA_FileWrite> Writing as TH2F the VDyPlot2 plot %s \n", h->GetTitle());}
                      idcurcopy2 = new TH2F(*((TH2F*)obj2));
                      sprintf(newname,"%s_%f", h->GetName(), h->GetTimeUTC());
                      idcurcopy2->SetName(newname);
                      idcurcopy2->Write();
                      delete idcurcopy2;
                     }
                  else if (obj2->InheritsFrom(TH1::Class()))
                     {obj2->Write();
                      TH1* h = (TH1 *)obj2;
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                         {printf("RA_FileWrite> Writing TH1 plot %s \n", h->GetTitle());}
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))  /*--- If SubPad encountered, loop on all the objects of this Subpad ---*/
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              VDyPlot* h = (VDyPlot *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                 {printf("RA_FileWrite> Writing as TH1F the VDyPlot plot %s \n", h->GetTitle());}
                              idcurcopy = new TH1F(*((TH1F*)obj3));
                              idcurcopy->SetBinError(0,h->GetTimeUTC());
                              idcurcopy->Write();
                              delete idcurcopy;
                             }
                          else if (obj3->InheritsFrom(VDyPlot2::Class()))
                             {
                              VDyPlot2* h = (VDyPlot2 *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                 {printf("RA_FileWrite> Writing as TH2F the VDyPlot2 plot %s \n", h->GetTitle());}
                              idcurcopy2 = new TH2F(*((TH2F*)obj3));
                              idcurcopy2->SetBinError(0,0,h->GetTimeUTC());
                              idcurcopy2->Write();
                              delete idcurcopy2;
                             }
                          else if (obj3->InheritsFrom(TH1::Class()))
                             {obj3->Write();
                              TH1* h = (TH1 *)obj3;
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                 {printf("RA_FileWrite> Writing TH1 plot %s \n", h->GetTitle());}
                             }
                          else
                             {obj3->Write();
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                 {printf("RA_FileWrite> Writing object %s  class=%s\n", obj3->GetName(), obj3->ClassName());}
                             }
                         }
                     }
/**
                  else
                     {obj2->Write();
                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                         {printf("RA_FileWrite> Writing object %s  class=%s\n", obj2->GetName(), obj2->ClassName());}
                     }
**/
                 }
             }
         }
     } 

/***
 TIter next(tmpdir->GetList());
 while (idcur = next())
    {
     if (idcur->InheritsFrom(TH1F::Class()))
        {
         idcurcopy = new TH1F(*((TH1F*)idcur));
         idcurcopy->Write();
         delete idcurcopy;
        }
     else
        {idcur->Write();}

     if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
        {TH1 *h = (TH1*)idcur; 
         printf("RA_FileWrite> %s: %s=%s written in %s\n", idcur->IsA()->GetName(), h->GetName(), h->GetTitle(), name);
        }
    }
***/

 tmpdir->cd();
}


//_____________________________________________________________________________
void RA_FileClose(char* name)
{
// Write out all histograms to a file
   
 TFile* f = (TFile*)(gROOT->GetListOfFiles()->FindObject(name));
 if (!f) {printf("RA_FileClose> file %s was not open\n",name); return;}
 f->Close();
 delete f;
}


//_____________________________________________________________________________
int RA_SavePlots(char* filename)
{
// Save the plots into a file with format specified by the suffix

 char *str1, *str2, *str3, *str4;
 char name[1024], jpgFileName[256], epsFileName[256], rootFileName[256];
 int suffixType, length;
#if ROOT_VERSION_CODE < ROOT_VERSION(5,0,0)
 char execConvert[1024];
 int xsize, ysize;
#endif

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->outPicture) strcpy(name, Dy->outPicture);
 else                strcpy(name, filename);

 str1 = strstr(name,CONFIG_SUFFIX);
 str2 = strstr(name,".jpg");
 str3 = strstr(name,".eps");
 str4 = strstr(name,ROOT_SUFFIX);
 length = strlen(name);
 if      (str1 && length <= str1-name+4) {suffixType=1; *str1 = '\0';}
 else if (str2 && length <= str2-name+4) {suffixType=1; *str2 = '\0';}
 else if (str3 && length <= str3-name+4) {suffixType=2; *str3 = '\0';}
 else if (str4 && length <= str4-name+5) {suffixType=3; *str4 = '\0';}
 else                                     suffixType=1;

 printf("RA_SavePlots> filename=%s  name=%s  suffix=%d \n", filename, name, suffixType);

 if (suffixType == 1)
    {
     if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) {printf("RA_SavePlots> No plot to save\n"); return(0);}
#if ROOT_VERSION_CODE >= ROOT_VERSION(5,0,0)
     //======= Real speed-up, suggested by Valeriy Onuchin 16-05-07 ======
     //   uses the "SaveAs" feature of ROOT 5, but with almost no compression
     sprintf(jpgFileName, "%s.jpg", name);
     printf("RA_SavePlots> Creating %s \n", jpgFileName);
     TImageDump *imgdump = new TImageDump(jpgFileName); 
     gDDCanvas->Paint(); 
     TImage *img = imgdump->GetImage();
     img->SetImageQuality(img->kImgFast);
//     img->SetImageQuality(img->kImgGood);
//     img->SetImageQuality(img->kImgBest); 
     img->SetImageCompression(60); 
     imgdump->Close();
     imgdump->Delete();
#else
     sprintf(jpgFileName, "%s.jpg", name);
     sprintf(epsFileName, "%s.eps", name);
     printf("RA_SavePlots> Creating %s and converting it into jpg\n", epsFileName);
     xsize = gDDCanvas->GetWindowWidth();
     ysize = gDDCanvas->GetWindowHeight();
     gDDCanvas->Print(epsFileName);
     sprintf(execConvert," convert -cache 512 -geometry %dx%d -density 288x288 -colors 256 -sharpen 0.2 %s %s",xsize,ysize,epsFileName,jpgFileName);
     gSystem->Exec(execConvert);
     gSystem->Exec(Form("rm -f %s",epsFileName));
#endif
    }

 else if (suffixType == 2)
    {if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) {printf("RA_SavePlots> No plot to save\n"); return(0);}
     sprintf(epsFileName, "%s.eps", name);
     printf("RA_SavePlots> Creating %s \n", epsFileName);
     gDDCanvas->Print(epsFileName);
    }

 else if (suffixType == 3)
    {sprintf(rootFileName, "%s%s", name, ROOT_SUFFIX);
     printf("RA_SavePlots> Creating %s \n", rootFileName);
//     gDDCanvas->SaveAs(rootFileName);
     RA_FileOpenCreate(rootFileName);
     RA_FileWrite(rootFileName);
     RA_FileClose(rootFileName);
    }

 else
    {printf("RA_SavePlots> Suffix must be .root , .jpg or .ps\n"); return(0);}

 return(1);
}


//_____________________________________________________________________________
void RA_AllPadUpdate()
{
// Update each pad (should be useless since gDDCanvas->Paint() do the same in RA_MainCanvasUpdate) 

 DyDebugPrint(__FUNCTION__,0);

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             ((TPad *)obj)->Modified(); ((TPad *)obj)->Update();
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             while ((obj2 = next()))
                {if (obj2->InheritsFrom(TPad::Class())) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}}
            }
        }
    }
}


//_____________________________________________________________________________
int RA_DeleteUnusedPlot (int nvisible)
{
// List the plots in memory

 TObject *obj;
 int nall, nplot;

 nall  = 0;
 nplot = 0;

 TIter  next(gROOT->GetList());
 while ((obj = next()))
    {if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         if (strstr(h->GetName(),REF_SUFFIX) == NULL) nall++;
        }
    }

 if (nall > nvisible)
    {
     TIter  next(gROOT->GetList());
     while ((obj = next()) && nplot < nall-nvisible)
        {
         if (obj->InheritsFrom(TH1::Class()))
            {TH1 *h = (TH1*)obj;
             if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;
             delete h;
             nplot++;
            }
        }
    }

 return(nplot);
}


//_____________________________________________________________________________
void RA_SetLineWidth_AllPlots (int linewidth)
{
 TObject *obj;

 if (gROOT == NULL) return;
 if (linewidth <= 0) return;

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(VDyPlot::Class()))
        {VDyPlot *h = (VDyPlot*)obj;
         h->SetLineWidth(linewidth);
        }
    }
 if (gDDCanvas)
   {gDDCanvas->Paint(); gDDCanvas->Update();}
}


//_____________________________________________________________________________
int RA_CheckAllPlot (int showall)
{
// List the plots in memory

 TObject *obj;
 int nplot;

 TIter    next(gROOT->GetList());
 nplot = 0;
 while ((obj = next()))
    {
     if (obj->InheritsFrom(VDyPlot::Class()))
        {VDyPlot *h = (VDyPlot*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : VDyPlot  : %s = %s  in pad %d \n", nplot, h->GetName(), h->GetTitle(), h->numpad);
        }
     else if (obj->InheritsFrom(VDyPlot2::Class()))
        {VDyPlot2 *h = (VDyPlot2*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : VDyPlot2 : %s = %s  in pad %d \n", nplot, h->GetName(), h->GetTitle(), h->numpad);
        }
     else if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         nplot++;
         if (showall) printf("  Plot%02d : TH1      : %s = %s \n", nplot, h->GetName(), h->GetTitle());
        }
    }

 if (showall > 1) gROOT->GetList()->Print();

 return(nplot);
}


// obsolete function
//_____________________________________________________________________________
void RA_DelAllPlot()
{
// Delete all the plots in memory which are not reference plots

 TObject *obj;

 DyDebugPrint(__FUNCTION__,0);

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *h = (TH1*)obj;
         if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;
         printf("RA_DelAllPlot> deleted plot %s=%s \n", h->GetName(), h->GetTitle());
         delete h;
        }
    }

 return;
}


//_____________________________________________________________________________
int RA_RefPlotRemove (char *filename)
{
// Delete all the reference plots in memory, for a given filename

 int hexist, stillone;
 int numPicture, subpad, color;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str, *str2, filename2[LSTRING_MAX];
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (filename) basename = gSystem->BaseName(filename);
 else basename = NULL;

 hexist = 0;
 stillone = 0;
 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         str2 = (char *)strstr(href->GetName(),REF_SUFFIX);
         if (str2 == NULL) continue;
         color = href->GetLineColor();
         str = (char *)strstr(href->GetName(),SEPTITLE);
         if (filename && strstr(href->GetName(),basename) == NULL) {stillone = 1; continue;}
         if (filename == NULL)
            {strncpy(filename2, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
             filename2[str2-str-strlen(SEPTITLE)] = '\0';
             basename = gSystem->BaseName(filename2);
            }
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             RA_SetActivePad(numPicture, subpad);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
             if (gDDCanvas && gPad)
                {sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                 timepave = (TPaveText*)(gPad->FindObject(dummy));
                 if (timepave) delete timepave;
                }
            }
         printf("RA_RefPlotRemove> deleted plot %s=%s \n", href->GetName(), href->GetTitle());
         delete href;
         hexist = 1;
        }
    }

 if (hexist == 1) RA_MainCanvasUpdate();

 return(stillone);
}


//_____________________________________________________________________________
int RA_RefPlot2DRemove (char *filename)
{
// Delete all the 2-dimensional reference plots in memory for a given filename

 int hexist;
 int numPicture, subpad, color;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (filename == NULL) return(0);

 basename = gSystem->BaseName(filename);

 hexist = 0;
 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         if (strstr(href->GetName(),REF_SUFFIX) == NULL)           continue;
         if (filename && strstr(href->GetName(),basename) == NULL) continue;
         if (strstr(href->GetTitle(),"__2D") == NULL        &&
             strstr(href->GetTitle(),"__FFTTIME") == NULL   &&
             strstr(href->GetTitle(),"__1DTIME") == NULL    &&
             strstr(href->GetTitle(),"__COHETIME") == NULL  &&
             strstr(href->GetTitle(),"__TRTIME") == NULL    &&
             strstr(href->GetTitle(),"__RAWTIME") == NULL)  continue;
         if (strstr(href->GetTitle(),"__RAW") != NULL || strstr(href->GetTitle(),"__COHE") != NULL)
            {int nx=href->GetNbinsX();
             int ny=href->GetNbinsY();
             if (nx<=1 || ny<=1) continue;
            }

         printf("RA_RefPlot2DRemove> deleted plot %s=%s \n", href->GetName(), href->GetTitle());
         color = href->GetLineColor();
         str = (char *)strstr(href->GetName(),SEPTITLE);
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             RA_SetActivePad(numPicture, subpad);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));
             if (timepave) delete timepave;
            }
         delete href;
         hexist = 1;
        }
    }

 if (hexist == 1) RA_MainCanvasUpdate();

 return(hexist);
}


// Obsolete function
//_____________________________________________________________________________
void RA_RefPlotCheck()
{
// Check if reference plots are superposed plots and change their name accordingly

 TObject *obj;
 char hname[64], *str;
 VDyPlot *hv;

 gROOT->GetList()->Print();

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {TH1 *href = (TH1*)obj;
         if ((str = (char *)strstr(href->GetName(),REF_SUFFIX)) == NULL) continue;
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         hv = (VDyPlot *)gROOT->Get(hname);
         if (hv) printf("RA_RefPlotCheck> %s is superposed to %s=%s in pad %d \n", href->GetName(), hv->GetName(), hv->GetTitle(), hv->numpad);
         else    printf("RA_RefPlotCheck> %s=%s is superposed to nothing! \n", href->GetName(), href->GetTitle());
        }
    }
}


//_____________________________________________________________________________
void RA_RefPlotModify (char *fileName, int color)
{
// Modify the color of a set of reference plots

 int numPicture, subpad;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char hname[64], dummy[64], *str;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return;

 basename = gSystem->BaseName(fileName);

 TIter    next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {
         TH1 *href = (TH1*)obj;
         if (strstr(href->GetName(),REF_SUFFIX) == NULL || strstr(href->GetName(),basename) == NULL) continue;
         href->SetLineColor(color);

         str = (char *)strstr(href->GetName(),SEPTITLE);
         strncpy(hname, href->GetName(), str-href->GetName());
         hname[str-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);
         if (obj2)
            {if (obj2->InheritsFrom(VDyPlot2::Class()))
                {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }
             else
                {VDyPlot *hv = (VDyPlot *)obj2;
                 numPicture = hv->numpad;
                 subpad = hv->numsubpad;
                }

             RA_SetActivePad(numPicture, subpad);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
           
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));
             if (timepave) timepave->SetTextColor(color);
            }
        }
    }

 RA_MainCanvasUpdate();
 return;
}


//_____________________________________________________________________________
int RA_RefPlotRead (char *fileName, int color)
{
// Read from file all the reference plots and put them in memory

 TKey *key;
 char dummy[256];
 const char *basename;
 int nref;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return(0);

 TFile *fp = new TFile(fileName);

 if (fp->GetListOfKeys() == NULL) {printf("RA_RefPlotRead> file %s not found\n", fileName); return(0);}

 nref = 0;
 TIter next(fp->GetListOfKeys());
 while ((key = (TKey*)next()))
    {
     if (strncmp(key->GetClassName(),"TH",2) == 0)
        {TH1 *href = (TH1*)fp->Get(key->GetName());
         if (strstr(href->GetName(),REF_SUFFIX) != NULL) continue;
         href->SetDirectory(gROOT);
         if (color > 1) href->SetLineColor(color);
         else           href->SetLineColor(51);
         basename = gSystem->BaseName(fileName);
         sprintf(dummy, "%s%s%s%s", href->GetName(), SEPTITLE, basename, REF_SUFFIX);
         href->SetName(dummy);
         printf("RA_RefPlotRead> found in %s plot %s=%s  color=%d\n", fileName, href->GetName(), href->GetTitle(), href->GetLineColor());
         nref++;
        }
    }
 fp->Close();

 return(nref);
}


//_____________________________________________________________________________
void RA_RefPlotDraw()
{
// Draws all the reference plots present in memory

 int numPicture, subpad, color, ok;
 TObject *obj, *obj2;
 TPaveText *timepave;
 char filename[LSTRING_MAX], hname[64], dummy[64], dummy2[LSTRING_MAX];
 char *str2, *str, *strtime, *dytimestring;

 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (Dy->refplot_indelebile == OFF) return;

 DyDebugPrint(__FUNCTION__,0);
 DyDebugTimeNeeded(__FUNCTION__,"start");

 numPicture = 0;

 TIter  next(gROOT->GetList());
 while ((obj = next()))
    {
     if (obj->InheritsFrom(TH1::Class()))
        {
         TH1 *href = (TH1*)obj;
         str2 = (char *)strstr(href->GetName(),REF_SUFFIX);
         if (str2 == NULL) continue;

         /*--- If a 2-dimensional reference plot but not 2D, we do not draw it ---*/
         strcpy(dummy2, href->GetTitle());
         if (strstr(dummy2,"__RAW")!=0 || strstr(dummy2,"__FFTTIME")!=0 || strstr(dummy2,"__1DTIME")!=0 ||
             strstr(dummy2,"__RAWTIME")!=0 || strstr(dummy2,"__TRFCTTIME")!=0 || strstr(dummy2,"__COHETIME")!=0)
            {
             int nx=href->GetNbinsX();
             int ny=href->GetNbinsY();
             if (nx > 1 && ny > 1) continue;
            }

         /*--- Build the name of the plot associated to the reference plot href ---*/
         str = (char *)strstr(href->GetName(),SEPTITLE);
         strtime = (char *)strstr(href->GetName(),"_");
         strncpy(filename, str+strlen(SEPTITLE), str2-str-strlen(SEPTITLE));
         filename[str2-str-strlen(SEPTITLE)] = '\0';
         strncpy(hname, href->GetName(), strtime-href->GetName());
         hname[strtime-href->GetName()] = '\0';
         obj2 = (TObject *)gROOT->Get(hname);

         /*--- If found, build the timestring for href and superpose href and its timestring to the plot ---*/
         if (obj2)
            {
             if (obj2->InheritsFrom(VDyPlot2::Class()))
               {VDyPlot2 *hv = (VDyPlot2 *)obj2;
                numPicture = hv->numpad;
                subpad = hv->numsubpad;
               }
             else
               {VDyPlot *hv = (VDyPlot *)obj2;
                numPicture = hv->numpad;
                subpad = hv->numsubpad;
               }
             RA_SetActivePad(numPicture, subpad);
             if (subpad > 0) RA_SetActiveSubPad(subpad); 
           
             color = href->GetLineColor(); 
             sprintf(dummy,"DyRefString%s%s", SEPTITLE, filename);
             timepave = (TPaveText*)(gPad->FindObject(dummy));

             if (!timepave)
                {
                 dytimestring = (char *)malloc(256);
                 ok = RA_GetTimeStringFromUTC(dytimestring, href->GetName());
                 if (ok)
                    {
                     timepave = new TPaveText(0.02, 0.07, 0.98, 0.11,"bNDC");
                     timepave->SetName(dummy);
                     timepave->AddText(dytimestring);
                     timepave->SetTextFont(DY_FONT_HELVETICA);
                     timepave->SetTextColor(color);
                     timepave->SetFillColor(0);
                     timepave->Draw("NDC");
                    }
                 free(dytimestring);
                }

             if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                {printf("RA_RefPlotDraw> drawing plot %s=%s=pad%d  color=%d \n",
                         href->GetName(), href->GetTitle(), numPicture, href->GetLineColor());
                }
             if (obj2->InheritsFrom(VDyPlot2::Class())) href->Draw("same colz");
             else                                       href->Draw("same");
            }
         else
            {printf("RA_RefPlotDraw> Could not find in which pad to put %s = %s  hname=%s\n", href->GetName(), href->GetTitle(), hname);}
        }
    }
 DyDebugTimeNeeded(__FUNCTION__,"stop");
 return;
}


/*------------------------------------------------- RA_DrawRefPlotOfPad ---*/

void RA_DrawRefPlotOfPad (int numpad)
{
 DyDebugPrint(__FUNCTION__,0);

 RA_SetActivePad(numpad, 0);

 TObject *obj2;
 TIter    next(gPad->GetListOfPrimitives());

 while ((obj2 = next()))
    {
     if (obj2->InheritsFrom(TH1::Class()))
       {
        TH1* href = (TH1 *)obj2;
printf("++++++ Found plot %s \n", href->GetName());
        if (strstr(href->GetName(),REF_SUFFIX) == NULL) continue;
printf("++++++ It is a ref plot \n");
        href->Draw("same");
       }
    }

 return;
}


/*------------------------------------------------- RA_RefPlotGet ---*/
/* Superpose to the plots the ones from reference file fileName */

int RA_RefPlotGet (char *fileName, int color)
{
 int padmodified, padmodified2, nsuperposed;
 double value, valueref, diff, mindiff, maxdiff;
 int ok, hnum, nx, ny;
 char dummy[256], dummy2[256], *dytimestring;
 TPaveText* timepave;
 TPaveText* timepave1;
 const char *basename;

 DyDebugPrint(__FUNCTION__,0);

 if (fileName == NULL) return(0);

 TFile *fp = new TFile(fileName);

 if (fp->GetListOfKeys() == NULL) {printf("RA_RefPlotGet> file %s not found\n", fileName); return(0);}

 //if (color < 2) color = 51;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;

             nsuperposed = 0;
             while ((obj2 = next()))
                {
                 if (obj2->InheritsFrom(TH1::Class()))
                    {
                     ((TPad *)obj)->cd();

                     TH1* h = (TH1 *)obj2;
printf("In pad: searching ref plot for plot_title=%s plot_name=%s \n", h->GetTitle(), h->GetName());
                     if (strstr(h->GetName(),REF_SUFFIX) != NULL) continue;

                     /*--- Search for the reference plot which has same title ---*/
                     TH1 *href = NULL;
                     TIter next(fp->GetListOfKeys());
                     TKey *key;
                     while ((key=(TKey*)next()))
                       {
/**
                        printf("Looking a refplot for %s --> key: %s points to an object of class: %s \n",
                                h->GetTitle(), key->GetName(),key->GetClassName());
**/
                        if (strncmp(key->GetClassName(),"TH",2) == 0)
                          {
                           href = (TH1*)fp->Get(key->GetName());
                           printf("h->title=%s  href->title=%s\n", h->GetTitle(), href->GetTitle());
                           if (strcmp(h->GetTitle(),href->GetTitle()) != 0) continue;
                           href->SetDirectory(gROOT);
                           basename = gSystem->BaseName(fileName);
                           sprintf(dummy, "%s%s%s%s", href->GetName(), SEPTITLE, basename, REF_SUFFIX);
                           href->SetName(dummy);
                           href->SetLineColor(color);
                           break;
                          }
                        }

                     if (href == NULL)
                       {printf("RA_RefPlotGet> References plot does not exist for %s=%s\n", h->GetName(), h->GetTitle());
                        continue;
                       }

                     nsuperposed++;

                     dytimestring = (char *)malloc(256);
/**
                     ok = 0;
                     if (href->GetXaxis()->GetTimeFormat())
                        {ok = RA_GetTimeString(dytimestring, href->GetXaxis()->GetTimeFormat());}
                     else 
                        {ok = RA_GetTimeStringFromUTC(dytimestring, href->GetBinError(0));
                          printf("ok=%d \n", ok);}
**/

                        ok = RA_GetTimeStringFromUTC(dytimestring, href->GetName());
printf("plot %s : ok=%d  href->GetName = %s \n", href->GetTitle(), ok, href->GetName());
                     if (ok)
                        {
                         timepave = new TPaveText(0.02, 0.07, 0.98, 0.11,"bNDC");
                         sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                         timepave->SetName(dummy);
                         timepave->AddText(dytimestring);
                         timepave->SetTextFont(DY_FONT_HELVETICA);
                         timepave->SetTextColor(color);
                         timepave->SetFillColor(0);
                         timepave->Draw("NDC");
                        }
                     free(dytimestring);
                     strcpy(dummy2, href->GetTitle());

                     /*--- If 2-dimensional plots but not 2D, we compute the difference between plot and reference plot ---*/
                     /*--- Then, we need to redraw the plot and to redraw the time string ---*/
                     if (strstr(dummy2,"__RAW")!=0 || strstr(dummy2,"__FFTTIME")!=0 || strstr(dummy2,"__1DTIME")!=0 ||
                         strstr(dummy2,"__RAWTIME")!=0 || strstr(dummy2,"__TRFCTTIME")!=0 || strstr(dummy2,"__COHETIME")!=0)
                        {
                         nx=h->GetNbinsX();
                         ny=h->GetNbinsY();

                         if (nx>1 && ny>1)
                            {
                             gPad->SetLogz(0);
                             for (int i=0;i<nx;i++)
                                {for (int j=0;j<ny;j++)
                                    {value    = h->GetBinContent(i+1,j+1);
                                     valueref = href->GetBinContent(i+1,j+1);
                                     diff = value - valueref;
                                     if (i==0 && j==0) {mindiff = diff; maxdiff = diff;}
                                     if (diff < mindiff) mindiff = diff;
                                     if (diff > maxdiff) maxdiff = diff;
                                     h->SetBinContent(i+1,j+1,diff);
                                    }
                                }

                             h->Draw("colz");
                             h->SetMinimum(mindiff);
                             h->SetMaximum(maxdiff);


                             dytimestring = (char *)malloc(256);
                             ok = RA_GetTimeStringFromUTC(dytimestring, href->GetName());
                             if (ok)
                                {
                                 timepave1 = new TPaveText(0.02, 0.07, 0.98, 0.11,"bNDC");
                                 sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                 timepave1->SetName(dummy);
                                 timepave1->AddText(dytimestring);
                                 if (Dy->nicefont == OFF) timepave1->SetTextFont(DY_FONT_TIMES);
                                 else                     timepave1->SetTextFont(DY_FONT_HELVETICA);
                                 timepave1->SetTextColor(color);
                                 timepave1->SetFillColor(0);
                                 timepave1->Draw("NDC");
                                }
                             free(dytimestring);

                             if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                {printf("RA_RefPlotGet> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+1);}
                             padmodified++;
                             break;
                            }

                          else if (ny == 1)
                             {
                              href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }
                         }

                      else
                         {
                          strcpy(dummy2, href->GetTitle());

                          if (strstr(dummy2,"__2D")!=0)
                             {
                              href->Draw("colz same");
                             }

                          else if (strstr(dummy2,"__1D")!=0)
                             {
                              href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }

                          else
                             {
                              href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                             }
                         }

                      if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                         {printf("RA_RefPlotGet> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+1);}
                      padmodified++;
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      nsuperposed = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(VDyPlot::Class()))
                             {
                              ((TPad *)obj2)->cd();
                              VDyPlot* h = (VDyPlot *)obj3;
printf("In subpad: searching ref plot for plot_title=%s plot_name=%s \n", h->GetTitle(), h->GetName());


                              /*--- Search for the reference plot which has same title ---*/
                              TH1 *href = NULL;
                              TIter next(fp->GetListOfKeys());
                              TKey *key;
                              while ((key=(TKey*)next()))
                               {
                                if (strncmp(key->GetClassName(),"TH",2) == 0)
                                // if (key->InheritsFrom(TH1::Class()))
                                  {
                                   href = (TH1*)fp->Get(key->GetName());
                                   printf("h->title=%s  href->title=%s\n", h->GetTitle(), href->GetTitle());
                                   if (strcmp(h->GetTitle(),href->GetTitle()) != 0) continue;
                                   href->SetDirectory(gROOT);
                                   basename = gSystem->BaseName(fileName);
                                   sprintf(dummy, "%s%s%s%s", h->GetName(), SEPTITLE, basename, REF_SUFFIX);
                                   href->SetName(dummy);
                                   href->SetLineColor(color);
                                   break;
                                  }
                               }

                              if (href == NULL)
                                {printf("RA_RefPlotGet> In subpad : References plot does not exist for %s=%s\n", h->GetName(), h->GetTitle());
                                 continue;
                                }

                              strcpy(dummy2, href->GetName());
                              hnum = atoi(dummy2+1);

                              if (hnum>=1000) nsuperposed++;
                              if (hnum>=1000)
                                 {
                                  dytimestring = (char *)malloc(256);
                                  ok = RA_GetTimeStringFromUTC(dytimestring, href->GetName());
                                  if (ok)
                                     {
                                      timepave = new TPaveText(0.02, 0.07, 0.98, 0.12,"bNDC");
                                      sprintf(dummy,"DyRefString%s%s", SEPTITLE, basename);
                                      timepave->SetName(dummy);
                                      timepave->AddText(dytimestring);
                                      if (Dy->nicefont == OFF) timepave->SetTextFont(DY_FONT_TIMES);
                                      else                     timepave->SetTextFont(DY_FONT_HELVETICA);
                                      timepave->SetTextColor(color);
                                      timepave->SetFillColor(0);
                                      timepave->Draw("NDC");
                                     }
                                  free(dytimestring);
                                 }

                              href->Draw("h same");
                              if (h->GetMaximum() < href->GetMaximum()) h->SetMaximum(href->GetMaximum());
                              if (h->GetMinimum() > href->GetMinimum()) h->SetMinimum(href->GetMinimum());
                              if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
                                 {printf("RA_RefPlotGet> FOUND href=%s=%s  (%d)\n", href->GetName(), href->GetTitle(), padmodified+padmodified2+1);}
                              padmodified2++;
                             }
                         }
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                     }
                 }
             if (padmodified > 0){((TPad *)obj)->Modified(); ((TPad *)obj)->Update();}
            }
        }
    }  /* end of if FindObject(gDDCanvas) */

 else
    {padmodified = RA_RefPlotRead(fileName,color);}

 fp->Close();

 printf("RA_RefPlotGet> Current plots in gROOT are:\n");
 gROOT->ls();

 return(padmodified+padmodified2);
}


/*------------------------------------------------- RA_GetTimeString ---*/
/* Get the GPS time string from the plot time format */

int RA_GetTimeString (char *dytimestring, const char *timeformat)
{
 double gtime;

 gtime = DyTimeFormatToGps(timeformat);

 if (gtime == 0.) return(0);

 DyGpsToTimeString ((int)gtime, NULL, 0, 0, OFF, dytimestring);

 return(1);
}

/*------------------------------------------------- RA_GetTimeStringFromUTC ---*/
/* Get the GPS time string from the UTC time stored in plot name */

int RA_GetTimeStringFromUTC (char *dytimestring, const char *plotname)
{
 double gtime;
 double timeutc;
 char *str, *str2;
 char dummy[256];

 str = strstr((char *)plotname,"_");
 if (!str) return(0);

 str2 = strstr(str,SEPTITLE);
 if (!str2) return(0);
 if (str2 == str) return(0);

 strncpy(dummy, str+1,str2-str-1);
 timeutc = atof(dummy);

 if (Dy->debug > 0)
   {printf("RA_GetTimeStringFromUTC> timeutc=%f from plotname=%s  dummy=%s  str=%s  str2=%s \n",
           timeutc, plotname, dummy, str, str2);
   }

 if (timeutc == 0)
   {printf("RA_GetTimeStringFromUTC> timeutc=0 from plotname=%s \n", plotname);
    return(0);
   }

 gtime = DyUTC2GPS(timeutc);

 if (gtime == 0.) return(0);

 DyGpsToTimeString ((int)gtime, NULL, 0, 0, OFF, dytimestring);

 return(1);
}


/*------------------------------------------------- RA_minSlider ---*/
/* Change the Ymin or Zmin of the plot */

void RA_minSlider (char *plotname, double percentage, int numpad)
{
 double zmin, zmax0, zmax, newzmin;

 if (plotname == NULL) return;
 if (numpad <= 0) return;

// if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return;

 TH1* h=(TH1*)gDirectory->Get(plotname);
 if (h == NULL) {printf("Plot %s not found in gDirectory \n", plotname); return;}

 gDDCanvas->cd(numpad);

 zmax0 = h->GetMaximum();

 h->SetMinimum(-1111);
 h->SetMaximum(-1111);

 zmin = h->GetMinimum();
 zmax = h->GetMaximum();
printf("1... zmin=%g  zmax=%g  percentage=%g\n", zmin, zmax, percentage);
 if (gPad->GetLogz() == 0) newzmin = zmax - (zmax-zmin)*10 * percentage;
 else                      newzmin = zmax - (zmax-zmin)*1000 * percentage;
 
printf("2... newzmin=%g  zmax0=%g  \n", newzmin, zmax0);

 h->SetMinimum(newzmin);
 h->SetMaximum(zmax0);
 h->Paint();

 gPad->Modified();
 gPad->Update();

 return;
}


/*------------------------------------------------- RA_maxSlider ---*/
/* Change the Ymax or Zmax of the plot */

void RA_maxSlider (char *plotname, double percentage, int numpad)
{
 double zmin0, zmin, zmax, newzmax;

 if (plotname == NULL) return;
 if (numpad <= 0) return;

// if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return;

 TH1* h=(TH1*)gDirectory->Get(plotname);
 if (h == NULL) {printf("Plot %s not found in gDirectory \n", plotname); return;}

 zmin0 = h->GetMinimum();

 h->SetMinimum(-1111);
 h->SetMaximum(-1111);

 zmin = h->GetMinimum();
 zmax = h->GetMaximum();
printf("11... zmin=%g  zmax=%g  percentage=%g\n", zmin, zmax, percentage);
 if (gPad->GetLogz() == 0) newzmax = zmin + (zmax-zmin)*10 * percentage;
 else                      newzmax = zmin + (zmax-zmin)*1000 * percentage;
printf("22... newzmax=%g  zmin0=%g  \n", newzmax, zmin0);

 h->SetMaximum(newzmax);
 h->SetMinimum(zmin0);

 h->Paint();

 gDDCanvas->cd(numpad);
 gPad->Modified();
 gPad->Update();

 return;
}


/*------------------------------------------------- RA_XpSlider ---*/
/* Change the X position of the plot centering */

void RA_XpSlider (char *plotname, double percentage, int numpad)
{
 int nbintot, newbinmin, newbinmax;
 double xmin, xmax, xmin_vis, xmax_vis, newxmin_vis, newxmax_vis, delta;
 double xmin2, xmax2;

 if (plotname == NULL) return;
 if (numpad <= 0) return;

// if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return;

 TH1* h=(TH1*)gDirectory->Get(plotname);
 if (h == NULL) {printf("Plot %s not found in gDirectory \n", plotname); return;}

 // Get the xmin and xmax of the plot
 nbintot = h->GetNbinsX();
 xmin    = h->GetXaxis()->GetBinLowEdge(0);
 xmax    = h->GetXaxis()->GetBinUpEdge(nbintot);
 xmin2   = h->GetXaxis()->GetXmin();
 xmax2   = h->GetXaxis()->GetXmax();

 // Get the xmin and xmax of what is currently visible
 gDDCanvas->cd(numpad);
 xmin_vis = gPad->GetUxmin();                  
 xmax_vis = gPad->GetUxmax();                  
 delta = xmax_vis - xmin_vis;

 newxmin_vis = xmin + (xmax-delta-xmin)*percentage;
 newxmax_vis = newxmin_vis + delta;

 printf("plot=%s  xmin=%g xmax=%g  xmin2=%g xmax2=%g  xmin_vis=%g xmax_vis=%g  newxmin_vis=%g newxmax_vis=%g \n",
         plotname, xmin, xmax, xmin2, xmax2, xmin_vis, xmax_vis, newxmin_vis, newxmax_vis);
 
 newbinmin   = h->GetXaxis()->FindBin(newxmin_vis);
 newbinmax   = h->GetXaxis()->FindBin(newxmax_vis);
 h->GetXaxis()->SetRange(newbinmin, newbinmax);

 gPad->Modified();
 gPad->Update();

// DyXformSetScanStep(0.2*deltax_vis/deltax);

 return;
}

/*------------------------------------------------- RA_GetSelectedPlot ---*/

char *RA_GetSelectedPlot()
{
 char plotname[256];

// gPad = (gPad->GetCanvas())->GetSelectedPad();

 if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return(NULL);

 if (gLastModifiedPlot != NULL)
   {printf("lastModifiedPlot=%s \n", gLastModifiedPlot->GetName());
    strcpy(plotname,gLastModifiedPlot->GetName());
   }
 else if (gLastModifiedPlot2 != NULL)
   {printf("lastModifiedPlot2=%s \n", gLastModifiedPlot2->GetName());
    strcpy(plotname,gLastModifiedPlot2->GetName());
   }

/**
 TObject *obj2;
 TIter    next(gPad->GetListOfPrimitives());

 while ((obj2 = next()))
   {
    if (obj2->InheritsFrom(TH1::Class()))
      {
       TH1 *h   = (TH1 *)obj2;
       return(h->GetName());
      }
   }
**/

 return(plotname);
}


/*------------------------------------------------- RA_ScanPlots ---*/
/* Change to time axis area visualized in the plot */

void RA_ScanPlots (double percentage)
{
 int padmodified, padmodified2;
 double value, plotymin, plotymax;
 TH1* horig;
 char *str, mPlotType[64], gLastPlotType[64];
 int nbintot, binmin_vis, binmax_vis, binmin, binmax, newbinmin, newbinmax;
 double xmin, xmax, deltax, xmin2, xmax2, xmin_vis, xmax_vis, deltax_vis, newxmin_vis, newxmax_vis;

// gPad = (gPad->GetCanvas())->GetSelectedPad();

 if (gLastModifiedPlot == NULL && gLastModifiedPlot2 == NULL) return;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas))
    {
     TObject *obj;
     TIter    next(gDDCanvas->GetListOfPrimitives());
     while ((obj = next()))
        {
         if (obj->InheritsFrom(TPad::Class()))
            {
             TObject *obj2;
             TIter    next(((TPad *)obj)->GetListOfPrimitives());
             padmodified = 0;
             horig = NULL;

             while ((obj2 = next()))
                {
                 if (obj2->InheritsFrom(TH1::Class()))
                    {
                     ((TPad *)obj)->cd();

                       TH1 *h   = (TH1 *)obj2;
                       str = (char *)strstr(h->GetTitle(),SEPTITLE);
                       if (str) strcpy(mPlotType,&str[0]+2);
                       else     strcpy(mPlotType,"NULL");
                       str = (char *)strstr(Dy->gLastTitle,SEPTITLE);
                       if (str) strcpy(gLastPlotType,&str[0]+2);
                       else     strcpy(gLastPlotType,"NULL");

                     if (strcmp(mPlotType, gLastPlotType) == 0 ||
                         (strstr(mPlotType,"TIME") != NULL && strstr(gLastPlotType,"TIME") != NULL) ||
                         (strcmp(mPlotType,"FFT") == 0 && strcmp(gLastPlotType,"COHE") == 0) ||
                         (strcmp(mPlotType,"COHE") == 0 && strcmp(gLastPlotType,"FFT") == 0) ||
                         (strcmp(mPlotType,"FFT") == 0 && strcmp(gLastPlotType,"TRFCT") == 0) ||
                         (strcmp(mPlotType,"COHE") == 0 && strcmp(gLastPlotType,"TRFCT") == 0))
                        {
                         nbintot = h->GetNbinsX();
                         xmin    = h->GetXaxis()->GetBinLowEdge(0);
                         xmax    = h->GetXaxis()->GetBinUpEdge(nbintot);
                         deltax  = xmax - xmin;

                         xmin2   = h->GetXaxis()->GetXmin();
                         xmax2   = h->GetXaxis()->GetXmax();

                         xmin_vis = gPad->GetUxmin();                  
                         xmax_vis = gPad->GetUxmax();                  
                         if (((TPad *)obj)->GetLogx() > 0)
                            {xmin_vis = pow(10,xmin_vis);
                             xmax_vis = pow(10,xmax_vis);
                            }
                         deltax_vis  = xmax_vis - xmin_vis;
                         binmin_vis  = h->GetXaxis()->GetFirst();
                         binmax_vis  = h->GetXaxis()->GetLast();
                         binmin      = h->GetXaxis()->FindBin(xmin_vis);
                         binmax      = h->GetXaxis()->FindBin(xmax_vis);

                         DyXformSetScanStep(0.2*deltax_vis/deltax);

                         newxmin_vis = percentage * (xmax-xmin-deltax_vis) + xmin;
                         newxmax_vis = newxmin_vis + deltax_vis; 
                         newbinmin   = h->GetXaxis()->FindBin(newxmin_vis);
                         newbinmax   = h->GetXaxis()->FindBin(newxmax_vis);
//                         printf("RA_ScanPlots> plot=%s  percentage=%g  nbintot=%d  xmin=%g  xmax=%g  xmin2=%g  xmax2=%g  \n",
//                                 h->GetName(), percentage, nbintot, xmin, xmax, xmin2, xmax2);
//                         printf("RA_ScanPlots> plot=%s  xmin_vis=%g  xmax_vis=%g  binmin=%d (%d)  binmax=%d (%d)  ||  newxmin_vis=%g  newxmax_vis=%g  newbinmin=%d  newbinmax=%d \n",
//                                 h->GetName(), xmin_vis, xmax_vis, binmin, binmin_vis, binmax, binmax_vis, newxmin_vis, newxmax_vis, newbinmin, newbinmax);
                         if (xmin_vis == xmin2 && xmax_vis == xmax2) continue;
//                         if (newxmin_vis <= xmin2 || newxmax_vis >= xmax2) continue;
                         if (newbinmin < 0) newbinmin = 0;
                         if (newbinmax > nbintot-1) newbinmax = nbintot-1;

//                         binmin_vis  = h->GetXaxis()->GetFirst();
//                         binmax_vis  = h->GetXaxis()->GetLast();
//                         xmin_vis2   = h->GetXaxis()->GetBinUpEdge(binmin_vis);
//                         xmax_vis2   = h->GetXaxis()->GetBinUpEdge(binmax_vis);
//                         newbinmin2  = (int)(newxmin_vis * nbintot / deltax);
//                         newbinmax2  = (int)(newxmax_vis * nbintot / deltax);

                         if (strcmp(h->IsA()->GetName(),"VDyPlot2") != 0)
                            {if (horig == NULL)
                                {horig = h;
                                 plotymin = h->GetBinContent(newbinmin);
                                 plotymax = h->GetBinContent(newbinmin); 
                                }
                             for (int i=newbinmin;i<=newbinmax;i++)
                                {value = h->GetBinContent(i);
                                 if (value > plotymax) plotymax = value;
                                 if (value<= 0. && ((TPad *)obj)->GetLogy() > 0) continue;
                                 if (value < plotymin ) plotymin = value;
                                }
                            }

                         h->GetXaxis()->SetRange(newbinmin, newbinmax);
                         padmodified++;
                        }
                     }

                  else if (obj2->InheritsFrom(TPad::Class()))
                     {
                      TObject *obj3;
                      TIter    next(((TPad *)obj2)->GetListOfPrimitives());
                      padmodified2 = 0;
                      while ((obj3 = next()))
                         {
                          if (obj3->InheritsFrom(TH1::Class()))
                             {
                              ((TPad *)obj2)->cd();
                              TH1* h = (TH1 *)obj3;
                              str = (char *)strstr(h->GetTitle(),SEPTITLE);
                              if (str) strcpy(mPlotType,&str[0]+2);
                              else     strcpy(mPlotType,"NULL");
                              str = strstr(Dy->gLastTitle,SEPTITLE);
                              if (str) strcpy(gLastPlotType,&str[0]+2);
                              else     strcpy(gLastPlotType,"NULL");

                              if (strcmp(mPlotType, gLastPlotType) == 0 ||
                                  (strcmp(mPlotType,"TRFCT") == 0 && strcmp(gLastPlotType,"COHE") == 0) ||
                                  (strcmp(mPlotType,"TRFCT") == 0 && strcmp(gLastPlotType,"FFT") == 0))
                                 {
                                  nbintot = h->GetNbinsX();
                                  xmin    = h->GetXaxis()->GetBinLowEdge(0);
                                  xmax    = h->GetXaxis()->GetBinUpEdge(nbintot);
                                  deltax  = xmax - xmin;

                                  xmin2   = h->GetXaxis()->GetXmin();
                                  xmax2   = h->GetXaxis()->GetXmax();

                                  xmin_vis = gPad->GetUxmin();                  
                                  xmax_vis = gPad->GetUxmax();                  
                                  if (((TPad *)obj2)->GetLogx() > 0)
                                     {xmin_vis = pow(10,xmin_vis);
                                      xmax_vis = pow(10,xmax_vis);
                                     }
                                  deltax_vis = xmax_vis - xmin_vis;
                                  binmin_vis = h->GetXaxis()->GetFirst();
                                  binmax_vis = h->GetXaxis()->GetLast();
                                  binmin     = h->GetXaxis()->FindBin(xmin_vis);
                                  binmax     = h->GetXaxis()->FindBin(xmax_vis);

                                  DyXformSetScanStep(0.2*deltax_vis/deltax);

                                  newxmin_vis = percentage * (xmax-xmin-deltax_vis) + xmin;
                                  newxmax_vis = newxmin_vis + deltax_vis; 
                                  newbinmin   = h->GetXaxis()->FindBin(newxmin_vis);
                                  newbinmax   = h->GetXaxis()->FindBin(newxmax_vis);

                                  if (xmin_vis == xmin2 && xmax_vis == xmax2) continue;
                                  if (newbinmin < 0) newbinmin = 0;
                                  if (newbinmax > nbintot-1) newbinmax = nbintot-1;
                                  if (strcmp(h->IsA()->GetName(),"VDyPlot2") != 0)
                                     {if (horig == NULL)
                                         {horig = h;
                                          plotymin = h->GetBinContent(newbinmin);
                                          plotymax = h->GetBinContent(newbinmin); 
                                         }
                                      for (int i=newbinmin;i<=newbinmax;i++)
                                         {value = h->GetBinContent(i);
                                          if (value > plotymax) plotymax = value;
                                          if (value<= 0. && ((TPad *)obj2)->GetLogy() > 0) continue;
                                          if (value < plotymin ) plotymin = value;
                                         }
                                     }
//                               printf("%s new range set: (%d %d) --> (%d %d) \n", h->GetName(), binmin, binmax, newbinmin, newbinmax);
                                  h->GetXaxis()->SetRange(newbinmin, newbinmax);
                                  padmodified2++;
                                 }
                             }
                         }  /* end of while on obj3 */
                     if (padmodified2 > 0) {((TPad *)obj2)->Modified(); ((TPad *)obj2)->Update();}
                    } /* end of if on obj2 */
                } /* end of while on obj2 */

             if (padmodified > 0)
                {
/**
                 if (horig && strcmp(horig->IsA()->GetName(),"VDyPlot2") != 0)
                    {if (((TPad *)obj)->GetLogy() == 0) plotymin = plotymin - 0.2 * (plotymax - plotymin);
                     plotymax = plotymax + 0.2 * (plotymax - plotymin);
                     horig->SetMinimum(plotymin);
                     horig->SetMaximum(plotymax);
                    }
**/
                 ((TPad *)obj)->Modified();
                 ((TPad *)obj)->Update();
                }
            } /* end of if on obj */
        } /* end of while on obj */
    } /* end of if gDDCanvas */

 return;
}


//_____________________________________________________________________________
void RA_NtupleNew(char* name,char* title,int nvar,char* tags)
{
// 
}

//_____________________________________________________________________________
void RA_NtupleFill(char* name,float* x)
{
//
}

//_____________________________________________________________________________
void RA_SetStyle(int iplot)
{
// Sets the global style gStyle when needed and the pad options

 struct DyPlot *plot;

 plot = Plot[iplot];

   DyDebugPrint(__FUNCTION__,0);

   if (plot->type == PLOTTR)
     {
      RA_SetActivePad(plot->numpad, 1);
      gPad->SetLogx(plot->logx);
      gPad->SetLogy(plot->logy);
      gPad->SetGridx(plot->gridx);
      gPad->SetGridy(plot->gridy);
      RA_SetActivePad(plot->numpad, 2);
      gPad->SetLogx(plot->logx2);
      gPad->SetLogy(plot->logy2);
      gPad->SetGridx(plot->gridx2);
      gPad->SetGridy(plot->gridy2);
      return;
     }

   RA_SetActivePad(plot->numpad, plot->numsubpad);
   gPad->SetLogx(plot->logx);
   gPad->SetLogy(plot->logy);
   gPad->SetLogz(plot->logz);
//   gPad->SetLogz(1);
   gPad->SetGridx(plot->gridx);
   gPad->SetGridy(plot->gridy);

}


//_____________________________________________________________________________
void RA_HUpdateEnd(char *plotName, char *options)
{
 if (!gPad)     return;
 if (!plotName) return;

 DyDebugPrint(__FUNCTION__,0);

 TH1* h=(TH1*)gDirectory->Get(plotName);
 if (!h) return;

 TString optbis = options;

/**
 if (h->GetNbinsX()<20480)
    {optbis = "h" + optbis; h->SetDrawOption(optbis.Data());}
 else
    {optbis = "l9" + optbis; h->SetDrawOption(optbis.Data());}
**/

 optbis = "h9" + optbis; h->SetDrawOption(optbis.Data());

 h->SetLabelFont(DY_FONT_HELVETICA,"X");
 h->SetLabelFont(DY_FONT_HELVETICA,"Y");
/**
 if (Dy->InputType == INCM || Dy->InputType == INDD || Dy->InputType == INSHM)
    {h->SetLabelFont(DY_FONT_HELVETICA,"X");
     h->SetLabelFont(DY_FONT_HELVETICA,"Y");
    }
**/

}


//_____________________________________________________________________________
int RA_HUpdate (int numpad)
{
// Update canvas if the pad contains the plot which updates the fastest
 int update_done;

 DyDebugPrint(__FUNCTION__,0);

 update_done = 0;

 RA_SetActivePad(numpad, 0);

 if (gPad == gLastPad)
    {
     RA_RefPlotDraw();
     if (RA_SystemCheckSwap() && gDDCanvas) gDDCanvas->Update();
     update_done = 1;
    }

 return(update_done);
}


//____________________________________________________________________________
void RA_PadUpdate(int numpad)
{
 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0) printf("+++++ PADUPDATE>  numpad=%d\n", numpad);

 DyDebugTimeNeeded(__FUNCTION__,"start");

/* ICI */
 gDDCanvas->cd(numpad);
 RA_RefPlotDraw();
/* RA_DrawRefPlotOfPad (numpad); */

/*
 if (Dy->UpdateDone == ON) RA_MainCanvasUpdate();
*/

 gPad->Modified();
 gPad->Update();

 Dy->UpdateDone = ON;

 DyDebugTimeNeeded(__FUNCTION__,"stop");

 return;
}


//_____________________________________________________________________________
void RA_HDraw(char* name, char* options, int superposed)
{
// Draws the plot named "name" in the current pad

 TLegend* leg;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("  RA_HDraw> plot=%s options=%s superposed=%d\n", name, options, superposed);}

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h && Dy->debug > 0) {printf("RA_HDraw> no plot %s \n", name); return;}

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 TString optbis=options;
 TAxis* ax = h->GetXaxis();
 
 // If plots superposed, we do not show first plot title
 leg = (TLegend*)gPad->FindObject("DyLegendString");
 if (leg) h->SetBit(TH1::kNoTitle);
 else     h->SetBit(TH1::kNoTitle,0);

/** ICI
 if (gPad->GetLogy() == 1 && h->GetMinimum() > 0 && h->GetMaximum()/h->GetMinimum() < 10)
    h->GetYaxis()->SetMoreLogLabels(1);
 else
    h->GetYaxis()->SetMoreLogLabels(0);
**/

 // If VDyPlot, take care of number of points displayed to choose the line option
 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
    {if ((unsigned int)(ax->GetLast()-ax->GetFirst())<3*gPad->GetWw()) {optbis = "h" + optbis;}
     else                                                              {optbis = "l" + optbis;}
     ((VDyPlot*)h)->Draw(optbis.Data());
    }

// If VDyPlot2, take care of right margin for the color palette
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0)
    {gPad->SetRightMargin(0.15);
     ((VDyPlot2*)h)->Draw(options);
    }

// If TH2F, take care of right margin for the color palette
 else if (strcmp(h->IsA()->GetName(),"TH2F") == 0)
    {gPad->SetRightMargin(0.15);
     ((TH2F*)h)->Draw(options);
    }

 /*--- draw legend if it exists ---*/

 if (leg && superposed == OFF) leg->Draw();
 
oldsty->cd();

}


//_____________________________________________________________________________
void RA_HSetXRange(char* name, float fmin, float fmax, float df)
{
// Set the X axis range of the 1D histogram named "name".
 int xmin, xmax;

 TH1* h = (TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetXRange> no plot %s\n",name); return;}

 if (df > 0)
    {xmin = (int)(fmin / df);
     xmax = (int)(fmax / df);
    }
 else
    {xmin = (int)fmin;
     xmax = (int)fmax;
    }

 TAxis* ax = h->GetXaxis();
 ax->SetRange(xmin, xmax);
}


//_____________________________________________________________________________
void RA_HSetYRange(char* name, float fmin, float fmax, float df)
{
// Set the Y axis range of the 2D histogram named "name".
 int ymin, ymax;

 TH1* h = (TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetYRange> no plot %s\n",name); return;}

 if (df > 0)
    {ymin = (int)(fmin / df);
     ymax = (int)(fmax / df);
    }
 else
    {ymin = (int)fmin;
     ymax = (int)fmax;
    }

 TAxis* ay = h->GetYaxis();
 ay->SetRange(ymin, ymax);
}


//_____________________________________________________________________________
void RA_HReset(char* name)
{
// Resets the histogram named "name"
  
 DyDebugPrint(__FUNCTION__,0);
 
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HReset> no plot %s\n",name); return;}

 if (!strcmp(h->IsA()->GetName(),"VDyPlot"))       ((VDyPlot*)h)->Reset();
 else if (!strcmp(h->IsA()->GetName(),"TH2F"))     ((TH2F*)h)->Reset();
 else if (!strcmp(h->IsA()->GetName(),"VDyPlot2")) ((VDyPlot2*)h)->Reset();
}


//_____________________________________________________________________________
void RA_H1Fill(char* name, float x , float weight)
{
// Fills the 1D histogram names "name"

 static int counter=0;

 VDyPlot* h=(VDyPlot*)gDirectory->Get(name);
 if (!h && counter == 0) {printf("RA_H1Fill> no plot %s\n",name); return;}
 counter++;
 if (counter > 100000) counter = 0;

// ICI counter really needed ?

 h->Fill(x,weight);
}


//_____________________________________________________________________________
void RA_H2Fill(char* name, float x, float y, float weight)
{
// Fills the 2D histogram names "name"

 static int counter=0;

 TH2F* h=(TH2F*)gDirectory->Get(name);
 if (!h && counter == 0) {printf("RA_H2Fill> no plot %s\n",name); return;}
 counter++;
 if (counter > 100000) counter = 0;

 h->Fill(x,y,weight);
}


//_____________________________________________________________________________
void RA_HDelete(char* name)
{
// Deletes the histogram named "name"

 int num;
 char h2name[256];

 DyDebugPrint(__FUNCTION__,0);

 if (!name) {gDirectory->Delete("*"); return;}

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) return;

 if (!strcmp(h->IsA()->GetName(),"VDyPlot"))       delete ((VDyPlot*)h);
 else if (!strcmp(h->IsA()->GetName(),"TH2F"))     delete ((TH2F*)h);
 else if (!strcmp(h->IsA()->GetName(),"VDyPlot2")) delete ((VDyPlot2*)h);

 sscanf(name+strlen(DYPLOT_PREFIX),"%d",&num);
 sprintf(h2name,"%s%d", DYPLOT_PREFIX, num+1000);
 TH1* h2=(TH1*)gDirectory->Get(h2name);
 if (!h2) return;

 if (!strcmp(h2->IsA()->GetName(),"VDyPlot"))       delete ((VDyPlot*)h2);
 else if (!strcmp(h2->IsA()->GetName(),"TH2F"))     delete ((TH2F*)h2);
 else if (!strcmp(h2->IsA()->GetName(),"VDyPlot2")) delete ((VDyPlot2*)h2);

}


// obsolete
//_____________________________________________________________________________
void RA_H1New(char* name,char* title,int nbins, float xmin, float xmax)
{
// Creates a new 1D histogram

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 VDyPlot* h = new VDyPlot(name,title,nbins,xmin,xmax);
 h->UnlinkTimePave();
 oldsty->cd();
}


//_____________________________________________________________________________
void RA_H1NewReuse(int pad, int subpad, char* name, char* title, int nbins, float xmin, float xmax)
{
// Reuses an already existing histogram in pad/subpad
// or if it is not good, creates a new 1D histogram

 char* oldname;

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 oldname = RA_PadContainsGoodHist (pad, subpad, 1, nbins, 0, name);

 if (!oldname)
    {
     VDyPlot *h0 = new VDyPlot(name,title,nbins,xmin,xmax);
     h0->numpad    = pad;
     h0->numsubpad = subpad;
     return;
    }

 else
    {
     VDyPlot* h = (VDyPlot*)gDirectory->Get(oldname);
     if (!h)
        {
         VDyPlot *h0 = new VDyPlot(name,title,nbins,xmin,xmax);
         h0->numpad    = pad;
         h0->numsubpad = subpad;
        }
     else
        {
         h->UnlinkTimePave();
         h->SetName(name);
         h->SetTitle(title);
         h->GetXaxis()->SetLimits(xmin,xmax);
         h->Reset();
         h->numpad    = pad;
         h->numsubpad = subpad;
        }
    }

 oldsty->cd();
}


//_____________________________________________________________________________
void RA_HSetMinimum(char* name, double ymin)
{
// Sets the minimum for histo "name"
   
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetMinimum> no plot %s\n",name); return;}

 h->SetMinimum(ymin);
}


//_____________________________________________________________________________
void RA_HSetMaximum(char* name, double ymax)
{
// Sets the maximum for histo "name"
   
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetMaximum> no plot %s\n",name); return;}

 h->SetMaximum(ymax);
}


//_____________________________________________________________________________
void RA_HSetTitle(char* name, char* title)
{
// Sets the title for histo "name"

 char *title2;

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTitle> no plot %s title=%s\n",name, title); return;}

 if (title == NULL) {h->SetBit(TH1::kNoTitle); return;}

 title2 = (char *)malloc(strlen(title)+1);
 strcpy(title2,title);
 char* p = strstr(title2,SEPTITLE);
// if (p != NULL) *p = '\0';
/* HOHO */

 h->SetTitle(title2);
}


//_____________________________________________________________________________
void RA_HSetUnitsX(char* name, char* units)
{
// Sets the units of horizontal axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetUnitsX> no plot %s\n",name); return;}

 h->GetXaxis()->SetTitle(units);
 h->GetXaxis()->SetTitleOffset(0.9);
}


//_____________________________________________________________________________
void RA_HSetUnitsY(char* name, char* units)
{
// Sets the units of vertical axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetUnitsY> no plot %s\n",name); return;}

 h->GetYaxis()->SetTitle(units);
 h->GetYaxis()->SetTitleOffset(1.63);
}


//_____________________________________________________________________________
void RA_HSetUnitsZ(char* name, char* units)
{
// Sets the units of vertical axis for histo "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTitle> no histogram %s\n",name); return;}

 h->GetZaxis()->SetTitle(units);
// h->GetZaxis()->RotateTitle(kTRUE);
// h->GetZaxis()->SetTitleOffset(1.63);
}


//_____________________________________________________________________________
void RA_HSetTimeString(char* name, char* timestring)
{
// Sets the time string displayed below the plot "name"

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeString> no plot %s\n",name); return;}

 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)       ((VDyPlot*)h)->SetTimeString(timestring);
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0) ((VDyPlot2*)h)->SetTimeString(timestring);
 else printf("RA_HSetTimeString> NO TIME STRING for plot %s=%s\n", h->GetName(), h->GetTitle());
}

//_____________________________________________________________________________
void RA_HSetTimeFlag(char* name, int flag)
{
// Sets the flag that allows drawing the time string

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeFlag> no plot %s\n",name); return;}

 if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)       ((VDyPlot*)h)->SetTimeFlag(flag); 
 else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0) ((VDyPlot2*)h)->SetTimeFlag(flag);
 else printf("RA_HSetTimeString> NO TIME FLAG for plot %s=%s\n", h->GetName(), h->GetTitle());
}

//_____________________________________________________________________________
void RA_HSetTimeOffset(char* name, double offset, int timeaxis)
{
// Sets the offset date for the time axis

 DyDebugPrint(__FUNCTION__,0);

 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_HSetTimeOffset> no plot %s\n",name); return;}

 if (offset > 0)
    {
     if (strcmp(h->IsA()->GetName(),"VDyPlot") == 0)
        {((VDyPlot*)h)->SetTimeUTC(offset);}  /* keep and use date later when zooming */
     else if (strcmp(h->IsA()->GetName(),"VDyPlot2") == 0)
        {((VDyPlot2*)h)->SetTimeUTC(offset);}  /* keep and use date later when zooming */
    }

 if (timeaxis == ON)
   {h->GetXaxis()->SetTimeDisplay(1);
    h->GetXaxis()->SetTimeOffset(offset,"gmt");
   }

}


//_____________________________________________________________________________
void RA_SetAxisNdiv(char* name, int ndiv)
{
// Sets the X axis ndivisions
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetAxisNdiv> no plot %s\n",name); return;}
 h->GetXaxis()->SetNdivisions(ndiv,"X");
}


//_____________________________________________________________________________
void RA_SetLimitsX(char* name, double xmin, double xmax)
{
// Sets the X axis limits (xmin and xmax) of a plot
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetLimitsX> no plot %s\n",name); return;}
 h->GetXaxis()->SetLimits(xmin,xmax);
}


//_____________________________________________________________________________
void RA_SetLimitsY(char* name, double ymin, double ymax)
{
// Sets the Y axis limits (ymin and ymax) of a plot
 TH1* h=(TH1*)gDirectory->Get(name);
 if (!h) {printf("RA_SetLimitsY> no plot %s\n",name); return;}
 h->GetYaxis()->SetLimits(ymin,ymax);
}


// obsolete
//_____________________________________________________________________________
void RA_H2New(char* name, char* title, int nbinx, float xmin, float xmax, int nbiny, float ymin, float ymax)
{
// Creates a new 2D histogram

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 VDyPlot2* h = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
// TH2F* h = new TH2F(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);

// Sets the kCanDelete bit so that any drawing
// replacing this plot will delete it
 h->SetBit(kCanDelete);

 oldsty->cd();
}


//_____________________________________________________________________________
void RA_H2NewReuse(int pad, int subpad, char* name, char* title, int nbinx, float xmin, float xmax, int nbiny, float ymin, float ymax)
{
// Reuses an already existing histogram in pad/subpad
// or if it is not good, creates a new 2D histogram

 char* oldname;

 DyDebugPrint(__FUNCTION__,0);

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();

 oldname = RA_PadContainsGoodHist(pad, subpad, 2, nbinx, nbiny, name);

 if (!oldname)
    {
     VDyPlot2* h0 = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
     h0->SetBit(kCanDelete);   // Sets the kCanDelete bit so that any drawing replacing this plot will delete it
     h0->numpad    = pad;
     h0->numsubpad = subpad;
     return;
    }
 else
    {
     VDyPlot2* h = (VDyPlot2*)gDirectory->Get(oldname);
     if (!h)
        {
         VDyPlot2 *h0 = new VDyPlot2(name,title,nbinx,xmin,xmax,nbiny,ymin,ymax);
         h0->numpad    = pad;
         h0->numsubpad = subpad;
        }
     else
        {
         h->UnlinkTimePave();
         h->SetName(name);
         h->SetTitle(title);
         h->GetXaxis()->SetLimits(xmin,xmax);
         h->GetYaxis()->SetLimits(ymin,ymax);
         h->Reset();
         h->numpad    = pad;
         h->numsubpad = subpad;
        }
    }

 oldsty->cd();
}


// ICI : Not used anymore
//_____________________________________________________________________________
void RA_H1SetBinColor(char* name, int bin, int colorValue)
{
// Set a vertical colored line at bin of a 1D plot
 if (bin < 0) return;
 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);
 if (!h) {printf("RA_H1SetBinColor> no plot %s\n",name); return;}
 h->SetIndicator(bin,colorValue);
// h->SetBinContent(bin,0);
}


//_____________________________________________________________________________
void RA_H1Pack(char* name, double* buff)
{
// Fills a 1D histogram with the contents of a buffer
 double lastValue;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);

 if (!h) {printf("RA_H1Pack> no plot %s\n",name); return;}

 /*--- 2=red=baddata, 3=green=nochannel, 4=blue=noframe, 17=gray ---*/

 int nx = h->GetNbinsX();

 lastValue = buff[0];
 if (lastValue >= DY_NOCHANNEL) lastValue = 0.;
 if (lastValue >= DY_NOFRAME)   lastValue = 0.;
 if (lastValue >= DY_BADDATA)   lastValue = 0.;
 if (lastValue < -1e+30)        lastValue = 0.;

 for (int i=0;i<nx;i++)
    {
     /*--- If channel is absent, set green color and put last correct value in the bin ---*/
     if (buff[i] >= DY_NOCHANNEL)
        {h->SetIndicator(i+1,DYCOL_NOCHANNEL);
         h->SetBinContent(i+1,lastValue);
        }

     /*--- If frame is absent, set blue color and put last correct value in the bin ---*/
     else if (buff[i] >= DY_NOFRAME)
        {h->SetIndicator(i+1,DYCOL_NOFRAME);
         h->SetBinContent(i+1,lastValue);
        }

     /*--- If data is corrupted, set red color and put last correct value in the bin ---*/
     else if (buff[i] >= DY_BADDATA)
        {h->SetIndicator(i+1,DYCOL_BADDATA);
         h->SetBinContent(i+1,lastValue);
        }

     /*--- If data is abnormal, set grey color and put last correct value in the bin ---*/
     else if (buff[i] < -1e+30)
        {h->SetIndicator(i+1,DYCOL_NOCHANNEL);
         h->SetBinContent(i+1,lastValue);
        }

     /*--- If the sampling is ok, put its value in the bin and keep it as last correct value ---*/
     else
        {h->SetIndicator(i+1,0);
         h->SetBinContent(i+1,buff[i]);
         lastValue = buff[i];
        }
    }
}


//_____________________________________________________________________________
void RA_H2Pack(char* name, double* buff)
{
// Fills a 2D histogram with the contents of a buffer
   
 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);

 if (!h) {printf("RA_H2Pack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();
 int ny=h->GetNbinsY();

 for (int i=0;i<nx;i++)
    {for (int j=0;j<ny;j++) h->SetBinContent(i+1,j+1,buff[i+nx*j]);}
}


//_____________________________________________________________________________
void RA_H2PackBits(char* name, double* buff)
{
// Fills a 2D histogram with the bits of the contents of a buffer

 int bitValue, valueInt;

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);
 
 if (!h) {printf("RA_H2PackBits> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();

 for (int i=0;i<nx;i++)
    {
     valueInt = (int)buff[i];
     for (int j=0;j<32;j++)
        {bitValue = valueInt&(1<<j);
         if (bitValue != 0) h->SetBinContent(i+1,j+1,1.);
         else               h->SetBinContent(i+1,j+1,0.3);
        }
    }
}


//_____________________________________________________________________________
void RA_H1Unpack(char* name, double* buff)
{
// Fills a buffer with the contents of a 1D histo
   
 DyDebugPrint(__FUNCTION__,0);

 VDyPlot* h = (VDyPlot*)gDirectory->Get(name);

 if (!h) {printf("RA_H1Unpack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();

 for (int i=0;i<nx;i++) buff[i] = h->GetBinContent(i+1);
}


//_____________________________________________________________________________
void RA_H2Unpack(char* name, double* buff)
{
// Fills a buffer with the contents of a 2D histo

 DyDebugPrint(__FUNCTION__,0);

 VDyPlot2* h = (VDyPlot2*)gDirectory->Get(name);

 if (!h) {printf("RA_H2Unpack> no plot %s\n",name); return;}

 int nx=h->GetNbinsX();
 int ny=h->GetNbinsY();

 for (int i=0;i<nx;i++)
    {for (int j=0;j<ny;j++) buff[i+nx*j] = h->GetBinContent(i+1,j+1);}
}


//_____________________________________________________________________________
void RA_SetActivePad(int numpad, int numsubpad)
{
// Sets the active pad as the number numpad (and numsubpad if any)

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("  SetActivePad> called with numpad=%d numsubpad=%d\n", numpad, numsubpad);}
   
 if (numpad <= 0) return;

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && gPad)
    {gDDCanvas->cd(numpad);
     gPad->SetFillColor(0);
     if (gPad && numsubpad > 0)
       {gPad->cd(numsubpad);
        gPad->SetFillColor(0);
       }
    }
}


//_____________________________________________________________________________
void RA_SetActiveSubPad(int num)
{
// Sets the active pad as the number num

 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) && gPad)
    {gPad->cd(num);
     gPad->SetFillColor(0);
     if (((Dy->debugType)&(1<<DBG_VDYPLOT))) printf("SetActiveSubPad> numsubpad=%d gPad=%ld\n",num, (long)gPad);
    }
}


//_____________________________________________________________________________
void RA_PadDivide(int nx,int ny)
{
// Divides the current pad in nx.ny subpads

 DyDebugPrint(__FUNCTION__,0);

 if (!gPad->IsEditable()) return;

 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 if (gPad) gPad->Divide(nx,ny);
 oldsty->cd();
}


//_____________________________________________________________________________
void RA_PadDivideWithTitle(int nx,int ny)
{
// Divides the current pad in nx.ny subpads
// Special version of the TPad::Divide() method which leaves a given space
// at the top of the pad in order to allow writing a title or a global time

 DyDebugPrint(__FUNCTION__,0);

 Float_t xmargin=0.005;
 Float_t ymargin=0.005;
 Int_t color=0;
 Float_t topmargin=0.06;
   
 if (!gPad->IsEditable()) return;

 TPad *padsav = (TPad*)gPad;
 gPad->cd();
 if (nx <= 0) nx = 1;
 if (ny <= 0) ny = 1;
 Int_t ix,iy;
 Double_t x1,y1,x2,y2;
 Double_t dy = (1-topmargin)/Double_t(ny);
 Double_t dx = 1/Double_t(nx);
 TPad *pad;
 char *name = new char [strlen(padsav->GetName())+6];
 Int_t n = 0;
 if (color == 0) color = padsav->GetFillColor();

 for (iy=0;iy<ny;iy++)
    {y2 = 1 - topmargin - iy*dy - ymargin;
     y1 = y2 - dy + 2*ymargin;
     if (y1 < 0) y1 = 0;
     if (y1 > y2) continue;
     for (ix=0;ix<nx;ix++)
        {x1 = ix*dx + xmargin;
         x2 = x1 +dx -2*xmargin;
         if (x1 > x2) continue;
         n++;
         sprintf(name,"%s_%d",padsav->GetName(),n);
         pad = new TPad(name,name,x1,y1,x2,y2,color);
         pad->SetNumber(n);
         pad->Draw();
        }
    }

 delete [] name;
 padsav->Modified();
 if (padsav) padsav->cd();
}


//_____________________________________________________________________________
void RA_PadClear(void* pad)
{
// Clear a pad, leaving all subpads and histograms intact
// Commented because it seems that not needed anymore
// and because it created a problem for restarting and using mTimePave
// of VDyPlot.

/***
 TObject *obj;
 TList* listofprim=((TPad*)pad)->GetListOfPrimitives();
 TIter    next(listofprim);
 while ((obj = next()))
   {
    if (obj->InheritsFrom(TPad::Class()))
       {RA_PadClear((void*)obj);}
    else
       {if (!obj->InheritsFrom(VDyPlot::Class()) && !obj->InheritsFrom(VDyPlot2::Class()) &&
             !obj->InheritsFrom(TH2F::Class()) &&
             !obj->InheritsFrom(TFrame::Class()) &&
             !obj->InheritsFrom(TPaveText::Class()) ) {listofprim->Remove(obj); obj->Delete();}
       }
   }
***/

}


//_____________________________________________________________________________
void RA_PadCompleteClear(int ipad)
{
// Clear a pad
 
 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;
  
 TStyle* oldsty = gStyle;
 TStyle* sty = (TStyle*)(gROOT->GetListOfStyles()->FindObject("dyStyle"));
 if (sty) sty->cd();
 gDDCanvas->cd(ipad);
 gPad->Clear();
 oldsty->cd();
}


//_____________________________________________________________________________
int RA_PadIsDivided(int npads)
{
// Return 1 if the current pad is already divided with npads subpads
// if npads == -1, returns 1 if there is at least one subpad
   
 TObject* idcur;
 int npadinpad=0;
   
// loop on all objects in the canvas to see if there are subpads
 TList* l = gPad->GetListOfPrimitives();
 TIter next(l);
 while ((idcur = next())) {if (!strcmp(idcur->IsA()->GetName(),"TPad")) npadinpad++;}
 if (npads==npadinpad || (npads==-1&&npadinpad>0)) return 1;

 return 0;
}


//_____________________________________________________________________________
void RA_SetTestTrigPad(int numpad, int numsubpad, double interval)
{
// Tests and sets the current pad to be the one triggering the canvas screen update 
  
 if (numpad==-1) {gLowestInterval = 999999999.; return;}

 if (numpad == 0) return;

 if (gDDCanvas == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;

 if (interval<=gLowestInterval && interval > 0.)
    {
     if (((Dy->debugType)&(1<<DBG_VDYPLOT))) printf("RA_SetTestTrigPad> pad%d subpad%d accepted \n", numpad, numsubpad);
     TVirtualPad* oldpad=gPad;
     gDDCanvas->cd(numpad);
     if (numsubpad) gPad->cd(numsubpad);
     gLastPad = gPad;
     gLowestInterval = interval;
     gPad = oldpad;
    }

 return;
}


//_____________________________________________________________________________
char* RA_PadContainsGoodHist(int numpad, int numsubpad, int type, int nbinsx, int nbinsy, char *name)
{
// Tests the pad number and subpad number for containing a plot of type
// "type" (1=1D, 2=2D) and of number of bins "nbins".
// Returns the name of the plot if ok.
// Allow to reuse the plot when one wants to keep the style
   
 static char oldname[255];

 DyDebugPrint(__FUNCTION__,0);

 if (gDDCanvas == NULL) return(NULL);
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return(NULL);

 TVirtualPad* curpad = gPad;

// Go to the right pad or subpad
 gDDCanvas->cd(numpad);
 if (numsubpad)
    {if (!RA_PadIsDivided(-1)) {curpad->cd(); return NULL;}
     gPad->cd(numsubpad);
    }
  
// Loop on all objects contained in the pad
 TObject *obj;
 TIter    next(gPad->GetListOfPrimitives());
 while ((obj = next()))
   {
    // If 1Dim plot (VDyPlot) with right number of bins, return it's name
    if (obj->InheritsFrom(VDyPlot::Class()) && type==1)
       {
        VDyPlot* h=(VDyPlot*)obj;
        if (h->GetNbinsX()==nbinsx && strcmp(name,h->GetName())==0)
           {
            strcpy(oldname,h->GetName());
            curpad->cd();
            return oldname;
           }
       }

    // If 2Dim plot (VDyPlot2) with right number of bins, return it's name
    else if (obj->InheritsFrom(VDyPlot2::Class()) && type==2)
       {
        VDyPlot2* h=(VDyPlot2*)obj;
        if (h->GetNbinsX()==nbinsx && h->GetNbinsY()==nbinsy && strcmp(name,h->GetName())==0)
           {
            strcpy(oldname,h->GetName());
            curpad->cd();
            return oldname;
           }
       }

    // If 2Dim plot (but not VDyPlot2) with right number of bins, return it's name
    else if (obj->InheritsFrom(TH2F::Class()) && type==2)
       {
        TH2F* h=(TH2F*)obj;
        if (h->GetNbinsX()==nbinsx && h->GetNbinsY()==nbinsy && strcmp(name,h->GetName())==0)
           {
            strcpy(oldname,h->GetName());
            curpad->cd();
            return oldname;
           }
       }

   }

 curpad->cd();
 return NULL;
}


//_____________________________________________________________________________
double RA_HGetXmin(char* name)
{
// Returns the lower limit of the X axis for a histogram named "name"
 TAxis* ax=((TH1*)gDirectory->Get(name))->GetXaxis();
 return ax->GetBinLowEdge(ax->GetFirst());
}


//_____________________________________________________________________________
double RA_HGetXmax(char* name)
{
// Returns the upper limit of the X axis for a histogram named "name"
 TAxis* ax=((TH1*)gDirectory->Get(name))->GetXaxis();
 return ax->GetBinUpEdge(ax->GetLast());
}


//_____________________________________________________________________________
double RA_HGetYmin(char* name)
{
// Returns the lower limit of the Y axis for a histogram named "name"
 TAxis* ay=((TH1*)gDirectory->Get(name))->GetYaxis();
 return ay->GetBinLowEdge(ay->GetFirst());
}


//_____________________________________________________________________________
double RA_HGetYmax(char* name)
{
// Returns the upper limit of the Y axis for a histogram named "name"
 TAxis* ay=((TH1*)gDirectory->Get(name))->GetYaxis();
 return ay->GetBinUpEdge(ay->GetLast());
}


//_____________________________________________________________________________
void RA_GlobalTitleDraw ()
{
// Draws the global title in the Pavelabel named "DyGlobalTitle"

 if (gDDCanvas == NULL || gPad == NULL) return;
 if (gROOT->GetListOfCanvases()->FindObject(gDDCanvas) == 0) return;
   
 TPad* padsave = (TPad*) gPad;
   
 static char stringTime[30];
 time_t utc;
 TPaveText* globalTitle;
 TString titlestring = "dataDisplay ";
 
 titlestring += DYVERSION;
 titlestring += " : started by ";

 gDDCanvas->cd();
 globalTitle = (TPaveText*)gDDCanvas->FindObject("DyGlobalTitle");
   
 if (!globalTitle)
    {globalTitle = new TPaveText(0.05,0.955,0.95,0.985,"");
     globalTitle->SetName("DyGlobalTitle");
     globalTitle->AddText(titlestring);
    }

 TText* firstline = globalTitle->GetLine(0);

 if (!strncmp(titlestring.Data(),firstline->GetTitle(), strlen(titlestring.Data())))
    {
     globalTitle->Clear();
     utc = time(NULL); 
     strftime(stringTime,30,"%b %e %Y %H:%M:%S UTC ",gmtime(&utc));
     titlestring += getenv("USER");
     titlestring += " on ";
     titlestring += stringTime;

     if (Dy->titleCmd != NULL)
        {titlestring += " : ";
         if (strncmp(Dy->titleCmd,"::",2) == 0) titlestring += Dy->titleCmd+2;
         else                                   titlestring += Dy->titleCmd;
        }

     globalTitle->AddText(titlestring);
     if (Dy->nicefont == OFF) globalTitle->SetTextFont(DY_FONT_TIMES);
     else                     globalTitle->SetTextFont(DY_FONT_HELVETICA);
//     globalTitle->SetTextSize(0.035);
     globalTitle->Draw();
    }
  
 if (padsave) padsave->cd();
}

   
//_____________________________________________________________________________
const char* RA_CanvasMacroString()
{
// Fills the global variable gDDCanvasString with the instructions
// created by the SaveAs command
// if no global canvas present, return 0

 DyDebugPrint(__FUNCTION__,0);

 gDDCanvasString = "";  // clear the canvas macro string
 if (!gDDCanvas) return gDDCanvasString.Data();

 gDDCanvas->SaveAs(gDyTmpMacro);
 ifstream incan(gDyTmpMacro,ios::in);
 TString linecan;

// Reload the saved canvas macro, with cleaning of some parts
 linecan.ReadLine(incan);
 while(linecan.Length()!=0)
   {
    if (!linecan.Contains("pt->") &&
        !linecan.Contains("TPaveText(") &&
        !linecan.Contains("->SetBinContent(") &&
        !linecan.Contains(REF_SUFFIX) ) gDDCanvasString += linecan + "\n";

    if (linecan.Contains("gDDCanvas_") && linecan.Contains("->Draw();"))
       {
        linecan.ReplaceAll("gDDCanvas_","");
        linecan.ReplaceAll("->Draw();","");
        linecan.ReplaceAll(" ","");
        TString stmpnum = linecan;
        stmpnum.ReplaceAll("_"," ");
        int f,g;
        f=0;
        g=0;
        char stmpnum1[5];
        sscanf(stmpnum.Data(),"%d %d",&f,&g);
        if (g) sprintf(stmpnum1,"%d",g);
        else sprintf(stmpnum1,"%d",f);
        gDDCanvasString += "gDDCanvas_"+linecan+"->SetNumber("+stmpnum1+");\n";
       }

    linecan.ReadLine(incan);
   }
   
 incan.close();
 remove(gDyTmpMacro);
 return gDDCanvasString.Data();
}


//_____________________________________________________________________________
void RA_InterpreterProcessLine(char * cline)
{
// Executes the commands contained in the string cline
// Processing is made by the interpreter

 fstream tmpout(gDyTmpMacro,ios::out);
 tmpout << cline;
 tmpout.close();
 gInterpreter->ExecuteMacro(gDyTmpMacro);
 remove(gDyTmpMacro);

 if (gPad != 0) {gDDCanvas = gPad->GetCanvas(); gDDCanvas->cd();}
}


//_____________________________________________________________________________
char* RA_CreateTempFileName(const char* suffix)
{
// Creates a unique temporary file name, including directory
   
 static char s[100];
// char* tmps = tempnam(0,"Dy_");
 char* tmps = (char *)malloc(10*sizeof(char));
 strcpy(tmps,"Dy_XXXXXX");
 mkstemp(tmps);
 remove(tmps);
 strcpy(s,tmps);
 strcat(s,suffix);
 free(tmps);
 return s;
}


//_____________________________________________________________________________
void RA_MacroCreateCanvas(char * cline)
{
// Executes the commands contained in the string cline
// Processing is made by the interpreter
 if (gDDCanvas) return;
 RA_InterpreterProcessLine(cline);
}


//_____________________________________________________________________________
void  RA_ObjectTablePrint() {}
// void  RA_ObjectTablePrint() {gObjectTable->Print(); TStorage::PrintStatistics();}


//_____________________________________________________________________________
void  RA_ProcessRootEvents(){gSystem->ProcessEvents();}
// void  RA_ProcessRootEvents(){}


//_____________________________________________________________________________
int RA_SystemCheckSwap()
{
// test of the swap space. If there is still enough swap, returns 1, else 0
// Only works for OSF1
// This routine is needed because of a "memory leak" in the Xwindows managing
// of ROOT. Hope will be obsoleted in a near future

#ifdef R__ALPHA
 struct tbl_swapinfo swbuf;
 int swap_pages=0,swap_free=0,i;

 i=0;
 while(table(TBL_SWAPINFO,i,&swbuf,1,sizeof(struct tbl_swapinfo))>0)
    {swap_pages += swbuf.size;
     swap_free  += swbuf.free;
     i++;
    }

 if (swap_free < 0.1*swap_pages)
    {printf("RA_SystemCheckSwap> Warning : swap space left %d%%, cannot continue to display plots\n", (100*swap_free)/swap_pages);
     return 0;
    }
#endif
 return 1;
}

