#ifndef UROOTACCESS_DEFINED
#define UROOTACCESS_DEFINED

/*--- Wrappers allowing access to ROOT plotting functions ---*/

#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

void RA_Main();       /*--- Run the root event loop with timer to force check_forms ---*/
void RA_ExecLogon();  /*--- Execute the logon macro, defined as in ROOT (in .rootrc) ---*/

double RA_GetSize (char *objname);
void RA_ZoomAllStat (double zoomstep);
void RA_ZoomAllLegends (double zoomstep);
void RA_ZoomAllLabels (double zoomstep, char *axisname);
void RA_ZoomAllTimeString (double zoomstep);
void RA_ZoomAllTitle (double zoomstep);
void RA_LogLinAll (char *axisname);
int  RA_GetLog (int plot2d);

void RA_HSetAllSize();
void RA_HSetLineWidth(char *name, int linewidth);
void RA_HSetColor(char *name, int color);
void RA_HSetLineStyle(char *name, int style);
void RA_HSetStat (char *name, int flag);
int  RA_HUpdate(int numpad);
void RA_HUpdateEnd(char *plotName, char *options);
void RA_PadUpdate(int padnumber);
void RA_DrawRefPlotOfPad (int numpad);
void RA_HSetLegend (char *name, char *text, int nsame, int nsuperposed, double xorig, double yorig, double leg_size);
void RA_HSetTitle2 (char *text, int color);
void RA_HSetTitle2Clear();
void RA_HSetWarningString (char *text);
void RA_HNewSetTimeString (char *text);

/*--- Choose font 20 (faster) when running and font 22 when stopped ---*/
/*--- Choose size 0.1 (bigger) when plotting subpads and size 0.05 otherwise ---*/

void RA_SetLabelFont (int fonttype, char *plotName);
void RA_SetLabelSize (double size, char *plotName);
void RA_SetCanvasColor (int fullwhite);
void RA_SetGrayscale(int arg);

/*--- Initialisation of the global canvas ---*/
void RA_InitROOT();
int RA_SetColorPalette (int choice);

/*--- Initialisation of the global canvas ---*/
void RA_InitDDWindow(char* title, int posx, int posy, int width, int height);
void RA_SetTestTrigPad(int numpad, int numsubpad, double interval);
void RA_MainCanvasClear();
void RA_MainCanvasDelete();
void RA_MainCanvasUpdate();
int  RA_MainCanvasCheck();

/*--- Open file, Write out all histograms, Close file ---*/
void RA_FileOpenCreate(char* name);
void RA_FileWrite(char* name);
void RA_FileClose(char* name);

/*--- Save canvas in root, eps or jpeg format ---*/
int RA_SavePlots(char* name);

/*--- Get, delete or superpose to current plots the reference plots read from a root file ---*/
void RA_AllPadUpdate();
int  RA_DeleteUnusedPlot (int nvisible);
void RA_SetLineWidth_AllPlots (int linewidth);
int  RA_CheckAllPlot (int showall);
void RA_DelAllPlot();
int  RA_RefPlotRemove (char *filename);
int  RA_RefPlot2DRemove (char *filename);
void RA_RefPlotCheck();
void RA_RefPlotModify (char *fileName, int color);
int  RA_RefPlotRead (char *fileName, int color);
int  RA_RefPlotGet (char *fileName, int color);
void RA_RefPlotDraw ();

/*--- Get GPS time string from the plot time format or from a UTC time stored in plot name ---*/
int RA_GetTimeString (char *dytimestring, const char *timeformat);
int RA_GetTimeStringFromUTC (char *dytimestring, const char *plotname);

/*--- Change the Ymin/Zmax (or Zmin/Zmax) of the plot ---*/
void RA_minSlider (char *plotname, double percentage, int numpad);
void RA_maxSlider (char *plotname, double percentage, int numpad);

/*--- Change center of the plot in X axis ---*/
void RA_XpSlider (char *plotname, double percentage, int numpad);

char *RA_GetSelectedPlot();

/*--- Scan plots along time axis ---*/
void RA_ScanPlots (double percentage);

/*--- C_HBOOKN ---*/
void RA_NtupleNew(char* name,char* title,int nvar,char* tags);

/*--- C_HFN ---*/
void RA_NtupleFill(char* name,float* x);

/*--- C_HPLOPT ---*/
void RA_SetStyle(int iplot);

/*--- C_HPLOT ---*/
void RA_HDraw(char* name, char* options, int nsame);

/*--- C_HPLZOM ---*/
void RA_HSetXRange(char* name, float fmin, float fmax, float df);
void RA_HSetYRange(char* name, float fmin, float fmax, float df);

/*--- C_HRESET ---*/
void RA_HReset(char* name);

/*--- C_HFILL ---*/
void RA_H1Fill(char* name, float x , float weight);

/*--- C_HFILL2 ---*/
void RA_H2Fill(char* name, float x, float y, float weight);

/*--- C_HDELET ---*/
void RA_HDelete(char* name);

/*--- C_HBOOK1 ---*/
void RA_H1New(char* name,char* title,int nbins, float xmin, float xmax);
void RA_H1NewReuse(int pad, int subpad, char* name,char* title,int nbins, float xmin, float xmax);

/*--- C_HPAK ---*/
void RA_H1SetBinColor(char* name, int bin, int colorValue);
void RA_H1Pack(char* name, double* buff);
void RA_H2Pack(char* name, double* buff);
void RA_H2PackBits(char* name, double* buff);

/*--- C_HMINIM ---*/
void RA_HSetMinimum(char* name, double ymin);

/*--- C_HMAXIM ---*/
void RA_HSetMaximum(char* name, double ymax);

/*--- C_HPLTIT ---*/
void RA_HSetTitle(char* name, char* title);
void RA_HSetUnitsX(char* name, char* units);
void RA_HSetUnitsY(char* name, char* units);
void RA_HSetUnitsZ(char* name, char* units);

/*--- Sets the time string displayed below the plot ---*/
void RA_HSetTimeString(char* name, char* timestring);

/*--- Sets the flag that allows drawing the time string ---*/
void RA_HSetTimeFlag(char* name, int flag);
void RA_HSetTimeOffset(char* name, double offset, int timeaxis);

/*--- Sets Ndivisions of a 1D plot ---*/
void RA_SetAxisNdiv(char* name, int ndiv);

/*--- Sets the limits (xmin and xmax) of a 1D plot and retrieve them ---*/
void RA_SetLimitsX(char* name, double tmin, double tmax);
void RA_SetLimitsY(char* name, double tmin, double tmax);
double RA_HGetXmin(char* name);
double RA_HGetXmax(char* name);
double RA_HGetYmin(char* name);
double RA_HGetYmax(char* name);


/*--- C_HBOOK2 ---*/
void RA_H2New(char* name, char* title, int nbinx, float xbin, float xmax, int nbiny, float ymin, float ymax);
void RA_H2NewReuse(int pad, int subpad, char* name, char* title, int nbinx, float xmin, float xmax, int nbiny, float ymin, float ymax);

/*--- C_HUNPAK ---*/
void RA_H1Unpack(char* name, double* buff);
void RA_H2Unpack(char* name, double* buff);

/*--- C_IACWK ---*/
void RA_SetActivePad(int numpad, int numsubpad);
void RA_SetActiveSubPad(int num);

/*--- C_HPLZON ---*/
void RA_PadDivide(int nx,int ny);
void RA_PadDivideWithTitle(int nx,int ny);
void RA_PadClear(void* pad);
void RA_PadCompleteClear(int ipad);

/*--- Is a pad already divided ? ---*/
int RA_PadIsDivided(int npads);

/*--- Does pad already contain a suitable histo ? ---*/
char* RA_PadContainsGoodHist(int numpad, int numsubpad, int type, int nbinsx, int nbinsy, char *name);

/*--- Creation of a string from macro generated by global canvas ---*/
const char* RA_CanvasMacroString();

/*--- Process one line by the C/C++ interpreter ---*/
void RA_InterpreterProcessLine(char * cline);

/*--- Process the line generating a canvas ---*/
void RA_MacroCreateCanvas(char * cline);

/*--- Check the remaining swap ---*/
int RA_SystemCheckSwap();

/*--- Create a temporary file name ---*/
char* RA_CreateTempFileName(const char* suffix);

/*--- Draw the global title of the canvas ---*/
void RA_GlobalTitleDraw();

/*--- Process the ROOT events (used for interactivity while running) ---*/
void    RA_ProcessRootEvents();
void    RA_ObjectTablePrint();

#ifdef __cplusplus
}
#endif

#endif
