#ifndef VDYPLOT2_DEFINED
#define VDYPLOT2_DEFINED

/************************************************************************/
/*                                                                      */
/* VDyPlot2                                                              */
/*                                                                      */
/* class describing a plot (representation) of a series                 */
/* Used to adapt the behaviour of TH2F to datadisplay needs             */
/*                                                                      */
/************************************************************************/

#include "TH1.h"
#include "TH2.h"
#include "TString.h"
#include "TPaveText.h"


class VDyPlot2 : public TH2F
{
private :
      int     mTimeFlag;
      TString mTimeString;
      TPaveText* mTimePave;
      double  mTimeUTC;
      double  mTimeX1;
      double  mTimeY1;
      double  mTimeX2;
      double  mTimeY2;
      TArrayS mIndicator;
      int     mOldFirstXbin;
      int     mOldLastXbin;
      int     mOldFirstYbin;
      int     mOldLastYbin;
      double  mOldMinY;
      double  mOldMaxY;
      double  mPreviousMinY;
      double  mPreviousMaxY;
      char    mPlotType[64];
      
public :
    int alreadyDrawn;
    int numpad;
    int numsubpad;
    int superposed;
    VDyPlot2();
    VDyPlot2(Text_t* name, Text_t* title, Int_t nbinsx, Axis_t xlow, Axis_t xup, Int_t nbinsy, Axis_t ylow, Axis_t yup);
    VDyPlot2(VDyPlot2& VDyPlot2);
    ~VDyPlot2() {if (mTimePave) {delete mTimePave; mTimePave=0;}};
            void     Copy(TObject& newvsp);
            void     Draw(Option_t* option);
            void     Paint(Option_t* option);
            void     UpdateTimeFreq();
            void     ClearTimePave() {if (mTimePave) {delete mTimePave; mTimePave=0;} }
            void     UnlinkTimePave() {mTimePave=0;}
            void     ExportAscii(const char* fileName="Default_file_name_=_Plot_name"); //*MENU*
            void     SuperposeExternalPlot(const char *fileName="ddref.root", const char *plotName="h1"); //*MENU*
            void     RemoveRefPlot(); //*MENU*
            int      GetTimeString (char *dytimestring, const char *timeformat);
    virtual char*    GetObjectInfo(Int_t px, Int_t py) const;
            char*    GetPlotType() {return(mPlotType);};
    virtual void     SavePrimitive(ofstream &out, Option_t *option);
            void     SetTimeString(char* timestr) {mTimeString = timestr;}
            void     SetTimeUTC(double timeutc) {mTimeUTC = timeutc;}
            double   GetTimeUTC() {return(mTimeUTC);}
            void     SetTimeFlag(int flag) {mTimeFlag = flag;}
            void     SetIndicator(int bin, short value);
            void     ZoomAsLast();

    ClassDef(VDyPlot2,1) // Description of a plot-representation of a series
};

R__EXTERN VDyPlot2* gLastModifiedPlot2;
R__EXTERN VDyPlot2* gLastModifiedPlotY2;

#endif

