/******************* dataDisplay.h **********************/
/*                                                      */
/*                 VIRGO DATA DISPLAY                   */
/*                                                      */
/*   MODULE     : dataDisplay.h                         */
/*   AUTHOR     : D. Verkindt, D. Buskulic (LAPP,Annecy)*/
/*                verkindt@lapp.in2p3.fr                */
/*                                                      */
/******************* dataDisplay.h **********************/

#define DYVERSIONDATE "16 fev 2017"
#define DYVERSION  "v10r8"
#define DYFDVERSION  "v5r0"

#if !defined(DATA_DISPLAYH)
#define DATA_DISPLAYH DONE

#ifdef __cplusplus
extern "C" {
#endif

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include <limits.h>

/*
#include "dysoundcard.h"
*/
#ifndef NOCM
#include <CmVersion.h>
#include <CmMessage.h>
#endif

#include "FrameL.h"
#include "Frv.h"
#include "URootAccess.h"

#include <fselector.h>
#include <forms.h>
#include <dysliders.h>
#include <mainbrowser.h>
#include <plot1d.h>
#include <plotraw.h>
#include <plottime.h>
#include <plotfft.h>
#include <plotffttime.h>
#include <plottrfcttime.h>
#include <plotcohetime.h>
#include <plot1dtime.h>
#include <plotrawtime.h>
#include <plottrfct.h>
#include <plotcohe.h>
#include <plot2d.h>
#include <plotaudio.h>
#include <plotgenepar.h>
#include <mainpanel.h>
#include <getdd.h>
#include <getshm.h>
#include <cmservers.h>
#include <userfuncbrowser.h>
#include <dyhelp.h>
#include <dydebug.h>
#include <dyoptions.h>
#include <dyoutput.h>
#include <reffile.h>

/*======================== Defines and Enums ============================*/

extern char FrVersion[];

#ifdef OSF1
#define __FUNCTION__ "not defined under OSF1"
#endif

#define CURRENT_MANAGER "D. Verkindt (verkindt@lapp.in2p3.fr)"

#define FrVectCD FrVect
#define FrVectD FrVect
#define FrVectCF FrVect
#define FrVectF FrVect

#define FrVectCDNew(nL,nC) FrVectNew(FR_VECT_C16,2,nL,0.,NULL,nC,0.,NULL);
#define FrVectDNew(nL,nC) FrVectNew(FR_VECT_8R,2,nL,0.,NULL,nC,0.,NULL);
#define FrVectCFNew(nL,nC) FrVectNew(FR_VECT_C8,2,nL,0.,NULL,nC,0.,NULL);
#define FrVectFNew(nL,nC) FrVectNew(FR_VECT_4R,2,nL,0.,NULL,nC,0.,NULL);

#define DY_MAX(A,B) ((A) > (B) ? (A) : (B))
#define DY_SWAP(a,b) temp=(a);(a)=(b);(b)=temp;

#define PI        3.1415926535897932385
#define TWOPI     6.2831853071795864769
#define TWOOVERPI 6.3661977236758134308E-1
#define PI2       1.5707963267948966192
#define BOLTZMANN 1.380658E-23
#define VLIGHT    299792458.
#define SQRT2     1.4142135623730950488
#define INVSQRT2  7.0710678118654752440E-1

#define UHMAXDUMP 10
#define SOUND_CONVERT 1
#define SOUND_STOP 2
#define SOUND_WAIT 3
#define AUDIO_BUF_SIZE 1024

#define DY_LEG_YMAX     0.98             /* max y position of legend in a plot              */
#define ACTIVE_TIMEMAX  86400            /* max number of seconds between Start and Stop    */
#define SHM_NFRAMEMAX   8192             /* max number of frames in shared memory           */
#define NCHMAX          500000           /*  max number of channels in the browser          */
#define LSTRING_MAX      1024            /*  max length of character strings                */
#define LBIGSTRING_MAX   2048            /*  max length of big character strings            */
#define NFILE_MAX         256            /*  max number of files read in a directory        */
#define NPLOT_MAX         128            /*  max number of plots                            */
#define DEFAULT_SHMNAME "siesta"         /*  default name of shared memory                  */
#define DEFAULT_DYNAME  "dy"             /*  default configuration name                     */
#define ROOT_SUFFIX     ".root"          /*  suffix for plot file name                      */
#define CONFIG_SUFFIX   ".cfg"           /*  suffix for configuration file name             */
#define REF_SUFFIX      "__REF"          /*  suffix for reference plots                     */
#define SEPTITLE        "__"             /*  separator for plot type in plot title          */
#define SEPSMS          "_"              /*  new separator for vVar in case of sms variable */
#define SEP             ".."             /*  separator for vVar in case of sms variable     */
#define SEP2VAR         ".vs."           /*  separator for title of plot with 2 var.        */
#define SEP2VARTR       ".over."         /*  separator for title of plot TrFct              */
#define DYEMPTY         "..."            /*  equivalent to empty string                     */
#define DDREFPLOTS      "ddref.root"     /*  default name of reference plots file           */
#define DYPLOT_PREFIX   "dy"             /*  prefix of plots name                           */
#define DY_NOUNITS      "no_units"       /*  put on Y axis when no units found for data vector */

#define AUDIOPLAY_OSF1     "/usr/bin/mme/audioplay "
#define AUDIOCONTROL_OSF1  "/usr/bin/mme/audiocontrol "
#define AUDIOPLAY          "/usr/bin/play "
#define AUDIOCONTROL       "/usr/bin/aumix "
#define AUDIOCH_NAME       "fraudio"

#define DY_BADDATA    2.00e+31
#define DY_NOFRAME    1.23e+32
#define DY_NOCHANNEL  3.14e+32
     
#define DYCOL_BADDATA    2
#define DYCOL_NOFRAME    4
#define DYCOL_NOCHANNEL  16

#define DYMSG_BADDATA    "CORRUPTED DATA"
#define DYMSG_NOFRAME    "MISSING FRAME"
#define DYMSG_NOCHANNEL  "MISSING CHANNEL"

#define XREF -50
#define YREF -80

/*  Menu items  */

#define CASCINA           "Cascina"
#define FFLDIR            "/virgoData/ffl"

#define READ_FFL          "Read FFL"
#define READ_FILE         "Read Files"
#define CONNECT_CMSERVER  "Connect Online"
#define CONNECT_DD        "Connect to DataSender"
#define READ_SHM          "Read Shared Memory"
#define READ_WAVE         "Read Audio Wave File"
#define READ_ASCII        "Read Ascii File"

#define LOAD_CONFIG       "Load Config"
#define SAVE_CONFIG       "Save Config"
#define SAVE_CONFIG_AS    "Save Config As..."

#define DY_RESET          "Reset"
#define DY_PADS           "Cols x Rows"
#define DY_SLIDERS        "Sliding along plots"
#define DY_ALIGN          "Align plots in time"
#define DY_USERFCT        "Define user functions"
#define DY_OUTPUT         "Output data files"
#define DY_OPTIONS        "Options"
#define DY_CHECKS         "Checks"
#define DY_DEBUG          "Debug"
#define DY_HELP           "Help"

#define DY_SETREFPLOT     "Set Reference Plots"
#define DY_REMOVEREFPLOT  "Remove Reference Plots"
#define DY_GETREFPLOT     "Get Reference Plots"
#define DY_SAVEREFPLOT    "Save Reference Plots"
#define DY_EDITREFPLOT    "Edit Reference Plots"

#define DYLOG             "/virgoLog/Dy"
#define DYDUMP_FILE       "dydump.txt"

#define OPSIGNAL          "zz_"
#define OPSEP             ","
#define VALUEIN           "valueIn"
#define USEROP            "ch"
#define TRIGGER           "trigger"
#define DYALGO_DIR        "/virgoData/DyAlgo"
#define ALGO_BEGIN        "/*ALGO_BEGIN"
#define ALGO_END          "ALGO_END*/"

#define TRIGCH            "Trigger from Channels"
#define TRIGASCII         "Trigger from ASCII file"
#define TRIGEVENTS        "Trigger from FrEvents"

#define DDSERVER          "dataSender"
#define DDFFLDIR          "ddffldir"
#define DDCHLIST          "ddchlist"
#define DDFILELIST        "ddfilelist"
#define DATAREQUEST       "DataRequest"
#define DDFILE            "/virgoData/ffl"
#define DDDIR             "/virgoData/ffl"
#define DDWAIT            "ddwait"
#define DYOUTPUTDIR       "./dyoutput"
#define DYOUTPUTFFL       "dyoutput.ffl"

#define CHANNELS_LIST          "ChannelsList"
#define CMTYPE_FDFRAME         "FdFrame"
#define CMTYPE_DYINFO          "dyinfo"
#define CMTYPE_DYQUIT          "CfgExit"
#define CMTYPE_CHLIST          "FdChannelsList"  
#define CMTYPE_CHLIST_REQUEST  "FdGetChannelsList"  
#define CMTYPE_FDADDFRAME      "FdAddCmOutput"
#define CMTYPE_FDREMOVEFRAME   "FdRemoveCmOutput"
#define CMTYPE_NSNAMES         "NSNames"

/****************************/

#define DYCM_QSIZE         20
#define DY_NOENDTIME      1e+12
#define DY_TIMEOUT1       120
#define DY_TIMEOUT2        10 
#define DY_TIMEOUT3         5 

#define DY_FONT_TIMES      22
#define DY_FONT_HELVETICA  62

#define NARG_MAX           64
#define NEVENT_MAX     256000     /* max number of events taken from an ASCII file */
#define DBGTIME_LEVELMAX    6     /* max number of levels used in DyDebugTimeNeeded */

enum {NEXT_EVENT, NEXT_UPDATE, CONTINUE,
      STOP, CLOSE, PAUSE, WAIT};          /* data display states (Dy->Order variable)  */

enum {OFF, ON, ON2, ON3};                 /* some states used by Dy->Clone variable    */

enum {INFILE, INCM, INDD,
      INVEGA, INSHM, INOFF,
      INWAVE, INASCII, INDY};             /* input type */

enum {MAINPANEL, HELPPANEL, MAINBROWSER,
      INPUTFILE, CMSERVERS, GETDD, DUMPDATA,
      PLOTTIME, PLOTFFT, PLOTTR,
      PLOTCOHE, PLOT1D, PLOT2D,
      PLOTAUDIO, PLOTFFTTIME, GENEPAR,
      USERFUNCBROWSER, USERFCTHELP, COFFEEPANEL,
      PLOTRAW, PLOT1DTIME, PLOTRAWTIME,
      PLOTTRTIME, PLOTCOHETIME, OUTPUTDATA,
      REFFILE, DYOPTIONS, DYDEBUG,
      GETSHM, DYHELP, DYSLIDERS};           /* Panels (and plots) types    */

enum {REPLACE_PLOT,ADD_PLOT};               /* States for the DySetPlot and DyEditPlotCallBack */

enum {NONE, CONT, LEGO, SURF, SCAT, COLZ};  /* options for 2D plots  */

enum {DBG_CALLS, DBG_SPEED, DBG_VDYPLOT, DBG_READ, DBG_OTHERS};     /* debug types */

enum {TRIG_CH, TRIG_ASCII, TRIG_EVENTS};    /* type of trigger source */
enum {PROCDATA, SIMDATA, ADCDATA, SERDATA}; /* type of data */


/*********     WAV BigEndian File Version ***********************/

struct soundhdr
{
  char  riff[4];        /* "RIFF"                                  */
  int  flength;        /* file length in bytes                    */
  char  wave[4];        /* "WAVE"                                  */
  char  fmt[4];         /* "fmt "                                  */
  int  block_size;     /* in bytes (generally 16)                 */
  short format_tag;     /* 1=PCM, 257=Mu-Law, 258=A-Law, 259=ADPCM */
  short num_chans;      /* 1=mono, 2=stereo                        */
  int  srate;          /* Sampling rate in samples per second     */
  int  bytes_per_sec;  /* bytes per second                        */
  short bytes_per_samp; /* 2=16-bit mono, 4=16-bit stereo          */
  short bits_per_samp;  /* Number of bits per sample               */
  char  data[4];        /* "data"                                  */
  int  dlength;        /* data length in bytes (filelength - 44)  */
};


/*=========================  ShmInfo  ================================*/

struct ShmInfo
{
/*  key_t key;   */               /* key to sign the shared memory segment */
  int  shmid;                 /* id of the shared memory segment (deduced from the key) */
  void *segptr;               /*address shared memory segment */
  long size;                  /* size of the shared memory segment (bytes) */
  char version[64];           /* version of FdShm */
  long dynamic;               /* flag>0 if optimize shm for variable size frames */
  long statusSize;            /* size of shminfo structure */
  long frameSizeMax;          /* maximum size of a frame in shm */
  long nFrameMax;             /* maximum number of frames in shm (if dynamic=0) */
  int latency;                /* max latency of the producer */

  long frameSh[SHM_NFRAMEMAX];    /* address of frames in shm */
  long frameSize[SHM_NFRAMEMAX];  /* size of frames in shm (bytes) */
  int  frameGPS[SHM_NFRAMEMAX];   /* GPS time of frames in shm */
  long iwrite;                /* write counter of the shm producer */

  long nConsumer;              /* number of consumers connected to shm */
  int  producerPid;            /* producer pid */
  char producerName[128];      /* producer Cm name */
  char outputName[128];        /* name of the common output of the consumers (triggermanager) */

  long iwriteMod;             /* write counter modulo number of frames in shm */
  long iwriteMax;             /* iwrite of the last frame in shm */
  long curSizeMax;            /* size of the current largest frame in shm */
};

/*=========================  DyCh  ================================*/

struct DyCh
{
  int type;                    /* channel type (ADC, SER, PROC...)               */
  double status;               /* channel status (ok, missing, corrupted...)     */
  char name[256];              /* name of the channel                            */
  double sampFreq;             /* initial sampling frequency                     */
  double resampFreq;           /* resampling frequency                           */
  int realunits;               /* flag is ON if vect contains calibrated data    */
  double slope;                /* slope  used to calibrate channel               */
  double bias;                 /* bias  used to calibrate channel                */
  double mean;                 /* current mean of data read                      */
  struct FrVectD  *vect;       /* input vector for this channel after processing */
  struct FrVectD  *subvect;    /* input vector for FFT computations              */
  char inputname[LSTRING_MAX]; /* name of the file from which channel is read    */
  struct FrFile *ifile;        /* file pointer used when reading data from file  */
  struct ShmInfo *shm;         /* shm pointer used when reading data from shm    */
  FILE           *wavefile;    /* wav file pointer used when reading audio file  */
  int inputtype;               /* type of input (INFILE, INSHM, INCM...)         */
  struct FrvBuf *buffer;       /* buffer used to process/resample input channel  */
  struct FrvFilterB *aafilter; /* filter structure used for anti-aliasing filter */
  struct FrvFilterB *bpfilter; /* filter structure used for passband filter      */
  double bpfilter_fmin;
  double bpfilter_fmax;
  int op_nch;
  char op_ich[64];
  struct DyCh *op_chin[64];
  char op_classname[128];
  char op_string[512];
  char op_cstring[256];
  double vect2d_duration;
  double vect2d_deltat;
};

/*========================  DyPlot  ============================*/

struct DyPlot
{
  int    autoedge;           /* flag to have auto edges updates in 2D/1D plots */
  int    autoscale;          /* flag to have Y axis auto scaling in TIME plots */
  int    bits;               /* flag: >0 if bits of values are displayed       */
  int    color;              /* color of the plot                              */
  int    colorpalette;       /* 0=rainbow 1=flame 2=green 3=violet             */
  double corrupt;            /* flag indicates if last data used are corrupted */
  int    crossCohe;          /* flag: >0 if plot is a crossCoherence (2D)      */
  double crossCohe_fmax;     /* upper edge of crossCoherence plot y axis       */
  double crossCohe_fmin;     /* lower edge of crossCoherence plot y axis       */
  double dc;                 /* moving average for DC removal                  */
  double decay;              /* >0  if average is computed with a decay        */
  int    decayfft;           /* flag: >0 if average computed with a decay      */
  double deltat;             /* time length of vector data needed by the plot  */
  double dfreq;              /* frequency resolution (Hz) of FFTTime plots     */
  long   dNbin;              /* nbr of bins to shift between two data read     */
  double dtime;              /* time resolution (sec) of FFTTime plots         */
  double dt;                 /* sampling period                                */
  struct FrVect *buffer;
  int ibuf;
  struct FrvBuf *frvbufx;    /* buffer used to do the FFTTIME plot             */
  struct FrvBuf *frvbufy;    /* buffer used to do the COHETIME plot            */
  struct DyCh *chx;          /* channel on x (may be input channel or combination of input channels) */
  struct DyCh *chy;          /* channel on y (may be input channel or combination of input channels) */
  double fft_duration;       /* time window over which FFT is done (FFTTime)   */
  double fft_deltat;         /* time step between two FFTs (FFTTime)           */
  double filter_fmax;        /* upper edge of pass-band filter                 */
  double filter_fmin;        /* lower edge of pass-band filter                 */
  double filter_order;       /* order of pass-band filter                      */
  double fmax;               /* max frequency bin of the spectrum plot         */
  double fmin;               /* min frequency bin of the spectrum plot         */
  struct FrvRFFT *fftx;      /* FFTW buffer for input channel x                */
  struct FrvTF   *tf;        /* structure for TrFct and Coherence plots        */
  double *buff;              /* buffer for time-frequency plot                 */
  int    gridx2;             /* flag to get x grid in phase plot               */
  int    gridx;              /* flag to get x grid                             */
  int    gridy2;             /* flag to get y grid in phase plot               */
  int    gridy;              /* flag to get y grid                             */
  int    gridz;              /* flag to get z grid                             */
  double gtime;              /* current GPS time of data shown by the plot     */
  int    hertz;              /* flag to get spectra in 1/Hz instead of 1/sqrt(Hz) */
  int    hidden;             /* flag>0 when plot is hidden in the list         */
  int    iFirst;             /* flag for first pass in plot drawing function   */
  int    itime;              /* time axis bin counter for time-frequency plot  */
  double lastrefresh_time;   /* time of data at the last refresh of the plot   */
  int    logx2;              /* flag to get x log scale in phase plot          */
  int    logx;               /* flag to get x log scale                        */
  int    logy2;              /* flag to get y log scale in phase plot          */
  int    logy;               /* flag to get y log scale                        */
  int    logz;               /* flag to get z log scale (2D plots)             */
  long   maxAverage;         /* max number of spectra averaged (fft plots)     */
  int    median_norm;        /* flag to have spectrogram normalized by median  */
  int    modulesqrt;         /* flag to have sqrt of module in COHE plots      */
  char   name[LSTRING_MAX];  /* name of the plot                               */
  char   namePhase[LSTRING_MAX]; /* name of the phase plot of the TRFCT        */
  char   *name_rms;          /* name of plot containing integrated rms         */
  struct FrVect *vect_rms;   /* vector to contain rms values computed in FFT   */
  long   nAverage;           /* current nbr of spectra averaged (fft plots)    */
  long   nBin;               /* number of bins used for stat in 1DTime plot    */
  int    nodc;               /* flag: >0 if no DC component in spectra         */
  double norm;               /* normalization factor for FFTs                  */
  int    idsame;             /* id of the plot to which is superposed this one */
  int    superposed;         /* flag>0 if plot is superposed to another one    */
  double timeshift;          /* time shift between ch1 and ch2 (2D,TRFCT,COHE plots) */
  char   options[LSTRING_MAX]; /* list of options used by Draw                 */
  int    numpad;             /* pad number where is done the plot              */
  int    numsubpad;          /* subpad number where is done the plot           */
  int    raw2d;              /* flag to say that raw vector is an image or not */
  int    ready;              /* flag to say that plot is ready to be displayed */
  long   refresh_fft;        /* number of spectra to do before to refresh plot */
  double refreshDt;          /* time period of plot's refresh                  */
  int    rmsfft;             /* flag: >0 if rms superposed to spectra          */
  int    sound16bits;        /* flag: >0 if audio output is on 16 bits         */
  int    soundonline;        /* flag: >0 if audio output online stream is on   */
  double audio_t0;           /* begin time for audio (s)                       */
  double audio_duration;     /* duration for audio (s)                         */
  long   audio_itime;        /* counter of sample written in file              */
  double audio_previousValueBuf; /* valueBuf computed at previous time step    */
  int    audio_resampFactor; /* interpolation factor when Fsamp<5512           */
  int    audio_encoding;     /* signal linear encoding 8 or 16 bits            */
  char   audio_outfileName[1024];       /* Audio output file name              */
  long   audio_data_nBytes;  /* Audio output file data length                  */
  long   audio_begin_nb;     /* Audio output file begin sampling nbr           */
  long   audio_end_nb;       /* Audio output file end sampling nbr             */
  unsigned char audio_bufstream[AUDIO_BUF_SIZE];   /* buffer for audio output stream 8 bits  */
  unsigned short audio_bufstreamS[AUDIO_BUF_SIZE]; /* buffer for audio output stream 16 bits */
  int    audio_nsamp;        /* counter to fill the audio device buffer        */
  double audio_delta;        /* xmax-xmin difference used to normalize audio   */
  int    audio_fd;           /* file descriptor for output audio device        */
  FILE  *audio_fout;         /* output file                                    */

  double stat_duration;      /* time window over which distrib is done (1DTime) */
  int    stat;               /* flag to get statistical info (1D plots)        */
  double timeSize;           /* larger time window size (s) for plotFFTTime    */
  char title[LSTRING_MAX];   /* plot title                                     */
  char   *title_rms;         /* title of plot containing integrated rms        */
  double tmin;               /* current starting time of time Window           */
  int    toberemoved;        /* flag to say that the plot must be removed      */
  double tsize;              /* time window size (s)                           */
  double tstep;              /* Shift in time between two FFTs (%)             */
  double tstep_fft;          /* Shift in time between two FFTs (%) in a FFTTIME plot */
  int    type2d;             /* flag to plot 2D histos as CONT, LEGO or SURF   */
  int    type;               /* type of the plot (time, fft, 1d, ...)          */
  int    x2av;               /* flag: >0 if average uses square of values      */
  double xmax;               /* upper edge of the plot                         */
  double xmin;               /* lower edge of the plot                         */
  long   xnBin;              /* number of bins used                            */
  double ymax;               /* upper edge of the plot (2D plots)              */
  double ymin;               /* lower edge of the plot (2D plots)              */
  long   ynBin;              /* number of bins used (2D plots)                 */
  double yoffset;            /* users's offset on Y axis of TIME plots         */
  double yscale;             /* users's scaling factor on Y axis of TIME plots */
  double xoffset;            /* users's offset on X axis of 1D plots           */
  double xscale;             /* users's scaling factor on X axis of 1D plots   */
  double zmax;               /* upper z limit of the plot (FFTTime plots)      */
  double zmin;               /* lower z limit of the plot (FFTTime plots)      */
};

/*========================  DyTool  ============================*/

struct DyTool
{
  double xt_size;
  double yt_size;
  double xl_size;
  double yl_size;
  double xt_offset;
  double yt_offset;
  int   ttstart;                  /* time when the Start button was pushed                */
  int   no_autostop;              /* flag>0 keeps dataDisplay running for any duration    */
  int   defaultcolor[16];         /* array containing the default colors of lines in plots*/
  int    colorset;                /* set of default plot line colors (default is 0)       */
  int   debug;                    /* debug level (if <0, no GPS origin line under plots)  */
  int   debugSleep;               /* debug sleep time (if >0, debugSleep seconds between each plot */
  int   debugType;                /* debug type (which part of code is debugged)          */
  struct FrameH *Frame;           /* a frame first read to get the channels               */ 
  char   curConfig[LSTRING_MAX];  /* generic current name used for .car .hsto files       */
  long   CmConnect;               /* flag to know if we are Cm connected or not           */
  char   fileName[LSTRING_MAX];   /* input file name                                      */
  char   shmName[LSTRING_MAX];    /* shared memory producer name                          */
  int    nshm;
  char   shmlist[32][32];
  int    cmnbr;                   /* Counter incremented at each Cm connection            */
  char   DyName[LSTRING_MAX];     /* name of the datadisplay (used for canvas and Cm)     */
  char   CmName[LSTRING_MAX];     /* Cm name of the datadisplay                           */
  char   CmServer[LSTRING_MAX];   /* Cm name of server to which dataDisplay is connected  */
  char   CmDomain[LSTRING_MAX];   /* Cm Domain used each time DyCmConnect is used         */
  int    CmFullList;              /* Flag>0 show full list of Cm names in the GUI         */
  double CmMftspan;               /* Time span of the buffer of frames received by Cm     */
  struct FrFile * CmMffile;       /* File pointer on the Cm received frames in memory     */ 
  int    ncm;
  char   cmlist[256][256];        /* list of visible Cm names in the CmDomain             */
  int    ndd;
  char   ddlist[256][256];        /* list of visible DataSender in the CmDomain           */
  int    nddfile;
  char   ddfilelist[256][256];    /* list of data files seen by the DataSender            */
  int    plotsNotDisplayed;       /* flag>0 if plots computed but not displayed           */
  int    QuitAtStop;              /* flag>0 if dataDisplay quits when it stops            */
  int    nocm;                    /* option to run dataDisplay without using Cm           */
  int    BgRun;                   /* flag>0 if ROOT plots are done in background and saved as jpg */
  char   *outPicture;             /* Name of output picture (with suffix .jpg or root.    */
  char   *titleCmd;               /* title of the canvas taken from command line          */
  int    colorpalette;            /* colorpalette type: 0=rainbow 1=flame 2=green 3=violet */
  char   chPrefix[32];            /* Channel prefix (H1, V1, etc...) used in the data read */

  char   conditionName[LSTRING_MAX];/* FrEvent names                                      */
  double conditionTh;               /* FrEvents threshold                                 */
  struct FrVect *triggerVect;     /* vector containing 1 if sampling triggers, 0 otherwise */
  int    triggerPause;            /* flag is positive if option of pause on trigger chosen*/
  int    triggerUsed;             /* flag is positive if dataDisplay runs with a trigger  */
  int    triggerEvent;            /* flag is positive if a trigger has seen an event      */
  int    triggerType;             /* type of trigger source: 0=channels 1=AsciiFile 2=FrEvents  */
  int    nevent_ascii;            /* number of events in ASCII file                       */
  double AsciiGps[NEVENT_MAX];    /* list of gps times of the events taken from ASCII file */
  char   trigAsciiFileName[LSTRING_MAX]; /* Name of ASCII file containing trigger GPS times */
  char   trigEventName[LSTRING_MAX];     /* Name of EVENT used to search for trigger GPS times */
  char   lastTrigString[LBIGSTRING_MAX]; /* last trigger string used                      */
  char   *trigMenuItem;           /* last item chosen in Trigger menu                     */

  char   lastsavedPlots[LSTRING_MAX]; /* last file name used to save plots                */
  char   *lastrefplot_dir;        /* last directory where reference plots file was read   */
  char   *lastsavedRefPlots;      /* last file name used to save reference plots          */
  char   lastsavedConfig[LSTRING_MAX]; /* last file name used to save config ie siesta cards  */
  char   *lastconfig_dir;         /* last directory where configuration file was read     */
  int    refplot_indelebile;      /* flag>0 if superposed reference plots are indelebile  */
  int    nicefont;                /* flag>0 if font 62 (instead of 20) is used while running  */
  char   *Dir;                    /* directory were are the frames files to read          */
  int    nfile;                   /* OBSOLETE : number of input files                     */
  char   filelist[256][256];      /* list of data files available as input                */
  int    nreffile;                /* number of reference plot files                       */
  int    refcolor[256];           /* color of the reference plots of file reffilename     */
  char   reffilename[256][256];   /* name of the file containing the reference plots      */
  char   typeName[NARG_MAX][64];  /* explicit type name of a plot                         */
  char   inputtypeName[NARG_MAX][64];  /* explicit type name of input source              */
  struct DyCh *selectedCh;        /* the channel selected in the Channel browser          */
  long   lastSearchI;             /* last line found in browser by the searchInBrowser    */
  char   lastSearchName[LSTRING_MAX]; /* last name found by the searchInBrowser           */
  double firstTime;               /* GPS time of first frame read                         */
  double endTime;                 /* end GPS time of the data to read                     */
  double min_refreshDt;           /* minimum of the plots refresh periods                 */
  double mindt;                   /* minimum of the data vector duration asked by plots   */
  double maxdt;                   /* maximum of the data vector duration asked by plots   */
  double startTime;               /* starting GPS time                                    */
  double startTimeCmd;            /* starting GPS time provided on command line           */
  double previousTime;            /* previous current GPS time of data read               */
  double duration;                /* time duration over which we look data                */
  double durationCmd;             /* time duration provided on command line               */
  double maxtimewindow;           /* largest time window used by the plots                */
  double maxtimestep;             /* largest time step used by the plots                  */
  long   y2d;                     /* flag for 2D plots management                         */
  int    WaitData;                /* flag to wait for new data once reached the end of file */
  int    Order;                   /* data taking status (CONTINUE, STOP, PAUSE,...)       */
  int    UpdateDone;              /* At least one plot has been updated                   */
  char   *AutoConf;               /* config name when dataDisplay get config from args    */
  int    AutoConfFlag;            /* flag is positive if dataDisplay started with a config*/
  long   previousInputType;       /* flag to keep previous input type                     */
  long   InputType;               /* flag to keep input type                              */
  long   ddfilelist_flag;         /* flag >0 if dataSender sent a file list               */
  int    DdNS;                    /* flag >0 if getting dataSenders list from Cm NS       */
  char   Ddffldir[LSTRING_MAX];   /* default ffl directory provided by dataSender         */
  char   *chlist;                 /* channels list string from a Cm server                */
  char   *Ddchlist;               /* channels list string from Data distribution server   */
  char   *Ddpasswd;               /* password for Data distribution server access         */
  char   DdDir[LSTRING_MAX];      /* Directory scanned by Data distribution server        */
  char   DdServer[LSTRING_MAX];       /* Cm name of the Data distribution server          */
  char   Ddinputfile[LBIGSTRING_MAX]; /* input file to be read by dataSender              */
  double startLatency;            /* latency at start of main loop reading data           */
  void*  frameChannel;            /* pointer to frame channel when used with vega         */
  double corrupt;
  char previoustrigVectList[LBIGSTRING_MAX]; /* used in DySetTriggerString to keep mem. of trig userop  */
  int    AutoDeselectPlot;        /* flag to automatically deselect plots browser entries */
  int    AutoDeselectCh;          /* flag to automatically deselect channels browsers entries */
  int    skipFrame;               /* flag to skip missing frames or not                   */
  int    nCol;                    /* user chosen number of columns in Root Canvas         */
  int    nRow;                    /* user chosen number of raws in Root Canvas            */
  int    fullwhite;               /* is ON when canvas color is white instead of grey     */ 
  int    linewidth;               /* width of the plots lines                             */
  int    squarePlot;              /* is ON when plots should be squares                   */
  char   *cmacrocopy;             /* string that contains Root Canvas taken from config   */
  int    nplotcreated;            /* number of plots created since start of dataDisplay   */
  int    nplot;                   /* number of plots (total)                              */
  int    nplotdrawn;              /* number of plots drawn in the pads                    */
  int    iplotref;                /* number of the plot used as reference for pads update */
  int    noBox;                   /* is ON if green boxes not displayed when data is missing */
  char   output_dir[LSTRING_MAX];
  int    output_flag;
  int    output_nframe_perfile;
  char   input_tag[LBIGSTRING_MAX];
  char   output_tag[LBIGSTRING_MAX];
  int    output_nframe_written;
  double output_nbytes_tot;
  struct FrFile *output_ffl;
  struct FrFile *output_file;
  char   gLastTitle[1024];
  char   gLastTitleY[1024];
  struct DyCh *ch[NCHMAX];        /* list of channels stored in browser    */
  int    nch;                     /* current number of channels in browser */
};


/*****************************************/
/*     DECLARATIONS OF FUNCTIONS         */
/*****************************************/

#ifndef NOCM
CmMessageStatus DyCmFrHandler(CmMessage message,char* sender,char* serverName);
CmConnectCondition DyCmWait (int timeout, char *servername);
CmMessageStatus DyNamesHandler (CmMessage message, char* sender, char* serverName);
CmMessageStatus DyCmInfoHandler (CmMessage message, char* sender, char* serverName);
CmMessageStatus DyCmQuitHandler (CmMessage message, char* sender, char* serverName);
CmMessageStatus DyDdHandlerFFL (CmMessage message, char* sender, char* servername);
CmMessageStatus DyDdHandlerChlist (CmMessage message, char* sender, char* servername);
CmMessageStatus DyDdHandlerFilelist (CmMessage message, char* sender, char* servername);
#endif

void DyRefPlotAdd(char *filename, int color);
void DyRefPlotRemove(char *filename);

double Dy_quickselect(double *arr, int n, int k);

void   DyAddFile (char *result, int arg);
struct DyCh *DyAddUserChannel (char *className, char *inputVect);
void   DyAddShm (char *shmname);
char   *DyAddToTag (char *tag, struct DyCh *ch);
void   DyAudioControl (FL_OBJECT *obj, long arg);
char   *DyBaseName (char *allname);
void   DyBasicProcessVect (int iplot);
void   DyChangeVarName (char *result, char *vVar, int datatype, int datatype_new, double freqClock);
struct DyCh *DyChClone (struct DyCh *chin);
void   DyChCopy (struct DyCh *chout, struct DyCh *chin);
void   DyChCloneOpch (struct DyCh *chout, struct DyCh *chin);
void   DyChCloseInput (struct DyCh *ch);
void   DyChCloseClone (struct DyCh *ch, struct DyCh *ch0);
void   DyChDescription (char *chfile, char *chname);
void   DyChFill (struct FrameH *frame, char *inputfile);
struct DyCh *DyChFillFromOpstring (struct DyCh *opch, char *opstring);
int    DyChFilter (const char *filtertag);
int    DyChFix2d (struct FrameH *frame, struct DyCh *ch);
void   DyChFree (struct DyCh *ch);
void   DyChGetPrefix ();
int    DyChOpenClone (struct DyCh *ch, struct DyCh *ch0);
void   DyChOpenInput (struct DyCh *ch);
static int DyChCmpName(const void *p1, const void *p2);
static int DyChCmpFreq(const void *p1, const void *p2);
void   DyChSort ();
void   DyChSortByFreq();
void   DyChRemove (int i0);
int    DyCheckData (struct FrVect *data);
double DyCheckMemory ();
void   DyCheckPlot (long id);
void   DyCheckRunning ();
char   *DyCheckString (char* name);
void   DyChecks ();
double DyCheckVect(struct FrVect *vect, struct DyCh *ch, double gtime, double duration);
void   DyChScaleOffset(struct DyCh *ch, double scale, double offset);
void   DyChZeroMean(struct DyCh *ch);
int    DyCmConnect ();
void   DyCmDisconnect ();
void   DyCmLoop();
void   DyCmSetCmDomain (char *DyCmDomain);
void   DyColorPalette (FL_OBJECT *obj, long arg);
int    DyConfigCreate (char *fileName, int noroot);
void   DyConfigGetCanvas (char *config);
int    DyConfigGetFile (char *result);
char   *DyConfigGetInBuffer (char *config_filename);
void   DyConfigGetInput (char **wordlist, int *iaddress, int nword);
void   DyConfigGetOpch (char **wordlist, int *iaddress, int nword);
void   DyConfigGetOptions (char **wordlist, int *iaddress, int nword);
void   DyConfigGetPads (char **wordlist, int *iaddress, int nword);
void   DyConfigGetPlot (char **wordlist, int *iaddress, int nword);
void   DyConfigGetRefPlots (char **wordlist, int *iaddress, int nword);
void   DyConfigGetTag (char **wordlist, int *iaddress, int nword);
void   DyConfigGetTiming (char **wordlist, int *iaddress, int nword);
void   DyConfigGetTriggers (char **wordlist, int *iaddress, int nword);
void   DyConfigPutInputs (FILE *fp);
void   DyConfigPutOpch (FILE *fp);
void   DyConfigPutOptions (FILE *fp);
void   DyConfigPutPads (FILE *fp);
void   DyConfigPutPlot (int iplot, FILE *fp);
void   DyConfigPutRefplots (FILE *fp);
void   DyConfigPutTiming (FILE *fp);
void   DyConfigPutTriggers (FILE *fp);
void   DyConstrain (double *value);
void   DyContinue ();
time_t DyConvertToUTC (int gps, int uLeapS);
void   DyCopyPlot (long idout,long idin,int fullcopy);
void   DyCopyPlotForSame (struct DyPlot *target, struct DyPlot *source);
int    DyCountChannels (struct FrameH *frame);
struct FrVect *DyCreateOpVect(struct DyCh *ch, struct FrVect **invects);
long   DyDdChlistRequest (char *DdServer, char *inputfile, int gpsStart);
void   DyDdContinueRequest (char *DdServer);
long   DyDdDataRequest (char *DdServer, char *inputfile, char *tag, int gpsStart, int nSeconds);
long   DyDdFFLRequest (char *DdServer);
void   DyDdPauseRequest (char *DdServer);
long   DyDdSendWaitingTime (double waiting_time);
void   DyDebugPrint (const char *msg, int printflag);
double DyDebugTimeNeeded (const char *fctname, const char *flag);
void   DyDecodeVar (char *input, char **smsName, char **varName);
int    DyFdRequestChannelsList (char *source, int gps);
int    DyFdRequestFrames (char *source, char* destination, char *tag, int queueSize, int nRetry);
int    DyFdRemoveFramesRequest (char *source, char* destination);
struct FrVect *DyFiltfilt(struct FrvFilterB *filter, struct FrVect *vect);

/****************************/

void   DyFileLoop();
char   *DyFilterString (char* namei, unsigned char separator);
char   *DyFrVectDumpToBuf (struct FrVect *vect, char *buf, long dumplevel);
void   DyFree();
int    DyGTimeFromDate (char *inputDate);
int    DyGetCmFrame();
void   DyGetDefaultFromPlot (long id);
void   DyGetMaxTimeStep();
char   *DyDirName (char *dirname, char *allname);
int    DyGetFrameHFromChList (char *chList, int *uleaps, double *frameDt, int *frameN, int *runN);
const char *DyGetFromChList (char *chList, char *chName, double *freq, char **smsData, double *valMin, double *valMax);
double DyGetGPSNow ();
void   DyGetInputVectors (struct DyCh *ch);
double DyGetLatency (double gps, double uleaps);
double DyGetMax (double x1, double x2);
double DyGetMin (double x1, double x2);
void   DyGetMinMax (struct FrVect *vect, double *vmin, double *vmax);
struct DyCh *DyGetOpString (const char *opstring);
int    DyGetShmList ();
int    DyGetSmsVarList (int *ifirst, char **wordlist, struct FrSerData *smsStation);
const char *DyGetTypeFromChList (char *chList, char *chName);
char   *DyGpsToTimeString (double origin, struct FrameH *frame, double deltat, int nAverage, int decayfft, char *title);
void   DyInfoLine (char *msg);
void   DyInfoPanel (char *msg, int alarm);
void   DyIni (char **arg1, int arg2);
int    DyJulianDay (int y, int m, int d);
void   DyLoadScript (char *className);
void   DyMakeAlgoString (FILE *fp);
void   DyMakeOpCString (char *opstring, char *opstring2, char *inputVect);
void   DyMakePlotOptions (long id, char *options, char *options2);
void   DyMakeTag (char *tag);
void   DyMakeTrigCString ( char *opstring, char *opstring2, char *inputVect);
void   DyMakeTrigEventList (char *opstring);
void   DyMakeTypeNames ();
int    DyMissingFrames (int previous, struct FrameH *frame);
double DyModule(double vin1, double vin2);
double DyPhase(double vin1, double vin2);
void   DyNext (long arg);
char   *DyOpStringReplaceChannels (char *opstring, char *vectlist);
void   DyOutputClose ();
void   DyOutputDefault ();
void   DyOutputWrite (struct FrameH *frame);
void   DyPassbandfilter(struct DyCh *ch);
void   DyPause ();
void   DyPlot1D (int iplot);
void   DyPlot1DEnd (int iplot);
void   DyPlot1DIni (int iplot);
void   DyPlot1DTIME (int iplot);
void   DyPlot1DTIMEEnd (int iplot);
void   DyPlot1DTIMEIni (int iplot);
void   DyPlot2D (int iplot);
void   DyPlot2DEnd (int iplot);
void   DyPlot2DIni (int iplot);
void   DyPlotAUDIO (int iplot);
void   DyPlotAUDIOEnd (int iplot);
void   DyPlotAUDIOIni (int iplot);
void   DyPlotAlignTime (long id);
void   DyPlotAutoAdjust1D (int iplot, double localxmin, double localxmax);
void   DyPlotAutoAdjust2D_X(int iplot, double localmin, double localmax);
void   DyPlotAutoAdjust2D_Y(int iplot, double localmin, double localmax);
void   DyPlotAllIni();
void   DyPlotCOHE (int iplot);
void   DyPlotCOHEEnd (int iplot);
void   DyPlotCOHEIni (int iplot);
void   DyPlotCOHETIME (int iplot);
void   DyPlotCOHETIMEPerVect(int iplot, struct FrVect *subvectx, struct FrVect *subvecty);
void   DyPlotCOHETIMEEnd (int iplot);
void   DyPlotCOHETIMEIni (int iplot);
void   DyPlotCheckFreq (int iplot);
int    DyPlotCheckPad ();
void   DyPlotCheckParam (int iplot);
void   DyPlotDraw (int iplot);
void   DyPlotDraw2 (int iplot);
void   DyPlotEnd ();
void   DyPlotFFT (int iplot);
void   DyPlotFFTEnd (int iplot);
void   DyPlotFFTIni (int iplot);
void   DyPlotFFTTIME (int iplot);
void   DyPlotFFTTIMEPerVect(int iplot, struct FrVect *subvect);
void   DyPlotFFTTIMEEnd (int iplot);
void   DyPlotFFTTIMEIni (int iplot);
void   DyPlotFree (long id, int fullfree);
int    DyPlotGetId (char *plotname);
int    DyPlotGetIsuperposed (int iplot);
int    DyPlotGetNsuperposed (int iplot);
int    DyPlotGetSameIndex (int iplot);
double DyPlotGetTimewindow (int i);
int    DyPlotGetType (char *plotname);
int    DyPlotGetUpdater();
void   DyPlotGetVectors (struct DyPlot *plot, struct FrameH *framein);
void   DyPlotIni(int iplot, double timewindow);
void   DyPlotInit (long id, long fullinit);
int    DyPlotLoop(double gtime);
void   DyPlotProcess(int iplot, double timewindow);
void   DyPlotRAW (int iplot);
void   DyPlotRAWEnd (int iplot);
void   DyPlotRAWIni (int iplot);
void   DyPlotRAWTIME (int iplot);
void   DyPlotRAWTIMEEnd (int iplot);
void   DyPlotRAWTIMEIni (int iplot);
int    DyPlotReady (int iplot);
void   DyPlotSetBrowserline (int iplot, char *browserline);
void   DyPlotSetColor (int iplot);
void   DyPlotSetDrawInfo (int iplot);
void   DyPlotSetDrawOptions (int iplot);
void   DyPlotSetAllSize();
void   DyPlotSetLegendSize(int iplot, double *xorig, double *yorig, double *leg_size);
void   DyPlotSetMinRefreshDt();
void   DyPlotSetMindt ();
void   DyPlotSetMaxdt ();
void   DyPlotSetMinMax (struct DyPlot *plot, double localymin, double localymax);
void   DyPlotSetSecondParam (int iplot);
void   DyPlotSetTime (double origin, char *plotName, double deltat, int nAverage, int timeaxis, struct FrameH *frame, int decayfft);
void   DyPlotSetTitle (int iplot);
void   DyPlotTIME (int iplot);
void   DyPlotTIMEEnd (int iplot);
void   DyPlotTIMEIni (int iplot);
void   DyPlotTRFCT (int iplot);
void   DyPlotTRFCTEnd (int iplot);
void   DyPlotTRFCTIni (int iplot);
void   DyPlotTRFCTTIME (int iplot);
void   DyPlotTRFCTTIMEPerVect(int iplot, struct FrVect *subvectx, struct FrVect *subvecty);
void   DyPlotTRFCTTIMEEnd (int iplot);
void   DyPlotTRFCTTIMEIni (int iplot);
void   DyProcessVect (struct DyCh *ch, int plottype);
void   DyProcessVectIni (struct DyCh *ch);
void   DyPutDefaultInPLOT1D (long id);
void   DyPutDefaultInPLOT1DTIME (long id);
void   DyPutDefaultInPLOT2D (long id);
void   DyPutDefaultInPLOTAUDIO (long id);
void   DyPutDefaultInPLOTCOHE (long id);
void   DyPutDefaultInPLOTCOHETIME (long id);
void   DyPutDefaultInPLOTFFT (long id);
void   DyPutDefaultInPLOTFFTTIME (long id);
void   DyPutDefaultInPLOTRAW (long id);
void   DyPutDefaultInPLOTRAWTIME (long id);
void   DyPutDefaultInPLOTTIME (long id);
void   DyPutDefaultInPLOTTR (long id);
void   DyPutDefaultInPLOTTRTIME (long id);
void   DyPutDefaultInPlot (long id);
void   DyRaz (long arg);
int    DyReadFirstFrame (char *chosen_file, long arg);
void   DyRemovePlot ();
void   DyResample(struct DyCh *ch, int plottype);
int    DySavePlot (char *savedPlots);
struct FrVect *DyGetSerDataVect(struct DyCh *ch, double gtime, double duration);
double DySetDefaultStartTime ();
double DySetDefaultEndTime ();
void   DySetDefaultColors (int flag);
void   DySetName();
void   DySetOnePlot (int plottype);
void   DySetOnePlotVarY ();
void   DySetPlot (FL_OBJECT *obj, long arg);
void   DySetRowCol (int ntot, int *nCol, int *nRow);
void   DySetTriggerList ();
void   DySetTriggerString ();
void   DySiestaLike (char *carfile);
void   DySortCmList ();
void   DySortDdList ();
void   DySortDdFileList ();
void   DySortShmList ();
void   DySoundFillHeader (struct soundhdr *hdr, long srate, long nBytes, short bits_per_sample);
void   DySoundReadHeader (char *buffer);
void   DyStart ();
void   DyStop ();
int    DySuperposePlot ();
double DyTimeFormatToGps (const char *timeformat);
void   DyUpdateChList ();
void   DyUpdateCmList ();
void   DyUpdateDataType (struct FrameH *frame);
void   DyUpdatePlotParam (struct FrameH *frame, int id);
void   DyUpdatePlotInput (int id);
void   DyUpdateShmList ();
double DyUTC2GPS (double utc);
void   DyVisuEnd ();
void   DyWindowsIni (long nwindows);
void   DyXformAddFile (FL_OBJECT *obj, long arg);
void   DyXformAddShm (FL_OBJECT *obj, long arg);
void   DyXformAddOneChannel (FL_OBJECT *obj, struct DyCh *opch, int i0);
void   DyXformAlignTime (FL_OBJECT *obj, long arg);
int    DyXformAtClose (FL_FORM *flForm, void *data);
void   DyXformAutoDeselect (FL_OBJECT *obj, long arg);
void   DyXformChannelFillBrowser ();
void   DyXformChDescription (FL_OBJECT *obj, long arg);
void   DyXformCheckGUI ();
void   DyXformChecks (FL_OBJECT *obj, long arg);
void   DyXformChRemove (FL_OBJECT *obj, long arg);
int    DyXformChSelect (char *chname);
void   DyXformChSort (FL_OBJECT *obj, long arg);
void   DyXformClonePlot (FL_OBJECT *obj, long arg);
void   DyXformClosePanel (FL_OBJECT *obj, long arg);
void   DyXformCmConnect (FL_OBJECT *obj, long arg);
void   DyXformCmFillBrowser ();
void   DyXformCmName (FL_OBJECT *obj, long arg);
void   DyXformCoffeePanel (FL_OBJECT *obj, long arg);
void   DyXformContinue (FL_OBJECT *obj, long arg);
void   DyXformDdFileFillBrowser ();
void   DyXformDdFillBrowser ();
void   DyXformDdGetChlist (FL_OBJECT *obj,long arg);
void   DyXformDdGetFile (FL_OBJECT *obj, long arg);
void   DyXformDeSelect (FL_OBJECT *obj, long arg);
void   DyXformEditPlot (FL_OBJECT *obj, long arg);
void   DyXformExchangeVar (FL_OBJECT *obj, long arg);
void   DyXformFileRemove (FL_OBJECT *obj, long arg);
void   DyXformFilterBrowser (FL_OBJECT *obj, long arg);
void   DyXformFindInBrowser (FL_OBJECT *obj, long arg);
void   DyXformGeneParToPlot (FL_OBJECT *obj, long arg);
void   DyXformGetButton (FL_OBJECT *obj, long arg);
void   DyXformGetChFromPlot (FL_OBJECT *obj, long arg);
void   DyXformGetConfigFile (FL_OBJECT *obj, long arg);
void   DyXformGetFirstLast (double *start, double *duration);
void   DyXformGetGeneParPanel (FL_OBJECT *obj, long arg);
int    DyXformGetMaxline (int browser);
void   DyXformGetOpString (FL_OBJECT *obj, long arg);
int    DyXformGetPasswd ();
void   DyXformGetPlotPanel (FL_OBJECT *, long);
void   DyXformGetRefPlot (FL_OBJECT *obj, long arg);
int    DyXformHelp (FL_OBJECT *ob, int, FL_Coord, FL_Coord, int, void *);
void   DyXformHelpAudio (FL_OBJECT *ob, long arg);
void   DyXformHelpCmConnect (FL_OBJECT *ob, long arg);
void   DyXformHelpDebug (FL_OBJECT *ob, long arg);
void   DyXformHelpSliders (FL_OBJECT *ob, long arg);
void   DyXformHelpMainBrowser (FL_OBJECT *ob, long arg);
void   DyXformHelpMainPanel (FL_OBJECT *ob, long arg);
void   DyXformHelpOptions (FL_OBJECT *ob, long arg);
void   DyXformHelpOutput (FL_OBJECT *ob, long arg);
void   DyXformHelpPanel (FL_OBJECT *obj, long arg);
void   DyXformHelpPlot1d (FL_OBJECT *ob, long arg);
void   DyXformHelpPlot1dtime (FL_OBJECT *ob, long arg);
void   DyXformHelpPlot2d (FL_OBJECT *ob, long arg);
void   DyXformHelpPlotGenePar (FL_OBJECT *ob, long arg);
void   DyXformHelpPlotfft (FL_OBJECT *ob, long arg);
void   DyXformHelpPlotffttime (FL_OBJECT *ob, long arg);
void   DyXformHelpPlotrawtime (FL_OBJECT *ob, long arg);
void   DyXformHelpPlottime (FL_OBJECT *ob, long arg);
void   DyXformHelpPlottrfct (FL_OBJECT *ob, long arg);
void   DyXformHelpPlotcohe (FL_OBJECT *ob, long arg);
void   DyXformHelpReffile (FL_OBJECT *ob, long arg);
void   DyXformHelpUserFct (FL_OBJECT *ob, long arg);
void   DyXformHideOneliner();
void   DyXformHideShowPlot(FL_OBJECT *obj, long arg);
void   DyXformIni (char **arg1,int *arg2);
void   DyXformLoadCompileMacro (FL_OBJECT *obj, long arg);
void   DyXformLoadInterpretMacro (FL_OBJECT *obj, long arg);
void   DyXformMakeNewMacro (FL_OBJECT *obj, long arg);
void   DyXformMenu (FL_OBJECT *obj, long arg);
void   DyXformNext (FL_OBJECT *obj, long arg);
void   DyXformNow (FL_OBJECT *obj, long arg);
void   DyXformOutputDefault (FL_OBJECT *obj, long arg);
void   DyXformOutputToPanel();
void   DyXformPadsConfig (FL_OBJECT *obj, long arg);
void   DyXformPanelToOutput (FL_OBJECT *obj, long arg);
void   DyXformPanelToPLOT1D (int id);
void   DyXformPanelToPLOT1DTIME (int id);
void   DyXformPanelToPLOT2D (int id);
void   DyXformPanelToPLOTAUDIO (int id);
void   DyXformPanelToPLOTCOHE (int id);
void   DyXformPanelToPLOTCOHETIME (int id);
void   DyXformPanelToPLOTFFT (int id);
void   DyXformPanelToPLOTFFTTIME (int id);
void   DyXformPanelToPLOTRAW (int id);
void   DyXformPanelToPLOTRAWTIME (int id);
void   DyXformPanelToPLOTTIME (int id);
void   DyXformPanelToPLOTTR (int id);
void   DyXformPanelToPLOTTRTIME (int id);
void   DyXformPanelToPlot (int id);
void   DyXformPause (FL_OBJECT *obj, long arg);
void   DyXformPlotDown (FL_OBJECT *obj, long arg);
void   DyXformPlotFillBrowser ();
void   DyXformPlotFillBrowser ();
void   DyXformPlotToPanel (long id);
void   DyXformPlotUp (FL_OBJECT *obj, long arg);
void   DyXformPutDefaultInAllPlots (FL_OBJECT *obj, long arg);
void   DyXformPutDefaultInPanel (FL_OBJECT *obj, long arg);
int    DyXformQuestion (char *dummy);
void   DyXformQuit (FL_OBJECT *obj, long arg);
int    DyXformQuitAtClose (FL_FORM *flForm, void *data);
void   DyXformRaisePanel (FL_OBJECT *obj, long arg);
void   DyXformRaz (FL_OBJECT *ob, long arg);
void   DyXformRemovePlot (FL_OBJECT *, long);
void   DyXformRefPlotEdit (FL_OBJECT *obj, long arg);
void   DyXformRefPlotFillBrowser ();
void   DyXformRefPlot2DRemove (FL_OBJECT *obj, long arg);
void   DyXforRefPlotRemove (FL_OBJECT *obj, long arg);
void   DyXformRemoveUserProcFiles ();
void   DyXformSuperposePlot (FL_OBJECT *obj, long arg);
void   DyXformSaveConfig (FL_OBJECT *obj, long arg);
void   DyXformSavePlot (FL_OBJECT *obj, long arg);
void   DyXformSaveRefPlot (FL_OBJECT *obj, long arg);
void   DyXformScanPlots (FL_OBJECT *obj, long arg);
void   DyXformSetButtons ();
void   DyXformSetClose ();
void   DyXformSetDebug ();
void   DyXformGetDebug (FL_OBJECT *ob, long arg);
void   DyXformSetDefaultColors (FL_OBJECT *obj, long arg);
void   DyXformSetDuration ();
void   DyXformSetFirstLast ();
void   DyXformSetGPSorDate (FL_OBJECT *obj, long arg);
void   DyXformSetMaxDuration (FL_OBJECT *obj, long arg);
void   DyXformSetOnePlot (FL_OBJECT *obj, long i);
void   DyXformSetOnePlotVarY (FL_OBJECT *obj, long arg);
void   DyXformSetDefaultOpString (FL_OBJECT *obj, long arg);
int    DyXformSetPlotType (FL_OBJECT *obj);
void   DyXformSetRefPlot (FL_OBJECT *obj, long arg);
void   DyXformSetStartTime (int uleaps);
void   DyXformSetScanStep (double step);
void   DyXformSetTrigger (FL_OBJECT *obj, long arg);
void   DyXformSetVar (FL_OBJECT *obj, long arg);
void   DyXformSetstartTime (FL_OBJECT *obj, long arg);
void   DyXformShowMessage (char *message);
void   DyXformShowOneliner ();
void   DyXformShowWriteAlarm ();
void   DyXformSortBrowser (FL_OBJECT *browser);
void   DyXformStart (FL_OBJECT *obj, long arg);
void   DyXformStop (FL_OBJECT *obj, long arg);
void   DyXformTransformPlot (FL_OBJECT *obj, long arg);
void   DyXformTrigger (int action);
void   DyXformUnsuperposePlot (FL_OBJECT *obj, long arg);
void   DyXformUpdateChList (FL_OBJECT *fl, long arg);
void   DyXformUpdateCmList (FL_OBJECT *obj, long arg);
void   DyXformUpdateCounters (double gtime, int uleaps);
void   DyXformUpdatePlotBrowser (long id, long add_or_remove);
void   DyXformUpdateShmList (FL_OBJECT *obj, long arg);
void   DyXformUpdateUserProcBrowser ();
void   DyXformUserFctHelp (FL_OBJECT *ob, long arg);
void   DyXformSlider (FL_OBJECT *obj, long arg);

void   FdShmClose (struct ShmInfo *shm, char *name);
long   FdShmCsmIni (struct ShmInfo *temp, long *shmiread);
void   FdShmFree (struct ShmInfo *shm, char *name);
long   FdShmGetIwrite (struct ShmInfo *shm);
long   FdShmGetnFrameMax (struct ShmInfo *shm);
void   FdShmInfoFree (struct ShmInfo *shm);
int    FdShmMakeKey (char *name);
void   FdShmSetDynamic (struct ShmInfo *shm, long dynamic);

struct FrVect *DyGetVect (struct DyCh *ch, double gtime, double duration);
struct FrameH *DyChListToFrame (struct FrameH *frame);
struct FrameH *DyCharChListToFrame (char *chList);
struct FrameH *DyOutputReadCache (int gtime, char *tag);
struct FrameH *DyWaveToFrame (char *wavefilename, double framedt);
struct FrameH *FdShmRead (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes);
struct FrameH *FdShmReadDynamic (struct ShmInfo *shm, long *shmiread, long compress, int *nBytes);
struct ShmInfo *FdShmOpen (char *idName, char *name);

long is_a_dir (char *arg);
long is_a_file (char *arg);
char *my_strcpy (char *dummy, char *str);
void quitHandler (int arg);


/**************************/
/*   Declarations extern  */
/**************************/

extern struct DyTool* Dy;
extern struct DyPlot* Plot[NPLOT_MAX];
extern struct FrVect *UFrCurVect;
extern struct FrameH *inputFrame;
extern int    inputFrameSize;
extern struct soundhdr wavehdr; 
extern struct timeval dytv; 
extern double ptime[DBGTIME_LEVELMAX+1];
extern char   fctnamep[DBGTIME_LEVELMAX+1][256];
 
extern FD_dysliders       *dysliders;
extern FD_mainbrowser     *mainbrowser;
extern FD_mainpanel       *mainpanel;
extern FD_userfuncbrowser *userfuncbrowser;
extern FD_plotaudio       *plotaudio;
extern FD_plotgenepar     *plotgenepar;
extern FD_plot1d          *plot1d;
extern FD_plot2d          *plot2d;
extern FD_plotraw         *plotraw;
extern FD_plottime        *plottime;
extern FD_plotfft         *plotfft;
extern FD_plotffttime     *plotffttime;
extern FD_plottrfcttime   *plottrfcttime;
extern FD_plotcohetime    *plotcohetime;
extern FD_plot1dtime      *plot1dtime;
extern FD_plotrawtime     *plotrawtime;
extern FD_plottrfct       *plottrfct;
extern FD_plotcohe        *plotcohe;
extern FD_getdd           *getdd;
extern FL_FORM            *helppanel;
extern FL_FORM            *coffeepanel;
extern FL_FORM            *userprochelp;
extern FD_cmservers       *cmservers;
extern FD_getshm          *getshm;
extern FD_dyhelp          *dyhelp;
extern FD_dydebug         *dydebug;
extern FD_dyoptions       *dyoptions;
extern FD_dyoutput        *dyoutput;
extern FD_reffile         *reffile;

#ifdef __cplusplus
}
#endif

#endif
