
/**************************************************************************/
/*                                                                        */
/* File: fselector.c                                                      */
/* Copyright (C) 2003, T.C. Zhao (Xform), L. Fournier, D. Verkindt        */
/* For the licensing terms see the LICENSE file.                          */
/*                                                                        */
/**************************************************************************/

/*********************************************************************/
/* basic dialog boxes                                                */
/* - message/alert/info/question                                     */
/* - file selector                                                   */
/*********************************************************************/
/* Origin: v3r04 of Cl package - 9.2.2002 - creation                 */
/* Modified:  28.3.2004 - for dataDisplay usage (v8 and after)       */
/* - selecteur de fichier ClXFSelector                               */
/*********************************************************************/

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
/*
#include <forms.h>
*/

#include "fselector.h"
#include "dataDisplay.h"

/**************************************************************/
/* externals */
/* from ulib.h */
extern char *fl_de_space_de(char *);
extern const char *fl_get_syserror_msg(void);

/* from flinternal.h */
extern void fl_object_qenter(FL_OBJECT *);

/**************************************************************/
/* character font dimension in pixels */
#define CL_FONT_SIZE            14

/* width between objects */
#define CL_SEP_WIDTH                (int)(CL_FONT_SIZE/3.0)
#define CL_SEP_HEIGHT               (int)(CL_FONT_SIZE/3.0)

/* button dimensions */
#define CL_BTN_WIDTH                (int)(9.6*CL_FONT_SIZE)
#define CL_BTN_HEIGHT               (int)(2.4*CL_FONT_SIZE)

/* input box dimensions */
#define CL_INP_WIDTH                (int)(12.6*CL_FONT_SIZE)
#define CL_INP_HEIGHT               (int)(2.4*CL_FONT_SIZE)

/* label dimensions */
#define CL_LBL_WIDTH                (int)(60.0*CL_FONT_SIZE)
#define CL_LBL_HEIGHT               (int)(2.4*CL_FONT_SIZE)

/* default object dimensions */
#define CL_OBJ_WIDTH                (int)(7.2*CL_FONT_SIZE + CL_SEP_WIDTH)
#define CL_OBJ_HEIGHT               (int)(2.4*CL_FONT_SIZE + CL_SEP_HEIGHT)

/* form client dimensions */
#define CL_CLN_WIDTH(form)          ((form)->w - 2*CL_SEP_WIDTH)
#define CL_CLN_HEIGHT(form)         ((form)->h - 2*CL_SEP_HEIGHT)

/* default message box timeout */
#define CL_XMSG_TIMEOUT             30000

/* default message length */
#define CL_MSG_LEN                  4096

/**************************************************************/
/* dynamic memory allocation */
#define ClPFree(p) \
  { if (p) { free((p)); p=NULL; } }
#define ClStrdup(p, s) \
  { if (p) free((p)); if (s) { p=strdup((s)); } else p=s; }
#define ClMalloc(p, size) \
  { if (p) free((p)); p=malloc((size)); }
#define ClCalloc(p, size, elSize) \
  { if (p) free(p); p=calloc((size), (elSize)); }
#define ClRealloc(p, newSize) \
  { p=realloc((p), (newSize)); }
#define ClStrlen(p) \
  (p?strlen(p):0);

/**************************************************************/
/* when not integrated with Cfg */
#define CFG_INFO 0

/**************************************************************/
/* internal debug level - set to 1 to activate debug messages */
static int goodie_debug=1;

/**************************************************************
 * common message function - TO BE CONNECTED TO CFGVIRGO
 **************************************************************/
static int goodie_msg(int errLvl, int dspLvl, char *tag, const char *fmt, ...) {
  char msg[CL_MSG_LEN];
  va_list args;

  if (!goodie_debug && errLvl==CFG_INFO)
    return(0);

  va_start(args, fmt);
  vsprintf(msg, fmt, args);
  va_end(args);
  return(fprintf(stderr, "INFO-%s\n", msg));
} /* ClXFormMsg */

/**************************************************************
 * preemptive handler to work around some window manager bugs
 * where iconification causes window to close. For normal closing,
 * we do not see any unmap events as winclose eats it
 **************************************************************/
/**
static int ClXGoodiesPreemptive(FL_FORM *form, void *ev) {
  XEvent *xev=(XEvent *)ev;

  if ((xev->type==UnmapNotify) && (form->u_vdata)) {
    fl_trigger_object(form->u_vdata);
    return(FL_PREEMPT);
  }

  return(!FL_PREEMPT);
} 
**/
/* ClXGoodiesPreemptive */

/******************************************************************
 *
 ******************************************************************/
/**
static int ClXGoodiesAtclose(FL_FORM *form, void *data) {
  fl_trigger_object(data?data:form->u_vdata);
  if (form->sort_of_modal)
    form->sort_of_modal=0;

  return FL_IGNORE;
}
**/
 /* ClXGoodiesAtclose */

/******************************************************************
 *
 ******************************************************************/
void ClXGoodiesParseLabel(FL_OBJECT *obj, const char *name) {
  const char *s=fl_get_resource(name, 0, FL_NONE, 0, 0, 0);

  if (s)
    fl_set_object_label(obj, s);
} /* ClXGoodiesParseLabel */

/******************************************************************
 *
 ******************************************************************/
void ClXGoodiesGetTitle(FL_FORM *form, const char *res) {
  const char *s=fl_get_resource(res, 0, FL_NONE, 0, 0, 0);
  if (s)
    fl_set_form_title(form, s);
} /* ClXGoodiesGetTitle */

/**************************************************************
 * adjust object size
 **************************************************************/
static FL_OBJECT *adjust_object_size(FL_OBJECT *obj) {

  if (obj) {
    int w, h;
    const char *text=fl_get_object_label(obj);
    fl_get_string_dimension(FL_NORMAL_STYLE, CL_FONT_SIZE, text, strlen(text),
      &w, &h);
    if (obj->w<w)
      fl_set_object_size(obj, w, obj->h);
    if (obj->h<h)
      fl_set_object_size(obj, obj->w, h);
  }

  return(obj);
} /* adjust_object_size */

/**************************************************************
 * COMMON MESSAGE FORM FUNCTIONS
 **************************************************************/
/* form cleanup, data=NULL */
static int ClXMsgAtCloseCallback(FL_FORM *form, void *data) {

  goodie_msg(0, CFG_INFO, "", "XMsg>AtCloseCallback>\"%s\" timeout #%ld",
    form->label, form->u_ldata);
  if (form->u_vdata)
    fl_object_qenter((FL_OBJECT *)form->u_vdata);
  if (form->u_ldata)
    fl_remove_timeout((int)form->u_ldata);
  fl_activate_all_forms();

  return(FL_OK);
} /* ClXMsgAtCloseCallback */

/**************************************************************/
/* raw preemptive callback */
static int ClXMsgRawCb(FL_FORM * form, void *ev) {
  XEvent *xev=(XEvent *)ev;

  if ((xev->type==UnmapNotify)
   || (xev->type==ClientMessage)) {
    goodie_msg(0, CFG_INFO, "", "XMsg>RawCallback>");
    if (form->u_vdata)
      fl_trigger_object((FL_OBJECT *)form->u_vdata);
    return FL_PREEMPT;
  }
  return (!FL_PREEMPT);
} /* ClXMsgRawCb */

/**************************************************************/
/* timeout : queues default button (no callback defined !) */
static void ClXMsgTimeout(int timeoutId, void *data) {
  FL_FORM *form=(FL_FORM *)data;

  goodie_msg(0, CFG_INFO, "", "XMsg>Timeout>#%ld", timeoutId);
  form->u_ldata=0;
  if (form->u_vdata)
    fl_trigger_object((FL_OBJECT *)form->u_vdata);
  else {
    fl_hide_form(form);
    fl_free_form(form);
  }

} /* ClXMsgTimeout */

/**************************************************************/
/* OK button callback function, data=0 */
static void ClXMsgOKCallback(FL_OBJECT *obj, long data) {

  obj->form->u_vdata=(void *)NULL;
  fl_hide_form(obj->form);
  fl_free_form(obj->form);
} /* ClXMsgOKCallback */

/**************************************************************/
/* create a modal dialog box */
static FL_FORM *ClXMsgCreate(char *title, const char *fmt, va_list args) {
  FL_FORM *form;
  FL_OBJECT *obj=(FL_OBJECT *)NULL;
  int mWidth=(2*CL_SEP_WIDTH+3*CL_BTN_WIDTH);
  int mHeight=(2*CL_SEP_HEIGHT+16*CL_BTN_HEIGHT);
  int x=CL_SEP_WIDTH, y=CL_SEP_HEIGHT;
  char msg[CL_MSG_LEN];
  char *msgPtr;

  /* modal dialog box except for info boxes */
  if (strncmp(title, "info", 4)!=0)
    fl_deactivate_all_forms();

  form=fl_bgn_form(FL_BORDER_BOX, mWidth, mHeight);
  mHeight=(2*CL_SEP_HEIGHT+1*CL_BTN_HEIGHT);
  vsprintf(msg, fmt, args);
  goodie_msg(0, CFG_INFO, "", "XMsg>Create>height=%d", mHeight);

  for (msgPtr=strtok(msg, "\r\n") ; msgPtr ; msgPtr=strtok(NULL, "\r\n")) {
    obj=fl_add_text(FL_NORMAL_TEXT, x, y, mWidth, CL_LBL_HEIGHT, msgPtr);
    fl_set_object_lsize(obj, CL_FONT_SIZE);
    fl_set_object_lalign(obj, FL_ALIGN_CENTER);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthEast);
    if (obj) {
      adjust_object_size(obj);
      if (mWidth < obj->w+2*CL_SEP_HEIGHT)
        mWidth=obj->w+2*CL_SEP_HEIGHT;
      mHeight+=obj->h;
      y+=obj->h;
    }
    goodie_msg(0, CFG_INFO, "", "XMsg>Create>height=%d", mHeight);
  }
  fl_set_form_size(form, mWidth, mHeight);

  return(form);
} /* ClXMsgCreate */

/**************************************************************/
static FL_FORM *ClXMsgDoForm(FL_FORM *form, char *title, int center) {

  if (form) {
    int x=(form->w-CL_BTN_WIDTH)/2 + 0;
    int y=form->h-CL_BTN_HEIGHT-CL_SEP_HEIGHT;
    FL_OBJECT *flOk=fl_add_button(FL_RETURN_BUTTON, x, y, CL_BTN_WIDTH, CL_BTN_HEIGHT, "OK");

    fl_set_object_lsize(flOk, CL_FONT_SIZE);
    fl_set_form_hotobject(form, flOk);
    form->u_vdata=(void *)flOk;

    fl_register_raw_callback(form, FL_ALL_EVENT, ClXMsgRawCb);
    fl_end_form();
    fl_set_form_atclose(form, ClXMsgAtCloseCallback, NULL);
    fl_show_form(form, center?FL_PLACE_CENTER:FL_PLACE_HOTSPOT, FL_TRANSIENT, title);
    form->u_ldata=fl_add_timeout(CL_XMSG_TIMEOUT, ClXMsgTimeout, (void *)form);

    /* modal dialog box */
    do {      
    } while ((fl_do_only_forms()!=flOk));

    goodie_msg(0, CFG_INFO, "", "XMsg>DoForm>end");
    fl_activate_all_forms();
    fl_hide_form(form);
    fl_free_form(form);
    form=(FL_FORM *)NULL;
  }

  return(form);
} /* ClXDoMsgForm */

/**************************************************************
 * INFO/MESSAGE/ALERT/QUESTION DIALOG BOXES
 **************************************************************/
/* information dialog box is NOT modal */
void ClXInfo(const char *fmt, ...) {
  FL_FORM *form;
  va_list args;
  char *title="info";

  va_start(args, fmt);
  if ((form=ClXMsgCreate(title, fmt, args))) {
    int x=(form->w - CL_BTN_WIDTH)/2 + 0;
    int y=form->h -CL_BTN_HEIGHT -CL_SEP_HEIGHT;
    FL_OBJECT *flOk=fl_add_button(FL_RETURN_BUTTON, x, y, CL_BTN_WIDTH, CL_BTN_HEIGHT,
      "OK");
    fl_set_object_lsize(flOk, CL_FONT_SIZE);
    fl_set_object_callback(flOk, ClXMsgOKCallback, 0);
    fl_set_form_hotobject(form, flOk);
    form->u_vdata=(void *)flOk;

    fl_register_raw_callback(form, FL_ALL_EVENT, ClXMsgRawCb);
    fl_end_form();
    fl_set_form_atclose(form, ClXMsgAtCloseCallback, NULL);
    fl_show_form(form, FL_PLACE_HOTSPOT, FL_TRANSIENT, title);
    form->u_ldata=fl_add_timeout(CL_XMSG_TIMEOUT, ClXMsgTimeout, (void *)form);
  }
  va_end(args);

} /* ClXInfo */

/**************************************************************/
void ClXMessage(const char *fmt, ...) {
  va_list args;

  va_start(args, fmt);
  ClXMsgDoForm(ClXMsgCreate("message", fmt, args), "message", 0);
  va_end(args);

} /* ClXMessage */

/**************************************************************/
void ClXAlert(const char *fmt, ...) {
  va_list args;

  va_start(args, fmt);
  ClXMsgDoForm(ClXMsgCreate("error", fmt, args), "error", 1);
  va_end(args);

} /* ClXAlert */

/**************************************************************
 * QUESTION
 **************************************************************/
int ClXQuestion(int def, const char *fmt, ...) {
  char *title="question";
  FL_FORM *form;
  FL_OBJECT *flOk=NULL;
  FL_OBJECT *flCancel=NULL;
  FL_OBJECT *obj=NULL;
  va_list args;

  va_start(args, fmt);
  if ((form=ClXMsgCreate(title, fmt, args))) {
    int x=(form->w - CL_BTN_WIDTH*2)/3 + 0;
    int y=form->h -CL_BTN_HEIGHT -CL_SEP_HEIGHT;
    flOk=fl_add_button(FL_RETURN_BUTTON, x, y, CL_BTN_WIDTH, CL_BTN_HEIGHT, "OK");
    fl_set_object_lsize(flOk, CL_FONT_SIZE);
    x+=(form->w - CL_BTN_WIDTH*2)/3 + CL_BTN_WIDTH;
    flCancel=fl_add_button(FL_NORMAL_BUTTON, x, y, CL_BTN_WIDTH, CL_BTN_HEIGHT, "Cancel");
    fl_set_object_lsize(flCancel, CL_FONT_SIZE);

    fl_set_form_hotobject(form, (def ? flOk : flCancel));
    form->u_vdata=(void *)flCancel;
    form->u_ldata=fl_add_timeout(CL_XMSG_TIMEOUT, ClXMsgTimeout, (void *)form);
    fl_register_raw_callback(form, FL_ALL_EVENT, ClXMsgRawCb);
    fl_end_form();
    fl_set_form_atclose(form, ClXMsgAtCloseCallback, NULL);
    fl_show_form(form, FL_PLACE_HOTSPOT, FL_TRANSIENT, title);

    /* modal dialog box */
    do {
      obj=fl_do_only_forms();
    } while ((obj!=flOk) && (obj!=flCancel));

    goodie_msg(0, CFG_INFO, "", "X>ShowQuestion>end with object %s",
      obj->label);
    fl_hide_form(form);
    fl_free_form(form);
    fl_activate_all_forms();
    form=(FL_FORM *)NULL;
  }

  va_end(args);
  return(obj==flOk);
} /* ClXQuestion */

/**************************************************************
 * FILE SELECTOR
 **************************************************************/
/* File selector */
static int dirmarker='D';	/* directory marker     */
static int bdevmarker='b';	/* special file marker  */
static int cdevmarker='c';	/* special file marker  */
static int fifomarker='p';	/* pipe marker          */
static int sockmarker='s';	/* socket marker        */
static int listdirfirst=1;	/* list directory first */

/**************************************************************/
typedef struct ClXFSelector_s {
  int timeoutId;                /* timeout identifier */
  FL_FORM *form;                /* dialog form */
  FL_OBJECT *OkBtn;             /* default button */
  /* dialog data */
  int rescan;			/* if update dir cache  */
  char retname[1024];           /* complete filename */
  char dname[1024];             /* current path */
  char filename[1024];          /* bare filename */
  char pattern[1024];           /* file pattern */
  /* objects */
  FL_OBJECT *patternInp;        /* file pattern input */
  FL_OBJECT *directoryInp;      /* directory input */
  FL_OBJECT *fileInp;           /* file name input */
  FL_OBJECT *fileBrs;           /* file list */
  FL_OBJECT *rescanBtn;         /* rescan button */
  FL_OBJECT *cancelBtn;         /* cancel button */
} ClXFSelector_t;

/**************************************************************/
/* form close event handler for dlgForm, data=dlg */
static int ClXFSelectorAtCloseCb(FL_FORM *form, void *data) {
  ClXFSelector_t *dlg=(ClXFSelector_t *)data;

/***
  goodie_msg(0, CFG_INFO, "", "XFSelector>AtCloseCb>\"%s\"", form->label);
***/

  if (dlg->timeoutId)
    fl_remove_timeout(dlg->timeoutId);
  ClPFree(form->u_vdata); /* set dialog pointer to NULL */

  return(FL_OK);
} /* ClXFSelectorAtCloseCb */

/**************************************************************
 * preemptive handler to work around some window mangers bugs
 * where iconification causes window to close. For normal closing,
 * we don't see any unmap events as winclose eats it
 **************************************************************/
static int ClXFSelectorRawCb(FL_FORM *form, void *ev) {
  XEvent *xev=(XEvent *)ev;

  if ((xev->type==UnmapNotify)
   || (xev->type==ClientMessage)) {
    ClXFSelector_t *dlg=(ClXFSelector_t *)form->u_vdata;
    goodie_msg(0, CFG_INFO, "", "XFSelector>RawCallback>%s",
      xev->type==UnmapNotify?"UnmapNotify":"ClientMessage");
    if (dlg->cancelBtn)
      fl_trigger_object(dlg->cancelBtn);
    return FL_PREEMPT;
  }
  return (!FL_PREEMPT);
} /* ClXFSelectorRawCb */

/******************************************************************
 *
 ******************************************************************/
static char *del_tail_slash(char *d) {
  int i=strlen(d)-1;

  if (d[i]=='/')
    d[i]='\0';
  return d;
} /* del_tail_slash */

/******************************************************************
 *
 ******************************************************************/
static char *append_slash(char *d) {
  int n=strlen(d);

  if (d[n-1]!='/') {
    d[n++]='/';
    d[n]='\0';
  }
  return d;
} /* append_slash */

/******************************************************************
 * Contract dirname from /usr/people/xxx to ~/ to save some length.
 * Also replace the middle part of a path with X if the path is longer
 * than limit. Useful when indicator is of limited length
 ******************************************************************/
/**
static void pat_replace(char str[], const char *pat, const char *rep) {
  char *t=strstr(str, pat);
  char *d=NULL;

  if (!t) return;
  ClStrdup(d,t+strlen(pat));

  *t='\0';
  strcat(str, rep);
  strcat(str, d);
  free(d);
}
**/

/******************************************************************/
/**
static const char *contract_dirname(ClXFSelector_t *dlg, const char *dir, int limit) {
  static char buf[FL_PATH_MAX];
  const char *home=getenv("HOME");
  char *p, *q;
  FL_OBJECT *ob=dlg->directoryInp;
  int l, len, strw, strh;

  fl_get_string_dimension(ob->lstyle, ob->lsize, dir, strlen(dir), &strw, &strh);
  if (strw < ob->w-4)
    return dir;

  strcpy(buf, dir);
  if (home)
    pat_replace(buf, home, "~");

  fl_get_string_dimension(ob->lstyle, ob->lsize, buf, strlen(buf), &strw, &strh);
  if (strw < ob->w)
    return buf;

  if ((l=strlen(buf))>limit) {
    int k=limit/3-3;

    p=strchr(buf+k, '/');
    q=buf+l-k;
    while ((*q!='/') && (q!=buf))
      q--;

    if (q>p+3) {
      *++p='.';
      *++p='.';
      *++p='.';
      *++p='\0';
      strcpy(p, q);
    }
  }

  if (strlen(buf)>limit) {
    p=buf+limit-3;
    *p++='.';
    *p++='.';
    *p++='.';
    *p='\0';
  }

  fl_get_string_dimension(ob->lstyle, ob->lsize, buf, strlen(buf), &strw, &strh);
  for (len=strlen(buf); len>0 && strw > ob->w-2; len--) {
    buf[len-1]='\0';
    fl_get_string_dimension(ob->lstyle, ob->lsize, buf, strlen(buf), &strw, &strh);
  }

  return(buf);
}
**/
 /* contract_dirname */

/**************************************************************/
static void add_browser_line(FL_OBJECT *obj, const char *fmt, ...) {
  if (obj && fmt) {
    va_list args;
    char buf[4096];

    fl_freeze_form(obj->form);
    va_start(args, fmt);
    vsprintf(buf, fmt, args);
    va_end(args);
    fl_add_browser_line(obj, buf);
    fl_unfreeze_form(obj->form);
  }
} /* add_browser_line */

/******************************************************************
 * Show all files in directory dname that match selection pattern.
 * Note fn indicates if we should highlight the current selection.
 ******************************************************************/
static int fill_entries(FL_OBJECT * br, const char *fn, int show) {
  FL_FORM *form=br->form;
  ClXFSelector_t *dlg=(ClXFSelector_t *)br->form->u_vdata;
  const FL_Dirlist *dirlist, *dl;
  int i, nEntries;

  if (!dlg)
    return(-1);

/***
  goodie_msg(0, CFG_INFO, "", "XFSelector>fill_entries>");
***/
  fl_set_cursor(form->window, XC_watch);

  if (!(dirlist=fl_get_dirlist(dlg->dname, dlg->pattern, &nEntries, dlg->rescan))) {
    char *p=strrchr(dlg->dname, '/');

    ClXAlert("Cannot read directory %s\n%s", dlg->dname, fl_get_syserror_msg());
    /* backup */
    if (p)
      *p='\0';
    fl_reset_cursor(form->window);
    return(-1);
  }

  fl_freeze_form(form);
/**
  fl_set_input(dlg->directoryInp, contract_dirname(dlg, dlg->dname, 38));
**/
  fl_set_input(dlg->directoryInp, dlg->dname);
  fl_clear_browser(br);

  /* list directories first */
  if (listdirfirst)
    for (dl=dirlist; dl->name; dl++)
      if (dl->type==FT_DIR)
        add_browser_line(br, "%c %s", dirmarker, dl->name);

  /* list all other entries */
  for (i=0, dl=dirlist; dl->name; i++, dl++) {
    if (dl->type==FT_DIR) {
      if (!listdirfirst)
        add_browser_line(br, "%c %s", dirmarker, dl->name);
    }
/**
    else if (dl->type==FT_LINK)
      add_browser_line(br, "%c %s", dirmarker, dl->name);
**/
    else if (dl->type==FT_FIFO)
      add_browser_line(br, "%c %s", fifomarker, dl->name);
    else if (dl->type==FT_SOCK)
      add_browser_line(br, "%c %s", sockmarker, dl->name);
    else if (dl->type==FT_BLK)
      add_browser_line(br, "%c %s", bdevmarker, dl->name);
    else if (dl->type==FT_CHR)
      add_browser_line(br, "%c %s", cdevmarker, dl->name);
    else {
      add_browser_line(br, "  %s", dl->name);
      if (fn && strcmp(dl->name, fn)==0) {
        int nLines=fl_get_browser_maxline(br);
        fl_select_browser_line(br, nLines);
        if (show) {
          int top=fl_get_browser_topline(br);
          int total=fl_get_browser_screenlines(br);
          if ((nLines < top) || (nLines > (top+total-1)))
            fl_set_browser_topline(br, nLines-total/2);
        }
      }
    }
  }

  fl_unfreeze_form(form);
  fl_reset_cursor(form->window);
  return(0);
} /* fill_entries */

/******************************************************************
 * directory input callback
 ******************************************************************/
static void ClXFSelectDirectoryInpCb(FL_OBJECT * ob, long data) {
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
  const char *p=fl_get_input(ob);
/***
  goodie_msg(0, CFG_INFO, "", "XFSelect>DirectoryInpCb>");
***/
  if (!p) return;
  if (strlen(p)<1) return;

  if (dlg && dlg->dname && p) {
    char tmpdir[FL_PATH_MAX + 2];

    fl_de_space_de(strcpy(tmpdir, p));
    if (strcmp(tmpdir, dlg->dname)==0)
      return;

    if (p[strlen(p)-1]=='/')
      return;

    fl_fix_dirname(tmpdir);
    if (!fl_is_valid_dir(tmpdir))
      return;

    strcpy(dlg->dname, tmpdir);
    if (fill_entries(dlg->fileBrs, (char *)NULL, 1) < 0)
       {del_tail_slash(dlg->dname);}
    else
       {fl_set_input(dlg->directoryInp, dlg->dname);}

/**
    else
      fl_set_input(dlg->directoryInp, contract_dirname(dlg, dlg->dname, 38));
**/
  }

  return;
} /* ClXFSelectDirectoryInpCb */

/******************************************************************
 * pattern button callback
 ******************************************************************/
static void ClXFSelectPatternInpCb(FL_OBJECT * ob, long q) {
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
  const char *s=fl_get_input(dlg->patternInp);
/***
  goodie_msg(0, CFG_INFO, "", "XFSelect>PatternInpCb>");
***/
  if (dlg && dlg->pattern && s && strcmp(dlg->pattern, s)!=0) {
    strcpy(dlg->pattern, s);
/*
    fl_set_input(dlg->patternInp, dlg->pattern);
*/
    fill_entries(dlg->fileBrs, dlg->filename, 1);
  }

  return;
} /* ClXFSelectPatternInpCb */

/******************************************************************
 * file input callback
 ******************************************************************/
static void ClXFSelectFileInpCb(FL_OBJECT * ob, long data) {
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
  const char *s=fl_get_input(dlg->fileInp);
/***
  goodie_msg(0, CFG_INFO, "", "XFSelect>FileInpCb>");
***/
  if (dlg)
    strcpy(dlg->filename, s);
} /* ClXFSelectFileInpCb */

/******************************************************************
 * rescan button callback
 ******************************************************************/
static void ClXFSelectRescanBtnCb(FL_OBJECT * ob, long arg) {
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
/***
  goodie_msg(0, CFG_INFO, "", "XFSelect>RescanBtnCb>");
***/
  if (dlg) {
    dlg->rescan=1;
    fill_entries(dlg->fileBrs, dlg->filename, 1);
    dlg->rescan=0;
  }
} /* ClXFSelectRescanBtnCb */

/******************************************************************
 * file list browser callback
 ******************************************************************
 * generate OK if valid selection (e.g. double-clicked)
 * Note that if another callback is defined, never generates OK
 ******************************************************************/
static void ClXFSelectFileBrsDblclickCb(FL_OBJECT *ob, long data) {
  int sLine=fl_get_browser(ob);
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
/***
  goodie_msg(0, CFG_INFO, "", "XFSelector>DblclickCb>%s - line %d selected",
    ob->label, sLine);
***/
  if (sLine < 0) sLine = -sLine;

  if ((dlg) && (sLine > 0)) {
    char *seltext=(char *)fl_get_browser_line(ob, sLine);
    int dir=((seltext[0]==dirmarker) && (seltext[1]==' '));

    seltext+=2;
    if (dir) {
      strcat(append_slash(dlg->dname), seltext);
      fl_fix_dirname(dlg->dname);
      if (fill_entries(dlg->fileBrs, (char *)NULL, 0)<0)
        del_tail_slash(dlg->dname);
      fl_set_input(dlg->fileInp, "");
    }
    else {
      strcpy(dlg->filename, seltext);
      fl_set_input(dlg->fileInp, seltext);
      fl_object_qenter(dlg->OkBtn);
    }
  }
} /* ClXFSelectFileBrsDblclickCb */

/******************************************************************
 * file list browser callback
 ******************************************************************
 * generate OK if valid selection (e.g. double-clicked)
 * Note that if another callback is defined, never generates OK
 ******************************************************************/
static void ClXFSelectFileBrsCb(FL_OBJECT * ob, long arg) {
  struct timespec tv;
  ClXFSelector_t *dlg=(ClXFSelector_t *)ob->form->u_vdata;
  int sLine=fl_get_browser(ob);
/***
  goodie_msg(0, CFG_INFO, "", "XFSelect>FileBrsCb>");
***/
  tv.tv_sec = 0;
  tv.tv_nsec = 200000000;
  nanosleep(&tv,NULL);

  if ((dlg) && (sLine > 0)) {
    char *seltext=(char *)fl_get_browser_line(ob, sLine);
    int dir=((seltext[0]==dirmarker) && (seltext[1]==' '));

    seltext+=2;
/*hihi*/
    if (dir) {
      strcat(append_slash(dlg->dname), seltext);
      fl_fix_dirname(dlg->dname);
      if (fill_entries(dlg->fileBrs, (char *)NULL, 0)<0)
        del_tail_slash(dlg->dname);
      fl_set_input(dlg->fileInp, "");
    }
    else {
      strcpy(dlg->filename, seltext);
      fl_set_input(dlg->fileInp, seltext);
    }
  }
} /* ClXFSelectFileBrsCb */



/**************************************************************
 * FILE SELECTOR
 **************************************************************/
int ClXFSelector(char *title, const char *directory,
                 const char *pattern, const char *filename, char *result, int size) {
  const char *tmp;
  char seltext[FL_PATH_MAX];
  FL_OBJECT *obj=(FL_OBJECT *)NULL;
  int x=CL_SEP_WIDTH, y=CL_SEP_HEIGHT;
  int w=CL_BTN_WIDTH+3*CL_INP_WIDTH+3*CL_SEP_WIDTH; /*--- width of the fselector panel  ---*/
  int h=CL_OBJ_HEIGHT*16+CL_SEP_HEIGHT;             /*--- height of the fselector panel ---*/
  int nFile, nSelected, length, i;
  ClXFSelector_t *dlg=(ClXFSelector_t *)NULL;

  /*--- Create the file selector panel ---*/

  if (!title) title="file selector";
  ClCalloc(dlg, sizeof(ClXFSelector_t), 1);
  dlg->form=fl_bgn_form(FL_BORDER_BOX, w, h);
  dlg->form->u_vdata=(void *)dlg;
/**
  goodie_msg(0, CFG_INFO, "", "XFSelector>Init>");
**/

  obj = fl_add_box(FL_UP_BOX,0,0,w,h,"");
  dlg->directoryInp=obj;
/*
  fl_set_object_color(obj,FL_DARKCYAN,FL_COL1);
*/
  fl_set_object_color(obj,FL_PALEGREEN,FL_COL1);

  x=CL_SEP_WIDTH+CL_BTN_WIDTH;
  obj=fl_add_input(FL_NORMAL_BUTTON, x, y, CL_CLN_WIDTH(dlg->form)-x, CL_INP_HEIGHT, "directory");
  dlg->directoryInp=obj;
  fl_set_object_callback(obj, ClXFSelectDirectoryInpCb, 0);
  fl_set_input_color(obj, FL_BLACK, FL_BLUE);
  fl_set_input_return(obj, FL_RETURN_ALWAYS);
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj, FL_WHITE, FL_WHITE);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_gravity(obj, NorthWestGravity, NorthEastGravity);
  y+=obj->h+CL_SEP_HEIGHT;

  x=CL_SEP_WIDTH+CL_BTN_WIDTH;
  obj=fl_add_input(FL_NORMAL_BUTTON, x, y, CL_CLN_WIDTH(dlg->form)-x, CL_INP_HEIGHT, "pattern");
  dlg->patternInp=obj;
  fl_set_input_color(obj, FL_BLACK, FL_BLUE);
  fl_set_input_return(obj, FL_RETURN_ALWAYS);
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_LEFT_BCOL,FL_LEFT_BCOL);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_callback(obj, ClXFSelectPatternInpCb, 0);
  fl_set_object_gravity(obj, NorthWestGravity, NorthEastGravity);
  y+=obj->h+CL_SEP_HEIGHT;

  x=CL_SEP_WIDTH+CL_BTN_WIDTH;
  obj=fl_add_input(FL_NORMAL_INPUT, x, y, CL_CLN_WIDTH(dlg->form)-x, CL_INP_HEIGHT, "filename");
  dlg->fileInp=obj;
  fl_set_input_color(obj, FL_BLACK, FL_BLUE);
  fl_set_input_return(obj, FL_RETURN_ALWAYS);
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_LEFT_BCOL,FL_LEFT_BCOL);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_callback(obj, ClXFSelectFileInpCb, 0);
  fl_set_object_gravity(obj, NorthWestGravity, NorthEastGravity);
  y+=obj->h+CL_SEP_HEIGHT;

  x=CL_SEP_WIDTH;
  obj=fl_add_browser(FL_MULTI_BROWSER, x, y, CL_CLN_WIDTH(dlg->form), CL_CLN_HEIGHT(dlg->form)-y-CL_BTN_HEIGHT, "file browser");
  dlg->fileBrs=obj;
  fl_set_browser_fontstyle (obj, FL_TIMES_STYLE);
  fl_set_browser_fontsize(obj, CL_FONT_SIZE);
  fl_set_browser_dblclick_callback(obj, ClXFSelectFileBrsDblclickCb, 0);
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_LEFT_BCOL,FL_GREEN);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_callback(obj, ClXFSelectFileBrsCb, 0);
  fl_set_object_gravity(obj, NorthWestGravity, SouthEastGravity);

  y=dlg->fileBrs->y+dlg->fileBrs->h+CL_SEP_HEIGHT;
  x=CL_SEP_WIDTH;
  obj=fl_add_button(FL_NORMAL_BUTTON, x, y, CL_CLN_WIDTH(dlg->form)/7, CL_BTN_HEIGHT, "Rescan");
  dlg->rescanBtn=obj;
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_WHEAT,FL_COL1);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_callback(obj, ClXFSelectRescanBtnCb, 0);
  fl_set_object_gravity(obj, SouthEastGravity, SouthEastGravity);
  x+=3*obj->w;

  obj=fl_add_button(FL_NORMAL_BUTTON, x, y, CL_CLN_WIDTH(dlg->form)/7, CL_BTN_HEIGHT, "Cancel");
  dlg->cancelBtn=obj;
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_WHEAT,FL_COL1);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_gravity(obj, SouthEastGravity, SouthEastGravity);
  x+=3*obj->w;

  obj=fl_add_button(FL_RETURN_BUTTON, x, y, CL_CLN_WIDTH(dlg->form)/7, CL_BTN_HEIGHT, "OK");
  dlg->OkBtn=obj;
  fl_set_object_lsize(obj, CL_FONT_SIZE);
  fl_set_object_color(obj,FL_WHITE,FL_COL1);
  fl_set_object_lstyle(obj,FL_TIMESBOLD_STYLE);
  fl_set_object_gravity(obj, SouthEastGravity, SouthEastGravity);

/**
  goodie_msg(0, CFG_INFO, "", "XFSelector>Show>show form");
**/
  fl_set_form_minsize(dlg->form, w, h);

  fl_register_raw_callback(dlg->form, FL_ALL_EVENT, ClXFSelectorRawCb);
  fl_end_form();
  fl_set_form_atclose(dlg->form, ClXFSelectorAtCloseCb, (void *)dlg);
  fl_show_form(dlg->form, FL_PLACE_FREE_CENTER, FL_TRANSIENT, title);

  /*--- List the files in directory, according to pattern of files suffix ---*/

  /* update directory only if requested dir is valid. This way, passing
     dir==0 has the effect of keeping where we were the last time */

  if (fl_is_valid_dir(directory))
    strcpy(dlg->dname, directory);
  else
    strcpy(dlg->dname, ".");
  fl_fix_dirname(dlg->dname);
  if (pattern && *pattern)
    strcpy(dlg->pattern, pattern);
  else
    strcpy(dlg->pattern, "*");
  if (filename && *filename)
    strcpy(dlg->filename, filename);
  else
    strcpy(dlg->filename, "");

  /* check cache settings */
  fl_set_input(dlg->fileInp, dlg->filename);
  fl_set_input(dlg->patternInp, dlg->pattern);
/**
  fl_set_input(dlg->directoryInp, contract_dirname(dlg, dlg->dname, 38));
**/
  fl_set_input(dlg->directoryInp, dlg->dname);

  /* fill the browser */
  fill_entries(dlg->fileBrs, dlg->filename, 1);

  /* rescan files each time the panel is opened */
  fl_trigger_object (dlg->rescanBtn); 
/* HIHI */

  /*--- Main while loop to manage the files selection ---*/

  /* modal dialog box */
  /* dlg->fileInp contains the files selected */

  do
   {
    obj=fl_do_only_forms();

    tmp=fl_get_input(dlg->fileInp);

/*hihi*/
    if (obj == dlg->OkBtn && tmp && !(*tmp))
      {
       int sLine=fl_get_browser(dlg->fileBrs);
       if (sLine < 0) sLine = -sLine;
       if (sLine > 0)
         {
          char *seltext=(char *)fl_get_browser_line(dlg->fileBrs, sLine);
          seltext+=2;
          strcat(append_slash(dlg->dname), seltext);
          fl_fix_dirname(dlg->dname);
          if (fill_entries(dlg->fileBrs, (char *)NULL, 0) < 0) del_tail_slash(dlg->dname);
          continue;
         }
      }

    /* can you say ugly */
    if (obj == dlg->OkBtn && ((tmp)) && *tmp)
      {
       if (*tmp != '/' && *tmp != '~')
         {
          strcat(append_slash(dlg->dname), tmp);
          fl_fix_dirname(dlg->dname);
         }
       else
         {
          strcpy(dlg->dname, tmp);
          fl_fix_dirname(dlg->dname);
         }

       if (fl_is_valid_dir(dlg->dname))
         {
          fill_entries(dlg->fileBrs, (char *)NULL, 1);
          fl_set_input(dlg->fileInp, "");
          obj=(FL_OBJECT *)NULL;
         }
       else
         {
          char *p;
          if ((p=strrchr(dlg->dname, '/')))
            {
             *p='\0';
             fl_set_input(dlg->fileInp, p+1);
            }
         }
      }
    else if (obj == dlg->cancelBtn)
      {fl_set_input(dlg->fileInp, "");}

  } while (obj != dlg->cancelBtn && (obj != dlg->OkBtn || (obj == dlg->OkBtn && strlen(fl_get_input(dlg->fileInp)) < 1)));

 /*--- If we went out of the do while loop because of pushing Cancel button, close panel and return 0 ---*/

 if (obj==dlg->cancelBtn)
    {
     fl_hide_form(dlg->form);
     fl_free_form(dlg->form);
     dlg->form=(FL_FORM *)NULL;
     return(0);
    }

  /********* get all selected files of the directory *********/

  nFile     = fl_get_browser_maxline(dlg->fileBrs);
  nSelected = 0;
  length    = 0;
  for (i=0; i<nFile; i++)
     {
      if (fl_isselected_browser_line(dlg->fileBrs,i+1))
         {
          tmp = fl_get_browser_line(dlg->fileBrs,i+1);

          if (tmp && *tmp)
             {
              strcpy(seltext, tmp+2);

              /* complete the returned filename with the directory */
              if (seltext[0] != '/')
                 append_slash(strcpy(dlg->retname, dlg->dname));
              else
                 dlg->retname[0]='\0';

              strcat(dlg->retname, seltext); 

              if (fl_is_valid_dir(dlg->retname) == 0)
                 {
                  nSelected++;
                  if (nSelected == 1) strcpy(dlg->filename, seltext);

                  length += strlen(seltext) + strlen(dlg->retname) + 1;
                  if (length >= size)
                     {printf("Not enough space in buffer. Selected only %d first files\n", nSelected);
                      break;
                     }
                  if (nSelected == 1) strcpy(result, dlg->retname);
                  else               {strcat(result, " "); strcat(result, dlg->retname);}
                 }
             }
         }
     }

  if (nSelected == 0)
     {if (fl_get_input(dlg->fileInp) == NULL)
         {strcpy(result, "");}
      else
         {strcpy(result,dlg->dname);
          strcat(result,"/");
          strcat(result,fl_get_input(dlg->fileInp));
         }
     }

  /*--- Close the file selector panel and return the result ---*/

 if (Dy->debug > 0)
  printf("ClXFSelector> nSelected=%d  result=%s  dname=%s  filename=%s  fileInp=%s\n",
          nSelected, result, dlg->dname, dlg->filename, fl_get_input(dlg->fileInp));

  fl_hide_form(dlg->form);
  fl_free_form(dlg->form);
  dlg->form=(FL_FORM *)NULL;

  return(strlen(result));
} /* ClXFSelector */



/******************************************************************
 *
 ******************************************************************/
void ClXFSelectorRescan(ClXFSelector_t *dlg) {
  dlg->rescan=1;
} /* ClXFSelectorInvalidateCache */

/******************************************************************
 *
 ******************************************************************/
FL_FORM *ClXFSelectorGetForm(ClXFSelector_t *dlg) {
  if (dlg)
    return dlg->form;
  else
    return(FL_FORM *)NULL;
} /* ClXFSelectorGetForm */

/******************************************************************
 *
 ******************************************************************/
void ClXFSelectorDone(ClXFSelector_t *dlg) {
  if (dlg && dlg->form && dlg->form->visible)
    fl_object_qenter(dlg->cancelBtn);
} /* ClXFSelectorDone */
/******************************************************************
 *
 ******************************************************************/
const char *ClXFSelectorGetDirectory(ClXFSelector_t *dlg) {
  return dlg->dname;
} /* ClXFSelectorGetDirectory */

/******************************************************************
 *
 ******************************************************************/
const char *ClXFSelectorGetPattern(ClXFSelector_t *dlg) {
  return dlg->pattern;
} /* ClXFSelectorGetPattern */

/******************************************************************
 *
 ******************************************************************/
const char *ClXFSelectorGetFilename(ClXFSelector_t *dlg) {
  return dlg->filename;
} /* ClXFSelectorGetFilename */

/******************************************************************
 *
 ******************************************************************/
void ClXFSelectorRefresh(ClXFSelector_t *dlg) {
  ClXFSelectRescanBtnCb(dlg->rescanBtn, (long)dlg);
} /* ClXFSelectorRefresh */

/******************************************************************
 * set file markers for file list browser
 ******************************************************************/
void ClXFSelectorSetFiletypeMarker(int dir, int fifo, int sock, int cdev, int bdev) {
  dirmarker=dir;
  fifomarker=fifo;
  cdevmarker=cdev;
  bdevmarker=bdev;
  sockmarker=sock;
}


