/**************************************************/
/*                                                */
/* File: UFrameL.c                                */
/* Copyright (C) 2003, D. Verkindt, B.Mours       */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <fcntl.h> 
#include <stdarg.h>
#include <dirent.h>
#include <assert.h>
#include <unistd.h>
#include "FrameL.h"
#include "URootProcess.h"
#include "UFrameL.h"
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

struct UJstructH *FrameHUJstructH;
struct UJstructH *UFrIFileUJstructH;
struct UJstructH *UFrWaveUJstructH;
struct UJstructH *UFrShmUJstructH;
struct UJstructH *UFrIFileOnlineUJstructH;
struct UJstructH *UFrUnpackUJstructH;
struct UJstructH *UFrVectOpUJstructH;

#ifndef NO_CM
struct UJstructH *UFrIFileCmUJstructH;
#endif

struct UFrBuilder *UFrCurFrBuilder = NULL;
double Time0;
int lastFrameN;


/*            *****************************************            */
/*            *  modified source FROM SIESTA UJobL.c  *            */
/*            *****************************************            */

/*---------------------------------------------- DyUJLibIni2 ----*/

void UJLibIni2 (char *jobName, long arg, char **argv)
{
 DyDebugPrint(__FUNCTION__,0);

 strcpy (UJobH.cards, jobName);
 strcpy (UJobH.histo, UJobH.cards);
 strcpy (UJobH.listing, UJobH.histo);
 UJobH.debugLvl = 0;

 strcat (UJobH.cards, CONFIG_SUFFIX);
 strcat (UJobH.listing, ".lis");
 strcat (UJobH.histo, ROOT_SUFFIX);

 if (arg == 0) UJtimeStart = time(NULL);

 UJobH.fOut = stdout;

 if (arg == 0)
    {
     if (Dy->debug > 0) printf("UJLibIni2> dataDisplay starts running at %s", ctime(&UJtimeStart));

     /*--- i/o package initialization ---*/
	
     UJ_aint   = (char *) &UJ_int;
     UJ_ashort = (char *) &UJ_short;
     UJ_achar  = (char *) &UJ_char;
	
     UJclockUJstructH   = UJDicAddS("UJclock", NULL, NULL, NULL, UJclockRead, UJclockFree);
     UJdelayUJstructH   = UJDicAddS("UJdelay", NULL, NULL, NULL, UJdelayRead, NULL);
     UJstructHUJstructH = UJDicAddS("UJstructH", NULL, NULL, NULL, NULL, NULL);
 
     UFrLibIni();
/*     UHLibIni(); */
     USLibIni(); 
    }

 return;
}


/*-------------------------------------------------------- UFrLibIni -*/
void UFrLibIni()
/*--------------------------------------------------------------------*/
{
 DyDebugPrint(__FUNCTION__,0);

 FrameHUJstructH   = UJDicAddS("FrameH", NULL, NULL, NULL,NULL, NULL);
 UFrIFileUJstructH = UJDicAddS("UFrIFile",UFrIFileIni,UFrIFileSim,UFrIFileEnd,UFrIFileRead,NULL);
 UFrWaveUJstructH  = UJDicAddS("UFrWave",UFrWaveIni,UFrWaveSim,UFrWaveEnd,UFrWaveRead,NULL);
 UFrShmUJstructH   = UJDicAddS("UFrShm",UFrShmIni,UFrShmSim,UFrShmEnd,UFrShmRead,NULL);

#ifndef NO_CM
 UFrIFileCmUJstructH = UJDicAddS("UFrIFileCm",UFrIFileCmIni,UFrIFileCmSim,UFrIFileCmEnd,UFrIFileCmRead,NULL);
#endif

 UFrIFileOnlineUJstructH = UJDicAddS("UFrIFileOnline",UFrIFileOnlineIni,UFrIFileOnlineSim,NULL,UFrIFileOnlineRead, NULL);
 UFrUnpackUJstructH      = UJDicAddS("UFrUnpack",UFrUnpackIni,UFrUnpackSim,UFrUnpackEnd,UFrUnpackRead,NULL);
 UFrVectOpUJstructH      = UJDicAddS("UFrVectOp",UFrVectOpIni,UFrVectOpSim,UFrVectOpEnd,UFrVectOpRead, NULL);

 return;
}

/*--------------------------------------------------- UFrWaveIni------*/
void UFrWaveIni(struct UFrWave *wave, double dt)
/*--------------------------------------------------------------------*/
{
 char *buf;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 wave->file = fopen(wave->fileName,"r");
 if (wave->file == NULL)
    {sprintf("Cannot open file %s", wave->fileName);
     DyInfoPanel(dummy,0);
     Dy->Order = STOP;
     return;
    }

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrWaveIni> Wavefile=%x=%s  timeOrigin=%ld  UFrCurFrameH=%x\n", wave->file, wave->fileName, Dy->timeOrigin, UFrCurFrameH);}

 buf = (char *)(&(wavehdr));
 fread(buf, 1, sizeof(wavehdr), wave->file);
 DySoundReadHeader(buf);

 if (wavehdr.riff[0] != 'R')
    {printf("UFrWaveIni> %s if not a wav file \n", wave->fileName); Dy->Order = STOP; return;}
 if (wavehdr.bits_per_samp != 8 && wavehdr.bits_per_samp != 16)
    {printf("UFrWaveIni> %s is not 8 or 16 bits wav file \n", wave->fileName); Dy->Order = STOP; return;}
 if (wavehdr.format_tag != 1 && wavehdr.format_tag != 259)
    {printf("UFrWaveIni> %s is not PCM format wav file \n", wave->fileName); Dy->Order = STOP; return;}

 if (wavehdr.num_chans == 256 * buf[30] + buf[31])
    {printf("UFrWaveIni> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in big endian\n", wavehdr.num_chans, buf[30], buf[31]);}
 else
    {printf("UFrWaveIni> num_chans=%d  buf[30]=%ud  buf[31]=%ud  We are in little endian\n", wavehdr.num_chans, buf[30], buf[31]);}

 wave->gtime            = Dy->startTime;
 wave->framedt          = dt;
 wave->vectnData        = (int)(wavehdr.srate * wave->framedt);
 wave->nBytes_per_frame = wave->vectnData * wavehdr.bytes_per_samp;
 wave->nData            = wavehdr.dlength / (wavehdr.bytes_per_samp / wavehdr.num_chans);
 printf("UFrWaveIni> Wave file contains %ld bytes with %d bytes per sample (%d bits) and %d channels --> nData=%ld\n",
        wavehdr.dlength, wavehdr.bytes_per_samp, wavehdr.bits_per_samp, wavehdr.num_chans, wave->nData);
 printf("UFrWaveIni> Frame length = %g --> vectnData=%d\n", wave->framedt, wave->vectnData);

 wave->i = 0;
 if (UFrCurFrameH) {FrameFree(UFrCurFrameH); UFrCurFrameH = NULL;}
 UFrCurFrameH = FrameHNew("frame_from_wave");
 FrAdcDataNew (UFrCurFrameH, AUDIOCH_NAME, wavehdr.srate, wave->vectnData, 16);

 return;
}


/*--------------------------------------------------- UFrWaveEnd------*/
void UFrWaveEnd(struct UFrWave *wave)
/*--------------------------------------------------------------------*/
{
 long i;

 DyDebugPrint(__FUNCTION__,0);

 DyOutputClose();
 
 fclose(wave->file);
 free(wave->fileName);
 free(wave->name);
 free(wave);
 
 return;
}


/*-------------------------------------------------- UFrWaveRead------*/
void UFrWaveRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrWave *dIF;

 UJ_READ_HEADER_WITH_NAME(UFrWave,dIF)

 UJ_READ_LONG(dIF->iClock);
 UJ_READ_S_CHAR(dIF->fileName);
  
 UJCObAdd(dIF->iClock,dIF, UFrWaveUJstructH, dIF->name);
    
 dIF->frameH = FrameHNew("dummy");

 strcpy(bankName, dIF->name);
 UJSetBA(strcat(bankName,".frameH"),"FrameH", dIF->frameH);
 strcpy(bankName, dIF->name);

 return;
}


/*--------------------------------------------------- UFrWaveSim ----*/
void UFrWaveSim(struct UFrWave *wave, long ttime)
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 int j, is;
 unsigned char valuec;
 short value;
 struct FrAdcData *adc;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Order == STOP || wave->i > wave->nData)
    {
     clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrWaveSim","no clock struct");
     clock->obj0->time = clock->totTime;
     return;
    }

 /*--- read data from wave file ---*/

 UFrCurFrameH->GTimeS = (int)(wave->gtime);
 UFrCurFrameH->dt     = wave->framedt;
 UFrCurFrameHSize     = wave->nBytes_per_frame;
 adc = FrAdcDataFind(UFrCurFrameH, AUDIOCH_NAME);

 is = 0;
 while (is < wave->vectnData)
    {
     if (wavehdr.bits_per_samp == 16) fread(&value, sizeof(short), 1, wave->file);
     else                             fread(&valuec, sizeof(unsigned char), 1, wave->file);

     if (wavehdr.num_chans == 1 || (wavehdr.num_chans == 2 && (wave->i)%2 == 0))
        {
         if (wavehdr.bits_per_samp == 16)
            {adc->data->dataS[is] = value;
             if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>0 && wave->i%100000 < 10)
                {printf("UFrWaveSim> i=%d  is=%d  value=%d  adc->data=%d\n", wave->i, is, value, adc->data->dataS[is]);}
            }
         else
            {adc->data->dataS[is] = (short)(valuec*256 - 32768);
             if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>0 && wave->i%100000 < 10)
                {printf("UFrWaveSim> i=%d  is=%d  valuec=%d  adc->data=%d\n", wave->i, is, valuec, adc->data->dataS[is]);}
            }
         is++;
        }
     wave->i++;
     if (wave->i > wave->nData) break;
    }

 if (is < wave->vectnData) {for (j=is; j<wave->vectnData; j++) adc->data->dataS[j]=0;}

 wave->gtime += wave->framedt;
 memcpy(wave->frameH, UFrCurFrameH, sizeof(struct FrameH)); 

 if (Dy->output_flag == ON) DyOutputWrite(UFrCurFrameH);

 return;
}


/*--------------------------------------------------- UFrIFileIni------*/
void UFrIFileIni(iFile, dt)
struct UFrIFile *iFile;
double dt;
/*--------------------------------------------------------------------*/
{
 double tStop, startTime;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 iFile->file = FrFileINew(iFile->fileName);
 if (iFile->file == NULL)
    {sprintf(dummy,"Cannot open file %s", iFile->fileName);
     DyInfoPanel(dummy,0);
     Dy->Order = STOP;
     return;
    }

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrIFileIni> iFile=%x=%s  timeOrigin=%ld  UFrCurFrameH=%x\n", iFile->file, iFile->fileName, Dy->timeOrigin, UFrCurFrameH);}

 if (Dy->timeOrigin > 0)
    {
     iFile->gtime = -1.;
     startTime    = Dy->timeOrigin+0.00001;
     UFrCurFrameH = FrameReadTChnl(iFile->file, startTime, "NONE");

     if (UFrCurFrameH == NULL)
        {startTime = Dy->timeOrigin+0.999;
         UFrCurFrameH = FrameReadTChnl(iFile->file, startTime, "NONE");
        }

     if (UFrCurFrameH == NULL)
        {iFile->gtime = FrFileITNextFrame(iFile->file, startTime);
         if (iFile->gtime > 0)
            {sprintf(dummy,"No frame readable at startTime GPS %.1f\nI have to start at %.1f (%s)", startTime, iFile->gtime, FrStrGTime(iFile->gtime));
             DyInfoPanel(dummy,0);
            }
        }
     else
        {iFile->gtime = startTime;}

     if (iFile->gtime < 0)
        {tStop = FrFileITEnd(iFile->file);
         sprintf(dummy, "No frame readable after startTime %ld\nMax GPS in those files = %.3f (%s)", Dy->timeOrigin, tStop, FrStrGTime(tStop));
         DyInfoPanel(dummy,1);
         iFile->gtime = startTime;
         return;
        }
     else
        {Dy->startTime  = (long)(iFile->gtime);
         Dy->timeOrigin = (long)(iFile->gtime);
        }
    }

 else
    {iFile->gtime = -1.;}

 iFile->lastFrameN = -1;
 
 FrFileIEnd(iFile->file);
 iFile->file = FrFileINew(iFile->fileName);

 return;
}


/*--------------------------------------------------- UFrIFileEnd------*/
void UFrIFileEnd(struct UFrIFile *iFile)
/*--------------------------------------------------------------------*/
{
 long i;

 DyDebugPrint(__FUNCTION__,0);

 DyOutputClose();
 
 if (iFile->trigAsciiFile != NULL) fclose(iFile->trigAsciiFile);

 for (i=0; i<UFr_MAX_ID; i++);
    {if (iFile->fSubName[i])  free(iFile->fSubName[i]);}

 FrFileIEnd(iFile->file);
 free(iFile->fileName);
 free(iFile->itag);
 free(iFile->name);
 free(iFile);
 
 return;
}


/*-------------------------------------------------- UFrIFileRead------*/
void UFrIFileRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrIFile *dIF;
 char tag[LBIGSTRING_MAX];

 UJ_READ_HEADER_WITH_NAME(UFrIFile,dIF)

 UJ_READ_LONG(dIF->iClock);
 UJ_READ_S_CHAR(dIF->fileName);

 DyMakeTag(tag);
 dIF->itag = strdup(tag);
 if (dIF->itag == NULL)  dIF->itag = strdup("*");

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrIFileRead> inputfile=%s  tag=%s\n",dIF->fileName, dIF->itag);}
  
 UJCObAdd(dIF->iClock,dIF, UFrIFileUJstructH, dIF->name);
    
 dIF->frameH = FrameHNew("dummy");

 strcpy(bankName, dIF->name);
 UJSetBA(strcat(bankName,".frameH"),"FrameH", dIF->frameH);
 strcpy(bankName, dIF->name);

 dIF->run       = USignalNew(strcat(bankName,".run"), YES);
 strcpy(bankName, dIF->name);
 dIF->frame     = USignalNew(strcat(bankName,".frame"), YES);
 strcpy(bankName, dIF->name);
 dIF->length    = USignalNew(strcat(bankName,".length"), YES);
 strcpy(bankName, dIF->name);
 dIF->time      = USignalNew(strcat(bankName,".time"), YES);
 strcpy(bankName, dIF->name);
 dIF->timeofrun = USignalNew(strcat(bankName,".timeofrun"), YES);
 dIF->timeofrun->value = -1.;

 return;
}

/*--------------------------------------------------DyFrFileIGetNextEventTime--*/
double DyFrFileIGetNextEventTime(FrFile *iFile,
                               char *name,
                               double tStart,
                               double aMin)
/*---------------------------------------------------------------------------*/
/* Return the next event with amplitude above ampMin                         */
/*---------------------------------------------------------------------------*/
{FrTOCevt *tocEvt;
 FrTag    *frtag;
 FrFileH  *firstFileH;
 double tEvent, tNext;
 int i;
 
 DyDebugPrint(__FUNCTION__,0);

 if(iFile == NULL) return(0);                 /*----- check input file ---*/

 FrTOCFrameFindNT(iFile, tStart);

 firstFileH = iFile->current;
 iFile->relocation = FR_NO;            /*--- to avoid relloaction problems---*/

 tNext = 1.e37;
 frtag = FrTagNew(name);
 if(frtag == NULL) return(0);
                                                /*--- Loop over all files--*/
 do
   {if(iFile->toc == NULL) FrTOCReadFull(iFile); /*--------- get the TOC----*/
    if(iFile->toc == NULL) return(0);

                  /*---------Find name in the channel list for this file----*/

    for(tocEvt = iFile->toc->event; tocEvt != NULL; tocEvt = tocEvt->next) 
      {if(FrTagMatch(frtag, tocEvt->name) == FR_NO) continue; 

       for(i=0; i<tocEvt->nEvent; i++)
         {if(tocEvt->amplitude[i] < aMin) continue;
          tEvent = tocEvt->GTimeS[i]+1.e-9*tocEvt->GTimeN[i];
          if(tEvent <= tStart) continue;
          if(tEvent < tNext) tNext = tEvent;}}

    if(tNext < 1.e13) break;}

 while (FrFileINext(iFile, 0, 0, firstFileH, FR_YES) == 0);

 FrTagFree(frtag); 
 if(tNext > 1.e36) tNext = 0.;
 
 return(tNext);}


/*--------------------------------------------------- UFrIFileSim ----*/
void UFrIFileSim(struct UFrIFile *iFile, long ttime)
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 char dummy[LSTRING_MAX], *trigline, *word, *str;
 double gtimeStop, gtimePrevious, currentSize;
 double next_gtime, next_event_gps;
 int nBytes, i, triggps, nowgps;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrIFileSim> Dy->startTime=%ld  Dy->timeOrigin=%ld  Dy->currentTime=%f  Dy->duration=%ld  time=%ld \n",
             Dy->startTime, Dy->timeOrigin, Dy->currentTime, Dy->duration, ttime);
    }

 if (Dy->Order == STOP)
    {
     clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
     clock->obj0->time = clock->totTime;
     return;
    }

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

 if (UFrCurFrameH)
    {
     if (iFile->frameH)
        {
         iFile->frameH->rawData     = NULL;
         iFile->frameH->procData    = NULL;
         iFile->frameH->simData     = NULL;
         iFile->frameH->event       = NULL;
         iFile->frameH->summaryData = NULL;
        }
     FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

  if (iFile->gtime < 0  &&  Dy->notoc == 0)
     {
      DyInfoLine("End of frames file (gtime<0)!");
      clock = UJCloGet();
      if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
      clock->obj0->time = clock->totTime;
      return;
     }

 if (iFile->gtime >= 0.  &&  Dy->notoc == 0)
    {
     if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_EVENTS)
        {
         next_event_gps = 0.;
         next_gtime = DyFrFileIGetNextEventTime (iFile->file, Dy->conditionName, iFile->gtime+0.00001, Dy->conditionTh);
         if (next_gtime > 0 && next_gtime > next_event_gps) next_event_gps = next_gtime;
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
             {printf("conditionName=%s  conditionTh=%s  current_event_gps=%f  next_event_gps=%f \n",
                       Dy->conditionName, Dy->conditionTh, iFile->gtime, next_event_gps);
             }
         if (next_event_gps > 0) iFile->gtime = next_event_gps;
        }

     if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_ASCII)
        {
         if (iFile->trigAsciiFile == NULL) iFile->trigAsciiFile = fopen(Dy->trigAsciiFileName,"r");
         if (iFile->trigAsciiFile == NULL)
            {sprintf(dummy,"Trigger file %s does not exist", Dy->trigAsciiFileName);
             DyInfoLine(dummy);
             Dy->triggerUsed == OFF;
             return;
            }
         trigline = (char *)malloc(LSTRING_MAX);
         Dy->triggerEvent = OFF;
         clock = UJCloGet();
         if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");

         while (Dy->triggerEvent == OFF && clock->obj0->time < clock->totTime)
             {
              trigline = fgets(trigline, LSTRING_MAX, iFile->trigAsciiFile);
              if (trigline)
                {word  = strtok(trigline," ");
                 while (word != NULL)
                    {triggps = atoi(word);
                     if (triggps == 0)
                        {str = strstr(word,"=");
                         if (str) triggps = atoi(str+1);
                        }
                     nowgps  = time(NULL) - FRGPSOFF + FRGPSDELTA + 3600;
                     if (triggps > 70000000 && triggps < nowgps) word = NULL;
                     else                                         word = strtok(NULL," ");
                    }
                 if (triggps > 70000000 && triggps < nowgps)
                    {iFile->gtime = triggps; Dy->triggerEvent = ON;
                     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
                        {printf("UFrIFileSim> Got GPS=%d from Trigger Ascii file %s\n", triggps, Dy->trigAsciiFileName);}
                     break;
                    }
                 else
                    {Dy->triggerEvent == OFF;}
                }
             else 
                {
                 DyInfoLine("End of Trigger Ascii file. I stop.");
                 clock = UJCloGet();
                 if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
                 clock->obj0->time = clock->totTime;
                 Dy->triggerEvent == OFF;
                }
            }
         free(trigline);
        }

     nBytes = iFile->file->nBytes;

     UFrCurFrameH = FrameReadTChnl(iFile->file, iFile->gtime+0.00001, iFile->itag);

     UFrCurFrameHSize = iFile->file->nBytes - nBytes;

     if (UFrCurFrameH && Dy->Frame && UFrCurFrameH->dt != Dy->Frame->dt)
        {
         sprintf(dummy,"Reading frames of %g s while configuration was for frames of %g s. Please, do \"Update Channels\" and restart", UFrCurFrameH->dt, Dy->Frame->dt);
         clock = UJCloGet();
         if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
         clock->obj0->time = clock->totTime;
         return;
        }

     if (Dy->skipFrame == ON)
        {
         if (UFrCurFrameH == NULL)
            {
             DyInfoLine("End of frames file (frame=NULL)!");
             clock = UJCloGet();
             if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
             clock->obj0->time = clock->totTime;
             return;
            }
        }
     else
        {
         if (UFrCurFrameH)
            {iFile->gtime   = UFrCurFrameH->GTimeS + UFrCurFrameH->dt;
             iFile->framedt = UFrCurFrameH->dt;
            }
         else
            {
             gtimeStop = FrFileITEnd(iFile->file);

             if (iFile->gtime+0.00001 < gtimeStop)
                {
                 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>2)
                    {sprintf(dummy,"Frame unreadable (supposed gtime=%f)", iFile->gtime);
                     DyInfoLine(dummy);
                    }
                 iFile->gtime += iFile->framedt;
                 return;
                }
             else
                {DyInfoLine("End of frames file!");
                 clock = UJCloGet();
                 if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
                 clock->obj0->time = clock->totTime;
                 return;
                }
            }
        }
    }


 if (Dy->notoc > 0)
    {
     nBytes = iFile->file->nBytes;

     UFrCurFrameH = FrameRead(iFile->file);

     UFrCurFrameHSize = iFile->file->nBytes - nBytes;

     if (UFrCurFrameH == NULL)
        {
         sprintf(dummy, "End of frames file or frame unreadable! (gtime=%f=%s)", iFile->gtime, FrStrGTime(iFile->gtime));
         DyInfoLine(dummy);
/*
         iFile->frameH->rawData = NULL;
*/
         clock = UJCloGet();
         if (clock == NULL) UJError(3,"UFrIFileSim","no clock struct");
         clock->obj0->time = clock->totTime;
         return;
        }
    }

 if (Dy->output_flag == ON) DyOutputWrite(UFrCurFrameH);

 iFile->lastFrameN = DyMissingFrames (iFile->lastFrameN, UFrCurFrameH);

 memcpy(iFile->frameH, UFrCurFrameH, sizeof(struct FrameH)); 

 iFile->run->value       = (double)iFile->frameH->run;
 iFile->frame->value     = (double)iFile->frameH->frame;
 iFile->length->value    = (double)iFile->frameH->dt;
 iFile->time->value      = 0.;
 if (iFile->timeofrun->value == -1.)  Time0 = iFile->time->value;
 iFile->timeofrun->value = 0. - Time0;

 if (Dy->skipFrame == ON)
    {
     if (iFile->gtime >= 0.  &&  Dy->notoc == 0)
        {
         iFile->gtime  =  FrFileITNextFrame(iFile->file, iFile->gtime+0.00001);

         if (iFile->gtime < 0)
            {
             gtimeStop     = FrFileITEnd(iFile->file);
             gtimePrevious = UFrCurFrameH->GTimeS;

             while (iFile->gtime < 0 && iFile->gtime < gtimeStop)
                {
                 iFile->gtime  = gtimePrevious +  UFrCurFrameH->dt;
                 gtimePrevious = iFile->gtime;
                 iFile->gtime  =  FrFileITNextFrame(iFile->file, iFile->gtime+0.00001);
                }
            }
        }
    }

 DyDebugTimeNeeded(__FUNCTION__,"stop",1);

 return;
}


/*---------------------------------------- UFrIFileCmIni ----*/

void UFrIFileCmIni (struct UFrIFileCm *iFile, double dt)
{
 struct UJclock *clock;
 int result;
 char dummy[LSTRING_MAX];

#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

 if (iFile->tag == NULL)  strcpy(iFile->tag,"*");

 if (Dy->Frame) Dy->Framedt = Dy->Frame->dt;
 else           Dy->Framedt = 1.;

 result = DyCmConnect(Dy->CmName);

 if (result == -1)
    {clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrIFileCmIni","no clock struct");
     clock->obj0->time = clock->totTime;
     Dy->Order = STOP;
     sprintf("dummy,Not been able to connect to %s", Dy->CmServer);
     DyInfoLine(dummy);
     return;
    }

 iFile->iFrame       = 0;
 iFile->lastFrameN   = -1;
 iFile->previousSize = 0;

 if (UFrCurFrameH)
    {FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 if (strncmp(Dy->CmName,DDISPLAY,strlen(DDISPLAY))==0 && Dy->InputType == INCM)
    {DyFdOutSendAdd (Dy->CmServer, Dy->CmName, iFile->tag, iFile->tag, 2, 0);
     sprintf(dummy,"Sent FdOutAdd request to %s", Dy->CmServer);
     DyInfoLine(dummy);
    }

 if (Dy->InputType == INDD)
    {DyDdDataRequest(Dy->DdServer, Dy->Ddinputfile, iFile->tag, Dy->timeOrigin, Dy->duration);
     sprintf(dummy,"Sent Data request to %s", Dy->DdServer);
     DyInfoLine(dummy);
     strcpy(Dy->CmServer, Dy->DdServer);
    }

 sprintf(dummy, "Waiting for frames from %s...", Dy->CmServer);
 DyInfoLine(dummy);

#endif

 return;
}


/*----------------------------------------------- UFrIFileCmRead------*/
void UFrIFileCmRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrIFileCm *dIF;
 
 UJ_READ_HEADER_WITH_NAME(UFrIFileCm,dIF);
 UJ_READ_LONG(dIF->iClock);
 UJ_READ_LONG(dIF->waitMax);
 UJ_READ_S_CHAR(dIF->CmName);
 UJ_READ_S_CHAR(dIF->tag);

 if (dIF->tag == NULL)
    {DyInfoLine("In CmRead: no channels tag! I thus request all channels");
     strcpy(dIF->tag,"*");
    }

 UJCObAdd(dIF->iClock, dIF, UFrIFileCmUJstructH, dIF->name);

 dIF->frameH = FrameHNew("dummy");

 strcpy(bankName, dIF->name);
 strcat(bankName,".frameH");
 UJSetBA(bankName,"FrameH", dIF->frameH);

 strcpy(bankName, dIF->name);

 dIF->run       = USignalNew(strcat(bankName,".run"), YES);
 strcpy(bankName, dIF->name);
 dIF->frame     = USignalNew(strcat(bankName,".frame"), YES);
 strcpy(bankName, dIF->name);
 dIF->length    = USignalNew(strcat(bankName,".length"), YES);
 strcpy(bankName, dIF->name);
 dIF->time      = USignalNew(strcat(bankName,".time"), YES);
 strcpy(bankName, dIF->name);
 dIF->timeofrun = USignalNew(strcat(bankName,".timeofrun"), YES);
 dIF->timeofrun->value = -1.;

 return;
}


/*------------------------------------------------- UFrIFileCmSim ----*/
void UFrIFileCmSim (struct UFrIFileCm *iFile, long clocktime)
/*--------------------------------------------------------------------*/
{
#ifndef NOCM

 CmConnectCondition condition;
 struct UJclock *clock;
 long iwait;
 int delta;
 static int frameAnalysed;
 double currentTime, dtime;
 char dummy[32], dummy2[LSTRING_MAX];
 int latency, diff_latency;

 DyDebugPrint(__FUNCTION__,0);

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

 if (UFrCurFrameH)
    {
     if (iFile->frameH)
        {iFile->frameH->rawData     = NULL;
         iFile->frameH->procData    = NULL;
         iFile->frameH->simData     = NULL;
         iFile->frameH->event       = NULL;
         iFile->frameH->summaryData = NULL;
        }
     FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 /*--- Check the time passed outside CmMessageWait ---*/

 gettimeofday(&dytv, NULL);
 currentTime = dytv.tv_sec + (dytv.tv_usec * 1.0e-6);
 dtime = currentTime - Dy->oldTime;

 if (Dy->oldTime > 0.)
    {
     Dy->nAnaTime++;
     Dy->AnaTime += dtime;
     if (dtime > DTMAX  &&  ((Dy->debugType)&(1<<DBG_SPEED)) && Dy->debug>0)
        {printf("UFrIFileCmSim> time to analyse frame %d = %.3fs ", frameAnalysed, dtime);
         if (Dy->nAnaTime > 0) printf(" (mean=%.3fs)\n", Dy->AnaTime/Dy->nAnaTime);
         else                  printf("\n");
        }
    }

 if (Dy->AnaTime > 30.)
    {sprintf(dummy2,"Mean time to analyse one %.3fs frame from %s = %.3fs", Dy->Framedt, Dy->CmServer, Dy->AnaTime/Dy->nAnaTime);
     DyInfoLine(dummy2);
     if ((Dy->AnaTime / Dy->nAnaTime > FRTIME_MAX*Dy->Framedt) && Dy->InputType != INDD)
        {Dy->Order = STOP;}
     else
        {Dy->nAnaTime = 0; Dy->AnaTime = 0.;}
    }


 /*--- Loop waiting for frames ---*/

 iwait = 1;
 while (Dy->Order != STOP) 
     {
      condition = CmMessageWaitWithTimeout(1.);
      if (condition == CmConnectBreakDetection) break;

      if (iwait%DY_TIMEOUT2 == 0 && Dy->InputType == INCM)
         {sprintf(dummy2,"No frame received since %ld seconds (no timeout)", iwait);
          if (strcmp(Dy->CmServer,DYEMPTY) != 0)
             {sprintf(dummy2, "%s - Sending again frames request to %s", dummy2, Dy->CmServer);
              DyFdOutSendAdd (Dy->CmServer, Dy->CmName, iFile->tag, iFile->tag, 2, 0);
             }
          DyInfoLine(dummy2);
         }

      if (Dy->InputType == INDD && Dy->duration>0)
         {delta = (int)Dy->currentTime - Dy->timeOrigin;
          if (iwait%DY_TIMEOUT2 == 0 && ((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>1)
             {printf("UFrIFileCmSim> delta=%d-%d=%d  duration=%ld  iwait=%d \n", (int)Dy->currentTime, Dy->timeOrigin, delta, Dy->duration, iwait);}
          if (delta >= Dy->duration-1) Dy->Order = STOP;
          else if (iwait%DY_TIMEOUT2 == 0)
             {sprintf(dummy2,"No frame received from %s since %ld seconds (no timeout)", Dy->CmServer, iwait);
              DyInfoLine(dummy2);
             }
         }
      iwait++;
      fl_check_forms();
     }

 if (UFrCurFrameH) frameAnalysed = UFrCurFrameH->frame;
 gettimeofday(&dytv, NULL);
 Dy->oldTime = dytv.tv_sec + (dytv.tv_usec * 1.0e-6);

 if (UFrCurFrameH) 
    {
     Dy->Framedt = UFrCurFrameH->dt;
     latency = time(NULL) - FRGPSOFF - FRGPSTAI + UFrCurFrameH->ULeapS - UFrCurFrameH->GTimeS;
     if (Dy->start_latency == 0) Dy->start_latency = latency;
     diff_latency = latency - Dy->start_latency;
     if (diff_latency > 2  &&  ((Dy->debugType)&(1<<DBG_CM)) && Dy->debug>1)
        {printf("UFrIFileCmSim> Current latency=%ds  (%ds above latency at start)", latency, diff_latency);}
     sprintf(dummy, "%d", latency);
     fl_set_input(mainpanel->latency, dummy);
   }

 if (Dy->Order == STOP)
    {clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrIFileCmSim","no clock struct");
     clock->obj0->time = clock->totTime;
     return;
    }

 iFile->lastFrameN = DyMissingFrames (iFile->lastFrameN, UFrCurFrameH);

 if (iFile->iFrame == 0 && UFrCurFrameH)
    {
     sprintf(dummy, "Reading frames from %s...", Dy->CmServer);
     DyInfoLine(dummy);
     Dy->timeOrigin  = (long)(UFrCurFrameH->GTimeS);
     Dy->startTime   = (long)(UFrCurFrameH->GTimeS);
     Dy->currentTime = (double)Dy->startTime;
     Dy->previousTime = (double)Dy->startTime;
     sprintf(dummy,"%ld",Dy->startTime);
     fl_set_input(mainpanel->startTime, dummy);
     if (FRGPSDELTA <= 13 && Dy->startTime>=820108813) fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime-1,0));
     else                                              fl_set_input(mainpanel->startUTCdate, FrStrUTC(Dy->startTime,0));
    }

 if (Dy->InputType == INDD)
    {
     if (Dy->nAnaTime > 0 && (iFile->iFrame == 5 || (iFile->iFrame > 0 &&  (iFile->iFrame)%60 == 0)))
        {if (((Dy->debugType)&(1<<DBG_SPEED)) && Dy->debug>0)
            {printf("UFrIFileCmSim> iFrame=%d  Sending Waiting time %f (%f/%d)\n",
                          iFile->iFrame, Dy->AnaTime/Dy->nAnaTime, Dy->AnaTime, Dy->nAnaTime);
            }
         DyDdSendWaitingTime(Dy->AnaTime/Dy->nAnaTime/2);
        }
    }

 if (UFrCurFrameH) memcpy(iFile->frameH, UFrCurFrameH, sizeof(struct FrameH));

 iFile->iFrame++;

 iFile->run->value       = (double)iFile->frameH->run;
 iFile->frame->value     = (double)iFile->frameH->frame;
 iFile->length->value    = (double)iFile->frameH->dt;
 iFile->time->value      = 0.;
 if(iFile->timeofrun->value == -1.) Time0 = iFile->time->value;
 iFile->timeofrun->value = 0. - Time0;

 DyDebugTimeNeeded(__FUNCTION__,"start",1);

#endif

 return;
}


/*---------------------------------------- UFrIFileCmEnd ----*/

void UFrIFileCmEnd (struct UFrIFileCm *dIF)
{
#ifndef NOCM

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->CmServer  &&  strcmp(Dy->CmServer,DYEMPTY) != 0 && Dy->InputType == INCM)
    {DyFdOutSendRemove (Dy->CmServer, Dy->CmName);}

 if (Dy->DdServer  &&  Dy->InputType == INDD)
    {DyFdOutSendRemove (Dy->DdServer, Dy->CmName);}

 DyCmDisconnect(Dy->CmName);

 DyOutputClose();

 if (dIF->CmName) free(dIF->CmName);
 if (dIF->tag)    free(dIF->tag);
 if (dIF->name)   free(dIF->name);

 free(dIF);

#endif

 return;
}


/*--------------------------------------------------- UFrIFileOnlineIni------*/
void UFrIFileOnlineIni(iFile, dt)
struct UFrIFileOnline *iFile;
double dt;
/*--------------------------------------------------------------------*/
{
 long i, i0, startTime;
 int fpf;
 DIR  *dirp;
 struct dirent *entry;
 char fileName[LSTRING_MAX], f0name[LSTRING_MAX], f1name[LSTRING_MAX], f2name[LSTRING_MAX];
 char s;
 char dummy[LSTRING_MAX];
 struct FrameH *frame;

 DyDebugPrint(__FUNCTION__,0);

 if (iFile->waitMax <= 0)    iFile->waitMax    = 1000000;
 if (iFile->startTime < 0)  iFile->startTime = 0;

 iFile->frames_per_file = 1;
 fpf                    = 1;
 startTime             = 0;

 /*--- Do not take the files beginning with a dot ---*/

 dirp  = opendir(iFile->directoryName);
 entry = readdir(dirp);
 while (strncmp(entry->d_name,".",1) == 0) entry = readdir(dirp);
 entry = readdir(dirp);

 /*--- Get the generic name (fname) and suffix (suffix) of the files to read online ---*/

 if ((entry = readdir(dirp)) != NULL)
    {
     strcpy(f0name,entry->d_name);
     i0 = strlen(f0name)-1;
     for (i=strlen(f0name)-1; i>=0; i--)
        {s = f0name[i];
         if (atoi(&s) != 0 || s == '0') {i0 = i; break;}
        }
	
     if (i0 < strlen(f0name)-1)
        {if (iFile->suffix) free(iFile->suffix);
         iFile->suffix = malloc(strlen(&f0name[i0+1])+1);
         strcpy(iFile->suffix, &f0name[i0+1]);
        }
     else
        {if (iFile->suffix) free(iFile->suffix);
         iFile->suffix = malloc(2);
         strcpy(iFile->suffix, "");
        }

     for (i=i0; i>=0; i--)
        {s = f0name[i];
         if (atoi(&s) == 0 && s != '0')
            {iFile->fname = (char *)calloc(i+2, sizeof(char));
             strncpy(iFile->fname, f0name, i+1);
             startTime   = atoi(&f0name[i+1]);
             break;
            }
        }
    }

 /*--- Get the number of frames per file (use 3 files as tests) ---*/

 if ((entry = readdir(dirp)) != NULL)
    {
     strcpy(f1name,entry->d_name);
     for (i=strlen(f1name)-1; i>=0; i--)
        {s = f1name[i];
         if (atoi(&s) != 0 || s == '0') {i0 = i; break;}
        }
     for (i=i0; i>=0; i--)
        {s = f1name[i];
         if (atoi(&s) == 0 && s != '0')
            {iFile->frames_per_file = atoi(&f1name[i+1]) - atoi(&f0name[i+1]);
             if (iFile->frames_per_file < 0) iFile->frames_per_file = -iFile->frames_per_file;
             break;
            }
        }
    }

 if ((entry = readdir(dirp)) != NULL)
    {
     strcpy(f2name,entry->d_name);
     for (i=strlen(f2name)-1; i>=0; i--)
        {s = f2name[i];
         if (atoi(&s) != 0 || s == '0') {i0 = i; break;}
        }
     for (i=i0; i>=0; i--)
        {s = f2name[i];
         if (atoi(&s) == 0 && s != '0')
            {fpf = atoi(&f2name[i+1]) - atoi(&f1name[i+1]);
             if (fpf < 0) fpf = -fpf;
             break;
            }
        }
    }

 if (iFile->frames_per_file > fpf)  iFile->frames_per_file = fpf;

 closedir(dirp);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrIFileOnlineIni> f0name=%s iFile->fname=%s startTime=%ld frames_per_file=%ld \n",
                                f0name, iFile->fname, startTime, iFile->frames_per_file);
     printf("UFrIFileOnlineIni> f1name=%s iFile->suffix=%s \n", f1name, iFile->suffix);
     printf("UFrIFileOnlineIni> fileName=%s\n",fileName);
    }

 sprintf(fileName, "%s/%s%ld%s", iFile->directoryName, iFile->fname,  startTime, iFile->suffix);

 iFile->file = FrFileINew(fileName);
 if (iFile->file == NULL)
    {sprintf(dummy, "Cannot open file %s", fileName);
     DyInfoLine(dummy); 
     iFile->gtime = 0;
    }
 else
    {frame = FrameRead(iFile->file);
     iFile->gtime = frame->GTimeS;
     FrFileIEnd(iFile->file);
    }

 while ( (iFile->file = FrFileINew(fileName)) != NULL )
    {startTime += iFile->frames_per_file;
     sprintf(fileName, "%s/%s%ld%s", iFile->directoryName, iFile->fname, startTime, iFile->suffix);
    }

 if (iFile->startTime > startTime  ||  iFile->startTime == 0)
    {iFile->startTime = startTime;}

 iFile->iFrame = iFile->startTime;
 lastFrameN    = -1;
 UFrCurFrameH  = NULL;

 iFile->iFirst = 1;

 if (Dy->timeOrigin > 0) iFile->gtime = (double)(Dy->timeOrigin);

 return;
}


/*-------------------------------------------------- UFrIFileOnlineRead------*/
void UFrIFileOnlineRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrIFileOnline *dIF;
 char tag[LBIGSTRING_MAX];

 UJ_READ_HEADER_WITH_NAME(UFrIFileOnline,dIF)

 UJ_READ_LONG(dIF->iClock);
 UJ_READ_S_CHAR(dIF->directoryName);
 UJ_READ_LONG(dIF->waitMax);
 UJ_READ_LONG(dIF->startTime);
/*
 UJ_READ_S_CHAR(dIF->itag);
*/

 DyMakeTag(tag);
 dIF->itag = strdup(tag);
 if (dIF->itag == NULL)  dIF->itag = strdup("*");

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrIFileOnlineRead> input directory=%s \n",dIF->directoryName);}
  
 UJCObAdd(dIF->iClock, dIF, UFrIFileOnlineUJstructH, dIF->name);
    
 dIF->frameH = FrameHNew("dummy");

 strcpy(bankName, dIF->name);
 strcat(bankName,".frameH");
 UJSetBA(bankName,"FrameH", dIF->frameH);

 strcpy(bankName, dIF->name);

 dIF->run       = USignalNew(strcat(bankName,".run"), YES);
 strcpy(bankName, dIF->name);
 dIF->frame     = USignalNew(strcat(bankName,".frame"), YES);
 strcpy(bankName, dIF->name);
 dIF->length    = USignalNew(strcat(bankName,".length"), YES);
 strcpy(bankName, dIF->name);
 dIF->time      = USignalNew(strcat(bankName,".time"), YES);
 strcpy(bankName, dIF->name);
 dIF->timeofrun = USignalNew(strcat(bankName,".timeofrun"), YES);
 dIF->timeofrun->value = -1.;

 return;
}


/*--------------------------------------------- UFrIFileOnlineSim ----*/
void UFrIFileOnlineSim (struct UFrIFileOnline *iFile, long time)
/*--------------------------------------------------------------------*/
{
 long totlen, nfile;
 char *fileName;
 struct UJclock *clock;
 long iwait, isafile;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1) printf("UFrOnlineSim> still frame=%x \n", UFrCurFrameH);

 if ((iFile->iFrame-iFile->startTime) % iFile->frames_per_file == 0)
    {
     totlen   = strlen(iFile->directoryName) + strlen(iFile->fname) + 10 + strlen(iFile->suffix) ;
     fileName = (char *)calloc(totlen+1, sizeof(char));
     sprintf(fileName, "%s/%s%ld%s", iFile->directoryName, iFile->fname, iFile->iFrame + iFile->frames_per_file, iFile->suffix);
     isafile = is_a_file(fileName);
     sprintf(dummy, "I try to open next file: %s (isafile=%ld)", fileName, isafile);
     DyInfoLine(dummy);

     iwait = 0;
     while (isafile == 0 && Dy->Order != STOP)
        {sleep(1);
         iwait++;
         if (iwait%DY_TIMEOUT2 == 0) {sprintf(dummy, "Trying to get file %s since %ld seconds (no timeout)", fileName, iwait); DyInfoLine(dummy);}
         isafile = is_a_file(fileName);
         fl_check_forms();
        }

     if (Dy->Order == STOP)
        {if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1) printf("UFrIFileOnlineSim> I stop.\n");
         clock = UJCloGet();
         if(clock == NULL) UJError(3,"UFrIFileOnlineSim","no clock struct");
         clock->obj0->time = clock->totTime;
         free(fileName);
         return;
        }

     sprintf(fileName, "%s/%s%ld%s", iFile->directoryName, iFile->fname, iFile->iFrame, iFile->suffix);
     iFile->file = FrFileINew(fileName);
     fl_set_object_label(mainpanel->currentInput, fileName);
     sprintf(dummy,"Reading file %s",fileName);
     DyInfoLine(dummy);
     free(fileName);
    }

 if (UFrCurFrameH)
    {
     if (iFile->frameH)
        {iFile->frameH->rawData     = NULL;
         iFile->frameH->procData    = NULL;
         iFile->frameH->simData     = NULL;
         iFile->frameH->event       = NULL;
         iFile->frameH->summaryData = NULL;
        }
     FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 if (iFile->iFirst == 1  &&  Dy->notoc == 0)
    {
     iFile->iFirst = 0;
     UFrCurFrameH = FrameRead(iFile->file);
     if (iFile->gtime < UFrCurFrameH->GTimeS) iFile->gtime = UFrCurFrameH->GTimeS;
     FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 if (iFile->gtime >= 0.  &&  Dy->notoc == 0)
    {
     UFrCurFrameH = FrameReadTChnl(iFile->file, iFile->gtime+0.00001, iFile->itag);

     if (UFrCurFrameH)
        {iFile->gtime   = UFrCurFrameH->GTimeS + UFrCurFrameH->dt;
         iFile->framedt = UFrCurFrameH->dt;
        }
    }

 if (Dy->notoc > 0) UFrCurFrameH = FrameRead(iFile->file);


 if (UFrCurFrameH == NULL  || (Dy->skipFrame==ON && iFile->gtime<0. && Dy->notoc==0))
    {
     nfile         = (iFile->iFrame - iFile->startTime) / iFile->frames_per_file;
     iFile->iFrame = iFile->startTime + nfile * iFile->frames_per_file; 
     FrFileIEnd(iFile->file);
     printf("UFrIFileOnlineSim> UFrCurFrameH is NULL. Looking for next file:\n (frame=%ld).\n",iFile->iFrame);
     UFrIFileOnlineSim(iFile, time);
     return;
    }

 if (Dy->output_flag == ON) DyOutputWrite (UFrCurFrameH);

 iFile->lastFrameN = DyMissingFrames (iFile->lastFrameN, UFrCurFrameH);

 memcpy(iFile->frameH, UFrCurFrameH, sizeof(struct FrameH));

 iFile->iFrame++;
      
 iFile->run->value       = (double)iFile->frameH->run;
 iFile->frame->value     = (double)iFile->frameH->frame;
 iFile->length->value    = (double)iFile->frameH->dt;
 iFile->time->value      = 0.;
 if (iFile->timeofrun->value == -1.) Time0 = iFile->time->value;
 iFile->timeofrun->value = 0. - Time0;

 if (Dy->skipFrame == ON)
    {
     if (iFile->gtime >= 0.  &&  Dy->notoc == 0)
        {iFile->gtime =  FrFileITNextFrame(iFile->file, iFile->gtime+0.00001);}
    }

 return;
}


/*------------------------------------------------- UFrUnpackIni------*/
void UFrUnpackIni(unpack, dt)
struct UFrUnpack *unpack;
double dt;
/*--------------------------------------------------------------------*/
{
 char *mark2, *mark3;

 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1) printf("UFrUnpackIni> vector=%s dt=%e\n", unpack->paramName, dt);

 unpack->dt = dt; 
 unpack->i  = 0;
 unpack->previousValue = 0.;

 /*--- split the name in frame name, type of channel and variable name ---*/

 mark2 = strstr(unpack->paramName, ".");
 if (mark2 == NULL) printf("UFrUnpackIni> marker 2 is missing in %s \n", unpack->paramName);

 strncpy(bankName, unpack->paramName, mark2-unpack->paramName);
 bankName[mark2-unpack->paramName] = '\0';
 mark2++;

 if (strncmp(bankName,"sms",3) == 0)
    {
     unpack->smsPeriod = atof(bankName+3);
     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3) printf("bankName=%s  smsPeriod=%f  paramName=%s\n", bankName, unpack->smsPeriod, unpack->paramName);
     mark3 = strstr(mark2,SEP);
     if (mark3 == NULL) printf("UFrUnpackIni> marker 3 is missing in %s \n", mark2);
     unpack->smsName = (char *)calloc(mark3-mark2+1,sizeof(char));
     strncpy(unpack->smsName, mark2, mark3-mark2);
     unpack->type    = UFRSMS;
     mark2           = mark3+2;
    }
 else if (strncmp(bankName,"adc",3) == 0) unpack->type = UFRADC;
 else if (strncmp(bankName,"sim",3) == 0) unpack->type = UFRSIM;
 else if (strncmp(bankName,"rec",3) == 0) unpack->type = UFRREC;
 else printf("UFrUnpackIni> %s is unknown type\n", bankName);

 unpack->param = (char *)calloc(strlen(mark2)+1,sizeof(char));
 strcpy(unpack->param, mark2);

 unpack->nError  = 0;
 unpack->nError2 = 0;
 unpack->frameNold = -1;
 unpack->data      = NULL;

 return;
}


/*--------------------------------------------------- UFrUnpackRead---*/
void UFrUnpackRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrUnpack *unpack;

 UJ_READ_HEADER_WITH_NAME(UFrUnpack, unpack)

 UJ_READ_LONG(unpack->iClock);
 UJ_READ_S_CHAR(unpack->paramName);
 UJ_READ_STRUCT(FrameH, unpack->frameH)

 strcpy(bankName,unpack->name);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UFrUnpackRead: vector=%s (name=%s bankName=%s)\n", unpack->paramName, unpack->name, bankName);}

 unpack->out = USignalNew(strcat(bankName,".out"), iFile->ascii);

 UJCObAdd(unpack->iClock,unpack,UFrUnpackUJstructH,unpack->name);

 return;
}


/*-------------------------------------------------- UFrUnpackSim-----*/
void UFrUnpackSim
(unpack, time)            /* update UFrame number  */
struct UFrUnpack *unpack;
long time;
/*--------------------------------------------------------------------*/
{
 struct FrAdcData *adc;
 struct FrSimData *sim;
 struct FrProcData *rec;
 struct FrSerData *sms;
 int result, status=1;
 double framefreq;
 char dummy[LSTRING_MAX], fullsername[LSTRING_MAX];

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i < 10)
     {printf("UFrUnpackSim>step0> vector=%s i=%d  Dy->startTime=%ld  Dy->timeOrigin=%ld  Dy->currentTime=%f  Dy->duration=%ld  time=%ld\n",
              unpack->param, unpack->i, Dy->startTime, Dy->timeOrigin, Dy->currentTime, Dy->duration, time);
     }

 Dy->unpacki = unpack->i;

 if (Dy->Order == STOP) status = DyCheckRunning();
 if (status == 0 && ((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>0)
    {printf("UFrUnpackSim> ***stop siesta loop: unpack->i=%ld unpack->param=%s\n",unpack->i,unpack->param);}

 Dy->corrupt = 0.;

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
    {printf("UFrUnpackSim>step1> paramName=%s  dt=%f  i=%d  type=%d  frameH=%x  UFrCurFrameH=%x\n",
     unpack->paramName, unpack->dt, unpack->i, unpack->type, unpack->frameH, UFrCurFrameH);
    }

 if (unpack->frameH == NULL)
    {unpack->i++;
     unpack->out->value = DY_NOFRAME * 1.01;
     Dy->corrupt = DY_NOFRAME * 1.01;
     return;
    }

 if (UFrCurFrameH == NULL)
    {unpack->i++;
     unpack->out->value = DY_NOFRAME * 1.01;
     Dy->corrupt = DY_NOFRAME * 1.01;
     return;
    }


 /*--- first deal with sms data ---*/
		 
 if (unpack->type == UFRSMS)
    {
     DyDebugPrint("UFrUnpackSim - SerData",0);

     result = FrSerDataGet(unpack->frameH, unpack->smsName, unpack->param, &(unpack->out->value));

     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
        {printf("UFrUnpackSim>step2> paramName=%s  smsPeriod=%f  i=%d  type=%d  smsName=%s  value=%g result=%d\n",
                 unpack->paramName, unpack->smsPeriod, unpack->i, unpack->type, unpack->smsName, unpack->out->value, result);
        }

     if (result < 0)
        {
         sprintf(fullsername,"%s_%s", unpack->smsName, unpack->paramName);
         adc = FrAdcDataFind(unpack->frameH, fullsername);
         if (adc != NULL)
            {unpack->type = UFRADC;
                printf("UFrUnpackSim> name=%s is in fact an adc %s in a frame of length %g s\n", unpack->param, fullsername, unpack->frameH->dt);
             if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>0)
                printf("UFrUnpackSim> name=%s is in fact an adc %s in a frame of length %g s\n", unpack->param, fullsername, unpack->frameH->dt);
            } 
         else
            {unpack->out->value = DY_NOCHANNEL * 1.01;
             Dy->corrupt = DY_NOCHANNEL * 1.01;
             unpack->i++;
             if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
                {printf("UFrUnpackSim>step3> RESULT<0 paramName=%s=%s_%s  smsPeriod=%f  i=%d  type=%d  unpack->dt=%g  (%g Hz)  framedt=%f  previousValue=%g\n",
                         unpack->paramName, unpack->smsName, unpack->param, unpack->smsPeriod, unpack->i, unpack->type,
                         unpack->dt, 1./unpack->dt, unpack->frameH->dt, unpack->previousValue);
                }
             if (unpack->smsPeriod > unpack->frameH->dt && unpack->i < unpack->smsPeriod)
                {unpack->out->value = unpack->previousValue;
                 Dy->corrupt = 0.;
                }
            }
        }

     else
        {if (unpack->i > unpack->smsPeriod)
            {if (unpack->smsPeriod > 0) printf("UFrUnpackSim> Sms channel %s_%s was absent %g times\n", unpack->smsName, unpack->param, unpack->i/unpack->smsPeriod);
             else                       printf("UFrUnpackSim> Sms channel %s_%s was absent %ld times\n", unpack->smsName, unpack->param, unpack->i);
            }
         if (strcmp(unpack->name,"signal1") == 0)
            {Dy->currentTime = unpack->frameH->GTimeS + unpack->i * unpack->frameH->dt;
             DyXformUpdateGTime(Dy->currentTime, unpack->frameH->ULeapS);
            }
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
            {printf("UFrUnpackSim>step4> RESULT=0 paramName=%s=%s_%s  smsPeriod=%f  i=%d  type=%d  unpack->dt=%g  (%g Hz)  framedt=%f  previousValue=%g  value=%g\n",
                     unpack->paramName, unpack->smsName, unpack->param, unpack->smsPeriod, unpack->i, unpack->type,
                     unpack->dt, 1./unpack->dt, unpack->frameH->dt, unpack->previousValue, unpack->out->value);
            }
         unpack->i = 0;
         unpack->previousValue = unpack->out->value;
        } 

     return;
    }
  
 /*--- now look for ADC, SIM or PROC data ---*/
 /*--- update pointer on data vector only when frame number or GPS time changes ---*/

 if (unpack->frameH->frame != unpack->frameNold || unpack->frameH->GTimeS+1e-9*unpack->frameH->GTimeN != unpack->GPSold+1e-9*unpack->GPSNold)
    {
     unpack->data = NULL;	   				 
     if (unpack->type == UFRADC)
        {
         DyDebugPrint("UFrUnpackSim - AdcData",0);
         adc = FrAdcDataFind(unpack->frameH, unpack->param);
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>2) printf("UFrUnpackSim> name=%s adc=%x\n", unpack->param, adc);
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>100) FrameDump(unpack->frameH, stdout,2);
         if (adc != NULL) unpack->data = adc->data;
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>2 && adc != NULL) printf("name=%s adc->data=%x\n", unpack->param, adc->data);
        }
     else if (unpack->type == UFRSIM)
        {
         DyDebugPrint("UFrUnpackSim - SimData",0);
         sim = FrSimDataFind(unpack->frameH, unpack->param);
         if (sim != NULL) unpack->data = sim->data;
        }
     else if (unpack->type == UFRREC)
        {
         DyDebugPrint("UFrUnpackSim - ProcData",0);
         rec = FrProcDataFind(unpack->frameH, unpack->param);
         if (rec == NULL) rec = FrProcDataFind(UFrCurFrameH, unpack->param);
         if (rec != NULL) unpack->data = rec->data;
         else             unpack->data = FrameFindVect(UFrCurFrameH, unpack->param);
        }
     if (unpack->data == NULL)
        {if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>0)
            {printf("UFrUnpackSim> Data vector %s not found in frame %d (gps%d)\n", unpack->param, unpack->frameH->frame, unpack->frameH->GTimeS);}
        }
     unpack->frameNold = unpack->frameH->frame;
     unpack->GPSold    = unpack->frameH->GTimeS;
     unpack->GPSNold    = unpack->frameH->GTimeN;
    }

 /*--- Keep in a global variable the pointer to data vector ---*/
 UFrCurVect = unpack->data;


 /*--- If data vector absent, put DY_NOCHANNEL value in the returned values ---*/ 
 if (unpack->data == NULL)
    {unpack->out->value = DY_NOCHANNEL * 1.01;
     Dy->corrupt = DY_NOCHANNEL * 1.01;
     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
        {printf("UFrUnpackSim00: value=%g i=%d frame=%d \n", unpack->out->value, unpack->i, unpack->frameH->GTimeS);}
    }

 else
    {
/**
printf("unpack->i=%d  unpack->frameH->GTimeS=%d unpack->frameH->GTimeN=%d unpack->data->dx=%x unpack->data->name=%s\n",
        unpack->i, unpack->frameH->GTimeS, unpack->frameH->GTimeN, unpack->data->dx, unpack->data->name);
**/
     if (strcmp(unpack->name,"signal1") == 0)
        {Dy->currentTime = unpack->frameH->GTimeS + unpack->data->dx[0] * unpack->i;
         if (Dy->currentTime - Dy->previousTime > unpack->frameH->dt / 10.)
            {Dy->previousTime = Dy->currentTime;
             DyXformUpdateGTime(Dy->currentTime, unpack->frameH->ULeapS);
            }
        }

     /*--- Check validity of data vector when first sampling is read ---*/ 
     if (unpack->i == 0 || unpack->i >= unpack->data->nData)
        {unpack->valid = FrVectIsValid(unpack->data);
         if (unpack->valid != 0)
            {sprintf(dummy, "Corrupted data in vector %s somewhere between GPS=%d and GPS=%d",
                     unpack->data->name, Dy->currentTime, Dy->currentTime + (int)(unpack->data->dx[0] * unpack->data->nData));
             DyInfoLine(dummy);
            }
        }

     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
        {printf("UFrUnpackSim01: nData=%d i=%d frame=%d \n", unpack->data->nData, unpack->i, unpack->frameH->GTimeS);}

     if (unpack->i >= unpack->data->nData)
        {unpack->i = (unpack->i)%(unpack->data->nData);}

     /*--- If sampling of data vector is valid, get the sampling value, except if auxiliary vector shows that sampling is missing ---*/
     if (unpack->data->next != NULL && strncmp(unpack->data->next->name,"Available_data",14) == 0 && unpack->data->next->data[unpack->i] == 0)
        {unpack->out->value = DY_NOCHANNEL * 1.01;
         Dy->corrupt = DY_NOCHANNEL * 1.01;
         if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>4)
            {printf("UFrUnpackSim> NOCHANNEL %s i=%d frame=%d \n", unpack->data->name, unpack->i, unpack->frameH->GTimeS);}
        }

     else if (unpack->valid != 0 && unpack->data->next != NULL && unpack->data->next->data[unpack->i] == -1)
        {unpack->out->value = DY_BADDATA * 1.01;
         Dy->corrupt = DY_BADDATA * 1.01;
          if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
             {printf("UFrUnpackSim03: value=%g i=%d frame=%d \n", unpack->out->value, unpack->i, unpack->frameH->GTimeS);}
        }

     else
        {
         if (unpack->data->type == FR_VECT_8R)  unpack->out->value = unpack->data->dataD[unpack->i];
         if (unpack->data->type == FR_VECT_4R)  unpack->out->value = unpack->data->dataF[unpack->i];
   
         if (unpack->data->type == FR_VECT_C)   unpack->out->value = unpack->data->data[unpack->i];
         if (unpack->data->type == FR_VECT_2S)  unpack->out->value = unpack->data->dataS[unpack->i];
         if (unpack->data->type == FR_VECT_8S)  unpack->out->value = unpack->data->dataL[unpack->i];
         if (unpack->data->type == FR_VECT_4S)  unpack->out->value = unpack->data->dataI[unpack->i];
        }

     if (unpack->out->value > 0  &&  unpack->out->value <= 2.e-31)
        {unpack->out->value = 2.e-31;}

     if (unpack->out->value < 0  &&  unpack->out->value >= -2.e-31)
        {unpack->out->value = -2.e-31;}

     if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>3 && unpack->i<10)
        {printf("UnpackSim02: value=%g i=%d frame=%d \n", unpack->out->value, unpack->i, unpack->frameH->GTimeS);}
    }

 (unpack->i)++;
 if (unpack->data != NULL && unpack->i >= unpack->data->nData) unpack->i = 0;
 
 return;
}


/*-------------------------------------------------- UFrUnpackEnd-----*/

void UFrUnpackEnd(struct UFrUnpack *unpack)
{
 DyDebugPrint(__FUNCTION__,0);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {printf("UnpackEnd: unpack->i=%ld  currentTime=%f\n", unpack->i, Dy->currentTime);}

 DyXformUpdateGTime(Dy->currentTime, unpack->frameH->ULeapS);

 if (unpack->param)      free(unpack->param);
 if (unpack->paramName)  free(unpack->paramName);
 if (unpack->smsName)    free(unpack->smsName);
 if (unpack->name)       free(unpack->name);
  
 free(unpack);
 
 return;
}


/*--------------------------------------------------- UFrShmIni------*/
void UFrShmIni (struct UFrShm *iFile, double dt)
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 int iwait;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 iwait = 0;
 Dy->Order = WAIT;
 while ((iFile->shm = FdShmOpen (iFile->producerName, Dy->CmName)) == NULL  &&  Dy->Order != STOP)
    {fl_check_forms();
     if (iwait%DY_TIMEOUT2 == 0)
        {sprintf(dummy,"Could not open shm %s since %d sec", iFile->producerName, iwait);
         DyInfoLine(dummy);
        }
     sleep(1);
     iwait++;
    }

 if (Dy->Order == STOP)
    {
     iFile->frameH->rawData = NULL;
/*
     clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrShmIni","no clock struct");
     clock->obj0->time = clock->totTime;
*/
    }
 else
    {
     sleep(1);
     FdShmCsmIni(iFile->shm, &(iFile->iread));
    }

 if (Dy->Frame) Dy->Framedt = Dy->Frame->dt;
 else           Dy->Framedt = 1.;

 if (UFrCurFrameH)
    {FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 return;
}


/*--------------------------------------------------- UFrShmEnd------*/
void UFrShmEnd (struct UFrShm *iFile)
/*--------------------------------------------------------------------*/
{
 DyDebugPrint(__FUNCTION__,0);

 FdShmFree (iFile->shm, Dy->CmName);
 free(iFile->producerName);
 free(iFile->name);
 
 return;
}


/*-------------------------------------------------- UFrShmRead------*/
void UFrShmRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UFrShm *dIF;

 UJ_READ_HEADER_WITH_NAME(UFrShm,dIF)
 UJ_READ_LONG(dIF->iClock);
 UJ_READ_S_CHAR(dIF->producerName);

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1) printf("UFrShmRead> producer=%s \n",dIF->producerName);
  
 UJCObAdd(dIF->iClock,dIF, UFrShmUJstructH, dIF->name);

 dIF->frameH = FrameHNew("dummy");

 strcpy(bankName, dIF->name);
 strcat(bankName,".frameH");
 UJSetBA(bankName,"FrameH", dIF->frameH);

 strcpy(bankName, dIF->name);

 return;
}


/*--------------------------------------------------- UFrShmSim ----*/
void UFrShmSim (struct UFrShm *iFile, long clocktime)
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 int nBytes;
 static int frameAnalysed;
 double currentTime, dtime;
 char dummy[32], dummy2[LSTRING_MAX];
 int latency, diff_latency, iwait, iwait2;
 struct ShmInfo *shm;

 DyDebugPrint(__FUNCTION__,0);

 if (UFrCurFrameH)
    {
     if (iFile->frameH)
        {iFile->frameH->rawData     = NULL;
         iFile->frameH->procData    = NULL;
         iFile->frameH->simData     = NULL;
         iFile->frameH->event       = NULL;
         iFile->frameH->summaryData = NULL;
        }
     FrameFree(UFrCurFrameH);
     UFrCurFrameH = NULL;
    }

 /*--- Check the time passed outside waiting for FdShmRead not null ---*/

 gettimeofday(&dytv, NULL);
 currentTime = dytv.tv_sec + (dytv.tv_usec * 1.0e-6);
 dtime = currentTime - Dy->oldTime;

 if (Dy->oldTime > 0.)
    {Dy->nAnaTime++;
     Dy->AnaTime += dtime;
     if (dtime > DTMAX  &&  ((Dy->debugType)&(1<<DBG_SPEED)) && Dy->debug>0)
        {printf("UFrShmSim> time to analyse frame %d = %.3fs ", frameAnalysed, dtime);
         if (Dy->nAnaTime > 0) printf(" (mean=%.3fs)\n", Dy->AnaTime/Dy->nAnaTime);
         else                  printf("\n");
        }
    }

 if (Dy->AnaTime > 30.)
    {sprintf(dummy,"Mean time to analyse one frame of %.3fs from %s = %.3fs", Dy->Framedt, iFile->producerName, Dy->AnaTime/Dy->nAnaTime);
     DyInfoLine(dummy);
     if (Dy->AnaTime / Dy->nAnaTime > FRTIME_MAX*Dy->Framedt)
        {Dy->Order = STOP;}
     else
        {Dy->nAnaTime = 0; Dy->AnaTime = 0.;}
    }

 /*--- Read frame from shared memory ---*/

 iwait = 1;
 while ((UFrCurFrameH = FdShmRead(iFile->shm, &(iFile->iread), -1, &nBytes)) == NULL  &&  Dy->Order != STOP) 
    {usleep(100000);
     fl_check_forms();
     if (iwait%(10*DY_TIMEOUT2) == 0)
        {iwait2 = 0;
         printf("UFrShmSim> Could not get frame since %d sec. I try to reconnect to shm %s\n", iwait/10, iFile->producerName);
         while ((shm = FdShmOpen(iFile->producerName, Dy->CmName)) == NULL && Dy->Order != STOP)
              {sleep(1);
               fl_check_forms();
               if (iwait2%DY_TIMEOUT2 == 0)
                  {sprintf(dummy2,"Could not get frame. I try to reconnect to shm %s since %d sec", iFile->producerName, iwait2);
                   DyInfoLine(dummy2);
                  }
                iwait2++;
              }
         if (Dy->Order != STOP) {FdShmFree(iFile->shm, Dy->CmName); iFile->shm = shm; sleep(1); FdShmCsmIni(iFile->shm, &(iFile->iread));}
        }
     iwait++;
    }

 if (((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
    {if (UFrCurFrameH) printf("UFrShmSim> iread=%ld frame=%d \n",iFile->iread,UFrCurFrameH->GTimeS);}

 /*--- Update oldTime and compute latency ---*/

 if (UFrCurFrameH) frameAnalysed = UFrCurFrameH->frame;
 gettimeofday(&dytv, NULL);
 Dy->oldTime = dytv.tv_sec + ( dytv.tv_usec * 1.0e-6 );

 if (UFrCurFrameH) 
    {
     Dy->Framedt = UFrCurFrameH->dt;
     latency = time(NULL) - FRGPSOFF - FRGPSTAI + UFrCurFrameH->ULeapS - UFrCurFrameH->GTimeS;
     if (Dy->start_latency == 0) Dy->start_latency = latency;
     diff_latency = latency - Dy->start_latency;
     if (diff_latency > 2  &&  ((Dy->debugType)&(1<<DBG_FRAME)) && Dy->debug>1)
        {printf("UFrShmSim> Current latency=%ds  (%ds with respect to start latency)", latency, diff_latency);}
     sprintf(dummy, "%d", latency);
     fl_set_input(mainpanel->latency, dummy);
    }

 if (Dy->Order == STOP)
    {clock = UJCloGet();
     if (clock == NULL) UJError(3,"UFrShmSim","no clock struct");
     clock->obj0->time = clock->totTime;
     return;
    }

 iFile->lastFrameN = DyMissingFrames (iFile->lastFrameN, UFrCurFrameH);

 memcpy(iFile->frameH, UFrCurFrameH, sizeof(struct FrameH)); 

 return;
}


/*-------------------------------------------------- UFrVectOpEnd-----*/

void UFrVectOpEnd (struct UFrVectOp *vectop)
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 for (i=0; i<vectop->nvect; i++)
    {if (vectop->inputName[i])  free(vectop->inputName[i]);}
 if (vectop->inputPar)    free(vectop->inputPar);
 if (vectop->inputVect)   free(vectop->inputVect);
 if (vectop->outName)     free(vectop->outName);
 if (vectop->className)   free(vectop->className);
 if (vectop->name)        free(vectop->name);
  
 free(vectop);
 
 return;
}


/*-------------------------------------------------- UFrVectOpRead------*/

void UFrVectOpRead (struct UJfile *iFile)
{
 struct UFrVectOp *dIF;
 int i, nword;
 char *word, *str, chaine[LBIGSTRING_MAX];

 UJ_READ_HEADER_WITHOUT_NAME(UFrVectOp,dIF)
 UJ_READ_LONG(dIF->iClock);
 UJ_READ_LONG(dIF->id)
 if (!dIF->name) dIF->name = (char *) calloc(10,sizeof(char));
 sprintf(dIF->name,"vectop%ld",dIF->id);
 UJ_READ_S_CHAR(dIF->className);
 UJ_READ_S_CHAR(dIF->inputVect);
 UJ_READ_S_CHAR(dIF->inputPar);
 UJ_READ_STRUCT(FrameH, dIF->frameH)

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("UFrVectOpRead:  algo=%s  vectors=%s\n", dIF->className, dIF->inputVect);}

 UJCObAdd(dIF->iClock,dIF, UFrVectOpUJstructH, dIF->name);

 /*--- Get list of input vector names and nvect ---*/

 strcpy(chaine,dIF->inputVect);
 word = strtok(chaine,OPSEP);
 nword = 0;
 while (word != NULL)
    {
     dIF->inputName[nword] = (char *)malloc(strlen(word)+1);
     strcpy(dIF->inputName[nword],word);
     if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2)
        {printf("UFrVectOpRead> inputName[%d]=%s  inputVect=%s\n", nword, dIF->inputName[nword], dIF->inputVect);}
     nword++;
     word = strtok(NULL,OPSEP);
    }
 dIF->nvect = nword;


 /*--- Get list of input parameters ---*/

 dIF->realUnits = 0;
 if (strstr(dIF->inputPar,"REALUNITS") != NULL) dIF->realUnits = 1;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2)
    {printf("UFrVectOpRead> inputPar=%s  realUnits=%d \n", dIF->inputPar, dIF->realUnits);}

 if (dIF->inputPar  &&  strlen(dIF->inputPar) > 2  &&  strstr(dIF->inputPar,"NULL") == 0)
    {
     strcpy(chaine,dIF->inputPar);
     word = strtok(chaine,"_");
     nword = 0;
     while (word != NULL)
        {sscanf(word,"%lf",&(dIF->inputParam[nword]));
         nword++;
         printf("UFrVectOpRead> wordPARAM=%s\n",word);
         word = strtok(NULL,"_");
        }
    }
 else
    {if (dIF->inputPar) free(dIF->inputPar);
     dIF->inputPar = NULL;
    }

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3)
    {for (i=0; i<dIF->nvect; i++) printf("UFrVectOpRead> inputName[%d]=%s \n",i,dIF->inputName[i]);}

 return;
}


/*--------------------------------------------------- FrVectOpIni------*/

void UFrVectOpIni (struct UFrVectOp *vectop, double dt)
{
 int i;
 double sampleRate;
 struct FrVect *vect;
 struct FrProcData *proc;
 char unitY[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (vectop->inputVect == NULL) return;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0)
    {printf("UFrVectOpIni for algo=%s vectors=%s\n", vectop->className, vectop->inputVect);}

 if (UFrCurFrameH == NULL) UFrCurFrameH = vectop->frameH;

 if (UFrCurFrameH->rawData == NULL) UFrCurFrameH = FrameCopy(Dy->Frame);

 vectop->outName = (char*)malloc(strlen(OPSIGNAL)+strlen(vectop->className)+strlen(vectop->inputVect)+LSTRING_MAX);
 strcpy(vectop->outName,OPSIGNAL);
 strcat(vectop->outName,vectop->className);

 /*--- Get nData max ---*/

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2)
    {printf("UFrVectOpIni> vectop->nvect=%d  vectop->inputName[0]=%s  UFrCurFrameH=%x\n", vectop->nvect, vectop->inputName[0], UFrCurFrameH);}

 vectop->nData = 0;
 for (i=0; i<vectop->nvect; i++)
    {
     strcat(vectop->outName,OPSEP);
     strcat(vectop->outName,vectop->inputName[i]);
    }

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_CH)
    {Dy->triggerEvent = OFF;
     vectop->lastValue = 0;
    }

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>1)
    {printf("UFrVectOpIni> vectop->nData=%d vectop->outName=%s\n",vectop->nData, vectop->outName);}

 return;
}


/*-------------------------------------------------- UFrVectOpSim-----*/

void UFrVectOpSim (struct UFrVectOp *vectop, long time)
{
 struct FrVect *invects[64], *outvect, *vectnew[64];
 struct FrProcData *proc;
 struct FrAdcData *adc;
 struct FrSerData *sms;
 double sampleRate;
 int i, k, i0, nvnew, nData;
 double data_available, value;
 char unitY[LSTRING_MAX];
 char *smsName=NULL, *varName=NULL, *adcName=NULL, *procName=NULL, *simName=NULL;

 DyDebugPrint(__FUNCTION__,0);

 if (UFrCurFrameH == NULL) return;
 if (vectop == NULL)       return;

 if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>2) printf("UFrVectOpSim>  algo=%s  vectors=%s\n", vectop->className, vectop->inputVect);

 nvnew = 0;
 data_available = 1.;

 for (i=0; i<vectop->nvect; i++)
    {
     invects[i] = FrameFindVect (UFrCurFrameH, vectop->inputName[i]);

     if (invects[i] == NULL)
        {
         DyDecodeVar(vectop->inputName[i], &smsName, &varName, &adcName, &procName, &simName);
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3) printf("UFrVectOpSim> inputName[%d]=%s  smsName=%s \n", i, vectop->inputName[i], smsName);
         sms = FrSerDataFind(UFrCurFrameH,smsName,NULL);
         if (sms != NULL)
            {FrSerDataGet(UFrCurFrameH, smsName, varName, &value);
             invects[i] = FrVectNew1D(vectop->inputName[i], FR_VECT_8R, 1, 1./sms->sampleRate, "time", NULL);
             invects[i]->dataD[0] = value;
             if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>3) printf("UFrVectOpSim> invect[%d]=%s nData=%d  dx=%f \n", i, invects[i]->name, invects[i]->nData, invects[i]->dx[0]);
            }
         else
            {data_available = DY_NOCHANNEL;
             if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("UFrVectOpSim> channel %s absent in frame %d\n", vectop->inputName[i], UFrCurFrameH->GTimeS);
             invects[i] = FrVectNew1D(vectop->inputName[i], FR_VECT_8R, 1, UFrCurFrameH->dt, "time", NULL);
             invects[i]->dataD[0] = 0.;
             continue;
            }
         if (smsName)  {free(smsName);  smsName  = NULL;}
         if (varName)  {free(varName);  varName  = NULL;}
         if (adcName)  {free(adcName);  adcName  = NULL;}
         if (procName) {free(procName); procName = NULL;}
         if (simName)  {free(simName);  simName = NULL;}
        }

     if (FrVectIsValid(invects[i]) != 0)
        {data_available = DY_BADDATA;
         if (((Dy->debugType)&(1<<DBG_CHANNEL)) && Dy->debug>0) printf("UFrVectOpSim> channel %s corrupted in frame %d\n", vectop->inputName[i], UFrCurFrameH->GTimeS);
         continue;
        }

     if (vectop->realUnits)
        {
         adc = FrAdcDataFind(UFrCurFrameH, vectop->inputName[i]);
         if (adc != NULL)
            {
             vectnew[nvnew] = FrVectNew1D (invects[i]->name, FR_VECT_8R, invects[i]->nData, 
                                           invects[i]->dx[0], invects[i]->unitX[0], invects[i]->unitY);
             for (k=0; k< vectnew[nvnew]->nData; k++)
                {vectnew[nvnew]->dataD[k] = adc->slope * FrVectGetV(invects[i],k) + adc->bias;}
             invects[i] = vectnew[nvnew];
             nvnew++;
            }
        }
    }

 nData = 0;
 strcpy(unitY,"NONE");
 for (i=0; i<vectop->nvect; i++)
    {if (invects[i] == NULL) continue;
     if (strcmp(unitY,"NONE") == 0 && (invects[i])->unitY != NULL) strcpy(unitY,(invects[i])->unitY);
     if (nData < (invects[i])->nData) nData = (invects[i])->nData;
    }
 if (nData > vectop->nData) vectop->nData = nData;

 if (UFrCurFrameH->dt > 0) sampleRate = (double)(vectop->nData) / UFrCurFrameH->dt;
 else                      sampleRate = (double)(vectop->nData);

 proc = FrProcDataFind(UFrCurFrameH, vectop->outName);
 if (proc == NULL)
    {proc = FrProcDataNew (UFrCurFrameH, vectop->outName, sampleRate, vectop->nData, -64);
     FrStrCpy(&(proc->data->unitY),unitY);
    }
 outvect = proc->data;
 for (i=0; i<vectop->nData; i++) outvect->dataD[i] = 0;

 if (data_available == 1.)
    {
     if (vectop->inputPar == NULL)
        {RA_UserProcess (vectop->className, vectop->nvect, invects, outvect, NULL);}
     else
        {RA_UserProcess (vectop->className, vectop->nvect, invects, outvect, vectop->inputParam);}
    }
 else
    {for (i=0; i<vectop->nData; i++) outvect->dataD[i] = data_available;
     printf("UFrVectOpSim> frame=%d (%d) : I put all %d data of %s to %g\n",
             UFrCurFrameH->frame, UFrCurFrameH->GTimeS, outvect->nData, outvect->name, data_available);
    }

 if (Dy->triggerUsed == ON && Dy->triggerType == TRIG_CH)
    {
     if (Dy->triggerVect) {FrVectFree(Dy->triggerVect); Dy->triggerVect = NULL;}
     Dy->triggerVect = FrVectNew1D ("Dytrigger", FR_VECT_C, outvect->nData, outvect->dx[0], outvect->unitX[0], outvect->unitY);
     Dy->triggerEvent = OFF;
 
     for (i=0; i<vectop->nData; i++)
        {
         if (outvect->dataD[i] == 1.)
            {
             if ((i> 0 && outvect->dataD[i-1] == 0) || (i==0 && outvect->dataD[i] == 1 && vectop->lastValue == 0))
                {printf("UFrVectOpSim> *** Start trigger=%s --> startTime=%.4f : frame %d (%d) i=%d nData=%d realUnits=%d value=%g\n",
                        Dy->lastTrigString, UFrCurFrameH->GTimeS + i * outvect->dx[0], UFrCurFrameH->frame, UFrCurFrameH->GTimeS,
                         i, vectop->nData, vectop->realUnits, FrVectGetV(invects[0],i));
                }
             Dy->triggerEvent = ON;
             Dy->triggerVect->data[i] = 1;
            }
         else
            {
             if ((i> 0 && outvect->dataD[i-1] == 1.) || (i == 0 && outvect->dataD[i] == 0. && vectop->lastValue == 1.))
                {printf("UFrVectOpSim> *** Stop trigger=%s --> endTime=%.4f : frame %d (%d) i=%d nData=%d realUnits=%d value=%g\n\n",
                        Dy->lastTrigString, UFrCurFrameH->GTimeS + i * outvect->dx[0], UFrCurFrameH->frame, UFrCurFrameH->GTimeS,
                        i, vectop->nData, vectop->realUnits, FrVectGetV(invects[0],i));
                }
             Dy->triggerVect->data[i] = 0;
            }
        }

     vectop->lastValue = outvect->dataD[vectop->nData-1];
    }

 for (i=0; i<nvnew; i++) FrVectFree(vectnew[i]);

 return;
}


