/**************************************************/
/*                                                */
/* File: UJobL.cc                                 */
/* Copyright (C) 2003, B. Mours, D. Verkindt      */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/* by B. Mours LAPP (Annecy)  */
/* 20 June 1999, minor modifications to UJError (A.Vicere', INFN Pisa) */
/* 08 Dec 1999, modifications to UJLibIni and UJCobSim (D. Verkindt, Annecy) */
/* 19 Nov 2002, modifications to UJCobSim (D. Verkindt, Annecy) */
 
char UJversion[] = "Nov 19, 2002";
 
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include "UJobL.h"
#include "USignalL.h"
#include "URootAccess.h"
#include "dataDisplay.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif

char bankName[UJBANKNAMESIZE];  /* local variables used for Read functions */
char UJbuf[UJBANKNAMESIZE];     /* local variables used for Read functions */

#define UJMSGMAX 1000
char UJmessage[UJMSGMAX];

struct UJobH UJobH;
struct UJxref *UJfirstXref = NULL;

short  UJ_short;
float  UJ_float;
double UJ_double;
int    UJ_id, UJ_int, UJ_type, UJ_instance, UJ_i, UJ_strlen;
char   UJ_char, *UJ_aint, *UJ_ashort, *UJ_achar;

time_t UJtimeStart;
char bankType[500];

struct UJclock *currentClock = NULL;
struct UJstructH *firstUJstructH = NULL;
struct UJstructH *UJclockUJstructH;
struct UJstructH *UJdelayUJstructH;
struct UJstructH *UJstructHUJstructH;
struct UJstructH *UJtimeUJstructH;

struct UJstructH *UJsHlist[UJMAPMAXSH];
long   UJnStructH=0;

BOOL   UJpauseAtTheEnd = NO;

struct UJtime *timeRef;


/*----------------------------------- Interaction with dataDisplay ---*/
/***
int DyCheckRunning()
{
 return(1);
}
***/


/*------------------------------------------------------- UJLibIni-*/
void UJLibIni(jobName, argc, argv)         
char *jobName;
long argc;
char *argv[];
/*--------------------------------------------------------------------*/
{
/*   optional parameter 1: cards file   (default = jobName)
     optional parameter 2: histo file   (default = param 1)
     optional parameter 3: listing file (default = param 2) 
     optional parameter 4: debug level  (default = 1)
*/

 /*--- deal with file name ---*/

 if (argc == 0) strcpy(UJobH.cards,"TERMINAL");

 if (argc > 1)  strcpy(UJobH.cards,argv[1]);
 else           strcpy(UJobH.cards,jobName);
 if (argc > 2)  strcpy(UJobH.histo,argv[2]);
 else           strcpy(UJobH.histo,UJobH.cards);
 if (argc > 3)  strcpy(UJobH.listing,argv[3]);
 else           strcpy(UJobH.listing,UJobH.histo);
 if (argc > 4)  sscanf(argv[4],"%ld",&UJobH.debugLvl);
 else           UJobH.debugLvl = 1;

 strcat(UJobH.cards,".car");
 strcat(UJobH.listing,".lis");
 strcat(UJobH.histo,".hsto");

 UJtimeStart = time(NULL);

 printf(" Start %s (%.24s)\n ",   jobName, ctime(&UJtimeStart));
/***
 printf("       cards file:%s\n", UJobH.cards);
 printf("     listing file:%s\n", UJobH.listing); 
 printf("       histo file:%s\n", UJobH.histo); 
 printf("      Debug level:%ld\n",UJobH.debugLvl);
 printf(" Output will be sent on file:%s\n", UJobH.listing);

 if (strcmp("TERMINAL.lis",UJobH.listing) != 0)
    {UJobH.fOut = fopen(UJobH.listing,"w");
     if (UJobH.fOut == NULL)
        {printf("\n !! Opening file error, output stay on screen\n");
         UJobH.fOut = stdout;
        }
    }
 else
    {UJobH.fOut = stdout;}

/***
 fprintf(UJobH.fOut," Start job: %s\n",ctime(&UJtimeStart));
 fprintf(UJobH.fOut,"   UJobL     Version:%s(%s%s %s)\n"
        ,UJversion,__DATE__,__TIME__,__FILE__);
***/

 /*--- i/o package initialization ---*/
	
 UJ_aint   = (char *) &UJ_int;
 UJ_ashort = (char *) &UJ_short;
 UJ_achar  = (char *) &UJ_char;
	
 UJclockUJstructH   = UJDicAddS("UJclock", NULL, NULL, NULL, UJclockRead, UJclockFree);
 UJdelayUJstructH   = UJDicAddS("UJdelay", NULL, NULL, NULL, UJdelayRead, NULL);
 UJstructHUJstructH = UJDicAddS("UJstructH", NULL, NULL, NULL, NULL, NULL);
 UJtimeUJstructH    = UJDicAddS("UJtime", NULL, NULL, NULL, UJtimeRead, NULL);

 timeRef = (struct UJtime *)malloc(sizeof(struct UJtime));

 timeRef->origin      = mktime(gmtime(&UJtimeStart)) - 315964800 + 7212;
 timeRef->current     = timeRef->origin;
 timeRef->originChar  = FrStrGTime(timeRef->origin);
 timeRef->currentChar = FrStrGTime(timeRef->current);

 UFrLibIni();
/*  UHLibIni(); */ 
 USLibIni();

 return;
} 

   
/*---------------------------------------------------- UJCardRead ----*/
void UJCardRead()
/*--------------------------------------------------------------------*/
{
 BOOL comment,found;
 long i,ibnk,iunk,irc,len;
 struct UJfile *dIF;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"\n Start UJCardRead on file %s\n",UJobH.cards);}

 dIF = UJfileNew(UJobH.cards,YES);
   
 ibnk = 0;
 iunk = 0;
 comment = NO;
 
 /*------ copy input cards to output file -----*/

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>1)
    {
     dIF->fp = fopen(UJobH.cards,"r");
     if(dIF->fp == NULL) UJError(3,"UJCardRead","\n !!!! Error in opening card file !!!!\n\n");
     fprintf(UJobH.fOut,"\n >>>>>> copy of input card file: <<<<<<<<<<< \n");
     while(fgets(bankName, UJBANKNAMESIZE, dIF->fp) != NULL) {fprintf(UJobH.fOut,"%s", bankName);}
     fprintf(UJobH.fOut,"\n >>>>>>>>>>> end of copy <<<<<<<<<<<<<<<<<<< \n");
     fclose(dIF->fp);
    }
	  
 /*----- now read all banks */
     
 comment = NO;

 dIF->fp = fopen(UJobH.cards,"r");

 while ((irc = fscanf(dIF->fp, "%s",bankType)) > 0 )
    {
     if (bankType[0] == '/')
	{if (bankType[1] == '*') comment = YES;}
		 
     len = strlen(bankType);
     if (len >= 2)
	{if (bankType[len-1] == '/')
	    {if (bankType[len-2] == '*')
                {comment = NO;
                 continue;
                }
            }
        }
     if (comment == YES) continue;
 
     /*--- tempory warning ---*/

     if (strcmp(bankType,"FrameH") == 0)
        {UJError(3,"UJCardRead","FrameH is an obsolete card. Remove it and restart");}
     if (strcmp(bankType,"FrDetector") == 0)
        {UJError(3,"UJCardRead","FrDetector is an obsolete card. Remove it and restart");}

     ibnk++;
     found = NO;

     for (i=0; i<UJnStructH; i++)
        {if (strcmp(bankType,UJsHlist[i]->type) == 0)
            {UJsHlist[i]->objRead(dIF);
             found = YES;
             break;
            }
        }

     if (found == NO)
        {fprintf(UJobH.fOut,"unknown bank:%s\n",bankType);
         UJError(3,"UJCardRead","unknown bank");
         ibnk--;
         iunk++;
        }
    }
 
 UJSetAll();      /*--- this sets all pointers ---*/
 
 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
     {fprintf(UJobH.fOut,"\n End of UJCardRead, %ld banks read, %ld unknown words\n\n",ibnk,iunk);}
 
 if (iunk > 0)  UJError(3,"UJCardRead","unknown banks");
  
 fclose(dIF->fp);

 UJfileFree(dIF);

 return;
}

 
/*---------------------------------------------------- UJCloGet -----*/
struct UJclock *UJCloGet()
/*--------------------------------------------------------------------*/
{
 return(currentClock);
}


/*---------------------------------------------------- UJCloSet -----*/
void UJCloSet(clock)           
struct UJclock *clock;
/*--------------------------------------------------------------------*/
{
 currentClock = clock;
 return;
}


/*---------------------------------------------------- UJCObAdd ----*/
void UJCObAdd(iClock, object, type, name)           
long iClock;
void *object;
char *name;
struct UJstructH *type;
/*--------------------------------------------------------------------*/
{
 struct UJclockO *clockO, *lastClockO;
 struct UJclock *clock;
  
 clockO = (struct UJclockO *)malloc(sizeof(struct UJclockO));
 if (clockO == NULL)  UJError(3,"UJCObAdd","malloc failed");

 clockO->object = object;
 clockO->type   = type;
 clockO->name   = name;
 clockO->iClock = iClock;
 clockO->next   = NULL;

 clock = currentClock;
 if (abs(iClock)+1 > clock->nClock)  printf("clock not available for %s (iClock=%ld)\n", name,iClock);
 if (abs(iClock)+1 > clock->nClock)  UJError(3,"UJCObAdd","clock not available");

 if (clock->clockO == NULL)
    {clock->clockO = clockO;}
 else
    {lastClockO       = clock->clockO;
     while (lastClockO->next != NULL)  lastClockO = lastClockO->next;
     lastClockO->next = clockO;
    }

 return;
}


 /*----------------------------------------------------- UJCObEnd ----*/
void UJCObEnd()
/*--------------------------------------------------------------------*/
{
 struct UJclockO *clockO;
 struct UJclockO *clockI;

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"  Start UJCObEnd after %d sec \n", time(NULL)-UJtimeStart);}
    
 if (currentClock == NULL)  UJError(3,"UJCObEnd","no clock struct");

 clockO = currentClock->clockO;

 while (clockO != NULL)
    {if (clockO->type->objEnd != NULL)  clockO->type->objEnd(clockO->object);
     clockI = clockO->next;
     free(clockO);
     clockO = clockI;
    }

 if (currentClock->name)  free(currentClock->name);
 if (currentClock)        free(currentClock);

 return;
}


/*----------------------------------------------------- UJCObIni ----*/
void UJCObIni()
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 struct UJclockO *clockO;
 long i,iClock;
 double fMax;
 char dummy[1024];

 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"  Start UJCObIni after %d sec \n", time(NULL)-UJtimeStart);}
    
 /*--- initialize clock parameters ---*/
		
 clock = currentClock;
 if (clock == NULL)  UJError(3,"UJCObIni","no clock struct");

 fMax = 0.;

 for (i=0; i<clock->nClock; i++)
    {if (clock->frequency[i] < fMax) continue;
     clock->iFast = i;
     fMax = clock->frequency[i];
    }

 for (i=0; i<clock->nClock; i++)
    {clock->nTic[i]      = (int)(0.5+clock->frequency[clock->iFast] / clock->frequency[i]);
     clock->roundFreq[i] = clock->frequency[clock->iFast] / clock->nTic[i];
     clock->dt[i]        =  1. / clock->roundFreq[i];
     if (fmod(clock->frequency[clock->iFast],clock->frequency[i]) != 0. && clock->frequency[i] != clock->roundFreq[i] && clock->frequency[i] > 1.)
        {printf("\n !!! Warning !!! \n");
         printf("Frequency of secondary clock %ld (%f Hz) is not a submultiple of main clock frequency (%f Hz)."
                "It has been forced to %g Hz  (nTic[%d]=%d) \n",
                i, clock->frequency[i], clock->frequency[clock->iFast], clock->roundFreq[i], i, clock->nTic[i]);
         sprintf(dummy, "WARNING: Frequency of secondary clock %ld (%f Hz) is not a submultiple of main clock frequency (%f Hz)."
                "It has been forced to %g Hz  (nTic[%d]=%d) \n",
                i, clock->frequency[i], clock->frequency[clock->iFast], clock->roundFreq[i], i, clock->nTic[i]);
         DyInfoPanel(dummy,0);
        }
    }

 /*--- initialize all clocking objects ---*/
	  
 clock->obj0 = NULL;
 clockO = clock->clockO;

 while (clockO != NULL)
    {iClock         =  abs(clockO->iClock);
     clockO->nTic   =  clock->nTic[iClock];
     clockO->subTic = -clock->nTic[iClock] * clock->delay[iClock];
     if (clockO->iClock > 0)
        {clockO->subTic = clockO->subTic + clock->nTic[iClock]-1;}
     clockO->time   = 0;
     if (iClock == 0)  clock->obj0 = clockO;

     if (clockO->type->objIni != NULL)
        {clockO->type->objIni(clockO->object,clock->dt[iClock]);}
     clockO = clockO->next;
    }
 
 if (clock->obj0 == NULL)  UJError(3,"UJCObIni","no obj at clock 0");

 return;
}


/*----------------------------------------------------- UJCObSim ----*/
void UJCObSim()
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 struct UJclockO *clockO;
 long iprint;
 int run_color=0;
 time_t previoustime=0;

 clock = currentClock;
 if (clock == NULL)  UJError(3,"UJCObSim","no clock struct");
    
 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"  Start UJCObSim after %d sec (totTime=%ld) \n",
             time(NULL)-UJtimeStart, clock->totTime);}

      
 /*--- start main loop ---*/
  
 iprint = 0;

 while (clock->obj0->time < clock->totTime)
    {
     clock->time->value = clock->obj0->time * clock->dt[0];

     if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>10)
        {if (clock->obj0->time == iprint)
            {iprint = iprint*2;
             if (iprint == 0) iprint = 1;
             fprintf(UJobH.fOut,"   UJCObSim: time=%8ld after %10d sec \n",
                     clock->obj0->time, time(NULL)-UJtimeStart);
            }
        }

     clockO = clock->clockO;

     while (clockO != NULL)
        {clockO->subTic++;
         if (clockO->subTic == clockO->nTic)
            {clockO->subTic = 0;
             if (clockO->type->objSim != NULL)
                {
                 if (Dy->triggerUsed == OFF || Dy->triggerEvent == ON || Dy->triggerType != TRIG_CH || 
                     ( strcmp(clockO->type->type,"UHplotFFT")!=0 && strcmp(clockO->type->type,"UHplotTime")!=0 &&
                       strcmp(clockO->type->type,"UHplot1D")!=0 && strcmp(clockO->type->type,"UHplot2D")!=0 &&
                       strcmp(clockO->type->type,"UHpTfct")!=0 && strcmp(clockO->type->type,"UHpCohe")!=0 &&
                       strcmp(clockO->type->type,"UHpCorr")!=0 && strcmp(clockO->type->type,"UHsound")!=0 &&
                       strcmp(clockO->type->type,"UHplotFFTTime")!=0 && strcmp(clockO->type->type,"UHpTfctTime")!=0 &&
                       strcmp(clockO->type->type,"UHpCoheTime")!=0 && strcmp(clockO->type->type,"UHplot1DTime")!=0 &&
                       strcmp(clockO->type->type,"UHplotRaw")!=0 && strcmp(clockO->type->type,"UHplotRawTime")!=0) )
                    {clockO->type->objSim(clockO->object,clockO->time);}
                }
             clockO->time++;
/**
             if (time(NULL) > previoustime)
                {previoustime = time(NULL);
                 if (run_color == 0)
                    {run_color = 1;
                     fl_set_object_color(mainpanel->latency,FL_DARKCYAN,FL_COL1);
                     fl_hide_object(mainpanel->logo);
                    }
                 else
                    {run_color = 0;
                     fl_set_object_color(mainpanel->latency,FL_SPRINGGREEN,FL_PALEGREEN);
                     fl_set_object_color(mainpanel->latency,FL_WHEAT,FL_YELLOW);
                     fl_show_object(mainpanel->logo);
                    }
                }
**/
             if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>10 && clockO == clock->obj0 && clock->obj0->time%100000 == 0 && UFrCurFrameH)
                {printf("******** clockO->time=%d  totTime=%d  clock->dt[0]=%f  clock->time->value=%f  frame-start=%d-%d=%d  duration=%d  "
                        "triggerUsed=%d  triggerType=%d  triggerEvent=%d\n",
                         clockO->time, clock->totTime, clock->dt[0], clock->time->value,
                         UFrCurFrameH->GTimeS, Dy->startTime, UFrCurFrameH->GTimeS-Dy->startTime, 
                         Dy->duration, Dy->triggerUsed, Dy->triggerType, Dy->triggerEvent);}
            }
         clockO = clockO->next;
        }

    }

    RA_ObjectTablePrint();
 
 return;
}


/*---------------------------------------------------UJClockFree------*/
void UJclockFree(clock)
struct UJclock *clock;
/*--------------------------------------------------------------------*/
{
  if (clock->name != NULL)  free(clock->name);
  free(clock);
  return;
}


/*---------------------------------------------------- UJclockRead ----*/
void UJclockRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 long i;

 UJ_READ_HEADER_WITH_NAME(UJclock, clock)
 
 UJ_READ_LONG(clock->totTime);
 UJ_READ_LONG(clock->nClock);

 if (clock->nClock > UJMAXCLOCK)
    {UJError(3,"UJclockRead","too many clock; You need to increase MAXCLOCK");}

 for (i=0; i<clock->nClock; i++)
    {UJ_READ_DOUBLE(clock->frequency[i]);
     clock->delay[i] = 0;
    }	
     
 clock->clockO = NULL;
  
 currentClock = clock;
 
 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"  totTime=%ld\n",clock->totTime);
     for (i=0; i<clock->nClock; i++)
        {fprintf(UJobH.fOut,"   frequency[%ld] = %g\n",i,clock->frequency[i]);}
    }    
  
 clock->time = USignalNew("TIME",YES);
 return;
}


/*---------------------------------------------------- UJdelayRead ----*/
void UJdelayRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 struct UJdelay *delay;

 UJ_READ_HEADER_WITHOUT_NAME(UJdelay, delay)
 
 UJ_READ_LONG(delay->iClock);
 UJ_READ_LONG(delay->nTic);

 if (delay->iClock >= currentClock->nClock)
    {UJError(3,"UJdelayRead","this clock is not available");}

 currentClock->delay[delay->iClock] = delay->nTic;
	
 if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>10)
    {fprintf(UJobH.fOut,"  The clock=%ld is delayed by%ld clock tics\n",
                        delay->iClock, delay->nTic);
    }
  
 return;
}


/*----------------------------------------------------- UJfileFree----*/
void UJfileFree(file)
struct UJfile *file;
/*--------------------------------------------------------------------*/
{  
 if (file->fileName != NULL)  free(file->fileName);
 if (file->buf      != NULL)  free(file->buf);

 free(file);
 return;
}

 
/*----------------------------------------------------- UJfileNew ----*/
struct UJfile *UJfileNew(fileName, ascii)
char *fileName;
BOOL ascii;
/*--------------------------------------------------------------------*/
{
 struct UJfile *UJfile;
 long i;
   
 UJfile = (struct UJfile *) malloc(sizeof(struct UJfile));
 if (UJfile == NULL) UJError(3,"UJfileNew","malloc struct file failed");
 
 UJfile->fileName = malloc(strlen(fileName)+1);
 if (UJfile->fileName == NULL) UJError(3,"UJfileNew","malloc name failed");
 strcpy(UJfile->fileName,fileName);

 UJfile->ascii   = ascii;
 UJfile->run     = -1;
 UJfile->fp      = NULL;
 UJfile->p       = NULL;
 UJfile->pMax    = UJfile->p + 99999999;  /* should better put the sys.val. "file_length_max" */
 UJfile->bufSize = 0;
 UJfile->buf     = NULL;

 /*--- decide if we write in a real file or not ---*/

 if (strcmp(UJfile->fileName,"STRING") != 0) 
    {UJfile->string = NO;
     if (UJfile->ascii == NO) {UJfileSetBSize(UJfile,500000);}
    }
 else
    {UJfile->string = YES;
     UJfileSetBSize(UJfile,500000);
    }

 /*--- reset dictionary output ---*/

 for (i=0; i<UJMAPMAXSH; i++)  UJfile->dicWrite[i] = NO;

 return(UJfile);
}


/*--------------------------------------------------- UJfileSetBSize -*/
void UJfileSetBSize(file, bufSize)
struct UJfile *file;
long bufSize;
/*  this function increase (only) the size of the output buffer to
    bufSize. It return the pointer to the output buffer               */
/*--------------------------------------------------------------------*/
{
 if (bufSize < file->bufSize) return;

 if (file->buf != NULL)  free(file->buf);

 file->buf = (char *) malloc(bufSize);
 if (file->buf == NULL)  UJError(3,"UJfileSetBSize","malloc failed");

 file->bufSize = bufSize;
 file->p       = file->buf;
 file->pMax    = file->p + bufSize;

 return;
}


/*---------------------------------------------------- UJsendMsg  --*/
char *UJsendMsg(class, instance, msgType, param) 
char *class, *instance, *msgType, *param;
/*--------------------------------------------------------------------*/
{
 long i,j;
 struct UJmap *map;
 void *iAdd;

 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"  UJsendMsg: class=%-12s instance=%-12s type=%-12s param=%s\n",
                        class, instance, msgType, param);
    }

 /*--- first find UJstructH for this bank ---*/
			
 for (j=0; j<UJnStructH; j++)
    {if (strcmp(class,UJsHlist[j]->type) == 0) break;}

 if (j == UJnStructH) 
    {sprintf(UJmessage,"  unknown class:%s\n",class);
     return(UJmessage);
    }

 if (UJsHlist[j]->objMsg == NULL) 
    {return("UJsendMsg: no handleMsg function");}

 /*--- find instance ---*/
 
  map = NULL;
  for (i=0; i<UJsHlist[j]->nUJmap; i++)
     {if (strcmp(instance,UJsHlist[j]->UJmap[i]->bname) == 0)
         {map = UJsHlist[j]->UJmap[i];
          break;
         }
     }

 if (map == NULL)  iAdd = NULL;
 else              iAdd = map->baddress;

 /*--- call function to handle message ---*/
 
 UJsHlist[j]->objMsg(iAdd, msgType, param, UJMSGMAX, UJmessage);

 return(UJmessage);
}


/*--------------------------------------------------- UJfindClockO ---*/
void* UJfindClockO(type, name)
char *type, *name;
/*--------------------------------------------------------------------*/
{
 struct UJclock* clock;
 struct UJclockO* clockO;

 clock  = UJCloGet();
 clockO = clock->clockO;

 while (clockO != NULL)
    {if ( strcmp(clockO->type->type,type) == 0)
        {if (strcmp(clockO->name, name) == 0)  return((void*) clockO->object);}
     clockO = clockO->next;
    }

 return(NULL);
}


/*------------------------------------------------- UJfindInstance  --*/
void *UJfindInstance(class, instance) 
char *class, *instance;
/*--------------------------------------------------------------------*/
{
 long i,j;
 struct UJmap *map;
 void *iAdd;

 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"  UJfindInstance: class=%-12s instance=%-12s \n",
                        class, instance);
    }

 /*--- first find UJstructH for this bank ---*/
			
 for (j=0; j<UJnStructH; j++)
    {if (strcmp(class,UJsHlist[j]->type) == 0) break;}

 if (j == UJnStructH)  return(NULL);       /*--- unknown class ---*/

 /*--- find instance ---*/
 
 map = NULL;
 for (i=0; i<UJsHlist[j]->nUJmap; i++)
    {if (strcmp(instance,UJsHlist[j]->UJmap[i]->bname) == 0)
        {map = UJsHlist[j]->UJmap[i];
         break;
        }
    }

 if (map == NULL)  iAdd = NULL;
 else              iAdd = map->baddress;

 return(iAdd);
}


/*------------------------------------------------------ UJSetAll ----*/
void UJSetAll()  /* set all banks address */
/*--------------------------------------------------------------------*/
{
 long i,j,k,ierr,noRef;
 void **reference, **address;
 struct UJxref *last;
 
 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"\n >>>> Start address relocation (UJSetAll)"
                        " UJnStructH=%ld\n",UJnStructH);
    }

 ierr  = 0;
 noRef = 0;
 
 for (j=0; j<UJnStructH; j++)
    {if (UJsHlist[j] == NULL) continue;
     if (UJobH.debugLvl > 100)
        {fprintf(UJobH.fOut," type:%-20s  number of instances: %ld\n",
	                    UJsHlist[j]->type, UJsHlist[j]->nUJmap);
        }
     for (i=0; i<UJsHlist[j]->nUJmap; i++)   
        {
         if (UJsHlist[j]->UJmap[i] == NULL) continue;

         if (UJobH.debugLvl > 100) 
            {
             fprintf(UJobH.fOut,"   name: %-20s addr:%12p nref=%2ld",
                                UJsHlist[j]->UJmap[i]->bname,
                                UJsHlist[j]->UJmap[i]->baddress,
                                UJsHlist[j]->UJmap[i]->nref);
             if (UJsHlist[j]->UJmap[i]->nref > 0)  fprintf(UJobH.fOut," ref:");
	     for (k=0; k<UJsHlist[j]->UJmap[i]->nref; k++)
	        {fprintf(UJobH.fOut,"%ld %p",j,UJsHlist[j]->UJmap[i]->bref[k]);}
	     fprintf(UJobH.fOut,"\n");
            }
	 
         if (UJsHlist[j]->UJmap[i]->baddress == NULL)
            {fprintf(UJobH.fOut," ****** ERROR ****** UJSetAll: bank not created:%s\n",
                                UJsHlist[j]->UJmap[i]->bname);
             printf(" *** ERROR *** UJSetAll: struct &%s not created\n",
                               UJsHlist[j]->UJmap[i]->bname);
             ierr++;
             continue;
            }

         for (k=0; k<UJsHlist[j]->UJmap[i]->nref; k++)	
            {reference  = UJsHlist[j]->UJmap[i]->bref[k];
             *reference = UJsHlist[j]->UJmap[i]->baddress;
            }
        }
    }

 if (ierr > 0)  UJError(3,"UJSetAll"," too many missing banks");
       
 /*--- set cross reference pointer ---*/

 last = UJfirstXref;
 while (last != NULL)
    {reference  = last->ref;
     address    = last->add;
     *reference = *address;
     last       = last->next;
    }

 return;
}

 
/*---------------------------------------------------- UJSetBA ------*/
void UJSetBA(bankName, bankType, address) /* store address for a bank*/
char *bankName, *bankType;
void *address;
/*--------------------------------------------------------------------*/
{
 long i,j;
 struct UJmap *map;

 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"  UJsetBA: instance=%-12s type=%-12s add=%p\n",
                         bankName,bankType,address);
    }

 /*--- first find UJstructH for this bank ---*/
			
 for (j=0; j<UJnStructH; j++)
    {if (strcmp(bankType,UJsHlist[j]->type) == 0) break;}

 if (j == UJnStructH) 
    {fprintf(UJobH.fOut," UJSetBA: for type:%s name%s\n", bankType, bankName);
     UJError(3,"UJSetBA","UJstructH not found");
    }
	
 /*--- search for an already register struct ---*/
			
 for (i=0; i<UJsHlist[j]->nUJmap; i++)
    {if (strcmp(bankName,UJsHlist[j]->UJmap[i]->bname) == 0)
        {map = UJsHlist[j]->UJmap[i];
         if (map->baddress != NULL)
            {fprintf(UJobH.fOut, " **** ERROR **** UJSetBA: bank already registered:%s\n",
                                 bankName);
             UJError(3,"UJSetBA","bank already registered");
            }
         break;
        }
    }
 
 /*--- bank is not already in UJmap, save it ---*/
 	 
 if (i == UJsHlist[j]->nUJmap)
    {map = (struct UJmap *) malloc(sizeof(struct UJmap));
     if (map == NULL) UJError(3,"UJSetBA","malloc failled");
     map->nref  = 0;
     map->bname = malloc(strlen(bankName)+1);
     if (map->bname == NULL)  UJError(3,"UJSetBA","malloc failed");
     strcpy(map->bname,bankName);
	 
     if (UJsHlist[j]->nUJmap == UJMAPMAXINSTANCE)  UJError(3,"UJSetBA","MAXUJMAP reached");
     UJsHlist[j]->nUJmap++;
     UJsHlist[j]->UJmap[i] = map;
    }

 /*--- store bank address ---*/
 
 map->baddress = address;

 return;
}


/*---------------------------------------------------- UJSetBAT------*/
void UJSetBAT(instance, type, address)   /* store address for a bank */
long instance, type;
void *address;
/*--------------------------------------------------------------------*/
{
 struct UJmap *map;

 if (type > UJnStructH)            UJError(3,"UJSetBAT","type>UJnStructH");
 if (instance > UJMAPMAXINSTANCE)  UJError(3,"UJSetBAT","MAXUJMAP reached");
 if (instance < 0)                 UJError(3,"UJSetBAT","wrong instance ");

 /*--- struct is not already in UJmap, save it ---*/

 map = UJsHlist[type]->UJmap[instance];

 if (map == NULL)
    {map = (struct UJmap *) malloc(sizeof(struct UJmap));
     if (map == NULL)  UJError(3,"UJSetBA","malloc failled");
     map->nref  = 0;
     map->bname = NULL;
     UJsHlist[type]->UJmap[instance] = map;
     UJsHlist[type]->nUJmap          = SIESTA_MAX(instance, UJsHlist[type]->nUJmap)+1;
    }
 else
    {if (map->baddress != NULL)  UJError(3,"UJSetBAT","bank already registered");}

 /*--- save data ---*/
 	 
 map->baddress = address;
 
 return;
}


/*------------------------------------------------------- UJSetBR ----*/
void UJSetBR(bankName, bankType, address)     /* store bank reference */
char *bankName, *bankType;
void *address;
/*--------------------------------------------------------------------*/
{
 void **reference;
 long i,j;
 struct UJmap *map;
 
 if (strcmp(bankName,"NULL") == 0) 
    {reference  = address;
     *reference = NULL;
     return;
    }

 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"  UJsetBR: instance=%-12s type=%-12s add=%p",
                        bankName,bankType,address);
    }

 /*--- first find UJstructH for this bank ---*/
			
 for (j=0; j<UJnStructH; j++)
     {if (strcmp(bankType,UJsHlist[j]->type) == 0) break;}

 if (j == UJnStructH)  UJError(3,"UJSetBR","type not found");

 /*--- find instance ---*/
 
 if (UJobH.debugLvl > 1000)
    {fprintf(UJobH.fOut,"    nUJmap=%ld\n", UJsHlist[j]->nUJmap);}
		   
 for (i=0; i<UJsHlist[j]->nUJmap; i++)
    {map = UJsHlist[j]->UJmap[i];
     if (map == NULL) continue;
     if (strcmp(bankName,map->bname) == 0) break;
    }
 
 /*--- if bank not already in UJmap, save it ---*/
	   
 if (i == UJsHlist[j]->nUJmap)
    {map = (struct UJmap*)malloc(sizeof(struct UJmap));
     if (map == NULL)  UJError(3,"UJSetBR","malloc failled");

     map->bname = malloc(strlen(bankName)+1);
     if (map->bname == NULL)  UJError(3,"UJSetBR","malloc failed");
     strcpy(map->bname,bankName);
     map->nref     = 0;
     map->baddress = NULL;

     UJsHlist[j]->UJmap[UJsHlist[j]->nUJmap] = map;
     UJsHlist[j]->nUJmap++;
    }
  
 /*--- finally store reference address ---*/

 map->bref[map->nref] = address;
 map->nref++;
 if (map->nref >= UJMAPMAXREF)  UJError(3,"UJSetBR","max ref reached");

 return;
}


/*------------------------------------------------------ UJSetBRT----*/
void UJSetBRT(instance, type, address)         /* store bank reference */
long type, instance;
void *address;
/*--------------------------------------------------------------------*/
{
 void **reference;
 struct UJmap *map;
 
 if (instance == -1) 
    {reference  = address;
     *reference = NULL;
     return;
    }

 /*--- first check type number validite ---*/
			
 if (type >= UJnStructH)           UJError(3,"UJSetBRT","type too big");
 if (type < 0)                     UJError(3,"UJSetBRT","type too small");
  
 if (instance > UJMAPMAXINSTANCE)  UJError(3,"UJSetBAT","MAXUJMAP reached");
  
 /*--- check that struct is already there ---*/
	  
 map = UJsHlist[type]->UJmap[instance];
 if (map == NULL)
    {map = (struct UJmap *) malloc(sizeof(struct UJmap));
     if (map == NULL)  UJError(3,"UJSetBRT","malloc failled");
     map->nref     = 0;
     map->bname    =  NULL;
     map->baddress = NULL;
     UJsHlist[type]->UJmap[instance] = map;
     UJsHlist[type]->nUJmap          = SIESTA_MAX(instance, UJsHlist[type]->nUJmap)+1;
    }
		
 /*--- store bank address ---*/
    
 map->bref[map->nref] = address;
 map->nref++;
 if (map->nref >= UJMAPMAXREF)  UJError(3,"UJSetBRT","max ref reached");

 return;
}


/*------------------------------------------------------ UJSetIni ----*/
void UJSetIni()                /* init the address relocation package */
/*--------------------------------------------------------------------*/
{
 long i,j;

 for (j=0; j<UJnStructH; j++)
    {for (i=0; i<UJsHlist[j]->nUJmap; i++)
        {if (UJsHlist[j]->UJmap[i] == NULL) continue;
         if (UJsHlist[j]->UJmap[i]->bname)  free(UJsHlist[j]->UJmap[i]->bname);
         free(UJsHlist[j]->UJmap[i]); 
         UJsHlist[j]->UJmap[i] = NULL;
        }
     UJsHlist[j]->nUJmap = 0;
    }
 
 return;
}


/*---------------------------------------------------- UJSetXR ------*/
void UJSetXR(address,ref) /* store cross reference address for a struct*/
void *address, *ref;
/*--------------------------------------------------------------------*/
{
 struct UJxref *xref;
 static struct UJxref *last = NULL;

 if (UJobH.debugLvl > 100)
    {fprintf(UJobH.fOut,"  UJsetXR: add=%p ref=%p\n",address,ref);}

 if (address == NULL) return;

 xref = (struct UJxref *) malloc(sizeof(struct UJxref));
 if (xref == NULL)  UJError(3,"UJSetXR","malloc failled");

 if (last == NULL)  UJfirstXref = xref;
 else               last->next  = xref;
 last = xref;

 xref->add  = address;
 xref->ref  = ref;
 xref->next = NULL;

 return;
}

  
/*------------------------------------------------------ UJEnd -------*/
void UJEnd()                                     /* print end of job  */
/*--------------------------------------------------------------------*/
{
 time_t tEnd;
 
 tEnd = time(NULL);

if (((Dy->debugType)&(1<<DBG_URPLOT)) && Dy->debug>0)
    {fprintf(UJobH.fOut,"\n UJEnd:  End of job at %s   Running time: %d sec.\n",
             ctime(&tEnd),tEnd-UJtimeStart);}

 USignalEnd();

/***
 if (UJpauseAtTheEnd == YES)
    {printf(" Please type return to end the job\n");
     while (getchar() != 10);
    }
***/

 return;
} 


/*------------------------------------------------------ UJFree ----*/
void UJFree()
/*--------------------------------------------------------------------*/
{
 long j;

 for (j=0; j<UJnStructH; j++)
    {
     free(UJsHlist[j]->type); 
     free(UJsHlist[j]); 
     UJsHlist[j] = NULL;
    }
 
 return;
}


/*----------------------------------------------------- UJError ------*/
void UJError(level, subroutine, message)
long level;
char *subroutine,  *message;
/*--------------------------------------------------------------------*/
/* handle error acording level of severity.                           */
/* input: subroutine: name of the subroutine calling verror           */
/*        message:    message to by reported                          */
/*        level:      1= info, 2= warning, 3=stop execution           */
/*--------------------------------------------------------------------*/
{
 long iCycle,id;
 char * type;
 static char * severity[] = {"info","warning","stop"};

 if (level >= 3)     type = severity[2];
 else if(level > 0)  type = severity[level-1];
 else                UJError(2,"UJError","unknown error level");

 if (UJobH.fOut) fprintf(UJobH.fOut,  "*** UJError::%s in %s %s\n", type, subroutine, message);
 
 if (level == 4)
    {UJCObEnd();                 /* call all "end" procedures   */
     RA_FileWrite("plot_file.root");
     RA_FileClose("plot_file.root");
    }

 if (level > 2)  exit(0);
 
 return;
}


/*---------------------------------------------------- UJDicAddS----*/
struct UJstructH *UJDicAddS(type, 
       funIni, funSim, funEnd, funRead, funFree)           
char *type;
void (*funIni)(),   (*funSim)(),  (*funEnd)(), 
     (*funRead)(), (*funFree)();
/*--------------------------------------------------------------------*/
{
 struct UJstructH *structH;

 structH = UJDicNewSH(type);
 structH->objIni   = funIni;
 structH->objSim   = funSim;
 structH->objEnd   = funEnd;
 structH->objRead  = funRead;
 structH->objFree  = funFree;
 structH->objMsg   = NULL;

 return(structH);
}

  
/*------------------------------------------------------- UJDicGetId-*/
long UJDicGetId(type, instance)
struct UJstructH *type;
void *instance;
/* this function return a unique id for a structure address  */
/*--------------------------------------------------------------------*/
{
 static long id;

 if (type == 0)  UJError(3,"UJDicGetId","structH = NULL");
 
 if (instance == NULL) return(-1);

 for (id=0; id< type->nAdd; id++)
    {if (type->bAddress[id] == instance) return(id);}

 type->bAddress[id] = instance;
 id                 = type->nAdd;
 type->nAdd++;

 return(id);
} 


/*------------------------------------------------------- UJDicNewSH-*/
struct UJstructH *UJDicNewSH(name)         
char *name;
/*--------------------------------------------------------------------*/
{
 struct UJstructH *structH;

 if (UJnStructH >= UJMAPMAXSH)  UJError(3,"UJCObAdd","wrong count");

 structH = (struct UJstructH*) malloc(sizeof(struct UJstructH));
 if (structH == NULL)  UJError(3,"UJDicNewSH","malloc failed");
 
 structH->type = malloc(strlen(name)+1);
 if (structH->type == NULL)  UJError(3,"UJDicNewSH","malloc name failed");

 strcpy(structH->type,name);
 structH->nUJmap = 0;
 structH->nAdd   = 0;
 structH->id     = UJnStructH;
  
 UJsHlist[UJnStructH] = structH;
 UJnStructH++;

 return(structH);
}


/*-------------------------------------------------------- UJQSRead---*/
long UJQSRead(iFile) /* read one quoted string */
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 long i;

 if (fscanf(iFile->fp,"%s\n",bankName) != 1)  return (-1);

 if (bankName[0] != '\"')  return(0);

 /*--- here we deal with quoted string          ---*/
 /*--- first we shift one character to the left ---*/

 for (i=0; i<strlen(bankName)-1; i++)  bankName[i] = bankName[i+1];

 if (bankName[i-1] == '\"')
    {bankName[i-1] = '\0';
     return(0);
    }

 /*--- read the end of the string ---*/

 bankName[i] = 0x20;
 while (bankName[i] !=  '\"')
    {i++; 
     bankName[i] = getc(iFile->fp);
    } 
 while(bankName[i] !=  '\"');
 bankName[i] = '\0';

 return(0);
}


/*----------------------------------------------------- UJtimeRead ----*/
void UJtimeRead(iFile)
struct UJfile *iFile;
/*--------------------------------------------------------------------*/
{
 char *timeC, *dateC;
 int fullyear;
 char *timechar, *timechar2;
 time_t tt;
 struct tm tp;

 UJ_READ_HEADER_WITHOUT_NAME(UJtime, timeRef)
 
 UJ_READ_S_CHAR(dateC)
 UJ_READ_S_CHAR(timeC)

 printf("dateC=%s  timeC=%s\n ",dateC, timeC);

 if (dateC == NULL  ||  timeC == NULL)
    {tt = time(NULL);}
 else
    {sscanf(dateC,"%d/%d/%d", &tp.tm_mday, &tp.tm_mon, &fullyear);
     sscanf(timeC,"%d:%d:%d", &tp.tm_hour, &tp.tm_min, &tp.tm_sec);
     tp.tm_year = fullyear - 1900;
     tp.tm_mon--;
     tt        = mktime(&tp);
     timechar  = asctime(&tp);
    }

 timechar2 = asctime(gmtime(&tt));
 printf("\n  tt=%d \n  local=%s  UTC=%s ", tt, timechar, timechar2);

 timeRef->origin      = mktime(gmtime(&tt)) - UJGPSOFF + 7212;
 timeRef->current     = timeRef->origin;
 timeRef->originChar  = FrStrGTime(timeRef->origin);
 timeRef->currentChar = FrStrGTime(timeRef->current);

 printf("\n  tt = %d \n  timeOrigin = %ld  (%s) \n\n", tt, timeRef->origin, timeRef->originChar);
     
 return;
}


/*----------------------------------------------------- UJtimeGet ----*/
long UJtimeGet()
/*--------------------------------------------------------------------*/
{    
 return(timeRef->origin);
}


/*----------------------------------------------------- UJtimeGetC ----*/
char *UJtimeGetC()
/*--------------------------------------------------------------------*/
{
 timeRef->originChar = FrStrGTime(timeRef->origin);    
 return(timeRef->originChar);
}


/*----------------------------------------------------- UJtimeGet ----*/
long UJtimeGetCurrent()
/*--------------------------------------------------------------------*/
{
 struct UJclock *clock;
 long nsec;
		
 clock = currentClock;
 if (clock == NULL)  UJError(3,"UJCObIni","no clock struct");

 nsec             = (long)((double)clock->obj0->time * clock->dt[0]);
 timeRef->current = timeRef->origin + nsec;

/*
 printf("%ld * clock->dt[0](%g) = %ld \n", clock->obj0->time,clock->dt[0],nsec);
 printf("timeRef->origin=%ld  timeRef->current=%ld \n",timeRef->origin,timeRef->current);
*/

 return(timeRef->current);
}


/*----------------------------------------------------- UJtimeGetCurrentC ----*/
char *UJtimeGetCurrentC()
/*--------------------------------------------------------------------*/
{
 UJtimeGetCurrent();

 timeRef->currentChar = FrStrGTime(timeRef->current);    

 return(timeRef->currentChar);
}
