/* *****************************************************************
*********** UJobL.h ***************************** UJobL.h **********
*********** UJobL.h ***************************** UJobL.h **********
***************************************************************** */
/* by B. Mours LAPP (Annecy) March 05, 1998 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include "UMathL.h"

#if !defined(UJOBLIB_DEFINED)
#define UJOBLIB_DEFINED DONE


extern BOOL UJpauseAtTheEnd;   /* used by UJEnd */

#define UJBANKNAMESIZE 300000
extern char bankName[UJBANKNAMESIZE];  /* local variables used for Read functions */
extern char UJbuf[UJBANKNAMESIZE];     /* local variables used for Read functions */

#define UJMAPMAXSH 200

extern struct UJstructH *UJsHlist[UJMAPMAXSH];
extern long   UJnstructH;
extern struct UJstructH *UJclockUJstructH;
extern struct UJstructH *UJdelayUJstructH;
extern struct UJstructH *UJstructHUJstructH;
extern struct UJstructH *UJstructHUJstructH;
extern struct UJstructH *UJtimeUJstructH;

#define UJMAXCLOCK 199
#define UJGPSOFF   315964800   /* (C time - GPS time = 3600*24*3567 = 315964800 seconds) */

#ifdef Linux
#define LONG long long
#else
#define LONG long
#endif

struct UJtime {             /* hold time origin information           */
   char   *name;            /* name of each instance                  */
   char   *originChar;      /* time origin in readable format         */
   LONG   origin;           /* time origin chosen (GPS time)          */
   char   *currentChar;     /* current time in readable format        */
   LONG   current;          /* current time w.r.t the origin          */
};

extern struct UJtime *timeRef;

struct UJclock {            /* hold general timing information        */
   char   *name;            /* name for each instance                 */
   LONG   totTime;          /* total processing time for clock 0      */
   long   nClock;           /* number of clocks                       */
   double frequency[UJMAXCLOCK]; /* clock frequencies in decreasing   */
                            /* order (secondary frequencies must be   */
                            /* sub-multiples of main clock frequency) */
                            /* ------ end_of_SIO parameters ----------*/
   struct UJclockO *clockO; /* first clocking object                  */
   struct UJclockO *obj0;   /* clocking object running for clock 0    */
   long   iFast;            /* index of fastest clock                 */
   LONG   nTic[UJMAXCLOCK];
   LONG   delay[UJMAXCLOCK];
   double roundFreq[UJMAXCLOCK];
   double dt[UJMAXCLOCK];
   struct USignal *time;    /* current time in sec.                   */
};

struct UJclockO {           /* describe clocking Object               */
  void *object;             /* pointer to the clocking object         */
  struct UJstructH *type;   /* object type                            */
  char *name;               /* object name                            */
  long   iClock;            /* clock id                               */
  LONG   time;              /* current time (in object clock count)   */
  LONG   subTic;            /* current tic clock (for lowest clock)   */
  LONG   nTic;              /* number of lowest frequency clock tic to
                               do one object tic                      */ 
  struct UJclockO *next;    /* next clocking element                  */
};

struct UJdelay {            /* delay the start of one clock           */
  long   iClock;            /* clock id                               */
  LONG   nTic;              /* # of clock tic to wait before start    */ 
                      /* -------------- end_of_SIO parameters --------*/
};

struct UJfile{             /* standard header for i/o file            */
  char *fileName;          /* file name                               */
  BOOL string;             /* if YES output is in a string            */
  BOOL ascii;              /* if YES output is in readable ASCCI      */
  BOOL dicWrite[UJMAPMAXSH];/* tell if the dict. info has been written*/
  long fmtVersion;         /* format version                          */
  long run;                /* current run number on file              */
  FILE *fp;                /* pointer to the file                     */
  char *buf;               /* pointer to the output string. The current
                              buffer length = UJfile->buf - UJfile->p */
  char *p;                 /* current pointer to the output string    */
  char *pStart;            /* pointer to the begining of a record     */
  char *pMax;              /* maximum value for the output string p.  */
  long bufSize;            /* output buffer size                      */
};

struct UJobH{         /* Job header, hold run time info               */
                      /* -------------- end_of_SIO parameters --------*/
   long   debugLvl;   /* debug level                                  */
   FILE *fOut;        /* pointer on output file (for debug)           */
   char cards[100];
   char listing[100];
   char histo[100];
     };

#define UJMAPMAXREF 400
#define UJMAPMAXINSTANCE 400

struct UJmap{         /* structure used by SIO */
                      /* -------------- end_of_SIO parameters --------*/
  char *bname;
  void *baddress;
  long nref;
  void  *bref[UJMAPMAXREF];
};

struct UJstructH {            /* hold info for this type of struct      */
   long   id;                 /* is structure when written on tape      */
   char   *type;              /* name for this type of structure        */
                              /* -------- end_of_SIO parameters --------*/
   void (*objIni)();          /* initialization function                */
   void (*objSim)();          /* simulation function                    */
   void (*objEnd)();          /* end function                           */
   void (*objRead)();         /* function to read the object on file    */
   void (*objFree)();         /* function to Free the object            */
   void (*objMsg)();          /* function to send a message             */
   struct UJmap *UJmap[UJMAPMAXINSTANCE]; /* array of UJmap struct      */
   long nUJmap;               /* number of UJmap structre               */
   void  *bAddress[UJMAPMAXINSTANCE];
   long nAdd;                 /* number of structure                    */
};

struct UJxref{/* structure used by SIO */
  void *add;
  void *ref;
  struct UJxref *next;
};


extern struct UJobH UJobH;

/***
int DyCheckRunning();
***/

void UJLibIni(char *jobName, long argc, char *argv[]);

void UJCardRead(void);

struct UJclock *UJCloGet();
void UJCloSet(struct UJclock *clock);
void UJCObAdd(long iClock, void *object, struct UJstructH *type, char *name);
void UJCObEnd(void);
void UJCObIni(void);
void UJCObSim(void);

void UJclockFree(struct UJclock *clock);
void UJclockRead(struct UJfile *iFile);

void UJdelayRead(struct UJfile *iFile);

void   UJfileFree(struct UJfile *file);
struct UJfile *UJfileNew(char *fileName, BOOL ascii);
void UJfileSetBSize(struct UJfile *file, long bufSize);

void* UJfindClockO(char *type, char *name);

char *UJsendMsg(char *class0, char *instance, char *msgType, char *param); 
 
void UJSetAll(void);
void UJSetBA( char *bankName, char *bankType, void *address);
void UJSetBAT(long instance,  long  bankType, void *address);
void UJSetBR( char *bankName, char *bankType, void *address);
void UJSetBRT(long instance,  long type     , void *address);
void UJSetXR(void *address, void *ref);

void UJFree(void);
void UJEnd(void);
void UJError(long level, char *subroutine, char *message);

struct UJstructH *UJDicAddS(char *name,
                           void (*objIni)(),  void (*objSim)(), 
	                       void (*objEnd)(),
                           void (*objRead)(), void (*objFree)());
struct UJstructH *UJDicNewSH(char *name);
long UJDicGetId(struct UJstructH *type, void *instance);

long UJQSRead(struct UJfile *iFile);
void UJtimeRead(struct UJfile *iFile);
long UJtimeGet();
char *UJtimeGetC();
long UJtimeGetCurrent();
char *UJtimeGetCurrentC();

/*-----------------------------------------------------------------------
  Data Format on Tape:
  --------------------

  The data format on tape is either an ASCII file or a binary file. 
  Data are written in ASCII format or in native binary format.
  Special macros (UJ_READ... and UJ_PUT...) are availables to 
  read/write the various C data type.  
  Binary format are the same for short (2 bytes), in (4 bytes),
  float (4 bytes) double (8 bytes) on HP-UX, SunOS,
  IBMRS6000, 680xx, Power PC. Long and pointer are different
  for DEC/Alpha and other machines. Therefore there use in stored
  data is prohibed. 
  The format conversion is done at reading time if needed.
  
  A data file should start with a file header with the following data:
   * for ASCII format:  
     - the word BSCII
   * for binary format:
     - byte 1 = the format version (current version number is 1)
     - byte 2 = sizeof(short)
     - byte 3 = sizeof(int)
     - byte 4 = sizeof(long)
     - byte 5 = sizeof(float)
     - byte 6 = sizeof(double)
     - bytes 7-8 = a short containing 0x1234 (to check the bytes swaping)
     - bytes 9-12 = a long word containting 0x01020304 
     - bytes 13-16 = a float containg PI
     - bytes 17-24 = a double containg PI
	
  The files is structured in record. One record content one C struct. 
  For each record there is a short header with the following data:
   * for ASCII format:  
     - a word to give the type of the C struct
   * for binary format: three 4 bytes words:
     - the first is the number of bytes in the record including the header
     - the second is the type the C struct. This number is dynamically
       generated by the write program (function UJDicGetId). 
       The correpondance with the structure name is given by the 
       dictionnary (UJstructH and UJstructE ) which are 
       included on tape.
     - the third is the instance number (a unique number per structure of 
       one type) This number is dynamically generated by the write program 
       using the function UJDicGetId.

  The content of the C struct follows this header. Data are written 
  in the same order as in the structure. 
  Character string are preceded by their length coded as a
  int (4 bytes) in binary format. When a pointer is given,
  the size of the data is given by the latest interger word.
  
  The record are grouped by Frame. It start with the FRameH record (which
  clear the instances number in the write programe) and end with the 
  EndOfFrame record which genereate the address realocation in the read program.

  The detector description is included in the Frames.
  
 ------- here we have the macros for the I/O package. ------------
 These macro assume that the following pointers are available:
  - read  function: struct FRiFile *iFile
  - write function: struct FRoFile *oFile
--------------------------------------------------------------- */
 
extern short  UJ_short;
extern float  UJ_float;
extern double UJ_double;
extern int    UJ_id, UJ_int, UJ_type, UJ_instance, UJ_i, UJ_strlen;
extern char   UJ_char, *UJ_aint, *UJ_ashort, *UJ_achar;

#define UJ_NEW_RECORD(TYPE, INSTANCE) \
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
	   {sprintf(UJbuf ,"%s ",#TYPE);\
	    UJ_strlen = strlen(UJbuf);\
	    memcpy(oFile->p, UJbuf, UJ_strlen);\
	    oFile->p += UJ_strlen;}\
     else  {fprintf(oFile->fp,"%s ",#TYPE);}} \
 else  \
   {if(oFile->string == NO) {oFile->p = oFile->buf;} \
    UJ_short = (short) TYPE ## UJstructH->id;  \
    oFile->pStart = oFile->p; \
    oFile->p +=4; \
    memcpy(oFile->p, &UJ_short, 2); \
    oFile->p += 2;  \
    UJ_short = UJDicGetId( TYPE ## UJstructH, INSTANCE); \
    memcpy(oFile->p, &UJ_short, 2); \
    oFile->p += 2; }
      
#define  UJ_PUT_STRUCT(TYPE, VALUE) \
 if(oFile->string == YES && oFile->p+100 > oFile->pMax) \
    {UJError(3,"UJ_PUT_STRUCT"#TYPE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
	   {sprintf(UJbuf ,"%p ",VALUE);\
	    UJ_strlen = strlen(UJbuf);\
	    memcpy(oFile->p, UJbuf, UJ_strlen);\
	    oFile->p += UJ_strlen;}\
     else  {fprintf(oFile->fp,"%p ",VALUE);}} \
 else  \
    {UJ_short = TYPE ## UJstructH->id; \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2;  \
     UJ_short = UJDicGetId(TYPE ## UJstructH, VALUE); \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2; }

#define  UJ_PUT_STRUCT_WITH_NAME(TYPE, VALUE) \
 if(oFile->string == YES && oFile->p+100 > oFile->pMax) \
    {UJError(3,"UJ_PUT_STRUCT_WITH_NAME"#TYPE,"UJ_buf overflow");} \
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
       {if(VALUE == NULL ) \
             {sprintf(UJbuf,"NULL ");} \
        else {sprintf(UJbuf,"%s ",VALUE->name);} \
	UJ_strlen = strlen(UJbuf);\
        memcpy(oFile->p, UJbuf, UJ_strlen);\
	oFile->p += UJ_strlen;}\
     else \
       {if(VALUE == NULL ) \
             {fprintf(oFile->fp,"NULL ");} \
        else {fprintf(oFile->fp,"%s ",VALUE->name);}}} \
 else  \
    {UJ_short = TYPE ## UJstructH->id; \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2; \
     UJ_short = UJDicGetId(TYPE ## UJstructH, VALUE); \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2; }

        /* we assume that the first element of a struct is a name */

#define  UJ_PUT_SHORT(VALUE) \
 if(oFile->string == YES && oFile->p+10 >  oFile->pMax) \
    {UJError(3,"UJ_PUT_SHORT"#VALUE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
	   {sprintf(UJbuf,"%d ",VALUE ); \
	    UJ_strlen = strlen(UJbuf);\
            memcpy(oFile->p, UJbuf, UJ_strlen);\
	    oFile->p += UJ_strlen;}\
     else  {fprintf(oFile->fp,"%d ",VALUE);}} \
 else  \
    {UJ_short = (short) VALUE; \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2; }
    
#define UJ_PUT_S_CHAR(NAME)  \
 if(oFile->string == YES && oFile->p+100 > oFile->pMax) \
    {UJError(3,"UJ_PUT_S_CHAR"#NAME,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
       {if(strchr(NAME,0x20) == NULL) \
             {sprintf(UJbuf,"%s ",NAME);} \
        else {sprintf(UJbuf,"\"%s\" ",NAME);} \
	UJ_strlen = strlen(UJbuf);\
        memcpy(oFile->p, UJbuf, UJ_strlen);\
	oFile->p += UJ_strlen;}\
     else \
       {if(strchr(NAME,0x20) == NULL) \
             {fprintf(oFile->fp,"%s ",NAME);} \
        else {fprintf(oFile->fp,"\"%s\" ",NAME);}}} \
 else  \
    {UJ_short = strlen(NAME)+1; \
     memcpy(oFile->p, &UJ_short, 2); \
     oFile->p += 2;  \
     memcpy( oFile->p, NAME, UJ_short); \
     oFile->p += UJ_short;}
 
#define UJ_PUT_CHAR(VALUE)  \
 if(oFile->string == YES && oFile->p+10 > oFile->pMax) \
    {UJError(3,"UJ_PUT_CHAR"#VALUE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
          {sprintf(UJbuf,"%c ",VALUE); \
           UJ_strlen = strlen(UJbuf);\
           memcpy(oFile->p, UJbuf, UJ_strlen);\
           oFile->p += UJ_strlen;}\
     else                    {fprintf(oFile->fp,"%c ",VALUE);}}\
 else  \
    {*oFile->p++ = (char) VALUE;}
   
#define UJ_PUT_LONG(VALUE)  UJ_PUT_INT(VALUE)

#define UJ_PUT_INT(VALUE) \
 if(oFile->string == YES && oFile->p+ 10 > oFile->pMax) \
    {UJError(3,"UJ_PUT_LONG"#VALUE,"UJ_buf overflow");} \
 if(oFile->ascii == YES)   \
     {if(oFile->string == YES) \
          {sprintf(UJbuf,"%d ",VALUE); \
           UJ_strlen = strlen(UJbuf);\
           memcpy(oFile->p, UJbuf, UJ_strlen);\
           oFile->p += UJ_strlen;}\
     else {fprintf(oFile->fp,"%d ",VALUE);}} \
 else  \
    {UJ_int = (int) VALUE; \
     memcpy(oFile->p, &UJ_int, 4); \
     oFile->p += 4;}  
   
#define UJ_PUT_FLOAT(VALUE) \
 if(oFile->string == YES && oFile->p+10 > oFile->pMax) \
    {UJError(3,"UJ_PUT_FLOAT"#VALUE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
          {sprintf(UJbuf,"%e ",VALUE); \
           UJ_strlen = strlen(UJbuf);\
           memcpy(oFile->p, UJbuf, UJ_strlen);\
           oFile->p += UJ_strlen;}\
     else {fprintf(oFile->fp,"%e ",VALUE);}} \
 else  \
    {UJ_float = (float) VALUE; \
     memcpy(oFile->p, &UJ_float, 4); \
     oFile->p += 4;}
  

#define UJ_PUT_DOUBLE(VALUE) \
 if(oFile->string == YES && oFile->p+10 > oFile->pMax) \
    {UJError(3,"UJ_PUT_DOUBLE"#VALUE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
          {sprintf(UJbuf,"%le ",VALUE); \
           UJ_strlen = strlen(UJbuf);\
           memcpy(oFile->p, UJbuf, UJ_strlen);\
           oFile->p += UJ_strlen;}\
     else {fprintf(oFile->fp,"%le ",VALUE);}} \
 else  \
    {UJ_double = (double) VALUE; \
     memcpy(oFile->p, &UJ_double, 8); \
     oFile->p += 8;}
  
#define  UJ_PUT_BOOL(VALUE) \
 if(oFile->string == YES && oFile->p+10 > oFile->pMax) \
    {UJError(3,"UJ_PUT_BOOL"#VALUE,"UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
       {if(VALUE == YES) sprintf(UJbuf,"YES "); \
        else             sprintf(UJbuf,"NO "); \
        UJ_strlen = strlen(UJbuf);\
        memcpy(oFile->p, UJbuf, UJ_strlen);\
        oFile->p += UJ_strlen;}\
     else  \
       {if(VALUE == YES) fprintf(oFile->fp,"YES "); \
        else             fprintf(oFile->fp,"NO ");}} \
 else  \
    {UJ_PUT_CHAR(VALUE);}
	 
#define UJ_WRITE_RECORD \
 if(oFile->string == YES && oFile->p+10 >  oFile->pMax) \
    {UJError(3,"UJ_WRITE_RECORD","UJ_buf overflow");}\
 if(oFile->ascii == YES)   \
    {if(oFile->string == YES) \
       {sprintf(oFile->p,"\n"); \
        oFile->p += 1;} \
     else  \
       {fprintf(oFile->fp,"\n");}} \
 else   \
    {UJ_int = (int) (oFile->p - oFile->pStart); \
     memcpy(oFile->pStart, &UJ_int, 4); \
     if(oFile->string == NO) \
       {if(fwrite(oFile->pStart, 1, UJ_int, oFile->fp) != UJ_int) \
                 {UJError(3,"UJ_WRITE_RECORD"," write error");}}}

/*----------------------- now read macros ---------------*/

#define UJ_READ_HEADER_WITH_NAME(TYPE,INSTANCE) \
  if(iFile->ascii == NO)  {UJ_READ_SHORT(UJ_instance)} \
  INSTANCE = (struct TYPE *) calloc(1,sizeof(struct TYPE)); \
  if(INSTANCE == NULL) UJError(3,#TYPE"RC","calloc failed"); \
  UJ_READ_S_CHAR(INSTANCE->name); \
  if(UJobH.debugLvl > 1) \
    {fprintf(UJobH.fOut," \n Read bank "#TYPE":%s\n",INSTANCE->name);} \
 if(iFile->ascii == YES) \
      {UJSetBA( INSTANCE->name, #TYPE, INSTANCE);} \
 else {UJSetBAT(UJ_instance, TYPE ## UJstructH->id, INSTANCE);}
 
#define UJ_READ_HEADER_WITHOUT_NAME(TYPE,INSTANCE) \
  if(iFile->ascii == NO)  {UJ_READ_SHORT(UJ_short)} \
  if(UJobH.debugLvl > 1) {fprintf(UJobH.fOut," \n Read bank "#TYPE"\n");} \
  INSTANCE = (struct TYPE *) calloc(1,sizeof(struct TYPE)); \
  if(INSTANCE == NULL) UJError(3,#TYPE"RC","malloc failed"); \
 
#define UJ_READ_S_CHAR(NAME) \
if(iFile->ascii == YES)  \
 {if(UJQSRead(iFile) != 0) \
       {UJError(3,"UJ_READ_S_CHAR","error reading "#NAME);} \
   NAME = (char *)malloc(strlen(bankName)+1); \
   if(NAME == NULL) {UJError(3,"UJ_READ_S_CHAR","malloc failed");} \
   strcpy(NAME,bankName);} \
 else {UJ_READ_SHORT(UJ_strlen); \
    NAME = (char *)malloc(UJ_strlen+1); \
    if(NAME == NULL) {UJError(3,"UJ_READ_S_CHAR","malloc failed");} \
    if(UJ_strlen != fread(NAME, sizeof(char),UJ_strlen, iFile->fp)) \
       {UJError(3,"UJ_READ_S_CHAR","error reading "#NAME);}} 

#define UJ_READ_LONG(VALUE) UJ_READ_INT(VALUE)

#define UJ_READ_INT(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%ld",&VALUE);} \
  else {UJ_i = fread(&UJ_int, 4,1, iFile->fp); \
        VALUE = UJ_int;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_LONG","error reading "#VALUE);} 

#define UJ_READ_LONG_HEXA(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%lx",&VALUE);} \
  else {UJ_i = fread(&UJ_int, 4,1, iFile->fp); \
        VALUE = UJ_int;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_LONG_HEXA","error reading "#VALUE);} 

#define UJ_READ_SHORT(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%d",&VALUE);} \
  else {UJ_i = fread(&UJ_short, sizeof(short),1, iFile->fp); \
        VALUE = UJ_short;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_SHORT","error reading "#VALUE);} 

#define UJ_READ_FLOAT(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%e",&VALUE);} \
  else {UJ_i = fread(&UJ_float, sizeof(float),1, iFile->fp); \
        VALUE = UJ_float;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_FLOAT","error reading "#VALUE);} 

#define UJ_READ_DOUBLE(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%le",&VALUE);} \
  else {UJ_i = fread(&UJ_double, sizeof(double),1, iFile->fp); \
        VALUE = UJ_double;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_DOUBLE","error reading "#VALUE);} 

#define UJ_READ_VECTCD(VALUE,NDATA) UJ_READ_VECTD(VALUE,2*NDATA)

#define UJ_READ_VECTD(VALUE,NDATA) \
  if(iFile->ascii == YES) \
     {for(UJ_i=0; UJ_i<NDATA; UJ_i++) \
         {if(1 != fscanf(iFile->fp,"%le",&VALUE)) \
              {UJError(3,"UJ_READ_VECTD","error reading "#VALUE);} \
          VALUE++;}} \
  else {if(NDATA != fread(&VALUE, sizeof(double),NDATA, iFile->fp)) \
              {UJError(3,"UJ_READ_VECTD","error reading "#VALUE);}} 

#define UJ_READ_BOOL(VALUE) \
  if(iFile->ascii == YES) \
       {UJ_i = fscanf(iFile->fp,"%s",bankName); \
        if (strncmp(bankName,"YES",3) == 0) \
             {VALUE = YES;} \
        else {VALUE = NO;}} \
  else {UJ_i = fread(&UJ_char, sizeof(char),1, iFile->fp); \
        VALUE = UJ_short;} \
  if(UJ_i != 1) {UJError(3,"UJ_READ_BOOL ","error reading "#VALUE);} 

#define UJ_READ_STRUCT(TYPE, STRUCT) \
  if(iFile->ascii == YES) \
    {if(1!= fscanf(iFile->fp,"%s\n",bankName)) \
       {UJError(3,"UJ_READ_STRUCT","error reading #STRUCT");} \
     UJSetBR(bankName, #TYPE, &STRUCT);} \
  else \
    {UJ_READ_SHORT(UJ_type) \
     UJ_READ_SHORT(UJ_instance) \
     UJSetBRT(UJ_instance, UJ_type, &STRUCT);} 

#endif

