#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "UMathL.h"
#ifdef MEMWATCH
#    include "memwatch.h"
#endif


/*------------------------------------------ UMrandom -----*/
/* random : fonction donnant un nombre pseudo_aleatoire    */
/*          entre 0 et 1                                   */
/* Cette fonction est basee sur le generateur de lehmer.   */
/* (Voir l'article de Stephen K. Park and Keith W Miller   */
/* dans Cmmunications of the ACM October 1988 Volume 31    */
/* Number 10)                                              */
/* En particulier on est assure d'obtenir une periode      */
/* "complete" sur une machine capable de representer les   */
/* entiers compris entre -2**31 et 2**31 - 1               */
/* Si l'argument d'entree est 0, la racine ("seed") sera   */
/* la derniere racine calculee (la premiere fois la racine */
/* vaut 2**31 - 2).                                        */
/* Si on veut changer de racine l'argument doit etre un    */
/* nombre compris entre 1 et 2**31 - 2.                    */
/* Author: A. Pellier      17/01/92                        */
/* Correction by: X. Grave 13/04/95                        */
/*-------------------------------------------------------- */
float UMrandom(i)
long i;
{
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 long hi, lo;
 
 if (i != 0)  seed = i;

 hi   = seed / q;
 lo   = seed - hi * q;
 seed = a*lo - p*hi;
 if (seed <= 0) seed = seed + m;

 return ((float)seed / (float)m);
}
 
/*----------------------------------------------------- UMrgausd -----*/
/* rangausd generates a pseudo random number (double) with normal     */
/* (Gaussian) distribution with zero mean and unit variance.          */
/* Author: B. Mours        27/02/91                                   */
/* Correction by: X. Grave 13/04/95                                   */
/*--------------------------------------------------------------------*/
double UMrgausd(i)
long i;
{
 static long iset=0;
 static double gset;
 double fac,r,x,y;
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 long hi, lo;

 if (i != 0)  seed = i;
 
 if (iset == 1 )
    {iset = 0;
     return(gset);
    }
 
 iset = 1;
 do {
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0) seed = seed + m;
     x    = -1. + 2. * (double)seed / (double)m;
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0) seed = seed + m;
     y    = -1. + 2. * (double)seed / (double)m;
     r    = x*x + y*y;
    }
 while (r >= 1.0);
 
 if (r == 0.0)  r = 0.000001;
 fac  = sqrt(-2.*log(r)/r);
 gset = x * fac;

 return(y*fac);
}


/*----------------------------------------------------- UMrgausf -----*/
/* rangausf generates a pseudo random number (float) with normal      */
/* (Gaussian) distribution with zero mean and unit variance.          */
/* Author: B. Mours        27/02/91                                   */
/* Correction by: X. Grave 13/04/95                                   */
/*--------------------------------------------------------------------*/
float UMrgausf(i)
long i;
{
 static long iset=0;
 static float gset;
 float fac,r,x,y;
 long a = 16807;
 long m = 2147483647;
 long q = 127773;
 long p = 2836;
 static long seed = 2147483646;
 double hi, lo;
 
 if (i != 0)  seed = i;
 
 if (iset == 1 )
    {iset = 0;
     return(gset);
    }
 
 iset = 1;
 do {
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0)  seed = seed + m;
     x    = -1. + 2. * (double)seed / (double)m;
     hi   = seed / q;
     lo   = seed - hi * q;
     seed = a*lo - p*hi;
     if (seed <= 0)  seed = seed + m;
     y    = -1. + 2. * (double)seed / (double)m;
     r    = x*x + y*y;
    }
 while (r >= 1.0);
 
 if (r == 0.0)  r = 0.000001;
 fac  = sqrt(-2.*log(r)/r);
 gset = x * fac;

 return(y*fac);
}

/*----------------------------------------------------- UACDfft --------*/
void UACDfft(Vect, msign)
struct FrVectCD *Vect;
long msign;
/*--------------------------------------------------------------------*/
/* cdfft computes the finite Fourier transform of a complex periodic  */
/* sequence, whose period n must be a power of 2 (n=2**|msign|).      */
/* To ensure optimum use of storage, the same array is used for input */
/* and output. If msign is positive the direct Fourier transform is   */
/* computed. If msign is negative, the inverse Fourier transform is   */
/* computed. This code is a c translation of the routine D704 (CFFT)  */
/* from CERNLIB. See the CERN writeup for more information.           */
/* Author: B. Mours        27/02/91                                   */
/*--------------------------------------------------------------------*/

{
 double c,s,ur,ui,ur0,wr,wi,tr,ti,*a,temp;
 long  m,n,nv2,nm1,i,j,k,le,l,le1,ip;
 long indexI,indexJ,indexIp;

 a   = Vect->dataD;

 if (msign == 0) return;
 m   = msign;
 if (msign < 0) m = -m;
 n   = 1;
 for (i=0; i<m; i++) {n=2*n;}
 nv2 = n/2;
 nm1 = n-1;
 j   = 0;

 for (i=0; i<nm1; i++)
    {indexI = 2*i;
     if (i < j)
        {indexJ      = 2*j;
         tr          = a[indexJ  ];
         ti          = a[indexJ+1];
         a[indexJ  ] = a[indexI  ];
         a[indexJ+1] = a[indexI+1];
         a[indexI  ] = tr;
         a[indexI+1] = ti;
        }
     k = nv2-1;
     while ( k < j )
        {j = j - k - 1;
         k = (k-1) / 2;
        }
    j = j + k + 1;
   }

 for (i=0; i<n; i=i+2)
    {indexI       = 2*i;
     tr           = a[2*(i+1)];
     ti           = a[2*(i+1)+1];
     a[2*(i+1)  ] = a[indexI  ] - tr;
     a[2*(i+1)+1] = a[indexI+1] - ti;
     a[indexI  ]  = a[indexI  ] + tr;
     a[indexI+1]  = a[indexI+1] + ti;
    }

 if (m == 1) return;
 c = 0;
 s = 1.;
 if (msign < 0) s = -1.;
 le = 2;

 for (l=2; l<=m; l++)
    {wr  = c;
     wi  = s;
     ur  = wr;
     ui  = wi;
     c   = sqrt(c*.5+.5);
     s   = wi/(c+c);
     le1 = le;
     le  = le1+le1;

     for (i=0; i<n; i=i+le)
        {ip      = i + le1;
         indexI  = 2*i;
         indexIp = 2*ip;
         tr      = a[indexIp];
         ti      = a[indexIp+1];
         a[indexIp  ] = a[indexI  ] - tr;
         a[indexIp+1] = a[indexI+1] - ti;
         a[indexI  ]  = a[indexI  ] + tr;
         a[indexI+1]  = a[indexI+1] + ti;
        }

     for (j=1; j<le1; j++)
        {for (i=j; i<n; i=i+le)
            {ip      = i + le1;
             indexI  = 2*i;
             indexIp = 2*ip;
             tr      = a[indexIp  ]*ur - a[indexIp+1]*ui;
             ti      = a[indexIp+1]*ur + a[indexIp  ]*ui;
             a[indexIp  ] = a[indexI  ] - tr;
             a[indexIp+1] = a[indexI+1] - ti;
             a[indexI  ]  = a[indexI  ] + tr;
             a[indexI+1]  = a[indexI+1] + ti;
            }

         ur0 = ur;
         ur  = ur*wr - ui*wi;
         ui  = ui*wr + ur0*wi;
        }
    }

 for (i=1; i<n/2; i++)
    {temp         = a[2*i];
     a[2*i]       = a[2*(n-i)];
     a[2*(n-i)]   = temp;
     temp         = a[2*i+1];
     a[2*i+1]     = a[2*(n-i)+1];
     a[2*(n-i)+1] = temp;
    }

 return;
}

