/* ******************************************************************
******** USignalL.h *************************** USignalL.h **********
******** USignalL.h *************************** USignalL.h **********
****************************************************************** */
/* by R. Flaminio and F. Marion LAPP (Annecy) July 30, 1998 */

#if !defined(SIGNALLIB_DEFINED)
#define SIGNALLIB_DEFINED DONE

#include "UMathL.h"
#include "UJobL.h"
#include <glob.h>

extern struct UJstructH *USadcUJstructH;
extern struct UJstructH *USadderUJstructH;
extern struct UJstructH *USaverageUJstructH;
extern struct UJstructH *UScosUJstructH;
extern struct UJstructH *UScosineUJstructH;
extern struct UJstructH *USdelayUJstructH;
extern struct UJstructH *USdivideUJstructH;
extern struct UJstructH *USfilter1UJstructH;
extern struct UJstructH *USfilter2UJstructH;
extern struct UJstructH *USfilter3UJstructH;
extern struct UJstructH *USfilter4UJstructH;
extern struct UJstructH *USfilter5UJstructH;
extern struct UJstructH *USfreqSelUJstructH;
extern struct UJstructH *USgeneratorUJstructH;
extern struct UJstructH *USifUJstructH;
extern struct UJstructH *USifMUJstructH;
extern struct UJstructH *USignalUJstructH;
extern struct UJstructH *USignal6UJstructH;
extern struct UJstructH *US6SetUJstructH;
extern struct UJstructH *US6SmearUJstructH;
extern struct UJstructH *USsweepUJstructH;
extern struct UJstructH *UStransFUJstructH;
extern struct UJstructH *USburgUJstructH;
extern struct UJstructH *USimpulseUJstructH;

#define USMAXINPUT 20

struct USadc{               /* describes ADC parameters              */ 
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  struct USignal *input;    /* input signal                          */ 
  double inputGain;         /* input gain (0 means autoset)          */
  double overallGain;       /* overall gain (out = in * overallGain) */
  double range;             /* adc input range                       */      
  double noise;             /* noise rms (in ADC count  unit)        */
                            /* no noise is applied if noise <= 0.    */
  long   nBits;             /* number of ADC bits                    */
                            /* (the dynamic is +- 2**(nBits-1)       */
                            /* ------ end_of_SIO parameters ---------*/
  double dynamic;           /* 2**nBit                               */
  double iGain;             /* real input Gain                       */
  double oGain;             /* real output gain                      */
  BOOL autoSet;            
  struct USignal *out;      /* adc ioutput values                    */ 
  };
  
struct USadder{             /* combine several signals                */
  char   *name;             /* name for each instance                 */
  long   iClock;            /* clock number                           */
  long   nSignal;           /* number of signals to combine           */
  struct USignal *input[USMAXINPUT]; /* input signals (size is nSignal) */
  double param[USMAXINPUT]; /* input parameters (size is nSignal)     */
                            /* ------ end_of_SIO parameters ----------*/
  struct USignal *out;      /* signal struct to hold output result    */
  struct USadder *next;
  };

struct USaverage{           /* average samples of input signal       */ 
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  struct USignal *input;    /* input signal                          */ 
  long n;                   /* number of samples to be averaged      */
                            /* ------ end_of_SIO parameters ---------*/
  long index;               /*                                       */
  double buf;               /*                                       */
  struct USignal *out;      /* output signal                         */ 
  };

struct UScos{               /* compute cosine and sine of a signal    */ 
  char   *name;             /* name for each instance                 */
  long   iClock;            /* clock number                           */
  struct USignal *input;    /* hold input signal                      */
                            /* ------ end_of_SIO parameters ----------*/
  struct USignal *cosine;   /* signal struct to hold output result    */
  struct USignal *sine;     /* signal struct to hold output result    */
  };

struct UScosine{            /* generate a cosine function            */
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  double amplitude;         /* signal amplitude                      */
  double frequency;         /* signal frequency (in hertz)           */
  double phase0;            /* initial phase (in radian)             */
                            /* ------ end_of_SIO parameters----------*/
  double dPhi;              /* phase increase                        */
  double phase;             /* current phase                         */
  struct USignal *out;      /* output signal structure               */
  };

struct USdelay{           /* delay the output signal by a given time  */
  char   *name;           /* name for each instance                   */
  long iClock;            /* clock number                             */
  struct USignal *input;  /* input signal structure                   */
  double time;            /* time delay (in sec.)                     */
                          /* ------ end_of_SIO parameters ------------*/
  long deep;              /* number of clock tic to delay             */
  double *buffer;         /* tempory storage                          */
  struct USignal *out;    /* output signal structure                  */
  };

struct USdivide{          /* divide two signals                       */
  char   *name;           /* name for each instance                   */
  long   iClock;          /* clock number                             */
  struct USignal *input1; /* input signal structure                   */
  struct USignal *input2; /* input signal structure                   */
                          /* ------ end_of_SIO parameters-------------*/
  struct USignal *out;    /* output signal structure                  */
  };

struct USfilter{
  char   *name;           /* name for each instance                   */
  struct USignal *input;  /* input signal structure                   */
  long iClock;            /* clock number                             */
  long nNum;              /* number of numerator coef. (= order+1)    */
  double *aReverse;       /* numerator coefficients,  decreasing order*/
  long nDen;              /* number of denominator coef.(= order+1)   */
  double *bReverse;       /* denominator coefficients,decreasing order*/
                          /* ------ end_of_SIO parameters ------------*/
  long degnum;            /* order of numerator = nNum -1             */
  long degden;            /* order of denominator = nDen -1           */
  double *a;              /* numerator coeff.,   normal order: a[0]=a0*/
  double *b;              /* denominator coeff., normal order: b[0]=b0*/
  struct FrVectD *ad;
  struct FrVectD *bd;
  struct FrVectD *c;
  struct FrVectD *x;
  struct FrVectD *xtemp;
  struct FrVectD *d;
  struct FrVectD *u;
  struct FrVectD *y;
  struct FrVectD *ytemp;
  struct USignal *out;      /* output signal structure               */
  };

struct USfilter1 {       
  char   *name;             /* name for each instance                */
  struct USignal *input;    /* input signal structure                */
  long   iClock;            /* clock number                          */
  double a1;                /* coef.                                 */
  double a0;                /* coef.                                 */
  double b1;                /* coef.                                 */
  double b0;                /* coef.                                 */
                            /* ------ end_of_SIO parameters----------*/
  double alfa1;             /* coef.                                 */
  double alfa0;             /* coef.                                 */
  double beta1;             /* coef.                                 */
  double beta0;             /* coef.                                 */
  double reg1;              /* register                              */
  double reg0;              /* register                              */
  struct USignal *out;      /* output signal structure               */
 };

struct USfilter2 {       
  char   *name;             /* name for each instance                */
  struct USignal *input;    /* input signal structure                */
  long   iClock;            /* clock number                          */
  double a2;                /* coef.                                 */
  double a1;                /* coef.                                 */
  double a0;                /* coef.                                 */
  double b2;                /* coef.                                 */
  double b1;                /* coef.                                 */
  double b0;                /* coef.                                 */
                            /* ------ end_of_SIO parameters----------*/
/*
  How does it work ? The transfer function (Laplace transform s=i*w) is:

                Yout(s)     a2 * s**2 + a1 * s + a0
                ------- =  -------------------------
                Yinp(s)     b2 * s**2 + b1 * s + b0

    Example 1 : first order low-pass filter at f0 with unity gain at dc
                a2=0.  a1=0.  a0=1.  b2=0.  b1=1/(2*pi*f0)  b0=1. 

    Example 2 : first order high-pass filter at f0 with unity gain at dc
                a2=0.  a1=1/(2*pi*f0)  a0=1.  b2=0.  b1=0.  b0=1.

    Example 3 : second order low-pass filter at f0 with quality factor Q 
                and unity gain at dc (e.g. pendulum)
                a2=0. a1=0. a0=1. b2=1/(2*pi*f0)**2  b1=1/(2*pi*f0*Q)  b0=1

    Example 4 : integrator with time constant t
                a2=0. a1=0. a0=1. b2=0. b1=t b0=0.
*/    
  double alfa2;             /* coef.                                 */
  double alfa1;             /* coef.                                 */
  double alfa0;             /* coef.                                 */
  double beta2;             /* coef.                                 */
  double beta1;             /* coef.                                 */
  double beta0;             /* coef.                                 */
  double reg2;              /* register.                             */
  double reg1;              /* register                              */
  double reg0;              /* register                              */
  struct USignal *out;      /* output signal structure               */
 };

struct USfilter3 {       
  char   *name;             /* name for each instance                */
  struct USignal *input;    /* input signal structure                */
  long   iClock;            /* clock number                          */
  double a3;                /* coef.                                 */
  double a2;                /* coef.                                 */
  double a1;                /* coef.                                 */
  double a0;                /* coef.                                 */
  double b3;                /* coef.                                 */
  double b2;                /* coef.                                 */
  double b1;                /* coef.                                 */
  double b0;                /* coef.                                 */
                            /* ------ end_of_SIO parameters----------*/
  double alfa3;             /* coef.                                 */
  double alfa2;             /* coef.                                 */
  double alfa1;             /* coef.                                 */
  double alfa0;             /* coef.                                 */
  double beta3;             /* coef.                                 */
  double beta2;             /* coef.                                 */
  double beta1;             /* coef.                                 */
  double beta0;             /* coef.                                 */
  double reg3;              /* register.                             */
  double reg2;              /* register.                             */
  double reg1;              /* register                              */
  double reg0;              /* register                              */
  struct USignal *out;      /* output signal structure               */
 };

struct USfilter4 {       
  char   *name;             /* name for each instance                */
  struct USignal *input;    /* input signal structure                */
  long   iClock;            /* clock number                          */
  double a4;                /* coef.                                 */
  double a3;                /* coef.                                 */
  double a2;                /* coef.                                 */
  double a1;                /* coef.                                 */
  double a0;                /* coef.                                 */
  double b4;                /* coef.                                 */
  double b3;                /* coef.                                 */
  double b2;                /* coef.                                 */
  double b1;                /* coef.                                 */
  double b0;                /* coef.                                 */
                            /* ------ end_of_SIO parameters----------*/
  double alfa4;             /* coef.                                 */
  double alfa3;             /* coef.                                 */
  double alfa2;             /* coef.                                 */
  double alfa1;             /* coef.                                 */
  double alfa0;             /* coef.                                 */
  double beta4;             /* coef.                                 */
  double beta3;             /* coef.                                 */
  double beta2;             /* coef.                                 */
  double beta1;             /* coef.                                 */
  double beta0;             /* coef.                                 */
  double reg4;              /* register.                             */
  double reg3;              /* register                              */
  double reg2;              /* register.                             */
  double reg1;              /* register                              */
  double reg0;              /* register                              */
  struct USignal *out;      /* output signal structure               */
 };
  
struct USfilter5 {       
  char   *name;             /* name for each instance                */
  struct USignal *input;    /* input signal structure                */
  long   iClock;            /* clock number                          */
  double a5;                /* coef.                                 */
  double a4;                /* coef.                                 */
  double a3;                /* coef.                                 */
  double a2;                /* coef.                                 */
  double a1;                /* coef.                                 */
  double a0;                /* coef.                                 */
  double b5;                /* coef.                                 */
  double b4;                /* coef.                                 */
  double b3;                /* coef.                                 */
  double b2;                /* coef.                                 */
  double b1;                /* coef.                                 */
  double b0;                /* coef.                                 */
                            /* ------ end_of_SIO parameters----------*/
  double alfa5;             /* coef.                                 */
  double alfa4;             /* coef.                                 */
  double alfa3;             /* coef.                                 */
  double alfa2;             /* coef.                                 */
  double alfa1;             /* coef.                                 */
  double alfa0;             /* coef.                                 */
  double beta5;             /* coef.                                 */
  double beta4;             /* coef.                                 */
  double beta3;             /* coef.                                 */
  double beta2;             /* coef.                                 */
  double beta1;             /* coef.                                 */
  double beta0;             /* coef.                                 */
  double reg5;              /* register.                             */
  double reg4;              /* register.                             */
  double reg3;              /* register                              */
  double reg2;              /* register.                             */
  double reg1;              /* register                              */
  double reg0;              /* register                              */
  struct USignal *out;      /* output signal structure               */
 };

struct USfreqSel{
  char   *name;             /* name for each instance                 */
  long   iClock;            /* clock number                           */
  struct USignal *input;    /* input signal structure                 */
  long   nBinP2;            /* number of clock tic to dump(power of 2)*/
  double freq;              /* frequency                              */ 
  double band;              /* bandwidth                              */
                            /* ------ end_of_SIO parameters ----------*/
  long nBin;                /* number of clock tic to dump            */
  double dnu;               /* frequency bin width                    */
  double dt;                /* sampling frequency                     */
  long i;                   /* vector index                           */
  double coef;              /* normalisation coef                     */
  long   imin;
  long   imax;
  struct FrVectD *vect;     /* working vector                         */
  struct FrVectD *window;   /* haning window                          */ 
  struct USignal *out;      /* output signal                          */ 
  };

struct USgenerator {        /* generate white and gaussian noise     */
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  double sigma;             /* spectral density (/sqrt(Hz))          */
                            /* the RMS of the gaussian distribution  */
                            /* will be sigma * sqrt(f) with f the    */
                            /* clock frequency                       */
                            /* ------ end_of_SIO parameters----------*/
  struct USignal *out;      /* output signal structure               */
 };
  
struct USif {               /* to provide run control                */ 
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  struct USignal *input;    /* signal used for the condition or TIME */
                            /* the keyword TIME can be used to put a */
                            /* condition on the ellapsed time(in sec)*/ 
  char   *condition;        /* condition: > or <                     */
  double value;             /* condition value                       */
  struct USignal *yes;      /* signal ouputed if the condition = YES */
  struct USignal *no;       /* signal ouputed if the condition = NO  */
                            /* ------ end_of_SIO parameters----------*/
  struct USignal *out;      /* output signal structure               */
  int type;                 
  };
struct USifM {              /* to provide run control                */ 
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  char   *stringtest;       /* string  */

  struct USignal *yes;      /* signal ouputed if the condition = YES */
  struct USignal *no;       /* signal ouputed if the condition = NO  */
                            /* ------ end_of_SIO parameters----------*/
  struct USifMtest *Test;   /* pointer to firts test   */
  struct USignal *out;      /* output signal structure               */
  int type;                 
  };

struct USifMtest{
  struct USifMtest *right; 
  struct USifMtest *left;
  int ope;
  struct USignal *input; 
};

struct USignal{ 
  char   *name;             /* name for each instance                 */
  double value;             /* signal value                           */
                            /* ------ end_of_SIO parameters----------*/
  struct USignal *next;
  };

struct USignal6             /*   */
{
                            /* ------ end_of_SIO parameters ----------*/
  char   *name;             /* name for each instance                 */
  struct USignal *s[6];     /* signal values                          */
  struct USignal6 *next;   
  };

struct US6Set      {        /* put 6 signals in one signal6 structure */
   char   *name;            /* name for each instance                 */
   long  iClock;            /* clock number                           */
   struct USignal *in[6];   /* signal values                          */
                            /* ------ end_of_SIO parameters ----------*/
   struct USignal6 *out;    /* output values                          */
   struct US6Set *next;     /* next position structure                */
};


struct US6Smear      {      /* smear a signal6  structure             */
   char   *name;            /* name for each instance                 */
   long  iClock;            /* clock number                           */
   struct USignal6 *in;     /* input  signal6  structure              */
   double rms[6];	    /* 6 rms values                           */
                            /* ------ end_of_SIO parameters ----------*/
   struct USignal6 *out;    /* smeared values                         */
   struct US6Smear *next;   /* next position structure                */
};

struct USsweep {            /* generate a linear function            */
  char   *name;             /* name for each instance                */
  long   iClock;            /* clock number                          */
  double start;             /* starting value                        */
  double slope;             /* slope (unit: sec-1)                   */
                            /* ------ end_of_SIO parameters----------*/
  double step;              /* increment value: slope*dt             */
  struct USignal *out;      /* output signal structure               */
 };
  
struct UStransF{
  long iClock;            /* clock number                             */
  long id;                /* histogram identifier                     */
  char* nameModulus;      /* modulus histogram name                   */
  char* namePhase;        /* phase histogram name                     */
  long nBin;              /* histogram bin number                     */
  double numin;           /* frequency lower bound                    */
  double numax;           /* frequency upper bound                    */
  long nNum;              /* number of numerator coef. (= order+1)    */
  double *aReverse;       /* numerator coefficients,  decreasing order*/
  long nDen;              /* number of denominator coef.(= order+1)   */
  double *bReverse;       /* denominator coefficients,decreasing order*/
                          /* ------ end_of_SIO parameters ------------*/
  long degnum;            /* order of numerator = nNum -1             */
  long degden;            /* order of denominator = nDen -1           */
  double *a;              /* numerator coeff.,   normal order: a[0]=a0*/
  double *b;              /* denominator coeff., normal order: b[0]=b0*/
  };

struct USburg {
  char *name;             /* name for each instance                   */
  long iClock;            /* clock number                             */
  struct USignal *input;  /* signal to be modeled with an AR series   */
  long nData;             /* data bunch to be used to estimate        */
  long pNum;              /* number of AR coefficients to use         */
                          /* ------ end_of_SIO parameters ------------*/
  long last;              /* position of the last value read          */
  long nDataRead;         /* data buffer available                    */
  struct USignal *out;    /* output whitened signal                   */
  double sigma;           /* scale of the gaussian innovation noise   */
  long pUsed;             /* effective number of coefficients computed*/
  double *a;              /* vector of AR coefficients                */
  double *inputData;      /* buffer of past data values               */
};

struct USimpulse {
  char *name;             /* name for each instance                   */
  long iClock;            /* clock number                             */
  double frequency;       /* frequency of occurrence of a new impulse */
  double scale;           /* common multiplying factor                */
  long nSignal;           /* number of columns in the ASCII files     */
  char *pattern;          /* pattern of the filenames to consider     */
                          /* ------ end_of_SIO parameters ------------*/
  double mu;              /* Poisson average for a single tick        */
  long fileNum;           /* index of the current file read           */
  FILE * file;            /* current file being read                  */
  struct USignal **out;   /* output stream with impulses              */
  glob_t pglob;           /* filenames matched                        */
};

void USLibIni(void);

void USadcEnd(struct USadc *adc);
void USadcFree(struct USadc *adc);
void USadcIni(struct USadc *adc, double dt);
void USadcRead(struct UJfile *iFile);
void USadcSim(struct USadc *adc, long time);

void USadderFree(struct USadder *adder);
void USadderIni(struct USadder *adder, double dt);
void USadderMsg(struct USadder *adder, char *msgType, 
                char *param, long reportL, char *report);
void USadderRead(struct UJfile *iFile);
void USadderSim(struct USadder *adder, long time);

void USaverageFree(struct USaverage *average);
void USaverageIni(struct USaverage *average, double dt);
void USaverageRead(struct UJfile *iFile);
void USaverageSim(struct USaverage *average, long time);

void UScosFree(struct UScos *cosine);
void UScosRead(struct UJfile *iFile);
void UScosSim(struct UScos *cosine, long time);

void UScosineFree(struct UScosine *cosine);
void UScosineIni(struct UScosine *cosine, double dt);
void UScosineRead(struct UJfile *iFile);
void UScosineSim(struct UScosine *cosine, long time);

void USdelayFree(struct USdelay *delay);
void USdelayIni(struct USdelay *delay, double dt);
void USdelayRead(struct UJfile *iFile);
void USdelaySim(struct USdelay *delay, long time);

void USdivideFree(struct USdivide *divide);
void USdivideRead(struct UJfile *iFile);
void USdivideSim(struct USdivide *divide, long time);

void USfilterFree(struct USfilter *filter);
void USfilterIni(struct USfilter *filter, double dt);
void USfilterRead(struct UJfile *iFile);
void USfilterSim(struct USfilter *filter, long time);

void USfilter1Free(struct USfilter1 *filter);
void USfilter1Ini(struct USfilter1 *filter, double dt);
void USfilter1Read(struct UJfile *iFile);
void USfilter1Sim(struct USfilter1 *filter, long time);

void USfilter2Free(struct USfilter2 *filter);
void USfilter2Ini(struct USfilter2 *filter, double dt);
void USfilter2Read(struct UJfile *iFile);
void USfilter2Sim(struct USfilter2 *filter, long time);

void USfilter3Free(struct USfilter3 *filter);
void USfilter3Ini(struct USfilter3 *filter, double dt);
void USfilter3Read(struct UJfile *iFile);
void USfilter3Sim(struct USfilter3 *filter, long time);

void USfilter4Free(struct USfilter4 *filter);
void USfilter4Ini(struct USfilter4 *filter, double dt);
void USfilter4Read(struct UJfile *iFile);
void USfilter4Sim(struct USfilter4 *filter, long time);

void USfilter5Free(struct USfilter5 *filter);
void USfilter5Ini(struct USfilter5 *filter, double dt);
void USfilter5Read(struct UJfile *iFile);
void USfilter5Sim(struct USfilter5 *filter, long time);

void USfreqSelEnd(struct USfreqSel *freqSel);
void USfreqSelIni(struct USfreqSel *freqSel, double dt);
void USfreqSelRead(struct UJfile *iFile);
void USfreqSelSim(struct USfreqSel *freqSel, long time);

void USgeneFree(struct USgenerator *generator);
void USgeneIni(struct USgenerator *generator, double dt);
void USgeneRead(struct UJfile *iFile);
void USgeneSim(struct USgenerator *generator, long time);


void USifIni(struct USif *usif, double dt);
void USifRead(struct UJfile *iFile);
void USifSim(struct USif *usif,long time);

int USifMstrcount(char *p,int t);
char *USifMstrchr(char *p,int t);
int USifMcutparen(char *p);
void USifMFreetest(struct USifMtest *test);
double USifMEval(struct USifMtest *test);
struct USifMtest *USifMreads(char *s);
void USifMIni(struct USifM *usifM, double dt);
void USifMRead(struct UJfile *iFile);
void USifMSim(struct USifM *usifM,long time);
void USifMFree(struct USifM *usifM);

void USignalEnd();
void USignalFree(struct USignal *signal);
struct USignal *USignalFind(char *name);
struct USignal *USignalNew(char *name, BOOL ascii);
void USignalRead(struct UJfile *iFile);

struct USignal6 *USignal6New(char *name, BOOL ascii);
struct USignal6 *USignal6Find(char *name);
void USignal6Free(struct USignal6 *s6);

void US6SetFree(struct US6Set *set6);
void US6SetRead(struct UJfile *iFile);
void US6SetSim(struct US6Set *set6, long time);

void US6SmearFree(struct US6Smear *position);
void US6SmearIni(struct US6Smear *position, double dt);
void US6SmearRead(struct UJfile *iFile);
void US6SmearSim(struct US6Smear *position, long time);

void USsweepFree(struct USsweep *sweep);
void USsweepIni(struct USsweep *sweep, double dt);
void USsweepRead(struct UJfile *iFile);
void USsweepSim(struct USsweep *sweep, long time);

void UStransFRead(struct UJfile *iFile);
void UStransFEnd(struct UStransF * transF);

void USburgIni(struct USburg *burg, double dt);
void USburgSim(struct USburg *burg, long time);
void USburgEnd(struct USburg * burg);
void USburgRead(struct UJfile *iFile);
void USburgFree(struct USburg *burg);

void USimpulseIni(struct USimpulse *impulse, double dt);
void USimpulseSim(struct USimpulse *impulse, long time);
void USimpulseEnd(struct USimpulse * impulse);
void USimpulseRead(struct UJfile *iFile);
void USimpulseFree(struct USimpulse *impulse);

#endif
