static char FdShmId[]="$Header: /scratch/cvsroot/Dy/src/shm_nocm/FdShm.c,v 1.1.1.9 2004-11-17 16:34:49 verkindt Exp $";

/**************************************************/
/*                                                */
/* File: FdShm.c                                  */
/* Copyright (C) 2003, D. Verkindt, B. Mours      */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/


/*************************** FdShm.c *****************************/
/*                                                               */
/*               FRAMES IN SHARED MEMORY                         */
/*                     LIBRARY                                   */
/* MODULE  : FdShm.c                                             */
/* AUTHOR  : D. Verkindt, LAPP (verkindt@lapp.in2p3.fr)          */
/* VERSION : v4r09 modified to be independant of Cfg and Fd      */
/* DATE: 25/02/2004                                              */
/*                                                               */
/*   This library allows a server (for instance the Main Frame   */
/*   Builder producer) to create a shared memory segment and     */
/*   to put frames into this shared memory. It allows also       */
/*   several consumers (for instance the Main Frame Builder      */
/*   consumers) to read those frames in shared memory.           */
/*                                                               */
/************************** FdShm.c ******************************/

#include <stdio.h>
#include <signal.h>
#include <dirent.h>
#include "FdShm.h"

#ifdef MEMWATCH
#include "memwatch.h"
#endif

#define FDSHMVERSION  "FdShm(v4r09)"
#define PRODUCERNAME  "producer"

/*
static FdShmPrinter_t FdShmPrinter = vprintf;
*/

/*----------------------------------------------- FdShmInstallPrinter ---*/
/*
FdShmPrinter_t FdShmInstallPrinter (FdShmPrinter_t printer)
{
  FdShmPrinter_t old;

  old = FdShmPrinter;
  FdShmPrinter = printer;

  return(old);
}
*/

/*-------------------------------------------------- FdShmDoPrint ---*/
/*
static void FdShmDoPrint (const char *format, ...)
{
 va_list args;

 if( !FdShmPrinter || !format ) return;

 va_start( args, format );
 FdShmPrinter( format, args );
 va_end( args );

 return;
}
*/



/*----------------------------------------- FdShmRewind ----------*/

long FdShmRewind (struct ShmInfo *shm, char *csmName, long nback)
{
 long csmN;

 if (nback >= shm->nFrameMax)
    {fprintf(stderr,"FdShmRewind> nback > nFrameMax (%d > %d)\n", nback, shm->nFrameMax);
     return(-1);
    }

 csmN = FdShmGetCsmN (shm,csmName);

 if (csmN <= 0)
    {fprintf(stderr,"FdShmRewind> csmN=%d for consumer %s\n", csmN, csmName);
     return(-1);
    }

 if (nback >= shm->iread[csmN])
    {fprintf(stderr,"FdShmRewind> nback > iread (%d > %d)\n", nback, shm->iread[csmN]);
     return(-1);
    }

 shm->iread[csmN] -= nback;

 return(shm->iread[csmN]);
}


/*------------------------------------------ FdShmInfoFree --------*/

void FdShmInfoFree (struct ShmInfo *shm)
{
 if (shm == NULL) return;

 if (shm->frameSh)    free(shm->frameSh);
 if (shm->frameSize)  free(shm->frameSize);

 free(shm);
}


/*------------------------------------------ FdShmGetCsmN --------*/

long FdShmGetCsmN (struct ShmInfo *shm, char *name)
{
 long csmN,i;

 csmN = -1;

 if (shm->nConsumer >= 0)
    {for (i=0; i<=shm->nConsumer; i++)
        {if (strcmp(name,shm->csmName[i]) == 0) {csmN = i; break;}}
    }

 return(csmN);
}


/*----------------------------------------- FdShmGetI ----------*/

long FdShmGetI (struct ShmInfo *shm, char *csmName)
{
 long i,csmN;

 csmN = FdShmGetCsmN (shm,csmName);

 if (csmN == 0)  i = shm->iwrite;
 else            i = shm->iread[csmN];

 return(i);
}


/*----------------------------------------- FdShmGetIwrite ----------*/

long FdShmGetIwrite (struct ShmInfo *shm)
{
 long i;
 i = shm->iwrite;
 return(i);
}


/*----------------------------------------- FdShmGetnFrameMax ----------*/

long FdShmGetnFrameMax (struct ShmInfo *shm)
{
 long nFrameMax;
 nFrameMax = shm->nFrameMax;
 return(nFrameMax);
}


/*----------------------------------------- FdShmGetOutputName ----------*/

char *FdShmGetOutputName (struct ShmInfo *shm)
{
 return(shm->outputName);
}


/*------------------------------------------ FdShmCheckConsumers --------*/

int FdShmCheckConsumers (struct ShmInfo *shm)
{
 long i,j, iread, iwrite, stillAlive;
 int lateCsm, delta;
 char *removedCsm;
 int nremoved, removedList[NOUTMAX+1];

 if (shm->nConsumer == 0) return(0);


 /*--- Check for consumers that would be dead ---*/

 removedCsm = (char *)malloc(2048);
 strcpy(removedCsm," ");

 iwrite = shm->iwrite;

 nremoved   = 0;
 stillAlive = 0;

 for (i=1; i<=shm->nConsumer; i++)
     {
      iread = shm->iread[i];
      if ((iread < iwrite - shm->nFrameMax  &&  iwrite%shm->nFrameMax == 0)  ||
          (iread < iwrite+1 - shm->nFrameMax  &&  shm->toBeRemoved[i] == 1))
         {
          stillAlive = kill (shm->csmPid[i],0);
          if (stillAlive != 0)
             {
              removedList[nremoved] = i;
              nremoved++;
             }
         }
     }

 for (j=0; j<nremoved; j++)
     {
      i = removedList[j];
      strcat(removedCsm, shm->csmName[i]);
      strcat(removedCsm, " ");
      FdShmRemoveCsm (shm, i);
     }

 if (strlen(removedCsm) <= 1)
    {free(removedCsm);
     removedCsm = NULL;
    }

 if (removedCsm) free(removedCsm);


 /*--- Check for consumers too late with respect to producer ---*/
 /*--- only if not Users or Shift ones ---*/

 lateCsm = 0;
 for (i=1; i<=shm->nConsumer; i++)
    {
     if (strstr(shm->csmName[i],"Users") != 0  ||
         strstr(shm->csmName[i],"Shift") != 0) continue;

     delta = shm->iwrite - shm->iread[i];

     if ((delta >= shm->nFrameMax)  &&  (delta%shm->nFrameMax == 0)
                                    &&  (shm->toBeRemoved[i] == 0) )
        {
         lateCsm = 1;
        }
    }

 return(lateCsm);
}


/*------------------------------------------ FdShmRemoveCsm --------*/

int FdShmRemoveCsm (struct ShmInfo *shm, int csmN)
{
 int j, stillAlive;

 if (csmN == 0) return(0);  /* do not remove the producer */

 stillAlive = kill (shm->csmPid[csmN],0);  /* do not remove an active process */
 if (stillAlive == 0  &&  shm->toBeRemoved[csmN] == 0) return(-1);

 shm->selfStatus = 1;

 if (shm->nConsumer > 1)
    {
     for (j=csmN; j<=shm->nConsumer-1; j++)
        {
         shm->iread[j]       = shm->iread[j+1];
         shm->csmPid[j]      = shm->csmPid[j+1];
         shm->toBeRemoved[j] = shm->toBeRemoved[j+1];
         strcpy(shm->csmName[j], shm->csmName[j+1]);
        }
    }
 shm->nConsumer--;
 shm->selfStatus = 0;

 return(csmN);
}


/*------------------------------------------ FdShmFree --------*/

void FdShmFree (struct ShmInfo *shm, char *name)
{
 long csmN;

 if (name == NULL  ||  shm == NULL) return;

 csmN = FdShmGetCsmN (shm,name);

 if (csmN == 0)
    {
     shmctl(shm->shmid, IPC_RMID, 0);
     printf ("FdShmFree> Shared memory segment marked for deletion\n");
    }
 else
    {
     shm->toBeRemoved[csmN] = 1;
     FdShmRemoveCsm (shm, csmN);
    }

 shmdt((void *)shm);

 return;
}


/*------------------------------------------ FdShmClose --------*/
/* only for consumers */

void FdShmClose (struct ShmInfo *shm, char *name)
{
 long csmN;

 if (name == NULL  ||  shm == NULL) return;

 csmN = FdShmGetCsmN (shm,name);

 if (csmN == 0)
    {
     return;
    }
 else
    {
     shm->toBeRemoved[csmN] = 1;
    }

 shmdt((void *)shm);

 return;
}


/*-------------------------------------------  FdShmNew -------*/

struct ShmInfo *FdShmNew (long frameSize, long nFrame, char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 long segsize;

 temp = (struct ShmInfo *) calloc(1,sizeof(struct ShmInfo));

 temp->key = FdShmMakeKey(idName);

 if (frameSize <= 0)  frameSize = FRAMESIZE_MAX;

 segsize = frameSize * nFrame;

 /*--- If called by the producer ---*/

 if (strcmp (name, PRODUCERNAME) == 0)
    {
     if ((temp->shmid = shmget(temp->key, segsize, IPC_CREAT|IPC_EXCL|0666)) == -1) 
        {
         printf("FdShmNew> Shm segment already exists: key=%d  shmid=%d ",
                              temp->key, temp->shmid);

         temp->shmid = shmget(temp->key, segsize, 0);

         shmctl(temp->shmid, IPC_RMID, 0);
         temp->shmid = shmget(temp->key, segsize, IPC_CREAT|IPC_EXCL|0666);

         printf("FdShmNew> Shm segment marked for deletion and recreated (shmid=%d)\n",
                             temp->shmid);

         if (temp->shmid == -1) 
	    {
             printf("FdShmNew> Shm segment problem when recreating it: key=%d  shmid=%d\n",
                                 temp->key, temp->shmid);
             free(temp);
             return(NULL);
            }
        }
     else
        {
         printf ("FdShmNew> Creating new shm segment: key=%d  shmid=%d\n", temp->key, temp->shmid);
        }
    }


 /*--- If called by a consumer ---*/
 else
    {
     if ((temp->shmid = shmget(temp->key, segsize, 0)) == -1) 
        {
         printf("FdShmNew> Shm segment does not exists: key=%d  shmid=%d\n",
                             temp->key, temp->shmid);
	 free(temp);
         return(NULL);
        }
    }

 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((temp->segptr = shmat(temp->shmid, 0, 0)) == (void *)-1)
    {
     printf("FdShmNew> Could not attach Shm segment shmid=%d\n", temp->shmid);
     FdShmFree(temp, name);
     free(temp);
     return(NULL);
    }

 if (temp->segptr == NULL)
    {
     printf("FdShmNew> shm segptr is NULL");
     FdShmFree(temp, name);
     free(temp);
     return(NULL);
    }

 if (strcmp (name, PRODUCERNAME) == 0)
    {printf("FdShmNew> shm segptr is %ld (shmid=%d)\n", (long)temp->segptr, temp->shmid);}

 /*--- If called by a producer ---*/

 if (strcmp (name, PRODUCERNAME) == 0)
    {
     FdShmInitStatus(temp, nFrame, frameSize);
     temp->size = segsize;
     temp->csmPid[0] = getpid();
     strcpy(temp->csmName[0], idName);
     memcpy(temp->segptr, temp, sizeof *temp);
    }

 segptr = temp->segptr;
 free(temp);
 temp = (struct ShmInfo *)segptr;

/***
fprintf(stderr,"FdShmNew: temp=%x  segptr=%x  temp->version=%s  temp->key=%d  temp->shmid=%d\n",
                    temp, (long)segptr, temp->version, temp->key, temp->shmid);
***/

 /*--- If called by a consumer ---*/

 if (strcmp (name, PRODUCERNAME) != 0)
    {
     temp->segptr = segptr;

     FdShmCsmIni (temp, name);

     if (temp->nConsumer > NOUTMAX)
        {
         printf("FdShmNew> Max nbr of consumer is %d. Cannot add %s\n", NOUTMAX, name);
         temp->nConsumer--;
         return(NULL);
        }
    }

 return(temp);
}


/*-------------------------------------------  FdShmOpen -------*/
/* only for consumers */

struct ShmInfo *FdShmOpen (char *idName, char *name)
{
 struct ShmInfo *temp;
 void *segptr;
 key_t key;
 int shmid;

 if(strcmp (name, PRODUCERNAME) == 0) return(NULL);

 key = FdShmMakeKey(idName);

 if ((shmid = shmget(key, 0, 0)) == -1) 
    {
     return(NULL);
    }


 /*--- Attach (map) the shared memory segment into the current process ---*/

 if ((segptr = shmat(shmid, 0, 0)) == (void *)-1)
    {
     printf("FdShmOpen> Could not attach Shm segment shmid=%d\n",temp->shmid);
     return(NULL);
    }

 if (segptr == NULL)
    {
     printf("FdShmOpen> shm segptr is NULL\n");
     return(NULL);
    }

 temp = (struct ShmInfo *)segptr;


/***
fprintf(stderr,"FdShmOpen: temp=%x  segptr=%x  temp->version=%s  temp->key=%d  temp->shmid=%d\n",
                    temp, (long)segptr, temp->version, temp->key, temp->shmid);
***/

 FdShmCsmIni (temp, name);

 if (temp->nConsumer > NOUTMAX)
    {
     printf("FdShmNew> Max nbr of consumer is %d. Cannot add %s\n",NOUTMAX, name);
     temp->nConsumer--;
     return(NULL);
    }

 return(temp);
}

/*------------------------------------------- FdShmMakeKey -------*/

int FdShmMakeKey (char *name)
{
 int i, length, key, val, nbytes;
 
 length = strlen(name);

 key = 0;
 for (i=0; i<length; i=i+4)
     {val = 0;
      if (length-i < 4) nbytes=length-i;
      else              nbytes = 4;
      memcpy(&val, &name[i], nbytes);
      key += val;
      if (key < 0) key += INT_MAX;
     }

 return(key);
}


/*-------------------------------------------  FdShmCsmIni -------*/

void FdShmCsmIni (struct ShmInfo *temp, char *name)
{
 int csmN, i;

 csmN = 0;
 for (i=1; i<=temp->nConsumer; i++)
    {if (strcmp(name, temp->csmName[i]) == 0) {csmN = i; break;}
    }

 if (csmN > 0)
    {temp->csmPid[csmN]      = getpid();
     temp->toBeRemoved[csmN] = 0;
    }
 else
    {
     temp->nConsumer++;
     strcpy(temp->csmName[temp->nConsumer], name);
     temp->toBeRemoved[temp->nConsumer] = 0;
     temp->csmPid[temp->nConsumer]      = getpid();
     temp->iread[temp->nConsumer]       = temp->iwrite;
     if (temp->iwrite < temp->nFrameMax)
        {temp->iread[temp->nConsumer] = 0;}
     else
        {temp->iread[temp->nConsumer] = temp->iwrite - temp->nFrameMax + 1;}
    }

 if (temp->nConsumer <= NOUTMAX)
    {if (csmN > 0)
        {
         printf("FdShmCsmIni> Consumer %s (%ld) back in the list. Positionned at iread=%ld\n",
                 temp->csmName[temp->nConsumer], temp->nConsumer, temp->iread[temp->nConsumer]);
        }
     else
        {
         printf("FdShmCsmIni> New consumer %s (%ld) added to the list. Positionned at iread=%ld\n",
                 temp->csmName[temp->nConsumer], temp->nConsumer, temp->iread[temp->nConsumer]);
        }
    }

 return;
}


/*---------------------------------------- FdShmInitStatus ---------*/

void FdShmInitStatus (struct ShmInfo *shm, long nFrame, long frameSize)
{
 long i;

 strcpy(shm->version, FDSHMVERSION);

 strcpy(shm->outputName,NOTRM);
 shm->statusSize   = sizeof *shm;
 shm->frameSizeMax = frameSize;
 shm->nFrameMax    = nFrame;
 shm->nConsumer    = 0;
 shm->selfStatus   = 0;
 shm->latency      = 10;

 shm->iwrite = 0;

 shm->frameSh[0] = shm->statusSize + (long)shm->segptr;

 for (i=1; i< shm->nFrameMax; i++)
     {shm->frameSh[i] = shm->frameSh[0] + i * shm->frameSizeMax;}

 return;
}


/*----------------------------------------- FdShmWrite ----------*/

long FdShmWrite (struct ShmInfo *shm, struct FrameH *frame, long compress)
{
 long nBytes, iwrite, address;

 iwrite = shm->iwrite % shm->nFrameMax;

 address = shm->frameSh[iwrite];

 nBytes = FrameWriteToBuf(frame, compress, (char *)address, shm->frameSizeMax);

 if (nBytes <= 0)
    {printf("FdShmWrite> frame not written in shm\n");}

 shm->frameSize[iwrite] = nBytes;
 shm->frameGPS[iwrite]  = frame->GTimeS;

 shm->iwrite++;

 return(nBytes);
}


/*----------------------------------------- FdShmWriteBuf ----------*/

void FdShmWriteBuf (struct ShmInfo *shm, char *buf, int nBytes)
{
 long iwrite, address;

 shm->iwrite++;
 iwrite = shm->iwrite % shm->nFrameMax;

 shm->frameSize[iwrite] = (long)nBytes;
 shm->frameGPS[iwrite]  = 0;

 address = shm->frameSh[iwrite];

 memcpy((char *)address, buf, nBytes);

 return;
}


/*------------------------------------------- FdShmRead -------------*/

struct FrameH *FdShmRead (struct ShmInfo *shm, char *name, long compress, int *nBytes)
{
 struct FrameH *frame;
 long csmN, address, iread, iwrite;
 char *debut;

 if (shm == NULL) return(NULL);

 csmN = FdShmGetCsmN (shm,name);

 iwrite = shm->iwrite;

 /*--- don't try to read a frame currently written ---*/
 if (shm->iread[csmN] >= iwrite)  {*nBytes = 0; return(NULL);}

 /*--- if too late with respect to producer, go behind it (skipping nFrameMax frames) ---*/
 if (shm->iread[csmN] <= iwrite - shm->nFrameMax)
    {
     printf ("FdShmRead> late in reading shm (%d/%d). Skeeping %d frames, I go to iread=%d\n",
                    shm->iread[csmN], iwrite, iwrite-shm->iread[csmN], iwrite-1);
     shm->iread[csmN] = iwrite - 1;
    }

 /*--- ok, read the frame in share memory ---*/

 iread = shm->iread[csmN] % shm->nFrameMax;

 *nBytes = shm->frameSize[iread];
 address = shm->frameSh[iread] + (long) shm - (long) shm->segptr;


 if (*nBytes <= 0 || address == (long)NULL) return(NULL);

 frame = FrameReadFromBuf((char *)address, *nBytes, compress);

 if (frame == NULL) *nBytes = 0;

 shm->iread[csmN]++;

 return(frame);
}


/*------------------------------------------- FdShmReadBuf -------------*/

char *FdShmReadBuf (struct ShmInfo *shm, char *name, int *nBytes)
{
 char *buf;
 long csmN, address, iread, iwrite;

 csmN = FdShmGetCsmN (shm,name);

 iwrite = shm->iwrite;

 /*--- don't try to read a frame currently written ---*/
 if (shm->iread[csmN] >= iwrite)  {*nBytes = 0; return(NULL);}

 /*--- if too late with respect to producer, go behind it ---*/
 if (shm->iread[csmN] <= iwrite - shm->nFrameMax)
    {shm->iread[csmN] = iwrite - 1;}

 /*--- ok, read the frame in share memory ---*/

 shm->iread[csmN]++;
 iread  = shm->iread[csmN] % shm->nFrameMax;

 *nBytes = shm->frameSize[iread];
 address = shm->frameSh[iread] + (long) shm - (long) shm->segptr;

 if (*nBytes <= 0  ||  address == (long)NULL) return(NULL);

 buf = (char *)malloc(*nBytes);

 if (buf == NULL) return(NULL);

 memcpy(buf,(char *)address, *nBytes);

 return(buf);
}


/*--------------------------------------- FdShmLook -------------*/

void FdShmLook (struct ShmInfo *shm)
{
 long i, csm;
 char *status;

 status = FdShmGetStatus(shm);
 printf("%s",status);
 free(status);

 return;
}



/*--------------------------------------- FdShmGetStatus -------------*/

char *FdShmGetStatus (struct ShmInfo *shm)
{
 long i, csm;
 char *status, *p;

 status = (char *)calloc(1024+shm->nConsumer*256+shm->nFrameMax*256, sizeof(char));
 p = status;

 p += sprintf(p,"\n ======================= FDSHMLOOK =========================\n\n");

 p += sprintf(p, " key=%ld  shmid=%d  segptr=%ld  SegmentSize=%ld  version=%s \n",
              (long)shm->key, shm->shmid, (long)shm->segptr, shm->size, shm->version);
 p += sprintf(p, " statusSize=%ld  frameSizeMax=%ld  nFrameMax=%ld  nConsumer=%ld\n",
              shm->statusSize, shm->frameSizeMax, shm->nFrameMax, shm->nConsumer);
 p += sprintf(p, " producer=%s [pid=%d iwrite=%ld]  outputName=%s  Fbm_latency=%d\n",
              shm->csmName[0], shm->csmPid[0], shm->iwrite, shm->outputName, shm->latency);
 for (csm=1; csm<=shm->nConsumer; csm++)
    {p += sprintf(p, " consumer%ld=%s [pid=%d iread=%ld] \n",
                  csm, shm->csmName[csm], shm->csmPid[csm], shm->iread[csm]);}
 p += sprintf(p, "\n");

 for (i=0; i<shm->nFrameMax; i++)
    {
     p += sprintf(p, "   ShmAddr[%02ld]=%08lx  Size[%02ld]=%08ld  GPS[%02ld]=%d",
                    i, shm->frameSh[i], i, shm->frameSize[i], i, shm->frameGPS[i]);

     for (csm=1; csm<=shm->nConsumer; csm++)
        {if (i == (shm->iread[csm])%(shm->nFrameMax))
            {p += sprintf(p, "  %s(%ld)", shm->csmName[csm], shm->iread[csm]);}
        }

     if (i == (shm->iwrite)%(shm->nFrameMax))
        {p += sprintf(p, "  %s_producer(%ld)",shm->csmName[0], shm->iwrite);}
     p += sprintf(p, "\n");
    }
 p += sprintf(p,"\n =======================================================\n\n\0");

 return(status);
}
