/*----------------------------------------------------------------------------*/
/* A test program to test the FrFileInMemory object                           */
/*----------------------------------------------------------------------------*/

#include <FrameL.h>
#include <CmMessage.h>

#define CMTYPE_FDADDFRAME      "FdAddCmOutput"
#define CMTYPE_FDFRAME         "FdFrame"
#define CHNAME  "V1:NTP217_dac"

struct FrFile *memFile;
struct FrameH *inputFrame;
int count;

/*------------------------------------------------------ DyFdRequestFrames --------*/

int DyFdRequestFrames (char *source, char* destination, char *tag, int queueSize, int nRetry)
{
 int ok;
 CmMessage message;

 message= CmMessageNew();

 CmMessagePutText(message, destination);
 CmMessagePutText(message, tag);
 CmMessagePutInt (message, queueSize);
 CmMessagePutInt (message, nRetry);
 CmMessagePutInt (message, 0);  /* no checksum computation for the frames that will be sent */

 CmMessageSetType (message, CMTYPE_FDADDFRAME);

 ok = CmMessageSend (message, source);

 CmMessageDelete (message);

 return(ok);
}


/*----------------------------------------------------------------------*/

CmMessageStatus DyCmFrHandler (CmMessage message, char *sender, char *serverName)
{
 int run, fnum, gtimeS, gtimeN, trig, nBytes;
 char *buf, *name;
 double tSpan;

 if (message == NULL)
    {
     printf("DyCmFrHandler> Cm message received is null\n");
     return(CmMessageOk);
    }

 /*----- decode Cm message -----*/

 name   = CmMessageGetText  (message);
 run    = CmMessageGetInt   (message);
 fnum   = CmMessageGetInt   (message);
 gtimeS = CmMessageGetInt   (message);
 gtimeN = CmMessageGetInt   (message);
 trig   = CmMessageGetInt   (message);
 nBytes = CmMessageGetInt   (message);
 buf    = CmMessageGetArray (message, CmMessageChar, &nBytes);

 printf("DyCmFrHandler> Received from %s  frame %s gtimeS=%d \n", sender, name, gtimeS);

 /*--- Put the last frame received in inputFrame ---*/

 if (inputFrame) {FrameFree(inputFrame); inputFrame=NULL;}
 inputFrame = FrameReadFromBuf (buf, nBytes, -1);

 /*--- If frame is null, wait for an other one ---*/

 if (inputFrame == NULL)
   {
    printf("DyCmFrHandler> Could not read the frame received!!\n");
    return(CmMessageOk);
   }

 /*--- Put buf in list of frames in memory ---*/

 if(count > 7 && count < 13)
   {printf("     *** count=%d  frame %d not written in memory \n", count, inputFrame->GTimeS);
    return(CmMessageBreak);
   }

 printf("In DyCmFrameHandler> count=%d \n", count);

 tSpan = FrFileIInMemoryAddBuffer(memFile, buf, nBytes, 4);
 if(tSpan <= 0) printf("DyCmFrHandler> Could not get frame %d from %s", gtimeS, sender);
 else           printf("DyCmFrHandler> Put a new frame in buffer : tSpan=%g  memFile_tspan=4 \n", tSpan);

 return(CmMessageBreak);
}


/*------------------------------------------ DyCmWait ---*/

CmConnectCondition DyCmWait (int timeout)
{
 int iwait;
 CmConnectCondition condition;

 if (timeout == 0) timeout = 99999999;

 iwait = 1;
 while (iwait < timeout)
    {
     condition = CmMessageWaitWithTimeout(1.);
     if (condition == CmConnectBreakDetection) break;
     iwait++;
    }

 return(condition);
}


/*------------------------------------------------------------- main ---------*/
int main(int argc, char *argv[])
/*------------------------------------------------------------- main ---------*/
{
  FrameH* frame;
  int size;
  double tStart, tEnd, gtime, starttime;
  struct FrVect *vect;
  CmConnectCondition condition;
  char cmserver[256];

  strcpy(cmserver,"FdIODy");
  if (argc > 1) strcpy(cmserver,argv[1]);

  if (!CmMessageOpenServer("test_cmframe"))
     {printf("Could not declare myself with Cm Name test_cmframe \n");
      exit(0);
     }
  else
    {printf("Declared to Cm server as test_cmframe\n");}

  CmMessageInstallHandler (DyCmFrHandler, CMTYPE_FDFRAME);

  memFile =  FrFileIInMemoryNew();
  int bufSize = 200000000;
  inputFrame = NULL;

  DyFdRequestFrames(cmserver, "test_cmframe", CHNAME, 0, 2);
  count = 0;

  printf("Waiting for first frame to get the starttime\n");
  condition = DyCmWait(10);
  if (condition == CmConnectTimeoutDetection)
    {printf("No frame received from %s \n", cmserver); exit(0);}


  starttime = FrFileITStart(memFile);
  printf("------------------ Got starttime=%.1f \n", starttime);

  printf("Now starting the gtime loop \n");

  for (gtime=starttime; count<20; gtime++)
     {
      tStart = FrFileITStart(memFile);
      tEnd   = FrFileITEnd(memFile);
      if (gtime >= tEnd) DyCmWait(10);
      gtime = tEnd -1;

      if (count > 0)
        {
         FrVect *vect = FrFileIGetVect(memFile, CHNAME, gtime, 2);
/*
         if (vect) FrVectDump(vect, stdout, 2);
*/
         printf("  count=%d  tStart=%.0f tEnd=%.0f  gtime=%.1f\n", count, tStart, tEnd, gtime);
         if (vect) printf("Got vector %s at gtime=%.1f  nData=%d \n\n", vect->name, gtime, vect->nData);
         else printf("Vector at %f is NULL\n\n", gtime);
        }
      count++;
     }

  return(0);
}

