/**************************************************/
/*                                                */
/* File: DyPlots.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/* regroup the following previous files:          */
/* DyPlot.c  DyPlotCompute.c                      */
/**************************************************/

#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include "dataDisplay.h"
#include "URootAccess.h"

#ifdef MEMWATCH
#    include "memwatch.h"
#endif



/**************************************************/
/*                                                */
/* File: DyPlot.c                                 */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*                        ***********                     */
/*                        *  PLOTS  *                     */
/*                        ***********                     */


/*-------------------------------------------- DyPlotGetType ---*/

int DyPlotGetType (char *plotname)
{
 int i, i0, type0;

 if (plotname == NULL) return(PLOTTIME);

 i0 = -1;
 for (i=1; i<=Dy->nplot; i++)
    {if (strcmp(Plot[i]->name, plotname) == 0) {i0 = i; break;}}

 if (i0 >= 0) type0 = Plot[i0]->type;
 else         type0 = Plot[1]->type;

 return(type0);
}


/*-------------------------------------------- DyPlotSetMinMax ---*/

void DyPlotSetMinMax (struct DyPlot *plot, double localymin, double localymax)
{
 double deltaMinMax;

 if (Plot[plot->idsame]->ymin > localymin) Plot[plot->idsame]->ymin = localymin;
 if (Plot[plot->idsame]->ymax < localymax) Plot[plot->idsame]->ymax = localymax;

 if (plot->superposed == OFF) {Plot[plot->idsame]->ymin = localymin; Plot[plot->idsame]->ymax = localymax;}

 if (Plot[plot->idsame]->ymax != 0 || Plot[plot->idsame]->ymin != 0)
   {deltaMinMax = (Plot[plot->idsame]->ymax-Plot[plot->idsame]->ymin)/10.;
    RA_HSetMinimum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymin - deltaMinMax);
    RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymax + deltaMinMax);
    RA_HSetMinimum(plot->name, localymin - deltaMinMax);
    RA_HSetMaximum(plot->name, localymax + deltaMinMax);
   }
 else
   {RA_HSetMinimum(Plot[plot->idsame]->name, -1);
    RA_HSetMaximum(Plot[plot->idsame]->name,  1);
   }

 return;
}


/*-------------------------------------------- DySavePlot ---*/

int DySavePlot (char *savedPlots)
{
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 ok = RA_SavePlots(savedPlots);
// RA_FileOpenCreate(savedPlots);
// RA_FileWrite(savedPlots);
// RA_FileClose(savedPlots);

 sprintf(dummy,"Plots saved in file %s", savedPlots);
 if (ok) DyInfoLine (dummy);

 return(ok);
}


/*------------------------------------------------- DyRefPlotAdd ---*/

void DyRefPlotAdd (char *filename, int color)
{
 int i, i0;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check if reference file is already in the list ---*/

 i0 = -1;
 for (i=0; i<Dy->nreffile; i++)
    {if (strcmp(Dy->reffilename[i],filename) == 0) {i0 = i; break;}}

 /*--- If not, add it ---*/

 if (i0 == -1)
   {
    if (color <= 1 && Dy->nreffile == 0) color = 51;
    else if (color <= 2)                 color = 42;
    else if (color <= 3)                 color = 42 + 10*Dy->nreffile;
    else if (color < 10)                 color = color + 1;
    else if (Dy->nreffile > 0)           color = 10 * ((color-6)/10 + 1) + 6;
    strcpy(Dy->reffilename[Dy->nreffile],filename);
    Dy->refcolor[Dy->nreffile] = color;
    RA_RefPlotGet (filename, color);
    Dy->nreffile++;
   }
 else
   {printf("DyRefPlotAdd> %s already present in reference files list \n", filename);}

 /*--- In any case, set the flag refplot_indelebile ---*/

 Dy->refplot_indelebile = ON;

 return;
}


/*------------------------------------------------- DyRefPlotRemove ---*/

void DyRefPlotRemove (char *filename)
{
 int i, i0;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Find the index of the reffile to delete ---*/

 for (i=0; i<Dy->nreffile; i++)
    {if (strcmp(Dy->reffilename[i],filename) == 0) {i0 = i; break;}}

 /*--- Move up all the array elements after i0 ---*/

 for (i=i0+1; i<Dy->nreffile; i++)
    {strcpy(Dy->reffilename[i-1],Dy->reffilename[i]);
     Dy->refcolor[i-1] = Dy->refcolor[i];
    }

 Dy->nreffile--;

 printf("------- Dy->nreffile=%d \n", Dy->nreffile);
 return;
}


/*---------------------------------------------------  DyPlotEnd ---*/

void DyPlotEnd()
{
 int i,ndeleted, ntrfct, nrms;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Call the End procedure for each plot ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     plot = Plot[i];

     if (plot->hidden == ON) continue;

     if (plot->type == PLOTTIME)      DyPlotTIMEEnd(i);
     if (plot->type == PLOTFFT)       DyPlotFFTEnd(i);
     if (plot->type == PLOT1D)        DyPlot1DEnd(i);
     if (plot->type == PLOT2D)        DyPlot2DEnd(i);
     if (plot->type == PLOTTR)        DyPlotTRFCTEnd(i);
     if (plot->type == PLOTCOHE)      DyPlotCOHEEnd(i);
     if (plot->type == PLOTRAW)       DyPlotRAWEnd(i);
     if (plot->type == PLOTFFTTIME)   DyPlotFFTTIMEEnd(i);
     if (plot->type == PLOT1DTIME)    DyPlot1DTIMEEnd(i);
     if (plot->type == PLOTRAWTIME)   DyPlotRAWTIMEEnd(i);
     if (plot->type == PLOTTRTIME)    DyPlotTRFCTTIMEEnd(i);
     if (plot->type == PLOTCOHETIME)  DyPlotCOHETIMEEnd(i);
     if (plot->type == PLOTAUDIO)     DyPlotAUDIOEnd(i);
     RA_HUpdateEnd(plot->name, plot->options);
    }

 /*--- Delete the unused plots ---*/

/* ICI */
/**
 ntrfct = 0;
 for (i=1; i<=Dy->nplot; i++) if (Plot[i]->type == PLOTTR) ntrfct++;
 nrms = 0;
 for (i=1; i<=Dy->nplot; i++) if (Plot[i]->rmsfft == ON) nrms++;

 ndeleted = RA_DeleteUnusedPlot(Dy->nplot+ntrfct+nrms);
**/

 return;
}


/*------------------------------------------- DyPlotSetSecondParam ---*/
/*--- Set the secondary parameters of the plot ---*/
/*--- (those not visible by user interface) ---*/

void DyPlotSetSecondParam(int iplot)
{
 struct DyPlot *plot;
 int ii,dNbin;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 if (plot->type == PLOTFFT) 
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->deltat    = plot->fft_duration * plot->tstep/100.;
      }
    else
      {
       plot->deltat = 1;
      }
    plot->xnBin     = plot->fft_duration * plot->chx->resampFreq;
    plot->dfreq     = plot->chx->resampFreq / plot->xnBin;
    plot->dNbin     = plot->xnBin * plot->tstep/100.;
    plot->refreshDt = plot->refresh_fft * plot->deltat;
   }

 else if (plot->type == PLOTTIME) 
   {
    plot->xnBin = plot->tsize * plot->chx->resampFreq;
    if (plot->chx->vect2d_duration == 0)
      {
       dNbin = plot->xnBin * plot->tstep/100.;
       for (ii=dNbin; ii<plot->xnBin; ii++)
          {
           if (plot->xnBin%ii == 0) break;
          }
       dNbin = ii;
       plot->dNbin = dNbin;
       plot->tstep = ((double)plot->dNbin / plot->chx->resampFreq) * 100. / plot->tsize;
       plot->deltat = plot->tsize * plot->tstep/100.;
      }
    else
      {
       plot->deltat = 1;
       plot->dNbin = plot->xnBin;
       plot->tstep = 100;
      }
    plot->refreshDt = plot->deltat;
   }

 else if (plot->type == PLOTTR)  
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->deltat    = plot->fft_duration * plot->tstep/100.;
      }
    plot->xnBin     = plot->fft_duration * plot->chx->resampFreq;
    plot->dfreq     = plot->chx->resampFreq / plot->xnBin;
    plot->dNbin     = plot->xnBin * plot->tstep/100.;
    plot->refreshDt = plot->refresh_fft * plot->deltat;
   }

 else if (plot->type == PLOTCOHE)
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->deltat    = plot->fft_duration * plot->tstep/100.;
      }
    plot->xnBin     = plot->fft_duration * plot->chx->resampFreq;
    plot->dfreq     = plot->chx->resampFreq / plot->xnBin;
    plot->dNbin     = plot->xnBin * plot->tstep/100.;
    plot->refreshDt = plot->refresh_fft * plot->deltat;
   }

 else if (plot->type == PLOT1D) 
   {
    plot->deltat = plot->refreshDt;
   }

 else if (plot->type == PLOT2D)
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->deltat = plot->refreshDt;
      }
   }

 else if (plot->type == PLOTAUDIO) 
   {
    plot->deltat = plot->audio_duration / 10;
   }

 else if (plot->type == PLOTFFTTIME) 
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->fft_duration = 1. / plot->dfreq;
       plot->fft_deltat   = plot->fft_duration * plot->tstep_fft/100;
       if (plot->dtime < plot->fft_deltat)
         {plot->fft_deltat = plot->dtime;
          plot->tstep_fft = 100 * (plot->fft_deltat / plot->fft_duration);
         }
      }
    plot->maxAverage   = (int)(plot->dtime / plot->fft_deltat + 0.5);
    plot->refreshDt    = plot->tsize * plot->tstep/100.;
    plot->ynBin        = plot->chx->resampFreq / plot->dfreq;
    plot->xnBin        = plot->tsize / plot->dtime;
    plot->deltat       = (int)(plot->refreshDt/plot->fft_deltat)*plot->fft_deltat;
   }

 else if (plot->type == PLOTRAW)  
   {
    plot->deltat    = plot->refreshDt;
   }

 else if (plot->type == PLOT1DTIME) 
   {
    plot->deltat    = plot->dtime;
    plot->xnBin     = plot->tsize / plot->dtime;
    plot->refreshDt = plot->tsize * plot->tstep/100.;
   }

 else if (plot->type == PLOTRAWTIME)
   {
    plot->deltat    = plot->refreshDt;
    plot->xnBin     = plot->tsize / plot->refreshDt;
   }

 else if (plot->type == PLOTTRTIME) 
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->fft_duration = 1. / plot->dfreq;
       plot->fft_deltat   = plot->fft_duration * plot->tstep_fft/100;
       if (plot->dtime < plot->fft_deltat)
         {plot->fft_deltat = plot->dtime;
          plot->tstep_fft = 100 * (plot->fft_deltat / plot->fft_duration);
         }
      }
    plot->maxAverage   = (int)(plot->dtime / plot->fft_deltat + 0.5);
    plot->refreshDt    = plot->tsize * plot->tstep/100.;
    plot->ynBin        = plot->chx->resampFreq / plot->dfreq;
    plot->ynBin        = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / plot->dfreq;
    plot->xnBin        = plot->tsize / plot->dtime;
    plot->deltat       = (int)(plot->refreshDt/plot->fft_deltat)*plot->fft_deltat;
   }

 else if (plot->type == PLOTCOHETIME)
   {
    if (plot->chx->vect2d_duration == 0)
      {
       plot->fft_duration = 1. / plot->dfreq;
       plot->fft_deltat   = plot->fft_duration * plot->tstep_fft/100;
       if (plot->dtime < plot->fft_deltat)
         {plot->fft_deltat = plot->dtime;
          plot->tstep_fft = 100 * (plot->fft_deltat / plot->fft_duration);
         }
      }
    plot->maxAverage   = (int)(plot->dtime / plot->fft_deltat + 0.5);
    plot->refreshDt    = plot->tsize * plot->tstep/100.;
    plot->ynBin        = plot->chx->resampFreq / plot->dfreq;
    plot->ynBin        = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / plot->dfreq;
    plot->xnBin        = plot->tsize / plot->dtime;
    plot->deltat       = (int)(plot->refreshDt/plot->fft_deltat)*plot->fft_deltat;
   }

 return;
}


/*------------------------------------------- DyPlotSetTitle ---*/

void DyPlotSetTitle(int iplot)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 if (plot->chy != NULL)
   {if (plot->type == PLOTTR || plot->type == PLOTTRTIME)
       sprintf(plot->title, "%s%s%s%s%s", plot->chx->name, SEP2VARTR, plot->chy->name, SEPTITLE, Dy->typeName[plot->type]);
    else if (plot->type == PLOTCOHE || plot->type == PLOTCOHETIME)
       sprintf(plot->title, "%s%s%s%s%s", plot->chx->name, SEP2VAR, plot->chy->name, SEPTITLE, Dy->typeName[plot->type]);
    else if (plot->type == PLOT2D)
       sprintf(plot->title, "%s%s%s%s%s", plot->chy->name, SEP2VAR, plot->chx->name, SEPTITLE, Dy->typeName[plot->type]);
   }
 else
    sprintf(plot->title, "%s%s%s", plot->chx->name, SEPTITLE, Dy->typeName[plot->type]);

 return;
}


/*------------------------------------ DyPlotSetBrowserline ---*/

void DyPlotSetBrowserline(int i, char *browserline)
{
 /*--- If the plot i is a superposed one ---*/

 if (Plot[i]->superposed == ON)
   {
    if (Plot[i]->hidden == ON)
      sprintf (browserline, "@C14 %d  %s  (s)", i, Plot[i]->title);
    else
      sprintf (browserline, "%d  %s  (s)", i, Plot[i]->title);
   }

 /*--- If the plot i is not a superposed one ---*/

 else
    {
    if (Plot[i]->hidden == ON)
      sprintf (browserline, "@C14 %d  %s", i, Plot[i]->title);
    else
      sprintf (browserline, "%d  %s", i, Plot[i]->title);
   }

 return;
}


/*------------------------------------ DyPlotAutoAdjust1D ---*/

void DyPlotAutoAdjust1D(int iplot, double localxmin, double localxmax)
{
 double *buff;
 int i, last;
 double delta;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 delta = localxmax - localxmin;
 localxmin = localxmin + delta/10.;
 localxmax = localxmax - delta/10.;

 /*--- Auto-adjust max on X axis ---*/

 while (plot->xmax < localxmax)
    {
     buff = (double *) calloc(plot->xnBin,sizeof(double));
     RA_H1Unpack(plot->name, buff);
     plot->xmax = 2 * plot->xmax - plot->xmin;
     RA_SetLimitsX(plot->name, plot->xmin, plot->xmax);
     for (i=0; i<plot->xnBin/2; i++)           buff[i] = buff[2*i]+buff[2*i+1];
     for (i=plot->xnBin/2; i<plot->xnBin; i++) buff[i] = 0.;
     RA_H1Pack(plot->name, buff);
     free(buff);
     DyXformCheckGUI();
     if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
    }

 /*--- Auto-adjust min on X axis ---*/

 while (plot->xmin > localxmin)
    {
     buff = (double *) calloc(plot->xnBin,sizeof(double));
     RA_H1Unpack(plot->name, buff);
     plot->xmin = 2 * plot->xmin - plot->xmax;
     RA_SetLimitsX(plot->name, plot->xmin, plot->xmax);
     last = plot->xnBin-1;
     for (i=0; i<plot->xnBin/2; i++) buff[last-i] = buff[last-2*i]+buff[last-2*i-1];
     for (i=0; i<plot->xnBin/2; i++) buff[i] = 0.;
     RA_H1Pack(plot->name, buff);
     free(buff);
     DyXformCheckGUI();
     if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
    }

 return;
}

/*------------------------------------ DyPlotAutoAdjust2D_X ---*/

void DyPlotAutoAdjust2D_X(int iplot, double localmin, double localmax)
{
 double *buff;
 int i, j, last, nx;
 double min0, max0, delta;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 /*--- Understimate min and max so that autoadjust is used when only really needed ---*/
 delta = localmax - localmin;
 localmin = localmin + delta/10.;
 localmax = localmax - delta/10.;

 /*--- Auto-adjust max only ---*/

 if (plot->xmax < localmax && plot->xmin < localmin)
   {
    while (plot->xmax < localmax)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         plot->xmax = plot->xmax + (plot->xmax - plot->xmin);
         RA_SetLimitsX(plot->name, plot->xmin, plot->xmax);

         nx = plot->xnBin;
         for (j=0; j<plot->ynBin; j++)
             {for (i=0; i<plot->xnBin/2; i++)
                  {buff[i+j*nx] = buff[2*i+j*nx]+buff[(2*i+1)+j*nx];}
              for (i=plot->xnBin/2; i<plot->xnBin; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
         DyXformCheckGUI();
         if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
        }
    }

 /*--- Auto-adjust min only ---*/

 else if (plot->xmax > localmax && plot->xmin > localmin)
   {
    while (plot->xmin > localmin)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         plot->xmin = plot->xmin - (plot->xmax - plot->xmin);
         RA_SetLimitsX(plot->name, plot->xmin, plot->xmax);

         last = plot->xnBin-1;
         nx   = plot->xnBin;
         for (j=0; j<plot->ynBin; j++)
             {for (i=0; i<plot->xnBin/2; i++)
                  {buff[(last-i)+j*nx] = buff[(last-2*i)+j*nx]+buff[(last-2*i-1)+j*nx];}
              for (i=0; i<plot->xnBin/2; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
        }
    }

 /*--- Auto-adjust min and max ---*/

 else if (plot->xmax < localmax && plot->xmin > localmin)
   {
    while (plot->xmax < localmax && plot->xmin > localmin)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         min0 = plot->xmin;
         max0 = plot->xmax;
         plot->xmin = plot->xmin - (max0-min0)/2;
         plot->xmax = plot->xmax + (max0-min0)/2;

         RA_SetLimitsX(plot->name, plot->xmin, plot->xmax);

         last = plot->xnBin-1;
         nx   = plot->xnBin;
         for (j=0; j<plot->ynBin; j++)
             {for (i=0; i<plot->xnBin/2; i++)
                  {buff[(last-i)+j*nx] = buff[(last-2*i)+j*nx]+buff[(last-2*i-1)+j*nx];}
              for (i=0; i<plot->xnBin/2; i++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
        }
    }

 return;
}

/*------------------------------------ DyPlotAutoAdjust2D_Y ---*/

void DyPlotAutoAdjust2D_Y(int iplot, double localmin, double localmax)
{
 double *buff;
 int i, j, last, nx;
 double min0, max0, delta;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 /*--- underestimate local min and max, so that autoadjust is done only when really needed ---*/
 delta = localmax - localmin;
 localmin = localmin + delta/10.;
 localmax = localmax - delta/10.;

 /*--- Auto-adjust max only ---*/

 if (plot->ymax < localmax && plot->ymin < localmin)
   {
    while (plot->ymax < localmax)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         plot->ymax = plot->ymax + (plot->ymax - plot->ymin);
         RA_SetLimitsY(plot->name, plot->ymin, plot->ymax);

         nx   = plot->xnBin;
         for (i=0; i<plot->xnBin; i++)
             {for (j=0; j<plot->ynBin/2; j++)
                  {buff[i+j*nx] = buff[i+2*j*nx]+buff[i+(2*j+1)*nx];}
              for (j=plot->ynBin/2; j<plot->ynBin; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
         DyXformCheckGUI();
         if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
        }
    }

 /*--- Auto-adjust min only ---*/

 else if (plot->ymax > localmax && plot->ymin > localmin)
   {
    while (plot->ymin > localmin)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         plot->ymin = plot->ymin - (plot->ymax - plot->ymin);
         RA_SetLimitsY(plot->name, plot->ymin, plot->ymax);

         last = plot->ynBin-1;
         nx   = plot->xnBin;
         for (i=0; i<plot->xnBin; i++)
             {for (j=0; j<plot->ynBin/2; j++)
                  {buff[i+(last-j)*nx] = buff[i+(last-2*j)*nx]+buff[i+(last-2*j-1)*nx];}
              for (j=0; j<plot->ynBin/2; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
         DyXformCheckGUI();
         if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
        }
    }

 /*--- Auto-adjust min and max ---*/

 else if (plot->ymax < localmax && plot->ymin > localmin)
   {
    while (plot->ymax < localmax && plot->ymin > localmin)
        {
         buff = (double *) calloc(plot->xnBin*plot->ynBin,sizeof(double));
         RA_H2Unpack(plot->name, buff);

         min0 = plot->ymin;
         max0 = plot->ymax;
         plot->ymin = plot->ymin - (max0-min0)/2;
         plot->ymax = plot->ymax + (max0-min0)/2;

         RA_SetLimitsY(plot->name, plot->ymin, plot->ymax);

         last = plot->ynBin-1;
         nx   = plot->xnBin;
         for (i=0; i<plot->xnBin; i++)
             {for (j=0; j<plot->ynBin/2; j++)
                  {buff[i+(last-j)*nx] = buff[i+(last-2*j)*nx]+buff[i+(last-2*j-1)*nx];}
              for (j=0; j<plot->ynBin/2; j++)
                  {buff[i+j*nx] = 0.;}
             }
         RA_H2Pack(plot->name, buff);
         free(buff);
         DyXformCheckGUI();
         if (Dy->Order == STOP) {Dy->Order = CONTINUE; break;}
        }
    }

 return;
}


/*------------------------------------ DyPlotGetId ---*/

int DyPlotGetId(char *plotname)
{
 int i, iplot;

 DyDebugPrint(__FUNCTION__,0);

 iplot = -1;
 for (i=0; i<=Dy->nplot; i++)
    {if (strcmp(Plot[i]->name,plotname) == 0) {iplot = i; break;}}

 if (iplot == -1) printf("DyPlotGetId> Plot %s was not found in the list\n", plotname);

 return(iplot);
}


/*------------------------------------ DyPlotDraw2 ---*/
/* ICI : obsolete ? */

void DyPlotDraw2(int iplot)
{
 struct DyPlot *plot;
 int j;

 if (Dy->plotsNotDisplayed == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 plot->ready = ON;

 /*--- Go into drawing process only for single plot or first plot of superposed set ---*/
 
 if (plot->superposed == ON) return;

 /*--- Check that all plots in the pad pointed by plot->numpad are ready to be drawn/updated ---*/
 /*--- and put all ready flag to OFF ---*/

 if (DyPlotReady(iplot) == OFF) return;

 /*--- Go into the pad and draw the plot ---*/

 RA_SetActivePad(plot->numpad, plot->numsubpad);
 RA_HDraw(plot->name, plot->options, plot->superposed);
 DyPlotSetDrawInfo(iplot);

 /*--- Draw also all the plots superposed to this one ---*/

 for (j=iplot+1; j<=Dy->nplot; j++)
    {if (Plot[j]->superposed == OFF) break;
     RA_HDraw(Plot[j]->name, Plot[j]->options, Plot[j]->superposed);
     DyPlotSetDrawInfo(j);
    }

 /*--- Update the pad, so that drawn plots are shown ---*/

 RA_PadUpdate(plot->numpad);

 return;
}


/*------------------------------------ DyPlotDraw ---*/

void DyPlotDraw(int iplot)
{
 struct DyPlot *plot;
 int all_ready, nsuperposed;

 if (Dy->plotsNotDisplayed == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 plot->ready = ON;

 /*--- Go into the pad and draw the plot ---*/

 if (plot->type == PLOTTR)
   {
    RA_SetActivePad(plot->numpad, 1);
    RA_HDraw(plot->name, plot->options, plot->superposed);
    DyPlotSetDrawInfo(iplot);
    RA_SetActivePad(plot->numpad, 2);
    RA_HDraw(plot->namePhase, plot->options, plot->superposed);
   }
 else
   {
    RA_SetActivePad(plot->numpad, 0);
    RA_HDraw(plot->name, plot->options, plot->superposed);
    if (plot->rmsfft == ON) RA_HDraw(plot->name_rms, "same", ON);
    DyPlotSetDrawInfo(iplot);
   }

 /*--- Check if all the superposed plots around iplot are ready to be shown ---*/

 nsuperposed = DyPlotGetNsuperposed(iplot) + 1;
 all_ready = DyPlotReady(iplot);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("  DyPlotDraw> Draw made for plot %s  gtime=%f  deltat=%g  PlotsReady=%d \n",
            plot->name, plot->gtime, plot->deltat, all_ready);
   }

 /*--- Update the pad, so that drawn plots are shown ---*/

 if ((all_ready == ON && nsuperposed>1) || (iplot == Dy->iplotref && nsuperposed==1))
   {if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
      {printf("DyPlotDraw> Doing update of numpad=%d  iplot=%d iplotref=%d all_ready=%d nsuperposed=%d \n",
                           plot->numpad, iplot, Dy->iplotref, all_ready, nsuperposed);
      }
    RA_PadUpdate(plot->numpad);
    Dy->UpdateDone = ON;
   }

 return;
}


/*------------------------------------ DyPlotGetUpdater ---*/
int DyPlotGetUpdater()
{
 int i, iplotref;

 iplotref = Dy->nplot;

 for (i=1; i<=Dy->nplot; i++)
    {if (Plot[i]->hidden == ON) continue;
     if (Plot[i]->superposed == ON) continue;
     if (Plot[i]->refreshDt == Dy->min_refreshDt)
       {iplotref = i;
        if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
          {printf("DyPlotGetUpdater> Found iplotref=%d min_refreshDt=%g\n", iplotref, Dy->min_refreshDt);}
       }
    }

 return(iplotref);
}


/*------------------------------------ DyPlotReady ---*/

int DyPlotReady(int iplot)
{
 int j, nready, nsuperposed;

 DyDebugPrint(__FUNCTION__,0);

 /*--- When at least one plot is not ready for update, put UpdateDone flag to OFF ---*/

 if (Plot[iplot]->ready == OFF) Dy->UpdateDone = OFF;

 /*--- Get the number of superposed plots, including the one not flagged by superposed=ON ---*/

 nsuperposed = DyPlotGetNsuperposed(iplot) + 1;

 /*--- Init the nready counter ---*/

 nready = 0;
 if (Plot[iplot]->ready == ON) nready++;

 /*--- Case of a single plot ready to be displayed ---*/

 if (nready == nsuperposed)
   {
    Dy->UpdateDone = ON; 
    Plot[iplot]->ready = OFF;
    return(ON);
   }

 /*--- Otherwise, are all the superposed plots ready to be displayed ? ---*/

 for (j=iplot+1; j<=Dy->nplot; j++)
    {if (Plot[j]->superposed == OFF) break;
     if (Plot[j]->ready == ON) nready++;
    }

 /*--- And if the plot is a superposed one, check also the previous plots in the list ---*/

 if (Plot[iplot]->superposed == ON)
   {for (j=iplot-1; j>0; j--)
      {if (Plot[j]->ready == ON) nready++;
       if (Plot[j]->superposed == OFF) break;
      }
   }

 /*--- Update the ready flags when all the superposed plots are ready to be displayed ---*/

 if (nready == nsuperposed)
   {
    Plot[iplot]->ready = OFF;
    for (j=iplot+1; j<=Dy->nplot; j++)
       {if (Plot[j]->superposed == OFF) break;
        Plot[j]->ready = OFF;
       }
    if (Plot[iplot]->superposed == ON)
      { 
       for (j=iplot-1; j>0; j--)
          {if (Plot[j]->superposed == OFF) break;
           Plot[j]->ready = OFF;
          }
      }
   }

 /*--- Return the readiness flag ---*/

 if (nready == nsuperposed)
   {
    Dy->UpdateDone = ON; 
    Plot[iplot]->ready = OFF;
    return(ON);
   }
 else
   {return(OFF);}
}


/*------------------------------------ DyPlotGetIsuperposed ---*/

int DyPlotGetIsuperposed(int iplot)
{
 int j, iref;

 DyDebugPrint(__FUNCTION__,0);

 if (Plot[iplot]->superposed == OFF) return(iplot);

 for (j=iplot; j>0; j--)
    {if (Plot[j]->superposed == OFF) {iref = j; break;}}

 return(iref);
}

/*------------------------------------ DyPlotGetSameIndex ---*/

int DyPlotGetSameIndex(int iplot)
{
 int j, nsame;

 DyDebugPrint(__FUNCTION__,0);

 if (Plot[iplot]->superposed == OFF) return(0);

 nsame = 0;
 for (j=iplot; j>0; j--)
    {if (Plot[j]->superposed == OFF) break;
     nsame++;
    }

 return(nsame);
}

/*------------------------------------ DyPlotNsuperposed ---*/

int DyPlotGetNsuperposed(int iplot)
{
 int j, nsuperposed;

 DyDebugPrint(__FUNCTION__,0);

 if (Plot[iplot]->superposed == ON) nsuperposed = 1;
 else                               nsuperposed = 0;

 for (j=iplot+1; j<=Dy->nplot; j++)
    {if (Plot[j]->superposed == OFF) break; nsuperposed++;}

 if (Plot[iplot]->superposed == ON)
    {for (j=iplot-1; j>0; j--)
        {if (Plot[j]->superposed == OFF) break; nsuperposed++;}
    }

 return(nsuperposed);
}

 
/*------------------------------------------- DyPlotSetMinRefreshDt ---*/

void DyPlotSetMinRefreshDt()
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 Dy->min_refreshDt = 9999999;

 for (i=1; i<= Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     if (Plot[i]->superposed == ON) continue;
     if (Plot[i]->refreshDt < Dy->min_refreshDt && Plot[i]->refreshDt > 0) Dy->min_refreshDt = Plot[i]->refreshDt;
    }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("DyPlotSetMinRefreshDt> Found min_refreshDt=%g \n", Dy->min_refreshDt);}

 return;
}


/*------------------------------------------- DyPlotSetMindt ---*/

 void DyPlotSetMindt()
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 Dy->mindt = 9999999;

 for (i=1; i<= Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     if (Plot[i]->superposed == ON) continue;
     if (Plot[i]->type == PLOTRAW) Plot[i]->deltat = Plot[i]->refreshDt;
     if (Plot[i]->deltat < Dy->mindt && Plot[i]->deltat > 0)             Dy->mindt = Plot[i]->deltat;
     if (Plot[i]->chx->vect2d_duration > 0) continue;
     if (Plot[i]->fft_duration < Dy->mindt && Plot[i]->fft_duration > 0) Dy->mindt = Plot[i]->fft_duration;
    }

/*
 if (Dy->InputType == INCM && Dy->mindt > 1.) Dy->mindt = 1.;
*/

 return;
}


/*------------------------------------------- DyPlotSetMaxdt ---*/

 void DyPlotSetMaxdt()
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 Dy->maxdt = 0;

 for (i=1; i<= Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
     if (Plot[i]->deltat > Dy->maxdt && Plot[i]->deltat > 0)             Dy->maxdt = Plot[i]->deltat;
     if (Plot[i]->chx->vect2d_duration > 0) continue;
     if (Plot[i]->fft_duration > Dy->maxdt && Plot[i]->fft_duration > 0) Dy->maxdt = Plot[i]->fft_duration;
    }

 return;
}


/*------------------------------------------- DySetOnePlotVarY ---*/
                                                 /*  called from DyXformSetOnePlotVarY  */
void DySetOnePlotVarY ()
{
 long id;
 char dummy[256];

 if (Dy->y2d == OFF) return;

 DyDebugPrint(__FUNCTION__,0);

 id = Dy->nplot;
 id++;

 if (Plot[id]->chx == NULL)
   {printf("DySetOnePlotVarY> The plot has not yet a variable x\n"); return;}

 /*--- Second channel of a 2D plot has been found ---*/

 if (Dy->selectedCh) Plot[id]->chy = DyChClone(Dy->selectedCh);
 else                {printf("DySetOnePlotVarY> not able to get variable y\n"); return;}

 /*--- Set the title of this newly created plot ---*/

 DyPlotSetTitle(id);

 /*--- Set the plot name, using an absolute counter ---*/

 Dy->nplotcreated++;
 sprintf(Plot[id]->name, "%s%d", DYPLOT_PREFIX, Dy->nplotcreated);

 /*--- Put default parameter in plot structure ---*/

 DyPutDefaultInPlot(id);

 /*--- Copy parameters from a plot which already exists in the list ---*/

 DyGetDefaultFromPlot(id);

 /*--- Update plots list in the GUI browser ---*/

 DyXformUpdatePlotBrowser(id, ADD_PLOT);

 /*--- Deselect all the selected lines in the channels browser ---*/

 DyXformAutoDeselect(mainbrowser->channels, 0);

 /*--- Put again the 2d flag to off ---*/

 Dy->y2d = OFF;

 /*--- Update the value of nplot ---*/

 Dy->nplot = id;

 sprintf(dummy, "Made a new %s plot", Dy->typeName[Plot[id]->type]);
 DyInfoLine (dummy);

 return;
}

/*------------------------------------------- DySetOnePlot ---*/
                                                        /*  called from DyXformSetPlot  */
void DySetOnePlot (int plottype)
{
 long id;
 char dummy[256];

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Frame == NULL) {printf("DyXformSetOnePlot> no frame in memory \n"); return;}

 id = Dy->nplot;
 id++;

 Plot[id]->type = plottype;

 /*--- If first pass for a 2D plot or if not a 2D plot ---*/

 if (Dy->y2d == OFF)
    {
     if (Dy->selectedCh) Plot[id]->chx = DyChClone(Dy->selectedCh);
     else                {printf("DyXformSetOnePlot> not been able to get variable x\n"); return;}
     DyPlotSetTitle(id);
    }

 /*--- If second pass for a 2D plot ---*/

 if (Dy->y2d == ON)
    {
     if (Dy->selectedCh) Plot[id]->chy = DyChClone(Dy->selectedCh);
     else                {printf("DyXformSetOnePlot> not been able to get variable y\n"); return;}
     DyPlotSetTitle(id);
    }


 /*--- Second variable of 2D or TrFct plot has been found ---*/

 /*--- y2d=OFF and the plot needs a second variable, then y2d=ON                   ---*/
 /*--- y2d=ON and the plot is ok, then y2d=ON2                                     ---*/
 /*--- y2d=ON2 means that plot has all the variables it needs, then y2d=OFF        ---*/

 if (Dy->y2d == ON)
    {if (Plot[id]->type == PLOT2D)       DyInfoLine ("Made a new 2D plot");
     if (Plot[id]->type == PLOTTR)       DyInfoLine ("Made a new Transfer Function plot");
     if (Plot[id]->type == PLOTCOHE)     DyInfoLine ("Made a new Coherence plot");
     if (Plot[id]->type == PLOTTRTIME)   DyInfoLine ("Made a new Transfer Function vs Time plot");
     if (Plot[id]->type == PLOTCOHETIME) DyInfoLine ("Made a new Coherence vs Time plot");
     Dy->y2d = ON2;
    }

 /*--- Need a second variable if it is a 2D or TrFct plot ---*/

 if (plottype == PLOT2D        && Dy->y2d == OFF)  Dy->y2d = ON;
 if (plottype == PLOTTR        && Dy->y2d == OFF)  Dy->y2d = ON;
 if (plottype == PLOTCOHE      && Dy->y2d == OFF)  Dy->y2d = ON;
 if (plottype == PLOTTRTIME    && Dy->y2d == OFF)  Dy->y2d = ON;
 if (plottype == PLOTCOHETIME  && Dy->y2d == OFF)  Dy->y2d = ON;

 /*--- Second channel of a 2D plot has been found, we put the 2d flag to off ---*/

 if (Dy->y2d == ON2) Dy->y2d = OFF;


 /*--- Channel of the plot has been found or all channels of the 2d plot have been found ---*/

 if (Dy->y2d == OFF)
    {
     /*--- Set the plot name, using an absolute counter ---*/

     Dy->nplotcreated++;
     sprintf(Plot[id]->name, "%s%d", DYPLOT_PREFIX, Dy->nplotcreated);

     /*--- Put default value for parameters of the plot ---*/

     DyPutDefaultInPlot(id);

     /*--- If a plot of same type already exists, copy its parameters instead of using default values ---*/

     DyGetDefaultFromPlot(id);

     /*--- Update the plots list in the GUI browser ---*/

     DyXformUpdatePlotBrowser(id, ADD_PLOT);

     /*--- Update the value of nplot ---*/

     Dy->nplot = id;

     sprintf(dummy, "Made a new %s plot: %s", Dy->typeName[Plot[id]->type], Plot[id]->title);
     DyInfoLine (dummy);
    }

 return;
}

/*-------------------------------------------- DyRemovePlot ---*/

void DyRemovePlot()
{
 int i0, i, j;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->nplot <= 0) return;

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->toberemoved == ON)
        {i0 = Dy->nplot + 1;
         for (j=i; j<=Dy->nplot; j++) if (Plot[j]->toberemoved == OFF) {i0 = j; break;}
         for (j=i; j<i0; j++)
            {
             RA_HDelete(Plot[j]->name);
             DyPlotFree(j, ON);
            }

         for (j=i0; j<=Dy->nplot; j++) DyCopyPlot(j-(i0-i),j,ON);

         Dy->nplot = Dy->nplot - (i0-i);
         if (i > 1  && Plot[i]->superposed == ON && Plot[i-1]->type != Plot[i]->type) Plot[i]->superposed = OFF; 
         if (i == 1 && Plot[i]->superposed == ON)                                     Plot[i]->superposed = OFF; 
        }
    }

 DyXformPlotFillBrowser();

 return;
}


/*------------------------------------------ DyPlotFree ---*/

void DyPlotFree (long id, int fullfree)
{
 struct DyPlot *plot;
 char dummy[LSTRING_MAX];

 plot = Plot[id];

 if (plot == NULL) return;

 sprintf(dummy,"DyPlotFree> plot%ld  %s", id, plot->name);
 DyDebugPrint(dummy,0);

 if (fullfree == ON)
   {if (plot->chx) {DyChFree(plot->chx); plot->chx = NULL;}
    if (plot->chy) {DyChFree(plot->chy); plot->chy = NULL;}
   }

 if (plot->frvbufx)  {FrvBufFree(plot->frvbufx); plot->frvbufx = NULL;}
 if (plot->frvbufy)  {FrvBufFree(plot->frvbufy); plot->frvbufy = NULL;}
 if (plot->buff)     {free(plot->buff);           plot->buff = NULL;}
 if (plot->buffer)   {FrVectFree(plot->buffer);   plot->buffer = NULL;}
 if (plot->fftx)     {FrvRFFTFree(plot->fftx);    plot->fftx = NULL;}
 if (plot->tf)       {FrvTFFree(plot->tf);        plot->tf = NULL;}
 if (plot->vect_rms) {FrVectFree(plot->vect_rms); plot->vect_rms = NULL;}
 if (plot->name_rms) free(plot->name_rms);
 if (plot->title_rms) free(plot->title_rms);

 return;
}



/*------------------------------------------ DyCopyPlot ---*/

void DyCopyPlot (long idout, long idin, int fullcopy)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyCopyPlot> in=%ld=%s  out=%ld=%s  fullcopy=%d", idin, Plot[idin]->name, idout, Plot[idout]->name, fullcopy);
 DyDebugPrint(dummy,0);

/*
ICI Check how is done the Copy of a plot
*/

 if (fullcopy == ON)
   {
    if (Plot[idin]->chx)
       {if (Plot[idout]->chx == NULL) Plot[idout]->chx = (struct DyCh *)calloc(1,sizeof(struct DyCh));
        DyChCopy(Plot[idout]->chx, Plot[idin]->chx);
       }

    if (Plot[idin]->chy)
       {if (Plot[idout]->chy == NULL) Plot[idout]->chy = (struct DyCh *)calloc(1,sizeof(struct DyCh));
        DyChCopy(Plot[idout]->chy, Plot[idin]->chy);
       }

    strcpy (Plot[idout]->title,  Plot[idin]->title);
    if (idout > Dy->nplot)
       {Dy->nplotcreated++;
        sprintf (Plot[idout]->name, "%s%d", DYPLOT_PREFIX, Dy->nplotcreated);
       }

    Plot[idout]->type   = Plot[idin]->type;
    Plot[idout]->hidden = Plot[idin]->hidden;
   }

 Plot[idout]->yoffset       = Plot[idin]->yoffset;
 Plot[idout]->yscale        = Plot[idin]->yscale;
 Plot[idout]->xoffset       = Plot[idin]->xoffset;
 Plot[idout]->xscale        = Plot[idin]->xscale;

 Plot[idout]->fft_duration  = Plot[idin]->fft_duration;
 Plot[idout]->tsize         = Plot[idin]->tsize;
 Plot[idout]->tstep         = Plot[idin]->tstep;
 Plot[idout]->tstep_fft     = Plot[idin]->tstep_fft;
 Plot[idout]->dfreq         = Plot[idin]->dfreq;
 Plot[idout]->dtime         = Plot[idin]->dtime;
 Plot[idout]->deltat        = Plot[idin]->deltat;
 Plot[idout]->refreshDt     = Plot[idin]->refreshDt;

 Plot[idout]->xnBin         = Plot[idin]->xnBin;
 Plot[idout]->ynBin         = Plot[idin]->ynBin;

 if (Plot[idin]->type != PLOT2D)
   {Plot[idout]->xmin          = Plot[idin]->xmin;
    Plot[idout]->xmax          = Plot[idin]->xmax;
    Plot[idout]->ymin          = Plot[idin]->ymin;
    Plot[idout]->ymax          = Plot[idin]->ymax;
   }

 Plot[idout]->timeshift     = Plot[idin]->timeshift;

 Plot[idout]->fmin          = Plot[idin]->fmin;
 Plot[idout]->fmax          = Plot[idin]->fmax;

 Plot[idout]->zmin          = Plot[idin]->zmin;
 Plot[idout]->zmax          = Plot[idin]->zmax;

 Plot[idout]->crossCohe_fmin = Plot[idin]->crossCohe_fmin;
 Plot[idout]->crossCohe_fmax = Plot[idin]->crossCohe_fmax;
 Plot[idout]->crossCohe      = Plot[idin]->crossCohe;

 Plot[idout]->filter_fmin    = Plot[idin]->filter_fmin;
 Plot[idout]->filter_fmax    = Plot[idin]->filter_fmax;

 Plot[idout]->maxAverage    = Plot[idin]->maxAverage;
 Plot[idout]->refresh_fft   = Plot[idin]->refresh_fft;
 Plot[idout]->audio_t0      = Plot[idin]->audio_t0;
 Plot[idout]->audio_duration = Plot[idin]->audio_duration;
 Plot[idout]->sound16bits   = Plot[idin]->sound16bits;
 Plot[idout]->soundonline   = Plot[idin]->soundonline;

 Plot[idout]->nodc          = Plot[idin]->nodc;
 Plot[idout]->bits          = Plot[idin]->bits;

 Plot[idout]->gridx         = Plot[idin]->gridx;
 Plot[idout]->gridy         = Plot[idin]->gridy;
 Plot[idout]->logx          = Plot[idin]->logx;
 Plot[idout]->logy          = Plot[idin]->logy;

 Plot[idout]->gridx2        = Plot[idin]->gridx2;
 Plot[idout]->gridy2        = Plot[idin]->gridy2;
 Plot[idout]->logx2         = Plot[idin]->logx2;
 Plot[idout]->logy2         = Plot[idin]->logy2;

 Plot[idout]->logz          = Plot[idin]->logz;
 Plot[idout]->stat          = Plot[idin]->stat;
 Plot[idout]->type2d        = Plot[idin]->type2d;
 Plot[idout]->autoedge      = Plot[idin]->autoedge;
 Plot[idout]->autoscale     = Plot[idin]->autoscale;
 Plot[idout]->modulesqrt    = Plot[idin]->modulesqrt;
 Plot[idout]->hertz         = Plot[idin]->hertz;
 Plot[idout]->rmsfft        = Plot[idin]->rmsfft;
 Plot[idout]->decayfft      = Plot[idin]->decayfft;
 Plot[idout]->decay         = Plot[idin]->decay;
 Plot[idout]->median_norm   = Plot[idin]->median_norm;

 Plot[idout]->superposed    = Plot[idin]->superposed;
 Plot[idout]->toberemoved   = Plot[idin]->toberemoved;

 return;
}


/*--------------------------------------------------- DyCopyPlotForSame ---*/
/*  called from DyXformEditPlot for each plot superposed to the plot edited */

void DyCopyPlotForSame (struct DyPlot *target, struct DyPlot *source)
{
 char dummy[LSTRING_MAX];

 if (source == NULL || target == NULL) return;

 sprintf(dummy,"DyCopyPlotForSame> in=%s  out=%s", source->name, target->name);
 DyDebugPrint(dummy,0);

 /*--- In case the sampling frequency of the target is too low for the source deltat ---*/
/**
 ICI
 if (target->chx != NULL && source->deltat < 2./target->chx->resampFreq && source->type != PLOT1D)
   {source->deltat = 2./target->chx->resampFreq;
    source->tsize = source->deltat * 100./source->tstep;
    if (target->type == PLOTFFT || target->type == PLOTTR || target->type == PLOTCOHE)
       source->fft_duration = source->deltat * 100./source->tstep;
   }
**/

 target->superposed = ON;

/*
 target->logx   = source->logx;
 target->logy   = source->logy;

 target->ymin   = source->ymin;
 target->ymax   = source->ymax;
 target->tstep  = source->tstep;
 target->tsize  = source->tsize;
 target->deltat = source->deltat;
*/

 if (target->type == PLOTTIME)
   {target->tstep  = source->tstep;
    target->tsize  = source->tsize;
    target->deltat = source->deltat;
   }

/*
 if (target->type == PLOTFFT || target->type == PLOTTR || target->type == PLOTCOHE)
    {target->maxAverage   = source->maxAverage;
     target->refresh_fft  = source->refresh_fft;
     target->fft_duration = source->fft_duration;
    }
*/

/*
 target->refreshDt = source->refreshDt;
*/

/**
 if (target->type == PLOT1D || target->type == PLOT2D)
   {target->refreshDt = target->deltat;}
**/

 return;
}


/*--------------------------------------------------- DySuperposePlot ---*/
/*   Called when Superpose button is pushed.                             */
/* Superpose to the first plot selected all the other plots selected     */

int DySuperposePlot ()
{
 char strnew[LSTRING_MAX];
 long i, i0, iref, iref0, imax;
 double maxFreq;
 struct DyPlot *dummyplot;

 DyDebugPrint(__FUNCTION__, 0);

 if (Dy->nplot <= 0) return(-1);

 /*--- First plot selected is the reference one over which the others will be superposed ---*/

 iref = -1;
 for (i=1; i<=Dy->nplot; i++)
    {if (Plot[i]->toberemoved == ON) {iref = i; break;}}

 /*--- If no plot selected, do nothing ---*/
 if (iref == -1) return(-1);

 /*--- If the first plot selected is of wrong type, return ---*/

 if (Plot[iref]->type == PLOT2D || Plot[iref]->type == PLOTFFTTIME || Plot[iref]->type == PLOT1DTIME ||
    Plot[iref]->type == PLOTTRTIME || Plot[iref]->type == PLOTCOHETIME)
   {DyInfoLine("Plot 2D, FFTTIME, 1DTIME, TRFCTTIME OR COHETIME can not be superposed"); return(-2);}

/*--- iref0 is the main reference for the parameters alignment ---*/
/*--- iref is temporary reference to put together all the plots superposed ---*/

 iref0 = iref;

 /*--- Superpose only plots of same type ---*/

 for (i=iref0+1; i<=Dy->nplot; i++)
    {if (Plot[i]->toberemoved == ON)
        {if (Plot[i]->type != Plot[iref]->type) Plot[i]->toberemoved = OFF;}
    }

 /*--- Which plot has the maximal sampling frequency ---*/

 maxFreq = 0.;
 imax    = 0;
 for (i=iref0+1; i<=Dy->nplot; i++)
    {if (Plot[i]->toberemoved == ON && Plot[i]->chx->resampFreq > maxFreq)
        {maxFreq = Plot[i]->chx->resampFreq; imax = i;}
    }

 /*--- Put the max freq plot as the new iref0 plot ---*/

 if (imax > iref  &&  Plot[imax]->chx->resampFreq > Plot[iref0]->chx->resampFreq)
    {
     dummyplot = Plot[iref0];
     Plot[iref0] = Plot[imax];
     Plot[imax] = dummyplot;
    }

 Plot[iref0]->superposed = OFF;
 Plot[iref0]->bits = OFF;

 /*--- Move the selected plots so that they are all just after the iref (first selected) plot ---*/

 i0 = 1;
 while (i0 > 0)
   {
    i0 = -1;
    for (i=iref+1; i<=Dy->nplot; i++)
       {if (Plot[i]->toberemoved == ON) {i0 = i; break;}}
    if (i0 <= 0) break;

    /*--- If needed, copy the plot in a temporary area and remove it from browser ---*/

    if (i0 > iref+1)
       {
        DyCopyPlot(NPLOT_MAX-1,i0,ON);
        for (i=i0-1; i>iref; i--) DyCopyPlot(i+1,i,ON);
        for (i=i0-1; i>iref; i--)
           {if (Plot[i]->toberemoved == ON) Plot[i+1]->toberemoved = ON;
            else                            Plot[i+1]->toberemoved = OFF;
           }
        DyCopyPlot(iref+1,NPLOT_MAX-1,ON);
       }

    DyCopyPlotForSame (Plot[iref+1], Plot[iref0]);

    /* ICI: Take as iref0 the plot with smallest or highest sampFreq and adapt refreshPeriod */

    iref++;
   }

 /*--- Update the idsame of plots ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->superposed == ON) Plot[i]->idsame = iref0;
     else                           Plot[i]->idsame = i;
     Plot[i]->toberemoved = OFF;
    }

 /*--- Fill the plots browser with the updated list of plots ---*/

 DyXformPlotFillBrowser();

 return(0);
}


/*------------------------------------------- DyUpdatePlotParam ---*/
/*
  Called when: an Update Channels is made
  Called why: to update plot parameters if corresponding channels parameters (like sampling frequency) have changed
  Call what: none 
*/

void DyUpdatePlotParam (struct FrameH *frame, int id)
{
 struct FrVect *vect;
 char dummy[LSTRING_MAX];

 if (frame == NULL) return;

 sprintf(dummy,"DyUpdatePlotParam> plot%d=%s=%s  frame=%d", id, Plot[id]->name, Dy->typeName[Plot[id]->type], frame->GTimeS);
 DyDebugPrint(dummy,0);

 /*--- Change of channels sampling frequency ---*/

 /*--- Update vect x parameters ---*/

 if (Plot[id]->chx == NULL) return;

 if (strstr(Plot[id]->chx->name,"_sample") != NULL) return; /* special case of 2d time vectors */

 vect = FrameFindVect(frame, Plot[id]->chx->name);
 if (vect) Plot[id]->chx->sampFreq = 1./vect->dx[0];

 /*--- Update vect y parameters ---*/

 if (Plot[id]->chy == NULL) return;

 if (strstr(Plot[id]->chy->name,"_sample") != NULL) return; /* special case of 2d time vectors */

 vect = FrameFindVect(frame, Plot[id]->chy->name);
 if (vect) Plot[id]->chy->sampFreq = 1./vect->dx[0];

 return;
}


/*--------------------------------- DyUpdatePlotInput ---*/

void DyUpdatePlotInput (int id)
{
 if (Plot[id]->chx == NULL) return;
 strcpy(Plot[id]->chx->inputname,Dy->ch[Dy->nch-1]->inputname);
 Plot[id]->chx->inputtype = Dy->ch[Dy->nch-1]->inputtype;

 if (Plot[id]->chy == NULL) return;
 strcpy(Plot[id]->chy->inputname,Dy->ch[Dy->nch-1]->inputname);
 Plot[id]->chy->inputtype = Dy->ch[Dy->nch-1]->inputtype;

 return;
}


/*------------------------------------------- DyPutDefaultInPlot ---*/

void DyPutDefaultInPlot (long id)
{
 char dummy[LSTRING_MAX];
 struct DyPlot *plot;

 plot = Plot[id];

 sprintf(dummy,"DyPutDefaultInPlot> plot%ld  name=%s  type=%s", id, plot->name, Dy->typeName[plot->type]);
 DyDebugPrint(dummy,0);

 /*--- Partial init of the Plot ---*/

 DyPlotInit(id,OFF);

 /*--- Init to some default values ---*/

 plot->superposed  = OFF;
 plot->toberemoved = OFF;

 plot->idsame    = id;
 plot->numpad    = id;
 plot->numsubpad = 0;

 plot->nodc      = OFF;
 plot->stat      = OFF;
 plot->gridx     = ON;
 plot->gridy     = ON;
 plot->gridx2    = ON;
 plot->gridy2    = ON;
 plot->logx      = OFF;
 plot->logy      = ON;
 plot->logz      = OFF;
 plot->logx2     = OFF;
 plot->logy2     = OFF;
 plot->autoscale = ON;

 plot->chx->realunits  = ON;
 plot->chx->resampFreq = plot->chx->sampFreq;

 if (plot->chy)
   {plot->chy->realunits  = ON;
    plot->chy->resampFreq = plot->chy->sampFreq;
   }

 /*--- Now put the default values specific to each plot type ---*/

 if (plot->type == PLOTTIME)     DyPutDefaultInPLOTTIME(id);
 if (plot->type == PLOTFFT)      DyPutDefaultInPLOTFFT(id);
 if (plot->type == PLOTTR)       DyPutDefaultInPLOTTR(id);
 if (plot->type == PLOTCOHE)     DyPutDefaultInPLOTCOHE(id);
 if (plot->type == PLOT1D)       DyPutDefaultInPLOT1D(id);
 if (plot->type == PLOT2D)       DyPutDefaultInPLOT2D(id);
 if (plot->type == PLOTRAW)      DyPutDefaultInPLOTRAW(id);
 if (plot->type == PLOTFFTTIME)  DyPutDefaultInPLOTFFTTIME(id);
 if (plot->type == PLOTTRTIME)   DyPutDefaultInPLOTTRTIME(id);
 if (plot->type == PLOTCOHETIME) DyPutDefaultInPLOTCOHETIME(id);
 if (plot->type == PLOT1DTIME)   DyPutDefaultInPLOT1DTIME(id);
 if (plot->type == PLOTRAWTIME)  DyPutDefaultInPLOTRAWTIME(id);
 if (plot->type == PLOTAUDIO)    DyPutDefaultInPLOTAUDIO(id);

 /*--- Special treatment for 2d time vectors ---*/

 if (plot->chx->vect2d_duration > 0)
   {
    printf("PutDefault> chx->name=%s  chx->vect2d_duration=%g \n", plot->chx->name, plot->chx->vect2d_duration);
    plot->tsize = plot->chx->vect2d_duration;
    plot->tstep = 100 * plot->deltat / plot->tsize;
    plot->xnBin = plot->tsize * plot->chx->resampFreq;
    plot->deltat = plot->chx->vect2d_deltat;
    plot->fft_duration = plot->chx->vect2d_duration;
   }

 /*--- Now limit the values range ---*/

 if (id > 0)
    {DyConstrain(&(Plot[id]->ymin));
     DyConstrain(&(Plot[id]->ymax));
    }

 return;
}


/*------------------------------------------- DyConstrain ---*/

void DyConstrain (double *value)
{
 if (*value >  1.e+33) *value =  1.e+33;
 if (*value < -1.e+33) *value = -1.e+33;
 if (*value<0  &&  *value > -1.e-33) *value = -1.e-33;
 if (*value>0  &&  *value <  1.e-33) *value =  1.e-33;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTRAW ---*/

void DyPutDefaultInPLOTRAW(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->type2d         = COLZ;
 plot->logx           = OFF;
 plot->logy           = OFF;
 plot->logz           = OFF;
 plot->chx->realunits = ON;
 plot->decayfft       = OFF;

 plot->zmin       = 0;
 plot->zmax       = 0;
 plot->maxAverage = 1;
 plot->refreshDt  = 1;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTRAWTIME ---*/

void DyPutDefaultInPLOTRAWTIME(long id)
{
 struct FrVect *vect;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->type2d         = COLZ;
 plot->logx           = OFF;
 plot->logy           = OFF;
 plot->logz           = ON;
 plot->chx->realunits = ON;

 plot->maxAverage = 1;
 plot->refreshDt  = 1; 
 plot->tsize      = 100 * plot->refreshDt;
 plot->tstep      = 20;

 vect = FrameFindVect(Dy->Frame, plot->chx->name);
 DyGetMinMax(vect, &plot->zmin, &plot->zmax);   
 
 return;
}

  
/*------------------------------------------- DyPutDefaultInPLOTTIME ---*/

void DyPutDefaultInPLOTTIME(long id)
{
 struct DyPlot *plot;
 struct FrVect *vectx;
 double vmin, vmax;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->logx = OFF;
 plot->logy = OFF;
 plot->bits = OFF;

 plot->xnBin  = 2000;
 plot->tsize  = (double)plot->xnBin / plot->chx->resampFreq;
 if (plot->tsize < 2) {plot->tsize = 2; plot->xnBin = (int)(plot->chx->resampFreq*plot->tsize);}
 if (Dy->InputType == INCM && plot->tsize>4) {plot->tsize = 4; plot->xnBin = (int)(plot->chx->resampFreq*plot->tsize);}
 plot->tstep  = 50;
 plot->deltat = plot->tsize * plot->tstep / 100.;

 plot->ymin = -1;
 plot->ymax =  1;

 vectx = FrameFindVect(Dy->Frame, plot->chx->name);
 DyGetMinMax(vectx, &vmin, &vmax);
 plot->ymin = vmin - 2.*(vmax - vmin);
 plot->ymax = vmax + 2.*(vmax - vmin);

 plot->yoffset = 0.;
 plot->yscale  = 0.;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTFFT ---*/

void DyPutDefaultInPLOTFFT(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->logx     = ON;
 plot->logx     = ON;
 plot->nodc     = ON;
 plot->rmsfft   = OFF;
 plot->decayfft = OFF;
 plot->hertz    = OFF;
 plot->x2av     = OFF;

 plot->xnBin        = 2000;
 plot->fft_duration = (double)plot->xnBin / plot->chx->resampFreq;
 if (plot->fft_duration < 2)
   {plot->fft_duration = 2; plot->xnBin = (int)(plot->chx->resampFreq*plot->fft_duration);}
 if (Dy->InputType == INCM && plot->fft_duration>4)
   {plot->fft_duration = 4; plot->xnBin = (int)(plot->chx->resampFreq*plot->fft_duration);}
 plot->tstep        = 50;
 plot->deltat       = plot->fft_duration * plot->tstep / 100.;
 plot->maxAverage   = 10;
 plot->refresh_fft  = 1;

 plot->fmin = 0;
 plot->fmax = plot->chx->resampFreq / 2.;

 plot->ymin = 0;
 plot->ymax = 0;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOT1D ---*/

void DyPutDefaultInPLOT1D(long id)
{
 struct DyPlot *plot;
 struct FrVect *vectx;
 double vmin, vmax;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->refreshDt = 2000 / plot->chx->resampFreq;
 if (plot->refreshDt < 2) plot->refreshDt = 2; 
 if (Dy->InputType == INCM && plot->refreshDt>4) plot->refreshDt = 4;
 plot->tsize     = plot->refreshDt;
 plot->tstep     = 100;

 plot->logx     = OFF;
 plot->logy     = ON;
 plot->stat     = ON;
 plot->nodc     = OFF;
 plot->autoedge = ON;

 plot->xnBin = 200;
 plot->xmin  = -1.e-15;
 plot->xmax  =  1.e-15;

 /*--- Get min and max for channel x ---*/

 vectx = FrameFindVect(Dy->Frame, plot->chx->name);
 DyGetMinMax(vectx, &vmin, &vmax);
 plot->xmin = vmin - 2.*(vmax - vmin);
 plot->xmax = vmax + 2.*(vmax - vmin);
 
 plot->xoffset = 0.;
 plot->xscale  = 0.;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOT2D ---*/

void DyPutDefaultInPLOT2D(long id)
{
 struct FrVect *vectx, *vecty;
 struct DyPlot *plot;
 double vmin, vmax;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->refreshDt = 2000. / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
 if (plot->refreshDt < 2) plot->refreshDt = 2; 
 if (Dy->InputType == INCM && plot->refreshDt>4) plot->refreshDt = 4;

 plot->type2d   = COLZ;
 plot->logy     = OFF;
 plot->stat     = OFF;
 plot->autoedge = ON;

 plot->xnBin = 200;
 plot->ynBin = 200;

 /*--- Get min and max for channel x ---*/

 vectx = FrameFindVect(Dy->Frame, plot->chx->name);
 DyGetMinMax(vectx, &vmin, &vmax);
 plot->xmin = vmin - 2.*(vmax - vmin);
 plot->xmax = vmax + 2.*(vmax - vmin);
 if (plot->xmin >= plot->xmax) {plot->xmin = -1e-15; plot->xmax = 1e-15;}

 /*--- Get min and max for channel y ---*/

 vecty = FrameFindVect(Dy->Frame,plot->chy->name);
 DyGetMinMax(vecty, &vmin, &vmax);
 plot->ymin = vmin - 2.*(vmax - vmin);
 plot->ymax = vmax + 2.*(vmax - vmin);
 if (plot->ymin >= plot->ymax) {plot->ymin = -1e-15; plot->ymax = 1e-15;}

 plot->xoffset = 0.;
 plot->xscale  = 0.;
 plot->yoffset = 0.;
 plot->yscale  = 0.;

return;
}


/*------------------------------------------- DyPutDefaultInPLOTTR ---*/

void DyPutDefaultInPLOTTR(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->logx = ON;
 plot->logy = ON;
plot->logx2 = ON;
 plot->nodc       = ON;
 plot->modulesqrt = OFF;
 plot->decayfft   = ON;

 plot->xnBin        = 2000;
 plot->fft_duration = (double)plot->xnBin / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
 if (plot->fft_duration < 2)
   {plot->fft_duration = 2; plot->xnBin = (int)(DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)*plot->fft_duration);}
 if (Dy->InputType == INCM && plot->fft_duration>4)
   {plot->fft_duration = 4; plot->xnBin = (int)(DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)*plot->fft_duration);}
 plot->tstep        = 50;
 plot->maxAverage   = 10;
 plot->refresh_fft  = 1;

 plot->fmin = 0.;
 plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;

 plot->ymin = 0;
 plot->ymax = 0;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTCOHE ---*/

void DyPutDefaultInPLOTCOHE(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 plot->logx = ON; 
 plot->logy      = OFF;
 plot->nodc      = ON;
 plot->autoscale = OFF;
 plot->crossCohe = OFF;
 plot->modulesqrt = OFF;

 plot->xnBin        = 2000;
 plot->fft_duration = (double)plot->xnBin / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
 if (plot->fft_duration < 2)
   {plot->fft_duration = 2; plot->xnBin = (int)DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);}
 if (Dy->InputType == INCM && plot->fft_duration>4)
   {plot->fft_duration = 4; plot->xnBin = (int)(DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)*plot->fft_duration);}
 plot->tstep        = 50;
 plot->maxAverage   = 100000;
 plot->refresh_fft  = 1;

 plot->fmin            = 0.;
 plot->fmax            = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;
 plot->crossCohe_fmin  = 0.;
 plot->crossCohe_fmax  = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;

 plot->ymin = 0;
 plot->ymax = 1;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTFFTTIME ---*/

void DyPutDefaultInPLOTFFTTIME(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 if (plot->chx->resampFreq > 9999.) plot->logy = ON;
 plot->logy     = OFF;
 plot->type2d   = COLZ;
 plot->logx     = OFF;
 plot->logy     = OFF;
 plot->logz     = ON;
 plot->decayfft = OFF;
 plot->median_norm = OFF;

 plot->xnBin        = 100;
 plot->ynBin        = 100;
 plot->fft_duration = (double)plot->ynBin / plot->chx->resampFreq;
 plot->tstep_fft    = 50;
 plot->dtime        = plot->fft_duration;

 if (plot->fft_duration < 0.1)
   {plot->fft_duration = 0.1; plot->ynBin = (int)(plot->chx->resampFreq*plot->fft_duration);
    plot->dtime = plot->fft_duration;
   }

 if (plot->fft_duration > 1)
   {plot->dtime = 1; plot->tstep_fft = plot->dtime / plot->fft_duration * 100.;}

 plot->dfreq        = 1. / plot->fft_duration;
 plot->tsize        = plot->xnBin * plot->dtime;
 plot->tstep        = 10;

 if (Dy->InputType == INCM && plot->tsize < 20)
   {plot->tsize = 20;
    plot->dtime = plot->tsize / plot->xnBin;
   }

 plot->fmin = 0;
 plot->fmax = plot->chx->resampFreq / 2.;

 plot->zmin = 0;
 plot->zmax = 0;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTTRTIME ---*/

void DyPutDefaultInPLOTTRTIME(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 if (DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) > 9999.) plot->logy = ON;
 plot->logy     = OFF;
 plot->type2d   = COLZ;
 plot->nodc     = ON;
 plot->logx     = OFF;
 plot->logy     = OFF;
 plot->logz     = ON;
 plot->decayfft = OFF;

 plot->xnBin        = 100;
 plot->ynBin        = 100;
 plot->fft_duration = (double)plot->ynBin / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);

 if (plot->fft_duration < 0.1)
   {plot->fft_duration = 0.1;
    plot->ynBin = (int)(DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)*plot->fft_duration);
   }

 plot->tstep_fft = 5;
 plot->dtime     = plot->fft_duration;
 plot->dfreq     = 1. / plot->fft_duration;
 plot->tsize     = plot->xnBin * plot->dtime;
 plot->tstep     = 20;
 if (plot->tstep < 5) plot->tstep = 5;
 if (plot->tstep > 50) plot->tstep = 50;

 if (Dy->InputType == INCM && plot->tsize < 20)
   {plot->tsize = 20;
    plot->dtime = plot->tsize / plot->xnBin;
   }

 plot->fmin = 0.;
 plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;

 plot->zmin = 0;
 plot->zmax = 0;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTCOHETIME ---*/

void DyPutDefaultInPLOTCOHETIME(long id)
{
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 if (DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) > 9999.) plot->logy = ON;
 plot->logy     = OFF;
 plot->type2d   = COLZ;
 plot->nodc     = ON;
 plot->logx     = OFF;
 plot->logy     = OFF;
 plot->logz     = ON;
 plot->decayfft = OFF;

 plot->xnBin        = 100;
 plot->ynBin        = 100;
 plot->fft_duration = (double)plot->ynBin / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);

 if (plot->fft_duration < 0.1)
   {plot->fft_duration = 0.1;
    plot->ynBin = (int)(DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)*plot->fft_duration);
   }

 plot->tstep_fft = 5;
 plot->dtime     = plot->fft_duration;
 plot->dfreq     = 1. / plot->fft_duration;
 plot->tsize     = plot->xnBin * plot->dtime;
 plot->tstep     = 20;
 if (plot->tstep < 5) plot->tstep = 5;
 if (plot->tstep > 50) plot->tstep = 50;

 if (Dy->InputType == INCM && plot->tsize < 20)
   {plot->tsize = 20;
    plot->dtime = plot->tsize / plot->xnBin;
   }

 plot->fmin = 0.;
 plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;

 plot->zmin = 0;
 plot->zmax = 1;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOT1DTIME ---*/

void DyPutDefaultInPLOT1DTIME(long id)
{
 struct DyPlot *plot;
 struct FrVect *vectx;
 double vmin, vmax;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 /*--- Get min and max for channel x ---*/
 vectx = FrameFindVect(Dy->Frame, plot->chx->name);
 DyGetMinMax(vectx, &vmin, &vmax);
 plot->ymin = vmin - 2.*(vmax - vmin);
 plot->ymax = vmax + 2.*(vmax - vmin);
 if (plot->ymin >= plot->ymax) {plot->ymin = -1; plot->ymax = 1;}


 if (plot->ymin == 0 && plot->ymax > 9999.) plot->logy = ON;
 if (plot->ymin > 0 && plot->ymax/plot->ymin > 9999.) plot->logy = ON;
 plot->logy      = OFF;
 plot->type2d    = COLZ;
 plot->logx      = OFF;
 plot->logz      = ON;
 plot->nodc      = ON;
 plot->autoedge  = ON;
 plot->autoscale = ON;

 plot->deltat = 256 / plot->chx->resampFreq;
 if (plot->deltat < 0.1) plot->deltat = 0.1; 
 if (Dy->InputType == INCM && plot->deltat>0.2) plot->deltat = 0.2;
 plot->dtime  = plot->deltat;
 plot->tsize  = 100. * plot->dtime;
 plot->tstep  = 20;
 plot->ynBin  = 100;

 plot->zmin = 0;
 plot->zmax = 0;

 return;
}


/*------------------------------------------- DyPutDefaultInPLOTAUDIO ---*/

void DyPutDefaultInPLOTAUDIO(long id)
{
 struct DyPlot *plot;
 int i;

 DyDebugPrint(__FUNCTION__, 0);

 plot = Plot[id];

 for (i=1; i<=Dy->nplot; i++)
    {if (Plot[i]->type == plot->type && Plot[i]->soundonline == ON) Plot[i]->soundonline = OFF;}

 plot->chx->realunits = OFF;
 plot->soundonline    = OFF; 
 plot->sound16bits    = OFF; 
 plot->audio_t0       = 0;
 plot->audio_duration = Dy->duration;
 if (plot->audio_duration <= 0 || plot->audio_duration > 10) plot->audio_duration = 10;

 return;
}


/*------------------------------------------- DyGetDefaultFromPlot ---*/
/*
  Called when: a plot is created and a plot of same type already exists
  Called why: to put in the new plot the same parameters as the existing plot
  Call what: NONE
*/

void DyGetDefaultFromPlot (long id)
{
 int i;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyGetDefaultFromPlot> plot%ld  name=%s  type=%s nplot=%d",
         id, Plot[id]->name, Dy->typeName[Plot[id]->type], Dy->nplot);
 DyDebugPrint(dummy, 0);

 if (Plot[id]->chx && Plot[id]->chx->vect2d_duration > 0) return;
 if (Plot[id]->chy && Plot[id]->chy->vect2d_duration > 0) return;


 /*--- If a plot is found with the same type and same chx->sampFreq as the one just created ---*/
 /*--- Copy its parameters into the new plot, with the flag OFF, which means copy only first level parameters ---*/

 for (i=1; i<=Dy->nplot; i++)
    {
     if (Plot[i]->hidden == ON) continue;
/*
     if (i != id && Plot[i]->type == Plot[id]->type && Plot[i]->chx->sampFreq == Plot[id]->chx->sampFreq)
*/
     if (Plot[i]->chx && Plot[i]->chx->vect2d_duration > 0) continue;
     if (Plot[i]->chy && Plot[i]->chy->vect2d_duration > 0) continue;
     if (Plot[id]->type == PLOTFFTTIME && Plot[i]->chx->sampFreq != Plot[id]->chx->sampFreq) continue;

     if (i != id && Plot[i]->type == Plot[id]->type)
        {
         DyCopyPlot(id,i,OFF);
         Plot[id]->superposed = OFF;
         if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
            {printf("  DyGetDefaultFromPlot> Put default in %s using parameters of %s \n", Plot[id]->name, Plot[i]->name);}
         break;
        }
    }

 return;
}


/*------------------------------------------- DyPlotAlignTime ---*/

void DyPlotAlignTime (long id)
{
 int i;
 double tsize0, tstep0, deltat0;
 struct DyPlot *plot, *plot0;

 plot0 = Plot[id];

 if (plot0->type == PLOTFFT || plot0->type == PLOTTR || plot0->type == PLOTCOHE)
   {tsize0  = plot0->fft_duration;
    tstep0  = plot0->tstep;
    deltat0 = tsize0 * tstep0/100.;
   }
 else if (plot0->type == PLOT1D || plot0->type == PLOT2D)
   {tsize0  = plot0->refreshDt;
    deltat0 = tsize0;
   }
 else
   {tsize0  = plot0->tsize;
    tstep0  = plot0->tstep;
    deltat0 = tsize0 * tstep0/100.;
   }  

 for (i=1; i<=Dy->nplot; i++)
    {
     if (i == id) continue;

     plot = Plot[i];
     if (plot->type == PLOTFFT || plot->type == PLOTTR || plot->type == PLOTCOHE)
        {plot->fft_duration = tsize0;
         plot->tstep        = tstep0;
         plot->deltat       = deltat0;
        }
     else if (plot->type == PLOT1D || plot->type == PLOT2D)
        {plot->refreshDt = tsize0;
         plot->deltat    = deltat0;
        }
     else if (plot->type == PLOTFFTTIME || plot->type == PLOTTRTIME || plot->type == PLOTCOHETIME)
        {plot->tsize  = tsize0;
         plot->tstep  = tstep0;
         plot->dtime = plot->tsize / plot->xnBin;
         plot->fft_duration = plot->dtime / plot->maxAverage;
         plot->dfreq = 1./ plot->fft_duration;
         plot->ynBin = (int)(plot->chx->resampFreq / plot->dfreq);
         plot->deltat = deltat0;
        }
     else
        {plot->tsize  = tsize0;
         plot->tstep  = tstep0;
         plot->deltat = deltat0;
        }
     DyPlotCheckParam(i);
     DyPlotSetSecondParam(i);
    }

 return;
}


/*------------------------------------------- DyPlotInit ---*/

void DyPlotInit (long id, long fullinit)
{
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyPlotInit> plot%ld  name=%s  type=%s fullinit=%ld", id, Plot[id]->name, Dy->typeName[Plot[id]->type], fullinit);
 DyDebugPrint(dummy, 0);

 /*
ICI Do we have a special init for the plot number 0 ?
*/

 if (id == 0) return;

 if (fullinit == ON)                    /* Init also parameters not seen in plot panel */
    {
     DyPlotFree(id, ON);
     strcpy(Plot[id]->name,"");
     Plot[id]->type        = NONE;
     Plot[id]->superposed  = OFF;
     Plot[id]->toberemoved = OFF;
     Plot[id]->ready       = OFF;
     if (id == 0)
       {Plot[id]->chx = (struct DyCh *) malloc(sizeof(struct DyCh));
        Plot[id]->chy = (struct DyCh *) malloc(sizeof(struct DyCh));
        strcpy(Plot[id]->chx->name,"");
        strcpy(Plot[id]->chy->name,"");
        Plot[id]->chx->sampFreq = 0.;
        Plot[id]->chy->sampFreq = 0.;
       }
    }

 Plot[id]->xnBin         = 0;
 Plot[id]->xmin          = 0.;
 Plot[id]->xmax          = 0.;
 Plot[id]->ynBin         = 0;
 Plot[id]->ymin          = 0.;
 Plot[id]->ymax          = 0.;
 Plot[id]->zmin          = 0.;
 Plot[id]->zmax          = 0.;
 Plot[id]->timeshift     = 0.;
 Plot[id]->refreshDt     = 0;
 Plot[id]->deltat        = 0;
 Plot[id]->tsize         = 0.;
 Plot[id]->tstep         = 0.;
 Plot[id]->fft_duration  = 0.;
 Plot[id]->maxAverage    = 0;
 Plot[id]->decayfft      = 0;
 Plot[id]->rmsfft        = 0;
 Plot[id]->dtime         = 0.;
 Plot[id]->dfreq         = 0;
 Plot[id]->filter_fmin   = 0.;
 Plot[id]->filter_fmax   = 0.;
 Plot[id]->filter_order  = 0.;

 Plot[id]->nodc          = OFF;
 Plot[id]->bits          = OFF;

 Plot[id]->gridx         = OFF;
 Plot[id]->gridy         = OFF;
 Plot[id]->logx          = OFF;
 Plot[id]->logy          = OFF;
 Plot[id]->logz          = OFF;
 Plot[id]->stat          = OFF;
 Plot[id]->type2d        = NONE;
 Plot[id]->autoedge      = OFF;
 Plot[id]->autoscale     = OFF;
 Plot[id]->modulesqrt    = OFF;
 Plot[id]->x2av          = OFF;
 Plot[id]->hidden        = OFF;

 Plot[id]->audio_t0       = 0;
 Plot[id]->audio_duration = 0;
 Plot[id]->soundonline    = OFF;
 Plot[id]->sound16bits    = ON;

 return;
}




/**************************************************/
/*                                                */
/* File: DyPlotCompute.c                          */
/* Copyright (C) 2013, D. Verkindt,               */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*----------------------------------------------------- DyPlotSetTime ---*/

void DyPlotSetTime (double origin, char *plotName, double deltat, int nAverage, int timeaxis, struct FrameH *frame, int decayfft)
{
 char title[256];
 time_t starttimeutc;

 if (origin-(int)origin >= 0.9999) origin = (double)((int)(origin+0.1));

 if (frame) starttimeutc = FrGPS2UTC((unsigned int)origin, frame->ULeapS);
 else       starttimeutc = FrGPS2UTC((unsigned int)origin,0);

 strcpy(title,DYEMPTY);
 DyGpsToTimeString (origin, frame, deltat, nAverage, decayfft, &title[0]);

/*
 if (timeaxis == ON) RA_HSetTimeOffset(plotName,(double)starttimeutc+origin-(int)origin, timeaxis);
*/

 RA_HSetTimeOffset(plotName,(double)starttimeutc+origin-(int)origin, timeaxis);
 RA_HSetTimeFlag(plotName,1);
 RA_HSetTimeString(plotName,title);

 return;
}


/*---------------------------------------------------- DyPlotCheckFreq -----*/

void DyPlotCheckFreq(int iplot)
{
 int i, plottype, resampFactor;
 char dummy[LSTRING_MAX];
 struct DyPlot *plot;

 plot = Plot[iplot];


 if (plot->chx != NULL)
   {for (i=0; i<Dy->nch; i++)
       {if (strcmp(plot->chx->name,Dy->ch[i]->name) == 0)
          {
           if (plot->chx->sampFreq != Dy->ch[i]->sampFreq)
             {printf("Plot%d channel %s : Updating sampFreq from %g to %g Hz",
                      iplot, plot->chx->name, plot->chx->sampFreq, Dy->ch[i]->sampFreq);
              if (plot->chx->resampFreq == plot->chx->sampFreq) plot->chx->resampFreq = Dy->ch[i]->sampFreq;
              plot->chx->sampFreq = Dy->ch[i]->sampFreq;
              if (plot->chx->resampFreq > Dy->ch[i]->sampFreq) plot->chx->resampFreq = Dy->ch[i]->sampFreq;
              if (Dy->ch[i]->sampFreq/plot->chx->resampFreq != (int)(Dy->ch[i]->sampFreq/plot->chx->resampFreq))
                 plot->chx->resampFreq = Dy->ch[i]->sampFreq / (int)(Dy->ch[i]->sampFreq/plot->chx->resampFreq);
             }
           break;
          }
       }
   }

 if (plot->chy != NULL)
   {for (i=0; i<Dy->nch; i++)
       {if (strcmp(plot->chy->name,Dy->ch[i]->name) == 0)
          {if (plot->chy->sampFreq != Dy->ch[i]->sampFreq)
             {printf("Plot%d channel %s : Updating sampFreq from %g to %g Hz\n",
                      iplot, plot->chy->name, plot->chy->sampFreq, Dy->ch[i]->sampFreq);
              if (plot->chy->resampFreq == plot->chy->sampFreq) plot->chy->resampFreq = Dy->ch[i]->sampFreq;
              plot->chy->sampFreq = Dy->ch[i]->sampFreq;
             }
           if (plot->chy->resampFreq > Dy->ch[i]->sampFreq) plot->chy->resampFreq = Dy->ch[i]->sampFreq;
           if (Dy->ch[i]->sampFreq/plot->chy->resampFreq != (int)(Dy->ch[i]->sampFreq/plot->chy->resampFreq))
              plot->chy->resampFreq = Dy->ch[i]->sampFreq / (int)(Dy->ch[i]->sampFreq/plot->chy->resampFreq);
           break;
          }
       }
   }

 return;
}


/*---------------------------------------------------- DyPlotCheckPad -----*/
/* Force the numpad and idsame of the superposed plots, return the total number of pads needed */

int DyPlotCheckPad()
{
 int i, numpad, idsame;

 numpad = 0;
 for (i=1; i<=Dy->nplot; i++)
    {if (Plot[i]->superposed == OFF && Plot[i]->hidden == OFF && Plot[i]->type != PLOTAUDIO) {numpad++; idsame = i;}
     Plot[i]->numpad = numpad;
     Plot[i]->idsame = idsame;
    }

 return(numpad);
}


/*---------------------------------------------------- DyPlotCheckParam -----*/

void DyPlotCheckParam(int iplot)
{
 int i, plottype, resampFactor;
 char dummy[LSTRING_MAX];
 struct DyPlot *plot;

 plot = Plot[iplot];
 plottype = Plot[iplot]->type;

 sprintf(dummy,"DyPlotCheckParam> plot%d: name=%s  type=%s", iplot, plot->name, Dy->typeName[plottype]);
 DyDebugPrint(dummy,0);

 /*--- Check resampling frequency ---*/
 if (plot->chx->resampFreq <= 0  ||  plot->chx->resampFreq > plot->chx->sampFreq)
    {plot->chx->resampFreq = plot->chx->sampFreq;}

 /*--- Check that the chosen resampFactor gives a submultiple of sampFreq ---*/
 resampFactor = (long)(plot->chx->sampFreq / plot->chx->resampFreq + 0.5);
 plot->chx->resampFreq = plot->chx->sampFreq / (double)resampFactor;
 while (plot->chx->sampFreq/plot->chx->resampFreq != (int)(plot->chx->sampFreq/plot->chx->resampFreq) && resampFactor > 1)
   {resampFactor--;
    plot->chx->resampFreq = plot->chx->sampFreq / (double)resampFactor;
   }

 if (plot->chy != NULL)
    {if (plot->chy->resampFreq <= 0  ||  plot->chy->resampFreq > plot->chy->sampFreq)
       {plot->chy->resampFreq = plot->chy->sampFreq;}
     resampFactor = (long)(plot->chy->sampFreq / plot->chy->resampFreq + 0.5);
     plot->chy->resampFreq = plot->chy->sampFreq / (double)resampFactor;
     while (plot->chy->sampFreq/plot->chy->resampFreq != (int)(plot->chy->sampFreq/plot->chy->resampFreq) && resampFactor > 1)
       {resampFactor--;
        plot->chy->resampFreq = plot->chy->sampFreq / (double)resampFactor;
       }
    }

 if (plottype == PLOTRAW)
   {
    if (plot->maxAverage < 0) plot->maxAverage = 0;  /* 0 is for infinite average */  
    if (plot->ymin > 0 && plot->ymax/plot->ymin > 10000.) {plot->logy = ON; plot->logz = ON;}
    if (plot->refreshDt > Dy->Frame->dt) plot->refreshDt = Dy->Frame->dt; /* ICI */
   }

 else if (plottype == PLOTRAWTIME) 
   {
    if (plot->xnBin < 2) plot->xnBin = plot->nBin / 4;
     if (plot->xmax < plot->xmin) plot->xmax = plot->xmin;
     if (plot->maxAverage < 0) plot->maxAverage = 0;  /* 0 is for infinite average */
     if (plot->tsize < 0.) plot->tsize = 256. * Dy->Frame->dt;
   }

 else if (plottype == PLOTTIME)
   {
    if (plot->tsize <= 0.) plot->tsize = 10000. / plot->chx->resampFreq;
    if (plot->tstep <= 0.  ||  plot->tstep > 100.)  plot->tstep = 100.;
    if (plot->tstep*plot->tsize/100. < 1./plot->chx->resampFreq) plot->tstep = 100*(2./plot->chx->resampFreq)/plot->tsize;
    if (plot->tstep <= 0.  ||  plot->tstep > 100.)  plot->tstep = 100.;
   }

 else if (plot->type == PLOTFFT)
   {
    if (plot->fft_duration <= 0.) plot->fft_duration = 10000. / plot->chx->resampFreq;
    if (plot->tstep <= 0. || plot->tstep > 100.) plot->tstep = 100.;
    if (plot->fmin < 0. || plot->fmin > plot->chx->resampFreq/2.) plot->fmin = 0.;
    if (plot->fmax < plot->fmin || plot->fmax > plot->chx->resampFreq/2.) plot->fmax = plot->chx->resampFreq/2.;
    if (plot->maxAverage < 0)  plot->maxAverage = 0;  /* 0 is for infinite average */
    if (plot->refresh_fft <= 0) plot->refresh_fft = plot->maxAverage;
   }

 else if (plottype == PLOTTR) 
   {
    if (plot->fft_duration <= 0.) plot->fft_duration = 10000. / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
    if (plot->tstep <= 0.  ||  plot->tstep > 100.)  plot->tstep = 100.;
    if (plot->fmin < 0.  ||  plot->fmin > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmin = 0.;}
    if (plot->fmax < plot->fmin  ||  plot->fmax > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;}
    if (plot->maxAverage < 0)  plot->maxAverage = 0;  /* 0 is for infinite average */
    if (plot->refresh_fft <= 0) plot->refresh_fft = plot->maxAverage;
   }

 else if (plottype == PLOTCOHE) 
   {
    if (plot->fft_duration <= 0.) plot->fft_duration = 10000. / DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
    if (plot->tstep <= 0.  ||  plot->tstep > 100.)  plot->tstep = 100.;
    if (plot->fmin < 0.  ||  plot->fmin > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmin = 0.;}
    if (plot->fmax < plot->fmin  ||  plot->fmax > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;}

    if (plot->crossCohe_fmin < 0.  ||  plot->crossCohe_fmin > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->crossCohe_fmin = 0.;}
    if (plot->crossCohe_fmax < plot->crossCohe_fmin  ||  plot->crossCohe_fmax > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->crossCohe_fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq) / 2.;}

    if (plot->maxAverage < 0) plot->maxAverage = 0;  /* 0 is for infinite average */
    if (plot->refresh_fft <= 0) plot->refresh_fft = plot->maxAverage;
   }

 else if (plottype == PLOTFFTTIME)
   {
    if (plot->dfreq  <= 0.) plot->dfreq  = 0.5;
    if (plot->fmin < 0. || plot->fmin > plot->chx->resampFreq/2.) plot->fmin = 0.;
    if (plot->fmax < plot->fmin || plot->fmax > plot->chx->resampFreq/2.) plot->fmax = plot->chx->resampFreq/2.;
    if (plot->tsize <= 0) plot->tsize = 100. * plot->dtime;
    if (plot->tstep <= 0. || plot->tstep > 100.) plot->tstep = 100.;
    if (plot->tstep_fft <= 0. || plot->tstep_fft > 100.) plot->tstep_fft = 100.;
   }

 else if (plottype == PLOTTRTIME) 
   {
    if (plot->dfreq  <= 0.) plot->dfreq  = 0.5;
    if (plot->fmin < 0. || plot->fmin > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmin = 0.;}
    if (plot->fmax < plot->fmin || plot->fmax > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.;}
    if (plot->tsize <= 0) plot->tsize = 100. * plot->dtime;
    if (plot->tstep <= 0. || plot->tstep > 100.) plot->tstep = 100.;
    if (plot->tstep_fft <= 0. || plot->tstep_fft > 100.) plot->tstep_fft = 100.;
   }

 else if (plottype == PLOTCOHETIME)
   {
    if (plot->dfreq  <= 0.) plot->dfreq  = 0.5;
    if (plot->fmin < 0. || plot->fmin > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmin = 0.;}
    if (plot->fmax < plot->fmin || plot->fmax > DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.)
       {plot->fmax = DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)/2.;}
    if (plot->tsize <= 0) plot->tsize = 100. * plot->dtime;
    if (plot->tstep <= 0. || plot->tstep > 100.) plot->tstep = 100.;
    if (plot->tstep_fft <= 0. || plot->tstep_fft > 100.) plot->tstep_fft = 100.;
   }

 else if (plottype == PLOT1DTIME) 
   {
    if (plot->deltat <= 0.)  plot->deltat = 1000. / plot->chx->resampFreq;
    if (plot->ynBin < 4) plot->ynBin = 4;
    if (plot->ynBin > 4096) plot->ynBin = 4096;
    if (plot->tsize < 0.) plot->tsize = 128 * plot->deltat;
    if (plot->ymax < plot->ymin) plot->ymax = plot->ymin;
   }

 else if (plottype == PLOT1D)
   {
    if (plot->xnBin < 2)  plot->xnBin = 100;
    if (plot->xmax <= plot->xmin) DyGetMinMax(plot->chx->vect, &(plot->xmin), &(plot->xmax));
    if (plot->xmax <= plot->xmin) {plot->xmin = -1e-15; plot->xmax = 1e-15;}
    if (plot->refreshDt <= 0) plot->refreshDt = 9999999;
    if (plot->refreshDt < 2./plot->chx->resampFreq) plot->refreshDt = 2./plot->chx->resampFreq;
   }

 else if (plottype == PLOT2D)
   {
    if (plot->xnBin <    4) plot->xnBin =    4;
    if (plot->xnBin > 4096) plot->xnBin = 4096;
    if (plot->ynBin <    4) plot->ynBin =    4;
    if (plot->ynBin > 4096) plot->ynBin = 4096;
    if (plot->xmax <= plot->xmin) DyGetMinMax(plot->chx->vect, &(plot->xmin), &(plot->xmax));
    if (plot->xmax <= plot->xmin) {plot->xmin = -1e-15; plot->xmax = 1e-15;}
    if (plot->ymax <= plot->ymin) DyGetMinMax(plot->chy->vect, &(plot->ymin), &(plot->ymax));
    if (plot->ymax <= plot->ymin) {plot->ymin = -1e-15; plot->ymax = 1e-15;}
    if (plot->refreshDt <= 0) plot->refreshDt = 9999999;
    if (plot->refreshDt < 2./DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq)) plot->refreshDt = 2./DyGetMin(plot->chx->resampFreq, plot->chy->resampFreq);
   }

 else if (plottype == PLOTAUDIO)
   {
    if (plot->audio_t0 < 0)  plot->audio_t0 = 0;
    if (plot->audio_duration <= 0)  plot->audio_duration = 10;
    if (plot->soundonline == ON)
       {for (i=1; i<=Dy->nplot; i++)
           {if (Plot[i]->type == plot->type && Plot[i]->soundonline == ON) Plot[i]->soundonline = OFF;}
        plot->soundonline = ON;
       }
   }

 return;
}


/*----------------------------------------------------- DyPlotSetDrawInfo ---*/

void DyPlotSetDrawInfo (int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Set warning string if data missing or corrupted in chx ---*/

 if (plot->chx)
   {
    if (plot->chx->status == DY_NOCHANNEL)
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (DYMSG_NOCHANNEL);
      }
    else if (plot->chx->status == DY_BADDATA)
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (DYMSG_BADDATA);
      }
    else 
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (NULL);
      }
   }

 /*--- Set warning string if data missing or corrupted in chy ---*/

 if (plot->chy)
   {
    if (plot->chy->status == DY_NOCHANNEL)
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (DYMSG_NOCHANNEL);
      }
    else if (plot->chy->status == DY_BADDATA)
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (DYMSG_BADDATA);
      }
    else 
      {RA_SetActivePad(plot->numpad, plot->numsubpad);
       RA_HSetWarningString (NULL);
      }
   }

 return;
}


/*----------------------------------------------------- DyPlotSetDrawOptions ---*/

void DyPlotSetDrawOptions (int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 strcpy(plot->options,"\0");

 if (plot->superposed == ON) strcat(plot->options," SAME");

 if (plot->type2d == COLZ)  strcat(plot->options," COLZ");
 if (plot->type2d == CONT)  strcat(plot->options," CONT");
 if (plot->type2d == LEGO)  strcat(plot->options," LEGO");
 if (plot->type2d == SURF)  strcat(plot->options," SURF");
 if (plot->type2d == SCAT)  strcat(plot->options," SCAT");
 if (plot->bits == ON)      strcat(plot->options," COLZ");
 if (plot->crossCohe == ON) strcat(plot->options," COLZ");

/**
 if (plot->type == PLOTTIME    || plot->type == PLOTFFTTIME || plot->type == PLOT1DTIME ||
     plot->type == PLOTRAWTIME || plot->type == PLOTTRTIME || plot->type == PLOTCOHETIME)
   {strcat(plot->options," TIMX");}
**/

 return;
}


/*------------------------------------------------- DyPlotSetColor ---------*/
void DyPlotSetColor(int iplot)
{
 struct DyPlot *plot;
 int nsame;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 /*--- If single plot or first one of superposed, set black color to this plot ---*/

 if (plot->superposed == OFF)
   {plot->color = Dy->defaultcolor[0];
    RA_HSetColor(plot->name, plot->color);
   }

 /*--- If it is a superposed plot, choose the right color ---*/

 else
   {nsame = DyPlotGetSameIndex(iplot);
    if (nsame < 10) plot->color = Dy->defaultcolor[nsame];
    else              plot->color = nsame + 1;
    RA_HSetColor(plot->name, plot->color);
   }

 RA_HSetLineWidth(plot->name, Dy->linewidth);

 return;
}


/*==================================================================================================*/
/*============================================== PLOT FFT ==========================================*/

/*---------------------------------------------------- DyPlotFFTIni -----*/

void DyPlotFFTIni(int iplot)
{
 long nsuperposed;
 char *oldname, dummy[LSTRING_MAX];
 struct DyPlot *plot;
 int i;
 double xleg, yleg, legsize;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;

 /*--- Initialize the FFTW buffer ---*/
 
 plot->fftx = FrvRFFTNew("AHNS", plot->xnBin, 1);
 if (plot->fftx == NULL) {printf("DyPlotFFTIni: Could not get fftx buffer\n"); return;}

 /*--- Set the decay time for a moving average ---*/

 if (plot->decayfft == ON)
    {FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);}
 else
    {FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 1, plot->xnBin/2+1, 0, plot->name);
 RA_H1NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin/2+1, plot->dfreq*(-.5), plot->dfreq*((double)(plot->xnBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetXRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);  /* Set the log scale on axis of plot iplot */
    }
 DyPlotSetDrawOptions (iplot);

 /*--- Create or reuse a ROOT plot to superpose rms to the spectrum ---*/

 if (plot->rmsfft > 0)
    {
     plot->name_rms = (char *) calloc(strlen(plot->name)+10,sizeof(char));
     sprintf(plot->name_rms,"%s_rms",plot->name);
     plot->title_rms = (char *) calloc(strlen(plot->title)+10,sizeof(char));
     sprintf(plot->title_rms,"%s_rms",plot->title);
     plot->vect_rms = FrVectNewTS(plot->name_rms, plot->chx->resampFreq, plot->xnBin/2, -64);
     for (i=0; i<plot->vect_rms->nData; i++) plot->vect_rms->dataD[i] = 0;
     RA_H1NewReuse(plot->numpad, plot->numsubpad, plot->name_rms, plot->title_rms,
                   plot->xnBin/2+1, plot->dfreq*(-0.5), plot->dfreq*((double)(plot->xnBin/2+1)-0.5));
    }

 /*--- If plot not reused, set the Y axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->ymin > 0. && plot->ymax > 0.)
    {RA_HSetMinimum(plot->name, plot->ymin);
     RA_HSetMaximum(plot->name, plot->ymax);
    }

 /*--- If autoscale is set, start with correct values of ymin and ymax ---*/

 if (plot->autoscale == ON)
   {plot->ymin = 1e+30;
    plot->ymax = 1e-30;
   }

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);
 if (plot->rmsfft == ON)
   {RA_HSetColor(plot->name_rms, plot->color);
    RA_HSetLineStyle(plot->name_rms, 2);
   }

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot);
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
 else
   {RA_HSetTitle(plot->name, plot->title);}
**/

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- X axis units of a spectrum ---*/

 RA_HSetUnitsX(plot->name, "Hz");

 /*--- Y axis units of the spectrum ---*/

 sprintf(dummy,"%s/sqrt(Hz)", DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) sprintf(dummy,"%s/sqrt(Hz)", plot->chx->vect->unitY);
 RA_HSetUnitsY(plot->name, dummy);

 return;
}


/*--------------------------------------------------- DyPlotFFT -----*/

void DyPlotFFT(int iplot)
{
 double localymin, localymax;
 double dt, rms_value, rmsSum;
 struct DyPlot *plot;
 int j, nfft_refresh;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Do not compute FFT for vector containing corrupted or missing data ---*/

 if (plot->chx->status != ON || plot->chx->vect == NULL || plot->chx->vect->next != NULL)
   {printf("DyPlotFFT> Input vector %s at time %f has missing or corrupted samples. Excluding it from FFT computation\n",
           plot->chx->name, plot->gtime);
    return;
   }

 /*--- Compute the FFT ---*/

 FrvRFFTFor(plot->fftx, plot->chx->vect);
 plot->nAverage++;

 /*--- Compute the rms ---*/

 if (plot->rmsfft == ON && plot->fftx->amplitudeA != NULL)
   {
    rmsSum = 0.;
    for (j=plot->xnBin/2 ;j>=0 ;j--)
       {
        rmsSum += plot->fftx->amplitudeA->dataD[j] * plot->fftx->amplitudeA->dataD[j] ;
        rms_value = sqrt(rmsSum / plot->deltat);
        plot->vect_rms->dataD[j] = rms_value;
       }
   }

 /*--- Show the plot with periodicity refreshDt ---*/

// dt = plot->gtime + plot->deltat - plot->lastrefresh_time;
 nfft_refresh = (int)(plot->refreshDt / plot->deltat);

// if (dt >= plot->refreshDt*0.99 && plot->fftx->amplitudeA != NULL)
 if (plot->nAverage%nfft_refresh == 0 && plot->fftx->amplitudeA != NULL)
    {
     /*--- If computed rms, put it in a superposed plot ---*/
     if (plot->rmsfft == ON)
       {RA_H1Pack(plot->name_rms, plot->vect_rms->dataD);}
       
     /*--- Put averaged FFT spectrum in the plot ---*/

     RA_H1Pack(plot->name, plot->fftx->amplitudeA->dataD);

     /*--- Compute the max on y axis ---*/

     if (plot->autoscale == ON)
       {DyGetMinMax(plot->fftx->amplitudeA, &localymin, &localymax);
        if (Plot[plot->idsame]->ymin > localymin) Plot[plot->idsame]->ymin = localymin;
        if (Plot[plot->idsame]->ymax < localymax && localymax>0) Plot[plot->idsame]->ymax = localymax;
        if (plot->rmsfft == ON)
           {DyGetMinMax(plot->vect_rms, &localymin, &localymax);
            if (Plot[plot->idsame]->ymin > localymin && localymin>0) Plot[plot->idsame]->ymin = localymin;
            if (Plot[plot->idsame]->ymax < localymax && localymax>0) Plot[plot->idsame]->ymax = localymax;
           }
        if (Plot[plot->idsame]->ymin > 0) RA_HSetMinimum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymin*0.9);
        RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymax*1.1);
       }

     /*--- Set time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->gtime, plot->name, plot->fft_duration, plot->nAverage, OFF, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime + plot->deltat;
    }

 /*--- Reset the averaging counter ---*/

 if (plot->nAverage == plot->maxAverage)
    {plot->nAverage = 0;
     if (plot->decayfft == OFF) plot->fftx->nDecay = 0;
    }

 return;
}


/*--------------------------------------------------- DyPlotFFTEnd -----*/

void DyPlotFFTEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/

/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors used ---*/

 DyPlotFree(iplot, OFF);
 
 return;
}


/*==================================================================================================*/
/*============================================= PLOT TIME ==========================================*/

/*---------------------------------------------------- DyPlotTIMEIni -----*/

void DyPlotTIMEIni(int iplot)
{
 int nsuperposed, i;
 char *oldname;
 struct DyPlot *plot;
 double xleg, yleg, legsize;

 plot = Plot[iplot];
 
 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters and flags ---*/

 if (plot->ymax <= plot->ymin) plot->autoscale = ON;

 /*--- When autoscale is used: set ymin and ymax ---*/

 if (plot->autoscale == ON)
   {DyGetMinMax(plot->chx->vect, &(plot->ymin), &(plot->ymax));
    if (plot->ymax <= plot->ymin)
       {plot->ymin = 1e+30;
        plot->ymax = -1e+30;
       }
    plot->ymin = 1e+30;
    plot->ymax = -1e+30;
   }

 /*--- Allocate buffer for data ---*/

 plot->buffer = FrVectNewTS(plot->chx->name, plot->chx->resampFreq, plot->xnBin, -64);
 plot->ibuf = 0;
 for (i=0; i<plot->xnBin; i++) plot->buffer->dataD[i] = -2e+30;

 /*--- Clear pad and create new plot ---*/

 if (plot->bits == ON)
    {
     /*  Keep the limits of the previous histogram in the pad if it is good  */
     oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, 32, plot->name);
     if (!oldname) RA_PadCompleteClear(plot->numpad);
     RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title, plot->xnBin, 0, plot->tsize, 32, 0., 32.);
     if (!oldname)
        {RA_SetStyle(iplot);}
    }

 else
    {
     /*  Keep the limits of the previous histogram in the pad if it is good  */
     oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 1, plot->xnBin, 0, plot->name);
     RA_H1NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title, plot->xnBin, 0., plot->tsize);
     if (!oldname)
        {if (plot->autoscale == OFF)
            {RA_HSetMinimum(plot->name, plot->ymin);
             RA_HSetMaximum(plot->name, plot->ymax);
            }
         RA_SetStyle(iplot);
        }
    }

 DyPlotSetDrawOptions (iplot);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot); 
 if (nsuperposed >= 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
 RA_HSetTitle(plot->name, plot->title);
**/

 /*--- Set a different x axis Ndivision if plot is over a long time scale ---*/

 if (plot->tsize > 3600.)  RA_SetAxisNdiv(plot->name,505);

 /*--- No stat box in this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set Units ---*/

 RA_HSetUnitsY(plot->name, DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON)
    {if (plot->bits == ON) RA_HSetUnitsY(plot->name, "bit number");
     else                  RA_HSetUnitsY(plot->name, plot->chx->vect->unitY);
    }

 return;
}


/*------------------------------------------------------ DyPlotTIME --------------*/

void DyPlotTIME(int iplot)
{
 int j;
 double deltaMinMax, localymin, localymax;
 double *buff;
 struct DyPlot *plot;
 int ibuf0;
 double lastvalue;
 int ii;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = OFF;

 /*--- Check input data ---*/

 if (plot->chx == NULL)
    {printf("No channel defined for plot %d (%s) in pad %d \n", iplot, plot->name, plot->numpad);
     return;
    }
 if (plot->chx->vect == NULL)
    {printf("No vector defined for channel %s of plot %d (%s) \n", plot->chx->name, iplot, plot->name);
     return;
    }

 /*--- Shift the buffer and  update the value of GTime ---*/

 if (plot->ibuf == 0) plot->buffer->GTime = plot->chx->vect->GTime;

 if (plot->ibuf+plot->dNbin > plot->xnBin)
   {
    if (plot->dNbin < plot->xnBin)
      {for (j=0; j<plot->xnBin-plot->dNbin; j++) plot->buffer->dataD[j] = plot->buffer->dataD[j+plot->dNbin];
       lastvalue = plot->buffer->dataD[plot->xnBin-plot->dNbin];
       for (j=plot->xnBin-plot->dNbin; j<plot->xnBin; j++) plot->buffer->dataD[j] = lastvalue;
       plot->ibuf -= plot->dNbin;
      }
    else
      {plot->ibuf = 0;}
    plot->buffer->GTime += (double)plot->dNbin / plot->chx->resampFreq;
   }

 /*--- Get new data into the buffer ---*/

 ibuf0 = plot->ibuf;

 for (j=0; j<plot->chx->vect->nData; j++)
    {plot->buffer->dataD[plot->ibuf] = plot->chx->vect->dataD[j];
     plot->ibuf++;
    }

 /*--- Put data in the plot ---*/

 buff = plot->buffer->dataD;

 if (plot->bits == ON) RA_H2PackBits(plot->name, buff);
 else                  RA_H1Pack(plot->name, buff);

 /*--- Compute min and max on y axis ---*/

 if (plot->autoscale == ON  &&  plot->bits == OFF)
   {
    DyGetMinMax(plot->buffer, &(localymin), &(localymax));
    DyPlotSetMinMax(plot, localymin, localymax);
   }

 /*--- Set time string and time axis offset and draw the plot ---*/

 DyPlotSetTime (plot->buffer->GTime, plot->name, 0, 0, ON, Dy->Frame, plot->decayfft);
 DyPlotDraw(iplot);

 return;
}


/*------------------------------------------------ DyPlotTIMEEnd -----*/

void DyPlotTIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/

 if (Dy->plotsNotDisplayed == OFF)
   {
/*
    printf("DyPlotTIMEEnd> buffer->GTime=%f  buffer->dataD[1]=%g \n", plot->buffer->GTime, plot->buffer->dataD[1]);
*/
    DyPlotSetTime (plot->buffer->GTime, plot->name, 0, 0, ON, Dy->Frame, plot->decayfft);
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    RA_PadUpdate(plot->numpad);
   }

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*=============================================== PLOT 1D ==========================================*/

/*------------------------------------------------- DyPlot1DIni -----*/

void DyPlot1DIni(int iplot)
{
 char *oldname;
 struct DyPlot *plot;
 int nsuperposed;
 double delta;
 double xleg, yleg, legsize;

 plot = Plot[iplot];
 
 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters and flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->ymax = -1e30;

 /*--- When autoedge is used: check that xnBin is even and set xmin and xmax ---*/

 if (plot->autoedge == ON)
   {if (plot->xnBin % 2 == 1)  plot->xnBin = plot->xnBin + 1;
    DyGetMinMax(plot->chx->vect, &(plot->xmin), &(plot->xmax));
    if (plot->xmin < plot->xmax)
      {delta = plot->xmax - plot->xmin;
       plot->xmin = plot->xmin - delta/10.;
       plot->xmax = plot->xmax + delta/10.;
      }
/*
    FrvRms(plot->chx->vect, &m, &r);
    plot->xmin = m-3*r;
    plot->xmax = m+3*r;
*/
    if (plot->xmax <= plot->xmin)
       {plot->xmin = -1e-15;
        plot->xmax = 1e-15;
       }
   }

 /*--- Keep the limits of the previous histogram in the pad if it is good ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, 0, 1, plot->xnBin, 0, plot->name);
 RA_H1NewReuse(plot->numpad, 0, plot->name, plot->title, plot->xnBin, plot->xmin, plot->xmax);
 if (!oldname)
    {RA_SetStyle(iplot);}

 DyPlotSetDrawOptions (iplot);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot); 
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
 else
   {RA_HSetTitle(plot->name, plot->title);}
**/

 /*--- Set stat box in this plot ---*/

 RA_HSetStat(plot->name, plot->stat);

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set Units ---*/

 RA_HSetUnitsX(plot->name, DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON)
    {RA_HSetUnitsX(plot->name, plot->chx->vect->unitY);}

 /*--- Set minimum to 0.5 (so that logy still works) ---*/

 RA_HSetMinimum(plot->name, 0.5);

 return;
}


/*--------------------------------------------------------- DyPlot1D -----------*/

void DyPlot1D(int iplot)
{
 struct DyPlot *plot;
 double *buff;
 double dt, localxmin, localxmax, ymax;
 int i;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 /*--- Do some initialization at first pas then put iFirst to OFF ---*/

 plot->iFirst = OFF;

 /*--- Check input data ---*/

 if (plot->chx == NULL)
    {printf("No channel defined for plot %d (%s) in pad %d \n", iplot, plot->name, plot->numpad);
     return;
    }
 if (plot->chx->vect == NULL)
    {printf("No vector defined for channel %s of plot %d (%s) \n", plot->chx->name, iplot, plot->name);
     return;
    }

 /*------ check the histogram x limits and auto-adjust them if needed ------*/

 if (plot->autoedge == ON)
    {
     DyGetMinMax(plot->chx->vect, &localxmin, &localxmax);
     DyPlotAutoAdjust1D(iplot, localxmin, localxmax);
     DyPlotAutoAdjust1D(plot->idsame, localxmin, localxmax);
    }

 /*--- Get input data ---*/

 for (i=0; i<plot->chx->vect->nData; i++) RA_H1Fill(plot->name, plot->chx->vect->dataD[i],1.);

 /*--- Compute the max on y axis ---*/

/*
ICI : voir si on peut en faire une fonction commune avec plotFFT  DyPlotSetMaxY
*/
 buff = (double *) calloc(plot->xnBin,sizeof(double));
 RA_H1Unpack(plot->name, buff);
 ymax = Plot[plot->idsame]->ymax;
 for (i=0; i<plot->xnBin; i++) {if (ymax < buff[i]) ymax = buff[i];}
 Plot[plot->idsame]->ymax = ymax;
 plot->ymax = ymax;
 RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymax*1.1);


 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {DyPlotSetTime (plot->gtime, plot->name, plot->gtime-Dy->startTime, 0, OFF, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 return;
}

/*------------------------------------------------ DyPlot1DEnd -----*/

void DyPlot1DEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    DyPlotSetTime (plot->gtime, plot->name, plot->deltat, 0, OFF, Dy->Frame, plot->decayfft);
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*=============================================== PLOT 2D ==========================================*/

/*---------------------------------------------------- DyPlot2DIni -----*/

void DyPlot2DIni(int iplot)
{
 char *oldname;
 struct DyPlot *plot;
 double delta;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init lastrefresh_time ---*/

 plot->lastrefresh_time = Dy->startTime;

 /*---- Set automatic update of the x and y edges if needed --------*/

 if (plot->xmax <= plot->xmin)  plot->autoedge = ON;
 if (plot->ymax <= plot->ymin)  plot->autoedge = ON;

 /*--- Check that xnBin and ynBin are even when autoedge is used ---*/

 if (plot->autoedge == ON)
    {if (plot->xnBin % 2 == 1)  plot->xnBin = plot->xnBin + 1;
     if (plot->ynBin % 2 == 1)  plot->ynBin = plot->ynBin + 1;
    }

 /*--- Get first estimate of min and max and init them with surestimations ---*/

 if (plot->autoedge == ON)
    {DyGetMinMax(plot->chx->vect, &(plot->xmin), &(plot->xmax));
     DyGetMinMax(plot->chy->vect, &(plot->ymin), &(plot->ymax));

     if (plot->xmin < plot->xmax)
       {delta = plot->xmax - plot->xmin;
        plot->xmin = plot->xmin - delta/10.;
        plot->xmax = plot->xmax + delta/10.;
       }
     if (plot->ymin < plot->ymax)
       {delta = plot->ymax - plot->ymin;
        plot->ymin = plot->ymin - delta/10.;
        plot->ymax = plot->ymax + delta/10.;
       }
    }

 /*--- Get the limits of the previous histogram in the pad if it is good  ---*/
 oldname = RA_PadContainsGoodHist(plot->numpad, 0, 2, plot->xnBin, plot->ynBin, plot->name);
 RA_H2NewReuse (plot->numpad, 0, plot->name, plot->title,
                plot->xnBin, plot->xmin, plot->xmax,
                plot->ynBin, plot->ymin, plot->ymax);
 if (!oldname)
   {RA_PadCompleteClear(plot->numpad);
    RA_SetStyle(iplot);
   }

 DyPlotSetDrawOptions (iplot);

 /*--- Set stat box in this plot ---*/

 RA_HSetStat(plot->name, plot->stat);

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set Title of the plot ---*/

 RA_HSetTitle(plot->name, plot->title);

 /*--- Set Units ---*/

 if (plot->chx->vect != NULL)
    {if (plot->chx->realunits == ON) RA_HSetUnitsX(plot->name, plot->chx->vect->unitY);
     else                            RA_HSetUnitsX(plot->name, DY_NOUNITS);
    }
 if (plot->chy->vect != NULL)
    {if (plot->chy->realunits == ON) RA_HSetUnitsY(plot->name, plot->chy->vect->unitY);
     else                            RA_HSetUnitsY(plot->name, DY_NOUNITS);
    }

 return;
}


/*------------------------------------------------------- DyPlot2D -------------*/

void DyPlot2D(int iplot)
{
 long i,j,last,nx;
 double tx, ty, dt;
 double xValue, yValue;
 double localxmin, localxmax, localymin, localymax, delta;
 double *buff;
 struct DyPlot *plot;

DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 /*--- Do some initialization at first pass, then put iFirst to OFF ---*/

 plot->iFirst = OFF;

 /*--- Check input data ---*/

 if (plot->chx == NULL)
    {printf("No channelx defined for plot %d (%s) in pad %d \n", iplot, plot->name, plot->numpad);
     return;
    }
 if (plot->chx->vect == NULL)
    {printf("No vector defined for channelx %s of plot %d (%s) \n", plot->chx->name, iplot, plot->name);
     return;
    }
 if (plot->chy == NULL)
    {printf("No channely defined for plot %d (%s) in pad %d \n", iplot, plot->name, plot->numpad);
     return;
    }
 if (plot->chy->vect == NULL)
    {printf("No vector defined for channely %s of plot %d (%s) \n", plot->chy->name, iplot, plot->name);
     return;
    }

 /*--- No superposition of plots 2D ---*/

 if (plot->superposed == ON) return;

 /*------ check the histogram limits ------*/

 if (plot->autoedge == ON)
    {
     /*--- compute localxmax, localxmin, localymax, localymin ---*/

     DyGetMinMax(plot->chx->vect, &localxmin, &localxmax);
     DyGetMinMax(plot->chy->vect, &localymin, &localymax);

     DyPlotAutoAdjust2D_X(iplot, localxmin, localxmax);
     DyPlotAutoAdjust2D_Y(iplot, localymin, localymax);
    }

 /*--- Fill the plot ---*/

 if (plot->chx->vect->nData == plot->chy->vect->nData)
   {for (i=0; i<plot->chx->vect->nData; i++)
       {xValue = plot->chx->vect->dataD[i];
        yValue = plot->chy->vect->dataD[i];
        RA_H2Fill(plot->name, xValue, yValue, 1);
       }
   }

 else if (plot->chx->vect->nData < plot->chy->vect->nData)
   {
    j = 0;
    ty = 0.;
    for (i=0; i<plot->chx->vect->nData; i++)
       {xValue = plot->chx->vect->dataD[i];
        tx = i / plot->chx->resampFreq;
        while (ty < tx && j < plot->chy->vect->nData)
          {yValue = plot->chy->vect->dataD[j];
           ty = j / plot->chy->resampFreq;
           RA_H2Fill(plot->name, xValue, yValue, 1);
           j++;
          }
       }
   }
 else if (plot->chx->vect->nData > plot->chy->vect->nData)
   {
    j = 0;
    tx = 0.;
    for (i=0; i<plot->chy->vect->nData; i++)
       {yValue = plot->chy->vect->dataD[i];
        ty = i / plot->chy->resampFreq;
        while (tx < ty && j < plot->chx->vect->nData)
          {xValue = plot->chx->vect->dataD[j];
           tx = j / plot->chx->resampFreq;
           RA_H2Fill(plot->name, xValue, yValue, 1);
           j++;
          }
       }
   }

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {DyPlotSetTime (plot->gtime, plot->name, plot->gtime-Dy->startTime, 0, OFF, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 return;
}

/*------------------------------------------------ DyPlot2DEnd -----*/

void DyPlot2DEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    DyPlotSetTime (plot->gtime, plot->name, plot->deltat, 0, OFF, Dy->Frame, plot->decayfft);
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*============================================== PLOT TRFCT ========================================*/

/*---------------------------------------------------- DyPlotTRFCTIni -----*/

void DyPlotTRFCTIni(int iplot)
{
 long nsuperposed;
 char *oldname, dummy[LSTRING_MAX], dummy1[LSTRING_MAX], dummy2[LSTRING_MAX];
 struct DyPlot *plot;
 double xleg, yleg, legsize;
 double *bufini;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;

 /*--- Initialize the FFTW buffer ---*/

 plot->tf = FrvTFNewT(NULL, plot->fft_duration, plot->maxAverage);

 if (plot->tf == NULL) {printf("DyPlotTRFCTIni: Could not get FrvTF structure \n"); return;}

 /*--- Divide the current Pad in two ---*/

 RA_SetActivePad(plot->numpad, 0);
 if (!RA_PadIsDivided(-1)  &&  plot->superposed == OFF) RA_PadDivide(1,2);

 /*--- Create or reuse Module and Phase ROOT plots ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, 1, 1, plot->xnBin/2+1, 0, plot->name);
 sprintf(dummy,"%s_Module", plot->title);
 RA_H1NewReuse(plot->numpad, 1, plot->name, dummy,
               plot->xnBin/2+1, plot->dfreq*(-.5), plot->dfreq*((double)(plot->xnBin/2+1)-0.5));

 sprintf(plot->namePhase,"%s%d",DYPLOT_PREFIX,1000+atoi(plot->name+strlen(DYPLOT_PREFIX)));
 sprintf(dummy,"%s_Phase", plot->title);
 RA_H1NewReuse(plot->numpad, 2, plot->namePhase, dummy,
               plot->xnBin/2+1, plot->dfreq*(-.5), plot->dfreq*((double)(plot->xnBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetXRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_HSetXRange (plot->namePhase, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);
    }

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Y axis range if autoscale is not set ---*/


 if (!oldname && plot->autoscale == OFF && plot->ymin > 0. && plot->ymax > 0.)
    {RA_HSetMinimum(plot->name, plot->ymin);
     RA_HSetMaximum(plot->name, plot->ymax);
    }

 /*--- If autoscale is set, start with correct values of ymin and ymax ---*/

 if (plot->autoscale == ON)
   {plot->ymin = 1e+60;
    plot->ymax = 1e-60;
   }

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot);
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
 else
   {RA_HSetTitle(plot->name, plot->title);}
**/

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- X axis units of module and phase ---*/

 RA_HSetUnitsX(plot->name, "Hz");
 RA_HSetUnitsX(plot->namePhase, "Hz");

 /*--- Y axis units of the module and phase ---*/

 strcpy(dummy1, DY_NOUNITS);
 strcpy(dummy2, DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) strcpy(dummy1, plot->chx->vect->unitY);
 if (plot->chy->vect != NULL && plot->chy->realunits == ON) strcpy(dummy2, plot->chx->vect->unitY);
 sprintf(dummy,"%s/%s",dummy1,dummy2);
 RA_HSetUnitsY(plot->name, dummy);
 RA_HSetUnitsY(plot->namePhase, "rad");

 bufini = (double *)calloc(plot->xnBin/2+1,sizeof(double));
 RA_H1Pack(plot->name, bufini);
 RA_H1Pack(plot->namePhase, bufini);
 DyPlotDraw(iplot);
 free(bufini);

 return;
}


/*-------------------------------------------------------- DyPlotTRFCT ------------*/

void DyPlotTRFCT(int iplot)
{
 int i, nfft_refresh;
 double localymin, localymax;
 double dt;
 struct DyPlot *plot;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Exclude missing samples from TRFCT computation ---*/

 if (plot->chx->status != ON || plot->chx->vect == NULL || plot->chx->vect->next != NULL)
   {printf("DyPlotTRFCT> Input vector %s at time %f has missing or corrupted samples. Excluding it from TRFCT computation\n",
           plot->chx->name, plot->gtime);
    return;
   }
 if (plot->chy->status != ON || plot->chy->vect == NULL || plot->chy->vect->next != NULL)
   {printf("DyPlotTRFCT> Input vector %s at time %f has missing or corrupted samples. Excluding it from TRFCT computation\n",
           plot->chy->name, plot->gtime);
    return;
   }

 /*--- Remove any missing sample information kept from previous iteration ---*/

 if (plot->tf->modulus != NULL && plot->tf->modulus->next != NULL)
   {for (i=0; i< plot->tf->modulus->nData; i++) plot->tf->modulus->next->data[i] = 1;}

 /*--- Compute the TRFCT ---*/

 FrvTFProc(plot->tf, plot->chx->vect, plot->chy->vect);
 plot->nAverage++;

 /*--- Show the plot with periodicity refreshDt ---*/

// dt = plot->gtime + plot->deltat - plot->lastrefresh_time;
 nfft_refresh = (int)(plot->refreshDt / plot->deltat);

// if (dt >= plot->refreshDt && plot->tf->modulus != NULL && plot->tf->phase != NULL)
 if (plot->nAverage%nfft_refresh == 0 && plot->tf->modulus != NULL && plot->tf->phase != NULL)
    {
     /*--- Put averaged FFT spectrum in the plot ---*/

     RA_H1Pack(plot->name, plot->tf->modulus->dataD);
     RA_H1Pack(plot->namePhase, plot->tf->phase->dataD);

     /*--- Compute the max on y axis ---*/

     if (plot->autoscale == ON)
       {DyGetMinMax(plot->tf->modulus, &localymin, &localymax);
        if (Plot[plot->idsame]->ymin > localymin && localymin>0) Plot[plot->idsame]->ymin = localymin;
        if (Plot[plot->idsame]->ymax < localymax && localymax>0) Plot[plot->idsame]->ymax = localymax;
        RA_HSetMinimum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymin*0.9);
        RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymax*1.1);
       }

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->gtime, plot->name, plot->fft_duration, plot->nAverage, OFF, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- Reset the averaging counter ---*/

 if (plot->nAverage == plot->maxAverage)
    {plot->nAverage = 0;
     if (plot->decayfft == OFF) FrvTFReset(plot->tf);
    }

 return;
}


/*--------------------------------------------------- DyPlotTRFCTEnd -----*/

void DyPlotTRFCTEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, 1);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    RA_SetActivePad(plot->numpad, 2);
    RA_HDraw(plot->namePhase,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);
 
 return;
}


/*==================================================================================================*/
/*============================================== PLOT COHE =========================================*/

/*---------------------------------------------------- DyPlotCOHEIni -----*/

void DyPlotCOHEIni(int iplot)
{
 long nsuperposed;
 char *oldname, dummy[LSTRING_MAX];
 struct DyPlot *plot;
 double xleg, yleg, legsize;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;

 /*--- Initialize the FFTW buffer ---*/

 plot->tf = FrvTFNewT("C", plot->fft_duration, plot->maxAverage);
 if (plot->tf == NULL) {printf("DyPlotCOHETIni: Could not get FrvTF strcture \n"); return;}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 1, plot->xnBin/2+1, 0, plot->name);
 RA_H1NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin/2+1, plot->dfreq*(-.5), plot->dfreq*((double)(plot->xnBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetXRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);
    }

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Y axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->ymin > 0. && plot->ymax > 0.)
    {RA_HSetMinimum(plot->name, plot->ymin);
     RA_HSetMaximum(plot->name, plot->ymax);
    }

 /*--- If autoscale is set, start with correct values of ymin and ymax ---*/

 if (plot->autoscale == ON)
   {plot->ymin = 1e+30;
    plot->ymax = 1e-30;
   }

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot);
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
 else
   {RA_HSetTitle(plot->name, plot->title);}
**/

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- X axis units of a spectrum ---*/

 RA_HSetUnitsX(plot->name, "Hz");

 /*--- Y axis units of the spectrum ---*/

/*
 sprintf(dummy,"%s/sqrt(Hz)", DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) sprintf(dummy,"%s/sqrt(Hz)", plot->chx->vect->unitY);
*/
 sprintf(dummy,"%s", DY_NOUNITS);
 RA_HSetUnitsY(plot->name, dummy);

 return;
}


/*--------------------------------------------------------- DyPlotCOHE -----------*/

void DyPlotCOHE(int iplot)
{
 double localymin, localymax;
 double dt;
 struct DyPlot *plot;
 int nfft_refresh;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Exclude missing samples from COHE computation ---*/

 if (plot->chx->status != ON || plot->chx->vect == NULL || plot->chx->vect->next != NULL)
   {printf("DyPlotCOHE> Input vector %s at time %f has missing or corrupted samples. Excluding it from COHE computation\n",
           plot->chx->name, plot->gtime);
    return;
   }
 if (plot->chy->status != ON || plot->chy->vect == NULL || plot->chy->vect->next != NULL)
   {printf("DyPlotCOHE> Input vector %s at time %f has missing or corrupted samples. Excluding it from COHE computation\n",
           plot->chy->name, plot->gtime);
    return;
   }

 /*--- Compute the COHErence ---*/

 FrvTFProc(plot->tf, plot->chx->vect, plot->chy->vect);
 plot->nAverage++;
/**
 if (plot->chx->vect != NULL && plot->chx->vect->next == NULL &&
     plot->chy->vect != NULL && plot->chy->vect->next == NULL)
   {FrvTFProc(plot->tf, plot->chx->vect, plot->chy->vect);
    plot->nAverage++;
   }
**/

 /*--- Show the plot only with periodicity refreshDt ---*/

// dt = plot->gtime + plot->deltat - plot->lastrefresh_time;
 nfft_refresh = (int)(plot->refreshDt / plot->deltat);

// if (dt >= plot->refreshDt && plot->tf->coherence != NULL)
 if (plot->nAverage%nfft_refresh == 0 && plot->tf->coherence != NULL)
    {
     /*--- Put averaged Coherence amplitude in the plot ---*/

     RA_H1Pack(plot->name, plot->tf->coherence->dataD);

     /*--- Compute the max on y axis ---*/

     if (plot->autoscale == ON)
       {DyGetMinMax(plot->tf->coherence, &localymin, &localymax);
        if (Plot[plot->idsame]->ymin < localymin && localymin>0) Plot[plot->idsame]->ymin = localymin;
        if (Plot[plot->idsame]->ymax < localymax && localymax>0) Plot[plot->idsame]->ymax = localymax;
        RA_HSetMinimum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymin*0.9);
        RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->ymax*1.1);
       }

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->gtime, plot->name, plot->fft_duration, plot->nAverage, OFF, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- Reset the averaging counter ---*/

 if (plot->nAverage == plot->maxAverage)
    {plot->nAverage = 0;
     if (plot->decayfft == OFF) FrvTFReset(plot->tf);
    }

 return;
}


/*--------------------------------------------------- DyPlotCOHEEnd -----*/

void DyPlotCOHEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*============================================== PLOT AUDIO ========================================*/

/*--------------------------------------------------------- DyPlotAUDIOIni ----------*/

void DyPlotAUDIOIni(int iplot)
{
 int tmp, sample_rate, j;
 double value, delta;
 char *str;
 struct DyPlot *plot;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[iplot];

 plot->iFirst = ON;

 /*--- Set new sampling frequency if needed ---*/

 plot->audio_resampFactor = 1;
 if (plot->chx->resampFreq < 5512.)
    {
     while (plot->audio_resampFactor * plot->chx->resampFreq < 5512.) plot->audio_resampFactor++;
     printf("DyPlotAUDIOIni> WARNING: Sampling frequency of %.2f Hz is too low for sound playing\n",plot->chx->resampFreq);
     printf("DyPlotAUDIOIni> Data will be interpolated to %.2f Hz to be in allowed frequency band: 5512 to 44100 Hz\n", plot->chx->resampFreq*plot->audio_resampFactor);
    }

 /*--- Set the min and max to normalize the audio output ---*/

 plot->xmin = plot->chx->vect->dataD[0];
 plot->xmax = plot->chx->vect->dataD[0];
 for (j=0; j<plot->chx->vect->nData; j++)
    {value = plot->chx->vect->dataD[j]; 
     plot->xmax = (value > plot->xmax) ? value : plot->xmax;
     plot->xmin = (value < plot->xmin) ? value : plot->xmin;
    }
 delta = plot->xmax - plot->xmin;
 plot->xmax = plot->xmax + delta/2;
 plot->xmin = plot->xmin - delta/2;
 plot->audio_delta = plot->xmax - plot->xmin;
 if (plot->audio_delta == 0) plot->audio_delta = 1;
printf("================== xmin=%g xmax=%g \n", plot->xmin, plot->xmax);
printf("delta=%g \n", plot->audio_delta);

 /*--- Initialise ---*/

 plot->audio_begin_nb = (long)(plot->audio_t0 * plot->chx->resampFreq);
 plot->audio_end_nb   = plot->audio_begin_nb + (long)(plot->audio_duration * plot->chx->resampFreq);
 plot->audio_itime    = 0;
 plot->audio_nsamp    = 0;
 plot->audio_fout     = NULL;
 plot->audio_fd       = 0;
 if (plot->sound16bits == ON) plot->audio_encoding = 16;
 else                         plot->audio_encoding = 8;
 plot->audio_data_nBytes = (unsigned long)(plot->audio_duration * plot->chx->resampFreq) * plot->audio_encoding/8;

 /*--- Audio output file ---*/

 sprintf(plot->audio_outfileName,"%s.wav", plot->chx->name);
 if (strlen(plot->audio_outfileName) >= 2)
    {str = strstr(plot->audio_outfileName,":");
     if (str) strcpy(plot->audio_outfileName, str+1);
    }

 printf("DyPlotAUDIOIni> t0=%g  duration=%g  resampFreq=%g  begin_nb=%ld  end_nb=%ld  encoding=%d bits  data_nBytes=%ld"
        "  AUDIO_BUF_SIZE=%d  outfileName=%s\n",
         plot->audio_t0, plot->audio_duration, plot->chx->resampFreq, plot->audio_begin_nb, plot->audio_end_nb, plot->audio_encoding, plot->audio_data_nBytes
, AUDIO_BUF_SIZE, plot->audio_outfileName);


 /*--- Set audio device for audio online output stream ---*/

 if (plot->soundonline == ON) return;

 if (plot->audio_encoding == 16)
    {
     if ((plot->audio_fd = open ("/dev/audio", O_WRONLY)) < 0)
        {printf("DyPlotAUDIOIni: could not open audio device\n");
         plot->audio_fd = 0;
        }
    }
 else
    {
     if ((plot->audio_fd = open ("/dev/dsp", O_WRONLY)) < 0)
        {printf("DyPlotAUDIOIni: could not open dsp audio device\n");
         plot->audio_fd = 0;
        }
    }

 if (plot->audio_fd)
    {
     if (plot->audio_encoding == 16)
        {tmp = AFMT_S16_NE;
         if (ioctl (plot->audio_fd, SNDCTL_DSP_SETFMT, &tmp)==-1) {perror("SNDCTL_DSP_SETFMT"); exit(-1);}
         if (tmp != AFMT_S16_NE)
            {printf("DyPlotAUDIOIni: The device doesn't support the 16 bit sample format.\n");
             plot->audio_fd = 0;
             return;
            }
        }
     printf("DyPlotAUDIOIni: audio device format set to %d bits\n", plot->audio_encoding);

     tmp = 1;
     if (ioctl (plot->audio_fd, SNDCTL_DSP_CHANNELS, &tmp)==-1) {perror("SNDCTL_DSP_CHANNELS"); exit(-1);}
     if (tmp != 1) {printf("DyPlotAUDIOIni: The device doesn't support mono mode.\n"); exit(-1);}

     sample_rate = plot->chx->resampFreq * plot->audio_resampFactor;
     if (ioctl (plot->audio_fd, SNDCTL_DSP_SPEED, &sample_rate)==-1) {perror("SNDCTL_DSP_SPEED"); exit(-1);}
     printf("DyPlotAUDIOIni> audio device sample rate set to %d \n", sample_rate);
    }

 return;
}


/*-------------------------------------------------------- DyPlotAUDIO ------------*/

void DyPlotAUDIO(int iplot)
{
 struct sample_16bits
    {
     unsigned char lsb;
     unsigned char msb;
    };
 union out_sample
    {
     unsigned short out_short;
     struct sample_16bits by;
    };
 int nwritten, z, j;
 double value, valueBuf, valueBuf2, timeNeeded;
 struct soundhdr hdr;
 union out_sample *pt;
 union out_sample pt_address;
 struct DyPlot *plot;
 struct timespec tv;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 if (plot->hidden == ON) return;

 pt = &pt_address;

 for (j=0; j<plot->chx->vect->nData; j++)
    {
     value = plot->chx->vect->dataD[j];

     valueBuf = (value - plot->xmin) / plot->audio_delta * (pow(2.0,(double)plot->audio_encoding) - 1.) + 0.5001;
/*
     pt->out_short = (unsigned short)((float)valueBuf);
if (plot->audio_itime%20001 == 0) printf("========== value=%g  valueBuf=%g  xmin=%g xmax=%g  outshort=%d \n", value, valueBuf, plot->xmin, plot->xmax, pt->out_short); 
*/
     /*--- Needed to linearly interpolate for resampling a too low frequency signal ---*/

     plot->audio_previousValueBuf = valueBuf;

    /*--- Start audio file recording only after begin_nb ---*/

    if (plot->audio_itime == plot->audio_begin_nb)
       {printf("DyPlotAUDIO> START AUDIO FILE RECORDING at itime=%ld\n", plot->audio_itime);
        plot->audio_fout = fopen(plot->audio_outfileName,"w");
        if (plot->audio_fout == NULL) printf("DyPlotAUDIO> output file %s could not be opened\n", plot->audio_outfileName);
        DySoundFillHeader(&hdr,(long)(plot->chx->resampFreq * plot->audio_resampFactor), plot->audio_data_nBytes, (short)(plot->audio_encoding));
        DySoundReadHeader((char *)(&hdr));
        if (plot->audio_fout) fwrite(&hdr, 1, sizeof(hdr), plot->audio_fout);
       }

    /*--- Loop for oversampling if needed ---*/

    for (z=0; z<plot->audio_resampFactor; z++)
       {
        /*--- Linearly interpolate for resampling a too low frequency signal ---*/

        if (plot->audio_resampFactor > 1)
           {valueBuf2 = ((plot->audio_resampFactor - z) * plot->audio_previousValueBuf + z * valueBuf) / plot->audio_resampFactor;
            pt->out_short = (unsigned short)((float)valueBuf2);
           }

        /*--- Put data in buffer for audio stream and in output file if requested ---*/

        if (plot->audio_encoding == 16)
           {if (plot->audio_fd) plot->audio_bufstreamS[plot->audio_nsamp] = pt->out_short;
            if (plot->audio_fout && z == 0) {fprintf(plot->audio_fout,"%c",pt->by.lsb); fprintf(plot->audio_fout,"%c",pt->by.msb);}
           }
        else
           {if (pt->by.msb != 0)
              {printf("DyPlotAUDIO> Error in char conversion for sample %d  outshort=%d  valueBuf=%g \n",
                       z, pt->out_short, valueBuf);
              }
            if (plot->audio_fd) plot->audio_bufstream[plot->audio_nsamp] = pt->by.lsb;
            if (plot->audio_fout && z == 0) fprintf(plot->audio_fout,"%c",pt->by.lsb);
           }

        if (plot->audio_fd)
          {if (plot->audio_nsamp == 0) DyDebugTimeNeeded ("DyPlotAUDIO", "start");
           plot->audio_nsamp++;
          }


        /*--- Output data to audio device each time the buffer is full ---*/

        if (plot->audio_fd && plot->audio_nsamp == AUDIO_BUF_SIZE)
          {
           if (plot->audio_encoding == 16)
             {if ((nwritten = write (plot->audio_fd, (char *)plot->audio_bufstreamS, 2*AUDIO_BUF_SIZE)) < 0) perror ("/dev/audio");}
           else
             {if ((nwritten = write (plot->audio_fd, plot->audio_bufstream, AUDIO_BUF_SIZE)) < 0) perror ("/dev/dsp");}
           timeNeeded = DyDebugTimeNeeded ("DyPlotAUDIO", "stop");
           if (plot->audio_nsamp > 5512.  &&  timeNeeded < plot->audio_nsamp / (plot->chx->resampFreq * plot->audio_resampFactor))
             {timeNeeded = 100000. * 0.8 * plot->audio_nsamp / (plot->chx->resampFreq * plot->audio_resampFactor) - timeNeeded;
              printf("DyPlotAUDIO> Waiting %g microseconds to align plots update on audio buffering speed\n", timeNeeded);
              tv.tv_sec = (int)timeNeeded;
              tv.tv_nsec = (int)(timeNeeded * 1000000000);
              nanosleep(&tv,NULL);
             }
           plot->audio_nsamp = 0;
          }
       }

    /*--- Stop audio data recording in file ---*/

    plot->audio_itime++;
if (j==0) printf("audio_itime=%ld  end_nb=%ld  begin_nb=%ld  \n", plot->audio_itime, plot->audio_end_nb, plot->audio_begin_nb);
   if (plot->audio_fout != NULL && plot->audio_itime >= plot->audio_end_nb)
      {
       printf("DyPlotAUDIO> END AUDIO FILE RECORDING at itime=%ld: %ld samples recorded\n", plot->audio_itime, plot->audio_itime-plot->audio_begin_nb);
       fclose(plot->audio_fout);
       plot->audio_fout = NULL;
      }

   } /* end of loop in plot->chx->vect->dataD */

 return;
}


/*--------------------------------------------------------- DyPlotAUDIOEnd ----------*/

void DyPlotAUDIOEnd(int iplot)
{
char cmd_play[2048];
 char cmd_volume[256];
 struct soundhdr hdr;
 long last;
 struct DyPlot *plot;

 plot = Plot[iplot];

 if (plot->hidden == ON) return;

 DyDebugPrint(__FUNCTION__,0);

 if (plot->audio_fd > 0) close(plot->audio_fd);

 if (plot->audio_itime < plot->audio_end_nb && plot->audio_fout != NULL)
    {last = ftell(plot->audio_fout);
     rewind(plot->audio_fout);
     DySoundFillHeader(&hdr,(long)(plot->chx->resampFreq * plot->audio_resampFactor), plot->audio_itime*plot->audio_encoding/8, (short
)plot->audio_encoding);
     fwrite(&hdr, 1, 44, plot->audio_fout);
     fseek(plot->audio_fout, last, SEEK_SET);
     printf("DyAUDIOEnd> END AUDIO FILE RECORDING: only %.2f sec (%ld samples) recorded at %.2f Hz\n",
            (plot->audio_itime-plot->audio_begin_nb)/plot->chx->resampFreq,
             (plot->audio_itime-plot->audio_begin_nb) * plot->audio_resampFactor, plot->chx->resampFreq * plot->audio_resampFactor);
    }

 if (plot->audio_fout) fclose(plot->audio_fout);
 plot->audio_fout = NULL;

 /*--- Run audioplay with right parameters ---*/

 sprintf(cmd_play,"%s -r %.2f %s", AUDIOPLAY, plot->chx->resampFreq*plot->audio_resampFactor, plot->audio_outfileName);
 sprintf(cmd_volume,"%s &", AUDIOCONTROL);

 printf("DyAUDIOEnd> An audio file has been created. You can play it with the following command:\n");
 printf("DyAUDIOEnd> play_command = %s\n",cmd_play);
 printf("DyAUDIOEnd> volume_control = %s\n",cmd_volume);

 return;
}


/*==================================================================================================*/
/*============================================= PLOT FFTTIME =======================================*/

/*------------------------------------------------------- DyPlotFFTTIMEIni -------------*/

void DyPlotFFTTIMEIni(int iplot)
{
 long nsuperposed;
 int step_ivect;
 char *oldname, dummy[LSTRING_MAX];
 struct DyPlot *plot;
 double xleg, yleg, legsize;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;
 plot->itime    = 0;
 plot->tmin     = Dy->startTime;

 /*--- Initialize the input buffer ---*/

/** ICI
printf("++++ plot->fft_deltat=%g  plot->chx->vect->dx[0]=%g  step_ivect=%d  plot->chx->vect->dx[1]=%g\n",
plot->fft_deltat, plot->chx->vect->dx[0], step_ivect, plot->chx->vect->dx[1]);
**/

 step_ivect = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 plot->frvbufx = FrvBufNew(plot->ynBin, step_ivect, FR_VECT_8R, -1, 0);

 /*--- Initialize the FFTW buffer ---*/
 
 plot->fftx = FrvRFFTNew("AHNS", plot->ynBin, 1);
 if (plot->fftx == NULL) {printf("DyPlotFFTTIMEIni: Could not get fftx buffer\n"); return;}

 /*--- Initialize the time-frequency buffer ---*/

 plot->buff = (double *) calloc(plot->xnBin*(plot->ynBin/2+1), sizeof(double));
 if (plot->buff == NULL)
    {printf("DyPlotFFTTIMEIni> plot->buff calloc (%ldx%ld doubles) failed \n", plot->xnBin, plot->ynBin/2+1);}

 /*--- Set the decay time for a moving average ---*/

 FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);
 if (plot->decayfft == ON)
    {FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin/2+1, plot->name);
 RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin, 0, plot->xnBin * plot->dtime,
               plot->ynBin/2+1,
               plot->dfreq*(-0.5), plot->dfreq*((double)(plot->ynBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetYRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);
    }

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Z axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->zmin > 0. && plot->zmax > 0.)
    {RA_HSetMinimum(plot->name, plot->zmin);
     RA_HSetMaximum(plot->name, plot->zmax);
    }

 /*--- If autoscale is ON, init the zmin and zmax values ---*/

 if (plot->autoscale == ON)
   {plot->zmin =   DY_BADDATA;
    plot->zmax = - DY_BADDATA;
   }

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set Title of the plot ---*/

 RA_HSetTitle(plot->name, plot->title);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot);
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }
**/

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Y axis units of a time-frequency plot ---*/

 RA_HSetUnitsY(plot->name, "Hz");

 /*--- Z axis units of the time-frequency plot ---*/

 sprintf(dummy,"%s/sqrt(Hz)", DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) sprintf(dummy,"%s/sqrt(Hz)", plot->chx->vect->unitY);
 RA_HSetUnitsZ(plot->name, dummy);

 return;
}


/*------------------------------------------------------- DyPlotFFTTIME -------------*/

void DyPlotFFTTIME(int iplot)
{
 struct DyPlot *plot;
 int step_ivect, nbuf;
 int ii;
 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Put in buffer the input vector and Compute the FFT for each sub vector ---*/

 step_ivect = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("DyPlotFFTTIME> step_ivect=%d  ynBin=%d  nData=%d  fft_deltat=%g deltat=%g \n",
            step_ivect, plot->ynBin, plot->chx->vect->nData, plot->fft_deltat, plot->deltat);
   }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
    {printf("DyPlotFFTTIME> Put in buffer vect=%s  GTime=%f nData=%d \n",
             plot->chx->vect->name, plot->chx->vect->GTime, plot->chx->vect->nData);
    }

 FrvBufFeed(plot->frvbufx, plot->chx->vect);

 nbuf = 1;
 while (FrvBufGetNext(plot->frvbufx) == 0 && Dy->Order != STOP)
   {
    if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
      {printf("DyPlotFFTTIME> Extract from buffer subvect %d/%d GTime=%f nData=%d (vect->nData=%d step_ivect=%d)\n",
              nbuf, plot->chx->vect->nData/step_ivect, plot->frvbufx->output->GTime, plot->frvbufx->output->nData, plot->chx->vect->nData, step_ivect);
      }
    DyPlotFFTTIMEPerVect(iplot, plot->frvbufx->output);
    nbuf++;
   }

 return;
}


/*------------------------------------------------------- DyPlotFFTTIMEPerVect -------------*/

void DyPlotFFTTIMEPerVect(int iplot, struct FrVect *subvect)
{
 int i, j, ib, nx, nx2;
 double dt;
 struct DyPlot *plot;
 double *buffback, *tempv, *median;
 double tv;
 int k;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Compute the FFT only if no missing data ---*/

 if (subvect != NULL && subvect->next == NULL)
    FrvRFFTFor(plot->fftx, subvect);

 /*--- Average the FFTs until maxAverage is reached ---*/

 plot->nAverage++;
 if (plot->nAverage < plot->maxAverage) return;

 /*--- Fill the buffer with this new computed FFT ---*/

 nx = plot->xnBin;
 for (j=0 ;j<plot->ynBin/2+1 ;j++)
    {ib = plot->itime + nx*j;
     plot->buff[ib] = plot->fftx->amplitudeA->dataD[j];
     if (plot->autoscale == ON)
       {if (plot->buff[ib] > 1e-30 && plot->zmin > plot->buff[ib]) plot->zmin = plot->buff[ib];
        if (plot->buff[ib] > 0 && plot->zmax < plot->buff[ib]) plot->zmax = plot->buff[ib];
       }
    }
 if (plot->median_norm == ON) {plot->zmin = 0.25; plot->zmax = 4;}

 /*--- Increment the time axis bin number ---*/

 plot->itime++;

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->fft_deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {
     /*--- Optionally apply a normalization by a median spectrum ---*/

     if (plot->median_norm == ON)
       {
        nx = plot->xnBin;
        nx2 = plot->itime;
        buffback = (double *)malloc(nx * (plot->ynBin/2+1) * sizeof(double));
        tempv    = (double *)malloc(nx2 * sizeof(double));
        median   = (double *)malloc((plot->ynBin/2+1) * sizeof(double));

        for (j=0; j<plot->ynBin/2+1; j++)
           {
            for (i=0; i<nx2; i++) tempv[i] = plot->buff[i+nx*j];
            for (i=0; i<nx2-1; i++)
               {for (k=i+1; k<nx2; k++)
                   {if (tempv[i] > tempv[k]) {tv = tempv[i]; tempv[i] = tempv[k]; tempv[k] = tv;}}
               }
            median[j] = tempv[nx2/2];
/*
            median[j] = Dy_quickselect(tempv,nx2,nx2/2);
*/
           }
        for (j=0; j<plot->ynBin/2+1; j++)
           {for (i=0; i<nx; i++)
              {ib = i + nx*j;
               buffback[ib] = plot->buff[ib];
               plot->buff[ib] = plot->buff[ib] / median[j];
               if (plot->buff[ib] < 0.25) plot->buff[ib] = 0.25;
               if (plot->buff[ib] > 4)    plot->buff[ib] = 4;
              }
           }
         free(tempv);
         free(median);
        }

     /*--- Put the time-frequency buffer in the plot ---*/

     RA_H2Pack(plot->name, plot->buff);

     /*--- Put back un-normalized values of spectrogram in buff ---*/

     if (plot->median_norm == ON)
       {
        for (j=0; j<plot->ynBin/2+1; j++)
           {for (i=0; i<nx; i++)
              {ib = i + nx*j;
               plot->buff[ib] = buffback[ib];
              }
           }
         free(buffback);
        }

     /*--- Set the min and max on z axis ---*/

     if (plot->autoscale == ON && plot->zmin > 0.) RA_HSetMinimum(plot->name, plot->zmin);
     if (plot->autoscale == ON && plot->zmax > 0.) RA_HSetMaximum(plot->name, plot->zmax);

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->tmin, plot->name, plot->fft_duration, plot->maxAverage, ON, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- Reset the counter of averages ---*/

 plot->nAverage = 0;
 if (plot->decayfft == OFF) plot->fftx->nDecay = 0;

 /*--- End of time window reached, then shift the buffer content by one dtime step ---*/

 if (plot->itime == plot->xnBin)
   {
    plot->itime--;
    plot->tmin += plot->dtime;
    nx = plot->xnBin;
    for (i=0; i<nx-1; i++)
       {for (j=0; j<plot->ynBin/2+1; j++)
           {ib = i + j*nx;
            plot->buff[ib] = plot->buff[ib+1];
           }
       }
    for (j=0; j<plot->ynBin/2+1; j++)
       {ib = nx-1+nx*j;
        plot->buff[ib] = plot->buff[ib-1];
       }
   }
/**
 else
   {
    for (i=plot->itime; i<nx; i++)
       {for (j=0; j<plot->ynBin/2+1; j++)
           {ib = i + j*nx;
            plot->buff[ib] = plot->buff[plot->itime-1];
           }
       }
   }
**/

 return;
}

/*------------------------------------------------------ DyPlotFFTTIMEEnd --------------*/

void DyPlotFFTTIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/

 DyPlotFFTTIME(iplot);

 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }

 /*--- Free the vectors and buffers sed ---*/

 DyPlotFree(iplot, OFF);
 
 return;
}


/*==================================================================================================*/
/*=============================================== PLOT RAW =========================================*/

/*---------------------------------------------------- DyPlotRAWIni -----*/

void DyPlotRAWIni(int iplot)
{
 int nsuperposed, nBin;
 char *oldname;
 struct DyPlot *plot;
 double xleg, yleg, legsize;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst   = ON;

 /*--- Init some counters and flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;
 if (plot->ymax <= plot->ymin) plot->autoscale = ON;

 /*--- When autoscale is used: set ymin and ymax ---*/

 if (plot->autoscale == ON)
   {
    DyGetMinMax(plot->chx->vect, &(plot->zmin), &(plot->zmax));
    if (plot->zmax <= plot->zmin)
      {plot->zmin = 1e+30;
       plot->zmax = -1e+30;
      }
   }

 /*--- Fix the format of vector if it contains an image or a series of images ---*/

 FrVectFixVirgoImage(plot->chx->vect);

 /*--- Set the flag saying it is an image or not ---*/

 plot->raw2d = OFF;
 if (plot->chx->vect->nData > plot->chx->vect->nx[0]) plot->raw2d = ON;

 /*--- Set some secondary parameters ---*/

 if (plot->raw2d == ON)
   {plot->xnBin = plot->chx->vect->nx[1];
    plot->xmin  = plot->chx->vect->startX[1];
    plot->xmax  = plot->xmin + plot->chx->vect->dx[1] * plot->chx->vect->nx[1];
    plot->ynBin = plot->chx->vect->nx[2];
    plot->ymin  = plot->chx->vect->startX[2];
    plot->ymax  = plot->ymin + plot->chx->vect->dx[2] * plot->chx->vect->nx[2];
   }
 else
   {plot->xnBin = plot->chx->vect->nx[0];
    plot->xmin  = plot->chx->vect->startX[0];
    plot->xmax  = plot->xmin + plot->chx->vect->dx[0] * plot->chx->vect->nx[0];
   }

 /*--- Allocate buffer for data ---*/

 if (plot->raw2d == ON) nBin = plot->xnBin * plot->ynBin;
 else                   nBin = plot->xnBin;

 plot->buff = (double *)calloc(nBin, sizeof(double));
 if (plot->buff == NULL) printf("DyPlotRAWni> plot->buff calloc (%ldx%ld doubles ) failed\n", plot->xnBin, plot->ynBin);

 /*--- Clear pad and create new plot ---*/

 if (plot->raw2d == ON)
    {
     oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin, plot->name);
     if (!oldname) RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
                   plot->xnBin, plot->xmin, plot->xmax, plot->ynBin, plot->ymin, plot->ymax);
     if (!oldname)
        {
         RA_PadCompleteClear(plot->numpad);
         if (plot->autoscale == OFF)
           {RA_HSetMinimum(plot->name, plot->zmin);
            RA_HSetMaximum(plot->name, plot->zmax);
           }
         RA_SetStyle(iplot);
        }
    }

 else
    {
     oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 1, plot->xnBin, 0, plot->name);
     RA_H1NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
                   plot->xnBin, plot->xmin, plot->xmax);
     if (!oldname)
        {if (plot->autoscale == OFF)
            {RA_HSetMinimum(plot->name, plot->zmin);
             RA_HSetMaximum(plot->name, plot->zmax);
            }
         RA_SetStyle(iplot);
        }
    }

 DyPlotSetDrawOptions (iplot);

 /*--- Set legend if several plots are superposed ---*/
/**
 nsuperposed = DyPlotGetNsuperposed(iplot); 
 if (nsuperposed > 0)
    {RA_SetActivePad(plot->numpad, plot->numsubpad);
     DyPlotSetLegendSize(iplot,&xleg, &yleg, &legsize);
     RA_HSetLegend(plot->name, plot->title, plot->superposed, nsuperposed, xleg, yleg, legsize);
    }  
 else
    {RA_HSetTitle(plot->name, plot->title);}
**/

 /*--- No stat box in this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Set color of the plot ---*/

 DyPlotSetColor(iplot);

 /*--- Set Units ---*/

 RA_HSetUnitsX(plot->name, DY_NOUNITS);
 RA_HSetUnitsY(plot->name, DY_NOUNITS);

 if (plot->chx->vect != NULL && plot->chx->realunits == ON)
    {
     if (plot->raw2d == ON)
       {
        RA_HSetUnitsX(plot->name, plot->chx->vect->unitX[1]);
        RA_HSetUnitsY(plot->name, plot->chx->vect->unitX[2]);
        RA_HSetUnitsZ(plot->name, plot->chx->vect->unitY);
       }
     else
       {
        RA_HSetUnitsX(plot->name, plot->chx->vect->unitX[0]);
        RA_HSetUnitsY(plot->name, plot->chx->vect->unitY);
       }
    }

 return;
}


/*------------------------------------------------------- DyPlotRAW ------------*/

void DyPlotRAW(int iplot)
{
 int i, im, nimage, nBin, j;
 double origin, value, localmin, localmax, dt;
 struct DyPlot *plot;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = OFF;

 /*--- Fix the format of vector if it contains an image or a series of images ---*/

 FrVectFixVirgoImage(plot->chx->vect);

 /*--- Set the total number of element of the buffer ---*/

 if (plot->raw2d == ON) nBin = plot->xnBin * plot->ynBin;
 else                   nBin = plot->xnBin;

 /*--- Fill the buffer with an average of the images ---*/

 nimage = plot->chx->vect->nData / nBin;

 for (im=0; im<nimage; im++)
    {
/* TOREMOVE */
/* ICI
 printf("im=%d/%d  nAverage=%d/%d \n", im, nimage, plot->nAverage, plot->maxAverage);
*/
     /*--- For now, we take only the first image ---*/
/**
     if (plot->raw2d == ON && im > 0) {origin += im*plot->chx->vect->dx[0]; continue;}
**/

     for (i=0; i<nBin; i++)
        {
         value = FrVectGetV(plot->chx->vect, i+im*nBin);
         if (plot->chx->vect->type == FR_VECT_C && value < 0) value += 256.;
         if (plot->x2av == ON) plot->buff[i] += value*value;
         else                  plot->buff[i] += value;
        }
     plot->nAverage++;

     /*--- Normalize data before putting in the plot ---*/
     for (i=0; i<nBin; i++)
        {if (plot->x2av == ON) plot->buff[i] = sqrt(plot->buff[i] / plot->nAverage);
         else                  plot->buff[i] = plot->buff[i] / plot->nAverage;
        }

     /*--- Set min and max of the buffer content ---*/

     if (plot->autoscale == ON)
       {
        localmin =  DY_BADDATA;
        localmax = -DY_BADDATA;
        for (j=0; j<nBin; j++)
           {if (j == nBin-1 && plot->buff[j] == 0.) continue;
            if (localmin > plot->buff[j]  &&  plot->buff[j] != 0.) localmin = plot->buff[j];
            if (localmax < plot->buff[j])                          localmax = plot->buff[j];
           }
       }

     /*---Show the plot with periodicity refreshDt ---*/

     dt = plot->gtime + plot->deltat - plot->lastrefresh_time;
/*
     printf("plot %s : gtime=%f  lastrefresh_time=%f  deltat=%g  dt=%g  refreshDt=%g \n",
             plot->title, plot->gtime, plot->lastrefresh_time, plot->deltat, dt, plot->refreshDt);
*/
     if (dt >= plot->refreshDt)
       {
        if (plot->raw2d == ON) RA_H2Pack(plot->name, plot->buff);
        else                   RA_H1Pack(plot->name, plot->buff);

        /*--- Compute the min and max on y or z axis ---*/
        if (plot->autoscale == ON)
          {
           if (Plot[plot->idsame]->zmin > localmin) Plot[plot->idsame]->zmin = localmin;
           if (Plot[plot->idsame]->zmax < localmax) Plot[plot->idsame]->zmax = localmax;
//           RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->zmin);
//           RA_HSetMaximum(Plot[plot->idsame]->name, Plot[plot->idsame]->zmax);
           RA_HSetMaximum(Plot[plot->idsame]->name, localmin);
           RA_HSetMaximum(Plot[plot->idsame]->name, localmax);
          }

        /*--- Set time string under the plot and draw the plot ---*/
        origin = plot->gtime;
        if (plot->raw2d == ON && plot->chx->vect->nDim > 2) origin += im*plot->chx->vect->dx[0];
        DyPlotSetTime (origin, plot->name, plot->deltat, plot->nAverage, OFF, Dy->Frame, plot->decayfft);
        DyPlotDraw(iplot);
        plot->lastrefresh_time = plot->gtime;
       }

    /*--- Denormalize after plot filling ---*/

    for (i=0; i<nBin; i++)
       {if (plot->x2av == ON) plot->buff[i] = plot->buff[i] * plot->buff[i] * plot->nAverage;
        else                  plot->buff[i] = plot->buff[i] * plot->nAverage;
       }

    /*--- Reset buffer when maxAverage reached ---*/

    if (plot->nAverage >= plot->maxAverage)
      {
       plot->nAverage = 0;
       for (j=0; j<nBin; j++) plot->buff[j] = 0.;
      }

   } /* end of loop on im */

 return;
}


/*----------------------------------------------------- DyPlotRAWEnd ---------------*/

void DyPlotRAWEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/

/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);
 
 return;
}



/*==================================================================================================*/
/*============================================= PLOT 1DTIME ========================================*/

/*---------------------------------------------------- DyPlot1DTIMEIni -----*/

void DyPlot1DTIMEIni(int iplot)
{
 char *oldname, dummy[LSTRING_MAX];
 struct DyPlot *plot;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->itime    = 0;
 plot->tmin     = Dy->startTime;

 /*--- When autoedge is used: check that xnBin is even and set xmin and xmax ---*/

 if (plot->autoedge == ON)
   {DyGetMinMax(plot->chx->vect, &(plot->ymin), &(plot->ymax));
    plot->ymin = plot->ymin - (plot->ymax - plot->ymin)/2;
    plot->ymax = plot->ymax + (plot->ymax - plot->ymin)/2;
    if (plot->ymax <= plot->ymin) {plot->ymin = -1; plot->ymax = 1;}
   }

 /*--- Initialize the time-frequency buffer ---*/

 plot->buff = (double *) calloc(plot->xnBin*(plot->ynBin), sizeof(double));
 if (plot->buff == NULL)
    {printf("DyPlot1DTIMEIni> plot->buff calloc (%ldx%ld doubles) failed \n", plot->xnBin, plot->ynBin);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin, plot->name);
 RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin, 0, plot->xnBin * plot->dtime,
               plot->ynBin, plot->ymin, plot->ymax);

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
   {RA_SetStyle(iplot);}

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Z axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->zmin >= 0. && plot->zmax > 0.)
    {RA_HSetMinimum(plot->name, plot->zmin);
     RA_HSetMaximum(plot->name, plot->zmax);
    }

 /*--- If autoscale is ON, init the zmin and zmax values ---*/

 if (plot->autoscale == ON)
   {plot->zmin = 0;
    plot->zmax = -1;
    if (!oldname) RA_HSetMinimum(plot->name, plot->zmin);
   }

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Y axis units of a 1d-time plot ---*/

 sprintf(dummy,"%s", DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) sprintf(dummy,"%s", plot->chx->vect->unitY);
 RA_HSetUnitsY(plot->name, dummy);

 /*--- Set Title of the plot ---*/

 RA_HSetTitle(plot->name, plot->title);

 return;

}


/*--------------------------------------------------------- DyPlot1DTIME -----------*/

void DyPlot1DTIME(int iplot)
{
 int i, j, k, ib, nx;
 double dt, value;
 struct DyPlot *plot;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Fill the buffer with this new computed FFT ---*/

 nx = plot->xnBin;
 for (k=0 ;k<plot->chx->vect->nData ;k++)
    {
     value = plot->chx->vect->dataD[k];
     if (value > plot->ymax || value < plot->ymin) continue;
     j = (int)(plot->ynBin * (value-plot->ymin) / (plot->ymax-plot->ymin));
     ib = plot->itime + nx*j;
     plot->buff[ib] = plot->buff[ib] + 1;
     if (plot->autoscale == ON && plot->zmax < plot->buff[ib]) plot->zmax = plot->buff[ib];
    }

 /*--- Increment the time axis bin number ---*/

 plot->itime++;

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {
     /*--- Put the time-frequency buffer in the plot ---*/

     RA_H2Pack(plot->name, plot->buff);

     /*--- Set the min and max on z axis ---*/

     if (plot->autoscale == ON && plot->zmax > 0.) RA_HSetMaximum(plot->name, plot->zmax);

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->tmin, plot->name, plot->deltat, 0, ON, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- End of time window reached, then shift the buffer content by one dtime step ---*/

 if (plot->itime == plot->xnBin)
   {
    plot->itime--;
    plot->tmin += plot->dtime;
    nx = plot->xnBin;
     for (i=0; i<nx-1; i++)
        {for (j=0; j<plot->ynBin; j++)
            {plot->buff[i+j*nx] = plot->buff[i+1+j*nx];}
        }
     for (j=0; j<plot->ynBin; j++)  plot->buff[nx-1+nx*j] = 0.;
   }

 return;
}


/*--------------------------------------------------------- DyPlot1DTIMEEnd ----------*/

void DyPlot1DTIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    DyPlotSetTime (plot->tmin, plot->name, plot->deltat, 0, ON, Dy->Frame, plot->decayfft);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*============================================ PLOT RAWTIME ========================================*/

/*---------------------------------------------------- DyPlotRAWTIMEIni -----*/

void DyPlotRAWTIMEIni(int iplot)
{
 char *oldname, dummy[LSTRING_MAX];
 struct DyPlot *plot;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst   = ON;

 /*--- Init some counters and flags ---*/

 plot->nAverage = 0;
 plot->lastrefresh_time = Dy->startTime;
 plot->itime    = 0;
 plot->tmin     = Dy->startTime;

 /*--- Set the flag saying it is an image or not ---*/

 plot->raw2d = OFF;
 if (plot->chx->vect->nData > plot->chx->vect->nx[0]) plot->raw2d = ON;

 /*--- Set some secondary parameters ---*/

 plot->ynBin = plot->chx->vect->nx[0];
 plot->ymin  = plot->chx->vect->startX[0];
 plot->ymax  = plot->ymin + plot->chx->vect->dx[0] * plot->chx->vect->nx[0];

 /*--- Initialize the time-frequency buffer ---*/

 plot->buff = (double *) calloc(plot->xnBin*(plot->ynBin), sizeof(double));
 if (plot->buff == NULL)
    {printf("DyPlotRAWTIMEIni> plot->buff calloc (%ldx%ld doubles) failed \n", plot->xnBin, plot->ynBin);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin, plot->name);
 RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin, 0, plot->xnBin * plot->dtime,
               plot->ynBin, plot->ymin, plot->ymax);

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
   {
    RA_PadCompleteClear(plot->numpad);
    if (plot->autoscale == OFF)
      {RA_HSetMinimum(plot->name, plot->zmin);
       RA_HSetMaximum(plot->name, plot->zmax);
      }
    RA_SetStyle(iplot);
   }

 DyPlotSetDrawOptions (iplot);

/*--- If autoscale is ON, init the zmin and zmax values ---*/

 if (plot->autoscale == ON)
   {plot->zmin = 1e+30;
    plot->zmax = -1e+30;
   }

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Y axis and Z axis units of a raw-time plot ---*/

 strcpy(dummy, DY_NOUNITS);
 if (plot->chx->vect != NULL) strcpy(dummy, (char *)plot->chx->vect->unitX);
 RA_HSetUnitsY(plot->name, dummy);

 strcpy(dummy, DY_NOUNITS);
 if (plot->chx->vect != NULL && plot->chx->realunits == ON) strcpy(dummy, (char *)plot->chx->vect->unitY);
 RA_HSetUnitsZ(plot->name, dummy);

 /*--- Set Title of the plot ---*/
 RA_HSetTitle(plot->name, plot->title);

 return;
}


/*--------------------------------------------------------- DYPlotRAWTIME -----------*/

void DyPlotRAWTIME(int iplot)
{
 int i, j, k, ib, nx;
 double dt, value;
 struct DyPlot *plot;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Fill the buffer with this new computed FFT ---*/

 nx = plot->xnBin;
 for (k=0 ;k<plot->chx->vect->nData ;k++)
    {
     value = plot->chx->vect->dataD[k];
     ib = plot->itime + nx*k;
     plot->buff[ib] = value;
     if (plot->autoscale == ON)
       {if (plot->zmax < value) plot->zmax = value;
        if (plot->zmin > value) plot->zmin = value;
       }
    }

 /*--- Increment the time axis bin number ---*/

 plot->itime++;

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {
     /*--- Put the time-frequency buffer in the plot ---*/

     RA_H2Pack(plot->name, plot->buff);

     /*--- Set the min and max on z axis ---*/

     if (plot->autoscale == ON && plot->zmax > 0.) RA_HSetMaximum(plot->name, plot->zmax);

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->tmin, plot->name, plot->deltat, 0, ON, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- End of time window reached, then shift the buffer content by one dtime step ---*/

 if (plot->itime == plot->xnBin)
   {
    plot->itime--;
    plot->tmin += plot->dtime;
    nx = plot->xnBin;
     for (i=0; i<nx-1; i++)
        {for (j=0; j<plot->ynBin; j++)
            {plot->buff[i+j*nx] = plot->buff[i+1+j*nx];}
        }
     for (j=0; j<plot->ynBin; j++)  plot->buff[nx-1+nx*j] = 0.;
   }

 return;
}

/*--------------------------------------------------------- DyPlotRAWTIMEEnd ----------*/

void DyPlotRAWTIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    DyPlotSetTime (plot->tmin, plot->name, plot->deltat, 0, ON, Dy->Frame, plot->decayfft);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*=========================================== PLOT TRFCTTIME =======================================*/

/*---------------------------------------------------- DyPlotTRFCTTIMEIni -----*/

void DyPlotTRFCTTIMEIni(int iplot)
{
 char *oldname;
 struct DyPlot *plot;
 int step_ivectx, step_ivecty;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;
 plot->itime    = 0;
 plot->tmin     = Dy->startTime;

/*--- Initialize the input buffer ---*/

 step_ivectx = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 step_ivecty = (int)(plot->fft_deltat / plot->chy->vect->dx[0]);
 plot->frvbufx = FrvBufNew(plot->ynBin, step_ivectx, FR_VECT_8R, -1, 0);
 plot->frvbufy = FrvBufNew(plot->ynBin, step_ivecty, FR_VECT_8R, -1, 0);

 /*--- Initialize the FFTW buffer ---*/

 plot->tf = FrvTFNewT(NULL, plot->fft_duration, plot->maxAverage);
 if (plot->tf == NULL) {printf("DyPlotTRFCTTIMEIni: Could not get FrvTF strcture \n"); return;}

 /*--- Initialize the time-frequency buffer ---*/

 plot->buff = (double *) calloc(plot->xnBin*(plot->ynBin/2+1), sizeof(double));
 if (plot->buff == NULL)
    {printf("DyPlotTRFCTTIMEIni> plot->buff calloc (%ldx%ld doubles) failed \n", plot->xnBin, plot->ynBin/2+1);}

 /*--- Set the decay time for a moving average ---*/

 FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);
 if (plot->decayfft == ON)
    {FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin/2+1, plot->name);
 RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin, 0, plot->xnBin * plot->dtime,
               plot->ynBin/2+1,
               plot->dfreq*(-0.5), plot->dfreq*((double)(plot->ynBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetYRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);
    }

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Z axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->zmin > 0. && plot->zmax > 0.)
    {RA_HSetMinimum(plot->name, plot->zmin);
     RA_HSetMaximum(plot->name, plot->zmax);
    }

 /*--- If autoscale is ON, init the zmin and zmax values ---*/

 if (plot->autoscale == ON)
   {plot->zmin =   DY_BADDATA;
    plot->zmax = - DY_BADDATA;
   }

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Y axis units of a time-frequency plot ---*/

 RA_HSetUnitsY(plot->name, "Hz");

 /*--- Set Title of the plot ---*/

 RA_HSetTitle(plot->name, plot->title);

 return;
}


/*------------------------------------------------------- DyPlotTRFCTTIME -------------*/

void DyPlotTRFCTTIME(int iplot)
{
 struct DyPlot *plot;
 int step_ivectx, step_ivecty, nbuf;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Put in buffer the input vector and Compute the FFT for each sub vector ---*/

 step_ivectx = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 step_ivecty = (int)(plot->fft_deltat / plot->chy->vect->dx[0]);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("DyPlotTRFCTTIME> step_ivectx=%d  ynBin=%d  nData=%d  fft_deltat=%g deltat=%g \n",
            step_ivectx, plot->ynBin, plot->chx->vect->nData, plot->fft_deltat, plot->deltat);
   }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
    {printf("DyPlotTRFCTTIME> Put in buffer vectx=%s  GTime=%f nData=%d and vecty=%s GTime=%f nData=%d \n",
             plot->chx->vect->name, plot->chx->vect->GTime, plot->chx->vect->nData,
             plot->chy->vect->name, plot->chy->vect->GTime, plot->chy->vect->nData);
    }

 FrvBufFeed(plot->frvbufx, plot->chx->vect);
 FrvBufFeed(plot->frvbufy, plot->chy->vect);

 nbuf = 2;
 while (FrvBufGetNext(plot->frvbufx) == 0 && FrvBufGetNext(plot->frvbufy) == 0 && Dy->Order != STOP)
   {
    if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
      {printf("DyPlotTRFCTTIME> Extract from buffer subvect %d GTime_x=%f nData_x=%d  GTime_y=%f nData_y=%d \n",
              nbuf, plot->frvbufx->output->GTime, plot->frvbufx->output->nData, plot->frvbufy->output->GTime, plot->frvbufy->output->nData);
      }
    DyPlotTRFCTTIMEPerVect(iplot, plot->frvbufx->output, plot->frvbufy->output);
    nbuf = nbuf+2;
   }

 return;
}


/*------------------------------------------------------- DyPlotTRFCTTIMEPerVect -------------*/

void DyPlotTRFCTTIMEPerVect(int iplot, struct FrVect *subvectx, struct FrVect *subvecty)
{
 int i, j, ib, nx, nx2;
 double dt;
 struct DyPlot *plot;
 double tv;
 int k;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Exclude missing samples from TRFCT computation ---*/

 if (plot->chx->vect == NULL || plot->chx->vect->next != NULL)
   {printf("DyPlotTRFCTTIME> Input vector %s at time %f has missing samples. Excluding it from COHE computation\n",
           plot->chx->name, plot->gtime);
    return;
   }
 if (plot->chy->vect == NULL || plot->chy->vect->next != NULL)
   {printf("DyPlotCOHETIME> Input vector %s at time %f has missing samples. Excluding it from COHE computation\n",
           plot->chy->name, plot->gtime);
    return;
   }

 /*--- Remove any missing sample information kept from previous iteration ---*/
 if (plot->tf->modulus != NULL && plot->tf->modulus->next != NULL)
   {for (i=0; i< plot->tf->modulus->nData; i++) plot->tf->modulus->next->data[i] = 1;}

 /*--- Compute the COHErence only of no missing data ---*/

 if (subvectx != NULL && subvectx->next == NULL)
   {FrvTFProc(plot->tf, subvectx, subvecty);}

 /*--- Average the FFTs until maxAverage is reached ---*/

 plot->nAverage++;
 if (plot->nAverage < plot->maxAverage) return;

 if (plot->tf->modulus == NULL) return;

 /*--- Fill the buffer with this new computed FFT ---*/

 nx = plot->xnBin;
 for (j=0 ;j<plot->ynBin/2+1 ;j++)
    {ib = plot->itime + nx*j;
     plot->buff[ib] = sqrt(plot->tf->modulus->dataD[j]);
     if (plot->autoscale == ON)
       {if (plot->buff[ib] > 1e-30 && plot->zmin > plot->buff[ib]) plot->zmin = plot->buff[ib];
        if (plot->buff[ib] > 0 && plot->zmax < plot->buff[ib]) plot->zmax = plot->buff[ib];
       }
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0 && j<200)
       {printf("DyPlotTRFCTTIMEPerVect> itime=%d  j=%d/%d  buff[%d]=%g  zmin=%g  nx=%d\n",
               plot->itime, j, plot->ynBin/2+1, ib, plot->buff[ib], plot->zmin, nx);
       } 
    }

 /*--- Increment the time axis bin number ---*/

 plot->itime++;

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->fft_deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {
     /*--- Put the time-frequency buffer in the plot ---*/

     RA_H2Pack(plot->name, plot->buff);

     /*--- Set the min and max on z axis ---*/

     if (plot->autoscale == ON && plot->zmin > 0.) RA_HSetMinimum(plot->name, plot->zmin);
     if (plot->autoscale == ON && plot->zmax > 0.) RA_HSetMaximum(plot->name, plot->zmax);

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->tmin, plot->name, plot->fft_duration, plot->maxAverage, ON, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- Reset the counter of averages ---*/

 plot->nAverage = 0;
 if (plot->decayfft == OFF) FrvTFReset(plot->tf);

 /*--- End of time window reached, then shift the buffer content by one dtime step ---*/

 if (plot->itime == plot->xnBin)
   {
    plot->itime--;
    plot->tmin += plot->dtime;
    nx = plot->xnBin;
    for (i=0; i<nx-1; i++)
       {for (j=0; j<plot->ynBin/2+1; j++)
           {ib = i + j*nx;
            plot->buff[ib] = plot->buff[ib+1];
           }
       }
    for (j=0; j<plot->ynBin/2+1; j++)
       {ib = nx-1+nx*j;
        plot->buff[ib] = plot->buff[ib-1];
       }
   }

 return;
}


/*--------------------------------------------------------- DyPlotTRFCTTIMEEnd ----------*/

void DyPlotTRFCTTIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*==================================================================================================*/
/*============================================= PLOT COHETIME ======================================*/

/*---------------------------------------------------- DyPlotCOHETIMEIni -----*/

void DyPlotCOHETIMEIni(int iplot)
{
 int step_ivectx, step_ivecty;
 char *oldname;
 struct DyPlot *plot;

 plot = Plot[iplot];

 DyDebugPrint(__FUNCTION__,0);

 plot->iFirst = ON;

 /*--- Init some counters or flags ---*/

 plot->lastrefresh_time = Dy->startTime;
 plot->nAverage = 0;
 plot->itime    = 0;
 plot->tmin     = Dy->startTime;

 /*--- Initialize the input buffer ---*/

 step_ivectx = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 step_ivecty = (int)(plot->fft_deltat / plot->chy->vect->dx[0]);
 plot->frvbufx = FrvBufNew(plot->ynBin, step_ivectx, FR_VECT_8R, -1, 0);
 plot->frvbufy = FrvBufNew(plot->ynBin, step_ivecty, FR_VECT_8R, -1, 0);

 /*--- Initialize the FFTW buffer ---*/
 
 plot->tf = FrvTFNewT("C", plot->fft_duration, plot->maxAverage);
 if (plot->tf == NULL) {printf("DyPlotCOHETIMEIni: Could not get FrvTF strcture \n"); return;}

 /*--- Initialize the time-frequency buffer ---*/

 plot->buff = (double *) calloc(plot->xnBin*(plot->ynBin/2+1), sizeof(double));
 if (plot->buff == NULL)
    {printf("DyPlotCOHETIMEIni> plot->buff calloc (%ldx%ld doubles) failed \n", plot->xnBin, plot->ynBin/2+1);}

 /*--- Set the decay time for a moving average ---*/

 FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);
 if (plot->decayfft == ON)
    {FrvRFFTSetDecay(plot->fftx, 1.-1./(double)plot->maxAverage);}

 /*--- Create or reuse a ROOT plot ---*/

 oldname = RA_PadContainsGoodHist(plot->numpad, plot->numsubpad, 2, plot->xnBin, plot->ynBin/2+1, plot->name);
 RA_H2NewReuse(plot->numpad, plot->numsubpad, plot->name, plot->title,
               plot->xnBin, 0, plot->xnBin * plot->dtime,
               plot->ynBin/2+1,
               plot->dfreq*(-0.5), plot->dfreq*((double)(plot->ynBin/2+1)-0.5));

 /*-- If plot not reused, set the frequency range and the plot style ---*/

 if (!oldname)
    {RA_HSetYRange (plot->name, plot->fmin, plot->fmax, plot->dfreq);
     RA_SetStyle(iplot);
    }

 DyPlotSetDrawOptions (iplot);

 /*--- If plot not reused, set the Z axis range if autoscale is not set ---*/

 if (!oldname && plot->autoscale == OFF && plot->zmin > 0. && plot->zmax > 0.)
    {RA_HSetMinimum(plot->name, plot->zmin);
     RA_HSetMaximum(plot->name, plot->zmax);
    }

 /*--- If autoscale is ON, init the zmin and zmax values ---*/

 if (plot->autoscale == ON)
   {plot->zmin =   DY_BADDATA;
    plot->zmax = - DY_BADDATA;
   }

 /*--- No stat shown on this plot ---*/

 RA_HSetStat(plot->name, OFF);

 /*--- Y axis units of a time-frequency plot ---*/

 RA_HSetUnitsY(plot->name, "Hz");

 /*--- Set Title of the plot ---*/

 RA_HSetTitle(plot->name, plot->title);

 return;
}



/*------------------------------------------------------- DyPlotCOHETIME -------------*/

void DyPlotCOHETIME(int iplot)
{
 struct DyPlot *plot;
 int step_ivectx, step_ivecty, nbuf;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Put in buffer the input vector and Compute the FFT for each sub vector ---*/

 step_ivectx = (int)(plot->fft_deltat / plot->chx->vect->dx[0]);
 step_ivecty = (int)(plot->fft_deltat / plot->chy->vect->dx[0]);

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)))
   {printf("DyPlotCOHETIME> step_ivectx=%d  ynBin=%d  nData=%d  fft_deltat=%g deltat=%g \n",
            step_ivectx, plot->ynBin, plot->chx->vect->nData, plot->fft_deltat, plot->deltat);
   }

 if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
    {printf("DyPlotCOHETIME> Put in buffer vectx=%s  GTime=%f nData=%d and vecty=%s GTime=%f nData=%d \n",
             plot->chx->vect->name, plot->chx->vect->GTime, plot->chx->vect->nData,
             plot->chy->vect->name, plot->chy->vect->GTime, plot->chy->vect->nData);
    }

 FrvBufFeed(plot->frvbufx, plot->chx->vect);
 FrvBufFeed(plot->frvbufy, plot->chy->vect);

 nbuf = 2;
 while (FrvBufGetNext(plot->frvbufx) == 0 && FrvBufGetNext(plot->frvbufy) == 0 && Dy->Order != STOP)
   {
    if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0)
      {printf("DyPlotCOHETIME> Extract from buffer subvect %d GTime_x=%f nData_x=%d  GTime_y=%f nData_y=%d \n",
              nbuf, plot->frvbufx->output->GTime, plot->frvbufx->output->nData, plot->frvbufy->output->GTime, plot->frvbufy->output->nData);
      }
    DyPlotCOHETIMEPerVect(iplot, plot->frvbufx->output, plot->frvbufy->output);
    nbuf = nbuf+2;
   }

 return;
}


/*------------------------------------------------------- DyPlotCOHETIMEPerVect -------------*/

void DyPlotCOHETIMEPerVect(int iplot, struct FrVect *subvectx, struct FrVect *subvecty)
{
 int i, j, ib, nx, nx2;
 double dt;
 struct DyPlot *plot;
 double tv;
 int k;

 plot = Plot[iplot];

 plot->iFirst = OFF;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Exclude missing samples from TRFCT computation ---*/

 if (plot->chx->vect == NULL || plot->chx->vect->next != NULL)
   {printf("DyPlotCOHETIME> Input vector %s at time %f has missing samples. Excluding it from COHE computation\n",
           plot->chx->name, plot->gtime);
    return;
   }
 if (plot->chy->vect == NULL || plot->chy->vect->next != NULL)
   {printf("DyPlotCOHETIME> Input vector %s at time %f has missing samples. Excluding it from COHE computation\n",
           plot->chy->name, plot->gtime);
    return;
   }

 /*--- Remove any missing sample information kept from previous iteration ---*/
 if (plot->tf->coherence != NULL && plot->tf->coherence->next != NULL)
   {for (i=0; i< plot->tf->coherence->nData; i++) plot->tf->coherence->next->data[i] = 1;}

 /*--- Compute the COHErence only of no missing data ---*/

 if (subvectx != NULL && subvectx->next == NULL)
   {FrvTFProc(plot->tf, subvectx, subvecty);}

 /*--- Average the FFTs until maxAverage is reached ---*/

 plot->nAverage++;
 if (plot->nAverage < plot->maxAverage) return;

 if (plot->tf->coherence == NULL) return;

 /*--- Fill the buffer with this new computed FFT ---*/

 nx = plot->xnBin;
 for (j=0 ;j<plot->ynBin/2+1 ;j++)
    {ib = plot->itime + nx*j;
     plot->buff[ib] = sqrt(plot->tf->coherence->dataD[j]);
     if (plot->autoscale == ON)
       {if (plot->buff[ib] > 1e-30 && plot->zmin > plot->buff[ib]) plot->zmin = plot->buff[ib];
        if (plot->buff[ib] > 0 && plot->zmax < plot->buff[ib]) plot->zmax = plot->buff[ib];
       }
     if (((Dy->debugType)&(1<<DBG_VDYPLOT)) && Dy->debug > 0 && j<200)
       {printf("DyPlotCOHETIMEPerVect> itime=%d  j=%d/%d  buff[%d]=%g  zmin=%g  nx=%d\n",
               plot->itime, j, plot->ynBin/2+1, ib, plot->buff[ib], plot->zmin, nx);
       } 
    }

 /*--- Increment the time axis bin number ---*/

 plot->itime++;

 /*--- Show the plot with periodicity refreshDt ---*/

 dt = plot->gtime + plot->fft_deltat - plot->lastrefresh_time;

 if (dt >= plot->refreshDt)
    {
     /*--- Put the time-frequency buffer in the plot ---*/

     RA_H2Pack(plot->name, plot->buff);

     /*--- Set the min and max on z axis ---*/

     if (plot->autoscale == ON && plot->zmin > 0.) RA_HSetMinimum(plot->name, plot->zmin);
     if (plot->autoscale == ON && plot->zmax > 0.) RA_HSetMaximum(plot->name, plot->zmax);

     /*--- Set the time string under the plot and draw the plot ---*/

     DyPlotSetTime (plot->tmin, plot->name, plot->fft_duration, plot->maxAverage, ON, Dy->Frame, plot->decayfft);
     DyPlotDraw(iplot);
     plot->lastrefresh_time = plot->gtime;
    }

 /*--- Reset the counter of averages ---*/

 plot->nAverage = 0;
 if (plot->decayfft == OFF) FrvTFReset(plot->tf);

 /*--- End of time window reached, then shift the buffer content by one dtime step ---*/

 if (plot->itime == plot->xnBin)
   {
    plot->itime--;
    plot->tmin += plot->dtime;
    nx = plot->xnBin;
    for (i=0; i<nx-1; i++)
       {for (j=0; j<plot->ynBin/2+1; j++)
           {ib = i + j*nx;
            plot->buff[ib] = plot->buff[ib+1];
           }
       }
    for (j=0; j<plot->ynBin/2+1; j++)
       {ib = nx-1+nx*j;
        plot->buff[ib] = plot->buff[ib-1];
       }
   }

 return;
}


/*--------------------------------------------------------- DyPlotCOHETIMEEnd ----------*/

void DyPlotCOHETIMEEnd(int iplot)
{
 struct DyPlot *plot;

 plot = Plot[iplot];

 /*--- Draw the plot ---*/
/**
 if (Dy->plotsNotDisplayed == OFF)
   {
    RA_SetActivePad(plot->numpad, plot->numsubpad);
    RA_HDraw(plot->name,plot->options,plot->superposed);
    DyPlotSetDrawInfo (iplot);
    RA_PadUpdate(plot->numpad);
   }
**/

 /*--- Free the vectors and buffers used ---*/

 DyPlotFree(iplot, OFF);

 return;
}


/*--------------------------------------------------------- DyPlotSetLegendSize ----------*/

void DyPlotSetLegendSize(int iplot, double *xorig, double *yorig, double *leg_size)
{
 char dummy[LSTRING_MAX];
 double xleg, coef;
 int i, maxlen, nchsame;
 int nPadsx, nPadsy;

 sprintf(dummy,"%s> iplot=%d/%d", __FUNCTION__, iplot, Dy->nplot);
 DyDebugPrint(dummy,0);

 if (iplot <= 0) return;

 nPadsx = Dy->nCol;
 nPadsy = Dy->nRow;

 // Set legend text size, according to the pads number
 *leg_size = 0.04;
 if (nPadsy > 2)                 *leg_size = 0.05;
 if (nPadsx == 3 && nPadsy == 4) *leg_size = 0.04;
 if (nPadsx == 2 && nPadsy == 1) *leg_size = 0.03;

 // Set coef for x position of legend box
 coef = 0.015;
 if (nPadsx == 1 && nPadsy == 2) coef = 0.010;
 if (nPadsx == 1 && nPadsy == 3) coef = 0.007;

 if (nPadsx == 2 && nPadsy == 1) coef = 0.023;
 if (nPadsx == 2 && nPadsy == 2) coef = 0.015;
 if (nPadsx == 2 && nPadsy == 3) coef = 0.015;
 if (nPadsx == 2 && nPadsy == 4) coef = 0.010;

 if (nPadsx == 3 && nPadsy == 1) coef = 0.030;
 if (nPadsx == 3 && nPadsy == 2) coef = 0.023;
 if (nPadsx == 3 && nPadsy == 3) coef = 0.020;
 if (nPadsx == 3 && nPadsy == 4) coef = 0.012;

 if (nPadsx == 4 && nPadsy == 2) coef = 0.030;
 if (nPadsx == 4 && nPadsy == 3) coef = 0.025;

 if (nPadsx == 5 && nPadsy == 2) coef = 0.030;
 if (nPadsx == 5 && nPadsy == 3) coef = 0.025;

 if (nPadsx >= 4 && nPadsy >= 4) coef = 0.020;

 // Get max length of plot names
 maxlen = 0;
 nchsame = 0;
 for (i=iplot; i<=Dy->nplot; i++)
    {
     if (i>iplot && Plot[i]->superposed == OFF) break;
     if (maxlen < strlen(Plot[i]->title)) maxlen = strlen(Plot[i]->title);
     nchsame++;
    }

 // Set x position of legend box, according to maxlen
 *xorig = 0.96-coef*maxlen;
 if (*xorig < 0.20) *xorig = 0.20;

 if (nPadsx > 1) *yorig = DY_LEG_YMAX-0.01- 0.037*nchsame;
 else            *yorig = DY_LEG_YMAX-0.01 - 0.040*nchsame;

 return;
}


/*------------------------------------- DyPlotSetAllSize ---*/
void DyPlotSetAllSize()
{
 char dummy[128];
 double xt_offset, xt_size, xl_size;
 double yt_offset, yt_size, yl_size;
 int nPadsx, nPadsy;

 DyDebugPrint(__FUNCTION__,0);

 nPadsx = Dy->nCol;
 nPadsy = Dy->nRow;

 // Set default Offset and title size on X and Y axis
 xt_size   = 0.03+0.003*nPadsx+0.003*nPadsy;
 yt_size   = 0.04+0.003*nPadsx+0.003*nPadsy;
 xt_offset = 1.2-0.05*nPadsy;
 yt_offset = 0.9-0.05*nPadsx;

 // Set default label size on X and Y axis
 xl_size   = 0.05;
 yl_size   = 0.05;

 if (nPadsx == 1 && nPadsy == 1)  {xt_size = 0.05;  yt_size = 0.05; xl_size = 0.05; yl_size = 0.05; xt_offset = 1.2;  yt_offset = 1.1;}
 if (nPadsx == 2 && nPadsy == 1)  {xt_size = 0.04;  yt_size = 0.04; xl_size = 0.04; yl_size = 0.04; xt_offset = 1.15; yt_offset = 1.1;}
 if (nPadsx == 3 && nPadsy == 1)  {xt_size = 0.04;  yt_size = 0.05; xl_size = 0.05; yl_size = 0.05; xt_offset = 1.15; yt_offset = 1.2;}
 if (nPadsx == 1 && nPadsy == 2)  {xt_size = 0.07;  yt_size = 0.07; xl_size = 0.07; yl_size = 0.07; xt_offset = 1;    yt_offset = 0.6;}
 if (nPadsx <= 2 && nPadsy > 2)   {xt_size = 0.08;  yt_size = 0.08; xl_size = 0.08; yl_size = 0.08; xt_offset = 0.9;  yt_offset = 0.6;}
 if (nPadsx > 2 && nPadsy > 2)    {xt_size = 0.07;  yt_size = 0.07; xl_size = 0.06; yl_size = 0.06;}
 if (nPadsx == 4 && nPadsy == 2)  {xt_size = 0.04;  yt_size = 0.05; xl_size = 0.05; yl_size = 0.05; xt_offset = 1.1;  yt_offset = 1.2;}
 if (nPadsx == 5 && nPadsy == 2)  {xt_size = 0.045; yt_size = 0.05; xl_size = 0.05; yl_size = 0.05; xt_offset = 1.1;  yt_offset = 1.4;}
 if (nPadsx == 3 && nPadsy == 4)  {xt_size = 0.06;  yt_size = 0.06;}

/**
 printf("+++++ nPadsx=%d nPadsy=%d ... xt_size=%g  yt_size=%g ... xl_size=%g  yl_size=%g ... xt_offset=%g  yt_offset=%g\n",
        nPadsx, nPadsy, xt_size, yt_size, xl_size, yl_size, xt_offset, yt_offset); 
**/

 /*--- offset of x and y titles ---*/
 Dy->xt_offset = xt_offset;
 Dy->yt_offset = yt_offset;

 /*--- Size of x and y titles ---*/
 Dy->xt_size = xt_size;
 Dy->yt_size = yt_size;

 /*--- Size of x and y labels ---*/
 Dy->xl_size = xl_size;
 Dy->yl_size = yl_size;

 return;
}



