/**************************************************/
/*                                                */
/* File: DyXform.c                                */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

#include "dataDisplay.h"
/*
#include "images/logo.xpm"
*/
#include "images/coffee.xpm"
#include "images/cristalBall.xpm"
#include "images/andromede.xpm"
#include "images/help_ffttime.xpm"
#include "RVersion.h"


#ifdef MEMWATCH
#    include "memwatch.h"
#endif

/*------------------------------------------- DyXformChSelect ---*/

int DyXformChSelect (char *chname)
{
 int i, i0, nch;

 nch = fl_get_browser_maxline(mainbrowser->channels);
 if (nch <= 0) return(-1);

 i0 = -1;
 for (i=1; i<=nch; i++)
    {if (strcmp(Dy->ch[i-1]->name, chname) == 0)
       {fl_select_browser_line(mainbrowser->channels,i);
        i0 = i;
        break;
       }
    }

 return(i0);
}


/*------------------------------------------- DyXformGetChFromPlot ---*/

void DyXformGetChFromPlot (FL_OBJECT *obj, long arg)
{
 int i, i0, i0min, nplot;

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 i0min = fl_get_browser_maxline(mainbrowser->channels);
 i0 = -1;

 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i))
        {
         if (Plot[i]->chx) i0 = DyXformChSelect(Plot[i]->chx->name);
         if (Plot[i]->chy) i0 = DyXformChSelect(Plot[i]->chy->name);
         if (i0min > i0) i0min = i0;
        }
    }

 if (i0 > 0) fl_set_browser_topline(mainbrowser->channels, i0min);

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*------------------------------------------- DyXformGeneParToPlot ---*/

void DyXformGeneParToPlot (FL_OBJECT *obj, long arg)
{
return;
}


/*------------------------------------------- DyXformGetOpString ---*/

void DyXformGetOpString (FL_OBJECT *obj, long arg)
{
 const char *opstring;
 int i, i0;
 struct DyCh *opch;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get the opstring from the mainbrowser input field ---*/

 opstring = fl_get_input(mainbrowser->opstring);
 if (opstring == NULL) return;

 /*--- Make the new channel from the opstring , or update the opstring of the user channel selected ---*/

 opch = DyGetOpString(opstring);

 if (opch == NULL)
   {printf("DyXformGetOpString> No new channel created for this combination of channel: opstring=%s\n", opstring);
    return;
   }

 /*--- Show the line in browser corresponding to the new channel created or updated ---*/

 i0 = -1;
 for (i=Dy->nch-1; i>= 0; i--)
    {if (strcmp(Dy->ch[i]->name, opch->name) == 0) {i0 = i; break;}}

 fl_set_browser_topline(mainbrowser->channels, i0);

 return;
}


/*-------------------------------------- DyXformAddOneChannel ---*/

void DyXformAddOneChannel (FL_OBJECT *obj, struct DyCh *opch, int i0)
{
 char dummy[LBIGSTRING_MAX];

 sprintf(dummy, "%s : %.2fHz  %s", opch->name, opch->sampFreq, opch->inputname);
 if (i0 < 0) fl_add_browser_line(obj, dummy);
 else        fl_replace_browser_line(obj, i0+1, dummy);

 return;
}


/*------------------------------------------- DyXformSetDebug ---*/

void DyXformSetDebug()
{
 char dummy[32];

 sprintf(dummy,"%d",Dy->debug);
 fl_set_input(dydebug->debuglvl,dummy);

 sprintf(dummy,"%d",Dy->debugSleep);
 fl_set_input(dydebug->debugsleep,dummy);

 fl_set_button(dydebug->dbg_calls,OFF);
 fl_set_button(dydebug->dbg_speed,OFF);
 fl_set_button(dydebug->dbg_read,OFF);
 fl_set_button(dydebug->dbg_vdyplot,OFF);
 fl_set_button(dydebug->dbg_others,OFF);
 if (Dy->debugType & (1 << DBG_CALLS))   fl_set_button(dydebug->dbg_calls,ON);
 if (Dy->debugType & (1 << DBG_SPEED))   fl_set_button(dydebug->dbg_speed,ON);
 if (Dy->debugType & (1 << DBG_READ))    fl_set_button(dydebug->dbg_read,ON);
 if (Dy->debugType & (1 << DBG_VDYPLOT)) fl_set_button(dydebug->dbg_vdyplot,ON);
 if (Dy->debugType & (1 << DBG_OTHERS))  fl_set_button(dydebug->dbg_others,ON);

 return;
}


/*------------------------------------------- DyXformShowWriteAlarm ---*/

void DyXformShowWriteAlarm (int alarm)
{
 if (alarm == 1) fl_show_object(mainpanel->writing_data);
 else            fl_hide_object(mainpanel->writing_data);

 return;
}


/*------------------------------------------- DyXformGetPasswd ---*/

int DyXformGetPasswd ()
{
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 answer = fl_show_input("You need to provide a password: ", "");
 if (answer) {Dy->Ddpasswd = (char *)malloc(64); strcpy(Dy->Ddpasswd,answer);}
 else        return(0);

 return(1);
}

/*------------------------------------------- DyXformSetButtons ---*/

void DyXformSetButtons ()
{
 DyDebugPrint(__FUNCTION__,0);

 fl_set_button(dyoptions->no_autostop, Dy->no_autostop);
 fl_set_button(dyoptions->trigused, Dy->triggerUsed);
 fl_set_button(dyoptions->trigpause, Dy->triggerPause);
 fl_set_button(dyoptions->noplot, Dy->plotsNotDisplayed);
 fl_set_button(dyoptions->squareplot, Dy->squarePlot);
 fl_set_button(dyoptions->autodeselectplot, Dy->AutoDeselectPlot);
 fl_set_button(dyoptions->autodeselectch, Dy->AutoDeselectCh);
 fl_set_button(dyoutput->output_flag, Dy->output_flag);
 fl_set_button(mainpanel->waitdata, Dy->WaitData);

 return;
}

/*---------------------------------------- DyXformGetConfigFile ---*/

void DyXformGetConfigFile (FL_OBJECT *obj, long arg)
{
 char pattern_str[LSTRING_MAX];
 char *result, *suffix;
 int result_length;

 DyDebugPrint(__FUNCTION__,0);

 result = (char *)malloc(LSTRING_MAX);

 sprintf(pattern_str,"*%s", CONFIG_SUFFIX);
 result_length = ClXFSelector("Choose a new config file", Dy->lastconfig_dir, pattern_str, "", result, LSTRING_MAX);
 if (result_length == 0 || result == NULL) return;

 suffix = strstr(result,".");

 printf("DyXformGetConfigFile> result=%s  suffix=%s \n", result, suffix);

/**
 if (strcmp(suffix,".car") == 0) DyCarGetConfigFile(result);
 else                            DyConfigGetFile(result);
**/

 DyConfigGetFile(result);

 free(result);

 return;
}

/*----------------------------------------- DyXformSaveConfig ---*/

void DyXformSaveConfig (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 int config_created;

 DyDebugPrint(__FUNCTION__,0);

 str = fl_show_input("Config filename", Dy->lastsavedConfig);
 if (str) strcpy(Dy->lastsavedConfig, (char *)str);

 config_created = DyConfigCreate(Dy->lastsavedConfig,0);

 if (config_created <= 0)
    {DyInfoLine("There is no configuration to save!");}
 else
    {sprintf(dummy,"Configuration saved in file %s", Dy->lastsavedConfig);
     DyInfoLine (dummy);
    }

 return;
}

/*----------------------------------------- DyXformDdFillBrowser ---*/

void DyXformDdFillBrowser ()
{
 int i;

 if (Dy->ndd <= 0) return;

 fl_clear_browser(getdd->datasender_list);
 for (i=0; i<Dy->ndd; i++) fl_add_browser_line(getdd->datasender_list, Dy->ddlist[i]);

 return;
}

/*----------------------------------------- DyXformDdFileFillBrowser ---*/

void DyXformDdFileFillBrowser ()
{
 int i;

 if (Dy->nddfile <= 0) return;

 fl_clear_browser(getdd->liste);
 for (i=0; i<Dy->nddfile; i++) fl_add_browser_line(getdd->liste, Dy->ddfilelist[i]);

 return;
}
/*----------------------------------------- DyXformCmFillBrowser ---*/

void DyXformCmFillBrowser ()
{
 int i;

 if (Dy->ncm <= 0) return;

 fl_clear_browser(cmservers->liste);
 for (i=0; i<Dy->ncm; i++) fl_add_browser_line(cmservers->liste, Dy->cmlist[i]);

 return;
}

/*----------------------------------------- DyXformShowOneliner ---*/

void DyXformShowOneliner (char *message, int fontsize)
{
 if (fontsize == 0) fontsize = FL_NORMAL_SIZE;
 else               fontsize = FL_LARGE_SIZE;

 fl_set_oneliner_font(FL_TIMESBOLD_STYLE, fontsize);
 fl_show_oneliner(message, XREF-mainpanel->mainpanel->w-30,YREF-mainpanel->mainpanel->h+30);

 return;
}

/*----------------------------------------- DyXformHideOneliner ---*/

void DyXformHideOneliner ()
{
 fl_hide_oneliner();
 fl_set_oneliner_font(FL_TIMESBOLD_STYLE, FL_NORMAL_SIZE);
 return;
}

/*----------------------------------------- DyXformPutDefaultInAllPlots ---*/

void DyXformPutDefaultInAllPlots (FL_OBJECT *obj, long arg)
{
 long id, nPlot, answer=1;

 answer = fl_show_question ("Do you really want to put default parameters in all the plots ?",1);
 if (!answer) return;

 nPlot = fl_get_browser_maxline(mainbrowser->plots);

 for (id=1; id<=nPlot; id++) DyPutDefaultInPlot(id);

 DyInfoLine ("Put default values in all Plots");
 DyXformClosePanel (plotgenepar->close,0);

 return;
}

/*------------------------------------------- DyXformSetOnePlot ---*/
/*---  called from Main Browser when a plot button is pushed ---*/

void DyXformSetOnePlot (FL_OBJECT *obj, long arg)
{
 long i, nvar, plottype, nplot_previous;
 char dummy[LSTRING_MAX];
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (fl_get_browser(mainbrowser->channels) == 0) {DyInfoLine ("No channel selected"); return;}

 /*--- Set the type of plot chosen ---*/

 plottype = DyXformSetPlotType(obj);

 /*--- Loop over channels browser and make the plot for the lines selected ---*/

 fl_deactivate_form(mainbrowser->mainbrowser);

 nplot_previous = Dy->nplot;

 browser = mainbrowser->channels;
 nvar = fl_get_browser_maxline(browser);
 for (i=0; i<nvar; i++)
    {if (fl_isselected_browser_line(browser,i+1))
       {DyXformSetVar(browser,i+1);
        DySetOnePlot(plottype);
       }
    }

 fl_activate_form(mainbrowser->mainbrowser);

 if (Dy->y2d == OFF)
    {sprintf(dummy, "Made %ld new %s plot", Dy->nplot-nplot_previous, Dy->typeName[Plot[Dy->nplot]->type]);
     DyInfoLine (dummy);
     DyXformAutoDeselect(mainbrowser->channels, 0);
    }

 return;
}


/*------------------------------------------- DyXformSetPlotType ---*/

int DyXformSetPlotType (FL_OBJECT *obj)
{
 int plottype;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainbrowser->plotRaw)     plottype = PLOTRAW;
 if (obj == mainbrowser->plotTime)    plottype = PLOTTIME;
 if (obj == mainbrowser->plotFFT)     plottype = PLOTFFT;
 if (obj == mainbrowser->plotTrFct)   plottype = PLOTTR;
 if (obj == mainbrowser->plotCohe)    plottype = PLOTCOHE;
 if (obj == mainbrowser->plot1D)      plottype = PLOT1D;
 if (obj == mainbrowser->plot2D)      plottype = PLOT2D;
 if (obj == mainbrowser->plotAudio)   plottype = PLOTAUDIO;
 if (obj == mainbrowser->plotFFTTime) plottype = PLOTFFTTIME;
 if (obj == mainbrowser->plot1DTime)  plottype = PLOT1DTIME;
 if (obj == mainbrowser->plotRawTime) plottype = PLOTRAWTIME;
 if (obj == mainbrowser->plotTrFctTime) plottype = PLOTTRTIME;
 if (obj == mainbrowser->plotCoheTime)  plottype = PLOTCOHETIME;

 return(plottype);
}


/*------------------------------------------- DyXformSetOnePlotVarY ---*/
                                                 /*  called from Main Browser  */
/* Called when the plot type selected requires two input channels */
/* and when a second channel is selected in the channels browser */

void DyXformSetOnePlotVarY (FL_OBJECT *obj, long arg)
{
 long i;

 if (Dy->y2d == OFF) return;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Take a second variable if a channel is selected ---*/

 i = fl_get_browser(obj);
 if (i <= 0) return;

 /*--- Create the 2 channels plot ---*/

 DyXformSetVar(obj,i);
 DySetOnePlotVarY();

 return;
}

/*------------------------------------------- DyXformAddFile ---*/

void DyXformAddFile (FL_OBJECT *obj, long arg)
{
 char *result, pattern_str[LSTRING_MAX];
 int size, result_length;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Prepare a result string sufficiently large ---*/

 size = LSTRING_MAX * NFILE_MAX;
 result = (char *)malloc(size);
 while (result == NULL)
    {size = size / 2;
     result = (char *)malloc(size);
    }

 /*--- Look for ffl files ---*/

 if (arg == 1)
    {strcpy(pattern_str, "*.ffl");
     if (is_a_dir(FFLDIR)) result_length = ClXFSelector("Choose ffl to read", FFLDIR, pattern_str, "", result, size);
     else                  result_length = ClXFSelector("Choose ffl to read", Dy->Dir, pattern_str, "", result, size);
    }

 /*--- Look for wave files ---*/

 else if (arg == -1)
    {strcpy(pattern_str, "*.wav");
     result_length = ClXFSelector("Choose wave file to read", Dy->Dir, pattern_str, "", result, size);
    }

 /*--- Look for ascii files ---*/

 else if (arg == -2)
    {strcpy(pattern_str, "*.txt");
     result_length = ClXFSelector("Choose an ascii file to read", Dy->Dir, pattern_str, "", result, size);
    }

 /*--- Look for frame files ---*/

 else
    {strcpy(pattern_str, "*.gwf");
     result_length = ClXFSelector("Choose frames files to read", Dy->Dir, pattern_str, "", result, size);
    }

 if (result_length == 0 || result == NULL) return;

 /*--- If current data were taken online or are going to be taken online, reset channels list ---*/
 if (Dy->InputType == INCM || Dy->previousInputType == INCM) DyXformRaz(NULL,1);

 /*--- Get channels of all the files contained in result string ---*/

 if (((Dy->debugType)&(1<<DBG_OTHERS))) printf("DyXformAddFile> Dy->Dir=%s  result=%s  arg=%d \n", Dy->Dir, result, arg);

 if (arg == -1)      DyAddFile(result,INWAVE);
 else if (arg == -2) DyAddFile(result,INASCII);
 else                DyAddFile(result,INFILE);

 Dy->Dir = DyDirName(Dy->Dir, result);

 free(result);

 return;
}


/*------------------------------------------- DyXformPadsConfig ---*/

void DyXformPadsConfig (FL_OBJECT *obj, long arg)
{
 long ntot;
 int nCol, nRow;
 char dummy[LSTRING_MAX];
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 sprintf(dummy,"%dx%d",Dy->nCol, Dy->nRow);
 answer = fl_show_input("Pads Configuration  (syntax: nCol times nRow)", dummy);
 if (!answer) return;

 if (strstr(answer,"x") == NULL)
    {
     sscanf(answer,"%ld", &ntot);
     DySetRowCol(ntot, &nCol, &nRow);
     Dy->nCol = nCol;
     Dy->nRow = nRow;
    }
 else
    {sscanf(answer,"%dx%d", &Dy->nCol, &Dy->nRow);}

 return;
}

/*------------------------------------------- DyXformChecks ---*/

void DyXformChecks (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];
 const char *answer, *str;
 char plotname[LSTRING_MAX], chname[LSTRING_MAX];
 int i, nplot, nch, ploti;

 if (arg == -1)
    {sprintf(dummy,"%d",Dy->debug);
     answer = fl_show_input("Debug Level", dummy);
     if (!answer) return;
     if (strstr(answer,"x") == NULL)
        {Dy->debug = atoi(answer);}
     else
        {sscanf(answer,"%dx%d", &Dy->nCol, &Dy->nRow);}
     return;
    }

 printf("\n");
 printf("**************************************************************** \n");
 printf("*             CHECKS of dataDisplay PARAMETERS                 * \n");
 printf("**************************************************************** \n");

 /*--- Check that channel browser has the same list as Dy ---*/

 nch = fl_get_browser_maxline(mainbrowser->channels); 
 printf("\n  ------------ CHANNELS BROWSER \n");
 printf("  %d channels in browser and Dy->nch=%d\n", nch, Dy->nch);
 for (i=0; i<nch; i++)
    {str = fl_get_browser_line(mainbrowser->channels,i+1);
     sscanf(str,"%s",chname);
     if (strcmp(chname, Dy->ch[i]->name) != 0)
       {printf("channel %d in channels browser is %s while Dy->ch[%d]=%s\n",
                i+1, chname, i, Dy->ch[i]->name);
       }
    }

 /*--- Check that plot browser has the same list as Plot ---*/

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 printf("\n  ------------ PLOTS BROWSER \n");
 printf("  %d plots in browser and Dy->nplot=%d\n", nplot, Dy->nplot);
 for (i=0; i<nplot; i++)
    {str = fl_get_browser_line(mainbrowser->plots,i+1);
     strcpy(plotname,"...");
     if (Plot[i+1]->hidden == ON) sscanf(str, "@C14 %d %s",&ploti, plotname);
     else                         sscanf(str, "%d %s",&ploti, plotname);
     if (strcmp(plotname,Plot[i+1]->title) != 0)
       {printf("plot %d in plots browser is %s while Plot[%d]=%s\n",
                i+1, plotname, i+1, Plot[i+1]->title);
       }
    }

 /*--- Check everything else ---*/

 DyChecks();

 return;
}


/*----------------------------------------------------- DyXformQuit ---*/

void DyXformQuit (FL_OBJECT *obj, long arg)
{
 int answer;

 /*--- Ask to the user if he really wants to quit ---*/

 if (arg != 10 && Dy->BgRun == OFF)
    {
     answer = fl_show_question("Do you want to quit ?",1);
     if (!answer) return;
    }

 /*--- if any plot present, do as if stop button has been pushed ---*/

 if (Dy->Order != STOP) fl_trigger_object(mainpanel->stop);

 /*--- Call finally the quitHandler to do all the end procedures ---*/

 quitHandler(arg);

 return;
}


/*------------------------------------------- DyXformQuitAtClose ---*/

int DyXformQuitAtClose(FL_FORM *flForm, void *data)
{
 quitHandler(-2);
 return(0);
}

/*------------------------------------------- DyXformSetClose ---*/

void DyXformSetClose()
{
 /*--- Set the function to call when a panel is closed ---*/
 fl_set_form_atclose(mainpanel->mainpanel, DyXformQuitAtClose, "mainpanel");
 fl_set_form_atclose(mainbrowser->mainbrowser, DyXformAtClose, "mainbrowser");
 fl_set_form_atclose(userfuncbrowser->userfuncbrowser, DyXformAtClose, "userfuncbrowser");
 fl_set_form_atclose(plotfft->plotfft, DyXformAtClose, "plotfft");
 fl_set_form_atclose(plotffttime->plotffttime, DyXformAtClose, "plotffttime");
 fl_set_form_atclose(plottrfcttime->plottrfcttime, DyXformAtClose, "plottrfcttime");
 fl_set_form_atclose(plotcohetime->plotcohetime, DyXformAtClose, "plotcohetime");
 fl_set_form_atclose(plot1dtime->plot1dtime, DyXformAtClose, "plot1dtime");
 fl_set_form_atclose(plottrfct->plottrfct, DyXformAtClose, "plottrfct");
 fl_set_form_atclose(plotcohe->plotcohe, DyXformAtClose, "plotcohe");
 fl_set_form_atclose(plottime->plottime, DyXformAtClose, "plottime");
 fl_set_form_atclose(plot1d->plot1d, DyXformAtClose, "plot1d");
 fl_set_form_atclose(plot2d->plot2d, DyXformAtClose, "plot2d");
 fl_set_form_atclose(plotraw->plotraw, DyXformAtClose, "plotraw");
 fl_set_form_atclose(plotrawtime->plotrawtime, DyXformAtClose, "plotrawtime");
 fl_set_form_atclose(plotaudio->plotaudio, DyXformAtClose, "plotaudio");
 fl_set_form_atclose(plotgenepar->plotgenepar, DyXformAtClose, "plotgenepar");
 fl_set_form_atclose(cmservers->cmservers, DyXformAtClose, "cmservers");
 fl_set_form_atclose(getdd->getdd, DyXformAtClose, "getdd");
 fl_set_form_atclose(getshm->getshm, DyXformAtClose, "getshm");
 fl_set_form_atclose(dyhelp->dyhelp, DyXformAtClose, "dyhelp");
 fl_set_form_atclose(dydebug->dydebug, DyXformAtClose, "dydebug");
 fl_set_form_atclose(dyoptions->dyoptions, DyXformAtClose, "dyoptions");
 fl_set_form_atclose(dysliders->dysliders, DyXformAtClose, "dysliders");
 fl_set_form_atclose(dyoutput->dyoutput, DyXformAtClose, "dyoutput");
 fl_set_form_atclose(reffile->reffile, DyXformAtClose, "reffile");
/*
 fl_set_form_atclose(userprochelp, DyXformAtClose, "userprochelp");
 fl_set_form_atclose(helppanel, DyXformAtClose, "helppanel");
 fl_set_form_atclose(coffeepanel, DyXformAtClose, "coffeepanel");
*/
 return;
}

/*------------------------------------------- DyXformAtClose ---*/

int DyXformAtClose(FL_FORM *flForm, void *data)
{
 if (flForm == mainbrowser->mainbrowser) DyXformClosePanel (mainbrowser->close,MAINBROWSER);
 if (flForm == userfuncbrowser->userfuncbrowser) DyXformClosePanel (userfuncbrowser->close,USERFUNCBROWSER);
 if (flForm == plotfft->plotfft)         DyXformClosePanel (plotfft->close,PLOTFFT);
 if (flForm == plotffttime->plotffttime) DyXformClosePanel (plotffttime->close,PLOTFFTTIME);
 if (flForm == plot1dtime->plot1dtime)   DyXformClosePanel (plot1dtime->close,PLOT1DTIME);
 if (flForm == plottrfcttime->plottrfcttime) DyXformClosePanel (plottrfcttime->close,PLOTTRTIME);
 if (flForm == plotcohetime->plotcohetime)   DyXformClosePanel (plotcohetime->close,PLOTCOHETIME);
 if (flForm == plottrfct->plottrfct)     DyXformClosePanel (plottrfct->close,PLOTTR);
 if (flForm == plotcohe->plotcohe)       DyXformClosePanel (plotcohe->close,PLOTCOHE);
 if (flForm == plottime->plottime)       DyXformClosePanel (plottime->close,PLOTTIME);
 if (flForm == plot1d->plot1d)           DyXformClosePanel (plot1d->close,PLOT1D);
 if (flForm == plot2d->plot2d)           DyXformClosePanel (plot2d->close,PLOT2D);
 if (flForm == plotraw->plotraw)         DyXformClosePanel (plotraw->close,PLOTRAW);
 if (flForm == plotrawtime->plotrawtime) DyXformClosePanel (plotrawtime->close,PLOTRAWTIME);
 if (flForm == plotaudio->plotaudio)     DyXformClosePanel (plotaudio->close,PLOTAUDIO);
 if (flForm == plotgenepar->plotgenepar) DyXformClosePanel (plotgenepar->close,GENEPAR);
 if (flForm == cmservers->cmservers)     DyXformClosePanel (cmservers->close,CMSERVERS);
 if (flForm == getdd->getdd)             DyXformClosePanel (getdd->close,GETDD);
 if (flForm == getshm->getshm)           DyXformClosePanel (getshm->close,GETSHM);
 if (flForm == helppanel)                DyXformClosePanel (NULL,HELPPANEL);
 if (flForm == userprochelp)             DyXformClosePanel (NULL,USERFCTHELP);
 if (flForm == coffeepanel)              DyXformClosePanel (NULL,COFFEEPANEL);
 if (flForm == dyhelp->dyhelp)           DyXformClosePanel (dyhelp->close,DYHELP);
 if (flForm == dydebug->dydebug)         DyXformClosePanel (dydebug->close,DYDEBUG);
 if (flForm == dyoptions->dyoptions)     DyXformClosePanel (dyoptions->close,DYOPTIONS);
 if (flForm == dysliders->dysliders)     DyXformClosePanel (dysliders->close,DYSLIDERS);
 if (flForm == dyoutput->dyoutput)       DyXformClosePanel (dyoutput->close,OUTPUTDATA);
 if (flForm == reffile->reffile)         DyXformClosePanel (reffile->close,REFFILE);

 return(FL_IGNORE);
}


/*------------------------------------------- DyXformGetDebug ---*/

void DyXformGetDebug (FL_OBJECT *obj, long arg)
{
 int debugType;

 DyDebugPrint(__FUNCTION__,0);

 debugType = 0;

 if (fl_get_button(dydebug->dbg_calls))   debugType |= (1 << DBG_CALLS);
 if (fl_get_button(dydebug->dbg_speed))   debugType |= (1 << DBG_SPEED);
 if (fl_get_button(dydebug->dbg_read))    debugType |= (1 << DBG_READ);
 if (fl_get_button(dydebug->dbg_vdyplot)) debugType |= (1 << DBG_VDYPLOT);
 if (fl_get_button(dydebug->dbg_others))  debugType |= (1 << DBG_OTHERS);

 Dy->debugType = debugType;

 Dy->debug = atoi(fl_get_input(dydebug->debuglvl));
 Dy->debugSleep = atoi(fl_get_input(dydebug->debugsleep));

 return;
}


/*-------------------------------------------- DyXformSlider ---*/

void DyXformSlider (FL_OBJECT *obj, long arg)
{
 int i, numpad, type0;
 double value;
 double min, max;
 float xmin, xmax;

 if (obj == dysliders->xpslider)
   {fl_get_scrollbar_bounds (obj, &xmin, &xmax);
    value = fl_get_scrollbar_value (obj);
    value = value / (xmax-xmin);
    type0 = DyPlotGetType(RA_GetSelectedPlot());
    printf("type0=%d=%s plotname=%s\n", type0, Dy->typeName[type0], RA_GetSelectedPlot());
    numpad = 0;
    for (i=1; i<=Dy->nplot; i++)
       {
        if (Plot[i]->superposed == ON) continue;
        numpad++;
        if (Plot[i]->hidden == ON) continue;
        if (Plot[i]->type != type0) continue;
        RA_XpSlider (Plot[i]->name, value, numpad);
       }
   }

 else if (obj == dysliders->xminslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = value / (max-min);
//    for (i=1; i<=Dy->nplot; i++) RA_XminSlider (Plot[i]->name, value);
   }
 else if (obj == dysliders->xmaxslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = value / (max-min);
//    for (i=1; i<=Dy->nplot; i++) RA_XmaxSlider (Plot[i]->name, value);
   }

 else if (obj == dysliders->yminslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = value / (max-min);
    numpad = 0;
    for (i=1; i<=Dy->nplot; i++)
       {
        if (Plot[i]->superposed == ON) continue;
        numpad++;
        if (Plot[i]->hidden == ON) continue;
        if (Plot[i]->type == PLOT2D || Plot[i]->type == PLOTFFTTIME) continue;
        RA_minSlider (Plot[i]->name, value, numpad);
       }
   }

 else if (obj == dysliders->ymaxslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = value / (max-min);
    numpad = 0;
    for (i=1; i<=Dy->nplot; i++)
       {
        if (Plot[i]->superposed == ON) continue;
        numpad++;
        if (Plot[i]->hidden == ON) continue;
        if (Plot[i]->type == PLOT2D || Plot[i]->type == PLOTFFTTIME) continue;
        RA_maxSlider (Plot[i]->name, value, numpad);
       }
   }

 else if (obj == dysliders->zminslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = 1. - value / (max-min);
    numpad = 0;
    for (i=1; i<=Dy->nplot; i++)
       {
        if (Plot[i]->superposed == ON) continue;
        numpad++;
        if (Plot[i]->hidden == ON) continue;
        if (Plot[i]->type == PLOTTIME || Plot[i]->type == PLOTFFT || Plot[i]->type == PLOTCOHE || Plot[i]->type == PLOTTR) continue;
        RA_minSlider (Plot[i]->name, value, numpad);
       }
   }

 else if (obj == dysliders->zmaxslider)
   {fl_get_slider_bounds (obj, &min, &max);
    value = fl_get_slider_value (obj);
    value = value / (max-min);
    numpad = 0;
    for (i=1; i<=Dy->nplot; i++)
       {
        if (Plot[i]->superposed == ON) continue;
        numpad++;
        if (Plot[i]->hidden == ON) continue;
        if (Plot[i]->type == PLOTTIME || Plot[i]->type == PLOTFFT || Plot[i]->type == PLOTCOHE || Plot[i]->type == PLOTTR) continue;
        RA_maxSlider (Plot[i]->name, value, numpad);
       }
   }

 return;
}


/*-------------------------------------------- DyXformScanPlots ---*/

void DyXformScanPlots (FL_OBJECT *obj, long arg)
{
 double value;
 float min, max;

 fl_get_scrollbar_bounds (obj, &min, &max);
 value = fl_get_scrollbar_value (obj);
 value = value / (max-min);

 RA_ScanPlots (value);

 return;
}


/*------------------------------------------- DyXformSetMaxDuration ---*/

void DyXformSetMaxDuration (FL_OBJECT *obj, long arg)
{
 int i, nplot;
 double maxduration;
 char dummy[64];

 maxduration = 0.;

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 for (i=1; i<=nplot; i++)
    {if (maxduration < Plot[i]->timeSize) maxduration = Plot[i]->timeSize;
     if (maxduration < Plot[i]->tsize)    maxduration = Plot[i]->tsize;
    }

 sprintf(dummy,"%d",(int)maxduration);
 fl_set_input(mainpanel->duration,dummy);

 return;
}


/*------------------------------------------- DyXformOutputDefault ---*/

void DyXformOutputDefault (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);
 DyOutputDefault();
 DyXformOutputToPanel();
 return;
}


/*------------------------------------------- DyXformOutputToPanel ---*/

void DyXformOutputToPanel()
{
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);
 sprintf(dummy,"%d", Dy->output_nframe_perfile);
 fl_set_input(dyoutput->output_nframe_perfile, dummy);
 fl_set_input(dyoutput->output_dir, Dy->output_dir);
 fl_set_input(dyoutput->output_tag, Dy->output_tag);
 fl_set_button(dyoutput->output_flag, Dy->output_flag);

 return;
}


/*------------------------------------------- DyXformPanelToOutput ---*/

void DyXformPanelToOutput (FL_OBJECT *obj, long arg)
{
 long answer=1;
 DIR  *dirp;
 char dummy[LSTRING_MAX];
 const char *output_dir, *output_tag;

 DyDebugPrint(__FUNCTION__,0);

 output_tag = fl_get_input(dyoutput->output_tag);
 output_dir = fl_get_input(dyoutput->output_dir);

 if (output_tag) strcpy(Dy->output_tag, output_tag);
 else            strcpy(Dy->output_tag," ");

 if (output_dir) strcpy(Dy->output_dir, output_dir);
 else            {strcpy(Dy->output_dir, "."); fl_set_input(dyoutput->output_dir,".");}
 Dy->output_nframe_perfile = atoi(fl_get_input(dyoutput->output_nframe_perfile));
 Dy->output_flag = fl_get_button(dyoutput->output_flag);

 dirp = opendir(Dy->output_dir);
 if (dirp != NULL)
    {sprintf(dummy, "Directory %s already exists. Do you want to erase its content ?", Dy->output_dir);
     answer = fl_show_question (dummy,1);
     if (answer) {sprintf(dummy,"rm -rf %s/Dy-*.gwf", Dy->output_dir); system(dummy);}
    }

 if (arg == 0 || arg == OUTPUTDATA) DyXformClosePanel(dyoutput->close,OUTPUTDATA);

 return;
}


/*------------------------------------------- DyXformSetDefaultColors ---*/

void DyXformSetDefaultColors (FL_OBJECT *obj, long arg)
{
 char dummy[16];
 int i;
 static int grayflag = 1;

 DyDebugPrint(__FUNCTION__,0);

 if (arg == 100)
   {RA_SetGrayscale(grayflag);
    grayflag = (int)(fabs(grayflag -1));
   }

 else if (arg != -1)
   {DySetDefaultColors(arg);
    for (i=1; i<=Dy->nplot; i++) DyPlotSetColor(i);
    RA_MainCanvasUpdate();
   }

 else
   {Dy->colorset++;
    if (Dy->colorset > 1) Dy->colorset = 0;
    DySetDefaultColors(Dy->colorset);
    for (i=1; i<=Dy->nplot; i++) DyPlotSetColor(i);
    RA_MainCanvasUpdate();
    sprintf(dummy, "ColorSet=%d", Dy->colorset);
    fl_set_object_label(dyoptions->colorset,dummy);
   }

 return;
}


/*------------------------------------------- DyColorPalette ---*/

void DyColorPalette (FL_OBJECT *obj, long arg)
{
 char dummy[16];

 DyDebugPrint(__FUNCTION__,0);

 if (arg == -1) Dy->colorpalette++;
 else           Dy->colorpalette = arg;

 Dy->colorpalette = RA_SetColorPalette(Dy->colorpalette);
 sprintf(dummy, "Color2D=%d", Dy->colorpalette);
 fl_set_object_label(dyoptions->color2d,dummy);

 return;
}


/*------------------------------------------- DyXformGetButton ---*/
                                            /*  called from mainpanel */
void DyXformGetButton (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 if (obj == dyoptions->fullwhite)         {Dy->fullwhite = fl_get_button (dyoptions->fullwhite); RA_SetCanvasColor(Dy->fullwhite);}
 if (obj == dyoptions->no_autostop)        Dy->no_autostop       = fl_get_button (dyoptions->no_autostop);
 if (obj == dyoptions->squareplot)         Dy->squarePlot        = fl_get_button (dyoptions->squareplot);
 if (obj == dyoptions->nicefont)           Dy->nicefont          = fl_get_button (dyoptions->nicefont);
 if (obj == dyoptions->trigused)           Dy->triggerUsed       = fl_get_button (dyoptions->trigused);
 if (obj == dyoptions->trigpause)          Dy->triggerPause      = fl_get_button (dyoptions->trigpause);
 if (obj == dyoptions->noplot)             Dy->plotsNotDisplayed = fl_get_button (dyoptions->noplot);
 if (obj == dyoptions->autodeselectplot)   Dy->AutoDeselectPlot  = fl_get_button (dyoptions->autodeselectplot);
 if (obj == dyoptions->autodeselectch)     Dy->AutoDeselectCh    = fl_get_button (dyoptions->autodeselectch);
 if (obj == dyoptions->noBox)              Dy->noBox             = fl_get_button (dyoptions->noBox);
 if (obj == dyoutput->output_flag)        {Dy->output_flag       = fl_get_button(dyoutput->output_flag); if (Dy->output_flag == OFF) fl_hide_object(mainpanel->writing_data);} 
 if (obj == mainpanel->waitdata)           Dy->WaitData          = fl_get_button (mainpanel->waitdata);
 if (obj == dyoptions->indelebile)
    {Dy->refplot_indelebile = fl_get_button (dyoptions->indelebile);
     if (Dy->Order == STOP) {RA_RefPlotDraw(); RA_MainCanvasUpdate();}
    }

 return;
}


/*------------------------------------------- DyXformAutoDeselect ---*/
                                            /*  called from plots browser */
void DyXformAutoDeselect (FL_OBJECT *obj, long arg)
{
 int nelement, i0, i;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainbrowser->channels && Dy->AutoDeselectCh == OFF) return;
 if (obj == mainbrowser->plots && Dy->AutoDeselectPlot == OFF) return;

 i0 = fl_get_browser(obj);
 if (i0 <= 0) return;

 nelement = fl_get_browser_maxline(obj);
 for (i=0; i<nelement; i++)
    {if (fl_isselected_browser_line(obj,i+1)) fl_deselect_browser_line(obj,i+1);}

 return;
}


/*------------------------------------------ DyXformNow ---*/

void DyXformNow (FL_OBJECT *obj, long arg)
{
 char dummy[32];

 DyDebugPrint(__FUNCTION__,0);

 Dy->startTime = DyGetGPSNow();

 sprintf(dummy,"%.0f", Dy->startTime);
 fl_set_input(mainpanel->startTime, dummy);
 fl_set_input(mainpanel->startUTCdate, FrStrUTC((int)Dy->startTime,0));

 return; 
}


/*------------------------------------------ DyAudioControl ---*/

void DyAudioControl (FL_OBJECT *obj, long arg)
{
 return;
}


/*------------------------------------------ DyXformIniStartTime ---*/

void DyXformIniStartTime (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 fl_set_input (mainpanel->startTime,"0");
 fl_set_input (mainpanel->startUTCdate, "0");
 return;
}

/*------------------------------------------ DyXformSetstartTime ---*/
/* obsolete */

void DyXformSetstartTime (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);
 sprintf(dummy,"%.3f",Dy->startTime);
 fl_set_input(mainpanel->startTime, dummy);
 fl_set_input(mainpanel->startUTCdate, FrStrUTC((int)Dy->startTime,0));

 sprintf(dummy, "%.3f", Dy->duration);
 fl_set_input (mainpanel->duration, dummy);

 return;
}

/*------------------------------------------ DyXformSetGPSorDate ---*/

void DyXformSetGPSorDate (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 if (arg == 0)
    {Dy->startTime = atof(fl_get_input(mainpanel->startTime));
     fl_set_input(mainpanel->startUTCdate, FrStrUTC((int)Dy->startTime,0));
    }
 else
    {Dy->startTime = DyGTimeFromDate((char *)fl_get_input(mainpanel->startUTCdate));
     sprintf(dummy,"%f",Dy->startTime);
     fl_set_input(mainpanel->startTime, dummy);
    }

 return;
}

 
/*------------------------------------------ DyXformFilterBrowser ---*/

void DyXformFilterBrowser (FL_OBJECT *obj, long arg)
{
 const char *filtertag, *str;
 int ndel, i, j, ns, nline;
 char selectlist[128][LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 /*--- Get the concerned browser ---*/

 if (obj != mainbrowser->filterch) return;

 /*--- Get filter tag ---*/

 filtertag = fl_get_input(obj);

 ns = 0;
 nline = fl_get_browser_maxline(mainbrowser->channels);
 for (i=1; i<=nline; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->channels,i))
       {str = fl_get_browser_line(mainbrowser->channels,i);
        strcpy(selectlist[ns],str); ns++;
       }
    }

 /*--- Apply the filter on the channels list ---*/

 ndel = DyChFilter(filtertag);

 /*--- Update the channels browser ---*/

 if (ndel > 0)
    {DyXformChannelFillBrowser();
     fl_set_browser_topline(mainbrowser->channels, 1);
    }

 if (ns > 0)
   {for (j=0; j<ns; j++)
       {for (i=1; i<nline; i++)
           {str = fl_get_browser_line(mainbrowser->channels,i);
            if (strcmp(str, selectlist[j]) == 0)
              {fl_select_browser_line(mainbrowser->channels,i); break;}
           }
       }
   }

 if (((Dy->debugType)&(1<<DBG_OTHERS)))
   {printf("Channel filtering done......... %d channels found over %d \n", Dy->nch-ndel, Dy->nch);}

 return;
}


/*------------------------------------------ DyXformFindInBrowser ---*/
/*--- Search for first occurence of an input string in the browser list ---*/

void DyXformFindInBrowser (FL_OBJECT *obj, long arg)
{
 const char *varName0, *varName, *name;
 char *str, name1[LSTRING_MAX], name2[LSTRING_MAX], dummy[LSTRING_MAX];
 int i, i0, nvar, length;
 FL_OBJECT *browser;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainbrowser->searchch)        browser = mainbrowser->channels;
 else if (obj == cmservers->searchCmName) browser = cmservers->liste;
 else                                     return;

 name = fl_get_input(obj);
 nvar = fl_get_browser_maxline(browser);

 /*--- Search for a prefix in the channels names ---*/
 /* not used anymore */
/*
 if (browser == mainbrowser->channels) DyChGetPrefix();
*/

 /*--- No search, show the top of the list ---*/

 if (name == NULL)
    {fl_set_browser_topline(browser,1);
     Dy->lastSearchI = 1;
     if (browser == mainbrowser->channels) return;
    }

 /*--- Search for name in the channels list with the prefix ---*/

 else if (strstr(name,":") != NULL)
    {
     strcpy(name1,DYEMPTY);
     strcpy(name2,DYEMPTY);
     length = 0;
     i0 = -1;
     for (i=0; i<nvar; i++)
        {
         varName = fl_get_browser_line(browser, i+1);

         if (strstr(name,"*") == NULL)
            {if (strncmp(varName, name, strlen(name)) == 0) {i0 = i; break;}}
         else
            {str = strstr(name,"*");
             strcpy(name2, str+1);
             length = str-name;
             strncpy(name1,name,length); name1[length] = '\0';
             if (strncmp(varName, name1, strlen(name1)) == 0 && strstr(varName+length,name2) != 0) {i0 = i; break;}
            }
        }

     if (i0 >= 0) fl_set_browser_topline(browser, i0+1);
    }

 /*--- Search for name in the channels list without any prefix ---*/

 else
    {
     strcpy(name1,DYEMPTY);
     strcpy(name2,DYEMPTY);
     length = 0;
     i0 = -1;
     for (i=0; i<nvar; i++)
        {
         varName0 = fl_get_browser_line(browser, i+1);
         str = strstr(varName0,":");
         if (str && *(str+1) != ' ') varName = str+1;
         else                        varName = varName0;

         if (strstr(name,"*") == NULL)
            {if (strncmp(varName, name, strlen(name)) == 0) {i0 = i; break;}}
         else
            {str = strstr(name,"*");
             strcpy(name2, str+1);
             length = str-name;
             strncpy(name1,name,length); name1[length] = '\0';
             if (strncmp(varName, name1, strlen(name1)) == 0 && strstr(varName+length,name2) != 0) {i0 = i; break;}
            }
        }

     if (i0 >= 0) fl_set_browser_topline(browser, i0+1);
    }

 return;
}


/*------------------------------------------ DyXformSetScanStep ---*/

void DyXformSetScanStep (double step)
{
 fl_set_scrollbar_increment(mainpanel->scanplots, step, step);
 fl_set_scrollbar_step(mainpanel->scanplots, step);
 return;
}

/*------------------------------------------- DyXformClonePlot ---*/
                                                 /*  called from Main Browser  */
void DyXformClonePlot (FL_OBJECT *obj, long arg)
{
 int nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i))
        {Dy->nplot++;
         DyCopyPlot (Dy->nplot,i,ON);
         DyXformUpdatePlotBrowser(Dy->nplot, ADD_PLOT);
        }
    }

 return;
}

/*-------------------------------------------- DyXformRemovePlot ---*/

void DyXformRemovePlot (FL_OBJECT *obj, long arg)
{
 long nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)) Plot[i]->toberemoved = ON;
     else                                                  Plot[i]->toberemoved = OFF;
    }

 DyRemovePlot();

 return;
}

/*-------------------------------------------- DyXformGetPlotPanel ---*/
/*                 called when the "Edit Plot" button is pressed   */

void DyXformGetPlotPanel (FL_OBJECT *obj, long arg)
{
 long id, i, nplot;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 id = fl_get_browser(mainbrowser->plots);

 if (id <= 0)
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++)
        {if (fl_isselected_browser_line(mainbrowser->plots,i)) {id = i; break;}}
    }

 if (id <= 0) return;

/* Init of the Plot 0 (intermediate plot)           */
/* Fill the Plot 0 with the current plot parameters */
/* Plot 0 content is then used to know which parameters were changed in the edition panel */

 DyPlotInit(0,ON); 
 DyCopyPlot(0,id,ON);     /*--- Keep in plot 0 the values of plot id which will be displayed in panel ---*/ 
 DyXformPlotToPanel(id);  /*--- Put plot id parameters into the panel ----*/

 DyXformRaisePanel(NULL,Plot[id]->type);

 sprintf(dummy, "Editing plot%ld: name=%s  type=%s  chx->name=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type], Plot[id]->chx->name);
 DyInfoLine (dummy);

 return;
}


/*-------------------------------------------- DyXformAlignTime ---*/

void DyXformAlignTime (FL_OBJECT *obj, long arg)
{
 long id;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check that at least one line is selected in the plots browser ---*/
 /*--- If not, take the first plot as the reference one ---*/

 id = fl_get_browser(mainbrowser->plots);
 if (id <= 0) id = 1;

 /*--- Align time parameters of the plots on those of the plot selected ---*/

 DyPlotAlignTime(id);

 return;
}


/*-------------------------------------------- DyXformEditPlot ---*/
/*                                  called when OK button of plot panel is pressed  */

void DyXformEditPlot (FL_OBJECT *obj, long arg)
{
 long id, nplot, i, j;
 char plotname[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check that at least one line is selected in the plots browser ---*/

 id = fl_get_browser(mainbrowser->plots);

 /*--- Redo a check ?? ---*/

 if (id <= 0)
    {nplot = fl_get_browser_maxline(mainbrowser->plots);
     for (i=1; i<=nplot; i++) {if (fl_isselected_browser_line(mainbrowser->plots,i)) {id = i; break;}}
    }
 if (id <= 0) return;

 /*--- For each plot selected, update the parameters which were changed in the panel ---*/

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) == 0) continue;
     if (Plot[i]->type != Plot[id]->type) continue;

     strcpy(plotname,Plot[i]->chx->name);
     DyXformPanelToPlot(i);
     strcpy(Plot[i]->chx->name,plotname);

     /*--- Update the plot line in the plots browser ---*/
     DyXformUpdatePlotBrowser(i, REPLACE_PLOT);

     /*--- Copy into the plots superposed to this one the parameters of this one ---*/
     if (Plot[i]->type < 10000)
        {
         /*--- This one is the first in the list of superposed plots ---*/
         if (i < nplot && Plot[i]->superposed == OFF && Plot[i+1]->superposed == ON)
            {j=i+1;
             while (j<=nplot && Plot[j]->superposed == ON)
                {DyCopyPlotForSame(Plot[j], Plot[i]);
                 j++;
                }
            }

         /*--- Or this one is in the middle of the list of superposed plots ---*/
         /*--- In this case, overwrite the edition with the parameters taken from first plot of the list of superposed plots ---*/
         else if (Plot[i]->superposed == ON)
            {j = i-1;
             while (j>0 && Plot[j]->superposed == ON) j--;
             DyCopyPlotForSame(Plot[i], Plot[j]);
            }
        }
    }

 /*--- Deselect all the lines in the plots browser and close the plot edition panel ---*/

 DyXformAutoDeselect(mainbrowser->plots,0);
 DyXformClosePanel(obj,arg);

 return;
}


/*--------------------------------------- DyXformPlotToPanel ---*/

void DyXformPlotToPanel (long id)
{
 long plottype;
 char dummy[LSTRING_MAX];

 sprintf(dummy,"DyXformPlotToPanel> plot%ld  name=%s  type=%s", id, Plot[id]->name, Dy->typeName[Plot[id]->type]);
 DyDebugPrint(dummy,0);

 plottype = Plot[id]->type;

 if (plottype == PLOTRAW)
    {
     strcpy(dummy,"\0");
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plotraw->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);            fl_set_input (plotraw->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);            fl_set_input (plotraw->ymax, dummy);
     sprintf(dummy,"%ld",Plot[id]->maxAverage);     fl_set_input (plotraw->maxAverage, dummy);
     sprintf(dummy,"%g",Plot[id]->refreshDt);       fl_set_input (plotraw->refreshDt, dummy);
     fl_set_button (plotraw->autoscale, Plot[id]->autoscale);
     fl_set_button (plotraw->realunits, Plot[id]->chx->realunits);
     fl_set_button (plotraw->decay,     Plot[id]->decay);
     fl_set_button (plotraw->gridx,     Plot[id]->gridx);
     fl_set_button (plotraw->gridy,     Plot[id]->gridy);
     fl_set_button (plotraw->gridz,     Plot[id]->gridz);
     fl_set_button (plotraw->logx,      Plot[id]->logx);
     fl_set_button (plotraw->logy,      Plot[id]->logy);
     fl_set_button (plotraw->logz,      Plot[id]->logz);
    }

 if (plottype == PLOTRAWTIME)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plotrawtime->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);       fl_set_input (plotrawtime->zmin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);       fl_set_input (plotrawtime->zmax, dummy);
     sprintf(dummy,"%g",Plot[id]->tsize);      fl_set_input (plotrawtime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);      fl_set_input (plotrawtime->tstep, dummy);
     sprintf(dummy,"%ld",Plot[id]->maxAverage); fl_set_input (plotrawtime->maxAverage, dummy);
     fl_set_button (plotrawtime->autoscale, Plot[id]->autoscale);
     fl_set_button (plotrawtime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plotrawtime->x2av, Plot[id]->x2av); 
     fl_set_button (plotrawtime->gridx, Plot[id]->gridx);
     fl_set_button (plotrawtime->gridy, Plot[id]->gridy);
     fl_set_button (plotrawtime->gridz, Plot[id]->gridz);
     fl_set_button (plotrawtime->logx,  Plot[id]->logx);
     fl_set_button (plotrawtime->logy,  Plot[id]->logy);
     fl_set_button (plotrawtime->logz,  Plot[id]->logz);
    }

 if (plottype == PLOTTIME)
    {
     strcpy(dummy,"\0");
     sprintf(dummy,"%g",Plot[id]->tsize);           fl_set_input (plottime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plottime->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plottime->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);            fl_set_input (plottime->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);            fl_set_input (plottime->ymax, dummy);
     sprintf(dummy,"%g",Plot[id]->yoffset);         fl_set_input (plottime->yoffset, dummy);
     sprintf(dummy,"%g",Plot[id]->yscale);          fl_set_input (plottime->yscale, dummy);
     sprintf(dummy,"%g",Plot[id]->filter_fmin);     fl_set_input (plottime->filter_fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->filter_fmax);     fl_set_input (plottime->filter_fmax, dummy);
     fl_set_button (plottime->bits, Plot[id]->bits);
     fl_set_button (plottime->nodc, Plot[id]->nodc);
     fl_set_button (plottime->autoscale, Plot[id]->autoscale);
     fl_set_button (plottime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plottime->gridx, Plot[id]->gridx);
     fl_set_button (plottime->gridy, Plot[id]->gridy);
     fl_set_button (plottime->logx,  Plot[id]->logx);
     fl_set_button (plottime->logy,  Plot[id]->logy);
    }

 if (plottype == PLOTFFT)
    {
     sprintf(dummy,"%g",Plot[id]->fft_duration);    fl_set_input (plotfft->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plotfft->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plotfft->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);            fl_set_input (plotfft->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);            fl_set_input (plotfft->fmax, dummy);
     sprintf(dummy,"%ld",Plot[id]->maxAverage);     fl_set_input (plotfft->maxAverage, dummy);
     sprintf(dummy,"%ld",Plot[id]->refresh_fft);    fl_set_input (plotfft->refresh_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);            fl_set_input (plotfft->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);            fl_set_input (plotfft->ymax, dummy);
     fl_set_button (plotfft->hertz,     Plot[id]->hertz);
     fl_set_button (plotfft->rmsfft,    Plot[id]->rmsfft);
     fl_set_button (plotfft->decayfft,  Plot[id]->decayfft);
     fl_set_button (plotfft->nodc,      Plot[id]->nodc);
     fl_set_button (plotfft->realunits, Plot[id]->chx->realunits);
     fl_set_button (plotfft->autoscale, Plot[id]->autoscale);
     fl_set_button (plotfft->gridx,     Plot[id]->gridx);
     fl_set_button (plotfft->gridy,     Plot[id]->gridy);
     fl_set_button (plotfft->logx,      Plot[id]->logx);
     fl_set_button (plotfft->logy,      Plot[id]->logy);
    }

 if (plottype == PLOTTR)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq);  fl_set_input (plottrfct->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->chy->resampFreq);  fl_set_input (plottrfct->chy_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->fft_duration);     fl_set_input (plottrfct->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);            fl_set_input (plottrfct->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);             fl_set_input (plottrfct->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);             fl_set_input (plottrfct->fmax, dummy);
     sprintf(dummy,"%ld",Plot[id]->maxAverage);      fl_set_input (plottrfct->maxAverage, dummy);
     sprintf(dummy,"%ld",Plot[id]->refresh_fft);     fl_set_input (plottrfct->refresh_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);             fl_set_input (plottrfct->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);             fl_set_input (plottrfct->ymax, dummy);
     sprintf(dummy,"%g",Plot[id]->timeshift);        fl_set_input (plottrfct->timeshift, dummy);
     fl_set_button (plottrfct->nodc, Plot[id]->nodc);
     fl_set_button (plottrfct->realunits,  Plot[id]->chx->realunits);
     fl_set_button (plottrfct->autoscale,  Plot[id]->autoscale);
     fl_set_button (plottrfct->modulesqrt, Plot[id]->modulesqrt);
     fl_set_button (plottrfct->decayfft,   Plot[id]->decayfft);
     fl_set_button (plottrfct->gridx,      Plot[id]->gridx);
     fl_set_button (plottrfct->gridy,      Plot[id]->gridy);
     fl_set_button (plottrfct->logx,       Plot[id]->logx);
     fl_set_button (plottrfct->logy,       Plot[id]->logy);
     fl_set_button (plottrfct->gridx2,     Plot[id]->gridx2);
     fl_set_button (plottrfct->gridy2,     Plot[id]->gridy2);
     fl_set_button (plottrfct->logx2,      Plot[id]->logx2);
     fl_set_button (plottrfct->logy2,      Plot[id]->logy2);
    }

 if (plottype == PLOTCOHE)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq);  fl_set_input (plotcohe->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->chy->resampFreq);  fl_set_input (plotcohe->chy_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->fft_duration);     fl_set_input (plotcohe->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);            fl_set_input (plotcohe->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);             fl_set_input (plotcohe->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);             fl_set_input (plotcohe->fmax, dummy);
     sprintf(dummy,"%ld",Plot[id]->maxAverage);      fl_set_input (plotcohe->maxAverage, dummy);
     sprintf(dummy,"%ld",Plot[id]->refresh_fft);     fl_set_input (plotcohe->refresh_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);             fl_set_input (plotcohe->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);             fl_set_input (plotcohe->ymax, dummy);
     sprintf(dummy,"%g",Plot[id]->timeshift);        fl_set_input (plotcohe->timeshift, dummy);
     sprintf(dummy,"%g",Plot[id]->crossCohe_fmin);   fl_set_input (plotcohe->crossCohe_fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->crossCohe_fmax);   fl_set_input (plotcohe->crossCohe_fmax, dummy);
     fl_set_button (plotcohe->crossCohe,  Plot[id]->crossCohe);
     fl_set_button (plotcohe->nodc,       Plot[id]->nodc);
     fl_set_button (plotcohe->realunits,  Plot[id]->chx->realunits);
     fl_set_button (plotcohe->autoscale,  Plot[id]->autoscale);
     fl_set_button (plotcohe->modulesqrt, Plot[id]->modulesqrt);
     fl_set_button (plotcohe->decayfft,   Plot[id]->decayfft);
     fl_set_button (plotcohe->gridx,      Plot[id]->gridx);
     fl_set_button (plotcohe->gridy,      Plot[id]->gridy);
     fl_set_button (plotcohe->logx,       Plot[id]->logx);
     fl_set_button (plotcohe->logy,       Plot[id]->logy);
/*
     fl_set_button (plotcohe->gridx2,     Plot[id]->gridx2);
     fl_set_button (plotcohe->gridy2,     Plot[id]->gridy2);
     fl_set_button (plotcohe->logx2,      Plot[id]->logx2);
     fl_set_button (plotcohe->logy2,      Plot[id]->logy2);
*/
    }

 if (plottype == PLOTFFTTIME)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plotffttime->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dfreq);           fl_set_input (plotffttime->dfreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dtime);           fl_set_input (plotffttime->dtime, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep_fft);       fl_set_input (plotffttime->tstep_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);            fl_set_input (plotffttime->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);            fl_set_input (plotffttime->fmax, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);            fl_set_input (plotffttime->zmin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);            fl_set_input (plotffttime->zmax, dummy);
     sprintf(dummy,"%g",Plot[id]->tsize);           fl_set_input (plotffttime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plotffttime->tstep, dummy);
     fl_set_button (plotffttime->autoscale, Plot[id]->autoscale);
     fl_set_button (plotffttime->decayfft,  Plot[id]->decayfft);
     fl_set_button (plotffttime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plotffttime->nodc,      Plot[id]->nodc);
     fl_set_button (plotffttime->gridx,     Plot[id]->gridx);
     fl_set_button (plotffttime->gridy,     Plot[id]->gridy);
     fl_set_button (plotffttime->gridz,     Plot[id]->gridz);
     fl_set_button (plotffttime->logx,      Plot[id]->logx);
     fl_set_button (plotffttime->logy,      Plot[id]->logy);
     fl_set_button (plotffttime->logz,      Plot[id]->logz);
     fl_set_button (plotffttime->median_norm, Plot[id]->median_norm);
    }

 if (plottype == PLOTTRTIME) 
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plottrfcttime->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->chy->resampFreq); fl_set_input (plottrfcttime->chy_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dfreq);           fl_set_input (plottrfcttime->dfreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dtime);           fl_set_input (plottrfcttime->dtime, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep_fft);       fl_set_input (plottrfcttime->tstep_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);            fl_set_input (plottrfcttime->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);            fl_set_input (plottrfcttime->fmax, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);            fl_set_input (plottrfcttime->zmin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);            fl_set_input (plottrfcttime->zmax, dummy);
     sprintf(dummy,"%g",Plot[id]->tsize);           fl_set_input (plottrfcttime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plottrfcttime->tstep, dummy);
     fl_set_button (plottrfcttime->autoscale, Plot[id]->autoscale);
     fl_set_button (plottrfcttime->decayfft,  Plot[id]->decayfft);
     fl_set_button (plottrfcttime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plottrfcttime->nodc,      Plot[id]->nodc);
     fl_set_button (plottrfcttime->gridx,     Plot[id]->gridx);
     fl_set_button (plottrfcttime->gridy,     Plot[id]->gridy);
     fl_set_button (plottrfcttime->gridz,     Plot[id]->gridz);
     fl_set_button (plottrfcttime->logx,      Plot[id]->logx);
     fl_set_button (plottrfcttime->logy,      Plot[id]->logy);
     fl_set_button (plottrfcttime->logz,      Plot[id]->logz);
    }

 if (plottype == PLOTCOHETIME)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plotcohetime->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->chy->resampFreq); fl_set_input (plotcohetime->chy_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dfreq);           fl_set_input (plotcohetime->dfreq, dummy);
     sprintf(dummy,"%g",Plot[id]->dtime);           fl_set_input (plotcohetime->dtime, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep_fft);       fl_set_input (plotcohetime->tstep_fft, dummy);
     sprintf(dummy,"%g",Plot[id]->fmin);            fl_set_input (plotcohetime->fmin, dummy);
     sprintf(dummy,"%g",Plot[id]->fmax);            fl_set_input (plotcohetime->fmax, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);            fl_set_input (plotcohetime->zmin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);            fl_set_input (plotcohetime->zmax, dummy);
     sprintf(dummy,"%g",Plot[id]->tsize);           fl_set_input (plotcohetime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plotcohetime->tstep, dummy);
     fl_set_button (plotcohetime->autoscale, Plot[id]->autoscale);
     fl_set_button (plotcohetime->decayfft,  Plot[id]->decayfft);
     fl_set_button (plotcohetime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plotcohetime->nodc,      Plot[id]->nodc);
     fl_set_button (plotcohetime->gridx,     Plot[id]->gridx);
     fl_set_button (plotcohetime->gridy,     Plot[id]->gridy);
     fl_set_button (plotcohetime->gridz,     Plot[id]->gridz);
     fl_set_button (plotcohetime->logx,      Plot[id]->logx);
     fl_set_button (plotcohetime->logy,      Plot[id]->logy);
     fl_set_button (plotcohetime->logz,      Plot[id]->logz);
    }

 if (plottype == PLOT1DTIME)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plot1dtime->chx_resampFreq, dummy);
     sprintf(dummy,"%ld",Plot[id]->ynBin);          fl_set_input (plot1dtime->ynBin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);            fl_set_input (plot1dtime->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);            fl_set_input (plot1dtime->ymax, dummy);
     sprintf(dummy,"%g",Plot[id]->zmin);            fl_set_input (plot1dtime->zmin, dummy);
     sprintf(dummy,"%g",Plot[id]->zmax);            fl_set_input (plot1dtime->zmax, dummy);
     sprintf(dummy,"%g",Plot[id]->tsize);           fl_set_input (plot1dtime->tsize, dummy);
     sprintf(dummy,"%g",Plot[id]->tstep);           fl_set_input (plot1dtime->tstep, dummy);
     sprintf(dummy,"%g",Plot[id]->dtime);           fl_set_input (plot1dtime->stat_duration, dummy);
     fl_set_button (plot1dtime->autoedge,  Plot[id]->autoedge);
     fl_set_button (plot1dtime->autoscale, Plot[id]->autoscale);
     fl_set_button (plot1dtime->realunits, Plot[id]->chx->realunits);
     fl_set_button (plot1dtime->nodc,      Plot[id]->nodc);
     fl_set_button (plot1dtime->gridx,     Plot[id]->gridx);
     fl_set_button (plot1dtime->gridy,     Plot[id]->gridy);
     fl_set_button (plot1dtime->gridz,     Plot[id]->gridz);
     fl_set_button (plot1dtime->logx,      Plot[id]->logx);
     fl_set_button (plot1dtime->logy,      Plot[id]->logy);
     fl_set_button (plot1dtime->logz,      Plot[id]->logz);
    }

 if (plottype == PLOT1D)
    {
     sprintf(dummy,"%g",Plot[id]->refreshDt);       fl_set_input (plot1d->refreshDt, dummy);
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plot1d->chx_resampFreq, dummy);
     sprintf(dummy,"%ld",Plot[id]->xnBin);          fl_set_input (plot1d->xnBin, dummy);
     sprintf(dummy,"%g",Plot[id]->xmin);            fl_set_input (plot1d->xmin, dummy);
     sprintf(dummy,"%g",Plot[id]->xmax);            fl_set_input (plot1d->xmax, dummy);
     sprintf(dummy,"%g",Plot[id]->xoffset);         fl_set_input (plot1d->xoffset, dummy);
     sprintf(dummy,"%g",Plot[id]->xscale);          fl_set_input (plot1d->xscale, dummy);
     fl_set_button (plot1d->autoedge,  Plot[id]->autoedge);
     fl_set_button (plot1d->realunits, Plot[id]->chx->realunits);
     fl_set_button (plot1d->gridx,     Plot[id]->gridx);
     fl_set_button (plot1d->gridy,     Plot[id]->gridy);
     fl_set_button (plot1d->logx,      Plot[id]->logx);
     fl_set_button (plot1d->logy,      Plot[id]->logy);
     fl_set_button (plot1d->stat,      Plot[id]->stat);
     fl_set_button (plot1d->nodc,      Plot[id]->nodc);
    }

 if (plottype == PLOT2D)
    {
     sprintf(dummy,"%g",Plot[id]->chx->resampFreq); fl_set_input (plot2d->chx_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->chy->resampFreq); fl_set_input (plot2d->chy_resampFreq, dummy);
     sprintf(dummy,"%g",Plot[id]->refreshDt);       fl_set_input (plot2d->refreshDt, dummy);
     sprintf(dummy,"%ld",Plot[id]->xnBin);          fl_set_input (plot2d->xnBin, dummy);
     sprintf(dummy,"%g",Plot[id]->xmin);            fl_set_input (plot2d->xmin, dummy);
     sprintf(dummy,"%g",Plot[id]->xmax);            fl_set_input (plot2d->xmax, dummy);
     sprintf(dummy,"%ld",Plot[id]->ynBin);          fl_set_input (plot2d->ynBin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymin);            fl_set_input (plot2d->ymin, dummy);
     sprintf(dummy,"%g",Plot[id]->ymax);            fl_set_input (plot2d->ymax, dummy);
     sprintf(dummy,"%g",Plot[id]->timeshift);       fl_set_input (plot2d->timeshift, dummy);
     sprintf(dummy,"%g",Plot[id]->xoffset);         fl_set_input (plot2d->xoffset, dummy);
     sprintf(dummy,"%g",Plot[id]->xscale);          fl_set_input (plot2d->xscale, dummy);
     sprintf(dummy,"%g",Plot[id]->yoffset);         fl_set_input (plot2d->yoffset, dummy);
     sprintf(dummy,"%g",Plot[id]->yscale);          fl_set_input (plot2d->yscale, dummy);
     fl_set_button (plot2d->autoedge,  Plot[id]->autoedge);
     fl_set_button (plot2d->realunits, Plot[id]->chx->realunits);
     fl_set_button (plot2d->gridx,     Plot[id]->gridx);
     fl_set_button (plot2d->gridy,     Plot[id]->gridy);
     fl_set_button (plot2d->logx,      Plot[id]->logx);
     fl_set_button (plot2d->logy,      Plot[id]->logy);
     fl_set_button (plot2d->logz,      Plot[id]->logz);
     fl_set_button (plot2d->stat,      Plot[id]->stat);
     if (Plot[id]->type2d == COLZ)  fl_set_button (plot2d->col,   ON);
     if (Plot[id]->type2d == CONT)  fl_set_button (plot2d->cont,  ON);
     if (Plot[id]->type2d == LEGO)  fl_set_button (plot2d->lego,  ON);
     if (Plot[id]->type2d == SURF)  fl_set_button (plot2d->surf,  ON);
     if (Plot[id]->type2d == SCAT)  fl_set_button (plot2d->scat,  ON);
    }

 if (plottype == PLOTAUDIO)
    {
     sprintf(dummy,"%g",Plot[id]->audio_t0);       fl_set_input (plotaudio->t0, dummy);
     sprintf(dummy,"%g",Plot[id]->audio_duration); fl_set_input (plotaudio->duration, dummy);
     fl_set_button (plotaudio->sound16bits, Plot[id]->sound16bits);
     fl_set_button (plotaudio->soundonline, Plot[id]->soundonline);
    }

 return;
}


/*----------------------------------------- DyXformGetGeneParPanel ---*/

void DyXformGetGeneParPanel (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];
 int i0, id, nPlot;
 static int firstTime=1;

 DyDebugPrint(__FUNCTION__,0);

 if (firstTime == 1)
    {
     firstTime = 0;

     nPlot = fl_get_browser_maxline(mainbrowser->plots);

     i0 = nPlot;
     for (id=1; id<=nPlot; id++)
        {
         if (Plot[id]->type == PLOTTIME    || Plot[id]->type == PLOTFFT    ||
             Plot[id]->type == PLOTTR      || Plot[id]->type == PLOTCOHE   ||
             Plot[id]->type == PLOTFFTTIME || Plot[id]->type == PLOTTRTIME || Plot[id]->type == PLOTCOHETIME)
            {i0 = id; break;}
        }

     strcpy(dummy,"\0");

     sprintf(dummy,"%g",Plot[i0]->tsize);    fl_set_input (plotgenepar->tsize, dummy);
     sprintf(dummy,"%g",Plot[i0]->tstep);    fl_set_input (plotgenepar->tstep, dummy);
     sprintf(dummy,"%g",Plot[i0]->chx->resampFreq); fl_set_input (plotgenepar->sampFreq, dummy);

     fl_set_button (plotgenepar->autoedge, Plot[i0]->autoedge);
     fl_set_button (plotgenepar->realunits, Plot[i0]->chx->realunits);
     fl_set_button (plotgenepar->nodc, Plot[i0]->nodc);
     fl_set_button (plotgenepar->gridx, Plot[i0]->gridx);
     fl_set_button (plotgenepar->gridy, Plot[i0]->gridy);
     fl_set_button (plotgenepar->logx, Plot[i0]->logx);
     fl_set_button (plotgenepar->logy, Plot[i0]->logy);
    }

 Plot[0]->chx->resampFreq  = atof(fl_get_input(plotgenepar->sampFreq));
 Plot[0]->tsize     = atof(fl_get_input(plotgenepar->tsize));
 Plot[0]->tstep     = atof(fl_get_input(plotgenepar->tstep));

 Plot[0]->autoscale = fl_get_button (plotgenepar->autoscale);
 Plot[0]->autoedge  = fl_get_button (plotgenepar->autoedge);
 Plot[0]->decayfft  = fl_get_button (plotgenepar->decayfft);
 Plot[0]->nodc      = fl_get_button (plotgenepar->nodc);
 Plot[0]->gridx     = fl_get_button (plotgenepar->gridx);
 Plot[0]->gridy     = fl_get_button (plotgenepar->gridy);
 Plot[0]->logx      = fl_get_button (plotgenepar->logx);
 Plot[0]->logy      = fl_get_button (plotgenepar->logy);
 Plot[0]->chx->realunits = fl_get_button (plotgenepar->realunits);

 DyInfoLine ("Edited general parameters of all plots");
 DyXformRaisePanel(NULL,GENEPAR);

 return;
}


/*----------------------------------------- DyXformPanelToPlot ---*/
/*--- Before calling this function, always copy the plot id to the plot 0 ---*/

void DyXformPanelToPlot (int id)
{
 long plottype;
 struct DyPlot *plot;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];
 plottype = Plot[id]->type;

 if (plottype == PLOTRAW)      DyXformPanelToPLOTRAW(id);
 if (plottype == PLOTRAWTIME)  DyXformPanelToPLOTRAWTIME(id);
 if (plottype == PLOTTIME)     DyXformPanelToPLOTTIME(id);
 if (plottype == PLOTFFT)      DyXformPanelToPLOTFFT(id);
 if (plottype == PLOTTR)       DyXformPanelToPLOTTR(id);
 if (plottype == PLOTCOHE)     DyXformPanelToPLOTCOHE(id);
 if (plottype == PLOTFFTTIME)  DyXformPanelToPLOTFFTTIME(id);
 if (plottype == PLOTTRTIME)   DyXformPanelToPLOTTRTIME(id);
 if (plottype == PLOTCOHETIME) DyXformPanelToPLOTCOHETIME(id);
 if (plottype == PLOT1DTIME)   DyXformPanelToPLOT1DTIME(id);
 if (plottype == PLOT1D)       DyXformPanelToPLOT1D(id);
 if (plottype == PLOT2D)       DyXformPanelToPLOT2D(id);
 if (plottype == PLOTAUDIO)    DyXformPanelToPLOTAUDIO(id);

 /*--- Check that modified parameters are in correct range and compatible betwen each other ---*/

 DyPlotCheckParam(id);

 /*--- Update secondary parameters according to the edition of primary parameters ---*/

 DyPlotSetSecondParam(id);

 return;
}


/*----------------------------------------- DyXformPanelToPLOTRAW ---*/

void DyXformPanelToPLOTRAW (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, zmin, zmax, refreshDt;
 int maxAverage, autoscale, chxrealunits, decay;
 int gridx, gridy, gridz, logx, logy, logz;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq  = atof(fl_get_input(plotraw->chx_resampFreq));
 zmin           = atof(fl_get_input(plotraw->ymin));
 zmax           = atof(fl_get_input(plotraw->ymax));
 maxAverage     = atol(fl_get_input (plotraw->maxAverage));
 refreshDt      = atof(fl_get_input(plotraw->refreshDt));

 autoscale      = fl_get_button (plotraw->autoscale);
 chxrealunits   = fl_get_button (plotraw->realunits);
 decay          = fl_get_button (plotraw->decay);
 gridx          = fl_get_button (plotraw->gridx);
 gridy          = fl_get_button (plotraw->gridy);
 gridz          = fl_get_button (plotraw->gridz);
 logx           = fl_get_button (plotraw->logx);
 logy           = fl_get_button (plotraw->logy);
 logz           = fl_get_button (plotraw->logz);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq) plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->zmin            != zmin)          plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)          plot->zmax = zmax;
 if (Plot[0]->maxAverage      != maxAverage)    plot->maxAverage = maxAverage;
 if (Plot[0]->refreshDt       != refreshDt)     plot->refreshDt = refreshDt;

 if (Plot[0]->autoscale       != autoscale)     plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits)  plot->chx->realunits = chxrealunits;
 if (Plot[0]->decay           != decay)         plot->decay = decay;
 if (Plot[0]->gridx           != gridx)         plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)         plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)         plot->gridz = gridz;
 if (Plot[0]->logx            != logx)          plot->logx = logx;
 if (Plot[0]->logy            != logy)          plot->logy = logy;
 if (Plot[0]->logz            != logz)          plot->logz = logz;

return;
}

/*----------------------------------------- DyXformPanelToPLOTRAWTIME ---*/

void DyXformPanelToPLOTRAWTIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, ymin, ymax, zmin, zmax, tsize, tstep;
 int ynBin, maxAverage, autoscale, chxrealunits, x2av;
 int gridx, gridy, gridz, logx, logy, logz;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq   = atof(fl_get_input(plotrawtime->chx_resampFreq));
 zmin            = atof(fl_get_input(plotrawtime->zmin));
 zmax            = atof(fl_get_input(plotrawtime->zmax));
 maxAverage      = atol(fl_get_input (plotrawtime->maxAverage));
 tsize           = atof(fl_get_input(plotrawtime->tsize));
 tstep           = atof(fl_get_input(plotrawtime->tstep));

 autoscale       = fl_get_button (plotrawtime->autoscale);
 chxrealunits    = fl_get_button (plotrawtime->realunits);
 x2av            = fl_get_button (plotrawtime->x2av);
 gridx           = fl_get_button (plotrawtime->gridx);
 gridy           = fl_get_button (plotrawtime->gridy);
 gridz           = fl_get_button (plotrawtime->gridz);
 logx            = fl_get_button (plotrawtime->logx);
 logy            = fl_get_button (plotrawtime->logy);
 logz            = fl_get_button (plotrawtime->logz);
 
 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq)   plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->ynBin           != ynBin)           plot->ynBin = ynBin;
 if (Plot[0]->ymin            != ymin)            plot->ymin = ymin;
 if (Plot[0]->ymax            != ymax)            plot->ymax = ymax;
 if (Plot[0]->zmin            != zmin)            plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)            plot->zmax = zmax;
 if (Plot[0]->maxAverage      != maxAverage)      plot->maxAverage = maxAverage;
 if (Plot[0]->tsize           != tsize)           plot->tsize = tsize;
 if (Plot[0]->tstep           != tstep)           plot->tstep = tstep;

 if (Plot[0]->autoscale       != autoscale)       plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits)    plot->chx->realunits = chxrealunits;
 if (Plot[0]->x2av            != x2av)            plot->x2av = x2av;
 if (Plot[0]->gridx           != gridx)           plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)           plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)           plot->gridz = gridz;
 if (Plot[0]->logx            != logx)            plot->logx = logx;
 if (Plot[0]->logy            != logy)            plot->logy = logy;
 if (Plot[0]->logz            != logz)            plot->logz = logz;

 return;
}


/*----------------------------------------- DyXformPanelToPLOTTIME ---*/

void DyXformPanelToPLOTTIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, yoffset, yscale, tsize, tstep, ymin, ymax, filter_fmin, filter_fmax;
 int bits, nodc, autoscale, chxrealunits, gridx, gridy, logx, logy; 

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq  = atof(fl_get_input(plottime->chx_resampFreq));
 yoffset        = atof(fl_get_input(plottime->yoffset));
 yscale         = atof(fl_get_input(plottime->yscale));
 tsize          = atof(fl_get_input(plottime->tsize));
 tstep          = atof(fl_get_input(plottime->tstep));
 ymin           = atof(fl_get_input(plottime->ymin));
 ymax           = atof(fl_get_input(plottime->ymax));
 filter_fmin    = atof(fl_get_input(plottime->filter_fmin));
 filter_fmax    = atof(fl_get_input(plottime->filter_fmax));

 bits           = fl_get_button (plottime->bits);
 nodc           = fl_get_button (plottime->nodc);
 autoscale      = fl_get_button (plottime->autoscale);
 chxrealunits   = fl_get_button (plottime->realunits);
 gridx          = fl_get_button (plottime->gridx);
 gridy          = fl_get_button (plottime->gridy);
 logx           = fl_get_button (plottime->logx);
 logy           = fl_get_button (plottime->logy);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {if (Plot[0]->chx->resampFreq != chxresampFreq) plot->chx->resampFreq = chxresampFreq;
    if (Plot[0]->tsize           != tsize)         plot->tsize = tsize;
    if (Plot[0]->tstep           != tstep)         plot->tstep = tstep;
   }
 if (Plot[0]->yoffset         != yoffset)       plot->yoffset = yoffset;
 if (Plot[0]->yscale          != yscale)        plot->yscale = yscale;
 if (Plot[0]->ymin            != ymin)          plot->ymin = ymin;
 if (Plot[0]->ymax            != ymax)          plot->ymax = ymax;
 if (Plot[0]->filter_fmin     != filter_fmin)   plot->filter_fmin = filter_fmin;
 if (Plot[0]->filter_fmax     != filter_fmax)   plot->filter_fmax = filter_fmax;

 if (Plot[0]->bits            != bits)          plot->bits = bits;
 if (Plot[0]->nodc            != nodc)          plot->nodc = nodc;
 if (Plot[0]->autoscale       != autoscale)     plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits)  plot->chx->realunits = chxrealunits;
 if (Plot[0]->gridx           != gridx)         plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)         plot->gridy = gridy;
 if (Plot[0]->logx            != logx)          plot->logx = logx;
 if (Plot[0]->logy            != logy)          plot->logy = logy;

 return;
}


/*----------------------------------------- DyXformPanelToPLOTFFT ---*/

void DyXformPanelToPLOTFFT (int id)
{
 struct DyPlot *plot;
 double fft_duration, tstep, chxresampFreq, fmin, fmax, ymin, ymax;
 int maxAverage, refresh_fft;
 int hertz, rmsfft, decayfft, nodc, chxrealunits, autoscale, gridx, gridy, logx, logy;
 
 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 fft_duration    = atof(fl_get_input(plotfft->tsize));
 tstep           = atof(fl_get_input(plotfft->tstep));
 chxresampFreq   = atof(fl_get_input(plotfft->chx_resampFreq));
 fmin            = atof(fl_get_input(plotfft->fmin));
 fmax            = atof(fl_get_input(plotfft->fmax));
 maxAverage      = atol(fl_get_input (plotfft->maxAverage));
 refresh_fft     = atol(fl_get_input(plotfft->refresh_fft));
 ymin            = atof(fl_get_input(plotfft->ymin));
 ymax            = atof(fl_get_input(plotfft->ymax));
 
 hertz          = fl_get_button (plotfft->hertz);
 rmsfft         = fl_get_button (plotfft->rmsfft);
 decayfft       = fl_get_button (plotfft->decayfft);
 nodc           = fl_get_button (plotfft->nodc);
 chxrealunits   = fl_get_button (plotfft->realunits);
 autoscale      = fl_get_button (plotfft->autoscale);
 gridx          = fl_get_button (plotfft->gridx);
 gridy          = fl_get_button (plotfft->gridy);
 logx           = fl_get_button (plotfft->logx);
 logy           = fl_get_button (plotfft->logy);

/*
 if (maxAverage <= 0 && decayfft == OFF) decayfft = ON;
 if (maxAverage > 0  && decayfft == ON)  decayfft = OFF; 
*/

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {if (Plot[0]->fft_duration    != fft_duration)  plot->fft_duration = fft_duration;
    if (Plot[0]->tstep           != tstep)         plot->tstep = tstep;
    if (Plot[0]->chx->resampFreq != chxresampFreq) plot->chx->resampFreq = chxresampFreq;
   }
 if (Plot[0]->fmin            != fmin)          plot->fmin = fmin;
 if (Plot[0]->fmax            != fmax)          plot->fmax = fmax;
 if (Plot[0]->maxAverage      != maxAverage)    plot->maxAverage = maxAverage;
 if (Plot[0]->refresh_fft     != refresh_fft)   plot->refresh_fft = refresh_fft;
 if (Plot[0]->ymin            != ymin)          plot->ymin = ymin;
 if (Plot[0]->ymax            != ymax)          plot->ymax = ymax;
 
 if (Plot[0]->hertz           != hertz)         plot->hertz = hertz;
 if (Plot[0]->rmsfft          != rmsfft)        plot->rmsfft = rmsfft;
 if (Plot[0]->decayfft        != decayfft)      plot->decayfft = decayfft;
 if (Plot[0]->nodc            != nodc)          plot->nodc = nodc;
 if (Plot[0]->chx->realunits  != chxrealunits)  plot->chx->realunits = chxrealunits;
 if (Plot[0]->autoscale       != autoscale)     plot->autoscale = autoscale;
 if (Plot[0]->gridx           != gridx)         plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)         plot->gridy = gridy;
 if (Plot[0]->logx            != logx)          plot->logx = logx;
 if (Plot[0]->logy            != logy)          plot->logy = logy;

 return;
}

/*----------------------------------------- DyXformPanelToPLOTTR ---*/

void DyXformPanelToPLOTTR (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, chyresampFreq, timeshift, tsize, tstep, fmin, fmax;
 int maxAverage, refresh_fft;
 double ymin, ymax;
 int nodc, chxrealunits, chyrealunits, autoscale, modulesqrt, decayfft;
 int gridx, gridy, logx, logy;
 int gridx2, gridy2, logx2, logy2;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq  = atof(fl_get_input(plottrfct->chx_resampFreq));
 chyresampFreq  = atof(fl_get_input(plottrfct->chy_resampFreq));
 tsize          = atof(fl_get_input(plottrfct->tsize));
 tstep          = atof(fl_get_input(plottrfct->tstep));
 fmin           = atof(fl_get_input(plottrfct->fmin));
 fmax           = atof(fl_get_input(plottrfct->fmax));
 maxAverage     = atol(fl_get_input (plottrfct->maxAverage));
 refresh_fft    = atol(fl_get_input(plottrfct->refresh_fft));
 ymin           = atof(fl_get_input(plottrfct->ymin));
 ymax           = atof(fl_get_input(plottrfct->ymax));
 timeshift      = atof(fl_get_input(plottrfct->timeshift));
 
 nodc           = fl_get_button (plottrfct->nodc);
 chxrealunits   = fl_get_button (plottrfct->realunits);
 chyrealunits   = fl_get_button (plottrfct->realunits);
 autoscale      = fl_get_button (plottrfct->autoscale);
 modulesqrt     = fl_get_button (plottrfct->modulesqrt);
 decayfft       = fl_get_button (plottrfct->decayfft);
 gridx          = fl_get_button (plottrfct->gridx);
 gridy          = fl_get_button (plottrfct->gridy);
 logx           = fl_get_button (plottrfct->logx);
 logy           = fl_get_button (plottrfct->logy);
 gridx2         = fl_get_button (plottrfct->gridx2);
 gridy2         = fl_get_button (plottrfct->gridy2);
 logx2          = fl_get_button (plottrfct->logx2);
 logy2          = fl_get_button (plottrfct->logy2);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {
    if (Plot[0]->chx->sampFreq  != chxresampFreq)  plot->chx->resampFreq = chxresampFreq;
    if (Plot[0]->chy->sampFreq  != chyresampFreq)  plot->chy->resampFreq = chyresampFreq;
    if (Plot[0]->fft_duration   != tsize)          plot->fft_duration = tsize;
    if (Plot[0]->tstep          != tstep)          plot->tstep = tstep;
   }
 if (Plot[0]->fmin           != fmin)           plot->fmin = fmin;
 if (Plot[0]->fmax           != fmax)           plot->fmax = fmax;
 if (Plot[0]->maxAverage     != maxAverage)     plot->maxAverage = maxAverage;
 if (Plot[0]->refresh_fft    != refresh_fft)    plot->refresh_fft = refresh_fft;
 if (Plot[0]->ymin           != ymin)           plot->ymin = ymin;
 if (Plot[0]->ymax           != ymax)           plot->ymax = ymax;
 if (Plot[0]->timeshift      != timeshift)      plot->timeshift = timeshift;
 
 if (Plot[0]->nodc           != nodc)         plot->nodc = nodc;
 if (Plot[0]->chx->realunits != chxrealunits) plot->chx->realunits = chxrealunits;
 if (Plot[0]->chy->realunits != chyrealunits) plot->chy->realunits = chyrealunits;
 if (Plot[0]->autoscale      != autoscale)    plot->autoscale = autoscale;
 if (Plot[0]->modulesqrt     != modulesqrt)   plot->modulesqrt = modulesqrt;
 if (Plot[0]->decayfft       != decayfft)     plot->decayfft = decayfft;
 if (Plot[0]->gridx          != gridx)        plot->gridx = gridx;
 if (Plot[0]->gridy          != gridy)        plot->gridy = gridy;
 if (Plot[0]->logx           != logx)         plot->logx = logx;
 if (Plot[0]->logy           != logy)         plot->logy = logy;
 if (Plot[0]->gridx2         != gridx2)       plot->gridx2 = gridx2;
 if (Plot[0]->gridy2         != gridy2)       plot->gridy2 = gridy2;
 if (Plot[0]->logx2          != logx2)        plot->logx2 = logx2;

 return;
}

/*----------------------------------------- DyXformPanelToPLOTCOHE ---*/

void DyXformPanelToPLOTCOHE (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, chyresampFreq, timeshift, tsize, tstep, fmin, fmax;
 int maxAverage, refresh_fft;
 double ymin, ymax, crossCohe_fmin, crossCohe_fmax;
 int crossCohe, nodc, chxrealunits, chyrealunits, autoscale, modulesqrt, decayfft;
 int gridx, gridy, logx, logy;
 int gridx2, gridy2, logx2, logy2;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq  = atof(fl_get_input(plotcohe->chx_resampFreq));
 chyresampFreq  = atof(fl_get_input(plotcohe->chy_resampFreq));
 tsize          = atof(fl_get_input(plotcohe->tsize));
 tstep          = atof(fl_get_input(plotcohe->tstep));
 fmin           = atof(fl_get_input(plotcohe->fmin));
 fmax           = atof(fl_get_input(plotcohe->fmax));
 maxAverage     = atol(fl_get_input (plotcohe->maxAverage));
 refresh_fft    = atol(fl_get_input(plotcohe->refresh_fft));
 ymin           = atof(fl_get_input(plotcohe->ymin));
 ymax           = atof(fl_get_input(plotcohe->ymax));
 timeshift      = atof(fl_get_input(plotcohe->timeshift));
 crossCohe_fmin = atof(fl_get_input(plotcohe->crossCohe_fmin));
 crossCohe_fmax = atof(fl_get_input(plotcohe->crossCohe_fmax));
 
 crossCohe      = fl_get_button (plotcohe->crossCohe);
 nodc           = fl_get_button (plotcohe->nodc);
 chxrealunits   = fl_get_button (plotcohe->realunits);
 chyrealunits   = fl_get_button (plotcohe->realunits);
 autoscale      = fl_get_button (plotcohe->autoscale);
 modulesqrt     = fl_get_button (plotcohe->modulesqrt);
 decayfft       = fl_get_button (plotcohe->decayfft);
 gridx          = fl_get_button (plotcohe->gridx);
 gridy          = fl_get_button (plotcohe->gridy);
 logx           = fl_get_button (plotcohe->logx);
 logy           = fl_get_button (plotcohe->logy);
/*
 gridx2         = fl_get_button (plotcohe->gridx2);
 gridy2         = fl_get_button (plotcohe->gridy2);
 logx2          = fl_get_button (plotcohe->logx2);
 logy2          = fl_get_button (plotcohe->logy2);
*/

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {
    if (Plot[0]->chx->sampFreq  != chxresampFreq)  plot->chx->resampFreq = chxresampFreq;
    if (Plot[0]->chy->sampFreq  != chyresampFreq)  plot->chy->resampFreq = chyresampFreq;
    if (Plot[0]->fft_duration   != tsize)          plot->fft_duration = tsize;
    if (Plot[0]->tstep          != tstep)          plot->tstep = tstep;
   }
 if (Plot[0]->fmin           != fmin)           plot->fmin = fmin;
 if (Plot[0]->fmax           != fmax)           plot->fmax = fmax;
 if (Plot[0]->maxAverage     != maxAverage)     plot->maxAverage = maxAverage;
 if (Plot[0]->refresh_fft    != refresh_fft)    plot->refresh_fft = refresh_fft;
 if (Plot[0]->ymin           != ymin)           plot->ymin = ymin;
 if (Plot[0]->ymax           != ymax)           plot->ymax = ymax;
 if (Plot[0]->timeshift      != timeshift)      plot->timeshift = timeshift;
 if (Plot[0]->crossCohe_fmin != crossCohe_fmin) plot->crossCohe_fmin = crossCohe_fmin;
 if (Plot[0]->crossCohe_fmax != crossCohe_fmax) plot->crossCohe_fmax = crossCohe_fmax;
 
 if (Plot[0]->crossCohe      != crossCohe)    plot->crossCohe = crossCohe;
 if (Plot[0]->nodc           != nodc)         plot->nodc = nodc;
 if (Plot[0]->chx->realunits != chxrealunits) plot->chx->realunits = chxrealunits;
 if (Plot[0]->chy->realunits != chyrealunits) plot->chy->realunits = chyrealunits;
 if (Plot[0]->autoscale      != autoscale)    plot->autoscale = autoscale;
 if (Plot[0]->modulesqrt     != modulesqrt)   plot->modulesqrt = modulesqrt;
 if (Plot[0]->decayfft       != decayfft)     plot->decayfft = decayfft;
 if (Plot[0]->gridx          != gridx)        plot->gridx = gridx;
 if (Plot[0]->gridy          != gridy)        plot->gridy = gridy;
 if (Plot[0]->logx           != logx)         plot->logx = logx;
 if (Plot[0]->logy           != logy)         plot->logy = logy;
 if (Plot[0]->gridx2         != gridx2)       plot->gridx2 = gridx2;
 if (Plot[0]->gridy2         != gridy2)       plot->gridy2 = gridy2;
 if (Plot[0]->logx2          != logx2)        plot->logx2 = logx2;
 if (Plot[0]->logy2          != logy2)        plot->logy2 = logy2;

 return;
}

/*----------------------------------------- DyXformPanelToPLOTFFTTIME ---*/

void DyXformPanelToPLOTFFTTIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, dtime, dfreq, zmin, zmax, fmin, fmax;
 int autoscale, chxrealunits, nodc, median_norm;
 int gridx, gridy, gridz, logx, logy, logz;
 double tsize, tstep, tstep_fft;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq   = atof(fl_get_input(plotffttime->chx_resampFreq));
 dfreq           = atof(fl_get_input(plotffttime->dfreq));
 dtime           = atof(fl_get_input(plotffttime->dtime));
 tstep_fft       = atof(fl_get_input(plotffttime->tstep_fft));
 fmin            = atof(fl_get_input(plotffttime->fmin));
 fmax            = atof(fl_get_input(plotffttime->fmax));
 tsize           = atof(fl_get_input(plotffttime->tsize));
 tstep           = atof(fl_get_input(plotffttime->tstep));
 zmin            = atof(fl_get_input(plotffttime->zmin));
 zmax            = atof(fl_get_input(plotffttime->zmax));

 autoscale       = fl_get_button (plotffttime->autoscale);
 chxrealunits    = fl_get_button (plotffttime->realunits);
 nodc            = fl_get_button (plotffttime->nodc);
 median_norm     = fl_get_button (plotffttime->median_norm);
 gridx           = fl_get_button (plotffttime->gridx);
 gridy           = fl_get_button (plotffttime->gridy);
 gridz           = fl_get_button (plotffttime->gridz);
 logx            = fl_get_button (plotffttime->logx);
 logy            = fl_get_button (plotffttime->logy);
 logz            = fl_get_button (plotffttime->logz);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {
    if (Plot[0]->chx->resampFreq != chxresampFreq)   plot->chx->resampFreq = chxresampFreq;
    if (Plot[0]->dfreq           != dfreq)           plot->dfreq = dfreq;
    if (Plot[0]->dtime           != dtime)           plot->dtime = dtime;
    if (Plot[0]->tstep_fft       != tstep_fft)       plot->tstep_fft = tstep_fft;
   }
 if (Plot[0]->fmin            != fmin)            plot->fmin = fmin;
 if (Plot[0]->fmax            != fmax)            plot->fmax = fmax;
 if (Plot[0]->tsize           != tsize)           plot->tsize = tsize;
 if (Plot[0]->tstep           != tstep)           plot->tstep = tstep;
 if (Plot[0]->zmin            != zmin)            plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)            plot->zmax = zmax;

 if (Plot[0]->autoscale       != autoscale)       plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits)    plot->chx->realunits = chxrealunits;
 if (Plot[0]->nodc            != nodc)            plot->nodc = nodc;
 if (Plot[0]->median_norm     != median_norm)     plot->median_norm = median_norm;
 if (Plot[0]->gridx           != gridx)           plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)           plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)           plot->gridz = gridz;
 if (Plot[0]->logx            != logx)            plot->logx = logx;
 if (Plot[0]->logy            != logy)            plot->logy = logy;
 if (Plot[0]->logz            != logz)            plot->logz = logz;

 return;
}


/*----------------------------------------- DyXformPanelToPLOTTRTIME ---*/

void DyXformPanelToPLOTTRTIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, chyresampFreq, dtime, dfreq, zmin, zmax, fmin, fmax;
 int autoscale, chxrealunits, chyrealunits, nodc;
 int gridx, gridy, gridz, logx, logy, logz;
 double tsize, tstep, tstep_fft;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq   = atof(fl_get_input(plottrfcttime->chx_resampFreq));
 chyresampFreq   = atof(fl_get_input(plottrfcttime->chy_resampFreq));
 dtime           = atof(fl_get_input(plottrfcttime->dtime));
 dfreq           = atof(fl_get_input(plottrfcttime->dfreq));
 tstep_fft       = atof(fl_get_input(plottrfcttime->tstep_fft));
 fmin            = atof(fl_get_input(plottrfcttime->fmin));
 fmax            = atof(fl_get_input(plottrfcttime->fmax));
 tsize           = atof(fl_get_input(plottrfcttime->tsize));
 tstep           = atof(fl_get_input(plottrfcttime->tstep));
 zmin            = atof(fl_get_input(plottrfcttime->zmin));
 zmax            = atof(fl_get_input(plottrfcttime->zmax));

 autoscale      = fl_get_button (plottrfcttime->autoscale);
 chxrealunits   = fl_get_button (plottrfcttime->realunits);
 chyrealunits   = fl_get_button (plottrfcttime->realunits);
 nodc           = fl_get_button (plottrfcttime->nodc);
 gridx          = fl_get_button (plottrfcttime->gridx);
 gridy          = fl_get_button (plottrfcttime->gridy);
 gridz          = fl_get_button (plottrfcttime->gridz);
 logx           = fl_get_button (plottrfcttime->logx);
 logy           = fl_get_button (plottrfcttime->logy);
 logz           = fl_get_button (plottrfcttime->logz);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq)   plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->chy->resampFreq != chyresampFreq)   plot->chy->resampFreq = chyresampFreq;
 if (Plot[0]->dtime           != dtime)           plot->dtime = dtime;
 if (Plot[0]->dfreq           != dfreq)           plot->dfreq = dfreq;
 if (Plot[0]->tstep_fft       != tstep_fft)       plot->tstep_fft = tstep_fft;
 if (Plot[0]->fmin            != fmin)            plot->fmin = fmin;
 if (Plot[0]->fmax            != fmax)            plot->fmax = fmax;
 if (Plot[0]->tsize           != tsize)           plot->tsize = tsize;
 if (Plot[0]->tstep           != tstep)           plot->tstep = tstep;
 if (Plot[0]->zmin            != zmin)            plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)            plot->zmax = zmax;

 if (Plot[0]->autoscale       != autoscale)    plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits) plot->chx->realunits = chxrealunits;
 if (Plot[0]->chy->realunits  != chyrealunits) plot->chy->realunits = chyrealunits;
 if (Plot[0]->nodc            != nodc)         plot->nodc = nodc;
 if (Plot[0]->gridx           != gridx)        plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)        plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)        plot->gridz = gridz;
 if (Plot[0]->logx            != logx)         plot->logx = logx;
 if (Plot[0]->logy            != logy)         plot->logy = logy;
 if (Plot[0]->logz            != logz)         plot->logz = logz;

 return;
}

/*----------------------------------------- DyXformPanelToPLOTCOHETIME ---*/

void DyXformPanelToPLOTCOHETIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, chyresampFreq, dtime, dfreq, zmin, zmax, fmin, fmax;
 int autoscale, chxrealunits, chyrealunits, nodc;
 int gridx, gridy, gridz, logx, logy, logz;
 double tsize, tstep, tstep_fft;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq   = atof(fl_get_input(plotcohetime->chx_resampFreq));
 chyresampFreq   = atof(fl_get_input(plotcohetime->chy_resampFreq));
 dtime           = atof(fl_get_input(plotcohetime->dtime));
 dfreq           = atof(fl_get_input(plotcohetime->dfreq));
 tstep_fft       = atof(fl_get_input(plotcohetime->tstep_fft));
 fmin            = atof(fl_get_input(plotcohetime->fmin));
 fmax            = atof(fl_get_input(plotcohetime->fmax));
 tsize           = atof(fl_get_input(plotcohetime->tsize));
 tstep           = atof(fl_get_input(plotcohetime->tstep));
 zmin            = atof(fl_get_input(plotcohetime->zmin));
 zmax            = atof(fl_get_input(plotcohetime->zmax));

 autoscale       = fl_get_button (plotcohetime->autoscale);
 chxrealunits    = fl_get_button (plotcohetime->realunits);
 chyrealunits    = fl_get_button (plotcohetime->realunits);
 nodc            = fl_get_button (plotcohetime->nodc);
 gridx           = fl_get_button (plotcohetime->gridx);
 gridy           = fl_get_button (plotcohetime->gridy);
 gridz           = fl_get_button (plotcohetime->gridz);
 logx            = fl_get_button (plotcohetime->logx);
 logy            = fl_get_button (plotcohetime->logy);
 logz            = fl_get_button (plotcohetime->logz);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq)   plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->chy->resampFreq != chyresampFreq)   plot->chy->resampFreq = chyresampFreq;
 if (Plot[0]->dtime           != dtime)           plot->dtime = dtime;
 if (Plot[0]->dfreq           != dfreq)           plot->dfreq = dfreq;
 if (Plot[0]->tstep_fft       != tstep_fft)       plot->tstep_fft = tstep_fft;
 if (Plot[0]->fmin            != fmin)            plot->fmin = fmin;
 if (Plot[0]->fmax            != fmax)            plot->fmax = fmax;
 if (Plot[0]->tsize           != tsize)           plot->tsize = tsize;
 if (Plot[0]->tstep           != tstep)           plot->tstep = tstep;
 if (Plot[0]->zmin            != zmin)            plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)            plot->zmax = zmax;

 if (Plot[0]->autoscale       != autoscale)    plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits) plot->chx->realunits = chxrealunits;
 if (Plot[0]->chy->realunits  != chyrealunits) plot->chy->realunits = chyrealunits;
 if (Plot[0]->nodc            != nodc)         plot->nodc = nodc;
 if (Plot[0]->gridx           != gridx)        plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)        plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)        plot->gridz = gridz;
 if (Plot[0]->logx            != logx)         plot->logx = logx;
 if (Plot[0]->logy            != logy)         plot->logy = logy;
 if (Plot[0]->logz            != logz)         plot->logz = logz;

 return;
}


/*----------------------------------------- DyXformPanelToPLOT1DTIME ---*/

void DyXformPanelToPLOT1DTIME (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, dtime, zmin, zmax, ymin, ymax;
 int ynBin, autoedge, autoscale, chxrealunits, nodc;
 int gridx, gridy, gridz, logx, logy, logz;
 double tsize, tstep;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq   = atof(fl_get_input(plot1dtime->chx_resampFreq));
 dtime           = atof(fl_get_input(plot1dtime->stat_duration));
 ynBin           = atoi(fl_get_input(plot1dtime->ynBin));
 tsize           = atof(fl_get_input(plot1dtime->tsize));
 ymin            = atof(fl_get_input(plot1dtime->ymin));
 ymax            = atof(fl_get_input(plot1dtime->ymax));
 zmin            = atof(fl_get_input(plot1dtime->zmin));
 zmax            = atof(fl_get_input(plot1dtime->zmax));
 tstep           = atof(fl_get_input(plot1dtime->tstep));

 autoedge        = fl_get_button (plot1dtime->autoedge);
 autoscale       = fl_get_button (plot1dtime->autoscale);
 chxrealunits    = fl_get_button (plot1dtime->realunits);
 nodc            = fl_get_button (plot1dtime->nodc);
 gridx           = fl_get_button (plot1dtime->gridx);
 gridy           = fl_get_button (plot1dtime->gridy);
 gridz           = fl_get_button (plot1dtime->gridz);
 logx            = fl_get_button (plot1dtime->logx);
 logy            = fl_get_button (plot1dtime->logy);
 logz            = fl_get_button (plot1dtime->logz);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq)   plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->dtime           != dtime)           plot->dtime = dtime;
 if (Plot[0]->ynBin           != ynBin)           plot->ynBin = ynBin;
 if (Plot[0]->tsize           != tsize)           plot->tsize = tsize;
 if (Plot[0]->ymin            != ymin)            plot->ymin = ymin;
 if (Plot[0]->ymax            != ymax)            plot->ymax = ymax;
 if (Plot[0]->zmin            != zmin)            plot->zmin = zmin;
 if (Plot[0]->zmax            != zmax)            plot->zmax = zmax;
 if (Plot[0]->tstep           != tstep)           plot->tstep = tstep;
 if (Plot[0]->autoedge        != autoedge)        plot->autoedge = autoedge;
 if (Plot[0]->autoscale       != autoscale)       plot->autoscale = autoscale;
 if (Plot[0]->chx->realunits  != chxrealunits)    plot->chx->realunits = chxrealunits;
 if (Plot[0]->nodc            != nodc)            plot->nodc = nodc;
 if (Plot[0]->gridx           != gridx)           plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)           plot->gridy = gridy;
 if (Plot[0]->gridz           != gridz)           plot->gridz = gridz;
 if (Plot[0]->logx            != logx)            plot->logx = logx;
 if (Plot[0]->logy            != logy)            plot->logy = logy;
 if (Plot[0]->logz            != logz)            plot->logz = logz;

 return;
}


/*----------------------------------------- DyXformPanelToPLOT1D ---*/

void DyXformPanelToPLOT1D (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, xmin, xmax, xoffset, xscale, refreshDt;
 int xnBin, autoedge, chxrealunits, gridx, gridy, logx, logy, stat, nodc;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];
 
 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq = atof(fl_get_input(plot1d->chx_resampFreq));
 xnBin         = atol(fl_get_input(plot1d->xnBin));
 xmin          = atof(fl_get_input(plot1d->xmin));
 xmax          = atof(fl_get_input(plot1d->xmax));
 xoffset       = atof(fl_get_input(plot1d->xoffset));
 xscale        = atof(fl_get_input(plot1d->xscale));
 refreshDt     = atof(fl_get_input(plot1d->refreshDt));

 autoedge      = fl_get_button (plot1d->autoedge);
 chxrealunits  = fl_get_button (plot1d->realunits);
 gridx         = fl_get_button (plot1d->gridx);
 gridy         = fl_get_button (plot1d->gridy);
 logx          = fl_get_button (plot1d->logx);
 logy          = fl_get_button (plot1d->logy);
 stat          = fl_get_button(plot1d->stat);
 nodc          = fl_get_button(plot1d->nodc);
 
 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (plot->chx->vect2d_duration == 0) /* edition of these parameters forbidden for 2d time vectors */
   {
    if (Plot[0]->chx->resampFreq != chxresampFreq) plot->chx->resampFreq = chxresampFreq;
    if (Plot[0]->refreshDt       != refreshDt)     plot->refreshDt = refreshDt;
   }
 if (Plot[0]->xnBin           != xnBin)         plot->xnBin = xnBin;
 if (Plot[0]->xmin            != xmin)          plot->xmin = xmin;
 if (Plot[0]->xmax            != xmax)          plot->xmax = xmax;
 if (Plot[0]->xoffset         != xoffset)       plot->xoffset = xoffset;
 if (Plot[0]->xscale          != xscale)        plot->xscale = xscale;

 if (Plot[0]->autoedge        != autoedge)      plot->autoedge = autoedge;
 if (Plot[0]->chx->realunits  != chxrealunits)  plot->chx->realunits = chxrealunits;
 if (Plot[0]->gridx           != gridx)         plot->gridx = gridx;
 if (Plot[0]->gridy           != gridy)         plot->gridy = gridy;
 if (Plot[0]->logx            != logx)          plot->logx = logx;
 if (Plot[0]->logy            != logy)          plot->logy = logy;
 if (Plot[0]->stat            != stat)          plot->stat = stat;
 if (Plot[0]->nodc            != nodc)          plot->nodc = nodc;

 return;
}


/*----------------------------------------- DyXformPanelToPLOT2D ---*/

void DyXformPanelToPLOT2D (int id)
{
 struct DyPlot *plot;
 double chxresampFreq, chyresampFreq, timeshift, xmin, xmax, ymin, ymax, refreshDt;
 double xscale, xoffset, yscale, yoffset;
 int xnBin, ynBin, autoedge, chxrealunits, chyrealunits;
 int type2d, gridx, gridy, logx, logy, logz, stat;
 
 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 chxresampFreq  = atof(fl_get_input(plot2d->chx_resampFreq));
 chyresampFreq  = atof(fl_get_input(plot2d->chy_resampFreq));
 xnBin          = atol(fl_get_input(plot2d->xnBin));
 xmin           = atof(fl_get_input(plot2d->xmin));
 xmax           = atof(fl_get_input(plot2d->xmax));
 ynBin          = atol(fl_get_input(plot2d->ynBin));
 ymin           = atof(fl_get_input(plot2d->ymin));
 ymax           = atof(fl_get_input(plot2d->ymax));
 refreshDt      = atof(fl_get_input(plot2d->refreshDt));
 timeshift      = atof(fl_get_input(plot2d->timeshift));
 xscale         = atof(fl_get_input(plot2d->xscale));
 xoffset        = atof(fl_get_input(plot2d->xoffset));
 yscale         = atof(fl_get_input(plot2d->yscale));
 yoffset        = atof(fl_get_input(plot2d->yoffset));

 autoedge       = fl_get_button (plot2d->autoedge);
 chxrealunits   = fl_get_button (plot2d->realunits);
 chyrealunits   = fl_get_button (plot2d->realunits);
 gridx          = fl_get_button (plot2d->gridx);
 gridy          = fl_get_button (plot2d->gridy);
 logx           = fl_get_button (plot2d->logx);
 logy           = fl_get_button (plot2d->logy);
 logz           = fl_get_button (plot2d->logz);
 stat           = fl_get_button (plot2d->stat);
 
 if (fl_get_button (plot2d->col))  type2d = COLZ;
 if (fl_get_button (plot2d->cont)) type2d = CONT;
 if (fl_get_button (plot2d->lego)) type2d = LEGO;
 if (fl_get_button (plot2d->surf)) type2d = SURF;
 if (fl_get_button (plot2d->scat)) type2d = SCAT;

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->chx->resampFreq != chxresampFreq) plot->chx->resampFreq = chxresampFreq;
 if (Plot[0]->chy->resampFreq != chyresampFreq) plot->chy->resampFreq = chyresampFreq;
 if (Plot[0]->xnBin           != xnBin)         plot->xnBin = xnBin;
 if (Plot[0]->xmin            != xmin)          plot->xmin = xmin;
 if (Plot[0]->xmax            != xmax)          plot->xmax = xmax;
 if (Plot[0]->ynBin           != ynBin)         plot->ynBin = ynBin;
 if (Plot[0]->ymin            != ymin)          plot->ymin = ymin;
 if (Plot[0]->ymax            != ymax)          plot->ymax = ymax;
 if (Plot[0]->refreshDt       != refreshDt)     plot->refreshDt = refreshDt;
 if (Plot[0]->timeshift       != timeshift)     plot->timeshift = timeshift;

 if (Plot[0]->autoedge       != autoedge)       plot->autoedge = autoedge;
 if (Plot[0]->chx->realunits != chxrealunits)   plot->chx->realunits = chxrealunits;
 if (Plot[0]->chy->realunits != chyrealunits)   plot->chy->realunits = chyrealunits;
 if (Plot[0]->gridx          != gridx)          plot->gridx = gridx;
 if (Plot[0]->gridy          != gridy)          plot->gridy = gridy;
 if (Plot[0]->logx           != logx)           plot->logx = logx;
 if (Plot[0]->logy           != logy)           plot->logy = logy;
 if (Plot[0]->logz           != logz)           plot->logz = logz;
 if (Plot[0]->stat           != stat)           plot->stat = stat;
 if (Plot[0]->type2d         != type2d)         plot->type2d = type2d;

 return;
}


/*----------------------------------------- DyXformPanelToPLOTAUDIO ---*/

void DyXformPanelToPLOTAUDIO (int id)
{
 struct DyPlot *plot;
 double audio_t0, audio_duration;
 int sound16bits, soundonline;

 DyDebugPrint(__FUNCTION__,0);

 plot = Plot[id];

 /*--- Put in temporary values the parameters taken from the panel ---*/

 audio_t0       = atof(fl_get_input(plotaudio->t0));
 audio_duration = atof(fl_get_input(plotaudio->duration));
 sound16bits    = fl_get_button (plotaudio->sound16bits);
 soundonline    = fl_get_button (plotaudio->soundonline);

 /*--- Update in plot only the parameters which have been changed by the edition ---*/

 if (Plot[0]->audio_t0       != audio_t0)       plot->audio_t0 = audio_t0;
 if (Plot[0]->audio_duration != audio_duration) plot->audio_duration = audio_duration;
 if (Plot[0]->sound16bits    != sound16bits)    plot->sound16bits = sound16bits;
 if (Plot[0]->soundonline    != soundonline)    plot->soundonline = soundonline;

 return;
}


/*--------------------------------------------------- DyXformSuperposePlot ---*/
/*   Called when Superpose button is pushed.                             */
/* Superpose to the first plot selected all the other plots selected     */

void DyXformSuperposePlot (FL_OBJECT *obj, long arg)
{
 long nplot, i; 

 DyDebugPrint(__FUNCTION__, 0);

 if (fl_get_browser(mainbrowser->plots) == 0) {DyInfoLine ("No plot selected"); return;}

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 /*--- Flag the plots selected in the browser ---*/

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)) Plot[i]->toberemoved = ON;
     else                                                  Plot[i]->toberemoved = OFF;
    }

 /*--- Do the superpose operation ---*/

 DySuperposePlot ();

 /*--- And finally deselect all the lines of the plots browser ---*/

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*------------------------------------------------------------ DyXformUnsuperposePlot ---*/
/*   Called when button Unsuperposed is pushed. Unsuperposed all the selected plots */

void DyXformUnsuperposePlot (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == ON)
        {Plot[i]->superposed = OFF;
         Plot[i]->idsame = i;
         DyPlotSetBrowserline(i,strnew);
         fl_replace_browser_line(mainbrowser->plots, i, strnew);
        }
    }

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*-------------------------------------------- DyXformExchangeVar ---*/
/*                  Exchange the 2 variables of the selected plots   */

void DyXformExchangeVar (FL_OBJECT *obj, long arg)
{
 struct DyCh *chdum;
 char strnew[LSTRING_MAX], dummy[LSTRING_MAX];
 long nplot, i;
 double sampfreq_dummy;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) &&
         (Plot[i]->type == PLOTTR || Plot[i]->type == PLOTCOHE || Plot[i]->type == PLOT2D || 
          Plot[i]->type == PLOTTRTIME || Plot[i]->type == PLOTCOHETIME) )
        {
         chdum = DyChClone(Plot[i]->chx);
         DyChCopy(chdum, Plot[i]->chx);
         DyChCopy(Plot[i]->chx, Plot[i]->chy);
         DyChCopy(Plot[i]->chy, chdum);
         DyChFree(chdum);
/***
         strcpy(dummy, Plot[i]->chx->name);
         strcpy(Plot[i]->chx->name, Plot[i]->chy->name);
         strcpy(Plot[i]->chy->name, dummy);

         sampfreq_dummy         = Plot[i]->chx->sampFreq;
         Plot[i]->chx->sampFreq = Plot[i]->chy->sampFreq;
         Plot[i]->chy->sampFreq = sampfreq_dummy;

         sampfreq_dummy         = Plot[i]->chx->resampFreq;
         Plot[i]->chx->resampFreq = Plot[i]->chy->resampFreq;
         Plot[i]->chy->resampFreq = sampfreq_dummy;
***/
         DyPlotSetTitle(i);
         DyPlotSetBrowserline(i,strnew);
         fl_replace_browser_line(mainbrowser->plots, i, strnew);
        }
    }

 DyXformAutoDeselect(mainbrowser->plots,0);

 return;
}


/*-------------------------------------------- DyXformPlotUp ---*/
/*                            Move upwards the selected plots   */

void DyXformPlotUp (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, j, flag, upMade;
 struct DyPlot *dummyplot;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot-1; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == OFF)
        {for (j=i+1; j<=nplot; j++)
            {if (Plot[j]->type == Plot[i]->type && Plot[j]->superposed == ON)
                {fl_select_browser_line(mainbrowser->plots,j);}
             else break;
            }
        }
    }

 flag = 1;
 upMade = 1;
 while(flag > 0 && upMade > 0)
    {
     upMade = 0;
     for (i=2; i<=nplot; i++)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
             fl_isselected_browser_line(mainbrowser->plots,i-1) == 0)
            {
             upMade = 1;
             if (Plot[i-1]->superposed == OFF) flag = 0;

             dummyplot = Plot[i];
             Plot[i] = Plot[i-1];
             Plot[i-1] = dummyplot;

             DyPlotSetBrowserline(i,strnew);
             fl_replace_browser_line(mainbrowser->plots, i, strnew);

             DyPlotSetBrowserline(i-1,strnew);
             fl_replace_browser_line(mainbrowser->plots, i-1, strnew);

             fl_deselect_browser_line(mainbrowser->plots,i);
             fl_select_browser_line(mainbrowser->plots,i-1);
            }
        }
    }

 return;
}


/*-------------------------------------------- DyXformPlotDown ---*/
/*                            Move downwards the selected plots   */

void DyXformPlotDown (FL_OBJECT *obj, long arg)
{
 char strnew[LSTRING_MAX];
 long nplot, i, j, flag, downMade;
 struct DyPlot *dummyplot;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot <= 0) return;

 for (i=1; i<=nplot-1; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) && Plot[i]->superposed == OFF)
        {for (j=i+1; j<=nplot; j++)
            {if (Plot[j]->type == Plot[i]->type && Plot[j]->superposed == ON)
                {fl_select_browser_line(mainbrowser->plots,j);}
             else break;
            }
        }
    }

 for (i=nplot-1; i>=1; i--)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
         fl_isselected_browser_line(mainbrowser->plots,i+1) == 0)
        {
         dummyplot = Plot[i];
         Plot[i] = Plot[i+1];
         Plot[i+1] = dummyplot;

         DyPlotSetBrowserline(i,strnew);
         fl_replace_browser_line(mainbrowser->plots, i, strnew);

         DyPlotSetBrowserline(i+1,strnew);
         fl_replace_browser_line(mainbrowser->plots, i+1, strnew);

         fl_deselect_browser_line(mainbrowser->plots,i);
         fl_select_browser_line(mainbrowser->plots,i+1);
        }
    }

 flag = 1;
 downMade = 1;
 while(flag > 0 && downMade > 0)
    {
     downMade = 0;
     for (i=nplot-1; i>=1; i--)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i) != 0 && 
             fl_isselected_browser_line(mainbrowser->plots,i+1) == 0)
            {
             downMade = 1;
             if (Plot[i+1]->superposed == OFF) {flag = 0; continue;}

             dummyplot = Plot[i];
             Plot[i] = Plot[i+1];
             Plot[i+1] = dummyplot;

             DyPlotSetBrowserline(i,strnew);
             fl_replace_browser_line(mainbrowser->plots, i, strnew);

             DyPlotSetBrowserline(i+1,strnew);
             fl_replace_browser_line(mainbrowser->plots, i+1, strnew);

             fl_deselect_browser_line(mainbrowser->plots,i);
             fl_select_browser_line(mainbrowser->plots,i+1);
            }
        }
    }

 return;
}

/*---------------------------------------- DyXformSetVar -----------*/

void DyXformSetVar (FL_OBJECT *obj, long iselect)
{
 const char *str=NULL;
 char dummy[LSTRING_MAX];
 char chname[LSTRING_MAX], chinput[LSTRING_MAX];
 double chfreq;
 int i;

 DyDebugPrint(__FUNCTION__,0);

 str = fl_get_browser_line(obj,iselect);
 sscanf(str,"%s",dummy);
 sscanf(str,"%s %lf %s",chname, &chfreq, chinput);

 /*--- Extract one channel ---*/
 /*--- Find the right value of iselect ---*/

 for (i=iselect-1; i<Dy->nch; i++)
    {if (strcmp(Dy->ch[i]->name, chname) == 0 && strcmp(Dy->ch[i]->inputname, chinput) == 0) {iselect = i+1; break;}}
 for (i=iselect-1; i>=0; i--)
    {if (strcmp(Dy->ch[i]->name, chname) == 0 && strcmp(Dy->ch[i]->inputname, chinput) == 0) {iselect = i+1; break;}}

 Dy->selectedCh = Dy->ch[iselect-1];

 return;
}


/*------------------------------------------- DyXformChSort ---*/

void DyXformChSort (FL_OBJECT *obj, long sorttype)
{
 char dummy[LSTRING_MAX];
 long i, i0;

 DyDebugPrint(__FUNCTION__,0);

 if (sorttype == 0) DyChSort();
 if (sorttype == 1) DyChSortByFreq();

 DyXformChannelFillBrowser();

 return;
}


/*------------------------------------------- DyXformChannelFillBrowser ---*/

void DyXformChannelFillBrowser ()
{
 char dummy[LSTRING_MAX], sourcelist[LBIGSTRING_MAX];
 long i, i0;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Remove mainbrowser panel if already open, because adding many new channels takes otherwise too much time ---*/

/**
 if (mainbrowser->mainbrowser && fl_form_is_visible(mainbrowser->mainbrowser)) DyXformClosePanel(mainbrowser->close,MAINBROWSER);
**/
 fl_freeze_form(mainbrowser->mainbrowser);

 /*--- Check if we read several files at the same time ---*/

 i0 = -1;
 strcpy(sourcelist,Dy->ch[0]->inputname);
 for (i=1; i<Dy->nch; i++)
    {if (strcmp(Dy->ch[i]->inputname,Dy->ch[0]->inputname) != 0)
       {i0 = i;
        strcat(sourcelist,"  ");
        strcat(sourcelist,Dy->ch[i]->inputname);
        strcat(sourcelist,"\0");
        break;
       } 
    }

 /*--- Clear the channels browser ---*/

 if (Dy->nch > 0) fl_clear_browser(mainbrowser->channels);

 /*--- Show last input on the mainpanel ---*/

 if (Dy->nch > 0) fl_set_input(mainpanel->currentsource, sourcelist);

 /*--- Fill the channels browser ---*/

 for (i=0; i<Dy->nch; i++)
    {if (i0 == -1) sprintf(dummy,"%s : %.2fHz", Dy->ch[i]->name, Dy->ch[i]->sampFreq);
     else          sprintf(dummy,"%s : %.2fHz  %s", Dy->ch[i]->name, Dy->ch[i]->sampFreq, Dy->ch[i]->inputname);
     fl_add_browser_line(mainbrowser->channels,dummy);
    }

 /*--- Get back the mainbrowser panel, once all the channels have been added in the browser ---*/

 fl_unfreeze_form(mainbrowser->mainbrowser);

 if (Dy->AutoConfFlag == OFF && !fl_form_is_visible(mainbrowser->mainbrowser)) DyXformRaisePanel(NULL,MAINBROWSER);

 return;
}

/*--------------------------------------------- DyXformIni ------*/

void DyXformIni (char **argv, int *myargc)
{
/*
 char** myargv;
 char dummy[LSTRING_MAX];
*/

 DyDebugPrint(__FUNCTION__,0);

 fl_set_border_width(-2);

 fl_initialize(myargc, argv, "dataDisplay", 0, 0);
 
 mainbrowser     = create_form_mainbrowser();
 mainpanel       = create_form_mainpanel();
 userfuncbrowser = create_form_userfuncbrowser();
 plot1d          = create_form_plot1d();
 plot2d          = create_form_plot2d();
 plottime        = create_form_plottime();
 plotfft         = create_form_plotfft();
 plotffttime     = create_form_plotffttime();
 plot1dtime      = create_form_plot1dtime();
 plottrfcttime   = create_form_plottrfcttime();
 plotcohetime    = create_form_plotcohetime();
 plotrawtime     = create_form_plotrawtime();
 plottrfct       = create_form_plottrfct();
 plotcohe        = create_form_plotcohe();
 plotaudio       = create_form_plotaudio();
 plotgenepar     = create_form_plotgenepar();
 getdd           = create_form_getdd();
 getshm          = create_form_getshm();
 cmservers       = create_form_cmservers();
 plotraw         = create_form_plotraw();
 dyhelp          = create_form_dyhelp();
 dydebug         = create_form_dydebug();
 dyoptions       = create_form_dyoptions();
 dysliders       = create_form_dysliders();
 dyoutput        = create_form_dyoutput();
 reffile         = create_form_reffile();

 /*--- Init some inputs and browsers ---*/

 DyXformUpdateCounters(0,0);
 fl_clear_browser (mainbrowser->channels);
 fl_clear_browser (mainbrowser->plots);
 fl_clear_browser (reffile->file_list);

 fl_set_button (mainpanel->waitdata,OFF);
 fl_set_button (dyoptions->autodeselectplot,ON);

 DyXformSetStartTime(0);
 DyXformSetDebug();

 /*--- Init the slider used for scanning zoomed plots ---*/

 fl_set_scrollbar_increment(mainpanel->scanplots, 0.001, 0.001);
 fl_set_scrollbar_bounds(mainpanel->scanplots,0.,1.);
 fl_set_scrollbar_step(mainpanel->scanplots,0.001);

 /*--- Init the user functions browser ---*/

 fl_clear_browser (userfuncbrowser->userFuncLoaded);
 DyXformUpdateUserProcBrowser();

 /*--- Init the CmServers panel and the dataSender panel ---*/
 printf("\n *** Your current CMDOMAIN is %s ***\n", getenv("CMDOMAIN"));
/**
 printf(" *** To access online data, you must type \042setenv CMDOMAIN %s\042 before starting dataDisplay ***\n", CASCINA);
 printf(" *** To connect to dataSender, you must type \042setenv CMDOMAIN %sLV\042 before starting dataDisplay ***\n\n", CASCINA);
**/
 fl_set_browser_dblclick_callback (cmservers->liste, DyXformCmConnect, 0);
 fl_set_browser_fontsize (cmservers->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (cmservers->liste, FL_TIMES_STYLE);

 fl_set_browser_fontsize (getdd->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (getdd->liste, FL_TIMES_STYLE);

 fl_set_browser_fontsize (getshm->liste, FL_MEDIUM_SIZE);
 fl_set_browser_fontstyle (getshm->liste, FL_TIMES_STYLE);

 /*--- Init the search input fields ---*/

 fl_set_input_return (cmservers->searchCmName,  FL_RETURN_ALWAYS);
 fl_set_input_return (mainbrowser->searchch,    FL_RETURN_ALWAYS);

 fl_set_input_return (mainpanel->startUTCdate,  FL_RETURN_ALWAYS);
 fl_set_input_return (mainpanel->startTime,   FL_RETURN_ALWAYS);

 /*--- Initialize options and buttons of the main panel ---*/

 fl_activate_object   (mainpanel->start);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->start, FL_BLACK);
 fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/

 /*--- Init the Input menu button ---*/

 fl_addto_menu (mainpanel->inputMenu, READ_FFL);
 fl_addto_menu (mainpanel->inputMenu, READ_FILE);
#ifndef NOCM
 fl_addto_menu (mainpanel->inputMenu, CONNECT_CMSERVER);
 fl_addto_menu (mainpanel->inputMenu, CONNECT_DD);
#endif
 fl_addto_menu (mainpanel->inputMenu, "------------------");
 fl_addto_menu (mainpanel->inputMenu, READ_SHM);
 fl_addto_menu (mainpanel->inputMenu, READ_WAVE);
 fl_addto_menu (mainpanel->inputMenu, READ_ASCII);

 /*--- Init the Config menu button ---*/

 fl_addto_menu (mainpanel->configMenu, LOAD_CONFIG);
 fl_addto_menu (mainpanel->configMenu, SAVE_CONFIG_AS);

 /*--- Init the tools menu ---*/

 fl_addto_menu (mainpanel->toolsMenu, DY_RESET);
 fl_addto_menu (mainpanel->toolsMenu, DY_PADS);
 fl_addto_menu (mainpanel->toolsMenu, DY_SLIDERS);
 fl_addto_menu (mainpanel->toolsMenu, DY_ALIGN);
 fl_addto_menu (mainpanel->toolsMenu, DY_USERFCT);
 fl_addto_menu (mainpanel->toolsMenu, DY_OUTPUT);
 fl_addto_menu (mainpanel->toolsMenu, DY_OPTIONS);
 fl_addto_menu (mainpanel->toolsMenu, DY_CHECKS);
 fl_addto_menu (mainpanel->toolsMenu, DY_DEBUG);
 fl_addto_menu (mainpanel->toolsMenu, DY_HELP);

 /*--- Init the reference plots menu ---*/

 fl_addto_menu (mainpanel->refplotMenu, DY_SETREFPLOT);
 fl_addto_menu (mainpanel->refplotMenu, DY_REMOVEREFPLOT);
 fl_addto_menu (mainpanel->refplotMenu, "------------------");
 fl_addto_menu (mainpanel->refplotMenu, DY_GETREFPLOT);
 fl_addto_menu (mainpanel->refplotMenu, DY_SAVEREFPLOT);
 fl_addto_menu (mainpanel->refplotMenu, DY_EDITREFPLOT);

 /*--- Init the Cm servers menu ---*/
/**
 fl_addto_menu (cmservers->csmMenu, "AddToMenu");
 fl_addto_menu (cmservers->csmMenu, "FbmStDy");
 fl_addto_menu (cmservers->csmMenu, "Fbt50FbDy");
 fl_addto_menu (cmservers->csmMenu, "FbmMainUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmMoniUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmSuspUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmDetUsers");
 fl_addto_menu (cmservers->csmMenu, "FbmMainShift");
 fl_addto_menu (cmservers->csmMenu, "FbmMoniShift");
 fl_addto_menu (cmservers->csmMenu, "FbmSuspShift");
 fl_addto_menu (cmservers->csmMenu, "FbmDetShift");
**/

 /*--- Init the Output parameters ---*/

 DyXformOutputToPanel();
 fl_set_button(mainpanel->writing_data,ON);
 fl_hide_object(mainpanel->writing_data);

 /*--- Init the Dy Debug panel ---*/

 fl_set_input_return (dydebug->debuglvl, FL_RETURN_ALWAYS);

 /*--- Get Virgo logo as a pixmap or bitmap picture ---*/

/**
 fl_set_pixmap_data(mainpanel->logo,logo);
**/
 fl_set_object_label (mainpanel->dyversion, DYVERSION);

 /*--- Init the goodies fonts ---*/

 fl_set_goodies_font(FL_TIMESBOLD_STYLE, FL_MEDIUM_SIZE);
 fl_set_oneliner_color(FL_BLACK, FL_WHEAT);
 fl_set_oneliner_font(FL_TIMESBOLD_STYLE, FL_LARGE_SIZE);
 fl_set_resource("flAlert.dismiss.label","Continue");

 return;
}


/*--------------------------------------------------- DyXformRaz ---*/

void DyXformRaz (FL_OBJECT *obj, long arg)
{
 long answer=1;
 int i;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Reset all ---*/

/*
 if (arg == 0 && Dy->AutoConfFlag == OFF)
    {answer = fl_show_question ("Do you really want to RESET All ?",1);
     if (!answer) return;
    }
*/

 /*--- reset plots list ---*/

 if (arg == 2) 
    {
     answer = fl_show_question ("Remove all the plots ?",1);
     if (!answer) return;
     fl_clear_browser(mainbrowser->plots);
    }

 /*--- reset reference plots files list ---*/

 if (arg == 6 || arg == 0)
    {
     for (i=0; i<Dy->nreffile; i++) RA_RefPlotRemove(Dy->reffilename[i]);
     Dy->nreffile = 0;
     fl_clear_browser(reffile->file_list);
     Dy->refplot_indelebile = OFF; fl_set_button(dyoptions->indelebile,OFF);
    }

 /*--- Initialize the channels list ---*/
 if (arg == 1 || arg == 0)
    {
     fl_clear_browser (mainbrowser->channels);
    }

 /*--- Initialize options and buttons of the main panel ---*/

 if (arg == 0)
    {
     /*--- reset counters and browsers ---*/
     DyXformUpdateCounters(0,0);
     fl_set_input (mainpanel->startTime, "0");
     fl_set_input (mainpanel->startUTCdate, "0");
     fl_set_input (mainpanel->duration, "-1");
     fl_set_input (mainpanel->UTCdate, "0");
     fl_clear_browser (mainbrowser->plots);

     /*--- reset buttons states ---*/
     fl_activate_object   (mainpanel->start);
     fl_set_object_lcol   (mainpanel->start, FL_BLACK);
     fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
    }

 /*--- Reset also internal variables ---*/

 DyRaz (arg);

 return;
}


/*------------------------------------------ DyInfoLine ---*/

void DyInfoLine (char *msg)
{
 char *dummy;
 char date_heure[24];
 time_t tt;

 dummy = (char *)malloc(strlen(msg)+256);

 tt = time(NULL);

 /*--- Build the info line ---*/

 strftime(date_heure,24,"%d_%b_%Y_%T",localtime(&tt));
 sprintf(dummy,"%s : %s", date_heure, msg);

/*--- Colored and bold text ---*/
/*
 printf("\033[35;1m%s \033[0m \n",dummy);
 printf("\033[92;1m%s \033[0m \n",dummy);
*/
 printf("\033[90;1m%s \033[0m \n",dummy);

 /*--- Show the info line at bottom of the mainpanel ---*/

 fl_set_object_label (mainpanel->infoLine, dummy);

/**
 if (Dy->Order == CONTINUE && strstr(dummy,"Stopped") == NULL)
   {if (tt%2 == 0) fl_set_object_color(mainpanel->infoLine, FL_MCOL,FL_MCOL);
    else           fl_set_object_color(mainpanel->infoLine, FL_WHITE,FL_WHITE);
   }
 else
   {fl_set_object_color(mainpanel->infoLine, FL_MCOL,FL_MCOL);}

 fl_check_forms();
**/

 free(dummy);

 return;
}


/*------------------------------------------ DyInfoPanel ---*/

void DyInfoPanel (char *msg, int alarm)
{
 char *str0=NULL, *str=NULL, *ret=NULL;

 if (msg == NULL) return;

 DyDebugPrint(__FUNCTION__,0);

 str0 = strstr(msg,"\n");
 if (str0 == NULL) str0 = msg + strlen(msg);

 if (str0-msg > 50)
    {str = msg;
     while (str && str < msg +strlen(msg) - 50)
        {ret = strstr(str+45, "\n");
         if (ret && (ret-str-45 < 10)) {str = ret; *str = '\n'; continue;}
         ret = strstr(str+45, ". ");
         if (ret && (ret-str-45 < 10)) {str = ret; *(str+1) = '\n'; continue;}
         str = strstr(str+45, " ");
         if (str) *str = '\n';
        }
    }

 if (alarm > 0) {fl_trigger_object(mainpanel->stop); fl_show_messages(msg);}
 else           {fl_trigger_object(mainpanel->pause); fl_show_messages(msg); fl_trigger_object(mainpanel->continuer);}

 return;
}


/*------------------------------------------  DyXformUpdateCounters ---*/

void DyXformUpdateCounters (double gtime, int uleaps)
{
 char dummy[LSTRING_MAX];
 double latency;

 DyDebugPrint(__FUNCTION__,0);

 latency = DyGetLatency(gtime,uleaps);
 
 sprintf (dummy, "%.4f", gtime);
 fl_set_input (mainpanel->currentGTime, dummy);

 sprintf (dummy, "%.3f", latency);
 fl_set_input (mainpanel->latency, dummy);

 fl_set_input (mainpanel->UTCdate, FrStrUTC((int)gtime,uleaps));

 if (latency > Dy->startLatency+DYCM_QSIZE && Dy->InputType == INCM)
   {DyInfoLine("Latency is too large. I stop before you get problems with the frames provider");
    Dy->Order = STOP;
   }

 return;
}


/*------------------------------------------  DyXformShowMessage ---*/

void DyXformShowMessage (char *message)
{
 fl_show_messages(message);
 return;
}

/*------------------------------------------  DyXformCheckGUI ---*/

void DyXformCheckGUI()
{
 fl_check_forms();
 return;
}

/*------------------------------------------  DyXformTrigger ---*/

void DyXformTrigger (int action)
{
 if      (action == STOP)        fl_trigger_object(mainpanel->stop);
 else if (action == PAUSE)       fl_trigger_object(mainpanel->pause);
 else if (action == NEXT_UPDATE) fl_trigger_object(mainpanel->nextUpdate);

 return;
}

/*------------------------------------------  DyXformQuestion ---*/

int DyXformQuestion (char *dummy)
{
 int answer;

 answer = fl_show_question (dummy,1);

 return(answer);
}

/*------------------------------------------ DyXformGetFirstLast ---*/
/* ICI obsolete ? */

void DyXformGetFirstLast (double *start, double *duration)
{
 const char *dum1, *dum2;

 DyDebugPrint(__FUNCTION__,0);

 /*--- If startTime field is empty, put default value 0 ---*/

 if (fl_get_input(mainpanel->startTime) == NULL)
    {fl_set_input(mainpanel->startTime,"0");
     fl_set_input (mainpanel->startUTCdate, "0");
    }

 /*--- If duration field is empty, put default value -1 ---*/

 if (fl_get_input(mainpanel->duration) == NULL)
    {fl_set_input (mainpanel->duration,"-1");}

 /*--- Get starting Time ---*/

 dum1 = fl_get_input(mainpanel->startTime);
 *start = atof(dum1);

 /*--- Get duration ---*/

 dum2 = fl_get_input(mainpanel->duration);
 *duration = atof(dum2);

 return;
}


/*------------------------------------------ DyXformSetStartTime ---*/

void DyXformSetStartTime (int uleaps)
{
 char dummy[64];

 if (Dy->startTime <= 0) Dy->startTime = DyGetGPSNow();

 /*--- Check that starTime is not below lowest time of inputs ---*/

 if (Dy->startTime < Dy->firstTime) Dy->startTime = Dy->firstTime;
 sprintf(dummy,"%.0f", Dy->startTime); 
 fl_set_input (mainpanel->startTime, dummy);    
 fl_set_input (mainbrowser->gps_of_update, dummy);    
 fl_set_input(mainpanel->startUTCdate, FrStrUTC((int)Dy->startTime,uleaps));

 /*--- Set also duration ---*/

 DyXformSetDuration();

 return;
}

/*------------------------------------------ DyXformSetDuration ---*/

void DyXformSetDuration ()
{
 char dummy[64];

 if (Dy->duration <= 0)
   {fl_set_input (mainpanel->duration, "-1");}
 else
   {sprintf(dummy,"%.3f", Dy->duration); 
    fl_set_input (mainpanel->duration, dummy);
   }

 return;
}


/*--------------------------------------------------- DyXformStart ---*/

void DyXformStart (FL_OBJECT *obj, long arg)
{
 long nPlot;

 DyDebugPrint(__FUNCTION__,0);

 /*--- If no input data, we stop ---*/

 if (Dy->InputType == INOFF && Dy->nch == 0 && Dy->frameChannel==0)
    {DyInfoLine ("Please, first choose a source of data by using the Input menu!");
     return;
    }

 /*--- If no plot defined, we stop. Except if we output frames on files ---*/

 nPlot = fl_get_browser_maxline(mainbrowser->plots);

 if (nPlot <= 0 && Dy->output_flag == OFF)
    {DyInfoLine ("There is nothing to display!");
     return;
    }

 /*--- Configure the main panel buttons ---*/

 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_deactivate_object (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_activate_object   (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_BLACK);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);

 if (Dy->InputType == INCM)
    {fl_deactivate_object (mainpanel->pause);
     fl_set_object_lcol (mainpanel->pause, FL_INACTIVE);
    }

 /*--- Prevent any modification on plots while running ---*/

 DyXformClosePanel(mainbrowser->close,MAINBROWSER);
 fl_deactivate_object (mainpanel->mainbrowser);
 fl_set_object_lcol   (mainpanel->mainbrowser, FL_INACTIVE);
 fl_deactivate_object (mainpanel->clear);
 fl_set_object_lcol   (mainpanel->clear, FL_INACTIVE);

 /*--- Really start reading frames ---*/

 DyStart();

 return;
}


/*--------------------------------------------------- DyXformPause ---*/

void DyXformPause (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 /*--- Set the pause configuration of the mainpanel buttons ---*/

 fl_deactivate_object (obj);
 fl_set_object_lcol   (obj, FL_INACTIVE);
 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);
 fl_deactivate_object (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_activate_object   (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_BLACK);
/*
 fl_activate_object   (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_BLACK);
*/
 fl_activate_object   (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_BLACK);

 /*--- Enter in the pause loop ---*/

 DyPause();
 return;
}


/*--------------------------------------------------- DyXformContinue ---*/

void DyXformContinue (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 fl_deactivate_object (obj);
 fl_set_object_lcol   (obj, FL_INACTIVE);
 fl_deactivate_object (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_activate_object   (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_BLACK);
 fl_activate_object   (mainpanel->stop);
 fl_set_object_lcol   (mainpanel->stop, FL_BLACK);

 DyContinue();
 return;
}


/*--------------------------------------------------- DyXformNext ---*/

void DyXformNext (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);
 DyNext(arg);
 return;
}


/*---------------------------------------------------  DyXformStop ---*/

void DyXformStop (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 /*--- Set the stop configuration of the mainpanel buttons ---*/

/*
 fl_deactivate_object (mainpanel->stop);
*/
 fl_set_object_lcol   (mainpanel->stop, FL_INACTIVE);
 fl_activate_object   (mainpanel->start);
 fl_set_object_lcol   (mainpanel->start, FL_BLACK);
 fl_deactivate_object (mainpanel->pause);
 fl_set_object_lcol   (mainpanel->pause, FL_INACTIVE);
 fl_deactivate_object (mainpanel->nextUpdate);
 fl_set_object_lcol   (mainpanel->nextUpdate, FL_INACTIVE);
/*
 fl_deactivate_object (mainpanel->nextEvent);
 fl_set_object_lcol   (mainpanel->nextEvent, FL_INACTIVE);
*/
 fl_deactivate_object (mainpanel->continuer);
 fl_set_object_lcol   (mainpanel->continuer, FL_INACTIVE);

 /*--- Allow modification on plots, once stopped ---*/

 fl_activate_object (mainpanel->mainbrowser);
 fl_set_object_lcol (mainpanel->mainbrowser, FL_BLACK);
 fl_activate_object (mainpanel->clear);
 fl_set_object_lcol (mainpanel->clear, FL_BLACK);

 /*--- Activate the stop procedure ---*/

 DyStop();
 return;
}


/*------------------------------------------ DyXformMenu ---*/

void DyXformMenu (FL_OBJECT *obj, long arg)
{
 const char *menuItem;
 int error;

 menuItem = fl_get_menu_text(obj);
 if (!menuItem) return;

 /*--- Read data from a ffl file ---*/

 if (strcmp(menuItem,READ_FFL) == 0)
    {
     DyXformAddFile(NULL,1);  /* Dy->InputType is set in DyReadFirstFrame called by DyAddFile */
    }

 /*--- Read data from a frame file ---*/

 if (strcmp(menuItem,READ_FILE) == 0)
    {
     DyXformAddFile(NULL,0);
    }

 /*--- Read data from an online Cm connection ---*/

 if (strcmp(menuItem,CONNECT_CMSERVER) == 0)
    {
     fl_set_button(cmservers->fullList,OFF);
     DyXformUpdateCmList(NULL,0);
     DyXformRaisePanel(NULL,CMSERVERS);
    }

 /*--- Read data from a Dd server ---*/

 if (strcmp(menuItem,CONNECT_DD) == 0)
    {
     fl_set_input(getdd->ddserver,Dy->DdServer);
     fl_set_input(getdd->inputfile,Dy->Ddinputfile);
     fl_clear_menu (getdd->datasender_list);
     Dy->DdNS = ON;
     DyXformUpdateCmList(getdd->datasender_list,GETDD);
     if (strstr(Dy->Ddinputfile,".") == NULL) fl_trigger_object(getdd->ok);
     DyXformRaisePanel(NULL,GETDD);
     Dy->DdNS = OFF;
    }

 /*--- Read data from a shared memory ---*/

 if (strcmp(menuItem,READ_SHM) == 0)
    {
     Dy->nshm = DyGetShmList();
     if (Dy->nshm == 0) {DyInfoPanel("No shared memory segments available",1); return;}
     else if (Dy->nshm == 1) DyAddShm(Dy->shmlist[0]);
     else {DyXformUpdateShmList(NULL,0); DyXformRaisePanel(NULL, GETSHM);}
    }

 /*--- Read data from an audio wave file ---*/

 if (strcmp(menuItem,READ_WAVE) == 0)
    {
     DyXformAddFile(NULL,-1);
    }

 /*--- Read data from an audio wave file ---*/

 if (strcmp(menuItem,READ_ASCII) == 0)
    {
     DyXformAddFile(NULL,-2);
    }


 /*--- Used from getdd panel, choice of a dataSender ---*/

 if (strncmp(menuItem,DDSERVER,10) == 0)
    {
     strcpy(Dy->DdServer, menuItem);
     fl_set_input(getdd->ddserver,Dy->DdServer);
     error = DyDdFFLRequest(Dy->DdServer);
     if (error < 0)
        {strcpy(Dy->Ddffldir,DDDIR);
         fl_clear_browser(getdd->liste);
         fl_add_browser_line(getdd->liste,"NO FFL DIRECTORY SENT BY DATASENDER");
        }
     strcpy(Dy->Ddinputfile, Dy->Ddffldir);
     fl_set_input(getdd->inputfile,Dy->Ddinputfile);
     if (error == 0) fl_trigger_object(getdd->ok);
    }


 /*--- Config Menu of the mainpanel ---*/

 if (strcmp(menuItem,LOAD_CONFIG) == 0)     DyXformGetConfigFile(NULL,0);
 if (strcmp(menuItem,SAVE_CONFIG_AS) == 0)  DyXformSaveConfig(NULL,0);

 /*--- Tools Menu of the mainpanel ---*/

 if (strcmp(menuItem,DY_RESET) == 0)   DyXformRaz(NULL,0);
 if (strcmp(menuItem,DY_PADS) == 0)    DyXformPadsConfig(NULL,0);
 if (strcmp(menuItem,DY_SLIDERS) == 0) DyXformRaisePanel(NULL,DYSLIDERS);
 if (strcmp(menuItem,DY_ALIGN) == 0)   DyXformAlignTime(NULL,0);
 if (strcmp(menuItem,DY_USERFCT) == 0) DyXformRaisePanel(NULL,USERFUNCBROWSER);
 if (strcmp(menuItem,DY_OUTPUT) == 0)  DyXformRaisePanel(NULL,OUTPUTDATA);
 if (strcmp(menuItem,DY_OPTIONS) == 0) DyXformRaisePanel(NULL,DYOPTIONS);
 if (strcmp(menuItem,DY_CHECKS) == 0)  DyXformChecks(NULL,0);
 if (strcmp(menuItem,DY_DEBUG) == 0)   DyXformRaisePanel(NULL,DYDEBUG);
 if (strcmp(menuItem,DY_HELP) == 0)    DyXformRaisePanel(NULL,DYHELP);

 /*--- Reference Plots Menu of the mainpanel ---*/
 if (strcmp(menuItem,DY_SETREFPLOT) == 0)  DyXformSetRefPlot(NULL,0);
 if (strcmp(menuItem,DY_REMOVEREFPLOT) == 0) DyXformRefPlotRemove(NULL,0);
 if (strcmp(menuItem,DY_GETREFPLOT) == 0)  DyXformGetRefPlot(NULL,0);
 if (strcmp(menuItem,DY_SAVEREFPLOT) == 0) DyXformSaveRefPlot(NULL,0);
 if (strcmp(menuItem,DY_EDITREFPLOT) == 0) DyXformRaisePanel(NULL,REFFILE);

 return;
}



/************/
/*  PANELS  */
/************/

/*------------------------------------------ DyXformRaisePanel ---*/

void DyXformRaisePanel (FL_OBJECT *obj, long arg)
{
 FL_FORM *form;
 long xmin, ymin, xsize, ysize, offset, xpos, ypos, width, height;
 char dummy[LSTRING_MAX];

 if (arg < NARG_MAX) sprintf(dummy,"DyXformRaisePanel> arg=%ld  panel=%s", arg, Dy->typeName[arg]);
 DyDebugPrint(dummy,0);

 if (Dy->BgRun == ON) return;

 xmin = -1200;
 ymin = -760;

 xsize = -mainpanel->mainpanel->w;
 ysize = -mainpanel->mainpanel->h;
 offset = 5;

 switch(arg)
   {
    case MAINPANEL:
       form   = mainpanel->mainpanel;
       width  = form->w;
       height = form->h;
       if (((Dy->debugType)&(1<<DBG_OTHERS)) && Dy->debug > 0) 
         {printf("DyXformRaisePanel> Mainpanel: xsize=%ld  ysize=%ld  xpos=%ld  ypos=%ld \n",
                 xsize, ysize, XREF-width, YREF-height);
         }
       fl_set_form_position(form, XREF-width, YREF-height);
       strcpy(dummy, Dy->DyName);
       if (Dy->titleCmd != NULL) strcpy(dummy,Dy->titleCmd);
       fl_show_form(form,FL_PLACE_POSITION,FL_FULLBORDER,dummy);
       break;

    case MAINBROWSER:
       form   = mainbrowser->mainbrowser;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "mainbrowser");
       fl_raise_form(form);
       break;

    case CMSERVERS:
       form   = cmservers->cmservers;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Cm Servers");
       fl_raise_form(form);
       break;

    case GETDD:
       form   = getdd->getdd;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "getdd");
       fl_raise_form(form);
       break;

    case GETSHM:
       form   = getshm->getshm;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Shared Memory segments");
       fl_raise_form(form);
       break;

    case PLOTRAW:
       form   = plotraw->plotraw;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotraw");
       fl_raise_form(form);
       break;

    case PLOTTIME:
       form   = plottime->plottime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottime");
       fl_raise_form(form);
       break;

    case PLOTFFT:
       form   = plotfft->plotfft;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotfft");
       fl_raise_form(form);
       break;

    case PLOTTR:
       form   = plottrfct->plottrfct;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottrfct");
       fl_raise_form(form);
       break;

    case PLOTCOHE:
       form   = plotcohe->plotcohe;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotcohe");
       fl_raise_form(form);
       break;

    case PLOTFFTTIME:
       form   = plotffttime->plotffttime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotffttime");
       fl_raise_form(form);
       break;

    case PLOTTRTIME:
       form   = plottrfcttime->plottrfcttime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plottrfcttime");
       fl_raise_form(form);
       break;

    case PLOTCOHETIME:
       form   = plotcohetime->plotcohetime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotcohetime");
       fl_raise_form(form);
       break;

    case PLOT1DTIME:
       form   = plot1dtime->plot1dtime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot1dtime");
       fl_raise_form(form);
       break;

    case PLOTRAWTIME:
       form   = plotrawtime->plotrawtime;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotrawtime");
       fl_raise_form(form);
       break;

    case PLOT1D:
       form   = plot1d->plot1d;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot1d");
       fl_raise_form(form);
       break;

    case PLOT2D:
       form   = plot2d->plot2d;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plot2d");
       fl_raise_form(form);
       break;

    case PLOTAUDIO:
       form   = plotaudio->plotaudio;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotaudio");
       fl_raise_form(form);
       break;

    case DYSLIDERS:
       form   = dysliders->dysliders;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "dysliders");
       fl_raise_form(form);
       break;

    case GENEPAR:
       form   = plotgenepar->plotgenepar;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "plotgenepar");
       fl_raise_form(form);
       break;

    case USERFUNCBROWSER:
       form   = userfuncbrowser->userfuncbrowser;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "userfuncbrowser");
       fl_raise_form(form);
       break;

    case DYHELP:
       form   = dyhelp->dyhelp;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_set_object_label (dyhelp->dyversion, DYVERSION);
       fl_clear_browser(dyhelp->browser);
       sprintf(dummy,"  dataDisplay (%s)\n"
                     "  Author: D. Verkindt (verkindt@lapp.in2p3.fr)\n"
                     "  Documentation:http://lapp.in2p3.fr/virgo/dataDisplay\n"
                     "  Libraries used:  Fr(%s), Root(%s) , GUI Xforms (v%dr%d)",
                     DYVERSION, FrVersion, ROOT_RELEASE, FL_VERSION, FL_REVISION);
/**
#ifndef NOCM
       sprintf(dummy,"%s , DyFd (%s)", dummy, DYFDVERSION);
#endif
**/
       sprintf(dummy,"%s\n", dummy);
       fl_add_browser_line(dyhelp->browser, dummy);
       fl_add_browser_line(dyhelp->browser,"  ********************** First Help ************************\n\n"
                             "  For detailed help, you can use the  [Help balloons]  button above.\n"
                             "  This will generate a message, for each field or button the mouse passes over.\n"
                             "  \n"
                             "  First, you must get some data (under frame or wave format).\n"
                             "  Use the 'Input Data' menu and choose a source of data      \n"
                             "  (for instance a 'Read File' or a 'Connect online').        \n"
                             "  Once you are selected a file or a source of data,        \n"
                             "  you should see the main browser panel opening, with\n"
                             "  the list of channels on the left side. Those channels\n"
                             "  can be selected to do operations, filtering, various plots\n"
                             "  and audio output. The main browser panel can be accessed\n"
                             "  using the 'Data and Plots' button of the main panel.\n\n"
                             "  *******************************************************************");
       sprintf(dummy,"  For more help, look at the README file in the /virgoApp/Dy/%s directory \n"
                     "  or look into /virgoApp/Dy/%s/doc. \n", DYVERSION, DYVERSION);
       fl_add_browser_line(dyhelp->browser,dummy);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "About dataDisplay");
       fl_raise_form(form);
       break;

    case HELPPANEL:
       form   = helppanel;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       if (xpos < xmin) xpos = xmin; if (ypos < ymin) ypos = ymin;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Help");
       fl_raise_form(form);
       break;

    case COFFEEPANEL:
       form   = coffeepanel;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Coffee");
       fl_raise_form(form);
       break;

    case USERFCTHELP:
       form   = userprochelp;
       width  = form->w;
       height = form->h;
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_set_form_position(form, XREF-width, YREF+ysize-offset-height);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Help for User Functions");
       fl_raise_form(form);
       break;

    case DYDEBUG:
       form   = dydebug->dydebug;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       if (((Dy->debugType)&(1<<DBG_SPEED)))   fl_set_button(dydebug->dbg_speed,ON); 
       if (((Dy->debugType)&(1<<DBG_VDYPLOT))) fl_set_button(dydebug->dbg_vdyplot,ON); 
       if (((Dy->debugType)&(1<<DBG_OTHERS)))  fl_set_button(dydebug->dbg_others,ON); 
       DyXformSetDebug();
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Debugging");
       fl_raise_form(form);
       break;

    case DYOPTIONS:
       form   = dyoptions->dyoptions;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       sprintf(dummy,"%g",RA_GetSize("stat"));       fl_set_input(dyoptions->zoomstat,dummy);
       sprintf(dummy,"%g",RA_GetSize("legend"));     fl_set_input(dyoptions->zoomlegend,dummy);
       sprintf(dummy,"%g",RA_GetSize("timestring")); fl_set_input(dyoptions->zoomtimestring,dummy);
       sprintf(dummy,"%g",RA_GetSize("title"));      fl_set_input(dyoptions->zoomtitle,dummy);
       sprintf(dummy,"%g",RA_GetSize("labelx"));     fl_set_input(dyoptions->zoomlabelx,dummy);
       sprintf(dummy,"%g",RA_GetSize("labely"));     fl_set_input(dyoptions->zoomlabely,dummy);
       sprintf(dummy,"%d",Dy->linewidth);            fl_set_input(dyoptions->linewidth,dummy);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Options");
       fl_raise_form(form);
       break;

    case OUTPUTDATA:
       DyXformOutputToPanel();
       form   = dyoutput->dyoutput;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Outputs");
       fl_raise_form(form);
       break;

    case REFFILE:
       form   = reffile->reffile;
       width  = form->w;
       height = form->h;
       if (form && fl_form_is_visible(form)) {fl_hide_form(form); break;}
       xpos = XREF-width; ypos = YREF+ysize-offset-height;
       if (xpos < xmin) xpos = 30; if (ypos < ymin) ypos = 30;
       fl_set_form_position(form, xpos, ypos);
       fl_show_form(form, FL_PLACE_POSITION, FL_FULLBORDER, "Dy Reference files");
       fl_raise_form(form);
       break;

    default:
       printf("DyXformRaisePanel> called with unvalid argument %ld\n", arg);
       break;
   }

 return;
}


/*------------------------------------------ DyXformClosePanel ---*/

void DyXformClosePanel (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];

 if (obj == NULL)                                        return;
 if (obj && obj->form && !fl_form_is_visible(obj->form)) return;
 if (obj && obj->visible != 1)                           return;

 if (arg == HELPPANEL)   {fl_hide_form(helppanel);    fl_free_form(helppanel);}
 if (arg == COFFEEPANEL) {fl_hide_form(coffeepanel);  fl_free_form(coffeepanel);}
 if (arg == USERFCTHELP) {fl_hide_form(userprochelp); fl_free_form(userprochelp);}

 if (obj == mainbrowser->close)     {fl_hide_form(mainbrowser->mainbrowser); arg = MAINBROWSER;}
 if (obj == cmservers->close)       {fl_hide_form(cmservers->cmservers); arg = CMSERVERS;}
 if (obj == getdd->close)           {fl_hide_form(getdd->getdd); arg = GETDD;}
 if (obj == getshm->close)          {fl_hide_form(getshm->getshm); arg = GETSHM;}
 if (obj == userfuncbrowser->close) {fl_hide_form(userfuncbrowser->userfuncbrowser); arg = USERFUNCBROWSER;}

 if (obj == plotraw->close     ||  obj == plotraw->ok)     {fl_hide_form(plotraw->plotraw); arg = PLOTRAW;}
 if (obj == plottime->close    ||  obj == plottime->ok)    {fl_hide_form(plottime->plottime); arg = PLOTTIME;}
 if (obj == plotfft->close     ||  obj == plotfft->ok)     {fl_hide_form(plotfft->plotfft); arg = PLOTFFT;}
 if (obj == plotffttime->close ||  obj == plotffttime->ok) {fl_hide_form(plotffttime->plotffttime); arg = PLOTFFTTIME;}
 if (obj == plot1dtime->close  ||  obj == plot1dtime->ok)  {fl_hide_form(plot1dtime->plot1dtime); arg = PLOT1DTIME;}
 if (obj == plottrfcttime->close  ||  obj == plottrfcttime->ok)  {fl_hide_form(plottrfcttime->plottrfcttime); arg = PLOTTRTIME;}
 if (obj == plotcohetime->close  ||  obj == plotcohetime->ok)  {fl_hide_form(plotcohetime->plotcohetime); arg = PLOTCOHETIME;}
 if (obj == plotrawtime->close ||  obj == plotrawtime->ok) {fl_hide_form(plotrawtime->plotrawtime); arg = PLOTRAWTIME;}
 if (obj == plottrfct->close   ||  obj == plottrfct->ok)   {fl_hide_form(plottrfct->plottrfct); arg = PLOTTR;}
 if (obj == plotcohe->close    ||  obj == plotcohe->ok)    {fl_hide_form(plotcohe->plotcohe); arg = PLOTCOHE;}

 if (obj == plot1d->close      ||  obj == plot1d->ok)      {fl_hide_form(plot1d->plot1d); arg = PLOT1D;}
 if (obj == plot2d->close      ||  obj == plot2d->ok)      {fl_hide_form(plot2d->plot2d); arg = PLOT2D;}
 if (obj == plotaudio->close   ||  obj == plotaudio->ok)   {fl_hide_form(plotaudio->plotaudio); arg = PLOTAUDIO;}
 if (obj == plotgenepar->close ||  obj == plotgenepar->ok) {fl_hide_form(plotgenepar->plotgenepar); arg = GENEPAR;}

 if (obj == dyhelp->close)    {fl_hide_form(dyhelp->dyhelp); arg = DYHELP;}
 if (obj == dydebug->close)   {fl_hide_form(dydebug->dydebug); arg = DYDEBUG;}
 if (obj == dyoptions->close) {fl_hide_form(dyoptions->dyoptions); arg = DYOPTIONS;}
 if (obj == dysliders->close) {fl_hide_form(dysliders->dysliders); arg = DYSLIDERS;}
 if (obj == dyoutput->close)  {fl_hide_form(dyoutput->dyoutput); arg = OUTPUTDATA;}
 if (obj == reffile->close)   fl_hide_form(reffile->reffile);

 sprintf(dummy,"DyXformClosePanel> arg=%ld  panel=%s", arg, Dy->typeName[arg]);
 DyDebugPrint(dummy,0);

 return;
}


/*******************/
/*  Cm CONNECTION  */
/*******************/

/*------------------------------------------ DyXformCmName ---*/

void DyXformCmName (FL_OBJECT *obj, long arg)
{
 int i0;
 const char *name;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(cmservers->liste);
 if (i0 == 0) return;
 if (i0 < 0) i0 = -i0;

 name = fl_get_browser_line(cmservers->liste, i0);
 fl_set_input(cmservers->cmname, name);

 return;
}


/*------------------------------------------ DyXformCmConnect ---*/

void DyXformCmConnect (FL_OBJECT *obj, long arg)
{
 const char *name;

 DyDebugPrint(__FUNCTION__,0);

 name = fl_get_input(cmservers->cmname);

 strcpy(Dy->CmServer, name);

 DyXformRaz(NULL,1);
 DyReadFirstFrame (NULL, INCM);

 if (obj != NULL)  DyXformClosePanel (cmservers->close, CMSERVERS);

 return;
}


/*------------------------------------------ DyXformUpdateCmList ---*/

void DyXformUpdateCmList (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 Dy->CmFullList = fl_get_button (cmservers->fullList);

 DyUpdateCmList();

 return;
}


/*******************/
/* Shm CONNECTION  */
/*******************/

/*------------------------------------------ DyXformAddShm ---*/

void DyXformAddShm (FL_OBJECT *obj, long arg)
{
 const char *shmname;
 int i0;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(getshm->liste);
 if (i0 == 0) return;
 if (i0 < 0) i0 = -i0;

 shmname = fl_get_browser_line(getshm->liste, i0);

 if (Dy->InputType == INCM) DyXformRaz(NULL,1);

 DyAddShm((char *)shmname);
 return;
}

/*------------------------------------------ DyXformUpdateShmList ---*/

void DyXformUpdateShmList (FL_OBJECT *obj, long arg)
{
 int i;

 DyDebugPrint(__FUNCTION__,0);

 Dy->nshm = DyGetShmList();

 if (Dy->nshm <= 0) return;

 fl_clear_browser(getshm->liste);
 for (i=0; i<Dy->nshm; i++) fl_add_browser_line(getshm->liste, Dy->shmlist[i]);

 return;
}



/*************************/
/*  Data Sender CONNECT  */
/*************************/

/*-------------------------------------------- DyXformDdGetFile ---*/

void DyXformDdGetFile (FL_OBJECT *obj, long arg)
{
 const char *filename;
 char dummy[LSTRING_MAX];
 int i0,i;

 DyDebugPrint(__FUNCTION__,0);

 i0 = fl_get_browser(getdd->liste);
 if (i0 <= 0) return;

 filename = fl_get_browser_line(getdd->liste, i0);

 if (filename)
    {if (strcmp(filename,"..") == 0)
        {for (i=strlen(Dy->DdDir)-1; i>0; i--)
            {if (Dy->DdDir[i] == '/')
                {Dy->DdDir[i]='\0'; break;}
            }
         if (strlen(Dy->DdDir) == 0) strcpy(Dy->DdDir,"/");
         strcpy(dummy, Dy->DdDir);
        }
     else
        {sprintf(dummy, "%s/%s", Dy->DdDir, filename);}

     fl_set_input(getdd->inputfile, dummy);

     if (strstr(filename,".") == NULL  || strcmp(filename,"..") == 0)
        {fl_deactivate_object(getdd->ok);
         DyXformDdGetChlist(getdd->ok,0);
         fl_activate_object(getdd->ok);
        }
    }

 return;
}


/*-------------------------------------------- DyXformDdGetChlist ---*/

void DyXformDdGetChlist (FL_OBJECT *obj, long arg)
{
 int result;
 const char *ddserver, *inputfile, *cmdomain;
 const char *answer;

 DyDebugPrint(__FUNCTION__,0);

 if (Dy->Ddpasswd == NULL)
    {answer = fl_show_input("You need to provide a password: ", "");
     if (answer)
        {Dy->Ddpasswd = (char *)malloc(64);
         if (strlen(answer) > 0) strcpy(Dy->Ddpasswd,answer);
         else                    strcpy(Dy->Ddpasswd,"NULL");
        }
     else return;
    }

 ddserver = fl_get_input(getdd->ddserver);
 if (!ddserver) return;
 strcpy(Dy->DdServer, (char *)ddserver);

 inputfile = fl_get_input(getdd->inputfile);
 if (!inputfile) return;
 strcpy(Dy->Ddinputfile, (char *)inputfile);

 sleep(2);
 DyXformRaz(NULL,1);
 result = DyReadFirstFrame (NULL, INDD);

 if (result == -2) strcpy(Dy->DdDir,Dy->Ddinputfile);

 if (obj != NULL && result >= 0)
    {
     fl_deselect_browser(getdd->liste); 
     DyXformClosePanel (getdd->close, GETDD);
    }

 return;
}




/***********/
/*  PLOTS  */
/***********/

/*-------------------------------------------- DyXformSavePlot ---*/

void DyXformSavePlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Save current plots in a file ---*/

 if (Dy->lastsavedPlots) strcpy(dummy, Dy->lastsavedPlots);
 str = fl_show_input("Plots saved in file", dummy);
 if (str) strcpy(Dy->lastsavedPlots, (char *)str);
 else     return;
 ok = DySavePlot (Dy->lastsavedPlots);
 if (ok)
    {sprintf(dummy, "Current plots saved in %s", Dy->lastsavedPlots);
     DyInfoLine (dummy);
    }

 return;
}


/*-------------------------------------------- DyXformSetRefPlot ---*/

void DyXformSetRefPlot (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX];
 int ok, color;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check for the last saved Ref plots file ---*/

 if (Dy->lastsavedRefPlots == NULL)
   {Dy->lastsavedRefPlots = (char *)malloc(LSTRING_MAX);
    strcpy(Dy->lastsavedRefPlots, DDREFPLOTS);
   }

 /*--- Save reference plots in a file ---*/

 ok = DySavePlot (Dy->lastsavedRefPlots);
 if (ok)
    {sprintf(dummy, "Reference plots saved in %s", Dy->lastsavedRefPlots);
     DyInfoLine (dummy);
     color = 0;
     DyRefPlotAdd (Dy->lastsavedRefPlots, color);
     DyXformRefPlotFillBrowser();
     sprintf(dummy, "Superposed the reference plots from %s (color=%d)", Dy->lastsavedRefPlots, color);
     DyInfoLine (dummy);
    }
 else
    {sprintf(dummy, "Could not save Reference plots in %s", Dy->lastsavedRefPlots);
     DyInfoLine (dummy);
    }

 return;
}

/*-------------------------------------------- DyXformSaveRefPlot ---*/

void DyXformSaveRefPlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char dummy[LSTRING_MAX];
 int ok;

 DyDebugPrint(__FUNCTION__,0);

 /*--- Check for the last saved Ref plots file ---*/

 if (Dy->lastsavedRefPlots == NULL)
   {Dy->lastsavedRefPlots = (char *)malloc(LSTRING_MAX);
    strcpy(Dy->lastsavedRefPlots, DDREFPLOTS);
   }

 /*--- Save reference plots in a file ---*/

 str = fl_show_input("Reference Plots saved in file", Dy->lastsavedRefPlots);
 if (!str) return;
 strcpy(Dy->lastsavedRefPlots, (char *)str);
 ok = DySavePlot (Dy->lastsavedRefPlots);
 if (ok)
    {sprintf(dummy, "Reference plots saved in %s", Dy->lastsavedRefPlots);
     DyInfoLine (dummy);
    }
 else
    {sprintf(dummy, "Could not save Reference plots in %s", Dy->lastsavedRefPlots);
     DyInfoLine (dummy);
    }

 return;
}


/*-------------------------------------------- DyXformGetRefPlot ---*/

void DyXformGetRefPlot (FL_OBJECT *obj, long arg)
{
 char dummy[LSTRING_MAX], filename[LSTRING_MAX];
 int color, nf, size, result_length;
 char *result, *str, pattern_str[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 size = LSTRING_MAX * NFILE_MAX;
 result = (char *)malloc(size);
 while (result == NULL)
    {size = size / 2;
     result = (char *)malloc(size);
    }

 sprintf(pattern_str, "*%s", ROOT_SUFFIX);
 result_length = ClXFSelector("Get Reference Plots from file", Dy->lastrefplot_dir, pattern_str, "", result, size);

 if (result_length == 0 || !result) return;

 nf = 0;
 str = strtok(result," ");
 while (str != NULL)
    {if (is_a_file(str) == 0) {str = strtok(NULL," "); continue;}
     nf++;
     if (nf == 1) Dy->lastrefplot_dir = DyDirName(Dy->lastrefplot_dir, str);
      printf("DyXformGetRefPlot> file%d = %s\n", nf, str);
      strcpy(filename, str);
      color = 0;
      DyRefPlotAdd (filename, color);
      DyXformRefPlotFillBrowser();
      sprintf(dummy, "Superposed the reference plots from %s (color=%d)", filename, color);
      DyInfoLine (dummy);
     str = strtok(NULL," ");
    }
 free(result);

 fl_set_button(dyoptions->indelebile,ON);

 return;
}


/*------------------------------------------ DyXformRefPlotFillBrowser ---*/

void DyXformRefPlotFillBrowser ()
{
 int i0, i;
 char dummy[LSTRING_MAX];

 DyDebugPrint(__FUNCTION__,0);

 fl_clear_browser(reffile->file_list);

 if (Dy->nreffile <= 0) return;

 i0 = -1;
 for (i=0; i<Dy->nreffile; i++)
    {sprintf(dummy,"%s  %d", Dy->reffilename[i], Dy->refcolor[i]);
     fl_add_browser_line (reffile->file_list, dummy);
    }

 return;
}


/*------------------------------------------- DyXformRefPlotRemove ---*/

void DyXformRefPlotRemove (FL_OBJECT *obj, long arg)
{
 long i0, nfile;
 const char *str;
 char filename[LSTRING_MAX];
 int color;

 DyDebugPrint(__FUNCTION__,0);

 nfile = fl_get_browser_maxline(reffile->file_list);
 if (nfile <= 0) return;

 i0 = fl_get_browser(reffile->file_list);
 if (i0 <= 0) i0 = nfile;

 str = fl_get_browser_line(reffile->file_list,i0);
 sscanf(str, "%s %d", filename, &color);

 DyRefPlotRemove(filename);
 if (arg == 0) RA_RefPlotRemove(filename);
 else          RA_RefPlot2DRemove(filename);
 DyXformRefPlotFillBrowser();

 printf("DyXformRefPlotRemove> Removed %s  color=%d  from reference files list\n", filename, color);

 return;
}


/*------------------------------------------- DyXformRefPlotEdit ---*/

void DyXformRefPlotEdit (FL_OBJECT *obj, long arg)
{
 int i, i0;
 const char *str, *answer;
 char dummy[LSTRING_MAX], filename[LSTRING_MAX], colorstring[16];

 i0 = fl_get_browser(reffile->file_list);
 if (i0 <= 0) return;

 str = fl_get_browser_line(reffile->file_list,i0);

 sscanf(str,"%s %s", filename, colorstring);

 answer = fl_show_input("Change color number:", colorstring);

 if (answer == NULL)      return;
 if (strlen(answer) == 0) return;

 for (i=0; i<Dy->nreffile; i++)
    {if (strcmp(filename,Dy->reffilename[i]) == 0)
       {Dy->refcolor[i] = atoi(answer);
        sprintf(dummy,"%s  %d", Dy->reffilename[i], Dy->refcolor[i]);
        fl_replace_browser_line(reffile->file_list,i0,dummy);
        RA_RefPlotModify(Dy->reffilename[i], Dy->refcolor[i]);
       }
    }

 return;
}


/*------------------------------------------ DyXformHideShowPlot ---*/

void DyXformHideShowPlot (FL_OBJECT *obj, long arg)
{
 const char *str;
 char browserline[LSTRING_MAX];
 long nplot, i, j;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 /*--- Hide the selected plots ---*/

 if (arg == 0)
    {
     for (i=1; i<=nplot; i++)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i))
           {Plot[i]->hidden = ON;
            DyPlotSetBrowserline(i, browserline);
            fl_replace_browser_line(mainbrowser->plots, i, browserline);
            fl_deselect_browser_line(mainbrowser->plots,i);
            for (j=i+1; j<=nplot; j++)
               {if (Plot[j]->superposed == OFF) break;
                Plot[j]->hidden = ON;
                DyPlotSetBrowserline(j, browserline);
                fl_replace_browser_line(mainbrowser->plots, j, browserline);
                fl_deselect_browser_line(mainbrowser->plots,j);
               }
           }
        }
    }

 /*--- Show the selected plots ---*/

 else
    {
     for (i=1; i<=nplot; i++)
        {
         if (fl_isselected_browser_line(mainbrowser->plots,i))
           {Plot[i]->hidden = OFF;
            DyPlotSetBrowserline(i, browserline);
            fl_replace_browser_line(mainbrowser->plots, i, browserline);
            fl_deselect_browser_line(mainbrowser->plots,i);
            if (Plot[i]->superposed == ON)
              {j = DyPlotGetIsuperposed(i);
               Plot[j]->hidden = OFF;
               DyPlotSetBrowserline(j, browserline);
               fl_replace_browser_line(mainbrowser->plots, j, browserline);
               fl_deselect_browser_line(mainbrowser->plots,j);
              }
           }
        }
    }

 return;
}


/*------------------------------------------ DyXformTransformPlot ---*/

void DyXformTransformPlot (FL_OBJECT *obj, long arg)
{
 const char *answer;
 char  browserLine[LSTRING_MAX];
 long nplot, i, i0, flag2ch, newtype;

 DyDebugPrint(__FUNCTION__,0);

 nplot = fl_get_browser_maxline(mainbrowser->plots);

 /*--- Check if  at least one plot is selected ---*/

 i0 = fl_get_browser(mainbrowser->plots);
 if (i0 <= 0) return;
/**
 i0 = 0;
 for (i=1; i<=nplot; i++)
    {if (fl_isselected_browser_line(mainbrowser->plots,i)) {i0 = i; break;}}
 if (i0 == 0) return;
**/

 /*--- Get and plot type name ---*/

 answer = fl_show_input("New plot type", Dy->typeName[Plot[i0-1]->type]);
 if (!answer) return;

 /*--- Check that the provided type name is a known type  ---*/

 newtype = -1;
 for (i=0; i<NARG_MAX; i++) {if (strcmp(answer,Dy->typeName[i]) == 0) {newtype = i; break;}}

 /*--- If not, try type names in other formats ---*/

 if (newtype == -1)
    {if (strcmp(answer,"Time") == 0 || strcmp(answer,"time") == 0)           newtype = PLOTTIME;
     if (strcmp(answer,"fft") == 0 || strcmp(answer,"spectrum") == 0)        newtype = PLOTFFT;
     if (strcmp(answer,"FFTtime") == 0 || strcmp(answer,"ffttime") == 0)     newtype = PLOTFFTTIME;
     if (strcmp(answer,"Trfct") == 0 || strcmp(answer,"trfct") == 0 || strcmp(answer,"transfer function") == 0) newtype = PLOTTR;
     if (strcmp(answer,"Trfcttime") == 0 || strcmp(answer,"trfcttime") == 0) newtype = PLOTTRTIME;
     if (strcmp(answer,"Cohe") == 0 || strcmp(answer,"cohe") == 0 || strcmp(answer,"coherence") == 0)  newtype = PLOTCOHE;
     if (strcmp(answer,"Cohetime") == 0 || strcmp(answer,"cohetime") == 0)   newtype = PLOTCOHETIME;
     if (strcmp(answer,"1d") == 0 || strcmp(answer,"histo") == 0)            newtype = PLOT1D;
     if (strcmp(answer,"1dtime") == 0 || strcmp(answer,"histotime") == 0)    newtype = PLOT1DTIME;
     if (strcmp(answer,"2d") == 0 || strcmp(answer,"histo2d") == 0)          newtype = PLOT2D;
     if (strcmp(answer,"Raw") == 0 || strcmp(answer,"raw") == 0 || strcmp(answer,"image") == 0)    newtype = PLOTRAW;
     if (strcmp(answer,"Rawtime") == 0 || strcmp(answer,"rawtime") == 0)   newtype = PLOTRAWTIME;
     if (strcmp(answer,"Audio") == 0 || strcmp(answer,"audio") == 0)       newtype = PLOTAUDIO;
    }

 /*--- If new type provided is not known, help the user and do not continue ---*/

 if (newtype == -1)
   {printf("Plot type \"%s\" is not known...\n", answer);
    printf("Available type are:\n");
    for (i=0; i<NARG_MAX; i++) {printf("%s | ", Dy->typeName[i]);}
    printf("\n");
    return;
   }

 /*--- Check if new plot type is a 2 channels plot ---*/

 flag2ch = OFF;
 if (newtype == PLOTTR || newtype == PLOTCOHE || newtype == PLOTTRTIME || newtype == PLOTCOHETIME ||
     newtype == PLOT2D ) flag2ch = ON;

 /*--- Transform into newtype plot all the plots selected ---*/

 for (i=1; i<=nplot; i++)
    {
     if (fl_isselected_browser_line(mainbrowser->plots,i))
       {
        if (((strstr(Plot[i]->title,SEP2VAR)!=NULL || strstr(Plot[i]->title,SEP2VARTR)!=NULL) && flag2ch == OFF) ||
            (strstr(Plot[i]->title,SEP2VAR)==NULL && strstr(Plot[i]->title,SEP2VARTR)==NULL && flag2ch == ON))
           {printf("DyXformTransformPlot> type=%s  newtype=%s ... I do not transform \n",
                   Dy->typeName[Plot[i]->type], answer);
            continue;
           }

        Plot[i]->type = newtype;
        DyPlotSetTitle(i);
        DyPutDefaultInPlot(i);
/**
        str = fl_get_browser_line(mainbrowser->plots,i);
        if (strncmp(str,"@C",2) == 0) sprintf (browserLine, "@C14 %d  %s  %s", i, Plot[i]->name, Plot[i]->title);
        else                          sprintf (browserLine, "%d  %s  %s", i, Plot[i]->name, Plot[i]->title);
**/
        DyPlotSetBrowserline(i,browserLine);
        browserLine[strlen(browserLine)] = '\0';
        fl_replace_browser_line(mainbrowser->plots, i, browserLine);
        fl_deselect_browser_line(mainbrowser->plots,i);
       }
    }

 return;
}


/*------------------------------------------ DyXformUpdatePlotBrowser ---*/

void DyXformUpdatePlotBrowser (long id, long arg)
{
 char browserLine[LSTRING_MAX];
 long nplot;

 DyDebugPrint(__FUNCTION__,0);

 if (id <= 0) {printf("DyXformUpdatePlotBrowser> plot id = %ld \n", id); return;}

 /*--- Add or replace one plot in the plots browser ---*/

 nplot = fl_get_browser_maxline(mainbrowser->plots);
 if (nplot >= NPLOT_MAX-1 && arg == ADD_PLOT) {printf("  Maximum number of plots reached. Cannot add more. \n"); return;}

 DyPlotSetBrowserline(id, browserLine);

 if (arg == ADD_PLOT)     fl_add_browser_line(mainbrowser->plots, browserLine);
 if (arg == REPLACE_PLOT) fl_replace_browser_line(mainbrowser->plots, id, browserLine);

 return;
}

/*------------------------------------------ DyXformPlotFillBrowser ---*/

void DyXformPlotFillBrowser()
{
 char browserLine[LSTRING_MAX];
 long i;

 DyDebugPrint(__FUNCTION__,0);

 fl_clear_browser(mainbrowser->plots);

 for (i=1; i<=Dy->nplot; i++)
    {
     DyPlotSetBrowserline(i,browserLine);
     fl_add_browser_line(mainbrowser->plots, browserLine);
    }

 return;
}



/*------------------------------------------- DyXformPutDefaultInPanel ---*/

void DyXformPutDefaultInPanel (FL_OBJECT *obj, long arg)
{
 long id;

 DyDebugPrint(__FUNCTION__,0);

 id = fl_get_browser(mainbrowser->plots);
 if (id <= 0) return;

/**
 DyPutDefaultInPlot(id);
 DyXformPlotToPanel(id);
**/

/*--- Allow to put default values not directly in the concerned plot ---*/
/*--- and to cancel plot edition even after pushing "Put default" button ---*/

 DyCopyPlot(NPLOT_MAX-1,id,ON);
 DyPutDefaultInPlot(NPLOT_MAX-1);
 DyXformPlotToPanel(NPLOT_MAX-1);

 return;
}



/***************/
/*  VARIABLES  */
/***************/

/*------------------------------------------- DyXformRootUtil ---*/

void DyXformRootUtil (FL_OBJECT *obj, long arg)
{
 double zoomvalue;
 int linewidth;

 DyDebugPrint(__FUNCTION__,0);

 if (obj == dyoptions->zoomstat)
   {zoomvalue = atof(fl_get_input(dyoptions->zoomstat));
    if (zoomvalue > 0) RA_ZoomAllStat (zoomvalue);
   }
 if (obj == dyoptions->zoomlegend)
   {zoomvalue = atof(fl_get_input(dyoptions->zoomlegend));
    if (zoomvalue > 0) RA_ZoomAllLegends (zoomvalue);
   }
 if (obj == dyoptions->zoomtimestring) 
   {zoomvalue = atof(fl_get_input(dyoptions->zoomtimestring));
    if (zoomvalue > 0) RA_ZoomAllTimeString (zoomvalue);
   }
 if (obj == dyoptions->zoomtitle) 
   {zoomvalue = atof(fl_get_input(dyoptions->zoomtitle));
    if (zoomvalue > 0) RA_ZoomAllTitle (zoomvalue);
   }
 if (obj == dyoptions->zoomlabelx)
   {zoomvalue = atof(fl_get_input(dyoptions->zoomlabelx));
    if (zoomvalue > 0) RA_ZoomAllLabels (zoomvalue, "xaxis");
   }
 if (obj == dyoptions->zoomlabely)
   {zoomvalue = atof(fl_get_input(dyoptions->zoomlabely));
    if (zoomvalue > 0) RA_ZoomAllLabels (zoomvalue, "yaxis");
   }
 if (obj == dyoptions->linewidth)
   {linewidth = atoi(fl_get_input(dyoptions->linewidth));
    if (linewidth > 0)
      {Dy->linewidth = linewidth;
       RA_SetLineWidth_AllPlots (Dy->linewidth);
      }
   }

 if (obj == dyoptions->loglinx)         RA_LogLinAll ("xaxis");
 if (obj == dyoptions->logliny)         RA_LogLinAll ("yaxis");

 return;
}


/*------------------------------------------- DyXformDeSelect ---*/

void DyXformDeSelect (FL_OBJECT *obj, long arg)
{
 DyDebugPrint(__FUNCTION__,0);

 if (obj == mainbrowser->deselectch)      fl_deselect_browser(mainbrowser->channels);
 if (obj == mainbrowser->deselectPlots)   fl_deselect_browser(mainbrowser->plots);

 return;
}



/*-------------------------------------------- DyXformChDescription ---*/

void DyXformChDescription (FL_OBJECT *obj, long arg)
{
 const char *str;
 char *str1, chname[LSTRING_MAX], chfile[LSTRING_MAX];
 int i, j, nline;

 DyDebugPrint(__FUNCTION__,0);

 nline = fl_get_browser_maxline(mainbrowser->channels);
 if (nline <= 0) return;

 /*--- Open the text file containing the channels descriptions ---*/

 strcpy(chfile,__FILE__);
 str1 = strstr(chfile,"/"); if (str1) *str1 = '\0';
 str1 = strstr(chfile,"/"); if (str1) *str1 = '\0';
 strcat(chfile,"/cmt/chlist.txt");

 /*--- Each user-defined channel found is described ---*/

 printf("\n --- Description of channels ---\n\n");
 for (i=1; i<=nline; i++)
    {if (fl_isselected_browser_line(mainbrowser->channels,i))
       {str = fl_get_browser_line(mainbrowser->channels,i);
        sscanf(str,"%s",chname);
        if (strncmp(chname,OPSIGNAL,strlen(OPSIGNAL)) == 0)
          {for (j=Dy->nch-1; j>=0; j--)
              {if (strcmp(chname,Dy->ch[j]->name) == 0) DyChDescription(chfile, chname);}
          }
        else
          {DyChDescription(chfile, chname);}
       }
    }
 printf("\n ------------------------------\n\n");

 return;
}



/*************/
/*   FILES   */
/*************/


/*-------------------------------------------- DyXformFileRemove ---*/

void DyXformFileRemove (FL_OBJECT *obj, long arg)
{
 const char *str;
 char inputname[LSTRING_MAX], chname[LSTRING_MAX];
 long nline, i; 
 double sampFreq;

 DyDebugPrint(__FUNCTION__,0);

 nline = fl_get_browser_maxline(mainbrowser->channels);
 if (nline <= 0) return;

 /*--- Get first line selected in the channels browser ---*/

 for (i=1; i<=nline; i++)
    {if (fl_isselected_browser_line(mainbrowser->channels,i))
       {str = fl_get_browser_line(mainbrowser->channels,i);
        sscanf(str,"%s : %lfHz %s",chname,&sampFreq,inputname);
        break;
       }
    }

 /*--- Remove all the channels whose inputname is identical to the one of the selected channel ---*/

 for (i=Dy->nch-1; i>=0; i--)
    {if (strcmp(Dy->ch[i]->inputname,inputname) == 0) DyChRemove(i);}

 /*--- Update the channels browser ---*/

 DyXformChannelFillBrowser();

 return;
}


/*-------------------------------------------- DyXformChRemove ---*/

void DyXformChRemove (FL_OBJECT *obj, long arg)
{
 const char *str;
 char chname[LSTRING_MAX];
 int i, j, nline;

 DyDebugPrint(__FUNCTION__,0);

 nline = fl_get_browser_maxline(mainbrowser->channels);
 if (nline <= 0) return;

 /*--- Each user-defined channel found is remove ---*/

 for (i=1; i<=nline; i++)
    {if (fl_isselected_browser_line(mainbrowser->channels,i))
       {str = fl_get_browser_line(mainbrowser->channels,i);
        sscanf(str,"%s",chname);
        if (strncmp(chname,OPSIGNAL,strlen(OPSIGNAL)) == 0)
          {for (j=Dy->nch-1; j>=0; j--)
              {if (strcmp(chname,Dy->ch[j]->name) == 0) DyChRemove(j);}
          }
       }
    }

 /*--- Update the channels browser ---*/

 DyXformChannelFillBrowser();

 return;
}


/*------------------------------------------ DyXformUpdateChList ---*/

void DyXformUpdateChList (FL_OBJECT *fl, long arg)
{
 double gps_of_update;

 DyDebugPrint(__FUNCTION__,0);

 gps_of_update = atof(fl_get_input(mainbrowser->gps_of_update));
 if (fabs(gps_of_update) < 700000000) gps_of_update = Dy->startTime + gps_of_update;

 Dy->startTime = gps_of_update;
 DyXformSetStartTime(0);

 DyUpdateChList();

 return;
}



/**************************************************/
/*                                                */
/* File: DyHelp.c                                 */
/* Copyright (C) 2003, D. Verkindt                */
/* For the licensing terms see the LICENSE file.  */
/*                                                */
/**************************************************/

/*                 **************************                 */
/*                 * HELPS, INIT and RESET  *                 */
/*                 **************************                 */

/*------------------------------------------ DyXformHelpPanel ---*/

void DyXformHelpPanel (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob, *browser,*close, *helpbutton;
 long xsize, ysize;
 char dummy[LSTRING_MAX];

 if (fl_form_is_visible(helppanel)) return;

 xsize      = 550;
 ysize      = 410;
 helppanel  = fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob         = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 helpbutton = fl_add_checkbutton(FL_PUSH_BUTTON,20,ysize-30,30,30,"Help Ballons");
   fl_set_object_lsize(helpbutton,FL_NORMAL_SIZE);
   fl_set_object_lstyle(helpbutton,FL_TIMESBOLD_STYLE);
 browser    = fl_add_browser(FL_HOLD_BROWSER,10,10,xsize-20,ysize-50,"Help");
    fl_set_object_color(browser,FL_LEFT_BCOL,FL_WHEAT);
    fl_set_object_lstyle(browser,FL_TIMESBOLD_STYLE);
 close = fl_add_button(FL_NORMAL_BUTTON,xsize-70,ysize-30,60,20,"Close");
    fl_set_object_color(close,FL_WHEAT,FL_COL1);
    fl_set_object_lstyle(close,FL_TIMESBOLD_STYLE);
    fl_set_object_callback(close,DyXformClosePanel,HELPPANEL);
 fl_end_form();

 fl_add_browser_line(browser,"\n");
 sprintf(dummy,"             dataDisplay (%s)\n ", DYVERSION);
 sprintf(dummy,"%s  using:  Fr(%s), Root(%s) , GUI Xforms (v%dr%d)",
                dummy, FrVersion, ROOT_RELEASE, FL_VERSION, FL_REVISION);
#ifndef NOCM
 sprintf(dummy,"%s , DyFd (%s)\n", dummy, DYFDVERSION);
#endif
 fl_add_browser_line(browser, dummy);
 fl_add_browser_line(browser,"********************   HELP SUMMARY   ***********************\n\n"
                             "  First, you must get some data (under frame or wave format).\n"
                             "  Use the 'Input Data' menu and choose a source of data      \n"
                             "  (for instance a 'Read File' or a 'Connect online').        \n"
                             "  Then, you will see in the bottom line of the mainpanel that\n"
                             "  a first frame has been read successfully. The data channels\n"
                             "  can be selected to do operations, filtering, various plots,\n"
                             "  and audio output, using 'Data and Plots' panel.\n\n");
 fl_add_browser_line(browser,"                              *********                      \n\n"
                             "  - For more help, you can look at the README file           \n");
 sprintf(dummy,"    in the /virgoApp/Dy/%s directory.                \n", DYVERSION);
 fl_add_browser_line(browser,dummy);
 fl_add_browser_line(browser,"  - For detailed help, you can use the 'Help ballons' button \n"
                             "    of the mainpanel. In this case, each field or button     \n"
                             "    shows a message each time the mouse passes on it.        \n");
 sprintf(dummy,"  - For help, see also paper documentation: /virgoApp/Dy/%s/doc/DyDoc.ps. \n\n", DYVERSION);
 fl_add_browser_line(browser,dummy);
 fl_add_browser_line(browser,"                              *********                     \n\n"
                             "  D. Verkindt                                               \n"
                             "  (verkindt@lapp.in2p3.fr)                                  \n\n");

 DyXformRaisePanel(NULL,HELPPANEL);
 return;
}


/*------------------------------------------ DyXformCoffeePanel ---*/

void DyXformCoffeePanel (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob;
 long xsize, ysize;

 if (fl_form_is_visible(coffeepanel)) return;

 xsize      = 670;
 ysize      = 190;
 if (arg == 3) {xsize=1020; ysize=370;}
 coffeepanel= fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
      fl_set_object_color(ob,FL_WHITE,FL_COL1);

 if (arg == 0)
    ob = fl_add_text(FL_NORMAL_TEXT,230,60,430,80,"Since April 2003, dataDisplay can also make the coffee !!");
 if (arg == 1)
    ob = fl_add_text(FL_NORMAL_TEXT,230,60,430,80,"Since 32nd of July 2004, dataDisplay can also read in the future !!");
 if (arg == 2)
    ob = fl_add_text(FL_NORMAL_TEXT,230,60,430,80,"Since 14st of September 2015, dataDisplay can detect Gravitational Waves!!");

    fl_set_object_color(ob,FL_WHITE,FL_MCOL);
    fl_set_object_lsize(ob,FL_MEDIUM_SIZE);
    fl_set_object_lalign(ob,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
/*    fl_set_object_boxtype(ob,FL_DOWN_BOX); */
    ob = fl_add_button(FL_RETURN_BUTTON,xsize-90,ysize-40,80,30,"Close");
    fl_set_object_color(ob,FL_WHEAT,FL_COL1);
    fl_set_object_lsize(ob,FL_NORMAL_SIZE);
    fl_set_object_lstyle(ob,FL_TIMESBOLD_STYLE);
    fl_set_object_callback(ob,DyXformClosePanel,COFFEEPANEL);
 if (arg==3) ob = fl_add_pixmap(FL_NORMAL_PIXMAP,10,10,910,310,"");
 else        ob = fl_add_pixmap(FL_NORMAL_PIXMAP,20,20,170,150,"");
 if (arg == 0) fl_set_pixmap_data(ob, coffee);
 if (arg == 1) fl_set_pixmap_data(ob, cristalBall);
 if (arg == 2) fl_set_pixmap_data(ob, andromede);
 if (arg == 3) fl_set_pixmap_data(ob, help_ffttime);
 fl_end_form();

 DyXformRaisePanel(NULL,COFFEEPANEL);
 return;
}


/*------------------------------------------ DyXformHelpMainPanel ---*/

void DyXformHelpMainPanel (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == mainpanel->inputMenu)
            fl_show_oneliner("To select a source of data is the first\n"
                             "action to do. You can read input data from\n"
                             "a frame file (Read FFL or Read File), \n"
                             "a Cm server (Connect Online),\n"
                             "a Data Distribution server (Connect to DataSender).\n"
                             "a shared memory (Read Shared Memory).\n"
                             "or a wave formatted file (Read Wave File).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->configMenu)
            fl_show_oneliner("Menu to load or save the\n"
                             "current configuration in a file",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->clear)
            fl_show_oneliner("Remove only the window showing the plots \n"
                             "This will reset any zoom or plots configuration done previously",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->mainbrowser)
            fl_show_oneliner("Shows the panel where you can:\n"
                             "- select the data to plot,\n"
                             "- manage, edit or remove plots,\n"
                             "- do operations with the channels...",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->refplotMenu)
            fl_show_oneliner("Menu of the actions that can be done with the reference \n"
                             "plots that you want to superpose to the current plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->toolsMenu)
            fl_show_oneliner("Menu of the various tools, options, checks and debug",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startTime)
            fl_show_oneliner("Put here the GPS time of first data to read\n"
                             "when the button Start is pushed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->startUTCdate)
            fl_show_oneliner("Show the starting UTC date in a standard format.\n"
                             "Put here the UTC date in some approximative time format\n"
                             "The starting GPS time will be automatically updated\n"
                             "and UTC date will be put into a standard format.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->duration)
            fl_show_oneliner("Put here the number of seconds\n"
                             "over which you want to read data \n"
                             "If this number is negative,\n"
                             "it will be interpreted as infinity.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentGTime)
            fl_show_oneliner("GPS time of the last data read.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->UTCdate)
            fl_show_oneliner("UTC date of the last data read.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainpanel->start)
            fl_show_oneliner("Start reading data and displaying plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->pause)
            fl_show_oneliner("Pause the data reading until button.\n"
                             "Continue or Next Refresh is pushed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->continuer)
            fl_show_oneliner("Continue to read data.\n"
                             "Active only after button Pause has been pushed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->nextUpdate)
            fl_show_oneliner("Continue to read data up to the next plot refresh.\n"
                             "Active only after button Pause has been pushed",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->stop)
            fl_show_oneliner("Stop reading data and close Cm Ethernet\n"
                             "or Shared Memory connection if needed.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->infoLine)
            fl_show_oneliner("Status line that shows the last action made\n"
                             "or the current status of the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->latency)
            fl_show_oneliner("Difference between the current GPS time and the data GPS time",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->scanplots)
            fl_show_oneliner("Slider to move along the X axis of the last zoomed plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->datenow)
            fl_show_oneliner("Button that sets the start UTC date and GPS time\n"
                             "to the current time",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->maxduration)
            fl_show_oneliner("Button that sets the duration of data read\n"
                             "to the max time window of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->waitdata)
            fl_show_oneliner("If this button is on, the dataDisplay will wait for\n"
                             "new incoming data when it reaches the end of file.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainpanel->currentsource)
            fl_show_oneliner("Here is displayed the current source of the data read by the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (mainpanel->inputMenu,     DyXformHelp);
     fl_set_object_posthandler (mainpanel->configMenu,    DyXformHelp);
     fl_set_object_posthandler (mainpanel->clear,         DyXformHelp);
     fl_set_object_posthandler (mainpanel->mainbrowser,   DyXformHelp);
     fl_set_object_posthandler (mainpanel->refplotMenu,   DyXformHelp);
     fl_set_object_posthandler (mainpanel->toolsMenu,     DyXformHelp);
     fl_set_object_posthandler (mainpanel->startTime,     DyXformHelp);
     fl_set_object_posthandler (mainpanel->startUTCdate,  DyXformHelp);
     fl_set_object_posthandler (mainpanel->currentGTime,  DyXformHelp);
     fl_set_object_posthandler (mainpanel->UTCdate,       DyXformHelp);
     fl_set_object_posthandler (mainpanel->duration,      DyXformHelp);
     fl_set_object_posthandler (mainpanel->start,         DyXformHelp);
     fl_set_object_posthandler (mainpanel->pause,         DyXformHelp);
     fl_set_object_posthandler (mainpanel->continuer,     DyXformHelp);
     fl_set_object_posthandler (mainpanel->nextUpdate,    DyXformHelp);
     fl_set_object_posthandler (mainpanel->stop,          DyXformHelp);
     fl_set_object_posthandler (mainpanel->infoLine,      DyXformHelp);
     fl_set_object_posthandler (mainpanel->latency,       DyXformHelp);
     fl_set_object_posthandler (mainpanel->scanplots,     DyXformHelp);
     fl_set_object_posthandler (mainpanel->datenow,       DyXformHelp);
     fl_set_object_posthandler (mainpanel->maxduration,   DyXformHelp);
     fl_set_object_posthandler (mainpanel->waitdata,      DyXformHelp);
     fl_set_object_posthandler (mainpanel->currentsource, DyXformHelp);
    }
 return;
}


/*------------------------------------------ DyXformHelpMainBrowser ---*/

void DyXformHelpMainBrowser (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == mainbrowser->channels)
            fl_show_oneliner("List of channels seen in the first in each input file.\n"
                             "It is updated each time you push button Stop.\n"
                             "If you want a time plot of several channels,\n"
                             "click on those channels to select them\n"
                             "and press the TIME button\n"
                             "To deselect a line, click on it. To deselect all the lines, click on 'Deselect'.\n"
                             "If two lines are selected and a 2D or TrFct plot\n"
                             "is chosen, X is the upper and Y is the lower line.\n"
                             "If only one line was selected, it is put in X\n"
                             "and next line selected will be in Y", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectch)
            fl_show_oneliner("Deselect all the channels selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->searchch)
            fl_show_oneliner("Search for a channel name.", 
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->filterch)
            fl_show_oneliner("Put as first lines of the channels list a selection of channels.\n"
                             "For instance '*BMS*DC' will show first all the channels containing\n"
                             "BMS in their name and finishing with DC",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->sortbyname)
            fl_show_oneliner("Sort the channels by alphabetic order.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->sortbyfreq)
            fl_show_oneliner("Sort the channels by increasing sampling frequency.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->getchinfo)
            fl_show_oneliner("Provide a description of the channels you have selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->removeProc)
            fl_show_oneliner("Remove from the list the selected channels.\n"
                             "Works only with the channels created with the 'Create channel' button",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->removefile)
            fl_show_oneliner("Remove from the list all the channels read from same source\n"
                             "as the ones of the selected channels.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->createch)
            fl_show_oneliner("Create a new channel defined by the operations showed in the operation string.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->setopstring)
            fl_show_oneliner("Put in the operation string the channels you selected.\n"
                             "The proposed operation can then be modified by you before pushing the 'Create channel' button.\n"
                             "For instance, if you write the string\n"
                             "mychannel = sin(Pr_B1_ACq)*Pr_B1_ACp , it will create a new channel\n"
                             "named mychannel that you can use to do plots, like any other channel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->opstring)
            fl_show_oneliner("Contains the operation string made with the channels you selected.\n"
                             "You can modify this operation before pushing the 'Create channel' button.\n"
                             "For instance, if you write the string\n"
                             "mychannel = sin(Pr_B1_ACq)*Pr_B1_ACp , it will create a new channel\n"
                             "named mychannel that you can use to do plots, like any other channel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->createtrigch)
            fl_show_oneliner("This button allows to set as a trigger\n"
                             "a condition based on logical and mathematical operations on the channels.\n"
                             "For instance: 'Pr_B5_DC>0.05 AND Qc_Moni_ScienceMode=1'\n"
                             "Then, plots will be computed only for data that verify this condition.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == mainbrowser->getchfromplot)
            fl_show_oneliner("Select in the channels list the channels used in the plots you have selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotTime)
            fl_show_oneliner("Create, for the data selected, a time\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotFFT)
            fl_show_oneliner("Create, for the data selected, a Spectrum\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotTrFct)
            fl_show_oneliner("Create, for the data selected, a Transfer Function\n"
                             "plot with suitable default parameters.\n"
                             "Transfer Function is FFT(X)/FFT(Y).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotCohe)
            fl_show_oneliner("Create, for the data selected, a Coherence Function\n"
                             "plot with suitable default parameters.\n"
                             "Coherence function has module 1 if the\n"
                             "two signals are correlated.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot1D)
            fl_show_oneliner("Create, for the data selected, a 1D\n"
                             "plot with suitable default parameters",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot2D)
            fl_show_oneliner("Create, for the data selected, a 2D\n"
                             "plot with suitable default parameters.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotFFTTime)
            fl_show_oneliner("Create, for the data selected, a time-frequency\n"
                             "plot with suitable default parameters.\n"
                             "It is based on a FFT made over timeWindow seconds\n"
                             "and moving by steps of timeStep seconds\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotTrFctTime)
            fl_show_oneliner("Create, for the data selected, a time-transfer_function\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the transfer function module\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotCoheTime)
            fl_show_oneliner("Create, for the data selected, a time-coherence_function\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the coherence function module\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plot1DTime)
            fl_show_oneliner("Create, for the data selected, a time-1d distribution\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the 1D distribution of the data\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotRawTime)
            fl_show_oneliner("Create, for the data selected, a time-raw\n"
                             "plot with suitable default parameters, where\n"
                             "on vertical axis are the data of the raw\n"
                             "vector taken from the frame (like spectra)\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotRaw)
            fl_show_oneliner("Create, for the data selected, a raw\n"
                             "plot with suitable default parameters, where\n"
                             "are plotted the data of the raw vector taken \n"
                             "from the frame (like spectra or camera images)\n"
                             "(see the edit panel of this plot).",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plotAudio)
            fl_show_oneliner("Create, for the data selected, a virtual plot used\n"
                             "to hear online or offline the selected data.\n"
                             "By default, a wave audio file will be written on disk.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->plots)
            fl_show_oneliner("List of plots created.\n"
                             "You can remove a plot from the list\n"
                             "or select it and edit its parameters.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->editPlot)
            fl_show_oneliner("Open an edition panel for the selected plot.\n"
                             "This panel shows you the parameters\n"
                             "of the plot that you can modify",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
/**
     if (ob == mainbrowser->editGenePar)
            fl_show_oneliner("Open an edition panel for the parameters.\n"
                             "common to almost all the plots.\n"
                             "Any change of parameter in this panel is propagated\n"
                             "to all the plots\n"
                             "Remark: Time window in this panel is\n"
                             "automatically changed to get nBin=a power of 2.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
**/
     if (ob == mainbrowser->removePlot)
            fl_show_oneliner("Remove the selected plot from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->savePlots)
            fl_show_oneliner("According to the filename extension, save in a\n"
                             "root, jpg or ps formatted file all the last plots made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->clear_plotbrowser)
            fl_show_oneliner("Remove ALL the plots from the list\n"
                             "Take care! There is no undo in the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->superposed)
            fl_show_oneliner("This button superpose the plots you have selected in\n"
                             "the browser. A sign (s) appears then in front of each plot\n"
                             "superposed, except the first one.\n"
                             "Selecting just one plot which has a sign (s) and clicking\n"
                             "on this button will remove it from the superposition.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->unsame)
            fl_show_oneliner("This button removes the superposition\n"
                             "for the selected plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->exchange)
            fl_show_oneliner("This button exchanges the variables in each of the 2D,\n"
                             " TRFCT or COHE plot selected in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->updateChList)
            fl_show_oneliner("This button updates the list of channels\n"
                             "according to the GPS time provided next to it or\n"
                             "according to the Start GPS time put in the mainpanel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->gps_of_update)
            fl_show_oneliner("Put here the GPS time of the data that will be\n"
                             "used to update the list of channels.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->copyPlot)
            fl_show_oneliner("Create a new plot with same\n"
                             "parameters as the plot selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->transform)
            fl_show_oneliner("Transform the selected plots into an other type of plots\n"
                             "with same number of channels. New type plot is set with default parameters.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->up)
            fl_show_oneliner("Move upwards in the list the selected plots.\n"
                             "The move takes into account the superposed plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->down)
            fl_show_oneliner("Move downwards in the list the selected plots.\n"
                             "The move takes into account the superposed plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->show)
            fl_show_oneliner("The selected plots are shown while running.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->hide)
            fl_show_oneliner("The selected plots are hidden while running.\n"
                             "This allow to temporarily hide some plots from the display.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->deselectPlots)
            fl_show_oneliner("Deselect all the plots currently selected.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == mainbrowser->rowcol)
            fl_show_oneliner("Change the plots row/columns configuration.\n"
                             "When you push this button, a field appears\n"
                             "If you put 3x4 in it, the plots will appear on 3 rows and 4 columns",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (mainbrowser->channels,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->deselectch,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->searchch,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->filterch,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->sortbyname,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->sortbyfreq,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->getchinfo,         DyXformHelp);
     fl_set_object_posthandler (mainbrowser->removeProc,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->removefile,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->createch,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->setopstring,       DyXformHelp);
     fl_set_object_posthandler (mainbrowser->opstring,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->createtrigch,      DyXformHelp);

     fl_set_object_posthandler (mainbrowser->getchfromplot,     DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotTime,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotFFT,           DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotTrFct,         DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotCohe,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotFFTTime,       DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotTrFctTime,     DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotCoheTime,      DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plot1DTime,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotRawTime,       DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotRaw,           DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plot1D,            DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plot2D,            DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plotAudio,         DyXformHelp);
     fl_set_object_posthandler (mainbrowser->plots,             DyXformHelp);
     fl_set_object_posthandler (mainbrowser->editPlot,          DyXformHelp);
/*
     fl_set_object_posthandler (mainbrowser->editGenePar,       DyXformHelp);
*/
     fl_set_object_posthandler (mainbrowser->removePlot,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->savePlots,         DyXformHelp);
     fl_set_object_posthandler (mainbrowser->clear_plotbrowser, DyXformHelp);
     fl_set_object_posthandler (mainbrowser->close,             DyXformHelp);
     fl_set_object_posthandler (mainbrowser->superposed,        DyXformHelp);
     fl_set_object_posthandler (mainbrowser->unsame,            DyXformHelp);
     fl_set_object_posthandler (mainbrowser->exchange,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->updateChList,      DyXformHelp);
     fl_set_object_posthandler (mainbrowser->gps_of_update,     DyXformHelp);
     fl_set_object_posthandler (mainbrowser->copyPlot,          DyXformHelp);
     fl_set_object_posthandler (mainbrowser->transform,         DyXformHelp);
     fl_set_object_posthandler (mainbrowser->up,                DyXformHelp);
     fl_set_object_posthandler (mainbrowser->down,              DyXformHelp);
     fl_set_object_posthandler (mainbrowser->show,              DyXformHelp);
     fl_set_object_posthandler (mainbrowser->hide,              DyXformHelp);
     fl_set_object_posthandler (mainbrowser->deselectPlots,     DyXformHelp);
     fl_set_object_posthandler (mainbrowser->rowcol,            DyXformHelp);
    }

 return;
}



/*------------------------------------------ DyXformHelpPlottime ---*/

void DyXformHelpPlottime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plottime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->chx_resampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->ymin)
            fl_show_oneliner("The lower y edge of the plot. To get an automatic\n"
                             "set of lower and upper edges, put 0",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->ymax)
            fl_show_oneliner("The upper y edge of the plot. To get an automatic\n"
                             "set of lower and upper edges, put 0",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->yoffset)
            fl_show_oneliner("A user defined offset that will be added to\n"
                             "the plot on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->yscale)
            fl_show_oneliner("A user defined scaling factor that will rescale\n"
                             "the plot on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->filter_fmin)
            fl_show_oneliner("Min frequency of the pass-band filter applied.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plottime->filter_fmax)
            fl_show_oneliner("Max frequency of the pass-band filter applied.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->autoscale)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->realunits)
            fl_show_oneliner("This button, if switched on, sets real units\n"
                             "of calibrated data on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->nodc)
            fl_show_oneliner("This button, if switched on, removes a mean value\n"
                             "from the data plotted.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->bits)
            fl_show_oneliner("This button, if switched on, allows to plot the bits\n"
                             "of each data sample. The time plot becomes a bits-time 2D plot.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->ok)
            fl_show_oneliner("Acknowledges the changes you made\n"
                             "and closes the panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancels any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plottime->bydefault, DyXformHelp);
     fl_set_object_posthandler (plottime->tsize,     DyXformHelp);
     fl_set_object_posthandler (plottime->chx_resampFreq,  DyXformHelp);
     fl_set_object_posthandler (plottime->tstep,     DyXformHelp);
     fl_set_object_posthandler (plottime->ymin,      DyXformHelp);
     fl_set_object_posthandler (plottime->ymax,      DyXformHelp);
     fl_set_object_posthandler (plottime->yscale,    DyXformHelp);
     fl_set_object_posthandler (plottime->yoffset,   DyXformHelp);
     fl_set_object_posthandler (plottime->filter_fmin, DyXformHelp);
     fl_set_object_posthandler (plottime->filter_fmax, DyXformHelp);
     fl_set_object_posthandler (plottime->gridx,     DyXformHelp);
     fl_set_object_posthandler (plottime->gridy,     DyXformHelp);
     fl_set_object_posthandler (plottime->logx,      DyXformHelp);
     fl_set_object_posthandler (plottime->logy,      DyXformHelp);
     fl_set_object_posthandler (plottime->realunits, DyXformHelp);
     fl_set_object_posthandler (plottime->autoscale, DyXformHelp);
     fl_set_object_posthandler (plottime->nodc,      DyXformHelp);
     fl_set_object_posthandler (plottime->bits,      DyXformHelp);
     fl_set_object_posthandler (plottime->ok,        DyXformHelp);
     fl_set_object_posthandler (plottime->close,     DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlotfft ---*/

void DyXformHelpPlotfft (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotfft->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->tsize)
            fl_show_oneliner("Size of the time window\n"
                             "(in seconds) used to compute each spectrum",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->chx_resampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->maxAverage)
            fl_show_oneliner("Maximum number of spectra to average",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->refresh_fft)
            fl_show_oneliner("Number of spectra computed between each plots' refresh",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->fmin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->fmax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->ymin)
            fl_show_oneliner("Lower (positive) value to display on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->ymax)
            fl_show_oneliner("Upper (positive) value to display on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotfft->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->realunits)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "calibrated values (instead of ADC values) on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->autoscale)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "an automatic scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->decayfft)
            fl_show_oneliner("If this button is switched on, it sets\n"
                             "a moving average based on the field 'Number of FFTs to average'",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->rmsfft)
            fl_show_oneliner("This button, if switched on, superposes to the spectrum plot\n"
                             "the plot of the rms computed from higher to lower frequencies.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->hertz)
            fl_show_oneliner("This button, if switched on, shows the spectrum plot \n"
                             "in 1/Hz units instead of the 1/sqrt(Hz) units.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotfft->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotfft->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plotfft->tsize,       DyXformHelp);
     fl_set_object_posthandler (plotfft->chx_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plotfft->maxAverage,  DyXformHelp);
     fl_set_object_posthandler (plotfft->refresh_fft, DyXformHelp);
     fl_set_object_posthandler (plotfft->tstep,       DyXformHelp);
     fl_set_object_posthandler (plotfft->fmin,        DyXformHelp);
     fl_set_object_posthandler (plotfft->fmax,        DyXformHelp);
     fl_set_object_posthandler (plotfft->ymin,        DyXformHelp);
     fl_set_object_posthandler (plotfft->ymax,        DyXformHelp);
     fl_set_object_posthandler (plotfft->nodc,        DyXformHelp);
     fl_set_object_posthandler (plotfft->gridx,       DyXformHelp);
     fl_set_object_posthandler (plotfft->gridy,       DyXformHelp);
     fl_set_object_posthandler (plotfft->logx,        DyXformHelp);
     fl_set_object_posthandler (plotfft->logy,        DyXformHelp);
     fl_set_object_posthandler (plotfft->realunits,   DyXformHelp);
     fl_set_object_posthandler (plotfft->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plotfft->decayfft,    DyXformHelp);
     fl_set_object_posthandler (plotfft->rmsfft,      DyXformHelp);
     fl_set_object_posthandler (plotfft->hertz,       DyXformHelp);
     fl_set_object_posthandler (plotfft->ok,          DyXformHelp);
     fl_set_object_posthandler (plotfft->close,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlotffttime ---*/

void DyXformHelpPlotffttime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotffttime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->chx_resampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plotffttime->dtime)
            fl_show_oneliner("Time resolution (in seconds)\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->dfreq)
            fl_show_oneliner("Frequency resolution (in Hertz)\n"
                             "of the time-frequency plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->fmin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->fmax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->tstep_fft)
            fl_show_oneliner("Size of the time window steps\n"
                             "(in % of FFT time window size)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->tstep)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->zmin)
            fl_show_oneliner("Min (positive) value on the Z axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->zmax)
            fl_show_oneliner("Max (positive) value on the Z axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotffttime->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotffttime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->realunits)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "calibrated values on the Z axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->autoscale)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "an automatic scaling of the Z axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->decayfft)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "a moving average that uses the number of averages defined",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->median_norm)
            fl_show_oneliner("This button, if switched on, normalizes the\n"
                             "spectrogram with median value computed at each frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotffttime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotffttime->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plotffttime->tsize,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->chx_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plotffttime->dtime,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->dfreq,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->tstep,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->tstep_fft,   DyXformHelp);
     fl_set_object_posthandler (plotffttime->fmin,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->fmax,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->zmin,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->zmax,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->nodc,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->gridx,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->gridy,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->gridz,       DyXformHelp);
     fl_set_object_posthandler (plotffttime->logx,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->logy,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->logz,        DyXformHelp);
     fl_set_object_posthandler (plotffttime->realunits,   DyXformHelp);
     fl_set_object_posthandler (plotffttime->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plotffttime->decayfft,    DyXformHelp);
     fl_set_object_posthandler (plotffttime->median_norm, DyXformHelp);
     fl_set_object_posthandler (plotffttime->ok,          DyXformHelp);
     fl_set_object_posthandler (plotffttime->close,       DyXformHelp);
    }

 return;
}



/*------------------------------------------ DyXformHelpPlot1dtime ---*/

void DyXformHelpPlot1dtime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot1dtime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-1d plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plot1dtime->stat_duration)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to do 1d statistics",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->chx_resampFreq)
            fl_show_oneliner("You can resample the data\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->ymin)
            fl_show_oneliner("Lower edge of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->ymax)
            fl_show_oneliner("Upper edeg of the 1d plot (y axis)\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->tstep)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->autoscale)
            fl_show_oneliner("This button, if switched on, allows to \n"
                             "automatically set the z axis min and max",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->realunits)
            fl_show_oneliner("This button, if switched on, put z axis of the\n"
                             "time-1d plot with the real units of the data",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plot1dtime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1dtime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot1dtime->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plot1dtime->tsize,       DyXformHelp);
     fl_set_object_posthandler (plot1dtime->stat_duration, DyXformHelp);
     fl_set_object_posthandler (plot1dtime->chx_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plot1dtime->ymin,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->ymax,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->zmax,       DyXformHelp);
     fl_set_object_posthandler (plot1dtime->autoscale,  DyXformHelp);
     fl_set_object_posthandler (plot1dtime->realunits,  DyXformHelp);
     fl_set_object_posthandler (plot1dtime->nodc,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->gridx,       DyXformHelp);
     fl_set_object_posthandler (plot1dtime->gridy,       DyXformHelp);
     fl_set_object_posthandler (plot1dtime->gridz,       DyXformHelp);
     fl_set_object_posthandler (plot1dtime->logx,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->logy,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->logz,        DyXformHelp);
     fl_set_object_posthandler (plot1dtime->ok,          DyXformHelp);
     fl_set_object_posthandler (plot1dtime->close,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlotrawtime ---*/

void DyXformHelpPlotrawtime (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotrawtime->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "of the time-raw plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
      if (ob == plotrawtime->maxAverage)
            fl_show_oneliner("Number of data raw vectors to\n"
                             "average (one vector per frame)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->zmax)
            fl_show_oneliner("Upper edge of the z axis of the time-raw plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->tstep)
            fl_show_oneliner("refresh period of the plot, in percent of time window",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->x2av)
            fl_show_oneliner("This button, if switched on, allows to average\n"
                             "squares instead of values (useful if raw data \n"
                             "averaged are spectra)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->autoscale)
            fl_show_oneliner("This button, if switched on, allows to \n"
                             "automatically set the z axis min and max",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->realunits)
            fl_show_oneliner("This button, if switched on, put z axis of the\n"
                             "time-raw plot with the real units of the data",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotrawtime->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->gridz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->logz)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Z axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotrawtime->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotrawtime->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plotrawtime->tsize,       DyXformHelp);
     fl_set_object_posthandler (plotrawtime->maxAverage,  DyXformHelp);
     fl_set_object_posthandler (plotrawtime->zmax,        DyXformHelp);
     fl_set_object_posthandler (plotrawtime->x2av,        DyXformHelp);
     fl_set_object_posthandler (plotrawtime->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plotrawtime->realunits,   DyXformHelp);
     fl_set_object_posthandler (plotrawtime->gridx,       DyXformHelp);
     fl_set_object_posthandler (plotrawtime->gridy,       DyXformHelp);
     fl_set_object_posthandler (plotrawtime->gridz,       DyXformHelp);
     fl_set_object_posthandler (plotrawtime->logx,        DyXformHelp);
     fl_set_object_posthandler (plotrawtime->logy,        DyXformHelp);
     fl_set_object_posthandler (plotrawtime->logz,        DyXformHelp);
     fl_set_object_posthandler (plotrawtime->ok,          DyXformHelp);
     fl_set_object_posthandler (plotrawtime->close,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlottrfct ---*/

void DyXformHelpPlottrfct (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plottrfct->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to compute FFT's",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->chx_resampFreq)
            fl_show_oneliner("You can resample the data of channel 1\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->chy_resampFreq)
            fl_show_oneliner("You can resample the data of channel 2\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->maxAverage)
            fl_show_oneliner("Number of spectra you average",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->refresh_fft)
            fl_show_oneliner("Number of spectra computed between two refreshes of the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->timeshift)
            fl_show_oneliner("Time shift (in seconds) of the channel2 with respect to the channel1",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plottrfct->fmin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->fmax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->ymin)
            fl_show_oneliner("Lower (positive) value to display on Y axis of the module plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->ymax)
            fl_show_oneliner("Upper (positive) value to display on Y axis of the module plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plottrfct->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->modulesqrt)
            fl_show_oneliner("This button, if switched on, compute square root of transfert function module",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridx2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->gridy2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logx2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->logy2)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of phase plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->realunits)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "calibrated values for both channels of the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->autoscale)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "automatic scale of the Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->decayfft)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "a moving average that uses the number of averages defined",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plottrfct->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plottrfct->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plottrfct->tsize,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->chx_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plottrfct->chy_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plottrfct->maxAverage,  DyXformHelp);
     fl_set_object_posthandler (plottrfct->tstep,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->refresh_fft, DyXformHelp);
     fl_set_object_posthandler (plottrfct->fmin,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->fmax,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->ymin,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->ymax,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->timeshift,   DyXformHelp);
     fl_set_object_posthandler (plottrfct->nodc,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->gridx,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->gridy,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->logx,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->logy,        DyXformHelp);
     fl_set_object_posthandler (plottrfct->gridx2,      DyXformHelp);
     fl_set_object_posthandler (plottrfct->gridy2,      DyXformHelp);
     fl_set_object_posthandler (plottrfct->logx2,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->logy2,       DyXformHelp);
     fl_set_object_posthandler (plottrfct->realunits,   DyXformHelp);
     fl_set_object_posthandler (plottrfct->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plottrfct->decayfft,    DyXformHelp);
     fl_set_object_posthandler (plottrfct->modulesqrt,  DyXformHelp);
     fl_set_object_posthandler (plottrfct->ok,          DyXformHelp);
     fl_set_object_posthandler (plottrfct->close,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlotcohe ---*/

void DyXformHelpPlotcohe (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotcohe->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->tsize)
            fl_show_oneliner("Size (in seconds) of the time window\n"
                             "used to compute FFT's",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->chx_resampFreq)
            fl_show_oneliner("You can resample the data of channel 1\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->chy_resampFreq)
            fl_show_oneliner("You can resample the data of channel 2\n"
                             "at a lower frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->tstep)
            fl_show_oneliner("Size (in % of time window size)\n"
                             "of the time window steps",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->maxAverage)
            fl_show_oneliner("Number of spectra you average",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->refresh_fft)
            fl_show_oneliner("Number of spectra computed between two refreshes of the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->timeshift)
            fl_show_oneliner("Time shift (in seconds) of the channel2 with respect to the channel1",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotcohe->fmin)
            fl_show_oneliner("Lower frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->fmax)
            fl_show_oneliner("Upper frequency to display\n"
                             "To select lower and upper edges is like doing a Zoom",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->ymin)
            fl_show_oneliner("Lower (positive) value to display on Y axis of the module plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->ymax)
            fl_show_oneliner("Upper (positive) value to display on Y axis of the module plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == plotcohe->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->modulesqrt)
            fl_show_oneliner("This button, if switched on, compute square root of transfert function module",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of amplitude plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->realunits)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "calibrated values for both channels of the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->autoscale)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "automatic scale of the Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->decayfft)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "a moving average that uses the number of averages defined",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->crossCohe)
            fl_show_oneliner("This button, if switched on, sets\n"
                             "a 2D plot showing the cross-correlation\n"
                             "between channel1 and channel2 for every frequency",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->crossCohe_fmin)
            fl_show_oneliner("Min frequency on the Y axis when\n"
                             "the cross-correlation 2D plot is done",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->crossCohe_fmax)
            fl_show_oneliner("Max frequency on the Y axis when\n"
                             "the cross-correlation 2D plot is done",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotcohe->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotcohe->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plotcohe->tsize,       DyXformHelp);
     fl_set_object_posthandler (plotcohe->chx_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plotcohe->chy_resampFreq, DyXformHelp);
     fl_set_object_posthandler (plotcohe->maxAverage,  DyXformHelp);
     fl_set_object_posthandler (plotcohe->tstep,       DyXformHelp);
     fl_set_object_posthandler (plotcohe->refresh_fft, DyXformHelp);
     fl_set_object_posthandler (plotcohe->fmin,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->fmax,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->ymin,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->ymax,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->timeshift,   DyXformHelp);
     fl_set_object_posthandler (plotcohe->nodc,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->gridx,       DyXformHelp);
     fl_set_object_posthandler (plotcohe->gridy,       DyXformHelp);
     fl_set_object_posthandler (plotcohe->logx,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->logy,        DyXformHelp);
     fl_set_object_posthandler (plotcohe->realunits,   DyXformHelp);
     fl_set_object_posthandler (plotcohe->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plotcohe->decayfft,    DyXformHelp);
     fl_set_object_posthandler (plotcohe->modulesqrt,  DyXformHelp);
     fl_set_object_posthandler (plotcohe->crossCohe,   DyXformHelp);
     fl_set_object_posthandler (plotcohe->crossCohe_fmin, DyXformHelp);
     fl_set_object_posthandler (plotcohe->crossCohe_fmax, DyXformHelp);
     fl_set_object_posthandler (plotcohe->ok,          DyXformHelp);
     fl_set_object_posthandler (plotcohe->close,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpPlot1d ---*/

void DyXformHelpPlot1d (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot1d->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xnBin)
            fl_show_oneliner("Number of x bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xmin)
            fl_show_oneliner("Min value for the 1D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xmax)
            fl_show_oneliner("Max value for the 1D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->refreshDt)
            fl_show_oneliner("Time period of\n"
                             "the plot refresh",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xoffset)
            fl_show_oneliner("User defined offset added to the values on the X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->xscale)
            fl_show_oneliner("User defined scale applied to the values on the Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->stat)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "statistical informations in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->autoedge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->realunits)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "calibrated values on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->nodc)
            fl_show_oneliner("This button, if switched on, removes\n"
                             "a mean value from the values on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot1d->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot1d->bydefault, DyXformHelp);
     fl_set_object_posthandler (plot1d->xnBin,     DyXformHelp);
     fl_set_object_posthandler (plot1d->xmin,      DyXformHelp);
     fl_set_object_posthandler (plot1d->xmax,      DyXformHelp);
     fl_set_object_posthandler (plot1d->refreshDt, DyXformHelp);
     fl_set_object_posthandler (plot1d->xoffset,   DyXformHelp);
     fl_set_object_posthandler (plot1d->xscale,    DyXformHelp);
     fl_set_object_posthandler (plot1d->stat,      DyXformHelp);
     fl_set_object_posthandler (plot1d->gridx,     DyXformHelp);
     fl_set_object_posthandler (plot1d->gridy,     DyXformHelp);
     fl_set_object_posthandler (plot1d->logx,      DyXformHelp);
     fl_set_object_posthandler (plot1d->logy,      DyXformHelp);
     fl_set_object_posthandler (plot1d->autoedge,  DyXformHelp);
     fl_set_object_posthandler (plot1d->realunits, DyXformHelp);
     fl_set_object_posthandler (plot1d->nodc,      DyXformHelp);
     fl_set_object_posthandler (plot1d->ok,        DyXformHelp);
     fl_set_object_posthandler (plot1d->close,     DyXformHelp);

    }

 return;
}


/*------------------------------------------ DyXformHelpPlot2d ---*/

void DyXformHelpPlot2d (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plot2d->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xnBin)
            fl_show_oneliner("Number of x bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xmin)
            fl_show_oneliner("Min x value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xmax)
            fl_show_oneliner("Max x value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xoffset)
            fl_show_oneliner("User defined offset added to values of the X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->xscale)
            fl_show_oneliner("User defined scale applied on values of the X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ynBin)
            fl_show_oneliner("Number of y bins used by the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ymin)
            fl_show_oneliner("Min y value for 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ymax)
            fl_show_oneliner("Max y value for the 2D plot entries",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->yoffset)
            fl_show_oneliner("User defined offset added to values of the Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->yscale)
            fl_show_oneliner("User defined scale applied on values of the Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->refreshDt)
            fl_show_oneliner("Time period of\n"
                             "the plot refresh",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->timeshift)
            fl_show_oneliner("Shift (in seconds) of the data on Y axis\n"
                             "with respect to the data on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->stat)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "statistical informations in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->logz)
            fl_show_oneliner("This button, if switched on, introduces\n"
                             "z log scale in the plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->col)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in color mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->cont)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in contour mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->lego)
            fl_show_oneliner("This button, if switch on, gives\n"
                             "a 2D plot in lego mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->surf)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in surface mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->scat)
            fl_show_oneliner("This button, if switched on, gives\n"
                             "a 2D plot in scatter mode",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->autoedge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->realunits)
            fl_show_oneliner("This button, if switched on, set calibrated\n"
                             "values on both X and Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plot2d->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plot2d->bydefault, DyXformHelp);
     fl_set_object_posthandler (plot2d->xnBin,     DyXformHelp);
     fl_set_object_posthandler (plot2d->xmin,      DyXformHelp);
     fl_set_object_posthandler (plot2d->xmax,      DyXformHelp);
     fl_set_object_posthandler (plot2d->xoffset,   DyXformHelp);
     fl_set_object_posthandler (plot2d->xscale,    DyXformHelp);
     fl_set_object_posthandler (plot2d->ynBin,     DyXformHelp);
     fl_set_object_posthandler (plot2d->ymin,      DyXformHelp);
     fl_set_object_posthandler (plot2d->ymax,      DyXformHelp);
     fl_set_object_posthandler (plot2d->yoffset,   DyXformHelp);
     fl_set_object_posthandler (plot2d->yscale,    DyXformHelp);
     fl_set_object_posthandler (plot2d->refreshDt, DyXformHelp);
     fl_set_object_posthandler (plot2d->timeshift, DyXformHelp);
     fl_set_object_posthandler (plot2d->stat,      DyXformHelp);
     fl_set_object_posthandler (plot2d->gridx,     DyXformHelp);
     fl_set_object_posthandler (plot2d->gridy,     DyXformHelp);
     fl_set_object_posthandler (plot2d->logx,      DyXformHelp);
     fl_set_object_posthandler (plot2d->logy,      DyXformHelp);
     fl_set_object_posthandler (plot2d->logz,      DyXformHelp);
     fl_set_object_posthandler (plot2d->col,       DyXformHelp);
     fl_set_object_posthandler (plot2d->cont,      DyXformHelp);
     fl_set_object_posthandler (plot2d->lego,      DyXformHelp);
     fl_set_object_posthandler (plot2d->surf,      DyXformHelp);
     fl_set_object_posthandler (plot2d->scat,      DyXformHelp);
     fl_set_object_posthandler (plot2d->autoedge,  DyXformHelp);
     fl_set_object_posthandler (plot2d->realunits, DyXformHelp);
     fl_set_object_posthandler (plot2d->ok,        DyXformHelp);
     fl_set_object_posthandler (plot2d->close,     DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpAudio ---*/

void DyXformHelpAudio (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotaudio->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters for this plot",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->t0)
            fl_show_oneliner("Begin time for data audio play\n"
                             "(relative to first frame read)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->duration)
            fl_show_oneliner("Duration of audio play",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotaudio->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotaudio->bydefault, DyXformHelp);
     fl_set_object_posthandler (plotaudio->t0,        DyXformHelp);
     fl_set_object_posthandler (plotaudio->duration,  DyXformHelp);
     fl_set_object_posthandler (plotaudio->ok,        DyXformHelp);
     fl_set_object_posthandler (plotaudio->close,     DyXformHelp);
    }

 return;
}

/*------------------------------------------ DyXformHelpPlotGenePar ---*/

void DyXformHelpPlotGenePar (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == plotgenepar->bydefault)
            fl_show_oneliner("To put the default\n"
                             "parameters in all the plots. Use it carefully.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->tsize)
            fl_show_oneliner("Change the size (in seconds) of the time window\n"
                             "used, in all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->sampFreq)
            fl_show_oneliner("Change the resampling of the data\n"
                             "on all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->tstep)
            fl_show_oneliner("Define the size (in % of time window size)\n"
                             "of the time window steps on all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->nodc)
            fl_show_oneliner("This button, if switched on, removes DC component\n"
                             "on all the concerned plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->autoscale)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on Y axis of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->autoedge)
            fl_show_oneliner("This button, if switched on, allows an\n"
                             "automatic set of min and max on X axis of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->realunits)
            fl_show_oneliner("This button, if switched on, sets the Y units\n"
                             "in real units instead of ADC counts of all the plots.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->gridx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->gridy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "gridlines along Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
/**
     if (ob == plotgenepar->logx)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->logy)
            fl_show_oneliner("This button, if switched on, puts\n"
                             "log scale on Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->minmax_resample)
            fl_show_oneliner("This button, if switched on, says that any data resampling\n"
                             "will be done using min and max instead of mean",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
**/
     if (ob == plotgenepar->all_clone)
            fl_show_oneliner("If switched on, this button allows to clone all\n"
                             " the parameters, even those you did not touched.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->ok)
            fl_show_oneliner("Closes the panel and\n"
                             "acknowledges the changes you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == plotgenepar->close)
            fl_show_oneliner("Closes the panel and\n"
                             "cancel any change you made",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (plotgenepar->bydefault,   DyXformHelp);
     fl_set_object_posthandler (plotgenepar->tsize,       DyXformHelp);
     fl_set_object_posthandler (plotgenepar->sampFreq,    DyXformHelp);
     fl_set_object_posthandler (plotgenepar->tstep,       DyXformHelp);
     fl_set_object_posthandler (plotgenepar->autoscale,   DyXformHelp);
     fl_set_object_posthandler (plotgenepar->autoedge,    DyXformHelp);
     fl_set_object_posthandler (plotgenepar->realunits,   DyXformHelp);
     fl_set_object_posthandler (plotgenepar->nodc,        DyXformHelp);
     fl_set_object_posthandler (plotgenepar->gridx,       DyXformHelp);
     fl_set_object_posthandler (plotgenepar->gridy,       DyXformHelp);
/***
     fl_set_object_posthandler (plotgenepar->logx,        DyXformHelp);
     fl_set_object_posthandler (plotgenepar->logy,        DyXformHelp);
     fl_set_object_posthandler (plotgenepar->minmax_resample, DyXformHelp);
**/
     fl_set_object_posthandler (plotgenepar->all_clone,   DyXformHelp);
     fl_set_object_posthandler (plotgenepar->ok,          DyXformHelp);
     fl_set_object_posthandler (plotgenepar->close,       DyXformHelp);
    }

 return;
}

/*------------------------------------------ DyXformHelpCmConnect ---*/

void DyXformHelpCmConnect (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == cmservers->liste)
            fl_show_oneliner("This browser shows the list of all Cm servers\n"
                             "currently available on the Ethernet network.\n"
                             "Selecte one of them. Then button OK will do the connection.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == cmservers->fullList)
            fl_show_oneliner("If on, this button show the full list of Cm names\n"
                             "provided by the Cm Name Server. Otherwise, only\n"
                             "default dataDisplay providers are shown.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == cmservers->update_cmlist)
            fl_show_oneliner("This button updates the liste of Cm names present in\n"
                             "the browser, by doing a request to the Cm Name Server.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == cmservers->searchCmName)
            fl_show_oneliner("Begin to type a name in this input field to search\n"
                             "for a Cm name in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);

     if (ob == cmservers->ok)
            fl_show_oneliner("This button do the connection to the Cm server\n"
                             "selected in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);


    }
 else
    {
     fl_set_object_posthandler (cmservers->liste,          DyXformHelp);
     fl_set_object_posthandler (cmservers->update_cmlist,  DyXformHelp);
     fl_set_object_posthandler (cmservers->fullList,       DyXformHelp);
     fl_set_object_posthandler (cmservers->searchCmName,   DyXformHelp);
     fl_set_object_posthandler (cmservers->ok,             DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpUserFct ---*/

void DyXformHelpUserFct (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == userfuncbrowser->userFuncLoaded)
            fl_show_oneliner("List of user functions currently loaded. This list\n"
                             "is the same as in the menu UserFct of mainbrowser panel.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->newFunc)
            fl_show_oneliner("To create a new file x.C that you can fill with your own\n"
                             "algorithm's code operating on the channels to create new\n"
                             "channels to plot.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->loadInterpret)
            fl_show_oneliner("Load a file x.C already compiled (x.so available).\n"
                             "Then, this user fonction x appears in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == userfuncbrowser->loadCompile)
            fl_show_oneliner("Compile and load a file x.C.\n"
                             "Then, this user fonction x appears in the browser.",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (userfuncbrowser->userFuncLoaded,  DyXformHelp);
     fl_set_object_posthandler (userfuncbrowser->newFunc,         DyXformHelp);
     fl_set_object_posthandler (userfuncbrowser->loadInterpret,   DyXformHelp);
     fl_set_object_posthandler (userfuncbrowser->loadCompile,     DyXformHelp);
    }

 return;
}



/*------------------------------------------ DyXformHelpSliders ---*/

void DyXformHelpSliders (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dysliders->xminslider)
            fl_show_oneliner("Using this slider from left to right moves the\n"
                             "min X of the plots from the lowest X to the max X",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dysliders->xmaxslider)
            fl_show_oneliner("Using this slider from left to right moves the\n"
                             "max X of the plots from the min X to the uppest X",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dysliders->xpslider)
            fl_show_oneliner("Using this slider from left to right moves the\n"
                             "the zoomed plots from the lowest X to the uppest X",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dysliders->xminslider,   DyXformHelp);
     fl_set_object_posthandler (dysliders->xmaxslider,   DyXformHelp);
     fl_set_object_posthandler (dysliders->xpslider,     DyXformHelp);
     fl_set_object_posthandler (dysliders->yminslider,   DyXformHelp);
     fl_set_object_posthandler (dysliders->ymaxslider,   DyXformHelp);
     fl_set_object_posthandler (dysliders->ypslider,     DyXformHelp);
     fl_set_object_posthandler (dysliders->zminslider,   DyXformHelp);
     fl_set_object_posthandler (dysliders->zmaxslider,   DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpOptions ---*/

void DyXformHelpOptions (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dyoptions->noplot)
            fl_show_oneliner("This button, when checked, allows to disable the\n"
                             "display for all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->trigused)
            fl_show_oneliner("This button, when checked, allows to use the\n"
                             "trigger defined with TRIGGER in Data and Plots panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->trigpause)
            fl_show_oneliner("This button, when checked, allows to have an automatic \n"
                             "pause between each event found by the trigger",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->color2d)
            fl_show_oneliner("Each time this button is pushed, a new colormap is \n"
                             "chosen for the plots 2D, FFTTIME, etc...",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->autodeselectplot)
            fl_show_oneliner("This button, when checked, allows an automatic deselection\n"
                             "of plots in the browser each time an action is made on the plots list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->autodeselectch)
            fl_show_oneliner("This button, when checked, allows an automatic deselection \n"
                             "of channels in the browsers each time a new plot is created",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->noBox)
            fl_show_oneliner("This button, when checked, removes the red (corrupted data),\n"
                             "green (missing channel) or blue (missing frame)\n"
                             "colored boxes in the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlegend)
            fl_show_oneliner("Increase or decrease (-) the font size\n"
                             "in the plots legends",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomstat)
            fl_show_oneliner("Increase or decrease the font size\n"
                             "in the plots statistics boxes",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlabelx)
            fl_show_oneliner("Increase or decrease the font size\n"
                             "in the plots labels on X axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomlabely)
            fl_show_oneliner("Increase or decrease the font size\n"
                             "in the plots labels on Y axis",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomtimestring)
            fl_show_oneliner("Increase or decrease the font size\n"
                             "of the time string under the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->zoomtitle)
            fl_show_oneliner("Increase or decrease the font size\n"
                             "of the plots title",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->loglinx)
            fl_show_oneliner("This button, when checked, put alternately log or\n"
                             "linear scale on X axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->logliny)
            fl_show_oneliner("This button, when checked, put alternately log or\n"
                             "linear scale on Y axis of all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->fullwhite)
            fl_show_oneliner("This button, when checked, puts white color over\n"
                             "the full Canvas of the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->indelebile)
            fl_show_oneliner("This button, when checked, allows to superpose the reference\n"
                             "plots in memory to the current plots while dataDisplay is running",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoptions->nicefont)
            fl_show_oneliner("This button, when checked, allows to use helvetica font (62)\n"
                             "instead of times font (22) in all the plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dyoptions->noplot,             DyXformHelp);
     fl_set_object_posthandler (dyoptions->trigused,           DyXformHelp);
     fl_set_object_posthandler (dyoptions->trigpause,          DyXformHelp);
     fl_set_object_posthandler (dyoptions->color2d,            DyXformHelp);
     fl_set_object_posthandler (dyoptions->autodeselectplot,   DyXformHelp);
     fl_set_object_posthandler (dyoptions->autodeselectch,     DyXformHelp);
     fl_set_object_posthandler (dyoptions->noBox,              DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomlegend,         DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomstat,           DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomlabelx,         DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomlabely,         DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomtimestring,     DyXformHelp);
     fl_set_object_posthandler (dyoptions->zoomtitle,          DyXformHelp);
     fl_set_object_posthandler (dyoptions->loglinx,            DyXformHelp);
     fl_set_object_posthandler (dyoptions->logliny,            DyXformHelp);
     fl_set_object_posthandler (dyoptions->fullwhite,          DyXformHelp);
     fl_set_object_posthandler (dyoptions->indelebile,         DyXformHelp);
     fl_set_object_posthandler (dyoptions->nicefont,           DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpOutput ---*/

void DyXformHelpOutput (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dyoutput->output_default)
            fl_show_oneliner("This button allows to put default values\n"
                             "in all the fields of this panel",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_dir)
            fl_show_oneliner("Name of the directory where will be written the\n"
                             "files containing data frames",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_nframe_perfile)
            fl_show_oneliner("Number of frames per file written on disk",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_flag)
            fl_show_oneliner("This button, when checked, allows the writing of data on disk",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dyoutput->output_tag)
            fl_show_oneliner("Write here the list of channels you want to write on disk\n"
                             "You can use wildcard like in \042Em*BD*01\042 to choose\n"
                             "all channels beginning with Em, containing BD and finishing with 01",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dyoutput->output_default,        DyXformHelp);
     fl_set_object_posthandler (dyoutput->output_dir,            DyXformHelp);
     fl_set_object_posthandler (dyoutput->output_nframe_perfile, DyXformHelp);
     fl_set_object_posthandler (dyoutput->output_flag,           DyXformHelp);
     fl_set_object_posthandler (dyoutput->output_tag,            DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpDebug ---*/

void DyXformHelpDebug (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == dydebug->dbg_vdyplot)
            fl_show_oneliner("This button, if checked, allows debugging of the\n"
                             "functions implied in the plots drawing and painting",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_speed)
            fl_show_oneliner("This button, if checked, allows to know how much time\n"
                             "is passed in various functions",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->dbg_others)
            fl_show_oneliner("This button, if checked, allows debugging of various\n"
                             "useful functions of the dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->debuglvl)
            fl_show_oneliner("The higher this value is the more debugging output\n"
                             "you get for each of the debugging types below",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == dydebug->checks)
            fl_show_oneliner("This button dump many parameters of the data inputs and plots\n"
                             "in the xterm window where you started dataDisplay",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (dydebug->dbg_vdyplot,  DyXformHelp);
     fl_set_object_posthandler (dydebug->dbg_speed,    DyXformHelp);
     fl_set_object_posthandler (dydebug->dbg_others,   DyXformHelp);
     fl_set_object_posthandler (dydebug->debuglvl,     DyXformHelp);
     fl_set_object_posthandler (dydebug->checks,       DyXformHelp);
    }

 return;
}


/*------------------------------------------ DyXformHelpReffile ---*/

void DyXformHelpReffile (FL_OBJECT *ob, long arg)
{
 if (arg == 0)
    {
     if (ob == reffile->file_list)
            fl_show_oneliner("This browser contains the list of reference plots files that\n"
                             "have been loaded into memory. The last number in the name is the code\n"
                             "color of the reference plots when they are superposed to the current plots",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->remove2d)
            fl_show_oneliner("This button removes from memory all the\n"
                             "2-dimensional reference plots of the files selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->removefile)
            fl_show_oneliner("This button removes from the list all the files selected",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->clear_filebrowser)
            fl_show_oneliner("This button removes all the files from the list",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->edit)
            fl_show_oneliner("This button allows to change the code color of the selected file\n"
                             "Colors code it the ROOT one (1=black, 2=red, ... 51=pink, 52=dark blue, ...)",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
     if (ob == reffile->addfile)
            fl_show_oneliner("This button allows to add a new file to the list.\n"
                             "The code color is defined automatically.\n"
                             "To change it, use the Edit button",
                             ob->form->x+ob->x, ob->form->y+ob->y + ob->h + 5);
    }
 else
    {
     fl_set_object_posthandler (reffile->file_list,         DyXformHelp);
     fl_set_object_posthandler (reffile->removefile,        DyXformHelp);
     fl_set_object_posthandler (reffile->clear_filebrowser, DyXformHelp);
     fl_set_object_posthandler (reffile->edit,              DyXformHelp);
     fl_set_object_posthandler (reffile->addfile,           DyXformHelp);
     fl_set_object_posthandler (reffile->remove2d,          DyXformHelp);
    }
 return;
}


/*------------------------------------------ DyXformUserFctHelp ---*/

void DyXformUserFctHelp (FL_OBJECT *obj, long arg)
{
 FL_OBJECT *ob, *browser,*close;
 long xsize, ysize;

 xsize      = 450;
 ysize      = 320;
 userprochelp  = fl_bgn_form(FL_NO_BOX, xsize, ysize);
 ob         = fl_add_box(FL_UP_BOX,0,0,xsize,ysize,"");
 browser    = fl_add_browser(FL_HOLD_BROWSER,10,10,xsize-20,ysize-60,"User Functions Help");
    fl_set_object_color(browser,FL_LEFT_BCOL,FL_WHEAT);
    fl_set_object_lstyle(browser,FL_TIMESBOLD_STYLE);
    fl_set_object_lsize(browser,FL_MEDIUM_SIZE);
 close = fl_add_button(FL_NORMAL_BUTTON,xsize-80,ysize-40,70,30,"Close");
    fl_set_object_color(close,FL_WHEAT,FL_COL1);
    fl_set_object_lstyle(close,FL_TIMESBOLD_STYLE);
    fl_set_object_lsize(close,FL_MEDIUM_SIZE);
    fl_set_object_callback(close,DyXformClosePanel,USERFCTHELP);
 fl_end_form();

 fl_add_browser_line(browser,"\n                        ****  USER FUNCTIONS HELP  **** \n\n"
                             " Using this panel or using the \"User Fct\" button, you can create any function \n"
                             " operating on a set of input channels and outputing a new channel whose name will \n"
                             " begin by default with UserOp. \n"
                             "    \n"
                             " When you click on the \"User Fct\" menu and choose \"Operation String\" \n"
                             " you get an input line where you can write any C function to combine channels. \n"
                             " This line begins by default with \"UserOp=\". This is the default name's start \n"
                             " of the new channel that you creates with this line. \n"
                             "    \n"
                             " Once you click on Ok, two files UserOp.C and UserOp.h are created. \n"
                             " You can modify them afterwards if you want to change the function. \n");
 fl_add_browser_line(browser,"                                     *********                \n"
                             " You can also create a new function by using the button \"New Function\" \n"
                             " of this panel. You can also Load and Compile a function you already had \n"
                             " written (files UserOp.C and UserOp.h) and that you want to use on some \n"
                             " channels to process them before passing the result to the plots. \n"
                             "   \n"
                             " Each time, you click on the \"Start\" button of the dataDisplay mainpanel, \n"
                             " the function that you have created are stored at the end of the dd.car \n"
                             " configuration file of the dataDisplay. Then, any UserOp.C and UserOp.h files \n"
                             " disappear since the information is kept in dd.car \n\n");

 DyXformRaisePanel(NULL,USERFCTHELP);
 return;
}


/*------------------------------------------ DyXformHelp ---*/
/*--- post-handler (for help ballons) ---*/

int DyXformHelp (FL_OBJECT *ob, int ev, FL_Coord mx, FL_Coord y, int key, void *xev)
{
 int help_selected;

 help_selected = fl_get_button(dyhelp->helpbutton);
/*
 help_selected = fl_get_button(mainpanel->help);
*/
 if (!help_selected) return(0);

 /*--- show help ballons ---*/

 if (ev == FL_ENTER)
    {
     DyXformHelpMainPanel(ob,0);     /*--- show help ballons for mainpanel panel   ---*/
     DyXformHelpMainBrowser(ob,0);   /*--- show help ballons for mainbrowser panel ---*/
     DyXformHelpPlottime(ob,0);      /*--- show help ballons for plottime panel    ---*/
     DyXformHelpPlotfft(ob,0);       /*--- show help ballons for plotfft panel     ---*/
     DyXformHelpPlotffttime(ob,0);   /*--- show help ballons for plotffttime panel ---*/
     DyXformHelpPlot1dtime(ob,0);    /*--- show help ballons for plot1dtime panel  ---*/
     DyXformHelpPlotrawtime(ob,0);   /*--- show help ballons for plotrawtime panel ---*/
     DyXformHelpPlottrfct(ob,0);     /*--- show help ballons for plottrfct panel   ---*/
     DyXformHelpPlotcohe(ob,0);      /*--- show help ballons for plotcohe panel    ---*/
     DyXformHelpPlot1d(ob,0);        /*--- show help ballons for plot1d panel      ---*/
     DyXformHelpPlot2d(ob,0);        /*--- show help ballons for plot2d panel      ---*/
     DyXformHelpAudio(ob,0);         /*--- show help ballons for plotaudio panel   ---*/
     DyXformHelpPlotGenePar(ob,0);   /*--- show help ballons for plotgenepar panel ---*/
     DyXformHelpCmConnect(ob,0);     /*--- show help ballons for cmservers panel   ---*/
     DyXformHelpUserFct(ob,0);       /*--- show help ballons for userfuncbrowser panel ---*/
     DyXformHelpOptions(ob,0);       /*--- show help ballons for options panel     ---*/
     DyXformHelpOutput(ob,0);        /*--- show help ballons for output panel      ---*/
     DyXformHelpDebug(ob,0);         /*--- show help ballons for debug panel       ---*/
     DyXformHelpReffile(ob,0);       /*--- show help ballons for reffile panel     ---*/
    }

 else if (ev != FL_MOTION)
    {fl_hide_oneliner();}

 return(0);
}


